/*
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such 
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into 
 * with Lombardi Software. No rights or licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * JavaScript file, contains code of FieldSet class.
 * 
 * Created by Sergey Rassokhin, 9/5/2007 
 */
 
/**
 * Represents FieldSetDecorator control. It is just wrapping <fieldset/> element 
 * for more comfortable using of it. 
 * 
 * @constructor
 * @class
 */
function FieldSet(captionText) {
	// <fieldset/> element of the control
	this.root = null;
	// <legend/> element
	this.caption = null;
	// <tbody/> element
	this.tbody = null;
	// Text of the fieldset caption
	this.captionText = captionText;
	this.fields = new Object();
	this.rows = new Object();
	this.adds = new Object();
	this.states = new Object();
} 

/**
 * Initializes control. Create all needed DOM elements.
 */
FieldSet.prototype.init = function() {
	this.root = document.createElement("fieldset");
	this.caption = document.createElement("legend");
	this.captionTextNode = document.createTextNode(this.captionText)
	this.caption.appendChild(this.captionTextNode);
	this.root.appendChild(this.caption);
	
// 	// create table
// 	var table = document.createElement("table");
//	table.className = "docAttachmentTableNoLines";
//	table.style.width = "100%";
//	table.style.margin = "0px";
//	table.style.marginBottom = "7px";
//	table.style.borderCollapse = "separate";
//	table.cellSpacing = "3px";	
//	var tbody = document.createElement("tbody");	
//	table.appendChild(tbody);
//	this.tbody = tbody;
//	this.root.appendChild(table);
}

/**
 * Sets the given caption text.
 */
FieldSet.prototype.setCaptionText = function(captionText) {
	this.caption.removeChild(this.captionTextNode);
	this.captionTextNode = document.createTextNode(captionText)
	this.caption.appendChild(this.captionTextNode);
}

/**
 * Add specified element into this control.
 */
FieldSet.prototype.addElement = function(name, control) {
	this[name] = control;
	this.root.appendChild(control.root != null ? control.root : control);
}

/**
 * Removes specified element from fielset control.
 */
FieldSet.prototype.removeElement = function(name) {
	this.root.removeChild(this[name]);
	this[name] = null;
}

/**
 * Adds row to embedded table in a subDiv in the fieldset.
 * 
 * @private
 * @argument {string} title - title, which will be shown at left column
 * @argument {DOM element} field - element, which will be shown at rights column
 * @argument {boolean} visible - initial visible state of row
 * @argument {string} id - <tr/> element id
 */
FieldSet.prototype.addRow = function (title, field, visible, rowId, required, propertyType, button ) {
	if (typeof this.documentPropertiesSubDiv == "undefined") {
	 	// create table
	 	var table = document.createElement("table");
		table.className = "docAttachmentTableNoLines";
		table.style.width = "100%";
		table.style.margin = "0px";
		//table.style.marginBottom = "7px";
		table.style.border = "none";
		//table.style.borderCollapse = "none";
		table.cellSpacing = "3px";	
		var tbody = document.createElement("tbody");	
		table.appendChild(tbody);
		this.documentPropertiesSubDiv = dojo.create("div", {
            style: {
                height: "0px",
				overflow:'auto'
			}
		});
		
		this.tbody = tbody;
		this.documentPropertiesSubDiv.id = "documentPropertiesSubDiv";
		this.documentPropertiesSubDiv.appendChild(table);	
		this.root.appendChild(this.documentPropertiesSubDiv);
	}
	
	var tr = document.createElement("tr");
	tr.id = rowId;
	tr.style.border = "none";
	tr.style.height = "40px";
	var td = document.createElement("td");
	field.title = title;
	if (required == 'true')
		title = '* ' + title;
	else
		title = '\u00a0\u00a0' + title;
	td.innerHTML = title;
	td.align = "left";
	td.width = "180px";
	td.style.border = "none";
	td.className = "inputFieldLabel";
	tr.appendChild(td);
	var td = document.createElement("td");
	field.req = required;
	field.propType = propertyType;
	if (propertyType == "boolean") {
		createHiddenInputElement(td, field.name, 'false');
	}	
	td.appendChild(field);
	td.width = "200px";
	td.style.border = "none";
	tr.appendChild(td);
	
	var td = document.createElement("td");
	if (button != null) {
		td.appendChild(button);
		this.adds[rowId] = button;
	}
	td.style.border = "none";
	td.width = "45px";
	tr.appendChild(td);
	
	var td = document.createElement("td");
	td.style.border = "none";
	tr.appendChild(td);
	
	tr.style.display = visible ? "" : "none";
	td.style.border = "none";
	this.tbody[rowId] = tr;
	this.rows[rowId] = field;
	this.tbody.appendChild(tr);
	var height = parseInt(this.documentPropertiesSubDiv.style.height, 10);
	if(height < 370 && height < document.documentElement.clientHeight - document.documentElement.clientHeight % 40 - 400)
		this.documentPropertiesSubDiv.style.height = (height + 40).toString() + "px";
	this.saveField(rowId);
}

FieldSet.prototype.insertAfterRow = function (row, title, field, visible, rowId, required, propertyType, button ) {
	if (typeof this.documentPropertiesSubDiv == "undefined") {
	 	// create table
	 	var table = document.createElement("table");
		table.className = "docAttachmentTableNoLines";
		table.style.width = "100%";
		table.style.margin = "0px";
		//table.style.marginBottom = "7px";
		table.style.border = "none";
		//table.style.borderCollapse = "none";
		table.cellSpacing = "3px";	
		var tbody = document.createElement("tbody");	
		table.appendChild(tbody);
		this.documentPropertiesSubDiv = dojo.create("div", {
            style: {
                height: "0px",
				overflow:'auto'
			}
		});
		
		this.tbody = tbody;
		this.documentPropertiesSubDiv.id = "documentPropertiesSubDiv";
		this.documentPropertiesSubDiv.appendChild(table);	
		this.root.appendChild(this.documentPropertiesSubDiv);
	}
	
	var tr = document.createElement("tr");
	tr.id = rowId;
	tr.style.border = "none";
	tr.style.height = "40px";
	var td = document.createElement("td");
	field.title = title;
	if (required == 'true')
		title = '* ' + title;
	else
		title = '\u00a0\u00a0' + title;
	td.innerHTML = title;
	td.align = "left";
	td.width = "180px";
	td.style.border = "none";
	td.className = "inputFieldLabel";
	tr.appendChild(td);
	var td = document.createElement("td");
	field.req = required;
	field.propType = propertyType;
	if (propertyType == "boolean") {
		createHiddenInputElement(td, field.name, 'false');
	}	
	td.appendChild(field);
	td.style.border = "none";
	td.width = "200px";
	tr.appendChild(td);

	var td = document.createElement("td");
	if (button != null) {
		td.appendChild(button);
		this.adds[rowId] = button;
	}
	td.style.border = "none";
	td.width = "45px";
	tr.appendChild(td);
	
	var td = document.createElement("td");
	td.style.border = "none";
	tr.appendChild(td);
	
	tr.style.display = visible ? "" : "none";
	td.style.border = "none";
	this.tbody[rowId] = tr;
	this.rows[rowId] = field;
	if (row.nextSibling == null)
		this.tbody.appendChild(tr);
	else
		this.tbody.insertBefore(tr, row.nextSibling);
	var height = parseInt(this.documentPropertiesSubDiv.style.height, 10);
	if(height < 370 && height < document.documentElement.clientHeight - document.documentElement.clientHeight % 40 - 400)
		this.documentPropertiesSubDiv.style.height = (height + 40).toString() + "px";
	this.saveField(rowId);
}

/**
 * Adds row to embedded table in the fieldset.
 * 
 * @private
 * @argument {string} title - title, which will be shown at left column
 * @argument {DOM element} field - element, which will be shown at rights column
 * @argument {boolean} visible - initial visible state of row
 * @argument {string} id - <tr/> element id
 */
FieldSet.prototype.addRowWithoutSubDiv = function (title, field, visible, rowId, required) {
	if (!this.tbody) {
	 	// create table
	 	var table = document.createElement("table");
		table.className = "docAttachmentTableNoLines";
		table.style.width = "100%";
		table.style.margin = "0px";
		//table.style.marginBottom = "7px";
		table.style.border = "none";
		//table.style.borderCollapse = "separate";
		table.cellSpacing = "3px";	
		var tbody = document.createElement("tbody");	
		table.appendChild(tbody);
		this.tbody = tbody;
		this.root.appendChild(table);		
	}
	
	var tr = document.createElement("tr");
	tr.id = rowId;
	var td = document.createElement("td");
	if (required == 'true')
		title = '* ' + title;
	else
		title = '\u00a0\u00a0' + title;
	td.innerHTML = title;
	td.align = "left";
	td.width = "180px";
	td.style.border = "none";
	td.className = "inputFieldLabel";
	tr.appendChild(td);
	var td = document.createElement("td");
	field.title = title;
	field.req = required;
	field.propType = 'fileContent';
	field.size = "80";
	td.style.border = "none";
	td.appendChild(field);
	tr.appendChild(td);
	tr.style.display = visible ? "" : "none";
	
	this.tbody[rowId] = tr;
	this.rows[rowId] = field;
	this.tbody.appendChild(tr);
	this.saveField(rowId);
}

/**
 * Removes row to embedded table.
 * 
 * @private
 * @argument {string} id - <tr/> element id
 */
FieldSet.prototype.removeRow = function (rowId) {
	if (!this.tbody)
		return;
	
	this.tbody.removeChild(this.tbody[rowId]);
	this.tbody[rowId] = null;
}

FieldSet.prototype.getRow = function (rowId) {
	if (!this.tbody)
		return null;
	
//	return this.tbody[rowId];
	return this.rows[rowId];
}

FieldSet.prototype.getRow1 = function (rowId) {
	if (!this.tbody)
		return null;
	
//	return this.rows[rowId];
	return this.tbody[rowId];
}

FieldSet.prototype.getAddButton = function (rowId) {
	if (!this.tbody)
		return null;
	
	return this.adds[rowId];
}

FieldSet.prototype.validateValues = function() {
	var valid = true;
	for (field in this.fields) {		
		var row = this.getRow(field);
		if(row.propType == 'integer') {
			var widget = dijit.byId(dijit.byId(field));
			var value = widget.value;
			valid = validateIntegers(row, field, value, false, valid);
		} else if(row.propType == 'datetime') {
			var index = field.lastIndexOf("$");
			var parentField;
			if (index != -1) {
				parentField = field.substring(0, index);
			} else
				parentField = field;
			var dtBox = dijit.byId(dijit.byId(field));
			if(dtBox.get('displayedValue') != "")
				var value = validateDate(dtBox.value);
			else {
				var value = "";
			}
			if(value == "invalid") {
				setVisible(document.getElementById('invalidValueError'), true);
				(document.getElementById(parentField)).childNodes[0].style.color='#FF0000';
				valid = false;
			} else {
				if(valid)
					setVisible(document.getElementById('invalidValueError'), false);
				(document.getElementById(parentField)).childNodes[0].style.color='#000000';
			}
		} else if(row.propType == 'decimal') {
			var widget = dijit.byId(dijit.byId(field));
			var value = widget.value;
			valid = validateDecimals(row, field, value, false, valid);
		}
		if(row.type == 'select-one' && row.values != 'undefined') {
			var widget = dijit.byId(dijit.byId(field));
			var checkVal = widget.get("displayedValue");
			if(checkVal != "")
				valid = validateChoices(row, field, row.values, checkVal, false, valid);
		}
	}
	if(valid)
		setVisible(document.getElementById('invalidValueError'), false);
	return valid;
}

FieldSet.prototype.validateFileContent = function(createDocument) {
	var valid = true;
	for (field in this.fields) {		
		if(field == 'fileContent') {
			var row = this.getRow(field);
			var value = row.value;
			if(value == null || value == "") {
				if(createDocument)
					valid = confirm(documentListView_noContentWithName);
				else
					valid = confirm(documentListView_noContentWithProperties);
			}
		}
	}
	return valid;
}

/**
* summary:
* escape the text to clean xss attacks
* using .createTextNode to utilize the browser escaping capabilities
* txt: the string to escape
*/
FieldSet.prototype.htmlEscape = function (txt) {
	var element = document.createElement('span');
	var content = document.createTextNode(txt);
	element.appendChild(content);
	return element.innerHTML;
}; 

FieldSet.prototype.validate = function() {
	var valid = true;
	for (field in this.fields) {
		var row = this.getRow(field);	
		if(row.propType == 'hidden') {
			continue;
		} else if(row.propType == "datetime") {
			var dtBox = dijit.byId(dijit.byId(field));
			if(dtBox.get('displayedValue') != "")
				var value = validateDate(dtBox.value);
			else {
				var value = "";
			}
		} else {
			var widget = dijit.byId(dijit.byId(field));
			if(row.propType == 'string' || row.propType == 'id')
				widget.setValue(this.htmlEscape(widget.value));
			var value = widget.value;
		}
		if(row.req == 'true') {
			if(value == null || value == "") {
				setVisible(document.getElementById('invalidFormError'), true);
				setVisible(document.getElementById('invalidValueError'), false);
				(document.getElementById(field)).childNodes[0].style.color='#FF0000';
				row.style.border = "1px solid #ff0000";
				valid = false;
			} else {
				if(value != "invalid") {
					(document.getElementById(field)).childNodes[0].style.color='#000000';
					row.style.border = 	"1px solid #bbb";
				}
			}
		} else {
			//setVisible(document.getElementById('invalidValueError' + field), false);
			(document.getElementById(field)).childNodes[0].style.color='#000000';
			//row.style.border = "1px solid #bbb";
		}
	}
	
	if(valid) {
		setVisible(document.getElementById('invalidFormError'), false);
	}
	return valid;
}

/**
  * Saves field parameters for further looking for them by field's name.
  */
FieldSet.prototype.saveField = function(rowId) {
 	var o = new Object(); 	
 	o.rowId = rowId;
 	this.fields[rowId] = o;	
 }

/**
 * Clears the control.
 */
FieldSet.prototype.clear = function() {
	this.init();
}

/**
 * Clears the control.
 */
FieldSet.prototype.clear1 = function() {
//	if (this.tbody)
//		this.tbody.innerHTML = "";
	for (field in this.fields) {
		this.removeRow(field);
	}
	if(this.documentPropertiesSubDiv)
		this.documentPropertiesSubDiv.style.height = "0px";
	this.rows = new Object();
	this.fields = new Object();
	this.adds = new Object();
}


FieldSet.prototype.saveState = function() {
	this.states = new Object();
	
	for (field in this.fields) {
		var fieldId = field;		
        index = fieldId.lastIndexOf("$");
        if (index != -1) {
        	fieldId = fieldId.substring(0, index);
        }
		var row = this.getRow(field);
		var values = this.states[fieldId];
		
		if (typeof values == "undefined") {
			values = [];
			this.states[fieldId] = values;
		}
		
		var obj = new Object();
		
		var dtBox = dijit.byId(dijit.byId(field));
		if(row.propType == "datetime") {
			obj.type = 'Date';
			if (!isNaN(dtBox.value.getTime()))
				obj.value = dtBox.value.getTime();
		} else if(row.propType == "boolean") {
			obj.type = 'Boolean';
			if(dtBox.checked)
				obj.value = dtBox.value;
		} else if(row.propType == "hidden" || row.propType == "file") {
			obj.type = row.propType;
			if (row.value != null && row.value.length > 0)
				obj.value = row.value;
		} else {
			obj.type = 'String';
			if(dtBox) {
				obj.value = dtBox.value;
			}
		}
		if (obj.type != undefined && obj.value != undefined)
			values.push(obj);
	}
}

FieldSet.prototype.popState = function() {	
	for (field in this.fields) {
		initializeFieldValue(this, field, this.states[field]);
	}	
}
