var arrayIndex=1;

// replace all dots in a dot name to specified char. The function is used
// to convert a dot name to underscore name or poundsign name
function convertName(name, newChar) {
    return name.replace(/\]/g, "").replace(/\[/g, newChar).replace(/\./g, newChar);
}

// construct a dot name out of an underscore name, based on a given
// template name which provides the pattern info about how the dot
// name should look like.
// For example,
//   underScoreName  = tw_local_a_1_bf_0_c
//   templateDotName = tw.local.a[0].bf_0.c
//   function return = tw.local.a[1].bf_0.c
function constructDotName(underScoreName, templateDotName) {
	var newDotName = "";
	var names = templateDotName.split("\.");
	for (var i = 0; i < names.length; i++) {
		if (names[i].length == 0)
			continue;
		
		var curName = names[i];
		if (curName.indexOf("[") == -1) {
			if (i != 0)
				newDotName = newDotName + "." +  underScoreName.substring(0, curName.length);
			else
				newDotName = underScoreName.substring(0, curName.length);
			
			underScoreName = underScoreName.substring(curName.length + 1);
		}else {
			if (curName.match(/^(.*)\[\d+\]$/g)) {
				var partialName = RegExp.$1;
				underScoreName = underScoreName.substring(partialName.length + 1);
				if (underScoreName.match(/^(\d+)_/g)) {
					partialName = partialName + "[" + RegExp.$1 + "]";
					underScoreName = underScoreName.substring((RegExp.$1).length + 1);
					newDotName = newDotName + "." +  partialName;
				}
			}
		}
	}
	
	return newDotName;
}

function createNewRow(tableArray, templateString) {
	var initArrayInput = dojo.query("#"+ tableArray.getAttribute("id")+"_INPUT", tableArray)[0];
	var arrayIndex = parseInt(initArrayInput.getAttribute("value")) + 1;
	initArrayInput.setAttribute("value", arrayIndex);
	var dotName = tableArray.getAttribute("templatename");
	
	// Replace parent
	var underScoreNameRE = new RegExp(convertName(dotName, '_'), "g");
	var poundsignNameRE = new RegExp(convertName(dotName, '#'), "g");
	
	// We need to construct the correct dot name. We cannot start with tableArray.id
	// (which is an underscored name) as it is not clear things like "_0_" is
	// array index or part of parameter's name
	var newDotName = constructDotName(tableArray.getAttribute("id"), dotName);
	var newUnderScoreName = convertName(newDotName, '_');
	var newPoundsignName = convertName(newDotName, '#');
	
	templateString = templateString.replace(underScoreNameRE, newUnderScoreName);
	templateString = templateString.replace(poundsignNameRE, newPoundsignName);
	
	// Replace index
	var newUnderScoreNameWtihIndex = newUnderScoreName + "_" + arrayIndex;
	var newPoundsignNameWithIndex = newPoundsignName + "#" + arrayIndex;
	underScoreNameRE = new RegExp(newUnderScoreName  + "_" + "(\\d+)", "g");
	poundsignNameRE = new RegExp(newPoundsignName + "#" + "(\\d+)", "g");
	
	templateString = templateString.replace(underScoreNameRE, newUnderScoreNameWtihIndex);
	templateString = templateString.replace(poundsignNameRE, newPoundsignNameWithIndex);
	
	var div = document.createElement('div');
	div.innerHTML = templateString;

	return div;
}

function updateRowTemplateForRecursiveType(recTable, templateString, recursivedepth) {
	var head = recTable.getAttribute("recursiveHead");
	var nodesList = recTable.getAttribute("recursion");
	var dotNameRE = new RegExp(head);
	var underScoreNameRE = new RegExp(head.replace(/\./g, "_"), "g");
	var poundsignNameRE = new RegExp(head.replace(/\./g, "#"), "g");
	var newDotName = head;
	while(recursivedepth > 0) {
		newDotName = newDotName + "." + nodesList;
		recursivedepth--;
	}
	var newUnderScoreName = newDotName.replace(/\./g, "_");
	var newPoundsignName = newDotName.replace(/\./g, "#");
	
	templateString = templateString.replace(dotNameRE, newDotName);
	templateString = templateString.replace(underScoreNameRE, newUnderScoreName);
	templateString = templateString.replace(poundsignNameRE, newPoundsignName);
	
	var div = document.createElement('div');
	div.innerHTML = templateString;
	
	return div;
}

// API for expanding a row in recursive table. 
function expandRecursive(arrayID, templateString) {

	var recTable=dojo.query("#"+arrayID)[0];
	var tableInput = dojo.query("#"+ arrayID+"_INPUT", recTable)[0];
	var recursivedepth = parseInt(tableInput.getAttribute("value"))
	var newRow = updateRowTemplateForRecursiveType(recTable, templateString, recursivedepth);
	
	// Need to clone the row since the add processing causes the scripts to be removed if we don't 
	var cloneRow = newRow.childNodes[0].rows[0].cloneNode(true);
	var scriptsToEval = cloneRow.getElementsByTagName("SCRIPT");   // Need to get the original scripts 
	
	addRowToTable(recTable, cloneRow);
	
	// Execute the scripts
	for(var i=0;i<scriptsToEval.length;i++)  
	{  
		eval(scriptsToEval[i].text);  
	}
	
	tableInput.setAttribute("value", recursivedepth + 1);
}


// Generic function for adding a TR element to a table
function addRowToTable(tableArray, rowToAdd) {
	// In IE, cannot simply do dom manipulations for table so we'll need to do this via the table apis
	var numRows = tableArray.rows.length;
	var newRow=tableArray.insertRow(numRows-1);
	newRow.setAttribute("class", rowToAdd.getAttribute("class"));
	newRow.setAttribute("className", rowToAdd.getAttribute("className")); //IE
	newRow.setAttribute("id", rowToAdd.getAttribute("id"));
	
	var cells = rowToAdd.childNodes;
	var cellIndex = 0;
	var newCells = new Array();
	
	// First create the new cells
	for (var index=0; index<cells.length; index++) {
		if (cells[index].nodeName == "TD") {
			var newCell = newRow.insertCell(cellIndex);
			newCells[cellIndex++] = newCell;
		}
	}
	
	// Now replace them
	for (var index=cells.length - 1; index>=0; --index) {
		if (cells[index].nodeName == "TD") {
			newCells[--cellIndex].parentNode.replaceChild(cells[index], newCells[cellIndex]);
		}
	}
}

// API for adding a row in a array table
function addRow(arrayID, template) {

	// Find the table Array node
	var tableArray=dojo.query("#"+arrayID)[0];
	var createdArray = createNewRow(tableArray, template);
	
	// Need to clone the row since the add processing causes the scripts to be removed if we don't 
	var cloneArray = createdArray.childNodes[0].rows[0].cloneNode(true);
	var scriptsToEval = createdArray.getElementsByTagName("SCRIPT");   // Need to get the original scripts 
	
	addRowToTable(tableArray, cloneArray);
	
	// Execute the scripts
	for(var i=0;i<scriptsToEval.length;i++)  
	{  
		eval(scriptsToEval[i].text);  
	}
	
	updateAddDeleteButtonStatus(tableArray)
}

function updateInputFields(rowTemplate, arrayTemplateID, bindingNameRoot, arrayIndex) {
	var inputFields = dojo.query("INPUT", rowTemplate);
	for (var i=0; i<inputFields.length;i++) {
		if (inputFields[i].getAttribute("name") != null)
			inputFields[i].name = inputFields[i].name.replace(new RegExp(arrayTemplateID.replace(/_/g, "#"), "g"), bindingNameRoot.replace(/_/g, "#"));
			inputFields[i].name = inputFields[i].name.replace(new RegExp(bindingNameRoot.replace(/_/g, "#") + "#" + "(\\d+)", "g"), bindingNameRoot.replace(/_/g, "#") + "#" + arrayIndex);
	}
}


function getOnClickAttribute(deleteButton) {
	var onclickFunction = deleteButton.getAttribute("onclick");
	
	if (typeof(onclickFunction) == "string") {
		return onclickFunction;
	}else {
		var onclickFunctionString = onclickFunction.toString();
		var regExp = new RegExp("{[\\n\\r\\t](.*)[\\n\\r\\t]}", "g");
		if (onclickFunctionString.match(regExp)) {
			return RegExp.$1;
		}
		else
			return onclickFunctionString;
	}
}


// Update Add a row button based on num of actual rows and min/max occurs
function updateArrayAddButtonStatus(tableArray, arrayIndex, maxOccurs ) {
	var addRow = tableArray.tBodies[0].rows[tableArray.tBodies[0].rows.length -1];
	var addRowButton = dojo.query("button", addRow)[0];
	
	if (maxOccurs != 2147483647) {
		if (maxOccurs <= arrayIndex + 1){
			addRowButton.disabled = true;
		}else if (maxOccurs > arrayIndex + 1) {
			addRowButton.disabled = false;
		}
	}
}


// Given an array table, update Add a row button and the delete buttons of
// each row based on num of actual rows and min/max occurs 
function updateAddDeleteButtonStatus(tableArray) {
	var arrayInput =  dojo.query("INPUT:first-child", tableArray)[0];
	var arrayIndex = parseInt(arrayInput.getAttribute("value"));
	var maxOccurs = parseInt(arrayInput.getAttribute("maxoccurs"));
	var minOccurs = parseInt(arrayInput.getAttribute("minoccurs"));
	
	updateArrayAddButtonStatus(tableArray, arrayIndex, maxOccurs);
	
	var allRows = tableArray.rows;
	for (var i=0; i<allRows.length; i++) {
		//Only arrayRow type of rows are needed to update
		if ( (allRows[i].getAttribute("class") != null && allRows[i].getAttribute("class").match(/arrayRow/g)) || 
			 (allRows[i].getAttribute("className") != null && allRows[i].getAttribute("className").match(/arrayRow/g))) {
			var deleteButtonCell = allRows[i].cells[allRows[i].cells.length -1];
			var deleteButton = dojo.query("Button", deleteButtonCell)[0];
			if (deleteButton != null) {
				if (arrayIndex + 1 <= minOccurs) 
					deleteButton.disabled = true;
				else 
					deleteButton.disabled = false;
			}
		}
	}
}


//API for deleting a row in an array table. 
function deleteRow(rowID) {
	var toBeDeleteRow = dojo.query("#"+rowID)[0];
	var arrayTable = toBeDeleteRow.parentNode;
	var rowIndex = toBeDeleteRow.rowIndex;
	var allRows = arrayTable.rows;

	
	if (rowIndex>=0 && rowIndex<allRows.length) {
		arrayTable.deleteRow(rowIndex);	
		
		// correct the delete index
		var arrayInput = dojo.query("INPUT:first-child", arrayTable.parentNode)[0];
		var deleteIndex = parseInt(arrayInput.getAttribute("deleteIndex")) + 1;
		arrayInput.setAttribute("deleteIndex", deleteIndex);
		updateAddDeleteButtonStatus(arrayTable.parentNode);
		
	}
}

//API for correcting the array indexes after deletes were  done. 
function postProcessArrays() {
	// Process all the array tables
	var tableArray = dojo.query("TABLE.arrayTable");
	
	for (var index=0; index<tableArray.length; index++) {
		// Get the input field that has INIT_ARRAY (first input)
		var inputField = dojo.query("INPUT:first-child", tableArray[index])[0];
		var deleteIndex = parseInt(inputField.getAttribute("deleteIndex"))
		if (deleteIndex > 0) {
			// calculate the correct value
			var arrayIndex = parseInt(inputField.getAttribute("value"));
			inputField.setAttribute("value", arrayIndex-deleteIndex);
			
			// cycle all the rows to fix up the input ids
			var allRows = tableArray[index].rows;

			for (var i=0; i<allRows.length; i++) {
				processInputIDs(allRows[i], tableArray[index].id, i);
			}			
		}
	}
}

function processInputIDs(curRow, bindingNameRoot, rowIndex) {
	// cycle all the input fields
    updateInputFields(curRow, bindingNameRoot, bindingNameRoot, rowIndex);

	// process all of the inner array items as indicated by the table tag with class arrayTable
	var innerTableArray = dojo.query("TABLE.arrayTable", curRow);
	for (var i=0; i<innerTableArray.length; i++) {	
		processInnerTableArrayInputIDs(innerTableArray[i], bindingNameRoot, bindingNameRoot, rowIndex);
	}
}

function processInnerTableArrayInputIDs(tableArray, arrayTemplateID, bindingNamePrefix, arrayInx) {

	var bindingNameRoot = tableArray.getAttribute("id").replace (new RegExp(bindingNamePrefix + "_" + "(\\d+)", "g"), bindingNamePrefix + "_" + arrayInx);
	var bindingName = bindingNameRoot+"_0_";
	
	// Change table id
	var oldtableArrayId = tableArray.getAttribute("id");
	tableArray.setAttribute("id", bindingNameRoot);

	// update the Input
	var initArrayInput = dojo.query("INPUT:first-child", tableArray)[0];
	initArrayInput.setAttribute("name", "INIT_ARRAY__"+ bindingNameRoot.replace(/_/g, "#"));
	initArrayInput.setAttribute("id", bindingNameRoot + "_INPUT");
		

	// process all of the inner array items as indicated by the table tag with class arrayTable
	var innerTableArray = dojo.query("TABLE.arrayTable", tableArray);
	for (var i=0; i<innerTableArray.length; i++) {	
		processInnerTableArrayInputIDs(innerTableArray[i], arrayTemplateID, bindingNamePrefix, arrayInx);
	}
}
