/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach view super class js

define(["dojo/_base/lang", "dojo/_base/array", "dojo/dom", "dojo/dom-class", "./CONSTANTS", "./commonHelper", "./domHelper", "./engine", "./collaborationPopup","./logger", "./dataBindingHelper", "./multiBindingListsHelper", "./validationHelper", "./coachViewHelper", "./bindingResolver", "./specificBindingHelper"], function(lang, array, dom, domClass, CONSTANTS, commonHelper, domHelper, engine, collaborationPopup, logger, dataBindingHelper, multiBindingListsHelper, validationHelper, coachViewHelper, bindingResolver, specificBindingHelper) {
	var CoachView = function (context, element, binding, option, readonly) {
		
		if(!!context) {
			this.context = context;
			this.context._init(this, engine.GlobalAssets.system, engine.GlobalAssets.team, element, binding, option, readonly);

			// if the view binds with a list and is framework managed, use following templates to handle the dynamic adding (e.g. list has been inserted a new item)
			this._contentBoxTemplates = [];
			// use this counter to work out the unique id (div id) for the content in the repeating ContentBox
			this._counter = 0;
					
			// hold all the handles of binding
			this._bindingHandles = {};
			
			this._impactedByMultipleLists = undefined;
			this._domIdToBindingListMap = {};
			// we monitor those list objects
			this._bindingStrToListMap = {};	

		    this._changeObject = lang.hitch(this, function(property, oldVal, newVal) {
		    	//Change handler for Stateful objects, calls the subclass change handler
		    	if(this._isReady()) {
			    	var event = {property: property, oldVal: oldVal, newVal: newVal, removedFrom: null, insertedInto: null};
			    	this._change(event);
		    	}
		    });
    
		}		
	}
	
	CoachView.prototype.constructor = CoachView;
	
    CoachView.prototype.view = function() {
    	// Super-class lifeCycle view method
    	var visibility = (!!this.context.options  &&  (!!this.context.options._metadata) && !!(this.context.options._metadata.visibility)) ? this.context.options._metadata.visibility.get("value"): "DEFAULT";
    	if (visibility == "DEFAULT") {
    		visibility = this.context.getInheritedVisibility();
    	}
    	if (visibility == "NONE") {
    		this.context.setDisplay(false);
    	} else {
    		this.context.setDisplay(true);
    	}

    };
    
    // Internal method
    CoachView.prototype._preLoad = function () {
    }
    
    CoachView.prototype.load = function() {
    	// Super-class lifeCycle load method
    };
    
    // Internal method
    CoachView.prototype._preUnload = function() {
    	for(var k in this._bindingHandles) {
			if(!!this._bindingHandles[k]) {
				if(!!this._bindingHandles[k].unbind){
					this._bindingHandles[k].unbind();
				}
				else if(!!this._bindingHandles[k].cancel){
					this._bindingHandles[k].cancel();
				}
			}
    	}
    	this._bindingHandles = null;
    }

    CoachView.prototype.unload = function() {
    	// Super-class lifeCycle unload method
    	// console.debug("unload() is called on viewid " + this.context.viewid + ", Class is " + this.context.element.getAttribute("Class"));
    };
    
    // Internal method to make the CoachView object ready for garbage collection
    CoachView.prototype._postUnload = function() {
    	// considering this.context could be referenced by something, set the content of context to null firstly then set context to null
    	this.context.element = null;
    	this.context.binding = null;
    	this.context.options = null;
    	this.context.subview = null;
    	this.context.bpm = null;
    	this.context._parent = null;
    	this.context._parentView = null;	    	
    	this.context = null;	    	
    }

	CoachView.prototype._change = function(rawEvent, eventType, eventProperty){
		var event = this._formalizeEvent(rawEvent, eventType, eventProperty);
		
		engine.callViewLifeCycleAPI(this, "change", CONSTANTS.ELAPSED_TIME_THRESHOLD_CHANGE, event);
		
		//post change,
		//one thing needs to do is to check if we need to update the visibility of children
		if (event.type == "config" && event.property == "_metadata.visibility"){
			var visibility;
			//go through its subview, if the value is default, then..
			var subviewScope;
			var subviewScopes;
			for (var subviewId in this.context.subview){
				subviewScopes = this.context.subview[subviewId];
				for (var realViewId in subviewScopes){
					subviewScope = subviewScopes[realViewId];
					if (subviewScope.context.options && subviewScope.context.options._metadata && subviewScope.context.options._metadata.visibility.get('value') == "DEFAULT"){						
						subviewScope._change(event);
					}			
				}
			}				
		}		
	}
			
	
	CoachView.prototype._formalizeEvent = function(rawEvent, eventType, eventProperty) {
		// every change event contains following properties
		var event = {property: null, oldVal: rawEvent.oldVal, newVal: rawEvent.newVal};
		
		if(typeof eventProperty != "undefined") {
			event.property = eventProperty;
		}else if(typeof rawEvent.property != "undefined") {
			event.property = rawEvent.property;
		}
		
		if(typeof eventType != "undefined") {
			event.type = eventType;
		}else if(typeof rawEvent.type != "undefined") {
			event.type = rawEvent.type;
		}

		if(typeof rawEvent.insertedInto != "undefined") {
			event.insertedInto = rawEvent.insertedInto;
		}
		if(typeof rawEvent.removedFrom != "undefined") {
			event.removedFrom = rawEvent.removedFrom;
		}
		if(typeof rawEvent.path != "undefined") {
			event._path = rawEvent.path;
		}
		return event;
	}
	
	// general comment - there are 3 levels of change for dataBinding:
	// 1. The value of resolved binding object is changed. Namely, same resolved binding object is still used. 
	//    When it happens, _bindingChanged() is invoked, event._isResolvedVarChanged doesn't exist.
	// 2. The resolved binding object itself is replaced. As by-product, the value of resolved binding object is also determined as changed.
	//    When it happens, _bindingChanged() is invoked, event._isResolvedVarChanged references to the new resolved object.
	// 3. A new binding String is assigned on this view instance. e.g. old one is "local.listA[2].p", new one is "local.listA[1].p", because "local.listA[0]" is deleted.
	//    This scenario is handled in _setRtDataBindingStr(). Basically rebind for "_bindingChanged" (via specificBindingHelper), 
	//    get a new resolved binding object (main purpose is to make sure _objectPath in resolved object is up to date).
	//    Since the only difference between new/old binding string is the index, no need to rebind for "_listUpdated" and "_watchRtDataBinding".	
	CoachView.prototype._bindingChanged = function(event, calledByRefreshView){
		if(event._isResolvedVarChanged) {
			// resolved binding object is completely replace. example: this view binds to local.a.b, when resolved object of local.a gets a new value, the resolved object of local.a.b becomes completely new.
			this.context.binding = event._newResolvedVar;
		}
						
		if(this.context.binding && this.context.binding.get("value") instanceof com.ibm.bpm.coach.List){
			// the change is on the list. as the result, this.context.binding.get("value") might point to different object.
			// because delete this.childrenCache[name] is called in set(name, value) in Stateful.js
			// so need to rebind "_listUpdated" and "_watchRtDataBinding" if they exist.
			var bindingHandle = this._bindingHandles["_listUpdated"];
			if (bindingHandle) {
				// if bindingHandle exists, there is fwk-managed contentBox (see _buildChildren() for detail).
				// need to re-bind and adjust the instances of contextBox
				if(bindingHandle.unbind){
					bindingHandle.unbind();
				}
				// bind on new object
				this._bindingHandles["_listUpdated"] = this.context.binding.get("value").bindAll(lang.hitch(this, this._listUpdated));
			}
			
			bindingHandle = this._bindingHandles["_watchRtDataBinding"];
			if (bindingHandle) {
				if(bindingHandle.unbind){
					bindingHandle.unbind();
				}
				// bind on new object
				this._bindingHandles["_watchRtDataBinding"] = this.context.binding.get("value").bindAll(lang.hitch(this, this._watchRtDataBinding));
			}
			
			// when _refreshView() calls _bindingChanged(), no need to execute following block because _refreshView() has updated runtime data binding strings and rebuild the repeated contentBoxes.
			if(!calledByRefreshView) {
				// If old list and new list have same length, need to do nothing.
				// Because the view instances that bind to list element (e.g local.listA[0], local.listB[1].p) will receive their notification and update accordingly -
				// resolved binding object, rebind (e.g. for _listUpdated callback) etc.
				// If the length is not different, need to delete all the repeated contentBox and re-create them.			
				if(event.oldVal && event.oldVal.items && event.newVal && event.newVal.items && event.oldVal.items.length == event.newVal.items.length) {
				} else {
					this._rebuildRepeatedContentBoxs();
				}
			}			
		}
		
		this._highlightBindingIssue();
		
		// event object should not be modified.
		this._change(event);
	}
	
	// predefined option
	CoachView.prototype._metadataChanged = function(event, metadataKey){
		if(event._isResolvedVarChanged) {
			this.context.options._metadata[metadataKey] = event._newResolvedVar;
		}

		// event object should not be modified.
		var eventType = "config";
		// event.property already exists. However, event.property means what really has changed. 
		// It could be the property of local variable, or property of config variable. The event we are preparing here
		// is for the change event handler. If a coach view instance's _metadata.visibility binds to local.varA,
		// event's property should be "_metadata.visibility" instead of "varA" when it's delivered via change event handler.
		var refinedEventProperty = "_metadata." + metadataKey;

		this._change(event, eventType, refinedEventProperty);
	}
	
	// normal option
	CoachView.prototype._optionChanged = function(event, optionKey){
		if(event._isResolvedVarChanged) {
			this.context.options[optionKey] = event._newResolvedVar;
		}
		// event object should not be modified.
		this._change(event, "config", optionKey);
	}
	
	CoachView.prototype._setRtDataBindingStr = function(newStr) {
		var oldRtBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
		if(oldRtBindingStr != newStr) {
			dataBindingHelper.rtDataBinding._set(this.context.element, newStr);
			
			// re-resolve the bindingStr to make sure this.context.binding is up to date
			this.context.binding = newStr ? bindingResolver.resolveBinding(newStr) : null;
			this._registerCallBackForDataBinding(newStr);
			this._highlightBindingIssue();
		}
	}
	
	CoachView.prototype._registerCallBackForDataBinding = function(bindingStr) {
		this._registerCallBack(bindingStr, this._bindingChanged, this, "_bindingChanged");
	}
	
	CoachView.prototype._registerCallBackForOptions = function() {
	    var options = this.context.options;
		for(var optionKey in options) {	
		    if (optionKey == "_metadata"){
				for (var metadataKey in options[optionKey]){
					var scopeObject = {viewInstance: this, metadataKey: metadataKey};
					var callbackFunc = function(event) {
						// "this" is scopeObject defined above
						this.viewInstance._metadataChanged(event, this.metadataKey);
					}
					var optionUniqueName = "_metadata." + metadataKey;
					var staticOptionStr = options[optionKey][metadataKey].staticOptionStr;
					var bindingStr = options[optionKey][metadataKey].bindingStr;
					if(staticOptionStr) {
						this._registerCallBack(staticOptionStr, callbackFunc, scopeObject, optionUniqueName);
					} else if(bindingStr) {
						this._registerCallBack(bindingStr, callbackFunc, scopeObject, optionUniqueName);
					}
				}
			}
			else{			
				var scopeObject = {viewInstance: this, optionKey: optionKey};
				var callbackFunc = function(event) {
					this.viewInstance._optionChanged(event, this.optionKey);
				}
				var staticOptionStr = options[optionKey].staticOptionStr;
				var bindingStr = options[optionKey].bindingStr;
				if(staticOptionStr) {
					this._registerCallBack(staticOptionStr, callbackFunc, scopeObject, optionKey);
				} else if(bindingStr) {
					this._registerCallBack(bindingStr, callbackFunc, scopeObject, optionKey);
				}
			}
		}
	}

	CoachView.prototype._registerCallBack = function(bindingStr, callbackFunc, scopeObject, handleName) {
		// unbind old handler
		var oldBindingHandler = this._bindingHandles[handleName];
		if(oldBindingHandler) {
			if(oldBindingHandler.unbind){
				try{
					oldBindingHandler.unbind();
				}catch(ex) {
					var errorMsg = "CoachView._registerCallBack() caught Exception when unbind " + bindingStr + ", Exception: " + ex;
					commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
				}
			}
		}
		
		if(bindingStr) {
			this._bindingHandles[handleName] = specificBindingHelper.bind(bindingStr, callbackFunc, scopeObject);
		}
	}
	
    CoachView.prototype.change = function(event) {
		// Default behavior.  Call view method if subview do not have any change logic
    	this.view();
    };	
	
    CoachView.prototype.collaboration = function(event) {
        
		var node = dom.byId(event.domId);
		
		/* now engine guarantee that event.domId is the right one. So we don't need to have following logic here any more. 
		if(!node || !dom.isDescendant(node, this.context.element)) {
			// notice that sometimes event.domId (detected in editor side) is the id generated by dojo, that id may not exist in the dom node of this coachView in viewer side.
			// For those cases, use coachView's dom node instead.
			node = this.context.element;
		}
		*/
		
    	if(event.type == "focus") {
			if(!this.originalOutline && !!node.style.outline && node.style.outline != "") {
				this.originalOutline = node.style.outline;
			}
			// node.style.border = "solid " + event.outlineColor;
			node.style.outline = "1px solid " + event.outlineColor;
			
			collaborationPopup.showCollaboratorInfo(node, event.color, event.bgColor, event.message, this.context.element);				
    	} else if(event.type == "blur") {
 			if(!!this.originalOutline) {
				node.style.outline = this.originalOutline;
				delete this.originalOutline;
			} else {
   				node.style.outline = "none";
			}
    		
    	} else if(event.type == "mouseOn") {
			collaborationPopup.showCollaboratorInfo(node, event.color, event.bgColor, event.message, this.context.element);
    	} else if(event.type == "mouseOff") {
			collaborationPopup.hideCollaboratorInfo(node);	    		
    	} else if(event.type == "dataChange") {
			collaborationPopup.showDataChange(node, event.message, this.context.element);
    	}    	
    };
    
	/**
	The default implementation provided by framework does nothing, it's each view's responsibility 
	to decide what to do when it receives the validation event.
	
	If validation errors occur during the process of boundary event, coach framework calls validate(event) 
	on all the involved views. When both parent and children views are involved, parent view is called 
	prior to children views. Framework guarantees that involved views are only called once per boundary event.
	
	Details about the input parameter event:		
	event.type can be either "error" or "clear".
	
	If event.type is "clear", event doesn't have any other property.		
	If event.type is "error", event has 2 more properties as following:
	
	- event.errors: list of errors. This view receives those errors because its data binding or part of data binding 
	  has validation errors. Each error object has 3 properties: binding, message and view_dom_ids.		
		a) binding is the path relatives to the data binding of current view.
		b) message is the localized message.
		c) view_dom_ids is the list of DOM id of the views that are either current view or the descendant views of current view and also receive the error message.		
	- errors_subcription: list of errors. This view receives those errors because it has called subscribeValidation() 
	  and some of its descendant views have validation errors on them. If the error of one view has been described 
	  in event.errors, it won't show in errors_subcription. Each error object has 2 properties: view_dom_id and messages.		
		a) view_dom_id is the DOM id of the view that contains the error.
		b) messages is a list of the localized messages.
	
	event example: 
	{		
	   type: "error",
	   errors: 
	           [
	            {
					binding: "ownerBirthday.year"
					message: "2030 is after current year of 2012",
					view_dom_ids: ["domId_PhoneBookView", "domId_yearView"]
	            }
	          ]
	   errors_subcription: 
	          [
	            {
					view_dom_id: "domId_advertisementView",
					messages: ["something wrong"]
	            }
	          ]
	}
	
	Since coach framework calls validate() on all involved views synchronously, the execution of this method 
	is expected to complete shortly.
	
	*/
    CoachView.prototype.validate = function(event) {
    }
    
	CoachView.prototype._buildChildren = function() {
		var currentViewScope = this;
		var currentDiv = currentViewScope.context.element;
	
		var binding = currentViewScope.context.binding;
		var bindToArray = (!!binding && binding.get("value") instanceof com.ibm.bpm.coach.List);			

		// view could hold subviews directly, those direct subview won't repeat even if parent view binds with a list
		var directViewList = domHelper.shallowQueryByClass(currentDiv, engine.CONSTANTS.COACH_VIEW, true);
		var oneDirectView;
		array.forEach(directViewList, function(item, index) {
			oneDirectView = engine.initView.apply(engine.GlobalAssets, [item, currentViewScope]);
		});			
	
		// find the direct ContentBox under this view
		var immediateContentBoxList = domHelper.shallowQueryByClass(currentDiv, engine.CONSTANTS.CONTENT_BOX, true);
		
		if(bindToArray) {
			if(!!binding && binding.get("value").length() == 0) {
				// view binds an empty list, no repeat will occurs, so no subview will call _addRelatedList() to report.
				// we have to simulate to collection information for the multiple binding list support here.
				this._simulateRepeatingToDetectMultiLists(immediateContentBoxList);
			}

			// Register listener for List changes to watch if rtDataBinding is dirty. Do it for both view managed and framework managed. 
			var oneHandler = binding.get("value").bindAll(lang.hitch(currentViewScope, currentViewScope._watchRtDataBinding));
			this._bindingHandles["_watchRtDataBinding"] = oneHandler;
		}			
		
		// only care about the framework managed ContentBox. view managed ContentBox will be built by its parent view.
		var fwkManagedContentBoxs = new Array();
		for(var k=0; k<immediateContentBoxList.length; k++) {
			if(!engine.isViewManagedContentBox(immediateContentBoxList[k])) {
				// Framework managed
				fwkManagedContentBoxs[fwkManagedContentBoxs.length] = immediateContentBoxList[k];
			}
		}
		
		if(fwkManagedContentBoxs.length == 0) {
			// no Framework managed contentBox, so job is done
			return;
		}
		
		if(bindToArray) {
			// save the template in the viewScope
			this._contentBoxTemplates = fwkManagedContentBoxs;
			
			// keep the templates in the dom since we need them to indicate where to insert 
			// when dynamically adding from nothing (the list was empty before adding).			
			// Hide those templates by changing its css class from "ContentBox" to "ContentBoxTemplate" so they don't take the space
			for(var i=0; i<this._contentBoxTemplates.length; i++) {
				domClass.replace(this._contentBoxTemplates[i], engine.CONSTANTS.CONTENTBOX_TEMPLATE, engine.CONSTANTS.CONTENT_BOX);
			}
			if(!!binding) {
				// Register listener for List changes
				var bindingHandle = binding.get("value").bindAll(lang.hitch(currentViewScope, currentViewScope._listUpdated));
				this._bindingHandles["_listUpdated"] = bindingHandle;
				
				// need to repeat the framework managed contentBox based on the data-binding						
				for(var k=0; k<binding.get("value").length(); k++) {
					this._repeatContentBoxs(k, false);
				}
			}
		} else {
			for(var i=0; i<fwkManagedContentBoxs.length; i++) {
				engine.buildContentBox(currentViewScope, fwkManagedContentBoxs[i]);
			}
		}	
	}
	
	// when binding list is empty, or ViewContext.containDiffLoopingContext() is called before view is fully initialized,
	// have to simulate to collect enough information to decide if this view involves multiple lists
	// this._impactedByMultipleLists is assigned in the function
	CoachView.prototype._simulateRepeatingToDetectMultiLists = function(immediateContentBoxList) {
		this._impactedByMultipleLists = false;

		if(multiBindingListsHelper.isBindingObjectList(this)) {
			if(!immediateContentBoxList) {
				immediateContentBoxList = domHelper.shallowQueryByClass(this.context.element, engine.CONSTANTS.CONTENT_BOX, true);			
			}
			
			for(var k=0; k<immediateContentBoxList.length; k++) {
				var oneDiv = immediateContentBoxList[k].cloneNode(true);						
				engine.adjustIdAndBinding(oneDiv, this, 0);		
				// this.context.element.parentNode.appendChild(oneDiv); // only for debug purpose
			}

	    	var listBindingStrArray = multiBindingListsHelper.analyzeRelatedListBindingStrs(this);
	    	this._impactedByMultipleLists = (listBindingStrArray.length > 0);
		}
	}
	
	// Repeat all the contentBox templates (of framework managed contentBoxs) for the specified bindingIndex.
	CoachView.prototype._repeatContentBoxs = function(bindingIndex, callView) {
		for(var i=0; i<this._contentBoxTemplates.length; i++) {
			var templateNode = this._contentBoxTemplates[i];
			var oneDiv = templateNode.cloneNode(true);
			// ContentBox template's css class is "ContentBoxTemplate", change it to "ContentBox"						
			domClass.replace(oneDiv, engine.CONSTANTS.CONTENT_BOX, engine.CONSTANTS.CONTENTBOX_TEMPLATE);
			oneDiv.setAttribute(engine.CONSTANTS.TEMPLATE_ID, templateNode.id);

			// content boxes are not always directly under their CoachView's element
			templateNode.parentNode.insertBefore(oneDiv, templateNode);

			engine.adjustIdAndBinding(oneDiv, this, bindingIndex);
			var newViewList = new Array();
			engine.buildContentBox(this, oneDiv, newViewList);
			
			if(callView == true) {
				for(var k=0; k<newViewList.length; k++) {
					engine.callView(newViewList[k]);
				}
				
				// check if those newly created view instance has special visibility setting
				if(!!engine.GlobalAssets.coach) {
					for(var i=0; i<newViewList.length; i++) {
						engine.GlobalAssets.coach.updateVisibilityIfApplicable(newViewList[i]);
					}
				}

				// check if it's needed to deliver the validation event to the newly created views
				var foundNewValidationEvent = false;
				for(var j=0; j<newViewList.length; j++) {
					foundNewValidationEvent = foundNewValidationEvent || validationHelper.validateView(newViewList[j]);
				}
				if(foundNewValidationEvent) {
					// the new view or its descendant views own validation event
					// just tell engine to deliver the new validation event. It's up to engine to decide when to deliver
					engine.deliverValidationEvents();
				}
			}
		}
	}
	
	// If bindingIndex is specified, delete the framework managed contentBox instances according to the binding index
	// If bindingIndex is not specified, delete all framework managed contentBox instances
	CoachView.prototype._deleteContentBoxs = function(bindingIndex) {
		var contentBoxGroup = this._findContentBoxGroup(bindingIndex);
		for(var k=0; k<contentBoxGroup.length; k++) {
			// delete the contentBox 
			engine.deleteContentBox(contentBoxGroup[k], this);
			// content boxes are not always directly under their CoachView's element
			contentBoxGroup[k].parentNode.removeChild(contentBoxGroup[k]);
		}
	}
	
	CoachView.prototype._watchRtDataBinding = function(changeEvent) {			
		if(changeEvent.removedFrom >= 0 && changeEvent.insertedInto >= 0 && changeEvent.removedFrom == changeEvent.insertedInto) {
			// event is "set", we don't care about it
			return;
		} else {
			if(changeEvent.removedFrom >= 0) { // one item has been removed from the list.	
				// notify validationHelper
				var dataBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
				
				validationHelper.listItemRemoved(dataBindingStr, changeEvent.removedFrom, changeEvent.oldVal);
				
				// do 2 things:
				// 1. mark templates dirty. The reason to only mark templates dirty instead of correcting them right way is performance, 
				//    dirty templates may not be used at all. Let the dirty templates accumulate, correct them when those templates are needed.
				// 2. update the runtime binding path immediately.
				var contentBoxGroup = this._findContentBoxGroup(changeEvent.removedFrom, true);
				var affectedSubviewArray = [];
				for(var k=0; k<contentBoxGroup.length; k++) {
					coachViewHelper.findSubviews(affectedSubviewArray, this, contentBoxGroup[k]);
				}
				
				var bindingStrToListMap = {};
				for(k=0; k<affectedSubviewArray.length; k++) {
					// mark template dirty
					affectedSubviewArray[k]._markTemplatesDirty();
					// correct the runtime binding path if needed
					dataBindingHelper.updateAllRtDataBinding(affectedSubviewArray[k], bindingStrToListMap);					
				}
			}
			
			if(changeEvent.insertedInto >= 0) {
				// when we support insert in the middle later, we need to do similar thing like delete
			}	
		}	
    };
	

    CoachView.prototype._listUpdated = function(changeEvent) {
    	/*
		console.debug("list change detected, object: ", changeEvent.object, 
		"removedFrom: ",changeEvent.removedFrom, 
		"insertedInto: ",changeEvent.insertedInto);
		*/
		
		if(changeEvent.removedFrom >= 0 && changeEvent.insertedInto >= 0 && changeEvent.removedFrom == changeEvent.insertedInto) {
			// don't need to do anything. All the impacted view instances will be updated automatically: replace the binding object if needed, fire change event.
		} else {
			// if both changeEvent.removedFrom and changeEvent.insertedInto exist, execute both
			if(changeEvent.removedFrom >= 0) { // one item has been removed from the list.	
				this._deleteContentBoxs(changeEvent.removedFrom);
			}
			
			if(changeEvent.insertedInto >= 0 && this._contentBoxTemplates.length != 0) {
				if(this._isTemplatesDirty) {
					// templates are dirty. Because some list item has been deleted e.g. my binding is local.listA[2].listB, but local.listA[1] was deleted.
					// As the result, my templates are dirty. Meanwhile, my relative that binds to local.listA[0].listB is still fine.
					// hard to correct the dirty templates. The meaningful way is to delete the branch that contains the dirty templates, force the re-build.
					// Once the related views are rebuilt, the newly inserted item is handled automatically.
					// Find a parent view that has non-dirty templates, then rebuild the contentBoxs from there.
					var parentView = this.context.parentView();
					while(parentView) {
						if(parentView._contentBoxTemplates.length != 0 && !parentView._isTemplatesDirty) {
							// find the healthy view
							break;
						}
						parentView = parentView.context.parentView();
					}
					if(parentView) {
						parentView._rebuildRepeatedContentBoxs();
					} else {
						// this branch should not be reached. Anyway, have it to be safe. If it does happen, refresh entire branch starting from the topLevel view.
						var warningMsg = "CoachView._listUpdated(), Rare scenario: The templates of view instance (DOM Id " + this.context.element.id 
							+ ") is dirty. However, coach framework failed to find a view that has healthy templates in its parental chain. Coach framework will call refreshView() on the topLevel view.";
						commonHelper.consoleLog(warningMsg, commonHelper.WARN);
						this._getTopLevelView()._refreshView(true);
					}
				} else {
					// now we only support to add to the end of the list, figure out the proper position to insert when list supports to add item at any position.
					this._repeatContentBoxs(changeEvent.insertedInto, true);
				}
			}	
		}	
    };
    
    // If bindingIndex is specified and continueToEnd is not true, find one contentBox instance for each template based on the bindingIndex.
	// If bindingIndex is specified and continueToEnd is true, for each template, find the contentBox instances whose position is equal or greater than bindingIndex.
    // If bindingIndex is NOT specified, find all the contentBox instances for all the contentBox templates owned by this CoachView
    CoachView.prototype._findContentBoxGroup = function(bindingIndex, continueToEnd) {
		var contentBoxGroup = new Array();
    	var rootNode = this.context.element;
		for(var i=0; i<this._contentBoxTemplates.length; i++) {
			// find the contentBox who was created using current template
			var contentBoxList = rootNode.querySelectorAll(' .' + engine.CONSTANTS.CONTENT_BOX + '[' + engine.CONSTANTS.TEMPLATE_ID + '=' + this._contentBoxTemplates[i].id + ']'); 
			if(bindingIndex == undefined) {
				// all instances qualify
				for(var k=0; k<contentBoxList.length; k++) {
					contentBoxGroup.push(contentBoxList[k]);
				}
			} else if(contentBoxList.length > bindingIndex) {
				if(continueToEnd) {
					for(var i=bindingIndex; i<contentBoxList.length; i++) {
						contentBoxGroup.push(contentBoxList[i]);
					}
				} else {
					contentBoxGroup.push(contentBoxList[bindingIndex]);
				}
			}
		}
		return contentBoxGroup;
    }
    
    CoachView.prototype._addRelatedList = function(viewDomId, relatedList) {
    	this._domIdToBindingListMap[viewDomId] = relatedList;
    }
    
    CoachView.prototype._monitorRelatedListsIfNeeded = function() {
    	this._impactedByMultipleLists = false;
    	
		if(!multiBindingListsHelper.isBindingObjectList(this)) {
    		return;
		}

		var myDataBindingObject = this.context.binding.get("value");			
    	var listBindingStrArray = multiBindingListsHelper.analyzeRelatedListBindingStrs(this);
    	if(listBindingStrArray.length > 0) {
    		this._impactedByMultipleLists = true;
    		var listStr;
			for(var k=0; k<listBindingStrArray.length; k++) {
				listStr = listBindingStrArray[k];
    			if(!this._bindingStrToListMap[listStr]) {
    				var resolvedList = bindingResolver.resolveBinding(listStr).get("value");
    				this._bindingStrToListMap[listStr] = resolvedList;
    				
					var bindingHandle = resolvedList.bindAll(lang.hitch(this, this._monitorListsLengthChange));
					this._bindingHandles["_monitorLengthOf_" + listStr] = bindingHandle;
    			}
			}
			this._bindingHandles["_monitorLengthOf_myOwnBinding"] = myDataBindingObject.bindAll(lang.hitch(this, this._monitorListsLengthChange));		    		
    		multiBindingListsHelper.processMonitoredLists(this, engine);
    	}
    }
    
    CoachView.prototype._monitorListsLengthChange = function(changeEvent) {
    	// we only care the length change, so either add or delete, not set
    	if(changeEvent.removedFrom != changeEvent.insertedInto) {
    		multiBindingListsHelper.processMonitoredLists(this, engine);
    	}
    }
    
    CoachView.prototype._checkIfImpactedByMultipleLists = function() {
		if(this._impactedByMultipleLists == undefined) {
		
			// this._impactedByMultipleLists is assigned in _simulateRepeatingToDetectMultiLists() or _monitorRelatedListsIfNeeded()
			// this._impactedByMultipleLists is still undefined, means that _monitorRelatedListsIfNeeded() is not called. So this view is not fully rendered yet.
			// As the result, maybe this view has some List binding that has not be discovered yet. 
			// So, have to trigger framework to evaluate the contentBox templates of this view to discover all the related List bindings.
			
			// have to simulate to collection information for the multiple binding list support here.
			this._simulateRepeatingToDetectMultiLists();
		}
		
		return this._impactedByMultipleLists;
    }
    
    CoachView.prototype._isReady = function() {
    	return !!this.context;
    }
    
	CoachView.prototype._getTopLevelView = function() {
		var aView = this;
		while(!!aView.context.parentView()) {
			aView = aView.context.parentView();
		}
		return aView;
	}
    
	// recursively check this view and its subviews, if there is templates for framework managed contentbox, make them dirty.
	CoachView.prototype._markTemplatesDirty = function() {
		if(this._contentBoxTemplates.length > 0) {
			// owns templates for framework managed contentBox
			this._isTemplatesDirty = true;
		}
		
		for (var subviewId in this.context.subview){
			for (var subviewDomId in this.context.subview[subviewId]){
				this.context.subview[subviewId][subviewDomId]._markTemplatesDirty();
			}
		}				
	}

	CoachView.prototype._rebuildRepeatedContentBoxs = function() {
		// only care about the templates of framework managed contentBox.
		if(this._contentBoxTemplates.length != 0) {
			//delete all framework managed content boxes				
			this._deleteContentBoxs();
			
			//repeat all framework managed content boxes: 		
			for(var k=0; k<this.context.binding.get("value").length(); k++) {
				this._repeatContentBoxs(k, true);
			}
		}
	}
	
	CoachView.prototype._refreshView = function(updateRtDataBindingStr) {
		if(!this._isReady()) {
			// this view instance is supposed to be recycled.
			return;
		}
	
		// step 1: delete all framework managed content boxes (owner view binds to list, so doesn't include the non-repeating contentBox)
		if(this._contentBoxTemplates.length != 0) {
			//delete all framework managed content boxes				
			this._deleteContentBoxs();
		}
		
		// step 2: update runtime Data Binding string
		if(updateRtDataBindingStr) {
			var topLevelParentView = this._getTopLevelView();
			// call updateAllRtDataBinding() on topLevelParentView. Actually all runtime binding path should be up-to-date. Do it just to be safe.
			dataBindingHelper.updateAllRtDataBinding(topLevelParentView);
		}
		
		// step 3: re-resolve dataBinding, re-bind on it
		var oldPlainBindingValue = engine.getPlainValue(this.context.binding);
		var propertyName = this.context.binding ? this.context.binding.property: null;
		var rtBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
		this.context.binding = rtBindingStr ? bindingResolver.resolveBinding(rtBindingStr) : null;
		this._registerCallBackForDataBinding(rtBindingStr);
		
		// step 4: re-resolve configure options, re-bind on them
		bindingResolver.resolveOptionsBinding(this.context.options);
		this._registerCallBackForOptions();
		
		// step 5: re-create framework managed content boxes (owner view binds to list)
		if(this._contentBoxTemplates.length != 0) {
			//repeat all framework managed content boxes: 		
			for(var k=0; k<this.context.binding.get("value").length(); k++) {
				this._repeatContentBoxs(k, true);
			}
		}

		// step 6: recursively refresh subviews
		var childView;
		for (var viewId in this.context.subview) {		    
			var subviews = this.context.subview[viewId];
			for(var divId in subviews) {
				childView = subviews[divId];
				if(childView && childView._isReady()) {
					childView._refreshView(false);
				}
			}
		}
		
		// step 7: trigger change() for dataBinding
		var newPlainBindingValue = engine.getPlainValue(this.context.binding);
		// when both newPlainBindingValue and oldPlainBindingValue are null (likely because the bindingStr is ""), commonHelper.isValuesEqual() returns commonHelper.UNKNOWN (in the sake of backward compatibility).
		// however, make no sense to fire change event for this scenario.
		if((newPlainBindingValue != null || oldPlainBindingValue != null) && commonHelper.isValuesEqual(newPlainBindingValue, oldPlainBindingValue) != commonHelper.YES) {
			// if failed to get propertyName from old binding, try to see if new binding has it
			if(!propertyName) {
				propertyName = this.context.binding ? this.context.binding.property: null;
			}
			var event = {property: propertyName, oldVal: oldPlainBindingValue, newVal: newPlainBindingValue, path: rtBindingStr};
			this._bindingChanged(event, true);
		}
		
		// step 8: trigger change() for configure options
		engine.notifyDataConfigBindingChange(this);
	}	
	
	CoachView.prototype._highlightBindingIssue = function() {
		if(!com_ibm_bpm_global.preference.highlightUnresolvedBinding) {
			return;
		}
		var dataBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
	    if(!!dataBindingStr) {
			var dataBinding = this.context.binding;
	    	// if dataBinding doesn't exist,
	    	// or dataBinding.boundObject is list, but can't get value from it (index is out of bound). 
	    	if(!dataBinding ||(dataBinding.boundObject instanceof com.ibm.bpm.coach.List && (dataBinding.get("value") == null || typeof dataBinding.get("value") == "undefined"))) {
		    	// we failed to resolve binding for this view. so highlight it here to let user be aware 
		    	this.context.element.style.outline = "1px dotted red";
		    } else {
				// remove the highlight which indicates binding issue if the issue is gone.
				if(this.context.element.style.outline == "1px dotted red") {
					this.context.element.style.outline = "none";
				}
			}
	    }
	}	

	CoachView.prototype._getBindingStrings = function() {
		var bindingStrArray = new Array();
		// make sure rtDataBinding is up-to-date before we call it	
		var dataBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
		bindingStrArray.push(dataBindingStr);
				
		return bindingStrArray;
	}

    return CoachView;
	
});

