/*
Licensed Materials - Property of IBM
5725-C95
 Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach data list class js

define(["dojo/_base/declare", "dojo/store/Memory", "dojo/store/Observable", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/json", "dojo/Stateful", "./commonHelper", "./dataBindingHelper", "./Stateful", "require", "./controller", "./logger", "./templateStore"], 
		function(declare, memory, observable, lang, array, json, dojoStateful, commonHelper, dataBindingHelper, stateful, require, controller, logger, templateStore) {
	//List object that uses dojo.store.Observable to "watch" the list for updates.
	//Maintains an internal representation of the list to map ids, so the user of this list does not need to supply unique ids
	//for each list item
	return declare("com.ibm.bpm.coach.List", [dojoStateful], {
		childrenCache: null,
		dataStore: null,
		propertyList: null,
		//index: 0,  //index is the index for the next available spot
		managedList: null,
		indexToId: null, // maitains a map from index to id. Think about remove from middle case; remove once, and remove again
		items: null,
		_systemCallbackHandle: null,
		_systemCallbackPropertyHandle: null,	
		
		//
		//1. bindAll: right now, it regisgter on the property list, should that also be on all properties
		//2. listAllSelectedIndices is plain array object, which are agreed.
		// following 4 properties for selection, they can be selected in design time for binding
		listAllSelectedIndices: null, //  (array) - return an array of all selected indices
		listSelectedIndex: null, // (int) - return the first element of listAllSelectedIndices
		listAllSelected: null, // (array) - return an array of selected elements (derived from listAllSelectedIndices and the array)
		listSelected: null, // (object) - return the selected element (derived from listSelectedIndex and the array)
		_masterList : null, // if the List is for listAllSelected, then the list have a masterList, which is the source of the data for this list
		
		constructor: function(listObj, path) {
		    
			this.childrenCache = {};
			this.managedList=[];
			this.indexToId = {};
			this._objectPath = path;
		
			//Store the items and populate other properites if this is an object contains items and listAllSelectedIndices
			if (!listObj){
				this.items = [];
			}
			else if (!!listObj.items && (!!listObj.listAllSelectedIndices || (!!listObj["selected"] && !!listObj["@metadata"]))){
				this.items = listObj.items;
				//defect 121645 - lists returned from AJAX services use "selected" instead of listAllSelectedIndecies
				if (!!listObj["listAllSelectedIndices"]) {
					this.listAllSelectedIndices = listObj.listAllSelectedIndices;
				} else if (!!listObj["selected"]) {
					this.listAllSelectedIndices = listObj.selected;
				}
				this._setDerivedProperties(true);
				
				// this.te is undefined when listObj.te doesn't exist. It's ok, we can try to figure out te when it is time to use it to create new element
				// when list is constructed as the response to server, likely litObj doesn't have "te" property
				if(listObj.te) {
					this.te = listObj.te;
					templateStore.save(path, listObj.te);
				}
			}
			else {
				//happens when the list is for ListAllSelected
				this.items = listObj;
			}
			
			//populate the internal managedList object with generated ids
			for (var index in this.items) {		
				this.managedList.push({"bpm_id":  this.updateId(index), "listItem": this.items[index]});
			};
			this.dataStore = observable(new memory({data: this.managedList, idProperty:"bpm_id"}));
			this.propertyList = this.dataStore.query();

			//register the system callback
			this._systemCallbackHandle = null,
			this._systemCallbackPropertyHandle = null,
			this.enableSystemNotifications("all");
		},
		
		disableSystemNotifications: function (property) {
			if (!property)
				this._systemCallbackHandle.cancel();
				
			if(!!this._systemCallbackPropertyHandle && property == "listAllSelectedIndices"){
				this._systemCallbackPropertyHandle.unwatch();
			}
		},
		
		enableSystemNotifications: function (property) {
			//set system call back, for notifications to controller
			if (!property || property == "all"){
				this._systemCallbackHandle = this.registerCallback(null, lang.hitch(this, function(changedObject, removedFrom, insertedInto) {
											controller.systemListChangeHandler(this._objectPath, changedObject, removedFrom, insertedInto);
										}));
			};
			
			//only register for listAllSelectedIndices
			if ( property == "all" || property == "listAllSelectedIndices"){
				this._systemCallbackPropertyHandle = this.registerCallback("listAllSelectedIndices", lang.hitch(this, function(property, oldValue, newValue) {
											controller.systemChangeHandler(this._objectPath, property, newValue);
										}));	
            }										
		},
		
		updateId: function(index){
			
		    var id;
            if (this.length() == 0){
				id = 1;
			}
			else{
				id = this.indexToId[this.length()-1] + 1;
			}			
			this.indexToId[index] = id;
			return id;
		},
				
		//registers a callback for this object that will be invoked if this object is modified
		registerCallback: function (property, callback) {
			var handle;
			if (callback){
				if (!property){				
				   handle =  this.propertyList.observe(callback, true);
				   handle.unbind = function (){						
						this.cancel();
					}
				}
				else {//if (property == "listSelected" || property  == "listAllSelectedIndices"		|| property == "listSelectedIndex" || property == "listAllSelected"){
					handle = this.watch(property, callback);
					
					handle.unbind = function (){						
						this.unwatch();
					}
					
				}
			}
			return handle;
		},
		
		//puts a new object on the list, replacing object at index
		put: function(index, object, options, disableSystemNotification) {
			logger.log("List", "put", "List: " + this._objectPath + ", index: " + index + ", object: " + object + ", options: " + options, logger.FINER);
		    if (index < 0 || index > this.length() - 1){
				return;
			}						   
			
			// check if it makes sense to accept the put request
			var currentPlainValue = dataBindingHelper.getPlainValue(this, index);
			if(commonHelper.isValuesEqual(currentPlainValue, object) == commonHelper.YES) {
				// new value is equal to current value, ignore this put request
				logger.log("List", "put", "Ignore this put request because new value is equal to current value. value = " + currentPlainValue, logger.FINE);
				return;
			}

		    //update items too
			if (this.items){
				this.items.splice(index, 1, object);		
			}	
			if (!!this.childrenCache[index]){
				delete this.childrenCache[index];
			}

			var result = null;
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				result = this.dataStore.put({"bpm_id": this.indexToId[index], listItem: object}, options); 
				this.enableSystemNotifications();
			} else {
				result = this.dataStore.put({"bpm_id": this.indexToId[index], listItem: object}, options); 
			}
			//Use the same id that is used by the object in this position
			return result;
		},
		
		//adds a new object to the end of the list
		add: function(object, options, disableSystemNotification) {		
			logger.log("List", "add", "List: " + this._objectPath + ", object: " + object + ", options: " + options, logger.FINER);
			var index = this.length();
			this.items.push(object);
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				this.dataStore.add({"bpm_id": this.updateId(index), listItem: object}, options)	;
				this.enableSystemNotifications();
			} else {
				this.dataStore.add({"bpm_id": this.updateId(index), listItem: object}, options)	;				
			}
			return index;
		},
		
		//removes the object at index from the list					
		remove: function(index, disableSystemNotification) {
			logger.log("List", "remove", "List: " + this._objectPath + ", index: " + index, logger.FINER);
			if (index < 0 || index >= this.length()){
				return;
			}
		
			if (this.items) {
				this.items.splice(index, 1);
			}
			
			var idToRemove = this.indexToId[index];
						
			//update all the index to Id mappings			
			for (var i=index; i < this.length(); i++){			   
				this.indexToId[i] = this.indexToId[i + 1];
				this.childrenCache[i] = this.childrenCache[i + 1];
			}
			delete this.indexToId[i];
			delete this.childrenCache[i];
			
			var result = null;
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				result = this.dataStore.remove(idToRemove);
				this.enableSystemNotifications();
			} else {
				result = this.dataStore.remove(idToRemove);
			}
			return result;
		},
		
		set: function(property, value, disableSystemNotification){
			logger.log("List", "set", "List: " + this._objectPath + ", property: " + property + ", value: " + value, logger.FINER);
			
			// dojo Statful calls set() during the execution of its constructor in version 1.8.3.
			// If first parameter (i.e. property) is an object and no other parameters are specified, we know set() is called by the constructor of dojo Stateful. 
			// The set() call can be ignored since com.ibm.bpm.coach.List doesn't handle it at all.
			// The set() call is ignored in follow way: when property is an complex object, currentPlainValue will be null, and value (input parameter) is also null.
			// currentPlainValue is equal to value, so we ignore the set() call accordingly.

			// check if it makes sense to accept the set request
			var currentPlainValue = dataBindingHelper.getPlainValue(this, property);
			if(commonHelper.isValuesEqual(currentPlainValue, value) == commonHelper.YES) {
				// new value is equal to current value, ignore this set request
				logger.log("List", "set", "Ignore this set request because new value is equal to current value. value = " + currentPlainValue, logger.FINE);
				return;
			}
			
			if (!!disableSystemNotification && property == "listAllSelectedIndices")
				this.disableSystemNotifications(property);	
				
			if (property != "listAllSelectedIndices" && property != "listSelectedIndex" 
			    && property != "listAllSelected" && property != "listSelected") {
				var index = this.idToIndex(parseInt(property));
				if(!!index && index >=0 && index < this.length()){
					this.put(index, value, null, disableSystemNotification);
				}
			} else {
				this.inherited(arguments);
			}
			if (!!disableSystemNotification && property == "listAllSelectedIndices")
				this.enableSystemNotifications(property);	
						
			if (property == "listAllSelectedIndices"){	                			
				this._setDerivedProperties(false);
			}	  
		},

		//gets the object at index or get the value of properties
		//also maintains a cache of any embedded lists or obejcts and makes them watchable.
		get: function(indexOrProperty) {
			var result;
			
			//special case for the 4 properties
			if (indexOrProperty == "listAllSelectedIndices" || indexOrProperty == "listSelectedIndex"){
				return this[indexOrProperty];  //not stateful or List
			}
			else if (indexOrProperty == "listSelected"){
				//Using get to get it from the main list, so that they points to the same wrapped object
				return this.get(this.listSelectedIndex);				
			}
			else if(indexOrProperty  == "listAllSelected"){
				var value = this.childrenCache[indexOrProperty];
				if (!value){
				
					//NOTE: that if listAllSelected also want to be a list,should we also give a listAllSelected property to it
					//but it does not make sense to do that
					value = new com.ibm.bpm.coach.List(this[indexOrProperty], this._objectPath + "." + indexOrProperty);
					value._masterList = this;
				}
				this.childrenCache[indexOrProperty] = value;	
				return value;
			}
			
			//The call is for getting an object in the items array in this list
			var index = indexOrProperty;
		
			//Special case: if this list object is for listAllSelected, then it has a main list where the data should come from, then get it from the mater list
			if (!!this._masterList){
			   return this._masterList.get(index);		
			}
			
			//first check the cache
			var property = this.childrenCache[index];

			if (typeof(property) == "undefined") {
				var obj = this.items[index];  
								
				//check if it is a simple type
				var dataTypes = ["number", "string", "boolean"];
				array.forEach(dataTypes, function(dataType) {
					if (typeof obj == dataType) {
						result = obj;
					}
				});

				if (typeof result == "undefined") {                                        
					if (!!obj && (lang.isArray(obj)|| !!obj.items && !!obj.listAllSelectedIndices)) {
						result = new com.ibm.bpm.coach.List(obj, this._objectPath + "[" +index + "]");
						//add result to the cache
						this.childrenCache[index] = result;
		        	} else if (typeof obj == "object") {
		        		if (obj == null || !obj.prototype) { // check that this is a literal
		        			result = obj;
		        		} else if (obj.constructor) {
		        			result = new obj.constructor();
		        		}
		        		
					//treat Date like  primitive type
					if (result != null && !(result instanceof Date)) {
						var objectPath = this._objectPath;
		    				require(["./Stateful"], function(stateful) {
		    					result = new stateful(result, objectPath + "[" +index + "]");
		    				});		
	        				//add result to the cache
	        				this.childrenCache[index] = result;
	        			}
	        		}
				} else {
					result = obj;
				}
			} else {
				result = property;
			}
			return result;
		}, //get 
		
		getById: function(bpm_id) {
			var index = this.idToIndex(bpm_id);
			if (!!index) {
				return this.get(index);
			}
			return null;
		},
		
		idToIndex: function(bpm_id) {
			for (var index in this.indexToId) {		
				if (this.indexToId[index] == bpm_id) {
					return index;
				}
			}
			return null;
		},
		
		//retuns the length of the list
		length: function() {
			return this.managedList.length; 
		},
		
		//list item add/remove 
		bindAll: function(callback, scope) {
		    var wrapperFunction = function(object, removedFrom, insertedInto) {
				var newVal, oldVal;		
				if (insertedInto > -1) {
					// "add" event
					newVal = object.listItem;
	             }
				if (removedFrom > -1){
					if(removedFrom == insertedInto) {
						// removedFrom and insertedInto have same value and the value is not -1, means that this is a "put" event.
						// we do not have the old value for the "put" event based on current design
						oldVal = null; 
					} else {
						// "delete" event
						oldVal = object.listItem;
					}			 
				 }
				var event = {property: null, oldVal: oldVal, newVal: newVal, removedFrom: removedFrom, insertedInto: insertedInto};
				try {
					callback.call(scope, event);
				} catch(ex) {
					commonHelper.consoleLog(ex, commonHelper.ERROR);
				}
			}
			return this.registerCallback(null, wrapperFunction);
				
	    },
		
	    bindAllWithIds: function(callback, scope) {
            var wrapperFunction = function(object, removedFrom, insertedInto) {
                var newVal, oldVal;     
                if (insertedInto > -1) {
                    newVal = object;
                 }
                if (removedFrom > -1){
                    oldVal = object;             
                 }
                var event = {property: null, oldVal: oldVal, newVal: newVal, removedFrom: removedFrom, insertedInto: insertedInto};
				try {
					callback.call(scope, event);
				} catch(ex) {
					commonHelper.consoleLog(ex, commonHelper.ERROR);
				}
            }
            return this.registerCallback(null, wrapperFunction);
                
        },
		
		bind: function (property, callback, scope){
		     var wrapperFunction = function(property, oldVal, newVal) {
				var event = {property: property, oldVal: oldVal, newVal: newVal};
				try {
					callback.call(scope, event);
				} catch(ex) {
					commonHelper.consoleLog(ex, commonHelper.ERROR);
				}
			}
			return this.registerCallback(property, wrapperFunction);
		},
		
		toJson: function() {
			logger.log("List", "toJson", "Serializing list: " + this._objectPath, logger.FINER);
			//first get all of the stateful properties and do a toJson on them and 
			//then print the simple properties
			//debugger;
			var resultList = [];
			for (var x=0; x< this.items.length; x++) {
				if ((typeof this.items[x] == 'object') && this.items[x] instanceof Date) {
					var curItem = this.items[x];
					require(["./Stateful"], function(stateful) {
						resultList.push("\"" + stateful.prototype.dateToString(curItem) + "\"");
					});
				}
                else if (this.items[x] == null) {
                    resultList.push(null);
                } else if (typeof this.items[x] == 'object' ||
					lang.isArray(this.items[x])||
					typeof this.items[x] == 'com.ibm.bpm.coach.Stateful' ||
					typeof this.items[x] == 'com.ibm.bpm.coach.List') {
						resultList.push(this.get(x).toJson());
				} else {
					resultList.push(json.toJson(this.items[x]));
				}
			}
			
			//include listAllSelectedIndices in the json, the original array data become vlaue of items
			if (!this.listAllSelectedIndices) {
				this.listAllSelectedIndices = [];
			}
			var result = "{ \"listAllSelectedIndices\" : " + "[" + this.listAllSelectedIndices.join(",") + "]";
			result += ", \"items\" : " + "[";
            // we can not use join() here since null values may be present which will be turned into empty value by join.
            for (var x = 0; x < resultList.length; x++) {
                result += resultList[x];
                if (x < resultList.length - 1) {
                    result += ",";
                }
            }
            result += "]}";
			//return "[" + resultList.join(",") + "]";
			logger.log("List", "toJson", result, logger.FINER);
			return result;
		},
		
		//All the values are none wrapped when set, the one get from get method will be wrapped
		_setDerivedProperties: function(initialize){
				//set other properties
				if (!this.listAllSelectedIndices || this.listAllSelectedIndices.length == 0){
					if (!initialize){
						this.set("listSelectedIndex", null);
						this.set("listSelected", null);
			            
						//this is cached so delete the cache
						if (!!this.childrenCache["listAllSelected"]){
							delete this.childrenCache["listAllSelected"];
						}
						this.set("listAllSelected", []);
					}
					return;
				}
				var listSelectedIndex = this.listAllSelectedIndices[0];
				var listSelected = this.items[listSelectedIndex]; 	
				
				var listAllSelected = [];			
				var selectedIndex;
                for (var index in this.listAllSelectedIndices){
					selectedIndex = this.listAllSelectedIndices[index];
					listAllSelected.push(this.items[selectedIndex]);
				}
								
				if (!!initialize){
					//do not trigger notification
					this.listSelectedIndex = listSelectedIndex;
					this.listSelected = listSelected;
					this.listAllSelected = listAllSelected;
				}
				else{			
					
					this.set("listSelectedIndex", listSelectedIndex);
					
					//when get, please get it from this.get("..."), is using this way, we do not need to cache it
					this.set("listSelected", listSelected);
		            
					//this is cached so delete the cache
					if (!!this.childrenCache["listAllSelected"]){
						delete this.childrenCache["listAllSelected"];
					}
					this.set("listAllSelected", listAllSelected);
				}

		},
		createListElement : function(){
			// Two example of this.local, the value of "te" property is the type of list element.
			// this.local = {var1:{listAllSelectedIndices:[],items:[],te:"t1"}}
			// this.local = {strList:{listAllSelectedIndices:[],items:[],te:"S"}}
			
			if(!this.te) {
				this.te = templateStore.getTypeByPath(this._objectPath);
			}
			
			if(this.te) {
				var fullTemplate = templateStore.getByType(this.te);
				// "" and 0 are valid template
				if(fullTemplate != undefined && fullTemplate != null) {
					return lang.clone(fullTemplate);
				}
			}
			return null;
		},
		
		/*
		 * Duplicate the array. It is only used internally by coach runtime
		 * @param obj maybe an array or an object in the same format as the inital data
		 */
		duplicate: function(obj){
			/* Make the obj to the following format. If the obj does not contains those field. use the value from this object
			 * {
				items: [],
				listAllSelectedIndices : [],
				te				
			}*/
			var newItems = [];
			var newListAllSelectedIndices = this.listAllSelectedIndices;
			var newTemplateElement = this.te;
			
			if (lang.isArray(obj)){
				newItems = obj;
			}
			else if ((!!obj) && (lang.isObject(obj))) {
				newItems = obj.items;
				if (obj.listAllSelectedIndices){
					newListAllSelectedIndices = obj.listAllSelectedIndices;
				}
				if (obj.te){
					newTemplateElement = obj.te;
				}
			}
			return {items : newItems, listAllSelectedIndices : newListAllSelectedIndices, te : newTemplateElement};
		}
		
	}); //com.ibm.bpm.coach.List
});

