/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Shimmer to cover the coach area when it's needed to support read-only mode

define(["dojo/_base/connect", "dojo/_base/event", "dojo/dom-class", "dojo/keys", "./logger"], function(connect, dojoEvent, domClass, keys, logger) {

	return function (id) {
		// constants
		var	CSS_SHIMMER = "CoachView_shimmer";
		var	CSS_SHIMMER_ASSISTANT = "CoachView_shimmerAssistant";
		
		var shimmerId = id;
		var shimmerAssistantId = id + "_assistant";
	
		this.shimmer = document.getElementById(shimmerId);
		if(!!this.shimmer){
			document.body.removeChild(this.shimmer);
		}
		this.shimmer = document.createElement('div');
		this.shimmer.id = shimmerId;
		this.shimmer.tabIndex = 1;					
		domClass.add(this.shimmer, CSS_SHIMMER);
		document.body.insertBefore(this.shimmer, document.body.firstChild);
		
		this.shimmerAssistant = document.getElementById(shimmerAssistantId);
		if(!!this.shimmerAssistant){
			document.body.removeChild(this.shimmerAssistant);
		}
		// The following elements can have focus and are tab stops by default: 
		// a, BODY, button, frame, iframe, img, input, isIndex, object, select, textArea.
		this.shimmerAssistant = document.createElement('button');
		this.shimmerAssistant.id = shimmerAssistantId;
		domClass.add(this.shimmerAssistant, CSS_SHIMMER_ASSISTANT);
		document.body.appendChild(this.shimmerAssistant);
   	      
   	    // fix 121150
   	    // Portal encounters some issues in 8500 if this.shimmer grabs focus once Coach turns to readonly mode. 
   	    // Coach team has to change the design to help Portal team out:
   	    // If shimmer is in an iFrame, shimmer expects the parent window (e.g. Portal) to guarantee the focus is 
   	    // not in any field covered by shimmer. parent window can achieve it by simply grab the focus in its hand.
   	    // If shimmer is not in an iFrame, shimmer grabs the focus by itself to guarantee that the focus is 
   	    // not in any field covered by it.
   	    if(!window.parent || window.parent == window) {
   	    	// not in iFrame
			this.shimmer.focus();
		}
  
		var localShimmerAssistant = this.shimmerAssistant;
		var shimmerKeyHandle = connect.connect(this.shimmer, "onkeypress", function(e) {
			if(e.charOrCode == keys.TAB && e.shiftKey != true) {
			    dojoEvent.stop(e);			
				localShimmerAssistant.focus();
			}
		});	      
		var localShimmer = this.shimmer;
		var shimmerAssistantKeyHandle = connect.connect(this.shimmerAssistant, "onkeypress", function(e) {
			if(e.charOrCode == keys.TAB && e.shiftKey == true) {
			    dojoEvent.stop(e);
				localShimmer.focus();
			}
		});
	    
	    this.dispose = function() {
	        connect.disconnect(shimmerKeyHandle);
	        connect.disconnect(shimmerAssistantKeyHandle);
            if(!!this.shimmer) {
			    document.body.removeChild(this.shimmer);
			    this.shimmer = null;
            }
            if(!!this.shimmerAssistant) {
			    document.body.removeChild(this.shimmerAssistant);
			    this.shimmerAssistant = null;
            }
			logger.log("Shimmer", "dipsose", "Shimmer is disposed", logger.INFO);	    
	    };
	    
		logger.log("Shimmer", "constructor", "Shimmer is on", logger.INFO);	    
	};
});



