/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// 

define(["dojo/_base/lang", "./commonHelper"], function(lang, commonHelper) {
				
	var bindingResolver = {};
	
	bindingResolver.init = function(globalAssets) {
		bindingResolver.globalAssets = globalAssets;
	}

	//##Data Binding start
	bindingResolver.resolveBinding = function(bindingString) {
		//Resolves a binding string in the form of BO.property

		if (bindingString == null || bindingString == "") return;

		var result;
		var refArray = bindingString.split(".");
		if (refArray[0] == "local") {
			if (!(bindingResolver.globalAssets["local"] instanceof com.ibm.bpm.coach.Stateful)) {
				bindingResolver.globalAssets["local"] = new com.ibm.bpm.coach.Stateful(bindingResolver.globalAssets["local"], "local");
			}
			refArray.splice(0, 1);
			result = bindingResolver._resolveBindingObject(bindingResolver.globalAssets["local"] , refArray);			
		}
		else if (bindingString.indexOf("config") == 0){
			//bind to parent's options, resolve it
			result = bindingResolver._resolveStaticOptionBinding(bindingString);
		}	
		else {
			//handle system, resource, epv
			//add a setter, as required for consistency, but no callback is registered
			//if we want to notify change, we will make them as stateful so we do not need to maintain the callback functions
			var get = function (propName) {return (propName=="value")?this.value:null};
			var set = function (propName, val) {
				var errorMsg = "bindingResolver.resolveBinding(), set method is not supported on: " + bindingString;
				commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			};

			var bindingString1 = bindingString.substring(refArray[0].length);
			var value = eval("this.globalAssets['"+refArray[0]+"']" + bindingString1);
			result = {value: value, get: get, set: set};
		}
		return result;
	}

	bindingResolver.resolveOptionsBinding = function(options) {
		for(var optionKey in options) {	
		    if (optionKey == "_metadata"){
				for (var metadataKey in options[optionKey]){
					bindingResolver._resolveOptionBinding(options, metadataKey, true);
				}
			}
			else{			
			  bindingResolver._resolveOptionBinding(options, optionKey, false);
			}		
		}	
	}	
		
	bindingResolver._resolveBindingObject = function (objectRoot, refArray){
		var parentObject = objectRoot;
		var boundObject = objectRoot;
		var prop = null;
		for (var i=0; i<refArray.length; i++) {
			parentObject = boundObject;
			if (!boundObject){
				//for binding like  local.myList.listSelected.pro1, if the list is empty, listSelected will be undefined
		//		logger.log("bindingResolver", "_resolveBindingObject ", "boundObject is undefined. property:" + refArray[i-1]);
				break;
			}
			if (refArray[i].indexOf("[]") != -1) {
				refArray[i] = refArray[i].substring(0, refArray[i].indexOf("[]"));
			}
			var subMatch = refArray[i].match(/\[.*\]/);
			var index = null;
			if (subMatch != null) {
				index = subMatch[0].substring(1, subMatch[0].length-1);
				refArray[i] = refArray[i].substring(0, subMatch.index);
			}
			prop = refArray[i];
			if(index == null) {
				boundObject = boundObject.get(refArray[i]);
			} else {
				parentObject = boundObject.get(refArray[i]);
				boundObject = boundObject.get(refArray[i]).get(index);
				prop = index;
			}
		}
		var set;
		var get;
		if ( parentObject instanceof com.ibm.bpm.coach.List && (parseInt(prop) >= 0)) {
			var id = parentObject.indexToId[prop];
			set = function(propName, val) {(propName=="value")? this.boundObject.set(id, val):0};
			get = function (propName) {return (propName=="value")? this.boundObject.getById(id):null};
		} else {
			set = function(propName, val) {(propName=="value")? this.boundObject.set(this.property, val):0};
			get = function (propName) {return (propName=="value")? this.boundObject.get(this.property):null};
		}
		
		var result = !!parentObject ? {boundObject: parentObject, property: prop, get: get, set: set} : undefined;
		return result;
	}

	/*
		resolve one configure option.
		this function is splited from original engine._resolveAndRegisterOptionBindingCallBack()
	*/
	bindingResolver._resolveOptionBinding = function(options, shortOptionKey, isMetadata, update, objectPath){

	        var optionValue;
			var binding;
			var optionKey = shortOptionKey;
			optionValue = options[optionKey];	
			
			if (!!isMetadata){
				optionKey = "_metadata." + shortOptionKey;
				optionValue = options["_metadata"][shortOptionKey];	
			}
			//fix a bug: the user may not give the option a value, 
			//for update case, nothing need to be done for non bo case
			if(!optionValue || !!update && !optionValue.bindingStr)
				return;
				
			//resolved previously, and not for update case
			if(!!optionValue.boundObject && !update){
				binding = optionValue;
			}
				
	         //resolve the object if it is ref option to local, epv, resource
			else {
			if(!!optionValue.bindingStr){
				var affected = (!update) || !objectPath || commonHelper.startsWithPath(optionValue.bindingStr, objectPath);
				if(affected){					
					binding = bindingResolver.resolveBinding(optionValue.bindingStr);					
					// fix JR45548
					if(!!binding) {
					//This is useful later when check the binding string of this option
					binding.bindingStr = optionValue.bindingStr;
					if(!!update){
						binding.changed = true;
					}					
					}
				}
			}
			
			else if (!!optionValue.staticOptionStr){
				//static value	
				binding = bindingResolver._resolveStaticOptionBinding (optionValue.staticOptionStr, optionValue.value);
				binding.staticOptionStr = optionValue.staticOptionStr; //to be used for clone case
				//consider the case:  option for config 2 actually pointing to option in config 1 with staticOptionStr  options.config1.showAddButton
				//the binding is stored in _resolveStaticOptionBinding. set it here too
			}
			if (!!binding){
				if((optionKey === "_metadata.visibility" || optionKey === "_metadata.labelVisibility") && !optionValue.isStandardValue){			
					binding.get = function (propName) {
						var result = (propName=="value") ? this.boundObject.get(this.property) : null;
						result = (!!result? result.toUpperCase().replace(/[^A-Z]/g,'') : result);
						return result;
					};					
				}
				//store the value
				if (!!isMetadata){
					options["_metadata"][shortOptionKey] = binding;
				}
				else
					options[shortOptionKey] = binding;
			}
			}
			
			// when the binding path is not resolvable, the object under context.options (e.g. this.context.options.myOption1, this.context.options._metadata.visibility) 
			// still hold the original plain object that contains the binding path. The plain object doesn't have get() and set() functions. 
			// However, the current practice is that stock controls (customer may copy the code from stock control) assume that this.context.options.myOption1.get function
			// exists as long as this.context.options.myOption1 exist.
			// To fix above issue, we add the dummy get() and set() function here. If the binding path becomes resolvable later, the whole wrapper object will be replaced.
			// New wrapper object will contain a functional get() and set().
			// If wrapper object is a function, do not add dummy get() and set().
			var wrapperObject = !!isMetadata ? options["_metadata"][shortOptionKey] : options[shortOptionKey];
			if (!!wrapperObject && !wrapperObject.get && !lang.isFunction(wrapperObject)){
				// dummy get() and set()
				wrapperObject.get = function(propName) {
					return;
				}
				wrapperObject.set = function(propName, val) {
					return;
				}
			}
			
			return binding;
	}
	
	bindingResolver._resolveStaticOptionBinding = function(staticOptionStr, value){
				var result;
				
				var pathArray = staticOptionStr.split(".");
				var target = pathArray[0] == "config" ? bindingResolver.globalAssets : bindingResolver.globalAssets.options;
				var parent;
				var optionKey;
				
				//the data structure property name is options( globalAssets.options;), but the object path start with config
				if (pathArray[0] == "config")
				  pathArray[0] = "options";				  
				  
				/* Path can be 
				 * config.config2.option1.prop1.name,   config.config2.option1, config.config2.option1.prop1.name, 
				 * In any case, we use .notition until we get option1, and then make option1 stateful if it is not yet
				 */
				var bindToProperties = false;
				var upTo = 3;
				if (staticOptionStr.indexOf("_metadata") > -1){
					upTo = 4;
				}

				// handle the scenario: config.config1.option1[3].name, or config.config1.option1[3]
				// option1 will be the stateful object, we need to reserve the index (e.g. 3) for further resolving.
				var planedStatefulObjectStr = pathArray[upTo - 1];
				var leftBracketPosition = planedStatefulObjectStr.indexOf("[");
				if(leftBracketPosition != -1) {
					var rightBracketPosition = planedStatefulObjectStr.indexOf("]");
					var numericValue = planedStatefulObjectStr.substring(leftBracketPosition+1, rightBracketPosition);					
					if(numericValue != "") {
						pathArray.splice(upTo, 0, numericValue);
					}
					pathArray[upTo - 1] = planedStatefulObjectStr.substring(0, leftBracketPosition);
				}
				
				bindToProperties = pathArray.length > upTo;
				
				for (var i=0; i<upTo; i++){
					parent = target;
					optionKey = pathArray[i];
					target = target[optionKey];
				}
				
				//if the option is a service, nothing need to do here. This will cover the following cases
				//1. option value is a service: code gen will generate a function
				//2. option value refer to another option whose value is a function: config.config2.myServiceOption
				if (lang.isFunction(target)){
					result = target;//it is not a real binding, just the function it self
					parent[optionKey] = result;
				}
				else{
					var binding;
					if (!!target.boundObject){
						binding = target;
					}
					else{
					//create the binding object for the option with getters and setters, so that the view can get it using this.context.options.option1.get('value')
						var boundObject = new com.ibm.bpm.coach.Stateful({value : target.value}, staticOptionStr);
						binding = bindingResolver._createBindingObject(boundObject, "value");
						
						// remember the staticOptionStr (if exists) of parent[optionKey] before assigning new object to parent[optionKey]
						var possibleStaticOptionStr = !!parent[optionKey] ? parent[optionKey].staticOptionStr : null;
						parent[optionKey] = binding;
						// if parent[optionKey].staticOptionStr exists before we assign a new object to parent[optionKey], we need to keep this staticOptionStr in new object because clone needs it.
						// when _resolveStaticOptionBinding() is called by _resolveAndRegisterOptionBindingCallBack(), staticOptionStr is set back. 
						// But _resolveStaticOptionBinding has other callers, that's why we handle here.
						if(!!possibleStaticOptionStr) {
							parent[optionKey].staticOptionStr = possibleStaticOptionStr;
						}
						
						//for system change handler: Nothing needs to be done in the Stateful object, instead
						//the controller should be able to understand it by the object path
					}
					if (bindToProperties){
						//options.config1.option1.name
						pathArray.splice(0, upTo - 1);
						pathArray[0] = "value";
						result = this._resolveBindingObject(binding.boundObject, pathArray );
					}
					else {
						result = binding;
					}
					//if the binding is to some property in the option value, ie, option value is a complex object, then we need to continue to resolve the binding
					
				}
				
			return result;
	}

	bindingResolver._createBindingObject = function(boundObject, property){
		var get = function (propName) {return ((propName=="value") ? this.boundObject.get(this.property) : null)};
		var set = function(propName, newVal) {
			if(propName =="value") {
				this.boundObject.set("value", newVal);
			}
		};
		
		//when initView ->resolvebinding, register callback on binding, if the property is null, it register on all property changes 
		var binding = {get: get, set: set, property : property, boundObject : boundObject};
	    return binding;	
	}
	
	
	return bindingResolver;
});
