/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/**********************************COLLABORATION UTILS START*********************************************/
define(['require', 'dojo/_base/kernel', 'dojo/io/script', "dojo/_base/lang", "dojo/_base/xhr", "dojo/has", "dojo/_base/event", "./engine", 'dojo/_base/json', './collaborationBanner',"./logger", "dojo/_base/config"], 
		function(require, dojo, ioScript, lang, xhr, has, event, engine, json, baseCollaborationBanner, logger, dojoConfig) {
	
var collab = new Object();


_collaborationChannelNamePrefix = "/bpm/ProcessPortal/notifications/collaboration/";
collab.collaboratonChannelNames = {fullSync:"/DATA_FULL_SYNC/",
					dataChange:"/DATA_CHANGE",
					userJoin:"/USER_JOIN_SESSION",
					userLeave:"/USER_LEAVE_SESSION",
					rosterChange:"/EDITOR_CHANGE"};
					
collab.taskSessionID = null;
collab.taskOwnerID = null;
collab.sessionID = null;
collab.sessionRoster = null;
collab.currentUserForCollaboration =null;
collab.connectedToBayeuxServer = false;
collab.connectionEstablished = false;
collab.cometdForCollab = null;
collab.skipUserLeavePublish = false; 
collab.unsubAllChannelsOnExit = false;
_collaboration_timeout= 2000;
_collaboration_timer= null;
_RosterChangeEvent_timeout= 1000;
_RosterChangeEvent_timer= null;
_collaboration_dataChange_subscription=null;
_collaboration_subscriptions=[];
collaborationBanner = null;
	

//dojo.registerModulePath("org", "/bpm/notification/org");
//dojo.registerModulePath("dojox.cometd", "/bpm/notification/dojox/cometd");

collab.RegisterForCollaboration = function(sessionID, TaskOwnerId, fullSynchReceivedCallbackMethod, 
								  publishFullSynchMethod, IncomingChangesetCallbackMethod,
								  rosterChangeCallbackMethod,setToReadOnly){

		if(collaborationBanner) {
			collab.exitCollaboration(true);
		}
	
		if(!collab.connectionEstablished){
			logger.log("collaboration", "RegisterForCollaboration", "cometd connection already exisit start registering...", logger.FINE);
		}
		
		if(!collab.currentUserForCollaboration) {
			collab.currentUserForCollaboration = getUserInfo();
		}
		
		collab.taskSessionID = sessionID;
		collab.taskOwnerID = TaskOwnerId;
		collab.sessionID = sessionID.task_id+"/"+sessionID.coach_session_id;
		
		//update the rosterlist
		
		if(!collab.sessionRoster){//no session cached check with server
			logger.log("collaboration", "RegisterForCollaboration", "no session cached check with server", logger.FINE);
			var taskID = sessionID.task_id.split("t")[1];//variable has tNNN format
			collab.sessionRoster = getRosterListByTaskID(taskID);

			if(!collab.sessionRoster){//no previous collaboration exists on this task
				logger.log("collaboration", "RegisterForCollaboration", "Server returned 0 participants, no previous collaboration exists on this task...", logger.FINE);
				collab.sessionRoster = {
					"taskOwnerID":TaskOwnerId,
					"editorID":TaskOwnerId,
                    "numCollabUsers":0,
					"allParticipants":[{
						"userName":collab.currentUserForCollaboration.userName,
						"fullName":collab.currentUserForCollaboration.fullName
					}]
				};
			}
		}
		
		//publishFullSynchMethod should return the data set to be passed to the joiner
		coachsidePublishFullSynch = publishFullSynchMethod;
		coachsideFullSynchReceivedCallbackMethod = fullSynchReceivedCallbackMethod;
		coachsideIncomingChangesetCallbackMethod = IncomingChangesetCallbackMethod;
		coachsideRosterChangeCallbackMethod = rosterChangeCallbackMethod;
		coachsideSetToReadOnly=setToReadOnly;
		var collabInitialized = collab.initCollaboration();
		
		if(collabInitialized) {
			return collab.sessionRoster;
		}
};

collab.publishData = function(data){
	logger.log("collaboration", "publishData", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, logger.FINE);
	collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, 
						  data);
};


collab.setSkipUserLeavePublish = function (flag){
	collab.skipUserLeavePublish = flag;
};

collab.setUnsubscribeAll = function(flag) {
	collab.unsubAllChannelsOnExit = flag;
};

collab.unsubscribeDataChangeset = function (){
	logger.log("collaboration", "_unsubscribeDataChangeset", "Unsubscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, logger.FINE);
	if(_collaboration_dataChange_subscription){
		collab.cometdForCollab.unsubscribe(_collaboration_dataChange_subscription);
		_collaboration_dataChange_subscription=null;
	}
	
};

function getUserInfo(userID){
	//var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();//TO-DO need to use Kam's code
	var user = null;
	var rest_url = dojoConfig.App._bpmContextRootMap.rest + "/v1/user/current?parts=all";
	if(userID) rest_url =  dojoConfig.App._bpmContextRootMap.rest + "/v1/user/userID?parts=all";
	
		xhr.get({
			url: rest_url,
			handleAs: 'json',
			headers: {
				"Content-Type": "application/json", 
				"Accept": "application/json"
			},
			sync: true,
			load:  function(data, xhrArgs){
				userData = data.data;
			},
			error: lang.hitch(this, handleFailedRestConnection)
		});
	return userData;
};

function getRosterListByTaskID(task_id){
	var servlet_url = dojoConfig.App._bpmContextRootMap.socialBusWeb +"/roster/task/"+task_id;
	var roster;
	xhr.get({
			url: servlet_url,
			handleAs: 'json',
			headers: {
				"Content-Type": "application/json", 
				"Accept": "application/json"
			},
			sync: true,
			preventCache: true,
			load:  function(data, xhrArgs){
				if(data.taskEditorID&&
					data.taskEditorID!=""&&
					data.taskOwnerID&&
					data.taskOwnerID!=""){
						roster={};
						roster.editorID = data.taskEditorID;
						roster.taskOwnerID = data.taskOwnerID;
						roster.numCollabUsers = data.numCollabUsers;
						if(data.participants&&data.participants.length&&data.participants.length>0){
							
							roster.allParticipants = [];
							
							for(var i in data.participants){
								roster.allParticipants.push(data.participants[i]);
							}
						}
				}				
			},
			error: lang.hitch(this, handleFailedRestConnection)
		});
	if(roster){
		if(!roster.allParticipants){
			roster.allParticipants =[{"userName":collab.currentUserForCollaboration.userName,
									  "fullName":collab.currentUserForCollaboration.fullName}];
		}
	}
	return roster;
};

function handleFailedRestConnection(message){
	logger.log("collaboration", "handleFailedRestConnection", message, logger.FINE);
}

function coachsidePublishFullSynch(){};
function coachsideFullSynchReceivedCallbackMethod(){};
function coachsideIncomingChangesetCallbackMethod(){};
function coachsideRosterChangeCallbackMethod(){};
function coachsideSetToReadOnly(){};

collab.initCollaboration = function() {
	//var hostSameFlag = _isHostSame();
	if(!_isCollaborationSupported()){
		logger.log("collaboration", "initCollaboration", "collaborationContainerDiv not found, this is not social portal env. Exit registerForCollaboration.", logger.FINE);
		return false;
	}
	var cometDConnectExist = window.parent["BPMCometDConnectionEstablished"];
	collab.cometdForCollab = window.parent["cometDForCollab"];

	if(cometDConnectExist && collab.cometdForCollab){
		logger.log("collaboration", "initCollaboration", "cometd channel already established ... ", logger.FINE);
		_publishAndSubscribeToChannels();

		if(collab.sessionRoster.allParticipants.length>1){
			startCollaboration();
		}
	}else{
		logger.log("collaboration", "initCollaboration", "cometd channel is not established yet, wait for 2 seconds ... ", logger.FINE);
		_collaboration_timer = setTimeout(lang.hitch(this, 'initCollaboration'), _collaboration_timeout);	
	}
	
	return true;
};

function _publishAndSubscribeToChannels(){
	//TO-DO: should also handle meta connect leaves
	_subscribeFullSynch(collab.currentUserForCollaboration.userName);
	_subscribeUserLeave();
	_subscribeRosterRoleChange();
	_publishUserJoined(	collab.sessionRoster.taskOwnerID,
						collab.sessionRoster.editorID,
						collab.currentUserForCollaboration.userName, 
						collab.currentUserForCollaboration.fullName);
	_subscribeUserJoin();
};
function handleConnectionBroken(message) {
	collab.connectionEstablished = false;
	logger.log("collaboration", "handleConnectionBroken", message, logger.FINE);
};


function startCollaboration(){
	   logger.log("collaboration", "startCollaboration", "Should start banner for collaboration now, editor is:"+ collab.sessionRoster.editorID, logger.FINE);
		if(!collaborationBanner){
			var currentUserID = collab.currentUserForCollaboration.userName;
			var participants = collab.sessionRoster.allParticipants;
			var partner;
			for(var i in participants){
				if(participants[i].userName!=currentUserID)
					partner = participants[i];//only support 1 partner
			}
			if(partner){
				var bannerParams = {};
				bannerParams.isTaskOwner = (collab.sessionRoster.taskOwnerID==currentUserID)? true:false;		
				bannerParams.isEditor = (collab.sessionRoster.editorID==currentUserID)? true:false;
				bannerParams._makeEditor = lang.hitch(this,setEditor);
				bannerParams._takeControl = lang.hitch(this,takeControl);
			
				bannerParams.partnerFullName = partner.fullName;
				bannerParams.partnerUsername = partner.userName;
			
				collaborationBanner = new com.ibm.bpm.coach.collaborationBanner(bannerParams);
			}
		}
};


collab.exitCollaboration = function(beforeUnloadObject, isForceExitByNewClient){
	if(beforeUnloadObject){
		_unsubscribeAllChannels();	
		collab.unsubscribeDataChangeset();
	}

	if(collab.sessionRoster || collaborationBanner) {
		if(isForceExitByNewClient){
			//set coach view to read only 
			coachsideSetToReadOnly(); 
		} else {
			//handle role change
			if(collab.currentUserForCollaboration.userName == collab.sessionRoster.taskOwnerID){
				if(collab.currentUserForCollaboration.userName != collab.sessionRoster.editorID){
					_publishRosterRoleChange(collab.sessionRoster.editorID);
					coachsideRosterChangeCallbackMethod(collab.sessionRoster);
				}
			}
		}

		if(collaborationBanner)	{
			collaborationBanner.remove();
			collaborationBanner = null;
		}
	}
	
	if(collab.currentUserForCollaboration && _checkForUserInCollaboration(collab.currentUserForCollaboration.userName)) {		
		if(isForceExitByNewClient) {
			collab.sessionRoster = null;
		}
		else if(collab.skipUserLeavePublish) {//do not publish user leave if going between coaches
			collab.skipUserLeavePublish = false;
		} else {
			_publishUserLeave(collab.currentUserForCollaboration.userName);
			collab.sessionRoster = null;
		}
	}

	if(collab.unsubAllChannelsOnExit) {
		collab.unsubAllChannelsOnExit = false;
		_unsubscribeAllChannels();
	}
};

function setEditor(evt){
	event.stop(evt);
	//Set the viewer Editor
	var currentUserID = collab.currentUserForCollaboration.userName;
	var participants = collab.sessionRoster.allParticipants;
	var viewer;
	
	for(var i in participants){
		if(participants[i].userName!=currentUserID)
			viewer = participants[i];//only support 1 partner
	}
	
	//collab.sessionRoster.editorID = viewer.userName;
	//change local roster and broadcast change
	if(viewer)
		_assignEditorAccess(viewer.userName);
	//call Callback function
	coachsideRosterChangeCallbackMethod(collab.sessionRoster);	
	_subscribeDataChangeset();
	//switch collaboration bar view
	collaborationBanner.switchToViewerMode(true);
	
	if(collab.sessionRoster.taskOwnerID == currentUserID)
		collaborationBanner.hideTakeControlButton(false);
	
};

function takeControl(evt){
	event.stop(evt);
	//Only task owner can take control	
	var currentUserID = collab.currentUserForCollaboration.userName;	
	var taskOwner = collab.sessionRoster.taskOwnerID;
	if(currentUserID==taskOwner){
		//broadcast change
		_assignEditorAccess(currentUserID);		
		logger.log("collaboration", "takeControl", "wait for 1 second before unsubscribing to the datachange channel ... ", logger.FINE);
		_RosterChangeEvent_timer = setTimeout(lang.hitch(collab, 'unsubscribeDataChangeset'), _RosterChangeEvent_timeout);	
		//call Callback function
		coachsideRosterChangeCallbackMethod(collab.sessionRoster);		
		//switch collaboration bar view
		collaborationBanner.switchToViewerMode(false);
		collaborationBanner.hideTakeControlButton(true);
	}
};

function publishFullSynchData(receiverName, data){
	logger.log("collaboration", "publishFullSynchData", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.fullSync + receiverName, logger.FINE);
	var computedData = {newRoster:collab.sessionRoster,dataSet:data};
	collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.fullSync + receiverName, 
						  computedData);
};

function _assignEditorAccess(newEditorID){
	logger.log("collaboration", "_assignEditorAccess", "started", logger.FINE);
	if(collab.sessionRoster&&(collab.sessionRoster.editorID != newEditorID)){		
		collab.sessionRoster.editorID = newEditorID;
		_publishRosterRoleChange(newEditorID);		
	}
};


function _subscribeDataChangeset(){
	logger.log("collaboration", "_subscribeDataChangeset", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, logger.FINE);
	if(!_collaboration_dataChange_subscription){
		_collaboration_dataChange_subscription = collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, 
								coachsideIncomingChangesetCallbackMethod);
		//_collaboration_subscriptions.push({"channelName":collab.collaboratonChannelNames.dataChange, "subscription":_collaboration_dataChange_subscription});
	}
	};



function _unsubscribeAllChannels(){
	if (collab.cometdForCollab && collab.cometdForCollab.batch) {
		collab.cometdForCollab.batch(function(){
			for (var i in _collaboration_subscriptions){
				collab.cometdForCollab.unsubscribe(_collaboration_subscriptions[i]["subscription"]);
			}
		});
	}
};

function _unsubscribeChannel(channelName){
	for (var i in _collaboration_subscriptions){
		if(_collaboration_subscriptions[i]["channelName"] == channelName)
			collab.cometdForCollab.unsubscribe(_collaboration_subscriptions[i]["subscription"]);
	}
};

function _subscribeFullSynch(userName){
	logger.log("collaboration", "_subscribeFullSynch", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.fullSync + userName, logger.FINE);
	_collaboration_subscriptions.push({"channelName":collab.collaboratonChannelNames.fullSync,
										"subscription":collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.fullSync + userName, 
							_handleFullSynchReceived)});
};

function _subscribeUserLeave(){
	logger.log("collaboration", "_subscribeUserLeave", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userLeave, logger.FINE);
	_collaboration_subscriptions.push({"channelName":collab.collaboratonChannelNames.userLeave, 
										"subscription":collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userLeave, 
							_handleUserLeave)});
};

function _publishUserLeave(userName){
	if(collab.cometdForCollab&&collab.cometdForCollab.publish&&userName){
		logger.log("collaboration", "_publishUserLeave", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userLeave+" : "+userName, logger.FINE);
		collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userLeave, 
							  {"userName":userName});
	}
};

function _subscribeUserJoin(){
	logger.log("collaboration", "_subscribeUserJoin", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userJoin, logger.FINE);
	_collaboration_subscriptions.push({"channelName":collab.collaboratonChannelNames.userJoin, 
										"subscription":collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userJoin, 
							_handleUserJoin)});
};

function _publishUserJoined(taskOwnerID, editorID,userName, fullName){
	logger.log("collaboration", "_publishUserJoined", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userJoin+" : "+userName, logger.FINE);
	collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userJoin, 
						  {"taskOwnerID":taskOwnerID, "editorID":editorID,"userName":userName,"fullName":fullName});
};

function _publishRosterRoleChange(newEditorID){
	logger.log("collaboration", "_publishRosterRoleChange", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID+collab.collaboratonChannelNames.rosterChange, logger.FINE);
	collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.rosterChange, 
						  {"editorID":newEditorID});
};

function _subscribeRosterRoleChange(){
	logger.log("collaboration", "_subscribeRosterRoleChange", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID+collab.collaboratonChannelNames.rosterChange, logger.FINE);
	_collaboration_subscriptions.push({"channelName":collab.sessionID+collab.collaboratonChannelNames.rosterChange, 
									"subscription":collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.rosterChange, 
						  _handleRosterChange)});
};

function _handleRosterChange(msg){
	var newEditorID = msg.data.editorID;
	if(collab.sessionRoster){
		collab.sessionRoster.editorID = newEditorID;
		var currentUserID = collab.currentUserForCollaboration.userName;
		if(currentUserID == newEditorID){		
			//switch collaboration bar view
			logger.log("collaboration", "_handleRosterChange", "wait for 1 second before unsubscribing to the datachange channel ... ", logger.FINE);
			_RosterChangeEvent_timer = setTimeout(lang.hitch(collab, 'unsubscribeDataChangeset'), _RosterChangeEvent_timeout);	
			if (collaborationBanner) {
			   collaborationBanner.switchToViewerMode(false);
			   collaborationBanner.hideTakeControlButton(true);
			}
		}else{
			_subscribeDataChangeset();
			if (collaborationBanner) {
			   collaborationBanner.switchToViewerMode(true);
			   if(currentUserID==collab.sessionRoster.taskOwnerID) collaborationBanner.hideTakeControlButton(false);		
			}
		}
		coachsideRosterChangeCallbackMethod(collab.sessionRoster);
		
	}else{
		logger.log("collaboration", "_handleRosterChange", "sessionRoster does not exist", logger.FINE);
	}
};
function _handleUserLeave(incomingData){

	logger.log("collaboration", "_handleUserLeave", json.toJson(incomingData), logger.FINE);
	userName = incomingData.data.userName;
	logger.log("collaboration", "_handleUserLeave", "user: " + userName, logger.FINE);
	if(collab.sessionRoster&&collab.sessionRoster.allParticipants){
		for(var i in collab.sessionRoster.allParticipants){
			if(collab.sessionRoster.allParticipants[i].userName == userName){
				collab.sessionRoster.allParticipants.splice(i, 1);
				collab.sessionRoster.numCollabUsers--;
			}
		}
	}
	coachsideRosterChangeCallbackMethod(collab.sessionRoster);
	var numberOfParticipants = 0;
	if(collab.sessionRoster&&collab.sessionRoster.allParticipants){
		for( var j in collab.sessionRoster.allParticipants){
			numberOfParticipants++;
		}
	}
	if(numberOfParticipants<2){
		collab.setSkipUserLeavePublish(true);
		collab.exitCollaboration();
	}
};


function _handleUserJoin(userInfo){
	logger.log("collaboration", "_handleUserJoin", "userInfo: " + json.toJson(userInfo), logger.FINE);
	logger.log("collaboration", "_handleUserJoin", " session roster: " + json.toJson(collab.sessionRoster), logger.FINE);
	var userName = userInfo.data.userName;
	var fullName = userInfo.data.fullName;
	
	var taskId = collab.taskSessionID.task_id.split("t")[1];
	collab.sessionRoster = getRosterListByTaskID(taskId);
	
	if(!collab.sessionRoster){//new session
		logger.log("collaboration", "_handleUserJoin", "sessionRoster does not exist start new sessionRoster...", logger.FINE);
		
		if(userName!=collab.currentUserForCollaboration.userName){//someone joined, let's start collaboration
			collab.sessionRoster = {
				"taskOwnerID": collab.taskOwnerID,
				"editorID": collab.taskOwnerID,
				"numCollabUsers": 1, 
				"allParticipants": [{
										"userName":collab.currentUserForCollaboration.userName,
										"fullName":collab.currentUserForCollaboration.fullName
									},{
										"userName":userName,
										"fullName":fullName
									}]
			};
			logger.log("collaboration", "_handleUserJoin", "new sessionRoster created...", logger.FINE);
			publishFullSynchData(userName,coachsidePublishFullSynch());
			logger.log("collaboration", "_handleUserJoin", "coachsidePublishFullSynch called...", logger.FINE);
		}
	} else {//collaboration is already started
		logger.log("collaboration", "_handleUserJoin", "Collaboration already started with the roster:", logger.FINE);
		logger.log("collaboration", "_handleUserJoin", json.toJson(collab.sessionRoster), logger.FINE);

		var userAlreadyExistInSession = _checkForUserInCollaboration(userName);
		var numOfParticipants = 0;

		for(var i in collab.sessionRoster.allParticipants){
			numOfParticipants++;
		}

		if(!userAlreadyExistInSession){ //add to roster if not already in
				collab.sessionRoster.allParticipants.push({
					"userName":userName,
					"fullName":fullName
				});

				collab.sessionRoster.numCollabUsers++;
				numOfParticipants++;
				//coachsideRosterChangeCallbackMethod(collab.sessionRoster);//shouldnt need this
		}

		if(collab.sessionRoster.editorID == collab.currentUserForCollaboration.userName ||
			numOfParticipants >= 1 || userAlreadyExistInSession) {
				//I am the editor, send out full synch
				publishFullSynchData(userName, coachsidePublishFullSynch());
			}
		
		if(collab.sessionRoster.editorID && collab.sessionRoster.editorID != "" &&
			collab.sessionRoster.editorID != collab.currentUserForCollaboration.userName) {
			_subscribeDataChangeset();
		} else {
			collab.unsubscribeDataChangeset();
		}
	}

	if(collab.sessionRoster && collab.sessionRoster.allParticipants.length > 1) {
		startCollaboration();	//do no subscribe for data change since you are the editor
	}
	
	coachsideRosterChangeCallbackMethod(collab.sessionRoster);
};

function _handleFullSynchReceived(message){
	
	collab.sessionRoster.taskOwnerID = message.data.newRoster.taskOwnerID;
	collab.sessionRoster.editorID = message.data.newRoster.editorID;
	collab.sessionRoster.allParticipants = [];
	
	for(var i in message.data.newRoster.allParticipants){
		collab.sessionRoster.allParticipants.push(message.data.newRoster.allParticipants[i]);
	}
	
	logger.log("collaboration", "_handleFullSynchReceived", "session roster: " + json.toJson(collab.sessionRoster), logger.FINE);

	if(collab.sessionRoster.allParticipants && collab.sessionRoster.allParticipants.length > 1) {
		startCollaboration();	
	}

	if(collab.sessionRoster.editorID&&collab.sessionRoster.editorID != collab.currentUserForCollaboration.userName)
		_subscribeDataChangeset();
	else
		collab.unsubscribeDataChangeset();
	var data = message.data.dataSet;
	coachsideRosterChangeCallbackMethod(collab.sessionRoster);
	coachsideFullSynchReceivedCallbackMethod(data);//subscribe for data change since the session already started before you joined		
};

function _checkForUserInCollaboration(userName){
	if(collab.sessionRoster && collab.sessionRoster.allParticipants){
		for(var i in collab.sessionRoster.allParticipants){
			if(collab.sessionRoster.allParticipants[i].userName == userName)
				return true;
		}
	}
	return false;
};

function _isCollaborationSupported(){
	try{
		var windowLocation = window.location.href;
		var parentLocation = parent.location.href;		
	
		if(windowLocation&&parentLocation){
			var parentParts = parentLocation.split("/");
			var winParts = windowLocation.split("/");
			if(parentParts && winParts&& 
				parentParts[0]==winParts[0]&&//this is the protocal part should be http: or https:
				parentParts[2]==winParts[2]){//this is the hostname+port part should be <hostname>:<port>
				logger.log("collaboration", "_isCollaborationSupported", "Check for hostname and port, succeeded... Proceed...", logger.FINE);
				if(window.parent&&window.parent["collaborationContainerDiv"]){
					logger.log("collaboration", "_isCollaborationSupported", "Check for window.parent and container div succeeded... Proceed with register for collaboration...", logger.FINE);
					return true;
				}else{
					logger.log("collaboration", "_isCollaborationSupported", "Check for window.parent and container div failed... Abort collaboration...", logger.FINE);
					return false;
				}
			}
			else{
				logger.log("collaboration", "_isCollaborationSupported", "Check for hostname and port, failed... Abort collaboration...", logger.FINE);				
				return false;
			}
		}
	}catch(err){//not supported
		logger.log("collaboration", "_isCollaborationSupported", "Not able to access parent object... Collaboration not supported....", logger.FINE);
		return false;
	}
	
	// if not able to get parent location info we should not proceed with collaboration.
	return false;
};

require(["./engine"], function(engine) {
	engine._registerInternalUnloadHandler(collab.exitCollaboration);
});

window["exitCollaboration"]= collab.exitCollaboration;
return collab;
});
