/*
Licensed Materials - Property of IBM
5725-C95
© Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

define(['dojo/_base/kernel', 
		"dojo/_base/declare", // declare
		"dijit/_Widget",
		"dijit/_Templated",
		"dojo/string",
		'dojo/io/script', 
		"dojo/_base/lang", 
		"dojo/_base/xhr",
		"dojo/_base/fx",
		"dojo/_base/array",
		"dojo/_base/connect",
		"dojo/dom-style",
		"dojo/dom-class",
		"dojo/dom-construct",
		"./engine",
		'dojo/_base/unload',
		'require',
		// baader: the template includes the online awareness icon for the avatar, because per definition collaboration
		//         is only possible with an online user. So for performance reasons no extra check is integrated. When ever
		//         off-line collaboration is implemented, that needs to change and the online tracking needs to be integrated. 
		"dojo/text!./templates/CollaborationBanner.html",
		"./logger", "dojo/_base/config"],
		function(dojo, declare, _Widget, _Templated, dojoString, ioScript, lang, xhr, fx, array, connect, domStyle, domClass, domConstruct, engine, baseUnload, require, template, logger, dojoConfig) {

return declare(
	"com.ibm.bpm.coach.collaborationBanner",
	[_Widget, _Templated],
	{
	
    templateString: template,
	
	containerDivId : "socialPortalCollaborationBanner",
	
    message: null,

	eventParam: null,
		
	_container: null,
	
	fade: true,
	
	isEditor: false,
	
	isTaskOwner: false,
	
	partnerFullName : "Unknown User",
	
	partnerUsername : "unknownUser",
	
	makeEditorCallBack: null,
	
	takeControlCallBack: null,
	
	_collaboration_connections: [],
	
	userAvatar : undefined,
	
	// workingWithLabel:"You are working on the task with",
	// editModeLabel:"Edit Mode:",
	// viewModeLabel:"View Mode:",
	
	workingWithLabel:"You are working with",
	editModeLabel:"Edit Mode:",
	viewModeLabel:"View Mode:",
	makeEditorLabel:"Make Editor",
	takeControlLabel:"Task Over Control",
	
	constructor: function(params){
			
		var msgs = this.getEngineMessages(params.partnerFullName);
		
		this.workingWithLabel = msgs.workingWithLabel;
		this.editModeLabel = msgs.editModeLabel;
		this.viewModeLabel = msgs.viewModeLabel;
		this.makeEditorLabel = msgs.makeEditorLabel;
		this.takeControlLabel = msgs.takeControlLabel;
		
		this.createContainer();
			
	},
	
	createContainer: function(){
		
		if(window.parent["collaborationContainerDiv"]){
			this.container = window.parent["collaborationContainerDiv"];
			this.container.innerHTML = "";
		} else {
			logger.log("collaborationBanner", "createContainer", "Error: socialPortalCollaborationBanner Container not found.", logger.FINE);
		}
	},
	
    postCreate: function() {	
		this._collaboration_connections.push(connect.connect(this.makeEditorLink, 'onclick', this, "_makeEditor"));
		this._collaboration_connections.push(connect.connect(this.takeControlLink, 'onclick', this, "_takeControl"));

		
		this.userAvatar = this.getUserAvatar(this.partnerUsername);
		if(this.userAvatar)
			this.partnerAvatarImage.src = "data:image/" + this.userAvatar.imageFormat + ";base64," + this.userAvatar.userAvatarImage;	
		
		/*
		this.workingWithLabel= engine.collab_workingWithLabel;
		this.editModeLabel = engine.collab_editModeLabel;
		this.viewModeLabel = engine.collab_viewModeLabel;
		*/
		//attach events
		if(this.isTaskOwner){
			if(this.isEditor){
			
				//toggle controls
				this.makeEditorLink.className = "collaboration-controls-span collaboration-show";
				this.takeControlLink.className = "collaboration-controls-span collaboration-hidden";
				
				//Set title area
				this.title_area_icon.className = "collaboration-title-area-icon-edit";
				this.title_area_label.innerHTML= this.editModeLabel;
				
			}else{
				this.makeEditorLink.className = "collaboration-controls-span collaboration-hidden";
				this.takeControlLink.className = "collaboration-controls-span collaboration-show";
				
				this.title_area_icon.className = "collaboration-title-area-icon-readOnly";
				this.title_area_label.innerHTML= this.viewModeLabel;
			}		
		}else{		
			if(this.isEditor){
				this.makeEditorLink.className = "collaboration-controls-span collaboration-show";
				this.takeControlLink.className = "collaboration-controls-span collaboration-hidden";
				
				this.title_area_icon.className = "collaboration-title-area-icon-edit";
				this.title_area_label.innerHTML=this.editModeLabel;
			}else{
				this.makeEditorLink.className = "collaboration-controls-span collaboration-hidden";
				this.takeControlLink.className = "collaboration-controls-span collaboration-hidden";
				
				this.title_area_icon.className = "collaboration-title-area-icon-readOnly";
				this.title_area_label.innerHTML=this.viewModeLabel;
			}		
		
		}
		
		this.makeEditorLink.innerHTML = this.makeEditorLabel;
		domStyle.set(this.domNode, 'opacity', 0);
		domConstruct.place(this.domNode, this.container);
		fx.anim(this.domNode, {opacity: 1}, 250);
		
		try { 
			domClass.add(this.container.parentNode.parentNode, "bpm_c12n"); 
		} catch (e) { 
			//TODO If this fails in IE, we can try window.top.document.body
		};
	},
	
	getEngineMessages : function(partnerFullName){
		var msgs = {};
		var pFullName = partnerFullName;
		
		require(["./engine","dojo/string","dojo/_base/json"], function(engine, dojoString, json) {
			if(engine&&engine.localizedMessages){
                // forcing to load messages if they are not yet initialized
                engine.forceLocalizedMessages();

				msgs.workingWithLabel = engine.localizedMessages.collab_workingWithLabel;
				msgs.editModeLabel = engine.localizedMessages.collab_editModeLabel;
				msgs.viewModeLabel = engine.localizedMessages.collab_viewModeLabel;
				logger.log("collaborationBanner", "getEngineMessages", json.toJson(dojoString), logger.FINE);
				
				if(pFullName){
					msgs.makeEditorLabel = dojoString.substitute(engine.localizedMessages.collab_makeEditorLabel,[pFullName]);
					if(msgs.makeEditorLabel)
						msgs.makeEditorLabel = msgs.makeEditorLabel.replace("{0}",pFullName);
				}
				msgs.takeControlLabel = engine.localizedMessages.collab_takeControl;
			}
		});
		
		return msgs;
	
	},
	
	switchToViewerMode : function(flag){
		if(flag){
			this.makeEditorLink.className = "collaboration-controls-span collaboration-hidden";			
			this.title_area_icon.className = "collaboration-title-area-icon-readOnly";
			this.title_area_label.innerHTML = this.viewModeLabel;
		}else{
			this.makeEditorLink.className = "collaboration-controls-span collaboration-show";
			this.title_area_icon.className = "collaboration-title-area-icon-edit";
			this.title_area_label.innerHTML = this.editModeLabel;
		}
		if(this.participants)
			this.participants.focus();
	},
	
	hideTakeControlButton : function(flag){
		//enable all Buttons
		if(flag)
			this.takeControlLink.className = "collaboration-controls-span collaboration-hidden";
		else
			this.takeControlLink.className = "collaboration-controls-span collaboration-show";
		if(this.participants)
			this.participants.focus();
	},
	
	_makeEditor: function(e){
		/*implement by the parent*/
		makeEditorCallBack();
	},
	
	_takeControl: function(e){
		/*implement by the parent*/
		takeControlCallBack();
	},
	
	
	fireEvent:function(){
		//	parent defined
		logger.log("collaborationBanner", "fireEvent", "This function needs to be implemented by the parent.", logger.FINE);
	},

	_handleActionClick:function(event){		
		this.fireEvent(event.target.id,this.eventParam);
	},

	getUserAvatar : function(username){		
	  if (username == null) {
	    return null;
	  }

	 var avatar = this.userAvatar;
	  if (!this.userAvatar){
	    xhr.get({
	      url: dojoConfig.App._bpmContextRootMap.rest + "/v1/avatar/" + encodeURIComponent(username),
	      handleAs: "json",
	      sync: true,
	      headers: {
	        "Content-Type": "application/json",
	        "Accept": "application/json"
	      },
	      load: function(data, xhrArgs){
				avatar ={};
				avatar = data.data;
				avatar.inProgress = false;
	      },
	      error: function(data, xhrArgs){
	    	  switch(xhrArgs.xhr.status){
				// In the case of a 404, assume no avatar is defined for this user and
				// don't treat as an error since the default avatar will be displayed.
				case 404:
					avatar = null;	
					break;
				default:
					logger.log("collaborationBanner", "getUserAvatar", "REST call loading avatar image caused ERROR '" + error + "'", logger.FINE);
			}
	      }
	    });
	  }
	  return avatar;
	},

	cleanCollaborationBannerConnections:function(){
		array.forEach(this._collaboration_connections, connect.disconnect);
	},
	close: function(){
		fx.anim(this.domNode,{opacity: 0}, 500, null, lang.hitch(this, 'remove'));
	},
	
	remove: function(){
		try{ 
			domClass.remove(this.container.parentNode.parentNode, "bpm_c12n"); 
		} catch (e) {
			//TODO If this fails in IE, we can try window.top.document.body
		}
		
		this.cleanCollaborationBannerConnections();
		fx.anim(this.domNode,{height: 0, margin: 0}, 250, null,lang.partial(domConstruct.destroy, this.domNode));
		if(this.domNode){
			domConstruct.destroy(this.domNode);
		}
		//TODO Look at destroying container if last notification
	}
	
});


});
