/*
Licensed Materials - Property of IBM
5725-C95
Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
 
// controller main js
define(["require", "./Stateful", "./engine", "dojo/_base/xhr", "dojo/_base/json", "dojo/_base/lang", "dojo/_base/connect", "dojo/_base/kernel", "dojo/dom", "dojo/has", "./collaboration", "./BPM_CONSTANTS", "./CONSTANTS", "./boundaryEventHelper", "./broadcastHelper", "./logger", "dojo/_base/config"], function(require, stateful, engine, xhr, json, lang, connect, kernel, dom, has, p_collaboration, BPM_CONSTANTS, CONSTANTS, boundaryEventHelper, broadcastHelper, logger, dojoConfig) {
	// controller start
	var controller = new Object();

	controller.inCollaboration = false;
    controller.readOnly = false;
    controller.destroyed = false;
	controller.notificationQueue = [];
	controller.notificationTimeoutHandler = null;
	controller.setAsEditorTimeoutHandler = null;

	controller.init = function() {
		//This function is called in the main routine of coach. 
		//Catch all the exceptions here to give views a chance to render themselves. 
		//Then customer at least won't get a blank page just because this init() fails.
		try {
		    controller.initNotificationBus();
		    controller.initFocus();
		    controller.subscribeToEvents();
		    controller.initProgressIcon();
		    controller.initCustomCollaborations();
		    controller.initBoundaryEvents();
            controller.initValidationErrors();
	    } catch(ex) {
	    	logger.log("controller", "init", "Exception initializing: " + ex, logger.INFO);
	    }
	}

	controller.initProgressIcon = function() {
		//initializes the icon that spins as boundary events are being processed
	    var img = document.createElement('img');
	    img.src = 'images/common/loading.gif';
        require(["./engine"], function(engine) {
		    img.alt = engine.localizedMessages.coachNG_loading;
        });	    
	    
	    controller.progressIcon = document.createElement('div');
	    controller.progressIcon.style.position = 'fixed';
	    controller.progressIcon.style.top = '10px';
	    controller.progressIcon.style.right = '10px';
	    controller.progressIcon.visibility = "hidden";
	    controller.progressIcon.style.display = "none";
	    controller.progressIcon.appendChild(img);
	    document.body.appendChild(controller.progressIcon);
	}

	controller.initCustomCollaborations = function() {
		controller.customCollabEvents = {};
	}

	controller.initBoundaryEvents = function() {
		controller.boundayEventQueue = [];
		controller.boundayEventInProgress = false;		
	}

    controller.initValidationErrors = function() {
        controller.validationErrors = null;
    }

	controller.showProgressIcon = function() {
		//shows the spinner and sets the progress flag
	    controller.progressIcon.style.display = "block";
	    controller.progressIcon.style.visibility = "visible";
	    
	    controller.boundayEventInProgress = true;
	}

	controller.hideProgressIcon = function() {
		//hides the spinner and sets the progress flag
	    controller.progressIcon.style.display = "none";
	    controller.progressIcon.style.visibility = "hidden";
	    
	    controller.boundayEventInProgress = false;
	}
		
	controller.clearBoundaryEventQueue = function(requestedByView) {
		if(!!controller.boundayEventQueue) {
			for(var k=0; k<controller.boundayEventQueue.length; k++) {
				var callbackFun = controller.boundayEventQueue[k].callback;
				var alwaysInvokeCallback = controller.boundayEventQueue[k].alwaysInvokeCallback;
				if(!requestedByView) {
					boundaryEventHelper.callbackForBoundaryEvent(callbackFun, alwaysInvokeCallback, BPM_CONSTANTS.BOUNDARY_EVENT_STATUS_ABANDONED);
				} else {
					boundaryEventHelper.callbackForBoundaryEvent(callbackFun, alwaysInvokeCallback, BPM_CONSTANTS.BOUNDARY_EVENT_STATUS_CANCELLED);
				}
			}
		}
		controller.boundayEventQueue = [];
	}	

	controller.initNotificationBus = function() {
		var _this = this;
		require(["./engine"], function(engine) {
			if(engine.GlobalAssets.systemData.zTaskId&&engine.GlobalAssets.systemData.zComponentId){
				controller.sessionId = {task_id: engine.GlobalAssets.systemData.zTaskId, coach_session_id: engine.GlobalAssets.systemData.zComponentId};
				//check to see if this coach is launched as part of a task
				var taskID = controller.sessionId.task_id.split("t")[1];
				if(taskID){
					// joining session with the given session id, if session does not exist, it is being initialized
					logger.log("controller", "initNotificationBus", "Initializing Bus with session=" + json.toJson(controller.sessionId) +
							   ", owner=" + engine.GlobalAssets.systemData.taskOwnerUserName + ", task=" + taskID, logger.FINE);
					var roster = p_collaboration.RegisterForCollaboration(controller.sessionId,
																  engine.GlobalAssets.systemData.taskOwnerUserName,
																  // fullSynchReceivedCallbackMethod
																  lang.hitch(_this, "handleFullSync"),
																  // publishFullSynchCallbackMethod
																  lang.hitch(_this, "publishFullSync"),
																  // incomingChangesetCallbackMethod
																  lang.hitch(_this, "handleBusNotifications"),
																  // rosterChangeCallbackMethod
																  lang.hitch(_this, "handleRosterChange"),
																  lang.hitch(_this,"setToReadOnly"));
					controller.handleRosterChange(roster);
				}
			}
		});
	}

	controller.publishFullSync = function() {
		//called when a new collaboration user joins
        var result = null;
        require(["./engine"], function(engine) {
            controller.inCollaboration = true;
            
            result = "{\"pendingEvents\":" + controller.getPendingCustomCollabEvents() + "," +
            "\"data\":" + controller.localDataToJson() + "," +
            "\"errors\":" + json.toJson(controller.validationErrors) + "}";
        });
        
		logger.log("controller", "publishFullSync", "Sending JSON: " + result, logger.FINER);
        return result;
	}

	controller.handleFullSync = function(data) {
		//called to receive the current coach data when we join the collaboration
		require(["./engine"], function(engine) {
		    // we assume that we are in collaboration if we are handling full sync
		    controller.inCollaboration = true;
			logger.log("controller", "handleFullSync", "Received JSON: " + data, logger.FINER);
		    var dataObject = json.fromJson(data);
		    var baseObject = dataObject.data;
		    var pendingEvents = dataObject.pendingEvents;
            var validationErrors = dataObject.errors;
            // handling validation errors
            if (!!validationErrors) {
                engine.clearValidationEvent();
                engine.handleValidationEvent(validationErrors);
            }
		    // setting new properties and refreshing views at the same time
		    for (var x in baseObject) {
                if (!!baseObject[x] && !!baseObject[x].items && !!baseObject[x].listAllSelectedIndices && Object.prototype.toString.call(baseObject[x].items) === '[object Array]') {
                    engine.GlobalAssets.local.set(x, baseObject[x], true);
                    engine.GlobalAssets.local.get(x).set("listAllSelectedIndices", baseObject[x].listAllSelectedIndices, true);
                } else {
                    if (!!baseObject[x]) {
                        controller.adjustDateProperties(baseObject[x]);
                    }
                    engine.GlobalAssets.local.set(x, baseObject[x], true);
                }
		    }
		    if (!!pendingEvents) {
			    for (var y=0; y<pendingEvents.length; y++) {
			    	if(!!pendingEvents[y]) {
			    		controller.processNotification(pendingEvents[y]);
			    	}
			    }		    	
		    }
		});    
	}

	controller.handleBusNotifications = function(data) {
		//called by the notification bus for all activity from the bus to the controller
        require(["./engine"], function(engine) {
	        var event = json.fromJson(data.data);
	        controller.processNotification(event);
        });
	}
	
	controller.setToReadOnly = function(){
		//called by the notification bus to force coach view to read only when the same user is joined collaboration somewhere else
        require(["./engine"], function(engine) {
            controller.readOnly = true;
            engine.setReadonly(true);
        });
	}
	
	controller._delaySetAsEditor = function() {
		// Need to check if there is one already exist, if yes, then don't do another one.
		if (controller.setAsEditorTimeoutHandler == null) {
			controller.setAsEditorTimeoutHandler = setTimeout(function() {
				controller.setAsEditorTimeoutHandler = null;
				if (controller.notificationTimeoutHandler == null) {
					require(["./engine"], function(engine) {
						engine.setReadonly(false);
					});
				} else {
					controller._delaySetAsEditor();
				}
			}, 1000);
		}
	}

	controller.handleRosterChange = function(sessionRoster) {
		//called by the bus when a user adds/leaves the collab
        require(["./engine"], function(engine) {
            var username = engine.GlobalAssets.system.user_loginName;
            if (sessionRoster && sessionRoster.allParticipants) {
    			logger.log("controller", "handleRosterChange", "sessionRoster: " + json.toJson(sessionRoster), logger.FINE);
                if (!!sessionRoster.editorID) {
                    controller.readOnly = sessionRoster.editorID != username;
					if (controller.readOnly) {
						// Need to make sure any unexecuted previously set as editor timout handler does not execute.
						if (controller.setAsEditorTimeoutHandler != null) {
							clearTimeout(controller.setAsEditorTimeoutHandler);
							controller.setAsEditorTimeoutHandler = null;
						}
						engine.setReadonly(controller.readOnly);
					} else {
						// make sure all previous notification has been processed before setting to editor
						if (controller.notificationTimeoutHandler != null) {
							controller._delaySetAsEditor();
						} else {
							engine.setReadonly(controller.readOnly);
						}
					}
                }
                var rosterSize = 0;
                var currentUserInCollab = false;
                for (var i in sessionRoster.allParticipants){
                    rosterSize++;
                    if (sessionRoster.allParticipants[i].fullName == engine.GlobalAssets.system.user_fullName) {
                        currentUserInCollab = true;
                    }
                }

                if (rosterSize > 1 && currentUserInCollab) {
                    controller.inCollaboration = true;
                } else {
                    controller.inCollaboration = false;
                }
            } else {
                controller.inCollaboration = false;
            }
        });
	}

	controller._adjustCollaborationURL = function(nextURL) {
		var tempParser = document.createElement('a');
		tempParser.href = nextURL;
		var tempStr = tempParser.protocol + '//' + tempParser.host;
		if (nextURL.indexOf(tempStr) > -1) {
			nextURL = nextURL.substring(tempStr.length);
		}
		return nextURL;
	}

	controller.processBoundaryEvent = function(boundaryEvent) {
		//processes the boundary event that was triggered by another collaboration user
		require(["./engine"], function(engine) {
			logger.log("controller", "processBoundaryEvent", "Processing boundary event: " + json.toJson(boundaryEvent), logger.FINE);
		    var event = boundaryEvent.event;
            if (event == 'taskCompleted') {
                engine.TaskCompleted(controller._adjustCollaborationURL(boundaryEvent.url), boundaryEvent.taskId);
            }
            if (event == 'taskPostponed') {
                engine.TaskCompleted(controller._adjustCollaborationURL(boundaryEvent.url));
            }
            if (event == 'taskClosed') {
                engine.TaskCompletedAlreadyClosed(controller._adjustCollaborationURL(boundaryEvent.url));
            }
            if (event == 'taskNotOwned') {
                engine.TaskCompletedNotOwned(controller._adjustCollaborationURL(boundaryEvent.url));
            }
            if (event == 'taskStateChanged') {
                // we need to indicate that we are not leaving collaboration session
                p_collaboration.setSkipUserLeavePublish(true);
                p_collaboration.setUnsubscribeAll(true);

                // rtc-67838. we need to force collaborating user to get the right coach with correct execution context
				var url = dojoConfig.App._bpmContextRootMap.teamworks + "/process.lsw?zWorkflowState=1&zTaskId=" +
                          engine.GlobalAssets.systemData.zTaskId + "&zResetContext=true";
                engine.StateChange(url);
            }
            if (event == 'taskDataChanged') {
                var baseObject = json.fromJson(boundaryEvent.data);
                // setting new properties and refreshing views at the same time
                if (!!engine.GlobalAssets.local.set) {
                    for (var x in baseObject) {
                        engine.GlobalAssets.local.set(x, baseObject[x], true);
                    }
                }
                // clearing validation events if any
                engine.clearValidationEvent();
            }
            if (event == 'taskDebug') {
                engine.handleDebugMode(controller._adjustCollaborationURL(boundaryEvent.url));
            }
            if (event == 'taskError') {
                engine.ErrorBoundaryEvent(boundaryEvent.errorMessage);
            }
            if (event == 'dataInputValidation') {
                engine.clearValidationEvent();
                engine.handleValidationEvent(boundaryEvent.validationErrors);
            }
		});
	}

	controller.subscribeToEvents = function() {
		//boundary
		connect.subscribe("com.ibm.bpm.coach.BoundaryEventTriggered", function(obj) {
	        controller.handleBoundaryEvent(obj.boundaryEventInfo, obj.validateBoundaryEventInfo, obj.callback, obj.alwaysInvokeCallback, true);
	    });
	}

	controller.publishNotification = function(notificationObject) {
		//send a message on the bus
	    if (controller.inCollaboration) {
	    	p_collaboration.publishData(json.toJson(notificationObject));
	    }
	}

	controller.handleBoundaryEvent = function(boundaryEventInfo, validateBoundaryEventInfo, callback, alwaysInvokeCallback, broadcast) {
		//called to handle the boundary event that the user triggers.  Adds the event to a queue for processing
		var boundaryEventInfo = {boundaryEventInfo: boundaryEventInfo, validateBoundaryEventInfo: validateBoundaryEventInfo,
                                callback: callback, alwaysInvokeCallback: alwaysInvokeCallback, broadcast: broadcast};
		controller.boundayEventQueue.push(boundaryEventInfo);
		controller.sendBoundaryEvent();
	}

	controller.sendBoundaryEvent = function() {
		//sends the boundary event (including all local data) to the server
		require(["./engine"], function(engine) {
			//only send the event if there is no other boundary event in progress
			if (controller.boundayEventInProgress == false && controller.boundayEventQueue.length > 0) {
			    controller.showProgressIcon();
				logger.log("controller", "sendBoundaryEvent", "Boundary event queue length: " + controller.boundayEventQueue.length, logger.FINER);
			    var event = controller.boundayEventQueue.shift();
			    var boundaryEventId = event.boundaryEventInfo.boundaryEventId;
			    var boundaryEventEndStateId = event.boundaryEventInfo.boundaryEventEndStateId;
                var fireValidation = event.boundaryEventInfo.fireValidation;
                var validateBoundaryEventSection = "";
                if (!!event.validateBoundaryEventInfo) {
                    validateBoundaryEventSection = "\"validateBoundaryEventInfo\":" + json.toJson(event.validateBoundaryEventInfo) + ",";
                }
			    var callback = event.callback;
			    var alwaysInvokeCallback = event.alwaysInvokeCallback;
			    var broadcast = event.broadcast;
				logger.log("controller", "sendBoundaryEvent", "Sending boundary event: " + event.boundaryEventInfo.boundaryEventId + ", " +
                                                              event.boundaryEventInfo.boundaryEventEndStateId, logger.FINE);
			    var localData = controller.localDataToJson();
				var coachData = "{\"boundaryEventInfo\":{" +
			                    "\"boundaryEventId\":\"" + boundaryEventId + "\"," +
			                    "\"boundaryEventEndStateId\":\"" + boundaryEventEndStateId + "\"," +
                                "\"fireValidation\":\"" + fireValidation + "\"}," +
                                validateBoundaryEventSection +
			                    "\"stateInfo\":{" +
			                    "\"zComponentId\":\"" + engine.GlobalAssets.systemData.zComponentId + "\"," +
			                    "\"zComponentName\":\"CoachNG\"," +
			                    "\"zTaskId\":\"" + engine.GlobalAssets.systemData.zTaskId + "\"," +
			                    "\"zWorkflowState\":\"" + engine.GlobalAssets.systemData.zWorkflowState + "\"," +
			                    "\"applicationId\":\"" + engine.GlobalAssets.systemData.applicationId + "\"," +
			                    "\"applicationInstanceId\":\"" + engine.GlobalAssets.systemData.applicationInstanceId + "\"," +
			                    "\"zDbg\":\"" + engine.GlobalAssets.systemData.zDbg + "\"}," +
			                    "\"data\":{\"local\":" + localData + "}}";
			    var xhrArgs = {
			        url: "ajaxCoach",
			        handleAs: "text",
			        headers: { 
			            "Content-Type":"application/json; charset=utf-8" 
			        },
			        postData: coachData,
			        load: function(data) {
						logger.log("controller", "sendBoundaryEvent", "Boundary event response: " + data, logger.FINER);
	                    // check for session timeout
	                    var sessionExpired = controller.checkSessionTimeOut(data);
	                    if (!sessionExpired) {
	                        // convert the response, because handleAs is text
	                        var response = json.fromJson(data);
		                    controller.hideProgressIcon();
		                    
		                    boundaryEventHelper.callbackForBoundaryEvent(callback, alwaysInvokeCallback, BPM_CONSTANTS.BOUNDARY_EVENT_STATUS_EXECUTED);

	                        // process response
	                        controller.processServerResponse(response, fireValidation);
	                        
	                        if (!!broadcast) {
	                            //get path from id
				                var boundaryEventPath = "";

	                        	if (!!engine.GlobalAssets.boundaryEventData)
	                        	{
	                        		for (var z in engine.GlobalAssets.boundaryEventData) {
	                        			if (engine.GlobalAssets.boundaryEventData[z] == boundaryEventId) {
	                        				boundaryEventPath = z;
	                        				break;
	                        			}
	                        		}
	                        	}
	                        	var event = {name: "notifyBoundaryEvent", path: boundaryEventPath, result: "success", coach: document.title, nextUrl: response.url};
	                        	broadcastHelper.broadcastMessage(event, engine.boundaryEventBroadcastTarget);
	                        } 
	                    } else {
	                    	//TODO: call a harmless rest call to trigger an auth challenge if session expired, and clear the queue	                    	
	                    }
			        },
			        error: function(error) {
			            controller.hideProgressIcon();
			            controller.clearBoundaryEventQueue(false);
			            logger.log("controller", "sendBoundaryEvent", "An unexpected error occurred: " + error, logger.INFO);
			            engine.ErrorBoundaryEvent(error);
			        }
			    }
			    
			    xhr.post(xhrArgs);				
			} else {
				logger.log("controller", "sendBoundaryEvent", "Boundary event already in progress, or nothing in the queue, waiting...", logger.FINER);
			}
	    });
	}

    controller.getPendingCustomCollabEvents = function () {
    	//Returns the json serialization for any pending custom collab and focus events.  This is useful in a late joiner case to make sure the 
    	//navigational state of tabs and tables is correct on the late joiner coach
    	var result = new Array(); 
        for (var viewId in controller.customCollabEvents) {
        	if (!!viewId) {
        		var event = controller.customCollabEvents[viewId];
        		if (!!event) {
        			result.push(event);
        		}
        	}
        }
        if (!!controller.currentUserFocusEvent) {
        	result.push(controller.currentUserFocusEvent);
        }
        return json.toJson(result);
    }
   
    controller.processCustomCollaborationEvent = function(event) {
		//called when notification event comes in from the bus where type = 'custom'
		require(["./engine"], function(engine) {
			engine.handleCustomCollaborationEvent(event);
		});
	}
	
    controller.checkSessionTimeOut = function(data) {
        var expired = false;
        require(["./engine"], function(engine) {
            // there is no way to intercept HTTP 302 response by means of dojo framework,
            // redirect happens completely transparent and we get contents of login page with HTTP 200 response
            if (data.indexOf("form method=\"POST\" action=\"j_security_check\"") != -1) {
                expired = true;
				logger.log("controller", "checkSessionTimeOut", "session has expired", logger.INFO);
                var msg = engine.localizedMessages.coachNG_session_expired;
                engine.ErrorBoundaryEvent(msg);
            }
        });
        return expired;
    }

    controller.processServerResponse = function(response, fireValidation) {
    	//processes the response of the boundary event from the server
        require(["./engine"], function(engine) {
            var notificationEvent;
            var event = response.event;
            var system = engine.GlobalAssets.system;
            if (event == 'taskCompleted') {
				logger.log("controller", "processServerResponse", "task is completed", logger.FINE);
                var nextUrl = response.url;
                var nextTaskId;
                if (!!response.taskId) {
                    nextTaskId = response.taskId;
                }
                notificationEvent = {type: 'boundary', event: 'taskCompleted', url: nextUrl, taskId: nextTaskId, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.TaskCompleted(nextUrl, nextTaskId);
            }
            if (event == 'taskPostponed') {
				logger.log("controller", "processServerResponse", "task is postponed", logger.FINE);
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskPostponed', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.TaskCompleted(nextUrl);
            }
            if (event == 'taskClosed') {
				logger.log("controller", "processServerResponse", "task is completed and already closed", logger.FINE);
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskClosed', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.TaskCompletedAlreadyClosed(nextUrl);
            }
            if (event == 'taskNotOwned') {
				logger.log("controller", "processServerResponse", "task is completed and not owned by the current user", logger.FINE);
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskNotOwned', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.TaskCompletedNotOwned(nextUrl);
            }
            if (event == 'taskStateChanged') {
				logger.log("controller", "processServerResponse", "task state is changed", logger.FINE);
                // we need to indicate that we are not leaving collaboration session
                if (controller.inCollaboration) {
                    p_collaboration.setSkipUserLeavePublish(true);
                    p_collaboration.setUnsubscribeAll(true);
                }
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskStateChanged', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.StateChange(nextUrl);
            }
            if (event == 'taskDataChanged') {
				logger.log("controller", "processServerResponse", "task data is changed", logger.FINE);
                //clear validation only if fireValidation is on
                if (fireValidation == 1){
                	engine.clearValidationEvent();
                }
                var delta = response.delta;
                controller.processDataDeltas(delta);

                var data = controller.localDataToJson();
                notificationEvent = {type: 'boundary', event: 'taskDataChanged', data: data, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                
                //we can now process the next boundary event that is queued
                controller.sendBoundaryEvent();
            }
            if (event == 'taskDebug') {
				logger.log("controller", "processServerResponse", "debug response is received", logger.FINE);
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskDebug', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.handleDebugMode(nextUrl);
            }
            if (event == 'taskError') {
                var message = response.errorMessage;
				logger.log("controller", "processServerResponse", "error has occured: " + message, logger.FINE);
                notificationEvent = {type: 'boundary', event: 'taskError', errorMessage: message, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.ErrorBoundaryEvent(message);
            }
            if (event == 'dataInputValidation') {
                logger.log("controller", "processServerResponse", "data input validation response is received", logger.FINE);
                notificationEvent = {type: 'boundary', event: 'dataInputValidation', validationErrors: response.validationErrors};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                // we need to keep track of validation errors for late joiner case
                controller.validationErrors = response.validationErrors;
                //clear views that have previous validation errors decorations before handling new validation errors
				engine.clearValidationEvent();
                engine.handleValidationEvent(response.validationErrors);
            }
        });
    }

    controller.localDataToJson = function() {
    	//returns the serialized local obejct (all local data)
        var result = "{}";
        require(["./engine"], function(engine) {
            // check for uninitialized data
            if (!!engine.GlobalAssets.local.toJson) {
                result = engine.GlobalAssets.local.toJson();
            }
        });
        return result;
    }

    controller.processDataDeltas = function(delta) {
    	//called when a boundary event is a data change (stay on same coach).  The response is the delta data that needs to be applied.
        var allPathsValid = true;
        require(["./engine"], function(engine) {
			logger.log("controller", "processDataDeltas", "processing deltas", logger.FINE);
            // check for uninitialized local data
            if (!!engine.GlobalAssets.local.set) {
                var restoreCollaboration = controller.inCollaboration;
                // stop sending collaboration events
                if (restoreCollaboration) {
                    controller.inCollaboration = false;
                }
                for (var x in delta) {
                    var pos = x.indexOf("tw.local.");
                    if (pos == 0) {
                        // stripping off 'tw.local.'
                        var property = x.substring(9);
                        // check if given property exists
                        if ((engine.GlobalAssets.local.get(property)) !== undefined) {
                            if (Object.prototype.toString.call(delta[x]) === '[object Array]') {
                                // retrieving the correct list property
                                var list = engine.GlobalAssets.local.get(property);
                                if (!!list) {
                                    engine.GlobalAssets.local.set(property, delta[x], true);
                                    engine.GlobalAssets.local.get(property).set("listAllSelectedIndices", [], true);
                                }
                            } else {
                                // rtc-90015 date strings must be converted into date objects
                                // in order to avoid inconsistencies in stateful objects and
                                // prevent issues in date/time pickers
                                controller.adjustDateProperties(delta[x]);
                                engine.GlobalAssets.local.set(property, delta[x], true);
                            }
                        } else {
                        	allPathsValid = false;
                        }
                    } else {
                    	allPathsValid = false;
                    }
                    
                }
                // restoring sending collaboration events if required
                if (restoreCollaboration) {
                    controller.inCollaboration = true;
                }
            } else {
            	allPathsValid = false;
            }
        });
        return allPathsValid;
    }

    controller.adjustDateProperties = function(value) {
        for (var x in value) {
            if (value.hasOwnProperty(x) && !!value[x]) {
                if (Object.prototype.toString.call(value[x]) === '[object Array]') {
                    controller.adjustDateProperties(value[x]);
                } else if (/[0-9]{4}(\-[0-9]{2}){2}T([0-9]{2}\:){2}[0-9]{2}/.test("" + value[x])) {
                    // converting date string to Date object
                    var datetime = value[x].split("T");
                    var date = datetime[0].split("-");
                    var time = datetime[1].split(":");
                    var epochValue = Date.UTC(parseInt(date[0]), parseInt(date[1],10)-1, parseInt(date[2],10),
                        parseInt(time[0],10), parseInt(time[1],10), parseInt(time[2],10));
                    value[x] = new Date(epochValue);
                } else if (typeof value[x] !== "string") {
                    // skip simple strings and check complex objects
                    controller.adjustDateProperties(value[x]);
                }
            }
        }
    }


	controller.processNotification = function(event) {
		controller.notificationQueue.push(event);
		if (controller.notificationTimeoutHandler == null) {
			// Do a 1 sec delay here to allow any previous collaboration to be shown for at least 1 sec
			controller.notificationTimeoutHandler = setTimeout(function() {
				controller._processNotification(controller.notificationQueue.shift());
			}, 1000);		
		}
	}
	
	controller._processNotification = function(event) {
		controller.notificationTimeoutHandler = null;
		if (controller.notificationQueue.length > 0) {
			// Do a 1 sec delay here to allow any previous collaboration to be shown for at least 1 sec
			controller.notificationTimeoutHandler = setTimeout(function() {
				controller._processNotification(controller.notificationQueue.shift());
			}, 1000);
		}
		//router for notification events from the bus
		if (event.type && controller.destroyed == false) {
            if (event.type == 'focus') {
                controller.processFocusEvent(event);
            } else if (event.type == 'data') {
                controller.processDataEvent(event);
            } else if (event.type == 'boundary') {
                controller.processBoundaryEvent(event);
            } else if (event.type == 'custom') {
                controller.processCustomCollaborationEvent(event);
            } else {
    			logger.log("controller", "processNotification", "could not process notification event: " + json.toJson(event), logger.FINE);
            }
		} else {
			logger.log("controller", "processNotification", "could not process notification event: " + json.toJson(event), logger.FINE);
		}
	}
	
	controller.initFocus = function() {
		controller.setFocusHandlers();
		
		//init data structures to handle focus collaboration
		controller.currentUserFocus = {};
		controller.previousUserFocus = {};
		controller.focusTextColor = "#ffffff";
		controller.colorList = ["#07e5e2", "#ec8705", "#59a043", "#1b75bc", "#8d0ee6", "#0818f1", "#e5c307", "#e50707", "#cd462f"];

		controller.userColorMap = {};
		controller.userFocusMap = {};
		controller.focusElementToUserMap = {};
		controller.isCurrentUserAnEditor = true;

		controller.focusTimestamp = new Date().getTime();
	}


	controller.handleFocusEvent = function (event) {
		require(["./engine"], function(engine) {
			if ((engine._isCoachUnloaded)) {
				// coach is already unloaded. no need to continue
				return;
			}

			//finds the id of the focus target and the id of the view it belongs to 
			//and passes the ids and focus event to the controller
	
			//IE
			if (!event.target) {
				event.target = event.srcElement;
			}
			
			var id = event.target.id;
			var viewId = null;
			var parent = event.target.parentNode;
			while (parent) {
				if (parent.className && parent.className.indexOf(engine.CONSTANTS.COACH_VIEW) != -1) {
					viewId = parent.id;
					break;
				}
				parent = parent.parentNode;
			}
			if (!id) {
				id = viewId;
			}
			var viewIdMetaData = engine.getViewIdMetaData(viewId);
			var currentFocusTimeStamp = new Date().getTime();
			e = {'type': 'focus', 'domId': id, 'viewId': viewId, 'userName': engine.GlobalAssets.system.user_fullName, 'viewIdMetaData': viewIdMetaData};
			controller.currentUserFocusEvent = e;
			
			//only change the previous focus if the coach stays on the field for some time
			//helps in the case where a user quickly tabs several fields after making a data change
			if ( (currentFocusTimeStamp - controller.focusTimestamp) > 600) {
				controller.previousUserFocus.domId = controller.currentUserFocus.domId;
				controller.previousUserFocus.viewId = controller.currentUserFocus.viewId;
				controller.previousUserFocus.viewIdMetaData = controller.currentUserFocus.viewIdMetaData;
			}
			
			controller.currentUserFocus.domId = id;
			controller.currentUserFocus.viewId = viewId;
			controller.currentUserFocus.viewIdMetaData = viewIdMetaData;
			controller.focusTimestamp = currentFocusTimeStamp;
			
			// publishing focus notification
		    controller.publishNotification(e);
		    
		    //clean up any highlights left by the previous editor
		    if (controller.isCurrentUserAnEditor == false) {
		    	for (var x in controller.userFocusMap) {
		    		if (!!x && !!controller.userFocusMap[x]) {
		    			var userFocus = controller.userFocusMap[x];
		    			engine.handleBlurEvent({viewId: userFocus.viewId, domId: userFocus.domId, viewIdMetaData: userFocus.viewIdMetaData});
		    		}
		    	}
		    	controller.isCurrentUserAnEditor = true;
		    }
		});
	}	

	controller.customCollaborationHandler = function(viewId, payload) {
		//creates a new event: type='custom', viewId = viewId and payload = payload
		//publishes the event on the notification bus
		require(["./engine"], function(engine) {
			var viewIdMeta = engine.getViewIdMetaData(viewId);
			var e = {'type': 'custom', 'viewId': viewId, 'payload': payload, 'viewIdMetaData': viewIdMeta};
			//keep track of custom collab events (per view) in case we need to send them later to a late joiner
			controller.customCollabEvents[viewId] = e;
			controller.publishNotification(e);
		});
	}
	
	controller.mouseOnHandler = function (event) {
		//detects a hover event.  Need to check if the current domId+viewId has any users who are focused on it.
		//If so, notify the framework to show a bubble on the element.
		//Note: this code no longer applies since the collab always has a shimer in front of the coach
		require(["./engine"], function(engine) {
	
			//IE
			if (!event.target) {
				event.target = event.srcElement;
			}
			
			if (!!event && !!event.target && event.target.id != "mainBody") {
				var id = event.target.id;
				var viewId = null;
				var parent = event.target.parentNode;
				while (parent) {
					if (parent.className && parent.className.indexOf(engine.CONSTANTS.COACH_VIEW) != -1) {
						viewId = parent.id;
						break;
					}
					parent = parent.parentNode;
				}
				if (!id) {
					id = viewId;
				}
				var focusedUsers = controller.focusElementToUserMap[id+viewId];
				if (focusedUsers && focusedUsers.length >0) {
					var e = 	{viewId: viewId, domId: id};
					e.color = controller.focusTextColor;
					e.bgColor = controller.userColorMap[focusedUsers[0]];
					if (focusedUsers.length >1) {
						e.message = focusedUsers[0] + ",...";
					} else {
						e.message = focusedUsers[0];
					}
					
					engine.handleMouseOnEvent(e);
				}
			}
		});
	}

	controller.mouseOffHandler = function (event) {
		require(["./engine"], function(engine) {
			//detected a hover leave event.  Need to check if the current domId+viewId has any users who are focused on it.
			//If so, notify the framework to hide the bubble on the element.
			//Note: this code no longer applies since the collab always has a shimer in front of the coach
	
			//IE
			if (!event.target) {
				event.target = event.srcElement;
			}
			
			if (!!event && !!event.target && event.target.id != "mainBody") {
				var id = event.target.id;
				var viewId = null;
				var parent = event.target.parentNode;
				while (parent) {
					if (parent.className && parent.className.indexOf(engine.CONSTANTS.COACH_VIEW) != -1) {
						viewId = parent.id;
						break;
					}
					parent = parent.parentNode;
				}
				if (!id) {
					id = viewId;
				}
				var focusedUsers = controller.focusElementToUserMap[id+viewId];
				if (focusedUsers && focusedUsers.length >0) {
					var e = 	{viewId: viewId, domId: id};
	
					engine.handleMouseOffEvent(e);
				}
			}
		});
	}


	controller.setFocusHandlers = function () {
		require(["./engine"], function(engine) {
			//sets a focus handler on the mainBody element to process all focus changes on the coach
			var mainBody = dom.byId("mainBody");
			
			if (mainBody.addEventListener) { //non-IE
				mainBody.addEventListener('focus', lang.hitch(engine.GlobalAssets,controller.handleFocusEvent), true); 
				//mainBody.onmouseover = controller.mouseOnHandler;
				//mainBody.onmouseout = controller.mouseOffHandler;
			} else {  // IE
			  //mainBody.onfocusin = controller.handleFocusEvent;
				mainBody.attachEvent('onfocusin', lang.hitch(engine.GlobalAssets, controller.handleFocusEvent));
				//mainBody.attachEvent('onmouseover', controller.mouseOnHandler);
				//mainBody.attachEvent('onmouseout', controller.mouseOffHandler);
			}
		});
	}

	controller.processFocusEvent = function(e) {
		require(["./engine"], function(engine) {
			//a focus event has come in, need to check if the user already is assigned a color, if not, assign one
			if (!controller.userColorMap[e.userName]) {
				controller.userColorMap[e.userName] = controller.colorList.pop();
			}
			var userColor = controller.userColorMap[e.userName];
			e.outlineColor = userColor;
			e.bgColor = userColor;
			e.color = controller.focusTextColor;
			
			//check the user focus map to see if this user has an existing focus
			var userFocus = controller.userFocusMap[e.userName];
			if (userFocus) {
				//use this key to remove the user from the focusElementToUserMap
				var listOfUsers = controller.focusElementToUserMap[userFocus.domId+userFocus.viewId];
				for (var x=0; x<listOfUsers.length; x++) {
					if (listOfUsers[x] == e.userName) {
						listOfUsers.splice(x,1);
						break;
					}
				}
				//if there are any other users who are currently focused on this element, send a focus event to the framework on their behalf,
				//otherwise ask the framework to blur this element
				if (listOfUsers.length > 0) {
					var altColor = controller.userColorMap[listOfUsers[0]];
					engine.handleFocusEvent({viewId: userFocus.viewId, domId: userFocus.domId, outlineColor: altColor, bgColor: altColor, color: controller.focusTextColor});
				} else {
					engine.handleBlurEvent({viewId: userFocus.viewId, domId: userFocus.domId, viewIdMetaData: userFocus.viewIdMetaData});
				}
			}
			//change the users new focus
			controller.userFocusMap[e.userName] = {domId: e.domId, viewId: e.viewId, viewIdMetaData: e.viewIdMetaData};
			//add the user to the focus map of the new dom
			if (!controller.focusElementToUserMap[e.domId+e.viewId]) {
				controller.focusElementToUserMap[e.domId+e.viewId] = [];
			}
			controller.focusElementToUserMap[e.domId+e.viewId].push(e.userName);
			
			engine.handleFocusEvent(e);
			
			//since we are processing a focus event, this implies that we are a viewer
			//not an editor, so we should reset the flag
			controller.isCurrentUserAnEditor = false;
		});
	}

	controller.processDataEvent = function(e) {
		//processes a data change event that has come in on the bus
		require(["./engine", "./Stateful"], function(engine, stateful) {
			//process data event
			var statefulObject = null; 
			if (e.path == "local") {
				logger.log("controller", "proccessing collab event for local.", logger.FINE);
				statefulObject = engine.GlobalAssets.local;
			}
			else {
				var path;
				
				//Now that option value are bindable and editable, even for static value (ie, not bo). 
				//Those static values are stored under engine.GlobalAssets.options
				if (e.path.indexOf("config") == 0){
					path = e.path.substring("config.".length);
					logger.log("controller", "proccessing collab event for config." +path, logger.FINE);
					statefulObject = engine.GlobalAssets.options;
					var propArray = path.split(".");
					for (var x=0; x<propArray.length; x++) {
						statefulObject = statefulObject[propArray[x]];
					}
					if (!!statefulObject && !!(statefulObject.boundObject)) {
						statefulObject = statefulObject.boundObject;
					}
				}
				else {
					path = e.path.substring("local.".length);
					logger.log("controller", "proccessing collab event for local." +path, logger.FINE);
					statefulObject = engine.GlobalAssets.local.get(path);
				}
			}
				
			logger.log("controller", "event is: " + json.toJson(e), logger.FINE);
			if(statefulObject instanceof com.ibm.bpm.coach.Stateful) {
				var processedValue = e.value;
				if (e.value[0] == "{" || e.value[0] == "[")
				{
					processedValue = kernel.eval('('+e.value+')'); 
				}
	
				//disable system notifications before doing the set to prevent an infinite
				//loop of notifications
				statefulObject.set(e.property, processedValue, true);			
			} else {
				//must be a list
                if (e.property == "listAllSelectedIndices") {
                    // listAllSelectedIndices is a special case for lists
                    processedValue = kernel.eval('('+e.value+')');
                    statefulObject.set(e.property, processedValue, true);
                } else if (e.insertedInto == -1) {
					//must be a delete
					if (e.removedFrom != -1) {
						statefulObject.remove(e.removedFrom, true)
					} else {
		    			logger.log("controller", "processDataEvent", "error could not process update: " + json.toJson(e), logger.FINE);
					}
				} else {
					//either an insert or a update
					processedValue = e.value;
					if (e.value[0] == "{" || e.value[0] == "[")
					{
						processedValue = kernel.eval('('+e.value+')'); 
					}
					if (e.removedFrom != -1) {
						//update
						statefulObject.put(e.insertedInto, processedValue.listItem, null, true);
					} else {
						//add
						statefulObject.add(processedValue.listItem, null, true);
					}
				}
//                var listObj = statefulObject;
//                var list = engine.GlobalAssets.local.get(path);
//                engine.GlobalAssets.local.set(list, listObj, true);
			}	
			e.color = controller.focusTextColor;
			e.bgColor = controller.userColorMap[e.userName];
            var message = engine.localizedMessages.coachNG_change_notification;
			e.message = message.replace("{0}", e.userName);
			engine.handleCollabDataChangeEvent(e);
		});
	}

	/*system change handlers, 
	 * handles properties change on a @Stateful object or
	 * property on a @List object
	*/
	controller.systemChangeHandler = function (path, prop, newValue) {
		require(["./engine"], function(engine) {
			var currTimeStamp = new Date().getTime();
			var curFocus;
			var curView;
			var curViewMeta;
			//only send the previous focus dom if the user has tabbed off the field
			//we know this if the focus change happens within a few milliseconds of the data change 
			if (!!controller.previousUserFocus.domId && ( (currTimeStamp - controller.focusTimestamp) < 600) ) {
				curFocus = controller.previousUserFocus.domId;
				curView = controller.previousUserFocus.viewId;
				curViewMeta = controller.previousUserFocus.viewIdMetaData;
			} else {
				curFocus = controller.currentUserFocus.domId;
				curView = controller.currentUserFocus.viewId;
				curViewMeta = controller.currentUserFocus.viewIdMetaData;
			}
			controller.previousUserFocus.domId = controller.currentUserFocus.domId;
			controller.previousUserFocus.viewId = controller.currentUserFocus.viewId;
			controller.previousUserFocus.viewIdMetaData = controller.currentUserFocus.viewIdMetaData;

            if (controller.readOnly == false) {
                // we need to check if coach is in editor mode
                // in order to prevent mirroring notifications
                if (typeof newValue == 'object' ||
                      Object.prototype.toString.call(newValue) === '[object Array]') {
                    var shortPath = path;
                    if (prop && prop != "") {
                        shortPath = shortPath + "." + prop;
                    }
                    if (shortPath.indexOf("local.") != -1)
                    {
                        shortPath = shortPath.substring("local.".length);
                    }
                    var storedValue = engine.GlobalAssets.local.get(shortPath);

                    if(storedValue instanceof Date) {
                        newValue = engine.GlobalAssets.local.dateToString(storedValue);
                    } else {
                        //There are properties on a List that requires this checking
                        newValue = storedValue instanceof com.ibm.bpm.coach.Stateful || storedValue instanceof com.ibm.bpm.coach.List
                            ? storedValue.toJson() : json.toJson(newValue);
                    }
                }

                var system = engine.GlobalAssets.system;
                var event = {type: 'data', path: path, property: prop, value: newValue, userName: system.user_fullName,
                    domId: curFocus, viewId: curView, viewIdMetaData: curViewMeta };

                // publishing data change (collaboration) notification
                
                // do not send notification if it is only a meta data change
                var metaDataOnly = false;
                if (!!path && path.indexOf("_metadata") != -1) {
                	logger.log("controller", "systemChangeHandler", "Meta data only, not sending", logger.FINER);
                	metaDataOnly = true;
                }
                
                if (!metaDataOnly) {
                 logger.log("controller", "systemChangeHandler", "Sending data change event on bus: " + json.toJson(event), logger.FINER);
                 controller.publishNotification(event);
                }
            }
		}); 
	}

	controller.systemListChangeHandler = function (path, object, removedFrom, insertedInto) {
		require(["./engine"], function(engine) {
			var currTimeStamp = new Date().getTime();
			var curFocus;
			var curView;
			var curViewMeta;
			//only send the previous focus dom if the user has tabbed off the field
			//we know this if the focus change happens within a few milliseconds of the data change 
			if (!!controller.previousUserFocus.domId && ( (currTimeStamp - controller.focusTimestamp) < 600) ) {
				curFocus = controller.previousUserFocus.domId;
				curView = controller.previousUserFocus.viewId;
				curViewMeta = controller.previousUserFocus.viewIdMetaData;
			} else {
				curFocus = controller.currentUserFocus.domId;
				curView = controller.currentUserFocus.viewId;
				curViewMeta = controller.currentUserFocus.viewIdMetaData;
			}
			controller.previousUserFocus.domId = controller.currentUserFocus.domId;
			controller.previousUserFocus.viewId = controller.currentUserFocus.viewId;
			controller.previousUserFocus.viewIdMetaData = controller.currentUserFocus.viewIdMetaData;

            if (controller.readOnly == false) {
                // we need to check if coach is in editor mode
                // in order to prevent mirroring notifications
                var objectString = ""
                if (insertedInto != -1) {
                    //this is not a remove operation
                    objectString = json.toJson(object);
                }
                var system = engine.GlobalAssets.system;
                var event = {type: 'data', path: path, removedFrom: removedFrom, insertedInto: insertedInto, value: objectString, userName: system.user_fullName,
                         domId: curFocus, viewId: curView, viewIdMetaData: curViewMeta };

                logger.log("controller", "systemListChangeHandler", "Sending list change event on bus: " + json.toJson(event), logger.FINER);
                // publishing data change notification (collaboration)
                controller.publishNotification(event);
            }
		});
	}

	controller.destroy = function() {
		logger.log("controller", "destroy", "removing controller objects", logger.FINE);
		if(!!controller.progressIcon) {
			controller.progressIcon.parentNode.removeChild(controller.progressIcon);
		}
        controller.destroyed = true;
		controller.currentUserFocus = null;
		controller.previousUserFocus = null;
		controller.focusTextColor = null
		controller.colorList = null;
		controller.userColorMap = null;
		controller.userFocusMap = null;
		controller.focusElementToUserMap = null;
		controller.focusTimestamp = null;
		controller.boundayEventQueue = null;
        controller.validationErrors = null;
		
		if (controller.notificationTimeoutHandler != null) {
			clearTimeout(controller.notificationTimeoutHandler);
			controller.notificationTimeoutHandler = null;
		}
		if (controller.setAsEditorTimeoutHandler != null) {
			clearTimeout(controller.setAsEditorTimeoutHandler);
			controller.setAsEditorTimeoutHandler = null;		
		}
	}

	//process an event originating from an external wrapper (like a portlet) 
	controller.processExternalEvent = function(event, returnTicket) {
		require(["./engine"], function(engine) {
			if (event.name == "setCoachData") {
				//sets the given data on the coach data model
				var result = controller.processDataDeltas(event.data);
				if (result == false) {
					event.result = "error";
				} else {
					event.result = "success";
				}
				event.data = null;
				broadcastHelper.broadcastMessage(event, returnTicket);
			} else if (event.name == "getCoachData") {
				//gets the specified data from the coach and returns it
				var result = {};
				event.result = "success";
				for (var x=0; x<event.data.length; x++) {
					var property = event.data[x];
					var value = null;
					var translatedProperty = null;
					if (property == "tw.local") {
						value = engine.GlobalAssets.local;
					} else {
                    				var pos = property.indexOf("tw.local.");
                    				if (pos == 0) {
                        				// stripping off 'tw.local.'
                    					translatedProperty = property.substring(9);
                        				 value = engine.GlobalAssets.local.get(translatedProperty);
                        			}
                        		}
            		if (value != null && value != undefined) {
						if (!!value.toJson) {
							//need to ensure stateful and list objects are traslated into normal JS objects
							value = json.fromJson(value.toJson());
						}
					} else {
						event.result = "error";
					}
					result[property] = value;
				}
				event.data = result;
				broadcastHelper.broadcastMessage(event, returnTicket);
			} else if (event.name == "triggerBoundaryEvent") {
				//Translate boundary event path to boundary event id
				var boundaryEventId = null;
				if (!!engine.GlobalAssets.boundaryEventData && !!engine.GlobalAssets.boundaryEventData[event.path]) {
                    // coach id must match the current coach
                    if (!!event.coach && event.coach == document.title) {
                        boundaryEventId = engine.GlobalAssets.boundaryEventData[event.path];
                    }
				}
				
				if (!!boundaryEventId) {
					//create closure callback that sends response including correlationid
					var callback = function() {
						event.result = "success";
						broadcastHelper.broadcastMessage(event, returnTicket);
					}
                    var boundaryEventInfo = engine.getBoundaryEventInfo(boundaryEventId);
                    var validateEventInfo = engine.getValidateBoundaryEventInfo();
					controller.handleBoundaryEvent(boundaryEventInfo, validateEventInfo, callback, false);
				} else {
					event.result = "skipped";
					broadcastHelper.broadcastMessage(event, returnTicket);
				}
			} else if (event.name == "getCoachContentBox") {
				event.data = dojo.contentBox(document.body);
				event.result = "success";
				broadcastHelper.broadcastMessage(event, returnTicket);
			}
		});		
	}
	// controller end
	
	return controller;
});
