/*
Licensed Materials - Property of IBM
5725-C95
 Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains a few utilities coach framework uses to manipulate DOM

define(["dojo/_base/array", "dojo/dom-class", "dojo/query", "./CONSTANTS"], function(array, domClass, query, CONSTANTS) {
				
	var domHelper = new Object();

	// only expect input className to be either "ContentBox" or "view"
	// once find a node whose class satisfies, won't go deeper on that node. So the nodes in the return array don't have direct or indirect containership.
	domHelper.shallowQueryByClass = function(node, className, sortResult) {
		var unsortedResult = new Array();
		var deepQueryResults = query(' .' + className, node);
		array.forEach(deepQueryResults, function(oneNode) {
			var parentNode = oneNode.parentNode;
			while(parentNode != node) {
				if(domClass.contains(parentNode, CONSTANTS.CONTENT_BOX) || domClass.contains(parentNode, CONSTANTS.COACH_VIEW)) {
					// if parentNode is view or ContentBox, the node is not what we want
					return;
				}
				parentNode = parentNode.parentNode;
			}
			unsortedResult.push(oneNode);
		});
		
		if(sortResult) {
			return domHelper.sortDomNodes(unsortedResult);
		} else {
			return unsortedResult;
		}		
	};
	
	// sort the nodes to match their orders in the DOM tree
	domHelper.sortDomNodes = function(nodeList) {		
		if(!!nodeList && nodeList.length > 1) {
			var testNode = nodeList[0];
			if(!!testNode.sourceIndex) { // IE
				nodeList.sort(function(nodeA, nodeB) {
					return nodeA.sourceIndex - nodeB.sourceIndex;
				});
			}else if(!!testNode.compareDocumentPosition) {
				nodeList.sort(function(nodeA, nodeB) {
					return 3 - (nodeA.compareDocumentPosition(nodeB) & 6);
				});
			}		
		}
		
		return nodeList;
	}


	return domHelper;
});
