/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach runtime engine main js

define(["dojo/parser", "dojo/_base/json", "dojo/_base/lang", "dojo/json", "dojo/_base/array", "dojo/query", "dojo/dom", "dojo/dom-class", "dojo/_base/xhr", "dojo/_base/connect", "./BPM_CONSTANTS", "./CONSTANTS", "./commonHelper", "./broadcastHelper", "./domHelper", "./dataBindingHelper", "./validationHelper", "./coachViewHelper", "./Stateful", "./Shimmer", "./logger", "./ViewContext", "./templateStore", "./bindingResolver"], function(parser, json, lang, json1, array, query, dom, domClass, xhr, connect, BPM_CONSTANTS, CONSTANTS, commonHelper, broadcastHelper, domHelper, dataBindingHelper, validationHelper, coachViewHelper, stateful, Shimmer, logger, ViewContext, templateStore, bindingResolver) {
	var engine = new Object();

	// Main modules for the framework starts
	
	// backward compatibility support
	engine.CONSTANTS = CONSTANTS;
	
	engine.GlobalAssets = {"topLevelViewScope": {}, "coach": null, "coachClass": null, "scopeClasses": {}, "watchHandles": [], "local" : {}, "epv": {},
					  "team": {"member": [], "manager": []}, "system": {}, "systemData": {}, "resource": {}, "options": {}, boundaryEventData: {} };
	
	engine.localizedMessages = {};
	
	engine._isCoachUnloaded = false;
	
	engine.isProcessingValidation = false;
	
	engine._internalUnloadHandler = [];
	
	engine.start = function(config) {
		// print out the dojo version, which may help view developer and support since the dojo version used by coach framework is not fixed per release.
		commonHelper.consoleLog("dojo version used by Coach is: " + commonHelper.getDojoVersion(), commonHelper.INFO);
		
		logger.init();
		
		engine.initLocalizedMessage();
		engine.initData.call(engine.GlobalAssets);
		templateStore.init(engine.GlobalAssets);
		bindingResolver.init(engine.GlobalAssets);
		engine.initConfig.call(engine.GlobalAssets);
		
		// reference to controller
		engine.controller = config.controller;
		engine.controller.init();
		
		// expose public APIs that are available globally
		engine.exposeGlobalAPIs();		
		engine.initViews.call(engine.GlobalAssets);
		engine.prepareView.call(engine.GlobalAssets);
		engine.addMessageListeners(config.whiteList);
		engine.completeInitialization();
	}
	
	engine.initLocalizedMessage = function() {
		commonHelper.timestamp("Enter engine.initLocalizedMessage()");
		logger.log("engine", "initLocalizedMessage", "is called", logger.INFO);
		var jsonResURL = "jsonResBundleLoader?bundle=JavaScriptMessages";
		
		xhr.get( {
		    url: jsonResURL,
		    handleAs: "json",
		    load: function(responseObject, ioArgs) {
		    	engine.localizedMessages = responseObject;		
				logger.log("engine", "initLocalizedMessage", "localizedMessages: " + engine.localizedMessages);
		    }
		});
	};

    // Loads localized messages in synchronous fashion if only they are not yet initialized.
    // This function is required to avoid timing issues in collaboration scenarios.
    engine.forceLocalizedMessages = function() {
        logger.log("engine", "forceLocalizedMessages", "is called", logger.INFO);
        var jsonResURL = "jsonResBundleLoader?bundle=JavaScriptMessages";

        var initialized = false;
        // check if messages are empty
        for (var prop in engine.localizedMessages) {
            if (engine.localizedMessages.hasOwnProperty(prop)) {
                initialized = true;
                break;
            }
        }
        // if messages are empty we need to pull them in a sync way
        if (initialized === false) {
            xhr.get( {
                url: jsonResURL,
                handleAs: "json",
                sync: true,
                load: function(responseObject, ioArgs) {
                    engine.localizedMessages = responseObject;
                    logger.log("engine", "forceLocalizedMessages", "localizedMessages: " + engine.localizedMessages);
                }
            });
        }
    }

    engine._getViewScope = function(parentViewScope, viewDomNode) {
		var viewDivId = viewDomNode.getAttribute("id");
		var viewId = viewDomNode.getAttribute(engine.CONSTANTS.VIEW_ID);
    	return parentViewScope.context.subview[viewId][viewDivId];
    }	    
    
    engine.exposeGlobalAPIs = function() {
    	// global object that provides access to functions that are globally available
    	com_ibm_bpm_global = {};
    	
    	/*
    		This property is available globally. It  holds the list of top level coach view instances in current window object.
    	*/
		com_ibm_bpm_global.topLevelCoachViews = engine.GlobalAssets.topLevelViewScope;
		
		// Note: this global API is not documented yet. Only let customer use it when it is really necessary.
		com_ibm_bpm_global.preference = {};
		com_ibm_bpm_global.preference.highlightUnresolvedBinding = true;
		
		com_ibm_bpm_global.coachView = {};
		
		/*
			This function is available globally. It returns the coach view instance that is described 
			by the input parameter "controlIds" which is a list of control ids.   
			The list of control ids should completely specify the view hierarchy, starting from the top level view, 
			all the way down to the coach view whose instance is expected to be returned.
			
			If  the list of control id doesn't describe the entire view hierarchy properly, this function returns null.
			
			If a coach view repeats, first instance of that view is chose to continue.
			
			For example, let's say we have following view hierarchy: 
				ViewA (Top Level View, control id is  id_ViewA)
				    ViewB  (control id is id_ViewB, binds to list)
				        ContentBox
				           ViewC (control id is id_ViewC)
			Then, 
			If controlIds is ["id_ViewA"], the return is the instance of ViewA.
			If controlIds is ["id_ViewA", "id_ViewB"], the return is the instance of ViewB.
			If controlIds is ["id_ViewA", "id_ViewB", "id_ViewC"], the return is the first instance of ViewC. 
				If ViewC doesn't have any instance under ViewB (e.g. the binding list of ViewB is empty), the return is null.	
			If controlIds is ["id_ViewB"], the return is null, because input parameter doesn't start with the control id of top level view.
			If controlIds is ["id_ViewA", "id_ViewC"]) returns null, because the control id of ViewB  is not on the input list.
		*/
		com_ibm_bpm_global.coachView.byControlId = function(controlIds) {
			return coachViewHelper.findFirstViewInstance(engine.GlobalAssets.topLevelViewScope, controlIds);
		}		
		
		/*
			This function is available globally. It returns the coach view instance that is uniquely described by the input parameter "domId". 
			If specified domId is not the id of a coach view instance's DOM node, this function returns null.
		*/
		com_ibm_bpm_global.coachView.byDomId = function(domId) {
			return coachViewHelper.findViewScope(domId, engine.GlobalAssets.topLevelViewScope);
		}
		
    }
	    
	
	engine.initViews = function() {
		commonHelper.timestamp("Enter engine.initViews()");
		logger.log("engine", "initViews", "is called", logger.INFO);
		
		// find out all top level views
		var topLevelViewList = query('#mainBody .' + engine.CONSTANTS.TOP_LEVEL);
		array.forEach(topLevelViewList, function(item, index) {
			var topLevelviewId = item.getAttribute(engine.CONSTANTS.VIEW_ID);
			commonHelper.timestamp("Call engine.initView() for top level view " + topLevelviewId);
			var oneTopLevelView = engine.initView.apply(this, [item, null]);
			commonHelper.timestamp("Finish engine.initView() for top level view " + topLevelviewId);
			this.topLevelViewScope[topLevelviewId] = oneTopLevelView;
		}, this);
		engine._isCoachUnloaded = false;
		
		// If no view has data binding, engine.GlobalAssets.local is not initialized yet.
		// So initialize it here accordingly
		if (engine.GlobalAssets.local == undefined || engine.GlobalAssets.local == null ||
		    !(engine.GlobalAssets.local instanceof com.ibm.bpm.coach.Stateful)) {
			engine.GlobalAssets.local = new stateful(engine.GlobalAssets.local, "local");
		}
		
	    var CoachClass = this.coachClass;
	    if(!!CoachClass) {
			try {			
				this.coach = new CoachClass();
		    } catch(ex) {
		    	var errorMsg = "engine.initViews() caught Exception when calling view constructor of Coach, Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
				return;
		    }
			try {			
				this.coach.load();
		    } catch(ex1) {
		    	var errorMsg1 = "engine.initViews() caught Exception when calling load() on Coach object, Exception: " + ex1;
		    	commonHelper.consoleLog(errorMsg1, commonHelper.ERROR);
				return;
		    }
	    }	    
		

		commonHelper.timestamp("Exit engine.initViews()");
	};

	 
	/*
	 * Start of configuration data handling
	 *
	 */
	 
	/**
	 * This function is used by code gen team to pass in the needed information to runtime 
	 * It is for the static value case, ie, user defined an option, the configuration value is static: type in by the user
	 */
	engine.resolveStaticOptionValue = function(configObjId, optionKey, optionValue){
	
	    //use the config object id and the option key to construct a unique key
		var optionValue = {staticOptionStr : "config." +  configObjId + "." + optionKey, value: optionValue };
		if(optionKey === "_metadata.visibility" || optionKey === "_metadata.labelVisibility" ){
			optionValue.isStandardValue = true;
		}
		return optionValue;
	}

	/**
	 * This function is used by code gen team to pass in the needed information to runtime 
	 * User defined an option, the configuration value for this option points to a bo or points to another option
	 * engine.resolveOptionValue.call(this, configId, "_metadata.visibility", "local.dataObject02"),
	 * engine.resolveOptionValue.call(this, configId, "_metadata.visibility", "config.config1._metadata.visibility"),
	 */
	engine.resolveOptionValue = function(configObjId, optionKey, optionBindingStr){
		var optionValue;
		if (!!optionBindingStr && optionBindingStr.indexOf("config.") == 0)
		{
			optionValue = {staticOptionStr : optionBindingStr};
		}
		else{
			//for bo case
			optionValue = {bindingStr : optionBindingStr};
		}
		return optionValue;
	}

	/**
	 * This function is used to get the value of an option
	 */
	engine.getOptionValue = function(configId, optionKey, isMetadata){
		var options = this.GlobalAssets.options[configId];
		if (!options){
			return;
		}
		//fix for defect 80359. we modified the template config object here so the change is limited to minor code path
		var optionValue, value;
		if (!!isMetadata){
			optionValue = options._metadata[optionKey];
		}
		else {
			optionValue = options[optionKey];
		}
		if (!!optionValue && (!!optionValue.staticOptionStr)){
			value = optionValue.value;
		}
		var binding = bindingResolver._resolveOptionBinding(options, optionKey, isMetadata);
		// for non-staticOptionStr cases, vlaue is still undefined
		if(!!value) {
			binding.value = value;
		}
		return !!binding ? binding.get("value") : null;
	}
	
	/* 
	 * This is for repeatting case
     * We first clone the config object
	 * If an option points to something under an array, then the index in binding will be adjusted
	 * If an option's value is a non bo object, we adjust the staticOptionStr to use the new config object id
	 * 
	 * static value      object                own: replace the configId portion of the staticOptionStr with the new configId
	 *                   function              same
	 * refrence to 
	 * 			local(non-array)         same
	 *          local(array)             points to the right index in array
	 *          
	 * reference to  static              same
	 *               static array        same array object
	 *               function            same
	 */
	engine.cloneAndAdjustDataConfig = function(viewElement, bindingIndex, maxNumOfUnresolvedRepeating, templateConfigIdToInstanceConfigIdMap){
		
		var tempViewCfgBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_CONFIG);
		if(!tempViewCfgBinding){
			return;
		}
	    var configObj = engine.GlobalAssets.options[tempViewCfgBinding];
		var clonedObjId = "config_" + viewElement.id;
		
		//clone the object and store it under GlobalAssets.options
		var clonedObj = {"_metadata" : {}, "templateConfigId" : tempViewCfgBinding, "configId": clonedObjId};
		
		// add info of newly cloned config to templateConfigIdToInstanceConfigIdMap
		engine._processOneConfig(templateConfigIdToInstanceConfigIdMap, clonedObj);
		
		var configValue;
		var bindingStr;
		var metaDataOptionValue;
		for(var configKey in configObj) {
			configValue = configObj[configKey];
		    
			if (configKey == "_metadata"){
			for(var metadataKey in configValue){				
				metaDataOptionValue = configObj[configKey][metadataKey];
				if (!metaDataOptionValue){
					continue;
				}
				//check if it points to an bo
				if (configValue[metadataKey].bindingStr){	
					bindingStr = configValue[metadataKey].bindingStr;
					//adjust index if needed
					bindingStr = dataBindingHelper.updateBindingStr(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating); 
					clonedObj[configKey][metadataKey] = {"bindingStr" : bindingStr };
					
				}
				//for static value case
				else if(!!configValue[metadataKey].staticOptionStr){
						// defect 96461
						var staticOptionStr1 = configValue[metadataKey].staticOptionStr;
						//adjust index if needed
						staticOptionStr1 = dataBindingHelper.updateBindingStr(staticOptionStr1, bindingIndex, maxNumOfUnresolvedRepeating); 

						// if metaDataOptionValue has been resolved, call its getter to get staticOptionValue, otherwise, use metaDataOptionValue.value as staticOptionValue
						var staticOptionValue = metaDataOptionValue.value;
						if(metaDataOptionValue.boundObject && metaDataOptionValue.get) {
							staticOptionValue = metaDataOptionValue.get("value");
						}
						clonedObj[configKey][metadataKey] =
                        {staticOptionStr: staticOptionStr1,
						 value: staticOptionValue
						}
						
						// following logic handles 2 scenarios: 
						// a) clonedObj[configKey][metadataKey].staticOptionStr is "config.configABC....". 
						//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ"), and we have added configABC --> configXYZ to templateConfigIdToInstanceConfigIdMap.
						//    Now it's time to update the config reference (replacing "configABC" with "configXYZ".
						// b) clonedObj[configKey][metadataKey].staticOptionStr is "config.configDEF....", 
						//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ").
						//    Now we need to check if configDEF has been cloned earlier, in the scope of from currentView up to the topLevelView. If so, replace "configDEF" with its cloned instance "configLMN". 
						clonedObj[configKey][metadataKey].staticOptionStr = engine._updateStaticOptionStr(staticOptionStr1, templateConfigIdToInstanceConfigIdMap);
				}
				else{
					clonedObj[configKey][metadataKey] = metaDataOptionValue;
				}
			}
			}
			else{
				if(!configValue){
					continue;
				}
				//check if it points to an object
				if (!!configValue.bindingStr){	
					bindingStr = configValue.bindingStr;
					//adjust index if needed
					bindingStr = dataBindingHelper.updateBindingStr(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating); 
					clonedObj[configKey] = {"bindingStr" : bindingStr };
				}
				//static value which maybe shared	
				else if (!!configValue.staticOptionStr) {
					var staticOptionStr1 = configValue.staticOptionStr;
					//adjust index if needed
					staticOptionStr1 = dataBindingHelper.updateBindingStr(staticOptionStr1, bindingIndex, maxNumOfUnresolvedRepeating); 
				
                   var optionValue = configValue.value;
                    if(configValue.boundObject && configValue.get) {
                        optionValue = configValue.get("value");
                    }
                    clonedObj[configKey] = {value: optionValue};
					
					// following logic handles 2 scenarios: 
					// a) staticOptionStr is "config.configABC....". 
					//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ"), and we have added configABC --> configXYZ to templateConfigIdToInstanceConfigIdMap.
					//    Now it's time to update the config reference (replacing "configABC" with "configXYZ".
					// b) staticOptionStr is "config.configDEF....", 
					//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ").
					//    Now we need to check if configDEF has been cloned earlier, in the scope of from currentView up to the topLevelView. If so, replace "configDEF" with its cloned instance "configLMN". 
					clonedObj[configKey].staticOptionStr = engine._updateStaticOptionStr(staticOptionStr1, templateConfigIdToInstanceConfigIdMap);
				}
				else{//simply copy over the value
					clonedObj[configKey] = 	configValue;			
				}
			}
		}	 
		
		engine.GlobalAssets.options[clonedObjId] = clonedObj;
		
		//update the view    
		viewElement.setAttribute(engine.CONSTANTS.DATA_CONFIG, clonedObjId);
	}
	
	engine._updateStaticOptionStr = function(staticOtionStr, templateConfigIdToInstanceConfigIdMap) {
		var pathArray = staticOtionStr.split(".");
		var referencedConfigId = pathArray[1];
		var newReferencedConfigId = templateConfigIdToInstanceConfigIdMap[referencedConfigId];
		if(!!newReferencedConfigId) {
			pathArray[1] = newReferencedConfigId;
			return pathArray.join(".");
		}
		
		return staticOtionStr;
	}
	
	// return the plain value represented by specified binding object (wrapper object that has boundObject, setter and getter functions)
	engine.getPlainValue = function(bindingObject) {
		if(!!bindingObject) {
			return dataBindingHelper.getPlainValue(bindingObject.boundObject, bindingObject.property);
		}
		return null;
	}
	
	engine.notifyDataConfigBindingChange = function(scopeObject){
		var dataConfig = scopeObject.context.options;
		var optionValue;
		if (!dataConfig){
			return;
		}
		
		var bindingForMetadata;
		for(var optionKey in dataConfig) {	
		    if (optionKey == "_metadata"){
				for (var metadataKey in dataConfig[optionKey]){
					bindingForMetadata = dataConfig[optionKey][metadataKey];
					// if config option binds to resource, system, epv etc, bindingForMetadata doesn't have boundObject at all. And this kind of binding is supposed to be readonly, no change event should be fired.
					if (!!bindingForMetadata.bindingStr && !!bindingForMetadata.changed && !!bindingForMetadata.boundObject){
						   //call the change method
						   var event = { type : "config", property: optionKey + "." + metadataKey, oldVal: null, newVal: engine.getPlainValue(bindingForMetadata)};
						   scopeObject._change(event);
                           // PMR 124679 Need to reset the .changed attribute
                           if ((typeof bindingForMetadata.changed != 'undefined') && (bindingForMetadata.changed == true)) {
                                scopeObject.context.options[optionKey][metadataKey].changed = false;
                           }
					}
				}
			}
			else{			
			   optionValue = dataConfig[optionKey];
				// if config option binds to resource, system, epv etc, optoinValue doesn't have boundObject at all. And this kind of binding is supposed to be readonly, no change event should be fired.
			   if (!!optionValue.bindingStr && !!optionValue.changed && !!optionValue.boundObject){
				   //call the change method
				   var event = { type : "config", property:optionKey, oldVal: null, newVal: engine.getPlainValue(optionValue)};
				   scopeObject._change(event);
                   // PMR 124679 Need to reset the .changed attribute
                   if ((typeof optionValue.changed != 'undefined') && (optionValue.changed == true)) {
                        scopeObject.context.options[optionKey].changed = false;
                   }
			   }
			}		
		}		
	}	
			 	
	//##Data Binding end
	
	engine.initView = function(viewElement, parentViewScope) {
		logger.log("engine", "initView ", "viewElement: " + viewElement.id);
		
		var tempViewDivId = viewElement.getAttribute("id");
		var tempViewId = viewElement.getAttribute(engine.CONSTANTS.VIEW_ID);
		var tempViewType = viewElement.getAttribute(engine.CONSTANTS.VIEW_DATA_TYPE);
		var tempViewDataBinding = engine.rtDataBinding._get(viewElement);
		var tempViewCfgBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_CONFIG);
				
	//##Data Binding start
		//data
		var dataBinding = bindingResolver.resolveBinding(tempViewDataBinding);

		//config
		var cfgBinding = this.options[tempViewCfgBinding]; 
		if (!!cfgBinding) {
			try {
				bindingResolver.resolveOptionsBinding(cfgBinding);
			} catch(ex) {
		    	var errorMsg = "engine.initView() caught Exception when calling bindingResolver.resolveOptionsBinding(), Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
		    	return;
			}
		}
		
		var subclass = this.scopeClasses[tempViewType];
		try {
			var viewScope = new subclass(new ViewContext(), viewElement, dataBinding, cfgBinding);
	    } catch(ex) {
	    	var errorMsg = "engine.initView() caught Exception when calling view constructor for " + tempViewDivId + ", Exception: " + ex;
	    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			return;
	    }
	    
		if(tempViewDataBinding) {
			viewScope._registerCallBackForDataBinding(tempViewDataBinding);
		}
		
		viewScope._registerCallBackForOptions();
	//##Data Binding end
		
		if (!!parentViewScope) {
			// Set the scope to the parent
			var viewId = viewElement.getAttribute(engine.CONSTANTS.VIEW_ID);
			var divId = viewElement.getAttribute("id");
			if(!parentViewScope.context.subview[viewId]) {
				parentViewScope.context.subview[viewId] = {};
			}
			parentViewScope.context.subview[viewId][divId] = viewScope;
			viewScope.context._parentView = parentViewScope;
		}
		
		// continue to build the children views
		viewScope._buildChildren();

		viewScope._preLoad();
		
		// Call the view onLoad lifecycle method
		engine.callViewLifeCycleAPI(viewScope, "load", CONSTANTS.ELAPSED_TIME_THRESHOLD_LOAD);
	    
	    viewScope._highlightBindingIssue();   
	    
	    // if the performance of following line is concern, we can collect bpmIds when the binding is resolved initially.
	    viewScope._bpmIds = dataBindingHelper.collectBpmIds(tempViewDataBinding);
	    
		return viewScope;
	};
	
	engine.callViewLifeCycleAPI = function(viewScope, apiName, elapsedTimeThreshold, parameter) {
		try {		
			var startTime = commonHelper.isDebugMode() ? (new Date()).getTime() : null;
			viewScope[apiName].call(viewScope, parameter);
			if(startTime) {
				var elapsedTime = (new Date()).getTime() - startTime;
				if(elapsedTime >= elapsedTimeThreshold) {
					var info = apiName + "() for " + viewScope.context.element.id + " used " + elapsedTime + "ms";
					commonHelper.timestamp(info);
				}
			}
	    } catch(ex) {
	    	try {
		    	var message = "Coach framework caught exception when calling Life cycle API " + apiName + "() on view " + viewScope.context.element.id;
		    	message +=  ", Exception: " + ex;
		    	commonHelper.consoleLog(message, commonHelper.ERROR);
	    	} catch(ex1) {
		    	commonHelper.consoleLog("Even failed to gracefully log this Exception: " + ex, commonHelper.ERROR);
	    	}
	    }
	}

	engine.prepareView = function() {
		commonHelper.timestamp("Enter engine.prepareView()");
		logger.log("engine", "prepareView", "is called", logger.INFO);
		for (var viewId in this.topLevelViewScope) {
			   var viewScope = this.topLevelViewScope[viewId];
			   engine.callView(viewScope);
		}
		
		if(!!this.coach) {
			try {			
				this.coach.view();
		    } catch(ex) {
		    	var errorMsg = "engine.prepareView() caught Exception when calling view() on Coach object, Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
		    }
		}
		
		commonHelper.timestamp("Exit engine.prepareView()");
	};

	engine.callView = function(viewScope) {
		// Call nested view view first
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					engine.callView(subViewScope);
				}
			}		
		}
		
	    // Call the view view lifecycle method		
		engine.callViewLifeCycleAPI(viewScope, "view", CONSTANTS.ELAPSED_TIME_THRESHOLD_VIEW);
	    
	    viewScope._monitorRelatedListsIfNeeded();
	    
	    // Display the content
		// viewScope.context.element.style.display = 'block';
		var viewClass = viewScope.context.element.getAttribute('class');
		if(!domClass.contains(viewScope.context.element, engine.CONSTANTS.COACH_VIEW_SHOW)) {
			domClass.add(viewScope.context.element, engine.CONSTANTS.COACH_VIEW_SHOW);
		}
	};

	//The function, adjustIdAndBinding, is called from 2 different places
	//one is from createView. The node( divNode )is cloned and passed into createView.
	//another is from _repeatContentBoxs, the node, divNode, is cloned in _repeatContentBoxs and passed into this function
	//I need to clone the data config object and adjust the binding str based on the index.
	//If clone the data before this function is called, then I need to do recursive traveral and clone 
	//Rather, do it here, so that it use the same traveral code
	engine.adjustIdAndBinding = function(divNode, parentViewScope, bindingIndex) {
		if(divNode.tagName != "DIV")
			return null;
		
		// var idSuffix = "_" + bindingIndex;  // bindingIndex is not enough as idSuffix due to the dynamic delete/add
		var idSuffix = "_r" + parentViewScope._counter;
		parentViewScope._counter++;

		var templateConfigIdToInstanceConfigIdMap = engine._prepareTemplateConfigIdToInstanceConfigIdMap(parentViewScope);
		if(engine.isContentBox(divNode)) {
			// this divNode is a contentBox
			engine._adjustIdAndBindingForContentBox(divNode, idSuffix, bindingIndex, 1, parentViewScope, templateConfigIdToInstanceConfigIdMap);
		} else {
			// this divNode is expected to be a view
			engine._adjustIdAndBindingForView(divNode, idSuffix, bindingIndex, 1, parentViewScope, templateConfigIdToInstanceConfigIdMap);
		}
	};
	
	engine._prepareTemplateConfigIdToInstanceConfigIdMap = function(viewScope) {
		var templateConfigIdToInstanceConfigIdMap = {};
		var oneView = viewScope;
		while(!!oneView) {
			var viewOptions = oneView.context.options; 
			engine._processOneConfig(templateConfigIdToInstanceConfigIdMap, viewOptions);
			oneView = oneView.context.parentView();
		} 
		return templateConfigIdToInstanceConfigIdMap;
	}
	
	engine._processOneConfig = function(templateConfigIdToInstanceConfigIdMap, aConfig) {
		if(!!aConfig && !!aConfig.templateConfigId) {
			templateConfigIdToInstanceConfigIdMap[aConfig.templateConfigId] = aConfig.configId;
		} 
	}
	
	engine._adjustIdAndBindingForContentBox = function(conentBoxDomNode, idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, templateConfigIdToInstanceConfigIdMap) {
		// adjust id
		conentBoxDomNode.setAttribute('id', conentBoxDomNode.id + idSuffix);
		
		// handle nested Views
		var immediateViewList = domHelper.shallowQueryByClass(conentBoxDomNode, engine.CONSTANTS.COACH_VIEW, false);
		for(var i=0; i<immediateViewList.length; i++) {
			var newMap = commonHelper.shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForView(immediateViewList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}
		
		// handle nested ContentBox
		var immediateContentBoxList = domHelper.shallowQueryByClass(conentBoxDomNode, engine.CONSTANTS.CONTENT_BOX, false);
		for(var i=0; i<immediateContentBoxList.length; i++) {
			var newMap = commonHelper.shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForContentBox(immediateContentBoxList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}			
	}
	
	
	engine._adjustIdAndBindingForView = function(viewDomNode, idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, templateConfigIdToInstanceConfigIdMap) {
		// adjust id
		viewDomNode.setAttribute('id', viewDomNode.id + idSuffix);

		var viewDataBinding = dataBindingHelper.rtDataBinding._get(viewDomNode);		
		
		// if view's data binding is a config option, need to make sure this config option is up-to-date (in case the config option has been cloned)
		if(viewDataBinding.indexOf("config") == 0) {
			var newConfigBindingStr = engine._updateStaticOptionStr(viewDataBinding, templateConfigIdToInstanceConfigIdMap);
			if(viewDataBinding != newConfigBindingStr) {
				dataBindingHelper.rtDataBinding._set(viewDomNode, newConfigBindingStr);
				viewDataBinding = newConfigBindingStr;
			}
		}

		// now adjust this view's databinding
		var numOfIndex = dataBindingHelper.getNumberOfIndex(viewDataBinding);
		if(numOfIndex >= maxNumOfUnresolvedRepeating) {
		
			if(maxNumOfUnresolvedRepeating == 1 && numOfIndex == 1) {
				// binding of viewDomNode recieves the index value (currentItem) from the binding list of its parent view
				// if the length of viewDomNode binding list could be differnt from that of parent view' binding list,
				// we need to manage this kind of situation.
				var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
				var involvedList = viewDataBinding.substring(0, viewDataBinding.indexOf(indexWithBracket));
				viewControlRepeating._addRelatedList(viewDomNode.id, involvedList);
			}				
		
			var newBinding = dataBindingHelper.updateBindingStr(viewDataBinding, bindingIndex, maxNumOfUnresolvedRepeating, numOfIndex);
			dataBindingHelper.rtDataBinding._set(viewDomNode, newBinding);
		} else {
			// now numOfIndex < maxNumOfUnresolvedRepeating, we will adjust the index when hande next repeat. 
		}
		
		// now handle the dataConfig
		engine.cloneAndAdjustDataConfig(viewDomNode, bindingIndex, maxNumOfUnresolvedRepeating, templateConfigIdToInstanceConfigIdMap);	
				
		// handle nested Views
		var immediateViewList = domHelper.shallowQueryByClass(viewDomNode, engine.CONSTANTS.COACH_VIEW, false);
		for(var i=0; i<immediateViewList.length; i++) {
			var newMap = commonHelper.shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForView(immediateViewList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}
		
		// handle nested ContentBox
		var immediateContentBoxList = domHelper.shallowQueryByClass(viewDomNode, engine.CONSTANTS.CONTENT_BOX, false);
		var newMaxNumOfUnresolvedRepeating = maxNumOfUnresolvedRepeating;
		var newViewControlRepeating = viewControlRepeating;
		if(dataBindingHelper.isBindingArray(viewDataBinding)) {
			newMaxNumOfUnresolvedRepeating++;
			// repeating of immediateContentBoxList[] is controlled by viewDomNode. but view object for viewDomNode is not
			// creaed yet. And newMaxNumOfUnresolvedRepeating won't be used in the called method because there is more than one
			// unresolved repeating at this moment.
			newViewControlRepeating = null;
		}
		for(var i=0; i<immediateContentBoxList.length; i++) {
			var newMap = commonHelper.shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForContentBox(immediateContentBoxList[i], idSuffix, bindingIndex, newMaxNumOfUnresolvedRepeating, newViewControlRepeating, newMap);
		}
		
	}

//************************* remove this part later, replace with the ones on dataBindingHelper	
	// rtDataBinding provides 3 helper method for resolving List related data-binding during runtime.
	engine.rtDataBinding = new Object();
	engine.rtDataBinding._get = function(viewElement) {
		var rtBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_BINDING_RT);
		if(!rtBinding) {
			return viewElement.getAttribute(engine.CONSTANTS.DATA_BINDING);
		} else {
			return rtBinding;
		}
	}
	engine.rtDataBinding._set = function(viewElement, bindingValue) {
		viewElement.setAttribute(engine.CONSTANTS.DATA_BINDING_RT, bindingValue);
	}
	engine.rtDataBinding._delete = function(viewElement) {	
		// viewElement.removeAttribute(engine.CONSTANTS.DATA_BINDING_RT);
	}
//************* end

	engine.isContentBox = function(node) {
		if(!!node) {
			return domClass.contains(node, engine.CONSTANTS.CONTENT_BOX);
		}
		return false;
	};
	
	// check if the provided contentBox is view managed
	engine.isViewManagedContentBox = function(contentBoxNode) {
		return contentBoxNode.getAttribute(engine.CONSTANTS.IS_VIEW_MANAGED) == "true";
	}
	
	// contentBoxDiv can be framework managed ContentBox or view managed ContentBox
	// newViewArray is optional. If provided, it stores the new views created by this method.
	engine.buildContentBox = function(parentView, contentBoxDiv, newViewArray) {
		var immediateViewList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.COACH_VIEW, true);
		var oneDirectView;
		array.forEach(immediateViewList, function(item, index) {
			oneDirectView = engine.initView.apply(engine.GlobalAssets, [item, parentView]);
			if(!!newViewArray) {
				newViewArray.push(oneDirectView);
			}
		});
		
		// find the direct ContentBox under this contentBox.
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.CONTENT_BOX, true);
		// only care about the framework managed ContentBox. view managed ContentBox will be built by its owner view.
		for(var k=0; k<immediateContentBoxList.length; k++) {
			if(!engine.isViewManagedContentBox(immediateContentBoxList[k])) {
				// Framework managed
				engine.buildContentBox(parentView, immediateContentBoxList[k], newViewArray);
			}
		}
	}
	
	// delete all subviews that reside in the specified contentBox, including 
	// 1) the views owned by specified contentBox directly
	// 2) the views under the framework or view managed contentBox that is owned by specified contentBox
	engine.deleteContentBox = function(contentBoxDiv, ownerView) {
		var directViewNodes = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.COACH_VIEW);
		array.forEach(directViewNodes, function(item, index) {
			engine.deleteView(item, ownerView);				
		}, ownerView);

		// find the direct ContentBox under this contentBox.
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.CONTENT_BOX, true);
		// delete all contentBox regardless it's framework managed or view managed.
		for(var k=0; k<immediateContentBoxList.length; k++) {
			engine.deleteContentBox(immediateContentBoxList[k], ownerView);
		}
	}
	
	engine.deleteView = function(subviewDivNode, ownerView) {
		var viewId = subviewDivNode.getAttribute(engine.CONSTANTS.VIEW_ID);
		var divId = subviewDivNode.getAttribute("id");
		var subviews = ownerView.context.subview[viewId];
		if(!!subviews && !!subviews[divId]) {
			engine.callUnload(subviews[divId]);
			delete subviews[divId];
		}
	}

	// Future consideration - engine holds a map between divId and viewScope object.
	// Pro: search vewScope object by div id (like following few methods) will perform better and looks smple.
	// Con: have to maitain the map properly when add/delete occurs
	// The performance of following methods (about finding viewScope) should be ok since it just traverse DOM tree straightly up/down.

	engine.findEndStateId = function(bdEventId) {
	    var systemData = engine.GlobalAssets.systemData;
	    if (systemData.boundaryEventInformation) {
	        for (var i=0; i<systemData.boundaryEventInformation.length; i++) {
	            if (systemData.boundaryEventInformation[i].boundaryEventID == bdEventId) {
	                return systemData.boundaryEventInformation[i].endStateID;
	            }
	        }
	    }
	    return "";
	}

    // helper function for retrieving boundary event information by the given boundary event id
    engine.getBoundaryEventInfo = function(bdEventId) {
        var systemData = engine.GlobalAssets.systemData;
        if (systemData.boundaryEventInformation) {
            for (var i=0; i<systemData.boundaryEventInformation.length; i++) {
                if (systemData.boundaryEventInformation[i].boundaryEventID == bdEventId) {
                    return {boundaryEventId: bdEventId,
                            boundaryEventEndStateId: systemData.boundaryEventInformation[i].endStateID,
                            fireValidation: systemData.boundaryEventInformation[i].fireValidation};
                }
            }
        }
        return {boundaryEventId: bdEventId, boundaryEventEndStateId: "", fireValidation: 0};
    }

    // helper function for retrieving validation boundary event id and end state id
    engine.getValidateBoundaryEventInfo = function() {
        var systemData = engine.GlobalAssets.systemData;
        if (!!systemData.validateBoundaryEventInformation) {
            // for now we assume that there may be only one validation boundary event per coach
            if (!!systemData.validateBoundaryEventInformation[0]) {
                return {validateBoundaryEventId: systemData.validateBoundaryEventInformation[0].boundaryEventID,
                        validateBoundaryEventEndStateId: systemData.validateBoundaryEventInformation[0].endStateID};
            }
        }
        return null;
    }

	// Helper API for handling debug response
	engine.handleDebugMode = function(nextURL) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);

		// redirect to the next URL
		window.location.assign(nextURL);
	}
	
	// Helper API for handling boundary event response
	engine.TaskCompleted = function(nextURL, nextTaskId) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			if (!!nextTaskId) {
				var event = {
					"name": "onCompleted",
					"taskID" : engine.GlobalAssets.systemData.zTaskId,
					"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
					"parameters": [nextTaskId]
				};
				broadcastHelper.broadcastMessage(event);
			}
			// Defect 70546.  We have to show task completed page in case of Process Admin Administrative service
			// if(!!nextTaskId) {
			if(!!nextURL) {
				//commenting this out for defect 79695, breaks auto-flow
				//if(window.parent.location.pathname == "/mum/enabler") {
				//      return;
				//}
				window.location.assign(nextURL);
			}
			// }
		} else {
			// redirect to the next URL in the non-embedded case
			if(!!nextURL) {
				if ((window.parent.location.pathname == "/mum/enabler") && (nextURL.indexOf("tm_process_finished.lsw") > -1)) {
					return;
				}
				window.location.assign(nextURL);
			}
		}
	}
	
	engine.TaskCompletedNotOwned = function(nextURL) {
		engine._taskCompletedOthers(nextURL);
	}

	engine.TaskCompletedAlreadyClosed = function(nextURL) {
		engine._taskCompletedOthers(nextURL);
	}
	
	engine._taskCompletedOthers = function(nextURL) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		if(!!nextURL) {
			window.location.assign(nextURL);
		}
	}	
	
	engine.TaskPostponed = function(nextURL) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onSuspended",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": []
			};
			broadcastHelper.broadcastMessage(event);
		} else {
			if(!!nextURL) {
				window.location.assign(nextURL);
			}
		}
	}


	engine.StateChange = function(nextURL) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		// redirect to the next URL
		window.location.assign(nextURL);
	}


	engine.ErrorBoundaryEvent = function(message) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onError",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": [message]
			};
			broadcastHelper.broadcastMessage(event);
		}
		// Showing the coachNG error page.  This is necessary in particular in Process Admin Administrative service case.
		window.location.assign("html/coachNGError.jsp");
	}

	// Helper API for handling collab related event
	engine.handleFocusEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var outlineColor = !!event.outlineColor ? event.outlineColor : "red"; // now use "red" as default color. refine later
			var message = !!event.userName ? event.userName : engine.localizedMessages.coachNG_unknown;
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "brown"; // now use "brown" as default background color. refine later
			
		    var collaborationEvent = {"type": "focus", "domId": domId, "outlineColor": outlineColor, "color": color, "bgColor": bgColor, "message": message};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleBlurEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);	
					
		    var collaborationEvent = {"type": "blur", "domId": domId};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleMouseOnEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "red"; // now use "red" as default background color. refine later
			var message = !!event.message ? event.message : engine.localizedMessages.coachNG_unknown;			
		    
		    var collaborationEvent = {"type": "mouseOn", "domId": domId, "color": color, "bgColor": bgColor, "message": message};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleMouseOffEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);			
		    
		    var collaborationEvent = {"type": "mouseOff", "domId": domId};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleCollabDataChangeEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "red"; // now use "red" as default background color. refine later
			var message = !!event.message ? event.message : engine.localizedMessages.coachNG_unknown;
			var userName = !!event.userName ? event.userName : engine.localizedMessages.coachNG_unknown_user;
			var value = !!event.value ? event.value : engine.localizedMessages.coachNG_unknown_value;			

		    var collaborationEvent = {"type": "dataChange", "domId": domId, "color": color, "bgColor": bgColor, "message": message, "userName": userName, "value": value};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}
	
	engine.handleCustomCollaborationEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {			
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, event);
		}
	}
	
	// controller calls this internal API when it receives validation information of one boundary event
	engine.handleValidationEvent = function(rawEvent) {
		try {		
			var topLevelViews = engine.GlobalAssets.topLevelViewScope;
			// prepare validationHelper, make sure it's clean
			validationHelper.initialize();
			var hasValidationEvent = validationHelper.validateAllViews(rawEvent, topLevelViews);
			if(hasValidationEvent) {	
				engine.deliverValidationEvents();
				// broadcast that all validation events have been delivered.
				connect.publish("com.ibm.bpm.coach.validationCompleted", {});
			} else {
				validationHelper.clearTransientInfo();	
			}
		} catch(ex) {
			logger.log("engine", "engine.handleValidationEvent", "ex: " + ex);
		}
	}

	// controller calls this internal API right before it submit boundary event.
	// engine will ask all the views that have validation errors to clear themself
	engine.clearValidationEvent = function() {
		var treeNode = validationHelper.viewTree.rootNode.firstChild;
		if(!!treeNode) {
			engine._deliverValidationEvents(treeNode, validationHelper.viewDomIdToViewMap, true);

			// now we have delivered all the clear events, it's time for framework to clear all the memory of last validation cycle.
			validationHelper.initialize();

			// broadcast that all validation events (clear) have been delivered.
			connect.publish("com.ibm.bpm.coach.validationCleared", {});
		}
	}
	
	engine.deliverValidationEvents = function() {
		if(engine.isProcessingValidation) {
			// engine is already in the middle of delivering validation events. All the new validation events 
			// prepared by the caller will be handled by the existing delivery process. So need to do nothing for now.
			return;
		}
	
		engine.isProcessingValidation = true;
		
		// new validation events may occur as the result of processing of other validation event. The new event may
		// locate at the tree node which current round of delivery has gone through. So we have to go through
		// the entire tree multiple times to make sure no event is left. However, in order to avoid the possible
		// overflow, we only tranverse the entire tree up to 10 times.
		for(var k=0; k<10; k++) {
			var treeNode = validationHelper.viewTree.rootNode.firstChild;
			var foundValidationEvent = engine._deliverValidationEvents(treeNode, validationHelper.viewDomIdToViewMap);
			if(!foundValidationEvent) {
				break;
			}
		}
		// clear the transient data in validationHelper. Leave other data alone in validationHelper for future use.
		// we don't call clearTransientInfo() earlier (e.g. after validateAllViews() is executed and before deliverValidationEvents() is executed), 
		// the main reason is for performance - new validation event may occurs during the execution of deliverValidationEvents(), the calculation of new valdation event
		// need the help from the transient data in validationHelper. 
		validationHelper.clearTransientInfo();
		engine.isProcessingValidation = false;		
	}
	
	engine._deliverValidationEvents = function(treeNode, viewDomIdToViewMap, isToClear) {
		var foundValidationEvent = false;
		if(!treeNode) {
			return foundValidationEvent;
		}
		
		if(!!isToClear) {
			var clearEvent = {"type": BPM_CONSTANTS.VALIDATION_EVENT_TYPE_CLEAR};
			// send clear event to all the valid views who have received validation events
			if(!!treeNode.validationEvent) {
				var aView = engine._findViewInstance(treeNode, viewDomIdToViewMap);
				if(!!aView) {
					engine.callViewLifeCycleAPI(aView, "validate", CONSTANTS.ELAPSED_TIME_THRESHOLD_VALIDATE, clearEvent);
				}
				if(!foundValidationEvent) {
					foundValidationEvent = true;
				}
			}			
		} else {
			// deliver the validation events if exists
			if(!treeNode.isHandled) {
				// handle treeNode's validationEvent if exists
				if(!!treeNode.validationEvent) {
					var aView = engine._findViewInstance(treeNode, viewDomIdToViewMap);
					if(!!aView) {
						engine.callViewLifeCycleAPI(aView, "validate", CONSTANTS.ELAPSED_TIME_THRESHOLD_VALIDATE, treeNode.validationEvent);
					}
					if(!foundValidationEvent) {
						foundValidationEvent = true;
					}
				}		
				treeNode.isHandled = true;
			}
		}
		
		// handle treeNode's children
		if(!!treeNode.firstChild) {
			var childResult = engine._deliverValidationEvents(treeNode.firstChild, viewDomIdToViewMap, isToClear);
			if(!foundValidationEvent) {
				foundValidationEvent = childResult;
			}	
		}
		
		// hanlde treeNode's sibling
		if(!!treeNode.next) {
			var siblingResult = engine._deliverValidationEvents(treeNode.next, viewDomIdToViewMap, isToClear);
			if(!foundValidationEvent) {
				foundValidationEvent = siblingResult;
			}	
		}
		
		return foundValidationEvent;
	}
	
	engine._findViewInstance = function(treeNode, viewDomIdToViewMap) {
		var cachedView = viewDomIdToViewMap[treeNode.domId];
		if(!!cachedView) {
			if(cachedView._isReady()) {
				return cachedView;
			}			
		}
		
		var aView = null;

		if(!!treeNode.locationInfo) {
			// viewIdMetaData exists
			aView = coachViewHelper.findViewByLocation(treeNode.locationInfo, engine.GlobalAssets.topLevelViewScope);
		} else {
			aView = coachViewHelper.findViewScope(treeNode.domId, engine.GlobalAssets.topLevelViewScope);
		}	

		if(!!aView) {
			viewDomIdToViewMap[treeNode.viewDomId] = aView;
			return aView;
		}
		
		return null;
	}

	engine._findCollaboratingView = function(event) {
		if(!!event.viewIdMetaData) {
			// if viewIdMetaData exists, use it to figure out view's DOM id
			return coachViewHelper.findViewByLocation(event.viewIdMetaData, engine.GlobalAssets.topLevelViewScope);
		} else if(!!event.viewId) {
			// event.viewId means the dom id of the closest viewScope node
			return coachViewHelper.findViewScope(event.viewId, engine.GlobalAssets.topLevelViewScope);
		} else {
			return null;
		}	
	}
	
	engine._findCollaboratingDomId = function(collaboratingView, event) {
		var node = dom.byId(event.domId);
		if(!!node && dom.isDescendant(node, collaboratingView.context.element)) {
			return event.domId;
		} else {
			// fallback to dom id of collaboratingView
			return collaboratingView.context.element.id;
		}
	}
	
	engine.getViewIdMetaData = function(viewDomId) {
		var viewScope = coachViewHelper.findViewScope(viewDomId, engine.GlobalAssets.topLevelViewScope);
		return coachViewHelper.getViewIdMetaData(viewScope);
	}
	
	engine.callAjaxService = function(ajaxUrl, args) {
		var paramsKey = "&params="
		var params = null;
		if (!!args) {
			if (!!(args.params)) {
				if (lang.isString(args.params)) {
					params = encodeURIComponent(args.params);					
				} else {
					params = encodeURIComponent(json1.stringify(args.params));
				}
			}
		}
		var tempStr = (!!params)? (ajaxUrl+paramsKey+params) : ajaxUrl;
		var tempArray = tempStr.split("?");
		
		var xhrArgs = {
			url: tempArray[0],
			handleAs: "json",
		    headers: {"Content-Type": "application/x-www-form-urlencoded",
		    	"Accept": "application/json"},
			sync: false,
			postData: tempArray[1],
			load: function(data){
				if ((!!args) && (!!(args.load)) && (lang.isFunction(args.load))) {
					args.load(data.data.data);
				}
	        },
	        error: function(err) {
	        	if ((!!args) && (!!(args.error)) && (lang.isFunction(args.error))) {
	        		args.error(err);
	        	} else {
			    	var errorMsg = "engine.callAjaxService() caught Error when handling " + ajaxUrl + ", Error: " + err;
			    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
	        	}
	        }
		};
		xhr.post(xhrArgs);
	}
	
	
	engine.setReadonly = function(readonly) {
		if(readonly) {
			// if engine.coachShimmer exists, that means already in readonly mode. So current readonly mode request should be ingored
			if(!engine.coachShimmer) {
				engine.coachShimmer = new Shimmer(engine.CONSTANTS.COACH_SHIMMER_ID_BASE);
			}
		} else {
			// if engine.coachShimmer doesn't exist, that means already in edit mode. So current edit mode request should be ignored
			if(!!engine.coachShimmer) {
				engine.coachShimmer.dispose();
                engine.coachShimmer = null;
			}
		}
	}

					  
	engine.completeInitialization = function() {
		logger.log("engine", "completeInitialization", "is called", logger.INFO);
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onInitialized",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": [{"name": "URL", "value": window.document.location.href}]
			};
			broadcastHelper.broadcastMessage(event);
		} 
		if (window.addEventListener) {
			window.addEventListener('unload', engine.onWindowClose, false);
		} else if (window.attachEvent) {
			window.attachEvent('onunload', engine.onWindowClose);
		} else {
			window.onunload = engine.onWindowClose;
		}		
		commonHelper.timestamp("Exit engine.completeInitialization()");
	}
	
	//support for portlet enablement
	engine.addMessageListeners = function(whiteList) {
		logger.log("engine", "addMessageListeners", "is called", logger.FINER);
		
		// Nesting engine.checkPostMessageOrigin() inside engine.addMessageListeners to hide whiteList in the closure.
		// check if the origin is a trusted one
		engine.checkPostMessageOrigin = function(origin) {
			// firstly, check if provided origin is same as mine
			// host covers both hostname and port
			var myOrigin = window.location.protocol + "//" + window.location.host;
			if(origin == myOrigin) {
				return true;
			}
	
			// then, check if provided origin matches any one in the whiteList
			if(!!whiteList) {
				for(var k=0; k<whiteList.length; k++) {
					if(origin == whiteList[k]) {
						return true;
					}
				} 
			} 			
			return false;
		}		
		
		var receivePostMessage = function(event) {
			logger.log("engine", "receivePostMessage", ["is called with event:", event], logger.FINER);
			
			if(engine.checkPostMessageOrigin(event.origin, whiteList)) {
				var eventObj = json.fromJson(event.data);
				var returnTicket = {window: event.source, origin: event.origin};
				
				if (eventObj.name == 'broadcastBoundaryEvent') {
					// initializing external messaging for boundary events
					// engine.boundaryEventBroadcastTarget are only used for coach to broadcast boundaryEvent, e.g. to portlet
					engine.boundaryEventBroadcastTarget = returnTicket;
				} else if (eventObj.name == 'USER_PROFILE_CHANGED') {
					dojo.publish("/com/ibm/bpm/social/USER_PROFILE_CHANGED", [eventObj.username]);
				} else {
					// Once Coach completes this external request, send the response back to the window (represented by returnTicket.window) 
					// with correct origin (represented by returnTicket.origin)
					engine.controller.processExternalEvent(eventObj, returnTicket);
				}
			} else {
				// Coach framework received postMessage from untrusted origin.
				commonHelper.consoleLog("Coach ignores a postMessage received from untrusted origin " + event.origin, commonHelper.WARN);
			}
		}
		
		if(typeof window.addEventListener != 'undefined') {
			window.addEventListener("message", receivePostMessage, false);
		} else if(typeof window.attachEvent != 'undefined') {
			window.attachEvent("onmessage", receivePostMessage);
		}	
	}
	
	engine.onWindowClose = function() {
		logger.log("engine", "onWindowClose", "is called", logger.INFO);
		for (var i = 0; i < engine._internalUnloadHandler.length; i++) {
			try {
				engine._internalUnloadHandler[i]();
			} catch (ex) {
				// log the exception and ignore it
		    	var errorMsg = "engine.onWindowClose() caught Exception when invoking internal unload handler, Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			}
		}
		engine.onCoachUnload.call(engine.GlobalAssets);
	}
	
	engine._registerInternalUnloadHandler = function(callback) {
		engine._internalUnloadHandler.push(callback);
	}

	engine.onCoachUnload = function() {
		logger.log("engine", "onCoachUnload", "is called", logger.INFO);
		if (!(engine._isCoachUnloaded)) {
		
			if(!!this.coach) {
				try {			
					this.coach.unload();
			    } catch(ex) {
			    	var errorMsg = "engine.onCoachUnload() caught Exception when calling unload() on Coach object, Exception: " + ex;
			    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			    }
				delete this.coach;
			}
						
			var topLevelViewIds = new Array();
			for (var viewId in this.topLevelViewScope) {
				topLevelViewIds.push(viewId);
				var viewScope = this.topLevelViewScope[viewId];
				engine.callUnload(viewScope);
			}
			for (var k=0; k<topLevelViewIds.length; k++) {
				delete this.topLevelViewScope[topLevelViewIds[k]];
			}
			
			if(!!engine.controller) {
				engine.controller.destroy();
			}
			
			engine._isCoachUnloaded = true;
		}
	}

	engine.callUnload = function(viewScope) {
		if(!viewScope) return;
		
	    // Call the view unload lifecycle method
		viewScope._preUnload();

		engine.callViewLifeCycleAPI(viewScope, "unload", CONSTANTS.ELAPSED_TIME_THRESHOLD_UNLOAD);

		// Then call nested view unload
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {

				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					engine.callUnload(subViewScope);
				}
			}		
		}
		
	    // Finally call _postUnload() to dispose viewScope (make it ready for garbage collection)
		// viewScope.context.subview is set to null in viewScope.postUnload(), so have to get viewScope.context.subview before calling viewScope.postUnload()
		viewScope._postUnload();
	}	
	
	return engine;
});
