// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-A09
// Copyright IBM Corporation 2010. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT

/**
 * Created Sep 5, 2010
 *
 * @author mfehrenbacher
 */

CustomTable = function(standardProps, standardColumns, cellCreators) {
  this.standardProps = standardProps;
  this.standardColumns = standardColumns;
  this.cellCreators = cellCreators;
  this.root = null;
  this.thead = null;
  this.tbody = null;
  this.customProps = null;
  this.customPropTypes = null;
}

/**
 * Creates DOM elements.
 * We must call this method after object instantiation, because this method
 * contructs all needed DOM elements.
 */
CustomTable.prototype.init = function() {
	var table = document.createElement("table");
	table.className = "docAttachmentTable";
	table.style.margin = "0px";
	table.style.marginBottom = "5px";
	table.style.width = "100%";
	var thead = document.createElement("thead");
	table.appendChild(thead);
  this.thead = thead;
  var tbody = document.createElement("tbody");
	table.appendChild(tbody);
	this.tbody = tbody;
	this.root = table;

}

CustomTable.prototype.setupColumns = function (customProps, customPropTypes) {
  var tr = document.createElement("tr");
  for (var i = 0; i < this.standardProps.length; i++) {
    var title = this.standardColumns != null ? this.standardColumns[this.standardProps[i]] : null;
    if (title == null) {
      tr.appendChild(createTD(this.propsArray[i]));
    } else {
      tr.appendChild(createTD(title));
    }
  }

  if (customProps == null) return;
  this.customProps = customProps;
  this.customPropTypes = customPropTypes;
  for (var prop in customProps) {
    if (prop == "") {
      continue;
    }
    var title = customProps[prop];
    if (title == null || title == "") {
      title = prop;
    }
    tr.appendChild(createTD(title));
	}
	this.thead.appendChild(tr);
}

/**
 * Add new row with cell values specified by given object.
 *
 * @argument {Object} obj - object that has to be added
 */
CustomTable.prototype.addRow = function (customProps, doc) {
	if (doc == null) return;

  var tr = document.createElement("tr");
	for (var i = 0; i < this.standardProps.length; i++) {
		var creator = this.cellCreators != null ? this.cellCreators[this.standardProps[i]] : null;
		var td = null;
		if (creator == null) {
			td = document.createElement("td");
			td.appendChild(document.createTextNode(doc[this.standardProps[i]]));
		} else {
			td = creator(doc);
		}
    tr.appendChild(td);
	}
  
  for (var prop in this.customProps) {
    if (prop == "") {
      continue;
    }
    var key = "prop_" + prop;
    var title = doc[key];
	if(this.customPropTypes){
		var dataType = this.customPropTypes[prop];
		if(dataType=="DATE"){
			var longNumber = new Number(title);
			if(!isNaN(longNumber)){
				var dateObj = new Date(longNumber);
				title=formatADate(dateObj,"yyyy-MM-dd");
			}
		} else if (dataType=="DATETIME"){
			var longNumber = new Number(title);
			if(!isNaN(longNumber)){
				var dateObj = new Date(longNumber);
				title=formatADate(dateObj,"yyyy-MM-dd hh:mm:ss");
			}
		} else if (dataType=="TIME"){
			var longNumber = new Number(title);
			if(!isNaN(longNumber)){
				var dateObj = new Date(longNumber);
				title=LZ(dateObj.getUTCHours())+":"+LZ(dateObj.getUTCMinutes())+":"+LZ(dateObj.getUTCSeconds());
			}
		}
	}
    if (title == null) {
      title = "";
    }
    tr.appendChild(createTD(title));
  }
  this.tbody.appendChild(tr);
}

/**
 * Clears table.
 */
CustomTable.prototype.clear = function() {
	this.root.removeChild(this.thead);
  this.root.removeChild(this.tbody);
	this.thead = null;
  this.tbody = null;
  this.thead = document.createElement("thead");
  this.tbody = document.createElement("tbody");
	this.root.appendChild(this.thead);
  this.root.appendChild(this.tbody);
}
