/*
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such 
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into 
 * with Lombardi Software. No rights or licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * JavaScript file, contains code of FieldSet class.
 * 
 * Created by Sergey Rassokhin, 9/5/2007 
 */
 
/**
 * Represents FieldSetDecorator control. It is just wrapping <fieldset/> element 
 * for more comfortable using of it. 
 * 
 * @constructor
 * @class
 */
function FieldSet(captionText) {
	// <fieldset/> element of the control
	this.root = null;
	// <legend/> element
	this.caption = null;
	// Text of the fieldset caption
	this.captionText = captionText;
} 

/**
 * Initializes control. Create all needed DOM elements.
 */
FieldSet.prototype.init = function() {
	this.root = document.createElement("fieldset");
	this.caption = document.createElement("legend");
	this.captionTextNode = document.createTextNode(this.captionText)
	this.caption.appendChild(this.captionTextNode);
	this.root.appendChild(this.caption);
}

/**
 * Sets the given caption text.
 */
FieldSet.prototype.setCaptionText = function(captionText) {
	this.caption.removeChild(this.captionTextNode);
	this.captionTextNode = document.createTextNode(captionText)
	this.caption.appendChild(this.captionTextNode);
}

/**
 * Add specified element into this control.
 */
FieldSet.prototype.addElement = function(name, control) {
	this[name] = control;
	this.root.appendChild(control.root != null ? control.root : control);
}

/**
 * Removes specified element from fielset control.
 */
FieldSet.prototype.removeElement = function(name) {
	this.root.removeChild(this[name]);
	this[name] = null;
}

/**
 * Clears the control.
 */
FieldSet.prototype.clear = function() {
	this.init();
}	