/*
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such 
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into 
 * with Lombardi Software. No rights or licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * JavaScript file, contains code of Table Control class.
 * 
 * Created by Sergey Rassokhin, 9/5/2007 
 */
 
 /**
  * Class represents HTML table control.
  *   
  * @param propsArray - Array of target object's properties
  * @param columns - Hash-object [prop-name -> Column Title]
  * @param columns - Hash-object [prop-name -> function(obj)]
  * @class
  * @constructor
  * @author Sergey Rassokhin 
  */
Table = function(propsArray, columns, cellCreators) {
  this.propsArray = propsArray;
  this.columns = columns;
  this.cellCreators = cellCreators;
  this.root = null;
  this.tbody = null;
}
 
/**
 * Creates DOM elements.
 * We must call this method after object instantiation, because this method
 * contructs all needed DOM elements.  	 
 */
Table.prototype.init = function() { 
	var table = document.createElement("table");
	table.className = "docAttachmentTable";
	table.style.margin = "0px";
	table.style.marginBottom = "5px";
	table.style.width = "100%";
	var thead = document.createElement("thead");
	var tr = document.createElement("tr");
	for (var i = 0; i < this.propsArray.length; i++) {
		var title = this.columns != null ? this.columns[this.propsArray[i]] : null;
		if (title == null) {
			tr.appendChild(createTD(this.propsArray[i]));
		} else {
			tr.appendChild(createTD(title));	
		}			
	}
	thead.appendChild(tr);
	table.appendChild(thead);
	var tbody = document.createElement("tbody");
	table.appendChild(tbody);
	this.tbody = tbody;
	this.root = table;
}

/**
 * Add new row with cell values specified by given object. 
 * 
 * @argument {Object} obj - object that has to be added 
 */
Table.prototype.addRow = function (obj) {
	if (obj == null) return;
	var tr = document.createElement("tr");
	for (var i = 0; i < this.propsArray.length; i++) {			
		var creator = this.cellCreators != null ? this.cellCreators[this.propsArray[i]] : null;
		var td = null;
		if (creator == null) {
			td = document.createElement("td");
			td.appendChild(document.createTextNode(obj[this.propsArray[i]]));
		} else {
			td = creator(obj);
		}						 
		tr.appendChild(td);
	}	
	this.tbody.appendChild(tr);
}  

/**
 * Clears table.
 */
Table.prototype.clear = function() {
	this.root.removeChild(this.tbody);
	this.tbody = null;
	this.tbody = document.createElement("tbody");
	this.root.appendChild(this.tbody);
}	
