/*
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such 
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into 
 * with Lombardi Software. No rights or licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * JavaScript file, contains code of Document Attachment Control in Coach.
 * 
 * Created by Sergey Rassokhin, 9/3/2007 
 */


/**
 * JavaScript class that represents Document Attachemnt control at Coach.
 * It creates all GUI elements, fills them, makes AJAX requests for data.
 * It can be used by instantiating it and call "init()" method. Note that you
 * must manually initialize set of following Read/Write properties before call
 * of init() method.
 * 
 * Avaliable Read/Write properties:
 *  	
 * 	{Hash-Object} matchingProperties 
 *			- properties for matching documents to display at this control
 * 	{Hash-Object} attachedProperties 	
 * 			- properties that will be attached for each uploaded document
 * 	{boolean} enableUpload
 * 			- enable/disable upload documents feature
 * 	{String} defaultName 	   	
 *			- default name for each uploaded document
 * 	{boolean} userEditable	   	
 *			- enable/disable ability to edit document name that will be uploaded
 * 	{boolean} hideInPortal	   	
 *			- indicates that we must hide documents that have been uploaded
 * 
 * @argument {DOM element} rootElement		   
 *			- root DOM elements where this control will be placed. 
 * @constructor
 * @class						 	 							      					 
 */
var documentAttachment_noDoc = "No Documents Attached";
var documentAttachment_button_addDoc = "Add Document";
var documentAttachment_uploadDoc = "Upload Document";
var documentAttachment_label_type = "Type";
var documentAttachment_label_file = "File";
var documentAttachment_label_title = "Title";
var documentAttachment_label_url = "URL";
var documentAttachment_button_ok = "OK";
var documentAttachment_button_cancel = "Cancel";
var documentAttachment_button_Browse = "Browse...";
var documentAttachment_link_Update = "Update";
var documentAttachment_fileName="File Name/URL";
var documentAttachment_titleName="Title/Name";
var documentAttachment_revision="Revision";
var documentAttachment_revisionDate="Revision Date";
var documentAttachment_modifiedBy="Last Modified By";
var documentAttachment_mimeType="MIME Type";
var documentAttachment_versionID="Version ID";
var documentAttachment_revisions="Revisions";
var documentAttachment_close="Close";
var documentAttachment_validationError="The value is not valid";
var documentAttachment_docNameMaxLength="The Title should be no more than 50 characters long";
var documentAttachment_fileNameMaxLength="File name is used as Title which exceeds 50 characters long. Please provide a valid Title";
var resourceBundle;

//Get the JS message from the servlet
var jsonResURL = "jsonResBundleLoader?bundle=JavaScriptMessages";

try{
dojo.xhrGet( {
    url: jsonResURL,
    handleAs: "json",
    load: function(responseObject, ioArgs) {
        if(responseObject != undefined && responseObject.__MISSING__ != "__MISSING__"){
	    	resourceBundle = responseObject;
			documentAttachment_noDoc = responseObject.documentAttachment_noDoc;
			documentAttachment_button_addDoc = responseObject.documentAttachment_button_addDoc;
			documentAttachment_uploadDoc = responseObject.documentAttachment_uploadDoc;
			documentAttachment_label_type = responseObject.documentAttachment_label_type;
			documentAttachment_label_file = responseObject.documentAttachment_label_file;
			documentAttachment_label_title = responseObject.documentAttachment_label_title;
			documentAttachment_label_url = responseObject.documentAttachment_label_url;
			documentAttachment_button_ok = responseObject.documentAttachment_button_ok;
			documentAttachment_button_cancel = responseObject.documentAttachment_button_cancel;
			documentAttachment_button_Browse = responseObject.documentAttachment_button_Browse;		
			documentAttachment_link_Update = responseObject.documentAttachment_link_Update;
			documentAttachment_fileName= responseObject.documentAttachment_fileName;
			documentAttachment_titleName= responseObject.documentAttachment_titleName;
			documentAttachment_revision= responseObject.documentAttachment_revision;
			documentAttachment_revisionDate= responseObject.documentAttachment_revisionDate
			documentAttachment_modifiedBy= responseObject.documentAttachment_modifiedBy;
			documentAttachment_mimeType= responseObject.documentAttachment_mimeType;
			documentAttachment_versionID= responseObject.documentAttachment_versionID;
			documentAttachment_revisions=responseObject.documentAttachment_revisions;
			documentAttachment_close=responseObject.documentAttachment_close;
			documentAttachment_validationError=responseObject.documentAttachment_validationError;
			documentAttachment_docNameMaxLength=responseObject.documentAttachment_docNameMaxLength;
			documentAttachment_fileNameMaxLength=responseObject.documentAttachment_fileNameMaxLength;
	    }else{
	    	initResourceBundleOnError();
	    }
		
    },
    error: dojo.hitch(this, initResourceBundleOnError),
    sync: true
});
}catch(err){
	//call will be failed if no security context, 
	//if do not catch, the failure will block the preview at the AE
	//do nothing at here, the validation messages will be default(en) at this case
}
 
function initResourceBundleOnError() {
	
	resourceBundle = {};
	resourceBundle.documentAttachment_noDoc = documentAttachment_noDoc;
	resourceBundle.documentAttachment_button_addDoc = documentAttachment_button_addDoc;
	resourceBundle.documentAttachment_uploadDoc = documentAttachment_uploadDoc;
	resourceBundle.documentAttachment_label_type = documentAttachment_label_type;
	resourceBundle.documentAttachment_label_file = documentAttachment_label_file;
	resourceBundle.documentAttachment_label_title = documentAttachment_label_title;
	resourceBundle.documentAttachment_label_url = documentAttachment_label_url;
	resourceBundle.documentAttachment_button_ok = documentAttachment_button_ok;
	resourceBundle.documentAttachment_button_cancel = documentAttachment_button_cancel;
	resourceBundle.documentAttachment_button_Browse = documentAttachment_button_Browse;		
	resourceBundle.documentAttachment_link_Update = documentAttachment_link_Update;
	resourceBundle.documentAttachment_fileName= documentAttachment_fileName;
	resourceBundle.documentAttachment_titleName= documentAttachment_titleName;
	resourceBundle.documentAttachment_revision= documentAttachment_revision;
	resourceBundle.vdocumentAttachment_revisionDate= documentAttachment_revisionDate
	resourceBundle.documentAttachment_modifiedBy= documentAttachment_modifiedBy;
	resourceBundle.documentAttachment_mimeType= documentAttachment_mimeType;
	resourceBundle.documentAttachment_versionID= documentAttachment_versionID;
	resourceBundle.documentAttachment_revisions=documentAttachment_revisions;
	resourceBundle.documentAttachment_close=documentAttachment_close;
	resourceBundle.documentAttachment_validationError=documentAttachment_validationError;
	resourceBundle.documentAttachment_docNameMaxLength=documentAttachment_docNameMaxLength;
	resourceBundle.documentAttachment_fileNameMaxLength=documentAttachment_fileNameMaxLength;
	
}

function DocumentAttachmentControl(root, id) {	
	this.rootElement = root;
	this.controlId = id;
	this.instanceId = null;
	this.userId = null;
	this.byInstance  = false;
  this.matchingMode = 0;
	this.matchingProperties = null;
	this.attachedProperties = null;
	this.enableUpload = null;
	this.defaultName = "";
	this.parentFolder = "";
	this.userEditable = null;
	this.hideInPortal = null;
  this.snapshotId         = "";
  this.processId        = "";
  this.connectionType     = 0;
  this.connectionUrl      = "";
  this.connectionRepoName = "";
  this.connectionUsername = "";
  this.passwordMode       = 0;
  this.password           = "";
  this.freeTextPassword   = "";
  this.remoteMatchingMode = 0;
  this.itemClassName      = "";
  this.maxResultSize      = 25;
  this.remoteMatchingProperties = null;
  this.columnDisplayProperties = null;
  this.columnDisplayPropertyTypes = null;
  this.viewerAssociated = false;
  this.viewerFunction = null;
  this.selectedRow = null;
  this.columns = {
		"filename" : documentAttachment_fileName, 
		"name" : documentAttachment_titleName,
		"revision" : documentAttachment_revision,
		"date" : documentAttachment_revisionDate,
		"author" : documentAttachment_modifiedBy, 
		"action" : "&nbsp;"
	};
  this.standardRemoteColumns = {
    "name" : documentAttachment_titleName,
    "mimeType" : documentAttachment_mimeType,
    "versionID" : documentAttachment_versionID
  };
  this.standardRemoteProps = [ "name", "mimeType", "versionID" ];
  var obj = this;
	this.cellCreators = {
		"name" : 
			function(doc) {
        var td;
        if (obj.connectionType != 1) {
          td = createTD(doc.name);
          td.style.cursor = "pointer";
          td.onclick = function() {
            obj.loadDocumentRevisions(doc);
          };
        } else {
          td = document.createElement("td");
          var a = document.createElement("a");
          var realPassword = obj.passwordMode == 0 ? obj.freeTextPassword : obj.password;
          var url = window._BPM_portalContextRoot + "/jsp/remotedocument?operation=ajax_loadRemoteDocument&snapshotId=" + obj.snapshotId +
                    "&processId=" + obj.processId + "&baseURL=" + encodeURIComponent(obj.connectionUrl) +
                    "&username=" + encodeURIComponent(obj.connectionUsername) + "&password=" + encodeURIComponent(realPassword) +
                    "&iciURN=" + encodeURIComponent(doc["iciURN"]);
          if (obj.viewerAssociated) {
            a.href = "#";
            a.onclick = function() {
              if (obj.selectedRow != null) {
                obj.selectedRow.className = obj.selectedRow.className.replace(/\bselectedRow\b/,'');
              }
              var tr = td.parentNode;
              tr.className += " selectedRow";
              obj.selectedRow = tr;

              obj.viewerFunction(url);
            };
          } else {
            a.href = url;
          }
          a.innerHTML = doc.name;
          td.appendChild(a);
        }
				return td;
			},
		"filename" :
			function(doc) {
        var url;
        if (doc.type == "0") {
          url = window._BPM_portalContextRoot + "/jsp/downloadDocument.do?docId=" + doc.id
        } else {
          url = doc.filename;
          url = obj.htmlStringDecoder(url);
        }
        var td = document.createElement("td");
				var a = document.createElement("a");
        if (obj.viewerAssociated) {
          a.href = "#";
          a.onclick = function () {
            obj.viewerFunction(url);
          }
        } else {
          a.href = url;
          a.target = "_blank"
        }
				a.innerHTML = doc.filename;
				td.appendChild(a)
				return td;
			},	
		"action" :
			function(doc) {
				var td = document.createElement("td");
				var href = document.createElement("a");
				href.href = "#";
				href.appendChild(document.createTextNode(documentAttachment_link_Update));
				td.appendChild(href);
				href.onclick = function() {
					obj.showUpdateForm(doc);
				};
				return td;
			}		
	};
}

/**
 * Initializes control. Create all embedded controls (Documents and Revisions 
 * tables, add and update documents forms).
 */
DocumentAttachmentControl.prototype.init = function() {			
	// Create elements
	this.noDocumentsMessage = this.createNoDocumentsElement();	
	this.documentsTable = this.createDocumentsTableElement();
  this.loadingNotification = this.createLoadingNotificationElement();
  this.remoteDocumentsTable = this.createRemoteDocumentsTableElement();
  this.revisionsFieldset = this.createRevisionsTableElement();
	this.uploadFieldset = this.createNewDocumentUploadFormElement();
  this.remoteUploadFieldset = this.createRemoteDocumentUploadFormElement();
  this.updateFieldset = this.createUpdateDocumentRevisionFormElement();
	this.addDocumentButton = this.createAddDocumentButtonElement();		
	// set all invisible
	setVisible(this.noDocumentsMessage, false);
	setVisible(this.documentsTable.root, false);
	setVisible(this.loadingNotification, false);
  setVisible(this.remoteDocumentsTable.root, false);
  setVisible(this.revisionsFieldset.root, false);
	setVisible(this.uploadFieldset.root, false);
	setVisible(this.remoteUploadFieldset.root, false);
	setVisible(this.updateFieldset.root, false);
	setVisible(this.addDocumentButton, false);	
	// place all elements
	this.rootElement.appendChild(this.noDocumentsMessage);
	this.rootElement.appendChild(this.documentsTable.root);
  this.rootElement.appendChild(this.loadingNotification);
  this.rootElement.appendChild(this.remoteDocumentsTable.root);
  this.rootElement.appendChild(this.revisionsFieldset.root);
	this.rootElement.appendChild(this.uploadFieldset.root);
	this.rootElement.appendChild(this.remoteUploadFieldset.root);
	this.rootElement.appendChild(this.updateFieldset.root);
	this.rootElement.appendChild(this.addDocumentButton);					
}	

/** Creates "No Attached Documents" message element. */
DocumentAttachmentControl.prototype.createNoDocumentsElement = function() {
	var noDocumentsMessage = document.createElement("div");
	noDocumentsMessage.style.padding = "5px";
	noDocumentsMessage.innerHTML = documentAttachment_noDoc;
        noDocumentsMessage.className = "twDocAttachmentNoDocs";
        return noDocumentsMessage;
}

/** Creates Documents Table element. */
DocumentAttachmentControl.prototype.createDocumentsTableElement = function() {
	var obj = this;
	var props = null;
	if (this.enableUpload) {
		props = ["name", "filename", "revision", "date", "author", "action"];	
	} else {
		props = ["name", "filename", "revision", "date", "author"];
	} 			
	var documentsTable = new Table(props, this.columns, this.cellCreators);
	documentsTable.init();
        documentsTable.className = "twDocAttachmentDocTable";
        return documentsTable;
}

DocumentAttachmentControl.prototype.createRemoteDocumentsTableElement = function() {
  var remoteDocumentsTable = new CustomTable(this.standardRemoteProps, this.standardRemoteColumns, this.cellCreators);
  remoteDocumentsTable.init();
  remoteDocumentsTable.className = "twDocAttachmentRemoteDocTable";
  return remoteDocumentsTable;
}

DocumentAttachmentControl.prototype.createLoadingNotificationElement = function() {
  var loadingDocumentsMessage = document.createElement("div");
  loadingDocumentsMessage.style.padding = "5px";
  loadingDocumentsMessage.innerHTML = "Loading document list...";
  loadingDocumentsMessage.className = "twDocAttachmentLoadingDocs";
  return loadingDocumentsMessage;
}

/** Creates Revisions Table element. */
DocumentAttachmentControl.prototype.createRevisionsTableElement = function() {
	var obj = this;
	var props = ["revision", "filename", "date", "author"];	
	var revisionsTable = new Table(props, this.columns, this.cellCreators);
	revisionsTable.init();
	var button = createButton(documentAttachment_close);
        button.className += " twDocAttachmentCloseButton";
        button.onclick = function() {
		setVisible(obj.addDocumentButton, true && obj.enableUpload);				
		setVisible(obj.revisionsFieldset.root, false);
	};
	var revisionsFieldset = new FieldSet("Revisions");
        revisionsFieldset.className = "twDocAttachmentRevisionBorder";
        revisionsFieldset.init();
	revisionsFieldset.addElement("table", revisionsTable);
	revisionsFieldset.addElement("button", button);
	return revisionsFieldset;
}

/** Creates New Documents Form element. */
DocumentAttachmentControl.prototype.createNewDocumentUploadFormElement = function() {	
	var obj = this;
	var uploadForm = new Form(this.controlId + "_upload");
    uploadForm.validationMessage=documentAttachment_validationError;
	uploadForm.actionUrl = window._BPM_portalContextRoot + "/jsp/addDocumentExecute.do";
	uploadForm.onclose = function() {
		setVisible(obj.addDocumentButton, true && obj.enableUpload);	
	};
	uploadForm.init();
	uploadForm.addHiddenField("bpdInstanceId", this.instanceId);
	uploadForm.addHiddenField("userID", this.userId);
	uploadForm.addHiddenField("inline", "true");
	var f = function() {
		var enabled = (uploadForm.getFieldValue("fileType") == "0" );
		uploadForm.setFieldVisible("fileContent", enabled);
		uploadForm.setFieldVisible("urlText", !enabled);
	};
	uploadForm.addSelectField("fileType", "0", documentAttachment_label_type , true, "30%",{"documentAttachment_label_file" : "0", "documentAttachment_label_url" : "1"}, f);		
	uploadForm.addInputFileField("fileContent", "", documentAttachment_label_file , true, "70", FILEValidator);
	uploadForm.addInputTextField("urlText", "", documentAttachment_label_url , false, "80%", URLValidator);
	if (this.userEditable || this.defaultName == "") {
		uploadForm.addInputTextField("docName", this.defaultName, documentAttachment_label_title , true, "30%", DocNameValidator);	
	} else {
		uploadForm.addHiddenField("docName", this.defaultName);
		uploadForm.addTextField("docNameText", this.defaultName, documentAttachment_label_title , true);
	}		
	// additional document properties
	uploadForm.addHiddenField("hideInPortal", this.hideInPortal);	
	for (prop in this.attachedProperties) {
		if (prop) {
			uploadForm.addHiddenField("docprop_" + prop, this.attachedProperties[prop]);
		}			
	}
	uploadForm.onclose = function() {    
		setVisible(obj.uploadFieldset.root, false);
		setVisible(obj.addDocumentButton, true && obj.enableUpload);		
	}; 
	uploadForm.onsubmit = function() {
		// if document name was not typed use filename(URL)
		if (uploadForm.getFieldValue("docName") == "") {
			if (uploadForm.getFieldValue("fileType") == 0) {
                                var m = uploadForm.getFieldValue("fileContent");
				// For security reason, specific browser will hide the full path for file uploaded.
				// Well, IE in earlier version still exposes the full path or as "fakepath" (from IE8)
                                if (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) { // IE browser
                                        var m_filename = uploadForm.getFieldValue("fileContent").match(/(.*)[\/\\]([^\/\\]+\.\w+)$/);
                                        if (m_filename != null)
                                                uploadForm.setFieldValue("docName", m_filename[2]);
                                        else
                                                uploadForm.setFieldValue("docName", m); // in case of IE changes in later versions
                                } else { // Other browsers (need further checking)
                                        uploadForm.setFieldValue("docName", m);
                                }
			} else {
				var m = uploadForm.getFieldValue("urlText").match(/(.*):[\/\\]+(.*)/);
				if (m) uploadForm.setFieldValue("docName", m[2]); 	
			}								
		}		
	};
	// overwrite default clear()
	var _clear = uploadForm.clear;
	uploadForm.clear = function() {
		_clear.apply(this, arguments);
		var enabled = (this.getFieldValue("fileType") == "0" );
		this.setFieldVisible("fileContent", enabled);
		this.setFieldVisible("urlText", !enabled);
	};
	// place it into fieldset
	var uploadFieldset = new FieldSet(documentAttachment_uploadDoc);
        uploadFieldset.className = "twDocAttachmentUploadBorder";
        uploadFieldset.init();	
	uploadFieldset.addElement("form", uploadForm);
	uploadForm.container = uploadFieldset.root; 
	return uploadFieldset;
}

DocumentAttachmentControl.prototype.createRemoteDocumentUploadFormElement = function() {
  var obj = this;
  var uploadForm = new Form(this.controlId + "_remoteupload");
  uploadForm.validationMessage=documentAttachment_validationError;
  uploadForm.actionUrl = window._BPM_portalContextRoot + "/jsp/addRemoteDocument.do";
  uploadForm.onclose = function() {
    setVisible(obj.addDocumentButton, true && obj.enableUpload);
  };
  uploadForm.init();
  uploadForm.addInputFileField("fileContent", "", documentAttachment_label_file, true, "70", FILEValidator);
  if (this.userEditable || this.defaultName == "") {
    uploadForm.addInputTextField("docName", this.defaultName, documentAttachment_label_title, true, "30%");
  } else {
    uploadForm.addHiddenField("docName", this.defaultName);
    uploadForm.addTextField("docNameText", this.defaultName, documentAttachment_label_title, true);
  }
  uploadForm.addHiddenField("parentFolder", this.parentFolder);
  // additional document properties
  for (var prop in this.attachedProperties) {
    if (prop) {
      uploadForm.addHiddenField("docprop_" + prop, this.attachedProperties[prop]);
    }
  }
  uploadForm.addHiddenField("itemClassName", this.itemClassName);
  uploadForm.addHiddenField("snapshotId", this.snapshotId);
  uploadForm.addHiddenField("processId", this.processId);
  uploadForm.addHiddenField("baseURL", this.connectionUrl);
  uploadForm.addHiddenField("repoName", this.connectionRepoName);
  uploadForm.addHiddenField("username", this.connectionUsername);
  uploadForm.addHiddenField("password", this.passwordMode == 0 ? this.freeTextPassword : this.password);

  uploadForm.onclose = function() {
    setVisible(obj.remoteUploadFieldset.root, false);
    setVisible(obj.addDocumentButton, true && obj.enableUpload);
  };
  uploadForm.onsubmit = function() {
    // if document name was not typed use filename(URL)
    if (uploadForm.getFieldValue("docName") == "") {
      var m = uploadForm.getFieldValue("fileContent");
	  // For security reason, specific browser will hide the full path for file uploaded.
	  // Well, IE in earlier version still exposes the full path or as "fakepath" (from IE8)
	  if (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) { // IE browser
    	  var m_filename = uploadForm.getFieldValue("fileContent").match(/(.*)[\/\\]([^\/\\]+\.\w+)$/);
    	  if (m_filename != null)
    		  uploadForm.setFieldValue("docName", m_filename[2]);
    	  else
    		  uploadForm.setFieldValue("docName", m); // in case of IE changes in later versions
	  } else { // Other browsers (need further checking)
      uploadForm.setFieldValue("docName", m);
    }
    }
  };
  // place it into fieldset
  var uploadFieldset = new FieldSet(documentAttachment_uploadDoc);
  uploadFieldset.className = "twDocAttachmentUploadBorder";
  uploadFieldset.init();
  uploadFieldset.addElement("form", uploadForm);
  uploadForm.container = uploadFieldset.root;
  return uploadFieldset;
}

/** Creates Update Documents Form element. */
DocumentAttachmentControl.prototype.createUpdateDocumentRevisionFormElement = function() {
	var obj = this;
	var updateForm = new Form(this.controlId + "_update");
    updateForm.validationMessage=documentAttachment_validationError;
	updateForm.actionUrl = window._BPM_portalContextRoot + "/jsp/addDocumentVersionExecute.do";
	updateForm.onclose = function() { 
		setVisible(obj.updateFieldset.root, false);
		setVisible(obj.addDocumentButton, true && obj.enableUpload);	
	}; 
	updateForm.init();
	updateForm.addHiddenField("bpdInstanceId", this.instanceId);
	updateForm.addHiddenField("userID", this.userId);
	updateForm.addHiddenField("inline", "true");
	updateForm.addHiddenField("parentId", "-1");
	updateForm.addHiddenField("fileType", "0");
	updateForm.addTextField("fileTypeText", "File", documentAttachment_label_type , true);
	updateForm.addInputFileField("fileContent", "", documentAttachment_label_file , true, "70", FILEValidator);
	updateForm.addInputTextField("urlText", "", documentAttachment_label_url , false, "80%", URLValidator);
	updateForm.addTextField("docName", "", documentAttachment_label_title , true);
	// place it into fieldset
	var updateFieldset = new FieldSet(documentAttachment_uploadDoc);
        updateFieldset.className = "twDocAttachmentUpdateBorder";
        updateFieldset.init();
	updateFieldset.addElement("form", updateForm);
	updateForm.container = updateFieldset.root;
	return updateFieldset;
}

/** Creates "Add Document" button element. */
DocumentAttachmentControl.prototype.createAddDocumentButtonElement = function() {
	var obj = this;
	var addDocumentButton = createButton(documentAttachment_button_addDoc);	
	addDocumentButton.onclick = function() {		
		setVisible(obj.addDocumentButton, false);
		setVisible(obj.updateFieldset.root, false);

    if (obj.connectionType != 1) {
      obj.uploadFieldset.form.clear();
      setVisible(obj.uploadFieldset.root, true);
    } else {
      obj.remoteUploadFieldset.form.clear();
      setVisible(obj.remoteUploadFieldset.root, true);
    }
  };
        addDocumentButton.className = "twDocAttachmentUploadButton";
        return addDocumentButton; 
}

/**
 * Show update document revision form.
 */
DocumentAttachmentControl.prototype.showUpdateForm = function(doc) {
	setVisible(this.uploadFieldset.root, false);
	setVisible(this.revisionsFieldset.root, false);
	setVisible(this.addDocumentButton, false);	
	this.updateFieldset.form.clear();
	this.updateFieldset.form.setFieldValue("parentId", doc.parent);
	this.updateFieldset.form.setFieldValue("fileTypeText", doc.type == 0 ? documentAttachment_label_file : documentAttachment_label_url);
	this.updateFieldset.form.setFieldValue("fileType", doc.type);	
	this.updateFieldset.form.setFieldValue("docName", doc.name);
	this.updateFieldset.form.setFieldVisible("fileContent", doc.type == 0);
	this.updateFieldset.form.setFieldVisible("urlText", doc.type == 1);	
	setVisible(this.updateFieldset.root, true);	
}


/**
 * Make AJAX request for documents. If response was successfully recieved 
 * makes Documents Table element visible and fills them with  documents.  
 */
DocumentAttachmentControl.prototype.loadDocuments = function() {
	// local documents
  if (this.connectionType != 1) {
    var outer = this;
    
    var args = "";
   	args += ("&bpdInstanceId=" + this.instanceId);
    args += ("&coachdesigner=false");
    if (this.matchingMode != "None") {
    	for (prop in this.matchingProperties) {
    		if (prop) {
    			args += ("&docprop_" + encodeURIComponent(prop) + "=" + encodeURIComponent(this.matchingProperties[prop]));		
    		} 
    	}    	
    	args += ("&matching=" + this.matchingMode);    		
    }    
    // Document matching mode:
    // TRUE - match all documents, FALSE - match attached documents only
   	args += ("&docMatching=" + !this.byInstance);

    dojo.xhrGet({
        url: window._BPM_portalContextRoot + "/jsp/documentattachment?operation=ajax_getDocuments" + args,
        handleAs: "json",
        load: dojo.hitch(this, function(o) {
			try {
				// clear table body
				outer.documentsTable.clear();
				// fill the table
				var documents = o;
				for(i = 0; i < documents.length; i++) {
    	    	   	outer.documentsTable.addRow(documents[i]);
	            }
				setVisible(outer.uploadFieldset.root, false);
        setVisible(outer.remoteUploadFieldset.root, false);
				setVisible(outer.updateFieldset.root, false);
				setVisible(outer.revisionsFieldset.root, false);
				if (documents.length > 0) {
	            	// show table and hide message
	            	setVisible(outer.noDocumentsMessage, false);
	            	setVisible(outer.documentsTable.root, true);
				} else {
					// show message and hide table
	            	setVisible(outer.documentsTable.root, false);
	            	setVisible(outer.noDocumentsMessage, true);
				}
				setVisible(outer.addDocumentButton, true && outer.enableUpload);
				var uploadForm = outer.uploadFieldset["form"];
				var enabled = (uploadForm.getFieldValue("fileType") == "0" );
				uploadForm.setFieldVisible("fileContent", enabled);
				uploadForm.setFieldVisible("urlText", !enabled);
			} catch (e) {
				alert(e);
			}
        }),
        error: dojo.hitch(this, function(o) {
        	alert("Error retrieving documents list");
        })
    });
  } else {
    // remote documents
    setVisible(this.remoteDocumentsTable.root, false);
    setVisible(this.loadingNotification, true);
    var outer = this;

    var realPassword = this.passwordMode == 0 ? this.freeTextPassword : this.password;

    var args = "";
    for (prop in this.remoteMatchingProperties) {
      if (prop) {
        args += ("&docprop_" + encodeURIComponent(prop) + "=" + encodeURIComponent(this.remoteMatchingProperties[prop]));
      }
    }
    
    for (prop in this.columnDisplayProperties) {
      if (prop) {
        args += ("&selprop_" + encodeURIComponent(prop) + "=" + encodeURIComponent(this.columnDisplayProperties[prop]));
      }
    }
    
    
    args += ("&matching=" + this.remoteMatchingMode);

    args += ("&snapshotId=" + this.snapshotId);
    args += ("&processId=" + this.processId);
    args += ("&url=" + encodeURIComponent(this.connectionUrl));
    args += ("&repositoryName=" + encodeURIComponent(this.connectionRepoName));
    args += ("&username=" + encodeURIComponent(this.connectionUsername));
    args += ("&password=" + encodeURIComponent(realPassword));
    args += ("&parentFolder=" + encodeURIComponent(this.parentFolder));

    args += ("&itemClassName=" + encodeURIComponent(this.itemClassName));
    args += ("&maxResultSize=" + this.maxResultSize);

    dojo.xhrGet({
        url: window._BPM_portalContextRoot + "/jsp/remotedocument?operation=ajax_getDocuments" + args,
        handleAs: "json",
        load: dojo.hitch(this, function(o) {
			try {
				// clear table body
				outer.remoteDocumentsTable.clear();
        outer.remoteDocumentsTable.setupColumns(this.columnDisplayProperties, this.columnDisplayPropertyTypes);
        // fill the table
				var documents = o;
				for(var i = 0; i < documents.length; i++) {
          outer.remoteDocumentsTable.addRow(this.remoteMatchingProperties, documents[i]);
        }
				setVisible(outer.uploadFieldset.root, false);
				setVisible(outer.remoteUploadFieldset.root, false);
				setVisible(outer.updateFieldset.root, false);
				setVisible(outer.revisionsFieldset.root, false);
        setVisible(outer.loadingNotification, false);
        if (documents.length > 0) {
          // show table and hide message
          setVisible(outer.noDocumentsMessage, false);
          setVisible(outer.remoteDocumentsTable.root, true);
				} else {
					// show message and hide table
          setVisible(outer.remoteDocumentsTable.root, false);
          setVisible(outer.noDocumentsMessage, true);
				}

        setVisible(outer.addDocumentButton, true && outer.enableUpload);
      } catch (e) {
        setVisible(outer.loadingNotification, false);
        alert(e);
			}
        }),
        error: dojo.hitch(this, function(o) {
        	alert("Error retrieving documents list");
        })
    });
  }
}

/**
 * Make AJAX request for document's revisions. If response was successfuly recieved 
 * makes Documents Revisions Table elemet visible and fills them with 
 * document's revisions.  
 *
 * @argument {Hash-Object} doc
 * 			- document which revisions have to be shown.
 */
DocumentAttachmentControl.prototype.loadDocumentRevisions = function(doc) {
	var outer = this;
	
    dojo.xhrGet({
        url: window._BPM_portalContextRoot  + "/jsp/documentattachment?operation=ajax_getRevisions&parent=" + doc.parent,
        handleAs: "json",
        load: dojo.hitch(this, function(o) {
			try {
				// clear table body
				outer.revisionsFieldset.table.clear();
				// fill the table
				var revisions = o;
				for(i = 0; i < revisions.length; i++) {
           			outer.revisionsFieldset.table.addRow(revisions[i]);
            	}
            	// show control
				outer.revisionsFieldset.setCaptionText(documentAttachment_revisions+ this.htmlStringDecoder(doc.name));
           		setVisible(outer.addDocumentButton, false);
            	setVisible(outer.uploadFieldset.root, false);
            	setVisible(outer.updateFieldset.root, false);
            	setVisible(outer.revisionsFieldset.root, true);
			} catch(e) {
				alert(e);
			}
        }),
        error: dojo.hitch(this, function(o) {
        	alert("Error during retrieving revisions list");
        })
    });
}

DocumentAttachmentControl.prototype.htmlStringDecoder = function (docName){
	var div = document.createElement('div');
	div.innerHTML = docName;
	if(div.innerText){
		return div.innerText;
	} else {
		return div.textContent;
	}
}
