/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/(function(
	userConfig,
	defaultConfig
){
	// summary:
	//		This is the "source loader" and is the entry point for Dojo during development. You may also load Dojo with
	//		any AMD-compliant loader via the package main module dojo/main.
	// description:
	//		This is the "source loader" for Dojo. It provides an AMD-compliant loader that can be configured
	//		to operate in either synchronous or asynchronous modes. After the loader is defined, dojo is loaded
	//		IAW the package main module dojo/main. In the event you wish to use a foreign loader, you may load dojo as a package
	//		via the package main module dojo/main and this loader is not required; see dojo/package.json for details.
	//
	//		In order to keep compatibility with the v1.x line, this loader includes additional machinery that enables
	//		the dojo.provide, dojo.require et al API. This machinery is loaded by default, but may be dynamically removed
	//		via the has.js API and statically removed via the build system.
	//
	//		This loader includes sniffing machinery to determine the environment; the following environments are supported:
	//
	//			* browser
	//			* node.js
	//			* rhino
	//
	//		This is the so-called "source loader". As such, it includes many optional features that may be discadred by
	//		building a customized verion with the build system.

	// Design and Implementation Notes
	//
	// This is a dojo-specific adaption of bdLoad, donated to the dojo foundation by Altoviso LLC.
	//
	// This function defines an AMD-compliant (http://wiki.commonjs.org/wiki/Modules/AsynchronousDefinition)
	// loader that can be configured to operate in either synchronous or asynchronous modes.
	//
	// Since this machinery implements a loader, it does not have the luxury of using a load system and/or
	// leveraging a utility library. This results in an unpleasantly long file; here is a road map of the contents:
	//
	//	 1. Small library for use implementing the loader.
	//	 2. Define the has.js API; this is used throughout the loader to bracket features.
	//	 3. Define the node.js and rhino sniffs and sniff.
	//	 4. Define the loader's data.
	//	 5. Define the configuration machinery.
	//	 6. Define the script element sniffing machinery and sniff for configuration data.
	//	 7. Configure the loader IAW the provided user, default, and sniffing data.
	//	 8. Define the global require function.
	//	 9. Define the module resolution machinery.
	//	10. Define the module and plugin module definition machinery
	//	11. Define the script injection machinery.
	//	12. Define the window load detection.
	//	13. Define the logging API.
	//	14. Define the tracing API.
	//	16. Define the AMD define function.
	//	17. Define the dojo v1.x provide/require machinery--so called "legacy" modes.
	//	18. Publish global variables.
	//
	// Language and Acronyms and Idioms
	//
	// moduleId: a CJS module identifier, (used for public APIs)
	// mid: moduleId (used internally)
	// packageId: a package identifier (used for public APIs)
	// pid: packageId (used internally); the implied system or default package has pid===""
	// pack: package is used internally to reference a package object (since javascript has reserved words including "package")
	// prid: plugin resource identifier
	// The integer constant 1 is used in place of true and 0 in place of false.

	// define a minimal library to help build the loader
	var	noop = function(){
		},

		isEmpty = function(it){
			for(var p in it){
				return 0;
			}
			return 1;
		},

		toString = {}.toString,

		isFunction = function(it){
			return toString.call(it) == "[object Function]";
		},

		isString = function(it){
			return toString.call(it) == "[object String]";
		},

		isArray = function(it){
			return toString.call(it) == "[object Array]";
		},

		forEach = function(vector, callback){
			if(vector){
				for(var i = 0; i < vector.length;){
					callback(vector[i++]);
				}
			}
		},

		mix = function(dest, src){
			for(var p in src){
				dest[p] = src[p];
			}
			return dest;
		},

		makeError = function(error, info){
			return mix(new Error(error), {src:"dojoLoader", info:info});
		},

		uidSeed = 1,

		uid = function(){
			// Returns a unique indentifier (within the lifetime of the document) of the form /_d+/.
			return "_" + uidSeed++;
		},

		// FIXME: how to doc window.require() api

		// this will be the global require function; define it immediately so we can start hanging things off of it
		req = function(
			config,       //(object, optional) hash of configuration properties
			dependencies, //(array of commonjs.moduleId, optional) list of modules to be loaded before applying callback
			callback      //(function, optional) lamda expression to apply to module values implied by dependencies
		){
			return contextRequire(config, dependencies, callback, 0, req);
		},

		// the loader uses the has.js API to control feature inclusion/exclusion; define then use throughout
		global = this,

		doc = global.document,

		element = doc && doc.createElement("DiV"),

		has = req.has = function(name){
			return isFunction(hasCache[name]) ? (hasCache[name] = hasCache[name](global, doc, element)) : hasCache[name];
		},

		hasCache = has.cache = defaultConfig.hasCache;

	has.add = function(name, test, now, force){
		(hasCache[name]===undefined || force) && (hasCache[name] = test);
		return now && has(name);
	};

	false && has.add("host-node", userConfig.has && "host-node" in userConfig.has ?
		userConfig.has["host-node"] :
		(typeof process == "object" && process.versions && process.versions.node && process.versions.v8));
	if(0){
		// fixup the default config for node.js environment
		require("./_base/configNode.js").config(defaultConfig);
		// remember node's require (with respect to baseUrl==dojo's root)
		defaultConfig.loaderPatch.nodeRequire = require;
	}

	false && has.add("host-rhino", userConfig.has && "host-rhino" in userConfig.has ?
		userConfig.has["host-rhino"] :
		(typeof load == "function" && (typeof Packages == "function" || typeof Packages == "object")));
	if(0){
		// owing to rhino's lame feature that hides the source of the script, give the user a way to specify the baseUrl...
		for(var baseUrl = userConfig.baseUrl || ".", arg, rhinoArgs = this.arguments, i = 0; i < rhinoArgs.length;){
			arg = (rhinoArgs[i++] + "").split("=");
			if(arg[0] == "baseUrl"){
				baseUrl = arg[1];
				break;
			}
		}
		load(baseUrl + "/_base/configRhino.js");
		rhinoDojoConfig(defaultConfig, baseUrl, rhinoArgs);
	}

	// userConfig has tests override defaultConfig has tests; do this after the environment detection because
	// the environment detection usually sets some has feature values in the hasCache.
	for(var p in userConfig.has){
		has.add(p, userConfig.has[p], 0, 1);
	}

	//
	// define the loader data
	//

	// the loader will use these like symbols if the loader has the traceApi; otherwise
	// define magic numbers so that modules can be provided as part of defaultConfig
	var	requested = 1,
		arrived = 2,
		nonmodule = 3,
		executing = 4,
		executed = 5;

	if(0){
		// these make debugging nice; but using strings for symbols is a gross rookie error; don't do it for production code
		requested = "requested";
		arrived = "arrived";
		nonmodule = "not-a-module";
		executing = "executing";
		executed = "executed";
	}

	var legacyMode = 0,
		sync = "sync",
		xd = "xd",
		syncExecStack = [],
		dojoRequirePlugin = 0,
		checkDojoRequirePlugin = noop,
		transformToAmd = noop,
		getXhr;
	if(1){
		req.isXdUrl = noop;

		req.initSyncLoader = function(dojoRequirePlugin_, checkDojoRequirePlugin_, transformToAmd_){
			if(!dojoRequirePlugin){
				dojoRequirePlugin = dojoRequirePlugin_;
				checkDojoRequirePlugin = checkDojoRequirePlugin_;
				transformToAmd = transformToAmd_;
			}
			return {
				sync:sync,
				xd:xd,
				arrived:arrived,
				nonmodule:nonmodule,
				executing:executing,
				executed:executed,
				syncExecStack:syncExecStack,
				modules:modules,
				execQ:execQ,
				getModule:getModule,
				injectModule:injectModule,
				setArrived:setArrived,
				signal:signal,
				finishExec:finishExec,
				execModule:execModule,
				dojoRequirePlugin:dojoRequirePlugin,
				getLegacyMode:function(){return legacyMode;},
				holdIdle:function(){checkCompleteGuard++;},
				releaseIdle:function(){checkIdle();}
			};
		};

		if(1){
			// in legacy sync mode, the loader needs a minimal XHR library to load dojo/_base/loader and dojo/_base/xhr

			var locationProtocol = location.protocol,
				locationHost = location.host,
				fileProtocol = !locationHost;
			req.isXdUrl = function(url){
				if(fileProtocol || /^\./.test(url)){
					// begins with a dot is always relative to page URL; therefore not xdomain
					return false;
				}
				if(/^\/\//.test(url)){
					// for v1.6- backcompat, url starting with // indicates xdomain
					return true;
				}
				// get protocol and host
				var match = url.match(/^([^\/\:]+\:)\/\/([^\/]+)/);
				return match && (match[1] != locationProtocol || match[2] != locationHost);
			};

			// note: to get the file:// protocol to work in FF, you must set security.fileuri.strict_origin_policy to false in about:config
			true || has.add("dojo-xhr-factory", 1);
			has.add("dojo-force-activex-xhr", 1 && !doc.addEventListener && window.location.protocol == "file:");
			has.add("native-xhr", typeof XMLHttpRequest != "undefined");
			if(has("native-xhr") && !has("dojo-force-activex-xhr")){
				getXhr = function(){
					return new XMLHttpRequest();
				};
			}else{
				// if in the browser an old IE; find an xhr
				for(var XMLHTTP_PROGIDS = ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP', 'Msxml2.XMLHTTP.4.0'], progid, i = 0; i < 3;){
					try{
						progid = XMLHTTP_PROGIDS[i++];
						if(new ActiveXObject(progid)){
							// this progid works; therefore, use it from now on
							break;
						}
					}catch(e){
						// squelch; we're just trying to find a good ActiveX progid
						// if they all fail, then progid ends up as the last attempt and that will signal the error
						// the first time the client actually tries to exec an xhr
					}
				}
				getXhr = function(){
					return new ActiveXObject(progid);
				};
			}
			req.getXhr = getXhr;

			has.add("dojo-gettext-api", 1);
			req.getText = function(url, async, onLoad){
				var xhr = getXhr();
				xhr.open('GET', fixupUrl(url), false);
				xhr.send(null);
				if(xhr.status == 200 || (!location.host && !xhr.status)){
					if(onLoad){
						onLoad(xhr.responseText, async);
					}
				}else{
					throw makeError("xhrFailed", xhr.status);
				}
				return xhr.responseText;
			};
		}
	}else{
		req.async = 1;
	}

	//
	// loader eval
	//
	var eval_ =
		// use the function constructor so our eval is scoped close to (but not in) in the global space with minimal pollution
		new Function('return eval(arguments[0]);');

	req.eval =
		function(text, hint){
			return eval_(text + "\r\n////@ sourceURL=" + hint);
		};

	//
	// loader micro events API
	//
	var listenerQueues = {},
		error = "error",
		signal = req.signal = function(type, args){
			var queue = listenerQueues[type];
			// notice we run a copy of the queue; this allows listeners to add/remove
			// other listeners without affecting this particular signal
			forEach(queue && queue.slice(0), function(listener){
				listener.apply(null, isArray(args) ? args : [args]);
			});
		},
		on = req.on = function(type, listener){
			// notice a queue is not created until a client actually connects
			var queue = listenerQueues[type] || (listenerQueues[type] = []);
			queue.push(listener);
			return {
				remove:function(){
					for(var i = 0; i<queue.length; i++){
						if(queue[i]===listener){
							queue.splice(i, 1);
							return;
						}
					}
				}
			};
		};

	// configuration machinery; with an optimized/built defaultConfig, all configuration machinery can be discarded
	// lexical variables hold key loader data structures to help with minification; these may be completely,
	// one-time initialized by defaultConfig for optimized/built versions
	var
		aliases
			// a vector of pairs of [regexs or string, replacement] => (alias, actual)
			= [],

		paths
			// CommonJS paths
			= {},

		pathsMapProg
			// list of (from-path, to-path, regex, length) derived from paths;
			// a "program" to apply paths; see computeMapProg
			= [],

		packs
			// a map from packageId to package configuration object; see fixupPackageInfo
			= {},

		packageMap
			// map from package name to local-installed package name
			= {},

		packageMapProg
			// list of (from-package, to-package, regex, length) derived from packageMap;
			// a "program" to apply paths; see computeMapProg
			= [],

		modules
			// A hash:(mid) --> (module-object) the module namespace
			//
			// pid: the package identifier to which the module belongs (e.g., "dojo"); "" indicates the system or default package
			// mid: the fully-resolved (i.e., mappings have been applied) module identifier without the package identifier (e.g., "dojo/io/script")
			// url: the URL from which the module was retrieved
			// pack: the package object of the package to which the module belongs
			// executed: 0 => not executed; executing => in the process of tranversing deps and running factory; executed => factory has been executed
			// deps: the dependency vector for this module (vector of modules objects)
			// def: the factory for this module
			// result: the result of the running the factory for this module
			// injected: (requested | arrived | nonmodule) the status of the module; nonmodule means the resource did not call define
			// load: plugin load function; applicable only for plugins
			//
			// Modules go through several phases in creation:
			//
			// 1. Requested: some other module's definition or a require application contained the requested module in
			//    its dependency vector or executing code explicitly demands a module via req.require.
			//
			// 2. Injected: a script element has been appended to the insert-point element demanding the resource implied by the URL
			//
			// 3. Loaded: the resource injected in [2] has been evalated.
			//
			// 4. Defined: the resource contained a define statement that advised the loader about the module. Notice that some
			//    resources may just contain a bundle of code and never formally define a module via define
			//
			// 5. Evaluated: the module was defined via define and the loader has evaluated the factory and computed a result.
			= {},

		cacheBust
			// query string to append to module URLs to bust browser cache
			= "",

		cache
			// hash:(mid)-->(function)
			//
			// Gives the contents of a cached resource; function should cause the same actions as if the given mid was downloaded
			// and evaluated by the host environment
			 = {},

		pendingCacheInsert
			// hash:(mid)-->(function)
			//
			// Gives a set of cache modules pending entry into cache. When cached modules are published to the loader, they are
			// entered into pendingCacheInsert; modules are then pressed into cache upon (1) AMD define or (2) upon receiving another
			// independent set of cached modules. (1) is the usual case, and this case allows normalizing mids given in the pending
			// cache for the local configuration, possibly relocating modules.
			 = {},

		dojoSniffConfig
			// map of configuration variables
			// give the data-dojo-config as sniffed from the document (if any)
			= {};

	if(1){
		var consumePendingCacheInsert = function(referenceModule){
				var p, item, match, now;
				for(p in pendingCacheInsert){
					item = pendingCacheInsert[p];
					match = p.match(/^url\:(.+)/);
					if(match){
						cache[toUrl(match[1], referenceModule)] =  item;
					}else if(p=="*now"){
						now = item;
					}else if(p!="*noref"){
						cache[getModuleInfo(p, referenceModule).mid] = item;
					}
				}
				if(now){
					now(createRequire(referenceModule));
				}
				pendingCacheInsert = {};
			},

			computeMapProg = function(map, dest, packName){
				// This routine takes a map target-prefix(string)-->replacement(string) into a vector
				// of quads (target-prefix, replacement, regex-for-target-prefix, length-of-target-prefix)
				//
				// The loader contains processes that map one string prefix to another. These
				// are encountered when applying the requirejs paths configuration and when mapping
				// package names. We can make the mapping and any replacement easier and faster by
				// replacing the map with a vector of quads and then using this structure in the simple machine runMapProg.
				dest.splice(0, dest.length);
				var p, i, item, reverseName = 0;
				for(p in map){
					dest.push([p, map[p]]);
					if(map[p]==packName){
						reverseName = p;
					}
				}
				dest.sort(function(lhs, rhs){
					return rhs[0].length - lhs[0].length;
				});
				for(i = 0; i < dest.length;){
					item = dest[i++];
					item[2] = new RegExp("^" + item[0].replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(c){ return "\\" + c; }) + "(\/|$)");
					item[3] = item[0].length + 1;
				}
				return reverseName;
			},

			fixupPackageInfo = function(packageInfo, baseUrl){
				// calculate the precise (name, baseUrl, main, mappings) for a package
				var name = packageInfo.name;
				if(!name){
					// packageInfo must be a string that gives the name
					name = packageInfo;
					packageInfo = {name:name};
				}
				packageInfo = mix({main:"main", mapProg:[]}, packageInfo);
				packageInfo.location = (baseUrl || "") + (packageInfo.location ? packageInfo.location : name);
				packageInfo.reverseName = computeMapProg(packageInfo.packageMap, packageInfo.mapProg, name);

				if(!packageInfo.main.indexOf("./")){
					packageInfo.main = packageInfo.main.substring(2);
				}

				// allow paths to be specified in the package info
				// TODO: this is not supported; remove
				mix(paths, packageInfo.paths);

				// now that we've got a fully-resolved package object, push it into the configuration
				packs[name] = packageInfo;
				packageMap[name] = name;
			},

			config = function(config, booting){
				for(var p in config){
					if(p=="waitSeconds"){
						req.waitms = (config[p] || 0) * 1000;
					}
					if(p=="cacheBust"){
						cacheBust = config[p] ? (isString(config[p]) ? config[p] : (new Date()).getTime() + "") : "";
					}
					if(p=="baseUrl" || p=="combo"){
						req[p] = config[p];
					}
					if(1 && p=="async"){
						// falsy or "sync" => legacy sync loader
						// "xd" => sync but loading xdomain tree and therefore loading asynchronously (not configurable, set automatically by the loader)
						// "legacyAsync" => permanently in "xd" by choice
						// "debugAtAllCosts" => trying to load everything via script injection (not implemented)
						// otherwise, must be truthy => AMD
						// legacyMode: sync | legacyAsync | xd | false
						var mode = config[p];
						req.legacyMode = legacyMode = (isString(mode) && /sync|legacyAsync/.test(mode) ? mode : (!mode ? "sync" : false));
						req.async = !legacyMode;
					}
					if(config[p]!==hasCache){
						// accumulate raw config info for client apps which can use this to pass their own config
						req.rawConfig[p] = config[p];
						p!="has" && has.add("config-"+p, config[p], 0, booting);
					}
				}

				// make sure baseUrl exists
				if(!req.baseUrl){
					req.baseUrl = "./";
				}
				// make sure baseUrl ends with a slash
				if(!/\/$/.test(req.baseUrl)){
					req.baseUrl += "/";
				}

				// now do the special work for has, packages, packagePaths, paths, aliases, and cache

				for(p in config.has){
					has.add(p, config.has[p], 0, booting);
				}

				// for each package found in any packages config item, augment the packs map owned by the loader
				forEach(config.packages, fixupPackageInfo);

				// for each packagePath found in any packagePaths config item, augment the packs map owned by the loader
				for(baseUrl in config.packagePaths){
					forEach(config.packagePaths[baseUrl], function(packageInfo){
						fixupPackageInfo(packageInfo, baseUrl + "/");
					});
				}

				// push in any paths and recompute the internal pathmap
				// warning: this cann't be done until the package config is processed since packages may include path info
				computeMapProg(mix(paths, config.paths), pathsMapProg);

				// aliases
				forEach(config.aliases, function(pair){
					if(isString(pair[0])){
						pair[0] = new RegExp("^" + pair[0].replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(c){return "\\" + c;}) + "$");
					}
					aliases.push(pair);
				});

				// mix any packageMap config item and recompute the internal packageMapProg
				computeMapProg(mix(packageMap, config.packageMap), packageMapProg);

				// push in any new cache values
				if(config.cache){
					consumePendingCacheInsert();
					pendingCacheInsert = config.cache;
					if(config.cache["*noref"]){
						consumePendingCacheInsert();
					}
				}

				signal("config", [config, req.rawConfig]);
			};

		//
		// execute the various sniffs
		//

		if(has("dojo-cdn") || 1){
			for(var dojoDir, src, match, scripts = doc.getElementsByTagName("script"), i = 0; i < scripts.length && !match; i++){
				if((src = scripts[i].getAttribute("src")) && (match = src.match(/(.*)\/?dojo\.js(\W|$)/i))){
					// if baseUrl wasn't explicitly set, set it here to the dojo directory; this is the 1.6- behavior
					userConfig.baseUrl = dojoDir = userConfig.baseUrl || defaultConfig.baseUrl || match[1];

					// see if there's a dojo configuration stuffed into the node
					src = (scripts[i].getAttribute("data-dojo-config") || scripts[i].getAttribute("djConfig"));
					if(src){
						dojoSniffConfig = req.eval("({ " + src + " })", "data-dojo-config");
					}
					if(0){
						var dataMain = scripts[i].getAttribute("data-main");
						if(dataMain){
							dojoSniffConfig.deps = dojoSniffConfig.deps || [dataMain];
						}
					}
				}
			}
		}

		if(0){
			// pass down doh.testConfig from parent as if it were a data-dojo-config
			try{
				if(window.parent != window && window.parent.require){
					var doh = window.parent.require("doh");
					doh && mix(dojoSniffConfig, doh.testConfig);
				}
			}catch(e){}
		}

		// configure the loader; let the user override defaults
		req.rawConfig = {};
		config(defaultConfig, 1);
		config(userConfig, 1);
		config(dojoSniffConfig, 1);

		if(has("dojo-cdn")){
			packs.dojo.location = dojoDir;
			packs.dijit.location = dojoDir + "../dijit/";
			packs.dojox.location = dojoDir + "../dojox/";
		}

	}else{
		// no config API, assume defaultConfig has everything the loader needs...for the entire lifetime of the application
		paths = defaultConfig.paths;
		pathsMapProg = defaultConfig.pathsMapProg;
		packs = defaultConfig.packs;
		aliases = defaultConfig.aliases;
		packageMap = defaultConfig.packageMap;
		packageMapProg = defaultConfig.packageMapProg;
		modules = defaultConfig.modules;
		cache = defaultConfig.cache;
		cacheBust = defaultConfig.cacheBust;

		// remember the default config for other processes (e.g., dojo/config)
		req.rawConfig = defaultConfig;
	}


	if(0){
		req.combo = req.combo || {add:noop};
		var	comboPending = 0,
			combosPending = [],
			comboPendingTimer = null;
	}


	// build the loader machinery iaw configuration, including has feature tests
	var	injectDependencies = function(module){
			// checkComplete!=0 holds the idle signal; we're not idle if we're injecting dependencies
			checkCompleteGuard++;
			forEach(module.deps, injectModule);
			if(0 && comboPending && !comboPendingTimer){
				comboPendingTimer = setTimeout(function() {
					comboPending = 0;
					comboPendingTimer = null;
					req.combo.done(function(mids, url) {
						var onLoadCallback= function(){
							// defQ is a vector of module definitions 1-to-1, onto mids
							runDefQ(0, mids);
							checkComplete();
						};
						combosPending.push(mids);
						injectingModule = mids;
						req.injectUrl(url, onLoadCallback, mids);
						injectingModule = 0;
					}, req);
				}, 0);
			}
			checkIdle();
		},

		contextRequire = function(a1, a2, a3, referenceModule, contextRequire){
			var module, syntheticMid;
			if(isString(a1)){
				// signature is (moduleId)
				module = getModule(a1, referenceModule, true);
				if(module && module.executed){
					return module.result;
				}
				throw makeError("undefinedModule", a1);
			}
			if(!isArray(a1)){
				// a1 is a configuration
				config(a1);

				// juggle args; (a2, a3) may be (dependencies, callback)
				a1 = a2;
				a2 = a3;
			}
			if(isArray(a1)){
				// signature is (requestList [,callback])
				if(!a1.length){
					a2 && a2();
				}else{
					syntheticMid = "require*" + uid();

					// resolve the request list with respect to the reference module
					for(var mid, deps = [], i = 0; i < a1.length;){
						mid = a1[i++];
						if(mid in {exports:1, module:1}){
							throw makeError("illegalModuleId", mid);
						}
						deps.push(getModule(mid, referenceModule));
					}

					// construct a synthetic module to control execution of the requestList, and, optionally, callback
					module = mix(makeModuleInfo("", syntheticMid, 0, ""), {
						injected: arrived,
						deps: deps,
						def: a2 || noop,
						require: referenceModule ? referenceModule.require : req
					});
					modules[module.mid] = module;

					// checkComplete!=0 holds the idle signal; we're not idle if we're injecting dependencies
					injectDependencies(module);

					// try to immediately execute
					// if already traversing a factory tree, then strict causes circular dependency to abort the execution; maybe
					// it's possible to execute this require later after the current traversal completes and avoid the circular dependency.
					// ...but *always* insist on immediate in synch mode
					var strict = checkCompleteGuard && req.async;
					checkCompleteGuard++;
					execModule(module, strict);
					checkIdle();
					if(!module.executed){
						// some deps weren't on board or circular dependency detected and strict; therefore, push into the execQ
						execQ.push(module);
					}
					checkComplete();
				}
			}
			return contextRequire;
		},

		createRequire = function(module){
			if(!module){
				return req;
			}
			var result = module.require;
			if(!result){
				result = function(a1, a2, a3){
					return contextRequire(a1, a2, a3, module, result);
				};
				module.require = mix(result, req);
				result.module = module;
				result.toUrl = function(name){
					return toUrl(name, module);
				};
				result.toAbsMid = function(mid){
					return toAbsMid(mid, module);
				};
				if(0){
					result.undef = function(mid){
						req.undef(mid, module);
					};
				}
				if(1){
					result.syncLoadNls = function(mid){
						var nlsModuleInfo = getModuleInfo(mid, module),
							nlsModule = modules[nlsModuleInfo.mid];
						if(!nlsModule || !nlsModule.executed){
							cached = cache[nlsModuleInfo.mid] || cache[nlsModuleInfo.cacheId];
							if(cached){
								evalModuleText(cached);
								nlsModule = modules[nlsModuleInfo.mid];
							}
						}
						return nlsModule && nlsModule.executed && nlsModule.result;
					};
				}

			}
			return result;
		},

		execQ =
			// The list of modules that need to be evaluated.
			[],

		defQ =
			// The queue of define arguments sent to loader.
			[],

		waiting =
			// The set of modules upon which the loader is waiting for definition to arrive
			{},

		setRequested = function(module){
			module.injected = requested;
			waiting[module.mid] = 1;
			if(module.url){
				waiting[module.url] = module.pack || 1;
			}
		},

		setArrived = function(module){
			module.injected = arrived;
			delete waiting[module.mid];
			if(module.url){
				delete waiting[module.url];
			}
			if(isEmpty(waiting)){
				clearTimer();
				1 && legacyMode==xd && (legacyMode = sync);
			}
		},

		execComplete = req.idle =
			// says the loader has completed (or not) its work
			function(){
				return !defQ.length && isEmpty(waiting) && !execQ.length && !checkCompleteGuard;
			},

		runMapProg = function(targetMid, map){
			// search for targetMid in map; return the map item if found; falsy otherwise
			for(var i = 0; i < map.length; i++){
				if(map[i][2].test(targetMid)){
					return map[i];
				}
			}
			return 0;
		},

		compactPath = function(path){
			var result = [],
				segment, lastSegment;
			path = path.replace(/\\/g, '/').split('/');
			while(path.length){
				segment = path.shift();
				if(segment==".." && result.length && lastSegment!=".."){
					result.pop();
					lastSegment = result[result.length - 1];
				}else if(segment!="."){
					result.push(lastSegment= segment);
				} // else ignore "."
			}
			return result.join("/");
		},

		makeModuleInfo = function(pid, mid, pack, url, cacheId){
			if(1){
				var xd= req.isXdUrl(url);
				return {pid:pid, mid:mid, pack:pack, url:url, executed:0, def:0, isXd:xd, isAmd:!!(xd || (packs[pid] && packs[pid].isAmd)), cacheId:cacheId};
			}else{
				return {pid:pid, mid:mid, pack:pack, url:url, executed:0, def:0, cacheId:cacheId};
			}
		},

		getModuleInfo_ = function(mid, referenceModule, packs, modules, baseUrl, packageMapProg, pathsMapProg, alwaysCreate){
			// arguments are passed instead of using lexical variables so that this function my be used independent of the loader (e.g., the builder)
			// alwaysCreate is useful in this case so that getModuleInfo never returns references to real modules owned by the loader
			var pid, pack, midInPackage, mapProg, mapItem, path, url, result, isRelative, requestedMid, cacheId=0;
			requestedMid = mid;
			isRelative = /^\./.test(mid);
			if(/(^\/)|(\:)|(\.js$)/.test(mid) || (isRelative && !referenceModule)){
				// absolute path or protocol of .js filetype, or relative path but no reference module and therefore relative to page
				// whatever it is, it's not a module but just a URL of some sort
				return makeModuleInfo(0, mid, 0, mid);
			}else{
				// relative module ids are relative to the referenceModule; get rid of any dots
				mid = compactPath(isRelative ? (referenceModule.mid + "/../" + mid) : mid);
				if(/^\./.test(mid)){
					throw makeError("irrationalPath", mid);
				}
				// find the package indicated by the mid, if any
				mapProg = referenceModule && referenceModule.pack && referenceModule.pack.mapProg;
				mapItem = (mapProg && runMapProg(mid, mapProg)) || runMapProg(mid, packageMapProg);
				if(mapItem){
					// mid specified a module that's a member of a package; figure out the package id and module id
					// notice we expect pack.main to be valid with no pre or post slash
					pid = mapItem[1];
					mid = mid.substring(mapItem[3]);
					pack = packs[pid];
					if(!mid){
						mid= pack.main;
					}
					midInPackage = mid;
					cacheId = pack.reverseName + "/" + mid;
					mid = pid + "/" + mid;
				}else{
					pid = "";
				}

				// search aliases
				var candidateLength = 0,
					candidate = 0;
				forEach(aliases, function(pair){
					var match = mid.match(pair[0]);
					if(match && match.length>candidateLength){
						candidate = isFunction(pair[1]) ? mid.replace(pair[0], pair[1]) : pair[1];
					}
				});
				if(candidate){
					return getModuleInfo_(candidate, 0, packs, modules, baseUrl, packageMapProg, pathsMapProg, alwaysCreate);
				}

				result = modules[mid];
				if(result){
					return alwaysCreate ? makeModuleInfo(result.pid, result.mid, result.pack, result.url, cacheId) : modules[mid];
				}
			}
			// get here iff the sought-after module does not yet exist; therefore, we need to compute the URL given the
			// fully resolved (i.e., all relative indicators and package mapping resolved) module id

			mapItem = runMapProg(mid, pathsMapProg);
			if(mapItem){
				url = mapItem[1] + mid.substring(mapItem[3] - 1);
			}else if(pid){
				url = pack.location + "/" + midInPackage;
			}else if(has("config-tlmSiblingOfDojo")){
				url = "../" + mid;
			}else{
				url = mid;
			}
			// if result is not absolute, add baseUrl
			if(!(/(^\/)|(\:)/.test(url))){
				url = baseUrl + url;
			}
			url += ".js";
			return makeModuleInfo(pid, mid, pack, compactPath(url), cacheId);
		},

		getModuleInfo = function(mid, referenceModule){
			return getModuleInfo_(mid, referenceModule, packs, modules, req.baseUrl, packageMapProg, pathsMapProg);
		},

		resolvePluginResourceId = function(plugin, prid, referenceModule){
			return plugin.normalize ? plugin.normalize(prid, function(mid){return toAbsMid(mid, referenceModule);}) : toAbsMid(prid, referenceModule);
		},

		dynamicPluginUidGenerator = 0,

		getModule = function(mid, referenceModule, immediate){
			// compute and optionally construct (if necessary) the module implied by the mid with respect to referenceModule
			var match, plugin, prid, result;
			match = mid.match(/^(.+?)\!(.*)$/);
			if(match){
				// name was <plugin-module>!<plugin-resource-id>
				plugin = getModule(match[1], referenceModule, immediate);

				if(1 && legacyMode == sync && !plugin.executed){
					injectModule(plugin);
					if(plugin.injected===arrived && !plugin.executed){
						checkCompleteGuard++;
						execModule(plugin);
						checkIdle();
					}
					if(plugin.executed){
						promoteModuleToPlugin(plugin);
					}else{
						// we are in xdomain mode for some reason
						execQ.unshift(plugin);
					}
				}



				if(plugin.executed === executed && !plugin.load){
					// executed the module not knowing it was a plugin
					promoteModuleToPlugin(plugin);
				}

				// if the plugin has not been loaded, then can't resolve the prid and  must assume this plugin is dynamic until we find out otherwise
				if(plugin.load){
					prid = resolvePluginResourceId(plugin, match[2], referenceModule);
					mid = (plugin.mid + "!" + (plugin.dynamic ? ++dynamicPluginUidGenerator + "!" : "") + prid);
				}else{
					prid = match[2];
					mid = plugin.mid + "!" + (++dynamicPluginUidGenerator) + "!waitingForPlugin";
				}
				result = {plugin:plugin, mid:mid, req:createRequire(referenceModule), prid:prid};
			}else{
				result = getModuleInfo(mid, referenceModule);
			}
			return modules[result.mid] || (!immediate && (modules[result.mid] = result));
		},

		toAbsMid = req.toAbsMid = function(mid, referenceModule){
			return getModuleInfo(mid, referenceModule).mid;
		},

		toUrl = req.toUrl = function(name, referenceModule){
			var moduleInfo = getModuleInfo(name+"/x", referenceModule),
				url = moduleInfo.url;
			return fixupUrl(moduleInfo.pid===0 ?
				// if pid===0, then name had a protocol or absolute path; either way, toUrl is the identify function in such cases
				name :
				// "/x.js" since getModuleInfo automatically appends ".js" and we appended "/x" to make name look likde a module id
				url.substring(0, url.length-5)
			);
		},

		nonModuleProps = {
			injected: arrived,
			executed: executed,
			def: nonmodule,
			result: nonmodule
		},

		makeCjs = function(mid){
			return modules[mid] = mix({mid:mid}, nonModuleProps);
		},

		cjsRequireModule = makeCjs("require"),
		cjsExportsModule = makeCjs("exports"),
		cjsModuleModule = makeCjs("module"),

		runFactory = function(module, args){
			req.trace("loader-run-factory", [module.mid]);
			var factory = module.def,
				result;
			1 && syncExecStack.unshift(module);
			if(has("config-dojo-loader-catches")){
				try{
					result= isFunction(factory) ? factory.apply(null, args) : factory;
				}catch(e){
					signal(error, module.result = makeError("factoryThrew", [module, e]));
				}
			}else{
				result= isFunction(factory) ? factory.apply(null, args) : factory;
			}
			module.result = result===undefined && module.cjs ? module.cjs.exports : result;
			1 && syncExecStack.shift(module);
		},

		abortExec = {},

		defOrder = 0,

		promoteModuleToPlugin = function(pluginModule){
			var plugin = pluginModule.result;
			pluginModule.dynamic = plugin.dynamic;
			pluginModule.normalize = plugin.normalize;
			pluginModule.load = plugin.load;
			return pluginModule;
		},

		resolvePluginLoadQ = function(plugin){
			// plugins is a newly executed module that has a loadQ waiting to run

			// step 1: traverse the loadQ and fixup the mid and prid; remember the map from original mid to new mid
			// recall the original mid was created before the plugin was on board and therefore it was impossible to
			// compute the final mid; accordingly, prid may or may not change, but the mid will definitely change
			var map = {};
			forEach(plugin.loadQ, function(pseudoPluginResource){
				// manufacture and insert the real module in modules
				var pseudoMid = pseudoPluginResource.mid,
					prid = resolvePluginResourceId(plugin, pseudoPluginResource.prid, pseudoPluginResource.req.module),
					mid = plugin.dynamic ? pseudoPluginResource.mid.replace(/waitingForPlugin$/, prid) : (plugin.mid + "!" + prid),
					pluginResource = mix(mix({}, pseudoPluginResource), {mid:mid, prid:prid, injected:0});
				if(!modules[mid]){
					// create a new (the real) plugin resource and inject it normally now that the plugin is on board
					injectPlugin(modules[mid] = pluginResource);
				} // else this was a duplicate request for the same (plugin, rid) for a nondynamic plugin

				// pluginResource is really just a placeholder with the wrong mid (because we couldn't calculate it until the plugin was on board)
				// mark is as arrived and delete it from modules; the real module was requested above
				map[pseudoPluginResource.mid] = modules[mid];
				setArrived(pseudoPluginResource);
				delete modules[pseudoPluginResource.mid];
			});
			plugin.loadQ = 0;

			// step2: replace all references to any placeholder modules with real modules
			var substituteModules = function(module){
				for(var replacement, deps = module.deps || [], i = 0; i<deps.length; i++){
					replacement = map[deps[i].mid];
					if(replacement){
						deps[i] = replacement;
					}
				}
			};
			for(var p in modules){
				substituteModules(modules[p]);
			}
			forEach(execQ, substituteModules);
		},

		finishExec = function(module){
			req.trace("loader-finish-exec", [module.mid]);
			module.executed = executed;
			module.defOrder = defOrder++;
			1 && forEach(module.provides, function(cb){ cb(); });
			if(module.loadQ){
				// the module was a plugin
				promoteModuleToPlugin(module);
				resolvePluginLoadQ(module);
			}
			// remove all occurences of this module from the execQ
			for(i = 0; i < execQ.length;){
				if(execQ[i] === module){
					execQ.splice(i, 1);
				}else{
					i++;
				}
			}
		},

		circleTrace = [],

		execModule = function(module, strict){
			// run the dependency vector, then run the factory for module
			if(module.executed === executing){
				req.trace("loader-circular-dependency", [circleTrace.concat(mid).join("->")]);
				return (!module.def || strict) ? abortExec :  (module.cjs && module.cjs.exports);
			}
			// at this point the module is either not executed or fully executed


			if(!module.executed){
				if(!module.def){
					return abortExec;
				}
				var mid = module.mid,
					deps = module.deps || [],
					arg, argResult,
					args = [],
					i = 0;

				if(0){
					circleTrace.push(mid);
					req.trace("loader-exec-module", ["exec", circleTrace.length, mid]);
				}

				// for circular dependencies, assume the first module encountered was executed OK
				// modules that circularly depend on a module that has not run its factory will get
				// the premade cjs.exports===module.result. They can take a reference to this object and/or
				// add properties to it. When the module finally runs its factory, the factory can
				// read/write/replace this object. Notice that so long as the object isn't replaced, any
				// reference taken earlier while walking the deps list is still valid.
				module.executed = executing;
				while(i < deps.length){
					arg = deps[i++];
					argResult = ((arg === cjsRequireModule) ? createRequire(module) :
									((arg === cjsExportsModule) ? module.cjs.exports :
										((arg === cjsModuleModule) ? module.cjs :
											execModule(arg, strict))));
					if(argResult === abortExec){
						module.executed = 0;
						req.trace("loader-exec-module", ["abort", mid]);
						0 && circleTrace.pop();
						return abortExec;
					}
					args.push(argResult);
				}
				runFactory(module, args);
				finishExec(module);
			}
			// at this point the module is guaranteed fully executed

			0 && circleTrace.pop();
			return module.result;
		},


		checkCompleteGuard =  0,

		checkComplete = function(){
			// keep going through the execQ as long as at least one factory is executed
			// plugins, recursion, cached modules all make for many execution path possibilities
			if(checkCompleteGuard){
				return;
			}
			checkCompleteGuard++;
			checkDojoRequirePlugin();
			for(var currentDefOrder, module, i = 0; i < execQ.length;){
				currentDefOrder = defOrder;
				module = execQ[i];
				execModule(module);
				if(currentDefOrder!=defOrder){
					// defOrder was bumped one or more times indicating something was executed (note, this indicates
					// the execQ was modified, maybe a lot (for example a later module causes an earlier module to execute)
					checkDojoRequirePlugin();
					i = 0;
				}else{
					// nothing happened; check the next module in the exec queue
					i++;
				}
			}
			checkIdle();
		},

		checkIdle = function(){
			checkCompleteGuard--;
			if(execComplete()){
				signal("idle", []);
			}
		};


	if(0){
		req.undef = function(moduleId, referenceModule){
			// In order to reload a module, it must be undefined (this routine) and then re-requested.
			// This is useful for testing frameworks (at least).
			var module = getModule(moduleId, referenceModule);
			setArrived(module);
			delete modules[module.mid];
		};
	}

	if(1){
		if(has("dojo-loader-eval-hint-url")===undefined){
			has.add("dojo-loader-eval-hint-url", 1);
		}

		var fixupUrl= function(url){
				url += ""; // make sure url is a Javascript string (some paths may be a Java string)
				return url + (cacheBust ? ((/\?/.test(url) ? "&" : "?") + cacheBust) : "");
			},

			injectPlugin = function(
				module
			){
				// injects the plugin module given by module; may have to inject the plugin itself
				var plugin = module.plugin;

				if(plugin.executed === executed && !plugin.load){
					// executed the module not knowing it was a plugin
					promoteModuleToPlugin(plugin);
				}

				var onLoad = function(def){
						module.result = def;
						setArrived(module);
						finishExec(module);
						checkComplete();
					};

				if(plugin.load){
					plugin.load(module.prid, module.req, onLoad);
				}else if(plugin.loadQ){
					plugin.loadQ.push(module);
				}else{
					// the unshift instead of push is important: we don't want plugins to execute as
					// dependencies of some other module because this may cause circles when the plugin
					// loadQ is run; also, generally, we want plugins to run early since they may load
					// several other modules and therefore can potentially unblock many modules
					plugin.loadQ = [module];
					execQ.unshift(plugin);
					injectModule(plugin);
				}
			},

			// for IE, injecting a module may result in a recursive execution if the module is in the cache

			cached = 0,

			injectingModule = 0,

			injectingCachedModule = 0,

			evalModuleText = function(text, module){
				// see def() for the injectingCachedModule bracket; it simply causes a short, safe curcuit
				injectingCachedModule = 1;
				if(has("config-dojo-loader-catches")){
					try{
						if(text===cached){
							cached.call(null);
						}else{
							req.eval(text, has("dojo-loader-eval-hint-url") ? module.url : module.mid);
						}
					}catch(e){
						signal(error, makeError("evalModuleThrew", module));
					}
				}else{
					if(text===cached){
						cached.call(null);
					}else{
						req.eval(text, has("dojo-loader-eval-hint-url") ? module.url : module.mid);
					}
				}
				injectingCachedModule = 0;
			},

			injectModule = function(module){
				// Inject the module. In the browser environment, this means appending a script element into
				// the document; in other environments, it means loading a file.
				//
				// If in synchronous mode, then get the module synchronously if it's not xdomainLoading.

				var mid = module.mid,
					url = module.url;
				if(module.executed || module.injected || waiting[mid] || (module.url && ((module.pack && waiting[module.url]===module.pack) || waiting[module.url]==1))){
					return;
				}
				setRequested(module);

				if(0){
					var viaCombo = 0;
					if(module.plugin && module.plugin.isCombo){
						// a combo plugin; therefore, must be handled by combo service
						// the prid should have already been converted to a URL (if required by the plugin) during
						// the normalze process; in any event, there is no way for the loader to know how to
						// to the conversion; therefore the third argument is zero
						req.combo.add(module.plugin.mid, module.prid, 0, req);
						viaCombo = 1;
					}else if(!module.plugin){
						viaCombo = req.combo.add(0, module.mid, module.url, req);
					}
					if(viaCombo){
						comboPending= 1;
						return;
					}
				}

				if(module.plugin){
					injectPlugin(module);
					return;
				} // else a normal module (not a plugin)


				var onLoadCallback = function(){
					runDefQ(module);
					if(module.injected !== arrived){
						// the script that contained the module arrived and has been executed yet
						// nothing was added to the defQ (so it wasn't an AMD module) and the module
						// wasn't marked as arrived by dojo.provide (so it wasn't a v1.6- module);
						// therefore, it must not have been a module; adjust state accordingly
						setArrived(module);
						mix(module, nonModuleProps);
					}

					if(1 && legacyMode){
						// must call checkComplete even in for sync loader because we may be in xdomainLoading mode;
						// but, if xd loading, then don't call checkComplete until out of the current sync traversal
						// in order to preserve order of execution of the dojo.required modules
						!syncExecStack.length && checkComplete();
					}else{
						checkComplete();
					}
				};
				cached = cache[mid] || cache[module.cacheId];
				if(cached){
					req.trace("loader-inject", ["cache", module.mid, url]);
					evalModuleText(cached, module);
					onLoadCallback();
					return;
				}
				if(1 && legacyMode){
					if(module.isXd){
						// switch to async mode temporarily; if current legacyMode!=sync, then is must be one of {legacyAsync, xd, false}
						legacyMode==sync && (legacyMode = xd);
						// fall through and load via script injection
					}else if(module.isAmd && legacyMode!=sync){
						// fall through and load via script injection
					}else{
						// mode may be sync, xd/legacyAsync, or async; module may be AMD or legacy; but module is always located on the same domain
						var xhrCallback = function(text){
							if(legacyMode==sync){
								// the top of syncExecStack gives the current synchronously executing module; the loader needs
								// to know this if it has to switch to async loading in the middle of evaluating a legacy module
								// this happens when a modules dojo.require's a module that must be loaded async because it's xdomain
								// (using unshift/shift because there is no back() methods for Javascript arrays)
								syncExecStack.unshift(module);
								evalModuleText(text, module);
								syncExecStack.shift();

								// maybe the module was an AMD module
								runDefQ(module);

								// legacy modules never get to defineModule() => cjs and injected never set; also evaluation implies executing
								if(!module.cjs){
									setArrived(module);
									finishExec(module);
								}

								if(module.finish){
									// while synchronously evaluating this module, dojo.require was applied referencing a module
									// that had to be loaded async; therefore, the loader stopped answering all dojo.require
									// requests so they could be answered completely in the correct sequence; module.finish gives
									// the list of dojo.requires that must be re-applied once all target modules are available;
									// make a synthetic module to execute the dojo.require's in the correct order

									// compute a guarnateed-unique mid for the synthetic finish module; remember the finish vector; remove it from the reference module
									// TODO: can we just leave the module.finish...what's it hurting?
									var finishMid = mid + "*finish",
										finish = module.finish;
									delete module.finish;

									def(finishMid, ["dojo", ("dojo/require!" + finish.join(",")).replace(/\./g, "/")], function(dojo){
										forEach(finish, function(mid){ dojo.require(mid); });
									});
									// unshift, not push, which causes the current traversal to be reattempted from the top
									execQ.unshift(getModule(finishMid));
								}
								onLoadCallback();
							}else{
								text = transformToAmd(module, text);
								if(text){
									evalModuleText(text, module);
									onLoadCallback();
								}else{
									// if transformToAmd returned falsy, then the module was already AMD and it can be script-injected
									// do so to improve debugability(even though it means another download...which probably won't happen with a good browser cache)
									injectingModule = module;
									req.injectUrl(fixupUrl(url), onLoadCallback, module);
									injectingModule = 0;
								}
							}
						};

						req.trace("loader-inject", ["xhr", module.mid, url, legacyMode!=sync]);
						if(has("config-dojo-loader-catches")){
							try{
								req.getText(url, legacyMode!=sync, xhrCallback);
							}catch(e){
								signal(error, makeError("xhrInjectFailed", [module, e]));
							}
						}else{
							req.getText(url, legacyMode!=sync, xhrCallback);
						}
						return;
					}
				} // else async mode or fell through in xdomain loading mode; either way, load by script injection
				req.trace("loader-inject", ["script", module.mid, url]);
				injectingModule = module;
				req.injectUrl(fixupUrl(url), onLoadCallback, module);
				injectingModule = 0;
			},

			defineModule = function(module, deps, def){
				req.trace("loader-define-module", [module.mid, deps]);

				if(0 && module.plugin && module.plugin.isCombo){
					// the module is a plugin resource loaded by the combo service
					// note: check for module.plugin should be enough since normal plugin resources should
					// not follow this path; module.plugin.isCombo is future-proofing belt and suspenders
					module.result = isFunction(def) ? def() : def;
					setArrived(module);
					finishExec(module);
					return module;
				};

				var mid = module.mid;
				if(module.injected === arrived){
					signal(error, makeError("multipleDefine", module));
					return module;
				}
				mix(module, {
					deps: deps,
					def: def,
					cjs: {
						id: module.mid,
						uri: module.url,
						exports: (module.result = {}),
						setExports: function(exports){
							module.cjs.exports = exports;
						}
					}
				});

				// resolve deps with respect to this module
				for(var i = 0; i < deps.length; i++){
					deps[i] = getModule(deps[i], module);
				}

				if(1 && legacyMode && !waiting[mid]){
					// the module showed up without being asked for; it was probably in a <script> element
					injectDependencies(module);
					execQ.push(module);
					checkComplete();
				}
				setArrived(module);

				if(!isFunction(def) && !deps.length){
					module.result = def;
					finishExec(module);
				}

				return module;
			},

			runDefQ = function(referenceModule, mids){
				// defQ is an array of [id, dependencies, factory]
				// mids (if any) is a vector of mids given by a combo service
				var definedModules = [],
					module, args;
				while(defQ.length){
					args = defQ.shift();
					mids && (args[0]= mids.shift());
					// explicit define indicates possible multiple modules in a single file; delay injecting dependencies until defQ fully
					// processed since modules earlier in the queue depend on already-arrived modules that are later in the queue
					// TODO: what if no args[0] and no referenceModule
					module = (args[0] && getModule(args[0])) || referenceModule;
					definedModules.push([module, args[1], args[2]]);
				}
				consumePendingCacheInsert(referenceModule);
				forEach(definedModules, function(args){
					injectDependencies(defineModule.apply(null, args));
				});
			};
	}

	var timerId = 0,
		clearTimer = noop,
		startTimer = noop;
	if(1){
		// Timer machinery that monitors how long the loader is waiting and signals an error when the timer runs out.
		clearTimer = function(){
			timerId && clearTimeout(timerId);
			timerId = 0;
		},

		startTimer = function(){
			clearTimer();
			req.waitms && (timerId = setTimeout(function(){
					clearTimer();
					signal(error, makeError("timeout", waiting));
			}, req.waitms));
		};
	}

	if(1){
		has.add("ie-event-behavior", !!doc.attachEvent && (typeof opera === "undefined" || opera.toString() != "[object Opera]"));
	}

	if(1 && (1 || 1)){
		var domOn = function(node, eventName, ieEventName, handler){
				// Add an event listener to a DOM node using the API appropriate for the current browser;
				// return a function that will disconnect the listener.
				if(!has("ie-event-behavior")){
					node.addEventListener(eventName, handler, false);
					return function(){
						node.removeEventListener(eventName, handler, false);
					};
				}else{
					node.attachEvent(ieEventName, handler);
					return function(){
						node.detachEvent(ieEventName, handler);
					};
				}
			},
			windowOnLoadListener = domOn(window, "load", "onload", function(){
				req.pageLoaded = 1;
				doc.readyState!="complete" && (doc.readyState = "complete");
				windowOnLoadListener();
			});

		if(1){
			// if the loader is on the page, there must be at least one script element
			// getting its parent and then doing insertBefore solves the "Operation Aborted"
			// error in IE from appending to a node that isn't properly closed; see
			// dojo/tests/_base/loader/requirejs/simple-badbase.html for an example
			var sibling = doc.getElementsByTagName("script")[0],
				insertPoint= sibling.parentNode;
			req.injectUrl = function(url, callback, owner){
				// insert a script element to the insert-point element with src=url;
				// apply callback upon detecting the script has loaded.

				startTimer();
				var node = owner.node = doc.createElement("script"),
					onLoad = function(e){
						e = e || window.event;
						var node = e.target || e.srcElement;
						if(e.type === "load" || /complete|loaded/.test(node.readyState)){
							disconnector();
							callback && callback();
						}
					},
					disconnector = domOn(node, "load", "onreadystatechange", onLoad);
				node.type = "text/javascript";
				node.charset = "utf-8";
				node.src = url;
				insertPoint.insertBefore(node, sibling);
				return node;
			};
		}
	}

	if(1){
		req.log = function(){
			try{
				for(var i = 0; i < arguments.length; i++){
					console.log(arguments[i]);
				}
			}catch(e){}
		};
	}else{
		req.log = noop;
	}

	if(0){
		var trace = req.trace = function(
			group,	// the trace group to which this application belongs
			args	// the contents of the trace
		){
			///
			// Tracing interface by group.
			//
			// Sends the contents of args to the console iff (req.trace.on && req.trace[group])

			if(trace.on && trace.group[group]){
				signal("trace", [group, args]);
				for(var arg, dump = [], text= "trace:" + group + (args.length ? (":" + args[0]) : ""), i= 1; i<args.length;){
					arg = args[i++];
					if(isString(arg)){
						text += ", " + arg;
					}else{
						dump.push(arg);
					}
				}
				req.log(text);
				dump.length && dump.push(".");
				req.log.apply(req, dump);
			}
		};
		mix(trace, {
			on:1,
			group:{},
			set:function(group, value){
				if(isString(group)){
					trace.group[group]= value;
				}else{
					mix(trace.group, group);
				}
			}
		});
		trace.set(mix(mix(mix({}, defaultConfig.trace), userConfig.trace), dojoSniffConfig.trace));
		on("config", function(config){
			config.trace && trace.set(config.trace);
		});
	}else{
		req.trace = noop;
	}

	var def = function(
		mid,		  //(commonjs.moduleId, optional) list of modules to be loaded before running factory
		dependencies, //(array of commonjs.moduleId, optional)
		factory		  //(any)
	){
		///
		// Advises the loader of a module factory. //Implements http://wiki.commonjs.org/wiki/Modules/AsynchronousDefinition.
		///
		//note
		// CommonJS factory scan courtesy of http://requirejs.org

		var arity = arguments.length,
			args = 0,
			defaultDeps = ["require", "exports", "module"];

		if(0){
			if(arity == 1 && isFunction(mid)){
				dependencies = [];
				mid.toString()
				.replace(/(\/\*([\s\S]*?)\*\/|\/\/(.*)$)/mg, "")
				.replace(/require\(["']([\w\!\-_\.\/]+)["']\)/g, function (match, dep){
					dependencies.push(dep);
			});
				args = [0, defaultDeps.concat(dependencies), mid];
			}
		}
		if(!args){
			args = arity == 1 ? [0, defaultDeps, mid] :
				(arity == 2 ? (isArray(mid) ? [0, mid, dependencies] : (isFunction(dependencies) ? [mid, defaultDeps, dependencies] : [mid, [], dependencies])) :
					[mid, dependencies, factory]);
		}
		req.trace("loader-define", args.slice(0, 2));
		var targetModule = args[0] && getModule(args[0]),
			module;
		if(targetModule && !waiting[targetModule.mid]){
			// given a mid that hasn't been requested; therefore, defined through means other than injecting
			// consequent to a require() or define() application; examples include defining modules on-the-fly
			// due to some code path or including a module in a script element. In any case,
			// there is no callback waiting to finish processing and nothing to trigger the defQ and the
			// dependencies are never requested; therefore, do it here.
			injectDependencies(defineModule(targetModule, args[1], args[2]));
		}else if(!has("ie-event-behavior") || !1 || injectingCachedModule){
			// not IE path: anonymous module and therefore must have been injected; therefore, onLoad will fire immediately
			// after script finishes being evaluated and the defQ can be run from that callback to detect the module id
			defQ.push(args);
		}else{
			// IE path: possibly anonymous module and therefore injected; therefore, cannot depend on 1-to-1,
			// in-order exec of onLoad with script eval (since it's IE) and must manually detect here
			targetModule = targetModule || injectingModule;
			if(!targetModule){
				for(mid in waiting){
					module = modules[mid];
					if(module && module.node && module.node.readyState === 'interactive'){
						targetModule = module;
						break;
					}
				}
				if(0 && !targetModule){
					for(var i = 0; i<combosPending.length; i++){
						targetModule = combosPending[i];
						if(targetModule.node && targetModule.node.readyState === 'interactive'){
							break;
						}
						targetModule= 0;
					}
				}
			}
			if(0 && isArray(targetModule)){
				injectDependencies(defineModule(getModule(targetModule.shift()), args[1], args[2]));
				if(!targetModule.length){
					combosPending.splice(i, 1);
				}
			}else if(targetModule){
				consumePendingCacheInsert(targetModule);
				injectDependencies(defineModule(targetModule, args[1], args[2]));
			}else{
				signal(error, makeError("ieDefineFailed", args[0]));
			}
			checkComplete();
		}
	};
	def.amd = {
		vendor:"dojotoolkit.org"
	};

	if(0){
		req.def = def;
	}

	// allow config to override default implemention of named functions; this is useful for
	// non-browser environments, e.g., overriding injectUrl, getText, log, etc. in node.js, Rhino, etc.
	// also useful for testing and monkey patching loader
	mix(mix(req, defaultConfig.loaderPatch), userConfig.loaderPatch);

	// now that req is fully initialized and won't change, we can hook it up to the error signal
	on(error, function(arg){
		try{
			console.error(arg);
			if(arg instanceof Error){
				for(var p in arg){
					console.log(p + ":", arg[p]);
				}
				console.log(".");
			}
		}catch(e){}
	});

	// always publish these
	mix(req, {
		uid:uid,
		cache:cache,
		packs:packs
	});


	if(0){
		mix(req, {
			// these may be interesting to look at when debugging
			paths:paths,
			aliases:aliases,
			packageMap:packageMap,
			modules:modules,
			legacyMode:legacyMode,
			execQ:execQ,
			defQ:defQ,
			waiting:waiting,

			// these are used for testing
			// TODO: move testing infrastructure to a different has feature
			pathsMapProg:pathsMapProg,
			packageMapProg:packageMapProg,
			listenerQueues:listenerQueues,

			// these are used by the builder (at least)
			computeMapProg:computeMapProg,
			runMapProg:runMapProg,
			compactPath:compactPath,
			getModuleInfo:getModuleInfo_
		});
	}

	// the loader can be defined exactly once; look for global define which is the symbol AMD loaders are
	// *required* to define (as opposed to require, which is optional)
	if(global.define){
		if(1){
			signal(error, makeError("defineAlreadyDefined", 0));
		}
	}else{
		global.define = def;
		global.require = req;
	}

	if(0 && req.combo && req.combo.plugins){
		var plugins = req.combo.plugins,
			pluginName;
		for(pluginName in plugins){
			mix(mix(getModule(pluginName), plugins[pluginName]), {isCombo:1, executed:"executed", load:1});
		}
	}

	if(1){
		var bootDeps = dojoSniffConfig.deps ||  userConfig.deps || defaultConfig.deps,
			bootCallback = dojoSniffConfig.callback || userConfig.callback || defaultConfig.callback;
		req.boot = (bootDeps || bootCallback) ? [bootDeps || [], bootCallback] : 0;
	}
	if(!1){
		!req.async && req(["dojo"]);
		req.boot && req.apply(null, req.boot);
	}
})
(this.dojoConfig || this.djConfig || this.require || {}, {
		async:0,
		hasCache:{
				'config-selectorEngine':"acme",
				'config-tlmSiblingOfDojo':1,
				'dojo-built':1,
				'dojo-loader':1,
				dom:1,
				'host-browser':1
		},
		packages:[
				{
					 location:"../../../com/ibm/bpm/coach",
					 name:"com.ibm.bpm.coach"
				},
				{
					 location:"../dojox",
					 name:"dojox"
				},
				{
					 location:"../dijit",
					 name:"dijit"
				},
				{
					 location:"../../../verspec",
					 name:"verspec"
				},
				{
					 location:".",
					 name:"dojo"
				}
		]
});require({cache:{
'dojo/dom-geometry':function(){
define(["./_base/sniff", "./_base/window","./dom", "./dom-style"],
		function(has, win, dom, style){
	// module:
	//		dojo/dom-geometry
	// summary:
	//		This module defines the core dojo DOM geometry API.

	var geom = {};  // the result object

	// Box functions will assume this model.
	// On IE/Opera, BORDER_BOX will be set if the primary document is in quirks mode.
	// Can be set to change behavior of box setters.

	// can be either:
	//	"border-box"
	//	"content-box" (default)
	geom.boxModel = "content-box";

	// We punt per-node box mode testing completely.
	// If anybody cares, we can provide an additional (optional) unit
	// that overrides existing code to include per-node box sensitivity.

	// Opera documentation claims that Opera 9 uses border-box in BackCompat mode.
	// but experiments (Opera 9.10.8679 on Windows Vista) indicate that it actually continues to use content-box.
	// IIRC, earlier versions of Opera did in fact use border-box.
	// Opera guys, this is really confusing. Opera being broken in quirks mode is not our fault.

		if(has("ie") /*|| has("opera")*/){
		// client code may have to adjust if compatMode varies across iframes
		geom.boxModel = document.compatMode == "BackCompat" ? "border-box" : "content-box";
	}
	
	// =============================
	// Box Functions
	// =============================

	/*=====
	dojo.getPadExtents = function(node, computedStyle){
		// summary:
		//		Returns object with special values specifically useful for node
		//		fitting.
		// description:
		//		Returns an object with `w`, `h`, `l`, `t` properties:
		//	|		l/t/r/b = left/top/right/bottom padding (respectively)
		//	|		w = the total of the left and right padding
		//	|		h = the total of the top and bottom padding
		//		If 'node' has position, l/t forms the origin for child nodes.
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.


	};
	=====*/

	/*=====
	dojo._getPadExtents = function(node, computedStyle){
		// summary:
		//		Existing alias for `dojo.getPadExtents`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.getBorderExtents = function(node, computedStyle){
		// summary:
		//		returns an object with properties useful for noting the border
		//		dimensions.
		// description:
		//		* l/t/r/b = the sum of left/top/right/bottom border (respectively)
		//		* w = the sum of the left and right border
		//		* h = the sum of the top and bottom border
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.


	};
	=====*/

	/*=====
	dojo._getBorderExtents = function(node, computedStyle){
		// summary:
		//		Existing alias for `dojo.getBorderExtents`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.getPadBorderExtents = function(node, computedStyle){
		// summary:
		//		Returns object with properties useful for box fitting with
		//		regards to padding.
		// description:
		//		* l/t/r/b = the sum of left/top/right/bottom padding and left/top/right/bottom border (respectively)
		//		* w = the sum of the left and right padding and border
		//		* h = the sum of the top and bottom padding and border
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.


	};
	=====*/

	/*=====
	dojo._getPadBorderExtents = function(node, computedStyle){
		// summary:
		//		Existing alias for `dojo.getPadBorderExtents`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.getMarginExtents = function(node, computedStyle){
		// summary:
		//		returns object with properties useful for box fitting with
		//		regards to box margins (i.e., the outer-box).
		//
		//		* l/t = marginLeft, marginTop, respectively
		//		* w = total width, margin inclusive
		//		* h = total height, margin inclusive
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.
	};
	=====*/

	/*=====
	dojo._getMarginExtents = function(node, computedStyle){
		// summary:
		//		Existing alias for `dojo.getMarginExtents`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.getMarginSize = function(node, computedStyle){
		// summary:
		//		returns an object that encodes the width and height of
		//		the node's margin box
		// node: DOMNode|String
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.
	};
	=====*/

	/*=====
	dojo._getMarginSize = function(node, computedStyle){
		// summary:
		//		Existing alias for `dojo.getMarginSize`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.getMarginBox = function(node, computedStyle){
		// summary:
		//		returns an object that encodes the width, height, left and top
		//		positions of the node's margin box.
		// node: DOMNode
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.
	};
	=====*/

	/*=====
	dojo._getMarginBox = function(node, computedStyle){
		// summary:
		//		Existing alias for `dojo.getMarginBox`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.setMarginBox = function(node, box, computedStyle){
		// summary:
		//		sets the size of the node's margin box and placement
		//		(left/top), irrespective of box model. Think of it as a
		//		passthrough to setBox that handles box-model vagaries for
		//		you.
		// node: DOMNode
		// box: Object
		//      hash with optional "l", "t", "w", and "h" properties for "left", "right", "width", and "height"
		//      respectively. All specified properties should have numeric values in whole pixels.
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.
	};
	=====*/

	/*=====
	dojo.getContentBox = function(node, computedStyle){
		// summary:
		//		Returns an object that encodes the width, height, left and top
		//		positions of the node's content box, irrespective of the
		//		current box model.
		// node: DOMNode
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.
	};
	=====*/

	/*=====
	dojo._getContentBox = function(node, computedStyle){
		// summary:
		//		Existing alias for `dojo.getContentBox`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.setContentSize = function(node, box, computedStyle){
		// summary:
		//		Sets the size of the node's contents, irrespective of margins,
		//		padding, or borders.
		// node: DOMNode
		// box: Object
		//      hash with optional "w", and "h" properties for "width", and "height"
		//      respectively. All specified properties should have numeric values in whole pixels.
		// computedStyle: Object?
		// 		This parameter accepts computed styles object.
		// 		If this parameter is omitted, the functions will call 
		//		dojo.getComputedStyle to get one. It is a better way, calling 
		//		dojo.computedStyle once, and then pass the reference to this 
		//		computedStyle parameter. Wherever possible, reuse the returned 
		//		object of dojo.getComputedStyle.
	};
	=====*/

	/*=====
	dojo.isBodyLtr = function(){
		// summary:
		//      Returns true if the current language is left-to-right, and false otherwise.
		// returns: Boolean
	};
	=====*/

	/*=====
	dojo._isBodyLtr = function(){
		// summary:
		//		Existing alias for `dojo.isBodyLtr`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.docScroll = function(){
		// summary:
		//      Returns an object with {node, x, y} with corresponding offsets.
		// returns: Object
	};
	=====*/

	/*=====
	dojo._docScroll = function(){
		// summary:
		//		Existing alias for `dojo.docScroll`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.getIeDocumentElementOffset = function(){
		// summary:
		//		returns the offset in x and y from the document body to the
		//		visual edge of the page for IE
		// description:
		//		The following values in IE contain an offset:
		//	|		event.clientX
		//	|		event.clientY
		//	|		node.getBoundingClientRect().left
		//	|		node.getBoundingClientRect().top
		//		But other position related values do not contain this offset,
		//		such as node.offsetLeft, node.offsetTop, node.style.left and
		//		node.style.top. The offset is always (2, 2) in LTR direction.
		//		When the body is in RTL direction, the offset counts the width
		//		of left scroll bar's width.  This function computes the actual
		//		offset.
	};
	=====*/

	/*=====
	dojo._getIeDocumentElementOffset = function(){
		// summary:
		//		Existing alias for `dojo.getIeDocumentElementOffset`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.fixIeBiDiScrollLeft = function(scrollLeft){
		// summary:
		//      In RTL direction, scrollLeft should be a negative value, but IE
		//      returns a positive one. All codes using documentElement.scrollLeft
		//      must call this function to fix this error, otherwise the position
		//      will offset to right when there is a horizontal scrollbar.
		// scrollLeft: NUmber
		// returns: Number
	};
	=====*/

	/*=====
	dojo._fixIeBiDiScrollLeft = function(scrollLeft){
		// summary:
		//		Existing alias for `dojo.fixIeBiDiScrollLeft`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.position = function(node, includeScroll){
		// summary:
		//		Gets the position and size of the passed element relative to
		//		the viewport (if includeScroll==false), or relative to the
		//		document root (if includeScroll==true).
		//
		// description:
		//		Returns an object of the form:
		//			{ x: 100, y: 300, w: 20, h: 15 }
		//		If includeScroll==true, the x and y values will include any
		//		document offsets that may affect the position relative to the
		//		viewport.
		//		Uses the border-box model (inclusive of border and padding but
		//		not margin).  Does not act as a setter.
		// node: DOMNode|String
		// includeScroll: Boolean?
		// returns: Object
	};
	=====*/

	geom.getPadExtents = function getPadExtents(/*DomNode*/node, /*Object*/computedStyle){
		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), px = style.toPixelValue,
			l = px(node, s.paddingLeft), t = px(node, s.paddingTop), r = px(node, s.paddingRight), b = px(node, s.paddingBottom);
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	var none = "none";

	geom.getBorderExtents = function getBorderExtents(/*DomNode*/node, /*Object*/computedStyle){
		node = dom.byId(node);
		var px = style.toPixelValue, s = computedStyle || style.getComputedStyle(node),
			l = s.borderLeftStyle != none ? px(node, s.borderLeftWidth) : 0,
			t = s.borderTopStyle != none ? px(node, s.borderTopWidth) : 0,
			r = s.borderRightStyle != none ? px(node, s.borderRightWidth) : 0,
			b = s.borderBottomStyle != none ? px(node, s.borderBottomWidth) : 0;
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	geom.getPadBorderExtents = function getPadBorderExtents(/*DomNode*/node, /*Object*/computedStyle){
		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node),
			p = geom.getPadExtents(node, s),
			b = geom.getBorderExtents(node, s);
		return {
			l: p.l + b.l,
			t: p.t + b.t,
			r: p.r + b.r,
			b: p.b + b.b,
			w: p.w + b.w,
			h: p.h + b.h
		};
	};

	geom.getMarginExtents = function getMarginExtents(node, computedStyle){
		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), px = style.toPixelValue,
			l = px(node, s.marginLeft), t = px(node, s.marginTop), r = px(node, s.marginRight), b = px(node, s.marginBottom);
		if(has("webkit") && (s.position != "absolute")){
			// FIXME: Safari's version of the computed right margin
			// is the space between our right edge and the right edge
			// of our offsetParent.
			// What we are looking for is the actual margin value as
			// determined by CSS.
			// Hack solution is to assume left/right margins are the same.
			r = l;
		}
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	// Box getters work in any box context because offsetWidth/clientWidth
	// are invariant wrt box context
	//
	// They do *not* work for display: inline objects that have padding styles
	// because the user agent ignores padding (it's bogus styling in any case)
	//
	// Be careful with IMGs because they are inline or block depending on
	// browser and browser mode.

	// Although it would be easier to read, there are not separate versions of
	// _getMarginBox for each browser because:
	// 1. the branching is not expensive
	// 2. factoring the shared code wastes cycles (function call overhead)
	// 3. duplicating the shared code wastes bytes

	geom.getMarginBox = function getMarginBox(/*DomNode*/node, /*Object*/computedStyle){
		// summary:
		//		returns an object that encodes the width, height, left and top
		//		positions of the node's margin box.
		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), me = geom.getMarginExtents(node, s),
			l = node.offsetLeft - me.l, t = node.offsetTop - me.t, p = node.parentNode, px = style.toPixelValue, pcs;
				if(has("mozilla")){
			// Mozilla:
			// If offsetParent has a computed overflow != visible, the offsetLeft is decreased
			// by the parent's border.
			// We don't want to compute the parent's style, so instead we examine node's
			// computed left/top which is more stable.
			var sl = parseFloat(s.left), st = parseFloat(s.top);
			if(!isNaN(sl) && !isNaN(st)){
				l = sl, t = st;
			}else{
				// If child's computed left/top are not parseable as a number (e.g. "auto"), we
				// have no choice but to examine the parent's computed style.
				if(p && p.style){
					pcs = style.getComputedStyle(p);
					if(pcs.overflow != "visible"){
						l += pcs.borderLeftStyle != none ? px(node, pcs.borderLeftWidth) : 0;
						t += pcs.borderTopStyle != none ? px(node, pcs.borderTopWidth) : 0;
					}
				}
			}
		}else if(has("opera") || (has("ie") == 8 && !has("quirks"))){
			// On Opera and IE 8, offsetLeft/Top includes the parent's border
			if(p){
				pcs = style.getComputedStyle(p);
				l -= pcs.borderLeftStyle != none ? px(node, pcs.borderLeftWidth) : 0;
				t -= pcs.borderTopStyle != none ? px(node, pcs.borderTopWidth) : 0;
			}
		}
				return {l: l, t: t, w: node.offsetWidth + me.w, h: node.offsetHeight + me.h};
	};

	geom.getContentBox = function getContentBox(node, computedStyle){
		// clientWidth/Height are important since the automatically account for scrollbars
		// fallback to offsetWidth/Height for special cases (see #3378)
		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), w = node.clientWidth, h,
			pe = geom.getPadExtents(node, s), be = geom.getBorderExtents(node, s);
		if(!w){
			w = node.offsetWidth;
			h = node.offsetHeight;
		}else{
			h = node.clientHeight;
			be.w = be.h = 0;
		}
		// On Opera, offsetLeft includes the parent's border
				if(has("opera")){
			pe.l += be.l;
			pe.t += be.t;
		}
				return {l: pe.l, t: pe.t, w: w - pe.w - be.w, h: h - pe.h - be.h};
	};

	// Box setters depend on box context because interpretation of width/height styles
	// vary wrt box context.
	//
	// The value of dojo.boxModel is used to determine box context.
	// dojo.boxModel can be set directly to change behavior.
	//
	// Beware of display: inline objects that have padding styles
	// because the user agent ignores padding (it's a bogus setup anyway)
	//
	// Be careful with IMGs because they are inline or block depending on
	// browser and browser mode.
	//
	// Elements other than DIV may have special quirks, like built-in
	// margins or padding, or values not detectable via computedStyle.
	// In particular, margins on TABLE do not seems to appear
	// at all in computedStyle on Mozilla.

	function setBox(/*DomNode*/node, /*Number?*/l, /*Number?*/t, /*Number?*/w, /*Number?*/h, /*String?*/u){
		// summary:
		//		sets width/height/left/top in the current (native) box-model
		//		dimensions. Uses the unit passed in u.
		// node:
		//		DOM Node reference. Id string not supported for performance
		//		reasons.
		// l:
		//		left offset from parent.
		// t:
		//		top offset from parent.
		// w:
		//		width in current box model.
		// h:
		//		width in current box model.
		// u:
		//		unit measure to use for other measures. Defaults to "px".
		u = u || "px";
		var s = node.style;
		if(!isNaN(l)){
			s.left = l + u;
		}
		if(!isNaN(t)){
			s.top = t + u;
		}
		if(w >= 0){
			s.width = w + u;
		}
		if(h >= 0){
			s.height = h + u;
		}
	}

	function isButtonTag(/*DomNode*/node){
		// summary:
		//		True if the node is BUTTON or INPUT.type="button".
		return node.tagName.toLowerCase() == "button" ||
			node.tagName.toLowerCase() == "input" && (node.getAttribute("type") || "").toLowerCase() == "button"; // boolean
	}

	function usesBorderBox(/*DomNode*/node){
		// summary:
		//		True if the node uses border-box layout.

		// We could test the computed style of node to see if a particular box
		// has been specified, but there are details and we choose not to bother.

		// TABLE and BUTTON (and INPUT type=button) are always border-box by default.
		// If you have assigned a different box to either one via CSS then
		// box functions will break.

		return geom.boxModel == "border-box" || node.tagName.toLowerCase() == "table" || isButtonTag(node); // boolean
	}

	geom.setContentSize = function setContentSize(/*DomNode*/node, /*Object*/box, /*Object*/computedStyle){
		// summary:
		//		Sets the size of the node's contents, irrespective of margins,
		//		padding, or borders.

		node = dom.byId(node);
		var w = box.w, h = box.h;
		if(usesBorderBox(node)){
			var pb = geom.getPadBorderExtents(node, computedStyle);
			if(w >= 0){
				w += pb.w;
			}
			if(h >= 0){
				h += pb.h;
			}
		}
		setBox(node, NaN, NaN, w, h);
	};

	var nilExtents = {l: 0, t: 0, w: 0, h: 0};

	geom.setMarginBox = function setMarginBox(/*DomNode*/node, /*Object*/box, /*Object*/computedStyle){
		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), w = box.w, h = box.h,
		// Some elements have special padding, margin, and box-model settings.
		// To use box functions you may need to set padding, margin explicitly.
		// Controlling box-model is harder, in a pinch you might set dojo.boxModel.
			pb = usesBorderBox(node) ? nilExtents : geom.getPadBorderExtents(node, s),
			mb = geom.getMarginExtents(node, s);
		if(has("webkit")){
			// on Safari (3.1.2), button nodes with no explicit size have a default margin
			// setting an explicit size eliminates the margin.
			// We have to swizzle the width to get correct margin reading.
			if(isButtonTag(node)){
				var ns = node.style;
				if(w >= 0 && !ns.width){
					ns.width = "4px";
				}
				if(h >= 0 && !ns.height){
					ns.height = "4px";
				}
			}
		}
		if(w >= 0){
			w = Math.max(w - pb.w - mb.w, 0);
		}
		if(h >= 0){
			h = Math.max(h - pb.h - mb.h, 0);
		}
		setBox(node, box.l, box.t, w, h);
	};

	// =============================
	// Positioning
	// =============================

	geom.isBodyLtr = function isBodyLtr(){
		return (win.body().dir || win.doc.documentElement.dir || "ltr").toLowerCase() == "ltr"; // Boolean
	};

	geom.docScroll = function docScroll(){
		var node = win.doc.parentWindow || win.doc.defaultView;   // use UI window, not dojo.global window
		return "pageXOffset" in node ? {x: node.pageXOffset, y: node.pageYOffset } :
			(node = has("quirks") ? win.body() : win.doc.documentElement,
				{x: geom.fixIeBiDiScrollLeft(node.scrollLeft || 0), y: node.scrollTop || 0 });
	};

		geom.getIeDocumentElementOffset = function getIeDocumentElementOffset(){
		//NOTE: assumes we're being called in an IE browser

		var de = win.doc.documentElement; // only deal with HTML element here, position() handles body/quirks

		if(has("ie") < 8){
			var r = de.getBoundingClientRect(), // works well for IE6+
				l = r.left, t = r.top;
			if(has("ie") < 7){
				l += de.clientLeft;	// scrollbar size in strict/RTL, or,
				t += de.clientTop;	// HTML border size in strict
			}
			return {
				x: l < 0 ? 0 : l, // FRAME element border size can lead to inaccurate negative values
				y: t < 0 ? 0 : t
			};
		}else{
			return {
				x: 0,
				y: 0
			};
		}
	};
	
	geom.fixIeBiDiScrollLeft = function fixIeBiDiScrollLeft(/*Integer*/ scrollLeft){
		// In RTL direction, scrollLeft should be a negative value, but IE
		// returns a positive one. All codes using documentElement.scrollLeft
		// must call this function to fix this error, otherwise the position
		// will offset to right when there is a horizontal scrollbar.

				var ie = has("ie");
		if(ie && !geom.isBodyLtr()){
			var qk = has("quirks"),
				de = qk ? win.body() : win.doc.documentElement;
			if(ie == 6 && !qk && win.global.frameElement && de.scrollHeight > de.clientHeight){
				scrollLeft += de.clientLeft; // workaround ie6+strict+rtl+iframe+vertical-scrollbar bug where clientWidth is too small by clientLeft pixels
			}
			return (ie < 8 || qk) ? (scrollLeft + de.clientWidth - de.scrollWidth) : -scrollLeft; // Integer
		}
				return scrollLeft; // Integer
	};

	geom.position = function(/*DomNode*/node, /*Boolean?*/includeScroll){
		node = dom.byId(node);
		var	db = win.body(),
			dh = db.parentNode,
			ret = node.getBoundingClientRect();
		ret = {x: ret.left, y: ret.top, w: ret.right - ret.left, h: ret.bottom - ret.top};
				if(has("ie") < 9){
			// On IE<9 there's a 2px offset that we need to adjust for, see dojo.getIeDocumentElementOffset()
			var offset = geom.getIeDocumentElementOffset();

			// fixes the position in IE, quirks mode
			ret.x -= offset.x + (has("quirks") ? db.clientLeft + db.offsetLeft : 0);
			ret.y -= offset.y + (has("quirks") ? db.clientTop + db.offsetTop : 0);
		}else if(has("ff") == 3){
			// In FF3 you have to subtract the document element margins.
			// Fixed in FF3.5 though.
			var cs = style.getComputedStyle(dh), px = style.toPixelValue;
			ret.x -= px(dh, cs.marginLeft) + px(dh, cs.borderLeftWidth);
			ret.y -= px(dh, cs.marginTop) + px(dh, cs.borderTopWidth);
		}
				// account for document scrolling
		// if offsetParent is used, ret value already includes scroll position
		// so we may have to actually remove that value if !includeScroll
		if(includeScroll){
			var scroll = geom.docScroll();
			ret.x += scroll.x;
			ret.y += scroll.y;
		}

		return ret; // Object
	};

	// random "private" functions wildly used throughout the toolkit

	geom.getMarginSize = function getMarginSize(/*DomNode*/node, /*Object*/computedStyle){
		node = dom.byId(node);
		var me = geom.getMarginExtents(node, computedStyle || style.getComputedStyle(node));
		var size = node.getBoundingClientRect();
		return {
			w: (size.right - size.left) + me.w,
			h: (size.bottom - size.top) + me.h
		}
	};

	geom.normalizeEvent = function(event){
		// summary:
		// 		Normalizes the geometry of a DOM event, normalizing the pageX, pageY,
		// 		offsetX, offsetY, layerX, and layerX properties
		// event: Object
		if(!("layerX" in event)){
			event.layerX = event.offsetX;
			event.layerY = event.offsetY;
		}
		if(!has("dom-addeventlistener")){
			// old IE version
			// FIXME: scroll position query is duped from dojo.html to
			// avoid dependency on that entire module. Now that HTML is in
			// Base, we should convert back to something similar there.
			var se = event.target;
			var doc = (se && se.ownerDocument) || document;
			// DO NOT replace the following to use dojo.body(), in IE, document.documentElement should be used
			// here rather than document.body
			var docBody = has("quirks") ? doc.body : doc.documentElement;
			var offset = geom.getIeDocumentElementOffset();
			event.pageX = event.clientX + geom.fixIeBiDiScrollLeft(docBody.scrollLeft || 0) - offset.x;
			event.pageY = event.clientY + (docBody.scrollTop || 0) - offset.y;
		}
	};

	// TODO: evaluate separate getters/setters for position and sizes?

	return geom;
});

},
'dojo/_base/html':function(){
define(["./kernel", "../dom", "../dom-style", "../dom-attr", "../dom-prop", "../dom-class", "../dom-construct", "../dom-geometry"], function(dojo, dom, style, attr, prop, cls, ctr, geom){
	// module:
	//		dojo/dom
	// summary:
	//		This module is a stub for the core dojo DOM API.

	// mix-in dom
	dojo.byId = dom.byId;
	dojo.isDescendant = dom.isDescendant;
	dojo.setSelectable = dom.setSelectable;

	// mix-in dom-attr
	dojo.getAttr = attr.get;
	dojo.setAttr = attr.set;
	dojo.hasAttr = attr.has;
	dojo.removeAttr = attr.remove;
	dojo.getNodeProp = attr.getNodeProp;

	dojo.attr = function(node, name, value){
		// summary:
		//		Gets or sets an attribute on an HTML element.
		// description:
		//		Handles normalized getting and setting of attributes on DOM
		//		Nodes. If 2 arguments are passed, and a the second argument is a
		//		string, acts as a getter.
		//
		//		If a third argument is passed, or if the second argument is a
		//		map of attributes, acts as a setter.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to get or set the attribute on
		// name: String|Object
		//		the name of the attribute to get or set.
		// value: String?
		//		The value to set for the attribute
		// returns:
		//		when used as a getter, the value of the requested attribute
		//		or null if that attribute does not have a specified or
		//		default value;
		//
		//		when used as a setter, the DOM node
		//
		// example:
		//	|	// get the current value of the "foo" attribute on a node
		//	|	dojo.attr(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.attr("nodeId", "foo");
		//
		// example:
		//	|	// use attr() to set the tab index
		//	|	dojo.attr("nodeId", "tabIndex", 3);
		//	|
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.attr("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.attr("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.attr("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);

		if(arguments.length == 2){
			return attr[typeof name == "string" ? "get" : "set"](node, name);
		}
		return attr.set(node, name, value);
	};

	// mix-in dom-class
	dojo.hasClass = cls.contains;
	dojo.addClass = cls.add;
	dojo.removeClass = cls.remove;
	dojo.toggleClass = cls.toggle;
	dojo.replaceClass = cls.replace;

	// mix-in dom-construct
	dojo._toDom = dojo.toDom = ctr.toDom;
	dojo.place = ctr.place;
	dojo.create = ctr.create;
	dojo.empty = function(node){ ctr.empty(node); };
	dojo._destroyElement = dojo.destroy = function(node){ ctr.destroy(node); };

	// mix-in dom-geometry
	dojo._getPadExtents = dojo.getPadExtents = geom.getPadExtents;
	dojo._getBorderExtents = dojo.getBorderExtents = geom.getBorderExtents;
	dojo._getPadBorderExtents = dojo.getPadBorderExtents = geom.getPadBorderExtents;
	dojo._getMarginExtents = dojo.getMarginExtents = geom.getMarginExtents;
	dojo._getMarginSize = dojo.getMarginSize = geom.getMarginSize;
	dojo._getMarginBox = dojo.getMarginBox = geom.getMarginBox;
	dojo.setMarginBox = geom.setMarginBox;
	dojo._getContentBox = dojo.getContentBox = geom.getContentBox;
	dojo.setContentSize = geom.setContentSize;
	dojo._isBodyLtr = dojo.isBodyLtr = geom.isBodyLtr;
	dojo._docScroll = dojo.docScroll = geom.docScroll;
	dojo._getIeDocumentElementOffset = dojo.getIeDocumentElementOffset = geom.getIeDocumentElementOffset;
	dojo._fixIeBiDiScrollLeft = dojo.fixIeBiDiScrollLeft = geom.fixIeBiDiScrollLeft;
	dojo.position = geom.position;

	dojo.marginBox = function marginBox(/*DomNode|String*/node, /*Object?*/box){
		// summary:
		//		Getter/setter for the margin-box of node.
		// description:
		//		Getter/setter for the margin-box of node.
		//		Returns an object in the expected format of box (regardless
		//		if box is passed). The object might look like:
		//			`{ l: 50, t: 200, w: 300: h: 150 }`
		//		for a node offset from its parent 50px to the left, 200px from
		//		the top with a margin width of 300px and a margin-height of
		//		150px.
		// node:
		//		id or reference to DOM Node to get/set box for
		// box:
		//		If passed, denotes that dojo.marginBox() should
		//		update/set the margin box for node. Box is an object in the
		//		above format. All properties are optional if passed.
		// example:
		//		Retrieve the margin box of a passed node
		//	|	var box = dojo.marginBox("someNodeId");
		//	|	console.dir(box);
		//
		// example:
		//		Set a node's margin box to the size of another node
		//	|	var box = dojo.marginBox("someNodeId");
		//	|	dojo.marginBox("someOtherNode", box);
		return box ? geom.setMarginBox(node, box) : geom.getMarginBox(node); // Object
	};

	dojo.contentBox = function contentBox(/*DomNode|String*/node, /*Object?*/box){
		// summary:
		//		Getter/setter for the content-box of node.
		// description:
		//		Returns an object in the expected format of box (regardless if box is passed).
		//		The object might look like:
		//			`{ l: 50, t: 200, w: 300: h: 150 }`
		//		for a node offset from its parent 50px to the left, 200px from
		//		the top with a content width of 300px and a content-height of
		//		150px. Note that the content box may have a much larger border
		//		or margin box, depending on the box model currently in use and
		//		CSS values set/inherited for node.
		//		While the getter will return top and left values, the
		//		setter only accepts setting the width and height.
		// node:
		//		id or reference to DOM Node to get/set box for
		// box:
		//		If passed, denotes that dojo.contentBox() should
		//		update/set the content box for node. Box is an object in the
		//		above format, but only w (width) and h (height) are supported.
		//		All properties are optional if passed.
		return box ? geom.setContentSize(node, box) : geom.getContentBox(node); // Object
	};

	dojo.coords = function(/*DomNode|String*/node, /*Boolean?*/includeScroll){
		// summary:
		//		Deprecated: Use position() for border-box x/y/w/h
		//		or marginBox() for margin-box w/h/l/t.
		//		Returns an object representing a node's size and position.
		//
		// description:
		//		Returns an object that measures margin-box (w)idth/(h)eight
		//		and absolute position x/y of the border-box. Also returned
		//		is computed (l)eft and (t)op values in pixels from the
		//		node's offsetParent as returned from marginBox().
		//		Return value will be in the form:
		//|			{ l: 50, t: 200, w: 300: h: 150, x: 100, y: 300 }
		//		Does not act as a setter. If includeScroll is passed, the x and
		//		y params are affected as one would expect in dojo.position().
		dojo.deprecated("dojo.coords()", "Use dojo.position() or dojo.marginBox().");
		node = dom.byId(node);
		var s = style.getComputedStyle(node), mb = geom.getMarginBox(node, s);
		var abs = geom.position(node, includeScroll);
		mb.x = abs.x;
		mb.y = abs.y;
		return mb;	// Object
	};

	// mix-in dom-prop
	dojo.getProp = prop.get;
	dojo.setProp = prop.set;

	dojo.prop = function(/*DomNode|String*/node, /*String|Object*/name, /*String?*/value){
		// summary:
		//		Gets or sets a property on an HTML element.
		// description:
		//		Handles normalized getting and setting of properties on DOM
		//		Nodes. If 2 arguments are passed, and a the second argument is a
		//		string, acts as a getter.
		//
		//		If a third argument is passed, or if the second argument is a
		//		map of attributes, acts as a setter.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node:
		//		id or reference to the element to get or set the property on
		// name:
		//		the name of the property to get or set.
		// value:
		//		The value to set for the property
		// returns:
		//		when used as a getter, the value of the requested property
		//		or null if that attribute does not have a specified or
		//		default value;
		//
		//		when used as a setter, the DOM node
		//
		// example:
		//	|	// get the current value of the "foo" property on a node
		//	|	dojo.prop(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.prop("nodeId", "foo");
		//
		// example:
		//	|	// use prop() to set the tab index
		//	|	dojo.prop("nodeId", "tabIndex", 3);
		//	|
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.prop("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.prop("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.prop("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);

		if(arguments.length == 2){
			return prop[typeof name == "string" ? "get" : "set"](node, name);
		}
		// setter
		return prop.set(node, name, value);
	};

	// mix-in dom-style
	dojo.getStyle = style.get;
	dojo.setStyle = style.set;
	dojo.getComputedStyle = style.getComputedStyle;
	dojo.__toPixelValue = dojo.toPixelValue = style.toPixelValue;

	dojo.style = function(node, name, value){
		// summary:
		//		Accesses styles on a node. If 2 arguments are
		//		passed, acts as a getter. If 3 arguments are passed, acts
		//		as a setter.
		// description:
		//		Getting the style value uses the computed style for the node, so the value
		//		will be a calculated value, not just the immediate node.style value.
		//		Also when getting values, use specific style names,
		//		like "borderBottomWidth" instead of "border" since compound values like
		//		"border" are not necessarily reflected as expected.
		//		If you want to get node dimensions, use `dojo.marginBox()`,
		//		`dojo.contentBox()` or `dojo.position()`.
		// node: DOMNode|String
		//		id or reference to node to get/set style for
		// name: String?|Object?
		//		the style property to set in DOM-accessor format
		//		("borderWidth", not "border-width") or an object with key/value
		//		pairs suitable for setting each property.
		// value: String?
		//		If passed, sets value on the node for style, handling
		//		cross-browser concerns.  When setting a pixel value,
		//		be sure to include "px" in the value. For instance, top: "200px".
		//		Otherwise, in some cases, some browsers will not apply the style.
		// returns:
		//		when used as a getter, return the computed style of the node if passing in an ID or node,
		//		or return the normalized, computed value for the property when passing in a node and a style property
		// example:
		//		Passing only an ID or node returns the computed style object of
		//		the node:
		//	|	dojo.style("thinger");
		// example:
		//		Passing a node and a style property returns the current
		//		normalized, computed value for that property:
		//	|	dojo.style("thinger", "opacity"); // 1 by default
		//
		// example:
		//		Passing a node, a style property, and a value changes the
		//		current display of the node and returns the new computed value
		//	|	dojo.style("thinger", "opacity", 0.5); // == 0.5
		//
		// example:
		//		Passing a node, an object-style style property sets each of the values in turn and returns the computed style object of the node:
		//	|	dojo.style("thinger", {
		//	|		"opacity": 0.5,
		//	|		"border": "3px solid black",
		//	|		"height": "300px"
		//	|	});
		//
		// example:
		//		When the CSS style property is hyphenated, the JavaScript property is camelCased.
		//		font-size becomes fontSize, and so on.
		//	|	dojo.style("thinger",{
		//	|		fontSize:"14pt",
		//	|		letterSpacing:"1.2em"
		//	|	});
		//
		// example:
		//		dojo.NodeList implements .style() using the same syntax, omitting the "node" parameter, calling
		//		dojo.style() on every element of the list. See: `dojo.query()` and `dojo.NodeList()`
		//	|	dojo.query(".someClassName").style("visibility","hidden");
		//	|	// or
		//	|	dojo.query("#baz > div").style({
		//	|		opacity:0.75,
		//	|		fontSize:"13pt"
		//	|	});

		switch(arguments.length){
			case 1:
				return style.get(node);
			case 2:
				return style[typeof name == "string" ? "get" : "set"](node, name);
		}
		// setter
		return style.set(node, name, value);
	};

	return dojo;
});

},
'url:com.ibm.bpm.coach/templates/CollaborationBanner.html':"<div class='collaboration-banner'>\n\t<table width=\"100%\">\n\t\t<tbody>\n\t\t\t<tr valign=\"center\" nowrap=\"nowrap\" height=\"30px\" width=\"100%\">\n\t\t\t\t<td nowrap=\"nowrap\">\n\t\t\t\t\t<span dojoattachpoint=\"title_area_icon\"></span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\">\t\t\n\t\t\t\t\t<span tabindex=\"0\" class='collaboration-title-area-icon-label' dojoattachpoint=\"title_area_label\"></span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\">\n\t\t\t\t\t<span role=\"wairole:alert\" tabindex=\"0\" dojoAttachPoint='participants'>${workingWithLabel} </span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\">\n\t\t\t\t\t<span class='collaboration-title-area-participants-avatar' dojoAttachPoint='partnerAvatar'>\t\t\t\t\t\n\t                   <div class=\"\">\t\t              \n\t\t                  <img class=\"collaboration-collaboratorImg\"  dojoattachpoint=\"partnerAvatarImage\" alt=\"User Avatar\" src=\"/portal/skins/Default/images/Avatar_16x16.png\">\t\t\n\t                   </div>\n                       <div class=\"collaboration-onlineAwarenessIcon\">\n                           <div class=\"collaboration-onlineAwarenessIconA11y\">&#9744</div>\n                       </div>  \n\t\t\t\t\t</span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\">\n\t\t\t\t\t<span role=\"wairole:alert\" tabindex=\"0\" class=\"collaboration-title-area-participants-fullName\" dojoattachpoint=\"partnerUserName\">${partnerFullName}</span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\" width=\"100%\">\n\t\t\t\t\t<div class=\"collaboration-controls collaboration-controls-link-style\" dojoattachpoint=\"controls\">\n\t\t\t\t\t\t<a tabindex=\"0\" id=\"makeEditorLink\" dojoattachpoint=\"makeEditorLink\" class=\"collaboration-controls-span collaboration-show\" href=\"javascript:\">${makeEditorLabel}</a>\n\t\t\t\t\t\t<a tabindex=\"0\" id=\"takeControlLink\" dojoattachpoint=\"takeControlLink\" class=\"collaboration-controls-span collaboration-hidden\" href=\"javascript:\">${takeControlLabel}</a>\n\t\t\t\t\t<div>\n\t\t\t\t</td>\n\t\t\t</tr>\n\t\t</tbody>\n\t</table>\n</div>",
'dojo/_base/array':function(){
define(["./kernel", "../has", "./lang"], function(dojo, has, lang){
	// module:
	//		dojo/_base/array
	// summary:
	//		This module defines the Javascript v1.6 array extensions.

	/*=====
	dojo.indexOf = function(arr, value, fromIndex, findLast){
		// summary:
		//		locates the first index of the provided value in the
		//		passed array. If the value is not found, -1 is returned.
		// description:
		//		This method corresponds to the JavaScript 1.6 Array.indexOf method, with one difference: when
		//		run over sparse arrays, the Dojo function invokes the callback for every index whereas JavaScript
		//		1.6's indexOf skips the holes in the sparse array.
		//		For details on this method, see:
		//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/indexOf
		// arr: Array
		// value: Object
		// fromIndex: Integer?
		// findLast: Boolean?
		// returns: Number
	};
	dojo.lastIndexOf = function(arr, value, fromIndex){
		// summary:
		//		locates the last index of the provided value in the passed
		//		array. If the value is not found, -1 is returned.
		// description:
		//		This method corresponds to the JavaScript 1.6 Array.lastIndexOf method, with one difference: when
		//		run over sparse arrays, the Dojo function invokes the callback for every index whereas JavaScript
		//		1.6's lastIndexOf skips the holes in the sparse array.
		//		For details on this method, see:
		//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/lastIndexOf
		//	arr: Array,
		//	value: Object,
		//	fromIndex: Integer?
		//	returns: Number
	};
	dojo.forEach = function(arr, callback, thisObject){
		//	summary:
		//		for every item in arr, callback is invoked. Return values are ignored.
		//		If you want to break out of the loop, consider using dojo.every() or dojo.some().
		//		forEach does not allow breaking out of the loop over the items in arr.
		//	arr:
		//		the array to iterate over. If a string, operates on individual characters.
		//	callback:
		//		a function is invoked with three arguments: item, index, and array
		//	thisObject:
		//		may be used to scope the call to callback
		//	description:
		//		This function corresponds to the JavaScript 1.6 Array.forEach() method, with one difference: when
		//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
		//		the callback function with a value of undefined. JavaScript 1.6's forEach skips the holes in the sparse array.
		//		For more details, see:
		//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/forEach
		//	example:
		//	| // log out all members of the array:
		//	| dojo.forEach(
		//	|		[ "thinger", "blah", "howdy", 10 ],
		//	|		function(item){
		//	|			console.log(item);
		//	|		}
		//	| );
		//	example:
		//	| // log out the members and their indexes
		//	| dojo.forEach(
		//	|		[ "thinger", "blah", "howdy", 10 ],
		//	|		function(item, idx, arr){
		//	|			console.log(item, "at index:", idx);
		//	|		}
		//	| );
		//	example:
		//	| // use a scoped object member as the callback
		//	|
		//	| var obj = {
		//	|		prefix: "logged via obj.callback:",
		//	|		callback: function(item){
		//	|			console.log(this.prefix, item);
		//	|		}
		//	| };
		//	|
		//	| // specifying the scope function executes the callback in that scope
		//	| dojo.forEach(
		//	|		[ "thinger", "blah", "howdy", 10 ],
		//	|		obj.callback,
		//	|		obj
		//	| );
		//	|
		//	| // alternately, we can accomplish the same thing with dojo.hitch()
		//	| dojo.forEach(
		//	|		[ "thinger", "blah", "howdy", 10 ],
		//	|		dojo.hitch(obj, "callback")
		//	| );
		//	arr: Array|String
		//	callback: Function|String
		//	thisObject: Object?
	};
	dojo.every = function(arr, callback, thisObject){
		// summary:
		//		Determines whether or not every item in arr satisfies the
		//		condition implemented by callback.
		// arr: Array|String
		//		the array to iterate on. If a string, operates on individual characters.
		// callback: Function|String
		//		a function is invoked with three arguments: item, index,
		//		and array and returns true if the condition is met.
		// thisObject: Object?
		//		may be used to scope the call to callback
		// returns: Boolean
		// description:
		//		This function corresponds to the JavaScript 1.6 Array.every() method, with one difference: when
		//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
		//		the callback function with a value of undefined. JavaScript 1.6's every skips the holes in the sparse array.
		//		For more details, see:
		//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/every
		// example:
		//	| // returns false
		//	| dojo.every([1, 2, 3, 4], function(item){ return item>1; });
		// example:
		//	| // returns true
		//	| dojo.every([1, 2, 3, 4], function(item){ return item>0; });
	};
	dojo.some = function(arr, callback, thisObject){
		// summary:
		//		Determines whether or not any item in arr satisfies the
		//		condition implemented by callback.
		// arr: Array|String
		//		the array to iterate over. If a string, operates on individual characters.
		// callback: Function|String
		//		a function is invoked with three arguments: item, index,
		//		and array and returns true if the condition is met.
		// thisObject: Object?
		//		may be used to scope the call to callback
		// returns: Boolean
		// description:
		//		This function corresponds to the JavaScript 1.6 Array.some() method, with one difference: when
		//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
		//		the callback function with a value of undefined. JavaScript 1.6's some skips the holes in the sparse array.
		//		For more details, see:
		//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/some
		// example:
		//	| // is true
		//	| dojo.some([1, 2, 3, 4], function(item){ return item>1; });
		// example:
		//	| // is false
		//	| dojo.some([1, 2, 3, 4], function(item){ return item<1; });
	};
	dojo.map = function(arr, callback, thisObject){
		// summary:
		//		applies callback to each element of arr and returns
		//		an Array with the results
		// arr: Array|String
		//		the array to iterate on. If a string, operates on
		//		individual characters.
		// callback: Function|String
		//		a function is invoked with three arguments, (item, index,
		//		array),	 and returns a value
		// thisObject: Object?
		//		may be used to scope the call to callback
		// returns: Array
		// description:
		//		This function corresponds to the JavaScript 1.6 Array.map() method, with one difference: when
		//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
		//		the callback function with a value of undefined. JavaScript 1.6's map skips the holes in the sparse array.
		//		For more details, see:
		//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/map
		// example:
		//	| // returns [2, 3, 4, 5]
		//	| dojo.map([1, 2, 3, 4], function(item){ return item+1 });
	};
	dojo.filter = function(arr, callback, thisObject){
		// summary:
		//		Returns a new Array with those items from arr that match the
		//		condition implemented by callback.
		// arr: Array
		//		the array to iterate over.
		// callback: Function|String
		//		a function that is invoked with three arguments (item,
		//		index, array). The return of this function is expected to
		//		be a boolean which determines whether the passed-in item
		//		will be included in the returned array.
		// thisObject: Object?
		//		may be used to scope the call to callback
		// returns: Array
		// description:
		//		This function corresponds to the JavaScript 1.6 Array.filter() method, with one difference: when
		//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
		//		the callback function with a value of undefined. JavaScript 1.6's filter skips the holes in the sparse array.
		//		For more details, see:
		//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/filter
		// example:
		//	| // returns [2, 3, 4]
		//	| dojo.filter([1, 2, 3, 4], function(item){ return item>1; });
	};
	=====*/

	// our old simple function builder stuff
	var cache = {}, u, array; // the export object

	function clearCache(){
		cache = {};
	}

	function buildFn(fn){
		return cache[fn] = new Function("item", "index", "array", fn); // Function
	}
	// magic snippet: if(typeof fn == "string") fn = cache[fn] || buildFn(fn);

	// every & some

	function everyOrSome(some){
		var every = !some;
		return function(a, fn, o){
			var i = 0, l = a && a.length || 0, result;
			if(l && typeof a == "string") a = a.split("");
			if(typeof fn == "string") fn = cache[fn] || buildFn(fn);
			if(o){
				for(; i < l; ++i){
					result = !fn.call(o, a[i], i, a);
					if(some ^ result){
						return !result;
					}
				}
			}else{
				for(; i < l; ++i){
					result = !fn(a[i], i, a);
					if(some ^ result){
						return !result;
					}
				}
			}
			return every; // Boolean
		}
	}
	// var every = everyOrSome(false), some = everyOrSome(true);

	// indexOf, lastIndexOf

	function index(up){
		var delta = 1, lOver = 0, uOver = 0;
		if(!up){
			delta = lOver = uOver = -1;
		}
		return function(a, x, from, last){
			if(last && delta > 0){
				// TODO: why do we use a non-standard signature? why do we need "last"?
				return array.lastIndexOf(a, x, from);
			}
			var l = a && a.length || 0, end = up ? l + uOver : lOver, i;
			if(from === u){
				i = up ? lOver : l + uOver;
			}else{
				if(from < 0){
					i = l + from;
					if(i < 0){
						i = lOver;
					}
				}else{
					i = from >= l ? l + uOver : from;
				}
			}
			if(l && typeof a == "string") a = a.split("");
			for(; i != end; i += delta){
				if(a[i] == x){
					return i; // Number
				}
			}
			return -1; // Number
		}
	}
	// var indexOf = index(true), lastIndexOf = index(false);

	function forEach(a, fn, o){
		var i = 0, l = a && a.length || 0;
		if(l && typeof a == "string") a = a.split("");
		if(typeof fn == "string") fn = cache[fn] || buildFn(fn);
		if(o){
			for(; i < l; ++i){
				fn.call(o, a[i], i, a);
			}
		}else{
			for(; i < l; ++i){
				fn(a[i], i, a);
			}
		}
	}

	function map(a, fn, o, Ctr){
		// TODO: why do we have a non-standard signature here? do we need "Ctr"?
		var i = 0, l = a && a.length || 0, out = new (Ctr || Array)(l);
		if(l && typeof a == "string") a = a.split("");
		if(typeof fn == "string") fn = cache[fn] || buildFn(fn);
		if(o){
			for(; i < l; ++i){
				out[i] = fn.call(o, a[i], i, a);
			}
		}else{
			for(; i < l; ++i){
				out[i] = fn(a[i], i, a);
			}
		}
		return out; // Array
	}

	function filter(a, fn, o){
		// TODO: do we need "Ctr" here like in map()?
		var i = 0, l = a && a.length || 0, out = [], value;
		if(l && typeof a == "string") a = a.split("");
		if(typeof fn == "string") fn = cache[fn] || buildFn(fn);
		if(o){
			for(; i < l; ++i){
				value = a[i];
				if(fn.call(o, value, i, a)){
					out.push(value);
				}
			}
		}else{
			for(; i < l; ++i){
				value = a[i];
				if(fn(value, i, a)){
					out.push(value);
				}
			}
		}
		return out; // Array
	}

	array = {
		every:       everyOrSome(false),
		some:        everyOrSome(true),
		indexOf:     index(true),
		lastIndexOf: index(false),
		forEach:     forEach,
		map:         map,
		filter:      filter,
		clearCache:  clearCache
	};

	1 && lang.mixin(dojo, array);

	/*===== return dojo.array; =====*/
	return array;
});

},
'dojo/_base/Deferred':function(){
define(["./kernel", "./lang"], function(dojo, lang){
	// module:
	//		dojo/_base/Deferred
	// summary:
	//		This module defines dojo.Deferred.

	var mutator = function(){};
	var freeze = Object.freeze || function(){};
	// A deferred provides an API for creating and resolving a promise.
	dojo.Deferred = function(/*Function?*/ canceller){
		// summary:
		//		Deferreds provide a generic means for encapsulating an asynchronous
		//		operation and notifying users of the completion and result of the operation.
		// description:
		//		The dojo.Deferred API is based on the concept of promises that provide a
		//		generic interface into the eventual completion of an asynchronous action.
		//		The motivation for promises fundamentally is about creating a
		//		separation of concerns that allows one to achieve the same type of
		//		call patterns and logical data flow in asynchronous code as can be
		//		achieved in synchronous code. Promises allows one
		//		to be able to call a function purely with arguments needed for
		//		execution, without conflating the call with concerns of whether it is
		//		sync or async. One shouldn't need to alter a call's arguments if the
		//		implementation switches from sync to async (or vice versa). By having
		//		async functions return promises, the concerns of making the call are
		//		separated from the concerns of asynchronous interaction (which are
		//		handled by the promise).
		//
		//		The dojo.Deferred is a type of promise that provides methods for fulfilling the
		//		promise with a successful result or an error. The most important method for
		//		working with Dojo's promises is the then() method, which follows the
		//		CommonJS proposed promise API. An example of using a Dojo promise:
		//
		//		|	var resultingPromise = someAsyncOperation.then(function(result){
		//		|		... handle result ...
		//		|	},
		//		|	function(error){
		//		|		... handle error ...
		//		|	});
		//
		//		The .then() call returns a new promise that represents the result of the
		//		execution of the callback. The callbacks will never affect the original promises value.
		//
		//		The dojo.Deferred instances also provide the following functions for backwards compatibility:
		//
		//			* addCallback(handler)
		//			* addErrback(handler)
		//			* callback(result)
		//			* errback(result)
		//
		//		Callbacks are allowed to return promises themselves, so
		//		you can build complicated sequences of events with ease.
		//
		//		The creator of the Deferred may specify a canceller.  The canceller
		//		is a function that will be called if Deferred.cancel is called
		//		before the Deferred fires. You can use this to implement clean
		//		aborting of an XMLHttpRequest, etc. Note that cancel will fire the
		//		deferred with a CancelledError (unless your canceller returns
		//		another kind of error), so the errbacks should be prepared to
		//		handle that error for cancellable Deferreds.
		// example:
		//	|	var deferred = new dojo.Deferred();
		//	|	setTimeout(function(){ deferred.callback({success: true}); }, 1000);
		//	|	return deferred;
		// example:
		//		Deferred objects are often used when making code asynchronous. It
		//		may be easiest to write functions in a synchronous manner and then
		//		split code using a deferred to trigger a response to a long-lived
		//		operation. For example, instead of register a callback function to
		//		denote when a rendering operation completes, the function can
		//		simply return a deferred:
		//
		//		|	// callback style:
		//		|	function renderLotsOfData(data, callback){
		//		|		var success = false
		//		|		try{
		//		|			for(var x in data){
		//		|				renderDataitem(data[x]);
		//		|			}
		//		|			success = true;
		//		|		}catch(e){ }
		//		|		if(callback){
		//		|			callback(success);
		//		|		}
		//		|	}
		//
		//		|	// using callback style
		//		|	renderLotsOfData(someDataObj, function(success){
		//		|		// handles success or failure
		//		|		if(!success){
		//		|			promptUserToRecover();
		//		|		}
		//		|	});
		//		|	// NOTE: no way to add another callback here!!
		// example:
		//		Using a Deferred doesn't simplify the sending code any, but it
		//		provides a standard interface for callers and senders alike,
		//		providing both with a simple way to service multiple callbacks for
		//		an operation and freeing both sides from worrying about details
		//		such as "did this get called already?". With Deferreds, new
		//		callbacks can be added at any time.
		//
		//		|	// Deferred style:
		//		|	function renderLotsOfData(data){
		//		|		var d = new dojo.Deferred();
		//		|		try{
		//		|			for(var x in data){
		//		|				renderDataitem(data[x]);
		//		|			}
		//		|			d.callback(true);
		//		|		}catch(e){
		//		|			d.errback(new Error("rendering failed"));
		//		|		}
		//		|		return d;
		//		|	}
		//
		//		|	// using Deferred style
		//		|	renderLotsOfData(someDataObj).then(null, function(){
		//		|		promptUserToRecover();
		//		|	});
		//		|	// NOTE: addErrback and addCallback both return the Deferred
		//		|	// again, so we could chain adding callbacks or save the
		//		|	// deferred for later should we need to be notified again.
		// example:
		//		In this example, renderLotsOfData is synchronous and so both
		//		versions are pretty artificial. Putting the data display on a
		//		timeout helps show why Deferreds rock:
		//
		//		|	// Deferred style and async func
		//		|	function renderLotsOfData(data){
		//		|		var d = new dojo.Deferred();
		//		|		setTimeout(function(){
		//		|			try{
		//		|				for(var x in data){
		//		|					renderDataitem(data[x]);
		//		|				}
		//		|				d.callback(true);
		//		|			}catch(e){
		//		|				d.errback(new Error("rendering failed"));
		//		|			}
		//		|		}, 100);
		//		|		return d;
		//		|	}
		//
		//		|	// using Deferred style
		//		|	renderLotsOfData(someDataObj).then(null, function(){
		//		|		promptUserToRecover();
		//		|	});
		//
		//		Note that the caller doesn't have to change his code at all to
		//		handle the asynchronous case.

		var result, finished, isError, head, nextListener;
		var promise = (this.promise = {});

		function complete(value){
			if(finished){
				throw new Error("This deferred has already been resolved");
			}
			result = value;
			finished = true;
			notify();
		}
		function notify(){
			var mutated;
			while(!mutated && nextListener){
				var listener = nextListener;
				nextListener = nextListener.next;
				if((mutated = (listener.progress == mutator))){ // assignment and check
					finished = false;
				}
				var func = (isError ? listener.error : listener.resolved);
				if(func){
					try{
						var newResult = func(result);
						if (newResult && typeof newResult.then === "function"){
							newResult.then(lang.hitch(listener.deferred, "resolve"), lang.hitch(listener.deferred, "reject"), lang.hitch(listener.deferred, "progress"));
							continue;
						}
						var unchanged = mutated && newResult === undefined;
						if(mutated && !unchanged){
							isError = newResult instanceof Error;
						}
						listener.deferred[unchanged && isError ? "reject" : "resolve"](unchanged ? result : newResult);
					}catch(e){
						listener.deferred.reject(e);
					}
				}else{
					if(isError){
						listener.deferred.reject(result);
					}else{
						listener.deferred.resolve(result);
					}
				}
			}
		}
		// calling resolve will resolve the promise
		this.resolve = this.callback = function(value){
			// summary:
			//		Fulfills the Deferred instance successfully with the provide value
			this.fired = 0;
			this.results = [value, null];
			complete(value);
		};


		// calling error will indicate that the promise failed
		this.reject = this.errback = function(error){
			// summary:
			//		Fulfills the Deferred instance as an error with the provided error
			isError = true;
			this.fired = 1;
			complete(error);
			this.results = [null, error];
			if(!error || error.log !== false){
				(dojo.config.deferredOnError || function(x){ console.error(x); })(error);
			}
		};
		// call progress to provide updates on the progress on the completion of the promise
		this.progress = function(update){
			// summary:
			//		Send progress events to all listeners
			var listener = nextListener;
			while(listener){
				var progress = listener.progress;
				progress && progress(update);
				listener = listener.next;
			}
		};
		this.addCallbacks = function(callback, errback){
			// summary:
			//		Adds callback and error callback for this deferred instance.
			// callback: Function?
			// 		The callback attached to this deferred object.
			// errback: Function?
			// 		The error callback attached to this deferred object.
			// returns:
			// 		Returns this deferred object.
			this.then(callback, errback, mutator);
			return this;	// dojo.Deferred
		};
		// provide the implementation of the promise
		promise.then = this.then = function(/*Function?*/resolvedCallback, /*Function?*/errorCallback, /*Function?*/progressCallback){
			// summary:
			//		Adds a fulfilledHandler, errorHandler, and progressHandler to be called for
			//		completion of a promise. The fulfilledHandler is called when the promise
			//		is fulfilled. The errorHandler is called when a promise fails. The
			//		progressHandler is called for progress events. All arguments are optional
			//		and non-function values are ignored. The progressHandler is not only an
			//		optional argument, but progress events are purely optional. Promise
			//		providers are not required to ever create progress events.
			//
			//		This function will return a new promise that is fulfilled when the given
			//		fulfilledHandler or errorHandler callback is finished. This allows promise
			//		operations to be chained together. The value returned from the callback
			//		handler is the fulfillment value for the returned promise. If the callback
			//		throws an error, the returned promise will be moved to failed state.
			//
			// returns: 
			//		Returns a new promise that represents the result of the
			//		execution of the callback. The callbacks will never affect the original promises value.
			// example:
			//		An example of using a CommonJS compliant promise:
			//		|	asyncComputeTheAnswerToEverything().
			//		|		then(addTwo).
			//		|		then(printResult, onError);
			//		|	>44
			//
			var returnDeferred = progressCallback == mutator ? this : new dojo.Deferred(promise.cancel);
			var listener = {
				resolved: resolvedCallback,
				error: errorCallback,
				progress: progressCallback,
				deferred: returnDeferred
			};
			if(nextListener){
				head = head.next = listener;
			}
			else{
				nextListener = head = listener;
			}
			if(finished){
				notify();
			}
			return returnDeferred.promise; // Promise
		};
		var deferred = this;
		promise.cancel = this.cancel = function (){
			// summary:
			//		Cancels the asynchronous operation
			if(!finished){
				var error = canceller && canceller(deferred);
				if(!finished){
					if (!(error instanceof Error)){
						error = new Error(error);
					}
					error.log = false;
					deferred.reject(error);
				}
			}
		};
		freeze(promise);
	};
	lang.extend(dojo.Deferred, {
		addCallback: function (/*Function*/ callback){
			// summary:
			// 		Adds successful callback for this deferred instance.
			// returns:
			// 		Returns this deferred object.
			return this.addCallbacks(lang.hitch.apply(dojo, arguments));	// dojo.Deferred
		},

		addErrback: function (/*Function*/ errback){
			// summary:
			// 		Adds error callback for this deferred instance.
			// returns:
			// 		Returns this deferred object.
			return this.addCallbacks(null, lang.hitch.apply(dojo, arguments));	// dojo.Deferred
		},

		addBoth: function (/*Function*/ callback){
			// summary:
			// 		Add handler as both successful callback and error callback for this deferred instance.
			// returns:
			// 		Returns this deferred object.
			var enclosed = lang.hitch.apply(dojo, arguments);
			return this.addCallbacks(enclosed, enclosed);	// dojo.Deferred
		},
		fired: -1
	});

	dojo.Deferred.when = dojo.when = function(promiseOrValue, /*Function?*/ callback, /*Function?*/ errback, /*Function?*/ progressHandler){
		// summary:
		//		This provides normalization between normal synchronous values and
		//		asynchronous promises, so you can interact with them in a common way
		// returns:
		// 		Returns a new promise that represents the result of the execution of callback 
		// 		when parameter "promiseOrValue" is promise.
		// 		Returns the execution result of callback when parameter "promiseOrValue" is value.
		// example:
		//		|	function printFirstAndLast(items){
		//		|		dojo.when(findFirst(items), console.log);
		//		|		dojo.when(findLast(items), console.log);
		//		|	}
		//		|	function findFirst(items){
		//		|		return dojo.when(items, function(items){
		//		|			return items[0];
		//		|		});
		//		|	}
		//		|	function findLast(items){
		//		|		return dojo.when(items, function(items){
		//		|			return items[items.length - 1];
		//		|		});
		//		|	}
		//		And now all three of his functions can be used sync or async.
		//		|	printFirstAndLast([1,2,3,4]) will work just as well as
		//		|	printFirstAndLast(dojo.xhrGet(...));

		if(promiseOrValue && typeof promiseOrValue.then === "function"){
			return promiseOrValue.then(callback, errback, progressHandler);
		}
		return callback ? callback(promiseOrValue) : promiseOrValue;	// Promise
	};

	return dojo.Deferred;
});

},
'dojo/store/JsonRest':function(){
define(["../_base/xhr", "../json", "../_base/declare", "./util/QueryResults"
], function(xhr, JSON, declare, QueryResults) {
  //  module:
  //    dojo/store/JsonRest
  //  summary:
  //    The module defines a JSON/REST based object store 

return declare("dojo.store.JsonRest", null, {
	// summary:
	//		This is a basic store for RESTful communicating with a server through JSON
	//		formatted data. It implements dojo.store.api.Store.

	constructor: function(/*dojo.store.JsonRest*/ options){
		// summary:
		//		This is a basic store for RESTful communicating with a server through JSON
		//		formatted data.
		// options:
		//		This provides any configuration information that will be mixed into the store
		declare.safeMixin(this, options);
	},
	// target: String
	//		The target base URL to use for all requests to the server. This string will be
	// 	prepended to the id to generate the URL (relative or absolute) for requests
	// 	sent to the server
	target: "",
	// idProperty: String
	//		Indicates the property to use as the identity property. The values of this
	//		property should be unique.
	idProperty: "id",
	// sortParam: String
	// 		The query parameter to used for holding sort information. If this is omitted, than
	//		the sort information is included in a functional query token to avoid colliding 
	// 		with the set of name/value pairs.
	
	get: function(id, options){
		//	summary:
		//		Retrieves an object by its identity. This will trigger a GET request to the server using
		//		the url `this.target + id`.
		//	id: Number
		//		The identity to use to lookup the object
		//	returns: Object
		//		The object in the store that matches the given id.
		var headers = options || {};
		headers.Accept = this.accepts;
		return xhr("GET", {
			url:this.target + id,
			handleAs: "json",
			headers: headers
		});
	},
	// accepts: String
	//		Defines the Accept header to use on HTTP requests
	accepts: "application/javascript, application/json", 
	getIdentity: function(object){
		// summary:
		//		Returns an object's identity
		// object: Object
		//		The object to get the identity from
		//	returns: Number
		return object[this.idProperty];
	},
	put: function(object, options){
		// summary:
		//		Stores an object. This will trigger a PUT request to the server
		//		if the object has an id, otherwise it will trigger a POST request.
		// object: Object
		//		The object to store.
		// options: dojo.store.api.Store.PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		//	returns: Number
		options = options || {};
		var id = ("id" in options) ? options.id : this.getIdentity(object);
		var hasId = typeof id != "undefined";
		return xhr(hasId && !options.incremental ? "PUT" : "POST", {
				url: hasId ? this.target + id : this.target,
				postData: JSON.stringify(object),
				handleAs: "json",
				headers:{
					"Content-Type": "application/json",
					Accept: this.accepts,
					"If-Match": options.overwrite === true ? "*" : null,
					"If-None-Match": options.overwrite === false ? "*" : null
				}
			});
	},
	add: function(object, options){
		// summary:
		//		Adds an object. This will trigger a PUT request to the server
		//		if the object has an id, otherwise it will trigger a POST request.
		// object: Object
		//		The object to store.
		// options: dojo.store.api.Store.PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		options = options || {};
		options.overwrite = false;
		return this.put(object, options);
	},
	remove: function(id){
		// summary:
		//		Deletes an object by its identity. This will trigger a DELETE request to the server.
		// id: Number
		//		The identity to use to delete the object
		return xhr("DELETE",{
			url:this.target + id
		});
	},
	query: function(query, options){
		// summary:
		//		Queries the store for objects. This will trigger a GET request to the server, with the
		//		query added as a query string.
		// query: Object
		//		The query to use for retrieving objects from the store.
		//	options: dojo.store.api.Store.QueryOptions?
		//		The optional arguments to apply to the resultset.
		//	returns: dojo.store.api.Store.QueryResults
		//		The results of the query, extended with iterative methods.
		var headers = {Accept: this.accepts};
		options = options || {};

		if(options.start >= 0 || options.count >= 0){
			headers.Range = "items=" + (options.start || '0') + '-' +
				(("count" in options && options.count != Infinity) ?
					(options.count + (options.start || 0) - 1) : '');
		}
		if(query && typeof query == "object"){
			query = xhr.objectToQuery(query);
			query = query ? "?" + query: "";
		}
		if(options && options.sort){
			var sortParam = this.sortParam;
			query += (query ? "&" : "?") + (sortParam ? sortParam + '=' : "sort(");
			for(var i = 0; i<options.sort.length; i++){
				var sort = options.sort[i];
				query += (i > 0 ? "," : "") + (sort.descending ? '-' : '+') + encodeURIComponent(sort.attribute);
			}
			if(!sortParam){
				query += ")";
			}
		}
		var results = xhr("GET", {
			url: this.target + (query || ""),
			handleAs: "json",
			headers: headers
		});
		results.total = results.then(function(){
			var range = results.ioArgs.xhr.getResponseHeader("Content-Range");
			return range && (range=range.match(/\/(.*)/)) && +range[1];
		});
		return QueryResults(results);
	}
});

});
},
'dojo/uacss':function(){
define(["./dom-geometry", "./_base/lang", "./ready", "./_base/sniff", "./_base/window"],
	function(geometry, lang, ready, has, baseWindow){
	// module:
	//		dojo/uacss
	// summary:
	//		Applies pre-set CSS classes to the top-level HTML node, based on:
	//			- browser (ex: dj_ie)
	//			- browser version (ex: dj_ie6)
	//			- box model (ex: dj_contentBox)
	//			- text direction (ex: dijitRtl)
	//
	//		In addition, browser, browser version, and box model are
	//		combined with an RTL flag when browser text is RTL. ex: dj_ie-rtl.

	var
		html = baseWindow.doc.documentElement,
		ie = has("ie"),
		opera = has("opera"),
		maj = Math.floor,
		ff = has("ff"),
		boxModel = geometry.boxModel.replace(/-/,''),

		classes = {
			"dj_quirks": has("quirks"),

			// NOTE: Opera not supported by dijit
			"dj_opera": opera,

			"dj_khtml": has("khtml"),

			"dj_webkit": has("webkit"),
			"dj_safari": has("safari"),
			"dj_chrome": has("chrome"),

			"dj_gecko": has("mozilla")
		}; // no dojo unsupported browsers

	if(ie){
		classes["dj_ie"] = true;
		classes["dj_ie" + maj(ie)] = true;
		classes["dj_iequirks"] = has("quirks");
	}
	if(ff){
		classes["dj_ff" + maj(ff)] = true;
	}

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = lang.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	// priority is 90 to run ahead of parser priority of 100
	ready(90, function(){
		if(!geometry.isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ");
			html.className = lang.trim(html.className + " " + rtlClassStr + "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl "));
		}
	});
	return has;
});

},
'dojo/dom':function(){
define(["./_base/sniff", "./_base/lang", "./_base/window"],
		function(has, lang, win){
	// module:
	//		dojo/dom
	// summary:
	//		This module defines the core dojo DOM API.

	// FIXME: need to add unit tests for all the semi-public methods

		try{
		document.execCommand("BackgroundImageCache", false, true);
	}catch(e){
		// sane browsers don't have cache "issues"
	}
	
	// =============================
	// DOM Functions
	// =============================

	/*=====
	dojo.byId = function(id, doc){
		// summary:
		//		Returns DOM node with matching `id` attribute or `null`
		//		if not found. If `id` is a DomNode, this function is a no-op.
		//
		// id: String|DOMNode
		//	 	A string to match an HTML id attribute or a reference to a DOM Node
		//
		// doc: Document?
		//		Document to work in. Defaults to the current value of
		//		dojo.doc.  Can be used to retrieve
		//		node references from other documents.
		//
		// example:
		//		Look up a node by ID:
		//	|	var n = dojo.byId("foo");
		//
		// example:
		//		Check if a node exists, and use it.
		//	|	var n = dojo.byId("bar");
		//	|	if(n){ doStuff() ... }
		//
		// example:
		//		Allow string or DomNode references to be passed to a custom function:
		//	|	var foo = function(nodeOrId){
		//	|		nodeOrId = dojo.byId(nodeOrId);
		//	|		// ... more stuff
		//	|	}
	=====*/

	/*=====
	dojo.isDescendant = function(node, ancestor){
		// summary:
		//		Returns true if node is a descendant of ancestor
		// node: DOMNode|String
		//		string id or node reference to test
		// ancestor: DOMNode|String
		//		string id or node reference of potential parent to test against
		//
		// example:
		//		Test is node id="bar" is a descendant of node id="foo"
		//	|	if(dojo.isDescendant("bar", "foo")){ ... }
	};
	=====*/

	// TODO: do we need this function in the base?

	/*=====
	dojo.setSelectable = function(node, selectable){
		// summary:
		//		Enable or disable selection on a node
		// node: DOMNode|String
		//		id or reference to node
		// selectable: Boolean
		//		state to put the node in. false indicates unselectable, true
		//		allows selection.
		// example:
		//		Make the node id="bar" unselectable
		//	|	dojo.setSelectable("bar");
		// example:
		//		Make the node id="bar" selectable
		//	|	dojo.setSelectable("bar", true);
	};
	=====*/

	var dom = {};   // the result object

		if(has("ie")){
		dom.byId = function(id, doc){
			if(typeof id != "string"){
				return id;
			}
			var _d = doc || win.doc, te = id && _d.getElementById(id);
			// attributes.id.value is better than just id in case the
			// user has a name=id inside a form
			if(te && (te.attributes.id.value == id || te.id == id)){
				return te;
			}else{
				var eles = _d.all[id];
				if(!eles || eles.nodeName){
					eles = [eles];
				}
				// if more than 1, choose first with the correct id
				var i = 0;
				while((te = eles[i++])){
					if((te.attributes && te.attributes.id && te.attributes.id.value == id) || te.id == id){
						return te;
					}
				}
			}
		};
	}else{
			dom.byId = function(id, doc){
			// inline'd type check.
			// be sure to return null per documentation, to match IE branch.
			return ((typeof id == "string") ? (doc || win.doc).getElementById(id) : id) || null; // DOMNode
		};
		}
		/*=====
	};
	=====*/

	dom.isDescendant = function(/*DOMNode|String*/node, /*DOMNode|String*/ancestor){
		try{
			node = dom.byId(node);
			ancestor = dom.byId(ancestor);
			while(node){
				if(node == ancestor){
					return true; // Boolean
				}
				node = node.parentNode;
			}
		}catch(e){ /* squelch, return false */ }
		return false; // Boolean
	};

	// TODO: do we need this function in the base?

		
	// Add feature test for user-select CSS property
	// (currently known to work in all but IE < 10 and Opera)
	has.add("css-user-select", function(global, doc, element){
		// Avoid exception when dom.js is loaded in non-browser environments
		if(!element){ return false; }

		var style = element.style;
		var prefixes = ["Khtml", "O", "ms", "Moz", "Webkit"],
			i = prefixes.length,
			name = "userSelect",
			prefix;

		// Iterate prefixes from most to least likely
		do{
			if(typeof style[name] !== "undefined"){
				// Supported; return property name
				return name;
			}
		}while(i-- && (name = prefixes[i] + "UserSelect"));

		// Not supported if we didn't return before now
		return false;
	});

	var cssUserSelect = has("css-user-select");
	dom.setSelectable = cssUserSelect ? function(node, selectable){
		// css-user-select returns a (possibly vendor-prefixed) CSS property name
		dom.byId(node).style[cssUserSelect] = selectable ? "" : "none";
	} : function(node, selectable){
		node = dom.byId(node);

		// (IE < 10 / Opera) Fall back to setting/removing the
		// unselectable attribute on the element and all its children
		var nodes = node.getElementsByTagName("*"),
			i = nodes.length;

		if(selectable){
			node.removeAttribute("unselectable");
			while(i--){
				nodes[i].removeAttribute("unselectable");
			}
		}else{
			node.setAttribute("unselectable", "on");
			while(i--){
				nodes[i].setAttribute("unselectable", "on");
			}
		}
	};
	/*
				*/
	
	return dom;
});

},
'dojo/dom-style':function(){
define(["./_base/sniff", "./dom"], function(has, dom){
	// module:
	//		dojo/dom-style
	// summary:
	//		This module defines the core dojo DOM style API.

	// =============================
	// Style Functions
	// =============================

	// getComputedStyle drives most of the style code.
	// Wherever possible, reuse the returned object.
	//
	// API functions below that need to access computed styles accept an
	// optional computedStyle parameter.
	// If this parameter is omitted, the functions will call getComputedStyle themselves.
	// This way, calling code can access computedStyle once, and then pass the reference to
	// multiple API functions.

	/*=====
	dojo.getComputedStyle = function(node){
		// summary:
		//		Returns a "computed style" object.
		//
		// description:
		//		Gets a "computed style" object which can be used to gather
		//		information about the current state of the rendered node.
		//
		//		Note that this may behave differently on different browsers.
		//		Values may have different formats and value encodings across
		//		browsers.
		//
		//		Note also that this method is expensive.  Wherever possible,
		//		reuse the returned object.
		//
		//		Use the dojo.style() method for more consistent (pixelized)
		//		return values.
		//
		// node: DOMNode
		//		A reference to a DOM node. Does NOT support taking an
		//		ID string for speed reasons.
		// example:
		//	|	dojo.getComputedStyle(dojo.byId('foo')).borderWidth;
		//
		// example:
		//		Reusing the returned object, avoiding multiple lookups:
		//	|	var cs = dojo.getComputedStyle(dojo.byId("someNode"));
		//	|	var w = cs.width, h = cs.height;
		return; // CSS2Properties
	}
	=====*/

	/*=====
	dojo.toPixelValue = function(node, value){
		// summary:
		//      converts style value to pixels on IE or return a numeric value.
		// node: DOMNode
		// value: String
		// returns: Number
	};
	=====*/

	/*=====
	dojo._toPixelValue = function(node, value){
		// summary:
		//		Existing alias for `dojo._toPixelValue`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.getStyle = function(node, name){
		// summary:
		//		Accesses styles on a node.
		// description:
		//		Getting the style value uses the computed style for the node, so the value
		//		will be a calculated value, not just the immediate node.style value.
		//		Also when getting values, use specific style names,
		//		like "borderBottomWidth" instead of "border" since compound values like
		//		"border" are not necessarily reflected as expected.
		//		If you want to get node dimensions, use `dojo.marginBox()`,
		//		`dojo.contentBox()` or `dojo.position()`.
		// node: DOMNode|String
		//		id or reference to node to get style for
		// name: String?
		//		the style property to get
		// example:
		//		Passing only an ID or node returns the computed style object of
		//		the node:
		//	|	dojo.getStyle("thinger");
		// example:
		//		Passing a node and a style property returns the current
		//		normalized, computed value for that property:
		//	|	dojo.getStyle("thinger", "opacity"); // 1 by default
	};
	=====*/

	/*=====
	dojo.setStyle = function(node, name, value){
		// summary:
		//		Sets styles on a node.
		// node: DOMNode|String
		//		id or reference to node to set style for
		// name: String|Object
		//		the style property to set in DOM-accessor format
		//		("borderWidth", not "border-width") or an object with key/value
		//		pairs suitable for setting each property.
		// value: String?
		//		If passed, sets value on the node for style, handling
		//		cross-browser concerns.  When setting a pixel value,
		//		be sure to include "px" in the value. For instance, top: "200px".
		//		Otherwise, in some cases, some browsers will not apply the style.
		//
		// example:
		//		Passing a node, a style property, and a value changes the
		//		current display of the node and returns the new computed value
		//	|	dojo.setStyle("thinger", "opacity", 0.5); // == 0.5
		//
		// example:
		//		Passing a node, an object-style style property sets each of the values in turn and returns the computed style object of the node:
		//	|	dojo.setStyle("thinger", {
		//	|		"opacity": 0.5,
		//	|		"border": "3px solid black",
		//	|		"height": "300px"
		//	|	});
		//
		// example:
		//		When the CSS style property is hyphenated, the JavaScript property is camelCased.
		//		font-size becomes fontSize, and so on.
		//	|	dojo.setStyle("thinger",{
		//	|		fontSize:"14pt",
		//	|		letterSpacing:"1.2em"
		//	|	});
		//
		// example:
		//		dojo.NodeList implements .style() using the same syntax, omitting the "node" parameter, calling
		//		dojo.style() on every element of the list. See: `dojo.query()` and `dojo.NodeList()`
		//	|	dojo.query(".someClassName").style("visibility","hidden");
		//	|	// or
		//	|	dojo.query("#baz > div").style({
		//	|		opacity:0.75,
		//	|		fontSize:"13pt"
		//	|	});
	};
	=====*/

	// Although we normally eschew argument validation at this
	// level, here we test argument 'node' for (duck)type,
	// by testing nodeType, ecause 'document' is the 'parentNode' of 'body'
	// it is frequently sent to this function even
	// though it is not Element.
	var getComputedStyle, style = {};
		if(has("webkit")){
			getComputedStyle = function(/*DomNode*/node){
			var s;
			if(node.nodeType == 1){
				var dv = node.ownerDocument.defaultView;
				s = dv.getComputedStyle(node, null);
				if(!s && node.style){
					node.style.display = "";
					s = dv.getComputedStyle(node, null);
				}
			}
			return s || {};
		};
		}else if(has("ie") && (has("ie") < 9 || has("quirks"))){
		getComputedStyle = function(node){
			// IE (as of 7) doesn't expose Element like sane browsers
			return node.nodeType == 1 /* ELEMENT_NODE*/ ? node.currentStyle : {};
		};
	}else{
		getComputedStyle = function(node){
			return node.nodeType == 1 ?
				node.ownerDocument.defaultView.getComputedStyle(node, null) : {};
		};
	}
		style.getComputedStyle = getComputedStyle;

	var toPixel;
		if(!has("ie")){
			toPixel = function(element, value){
			// style values can be floats, client code may want
			// to round for integer pixels.
			return parseFloat(value) || 0;
		};
		}else{
		toPixel = function(element, avalue){
			if(!avalue){ return 0; }
			// on IE7, medium is usually 4 pixels
			if(avalue == "medium"){ return 4; }
			// style values can be floats, client code may
			// want to round this value for integer pixels.
			if(avalue.slice && avalue.slice(-2) == 'px'){ return parseFloat(avalue); }
			var s = element.style, rs = element.runtimeStyle, cs = element.currentStyle,
				sLeft = s.left, rsLeft = rs.left;
			rs.left = cs.left;
			try{
				// 'avalue' may be incompatible with style.left, which can cause IE to throw
				// this has been observed for border widths using "thin", "medium", "thick" constants
				// those particular constants could be trapped by a lookup
				// but perhaps there are more
				s.left = avalue;
				avalue = s.pixelLeft;
			}catch(e){
				avalue = 0;
			}
			s.left = sLeft;
			rs.left = rsLeft;
			return avalue;
		}
	}
		style.toPixelValue = toPixel;

	// FIXME: there opacity quirks on FF that we haven't ported over. Hrm.

		var astr = "DXImageTransform.Microsoft.Alpha";
	var af = function(n, f){
		try{
			return n.filters.item(astr);
		}catch(e){
			return f ? {} : null;
		}
	};

		var _getOpacity =
			has("ie") < 9 || (has("ie") < 10 && has("quirks")) ? function(node){
			try{
				return af(node).Opacity / 100; // Number
			}catch(e){
				return 1; // Number
			}
		} :
			function(node){
			return getComputedStyle(node).opacity;
		};

	var _setOpacity =
				has("ie") < 9 || (has("ie") < 10 && has("quirks")) ? function(/*DomNode*/node, /*Number*/opacity){
			var ov = opacity * 100, opaque = opacity == 1;
			node.style.zoom = opaque ? "" : 1;

			if(!af(node)){
				if(opaque){
					return opacity;
				}
				node.style.filter += " progid:" + astr + "(Opacity=" + ov + ")";
			}else{
				af(node, 1).Opacity = ov;
			}

			// on IE7 Alpha(Filter opacity=100) makes text look fuzzy so disable it altogether (bug #2661),
			//but still update the opacity value so we can get a correct reading if it is read later.
			af(node, 1).Enabled = !opaque;

			if(node.tagName.toLowerCase() == "tr"){
				for(var td = node.firstChild; td; td = td.nextSibling){
					if(td.tagName.toLowerCase() == "td"){
						_setOpacity(td, opacity);
					}
				}
			}
			return opacity;
		} :
				function(node, opacity){
			return node.style.opacity = opacity;
		};

	var _pixelNamesCache = {
		left: true, top: true
	};
	var _pixelRegExp = /margin|padding|width|height|max|min|offset/; // |border
	function _toStyleValue(node, type, value){
		//TODO: should we really be doing string case conversion here? Should we cache it? Need to profile!
		type = type.toLowerCase();
				if(has("ie")){
			if(value == "auto"){
				if(type == "height"){ return node.offsetHeight; }
				if(type == "width"){ return node.offsetWidth; }
			}
			if(type == "fontweight"){
				switch(value){
					case 700: return "bold";
					case 400:
					default: return "normal";
				}
			}
		}
				if(!(type in _pixelNamesCache)){
			_pixelNamesCache[type] = _pixelRegExp.test(type);
		}
		return _pixelNamesCache[type] ? toPixel(node, value) : value;
	}

	var _floatStyle = has("ie") ? "styleFloat" : "cssFloat",
		_floatAliases = {"cssFloat": _floatStyle, "styleFloat": _floatStyle, "float": _floatStyle};

	// public API

	style.get = function getStyle(/*DOMNode|String*/ node, /*String?*/ name){
		var n = dom.byId(node), l = arguments.length, op = (name == "opacity");
		if(l == 2 && op){
			return _getOpacity(n);
		}
		name = _floatAliases[name] || name;
		var s = style.getComputedStyle(n);
		return (l == 1) ? s : _toStyleValue(n, name, s[name] || n.style[name]); /* CSS2Properties||String||Number */
	};

	style.set = function setStyle(/*DOMNode|String*/ node, /*String|Object*/ name, /*String?*/ value){
		var n = dom.byId(node), l = arguments.length, op = (name == "opacity");
		name = _floatAliases[name] || name;
		if(l == 3){
			return op ? _setOpacity(n, value) : n.style[name] = value; // Number
		}
		for(var x in name){
			style.set(node, x, name[x]);
		}
		return style.getComputedStyle(n);
	};

	return style;
});

},
'dojo/text':function(){
define(["./_base/kernel", "require", "./has", "./_base/xhr"], function(dojo, require, has, xhr){
	// module:
	//		dojo/text
	// summary:
	//		This module implements the !dojo/text plugin and the dojo.cache API.
	// description:
	//		We choose to include our own plugin to leverage functionality already contained in dojo
	//		and thereby reduce the size of the plugin compared to various foreign loader implementations.
	//		Also, this allows foreign AMD loaders to be used without their plugins.
	//
	//		CAUTION: this module is designed to optionally function synchronously to support the dojo v1.x synchronous
	//		loader. This feature is outside the scope of the CommonJS plugins specification.

	var getText;
	if(1){
		getText= function(url, sync, load){
			xhr("GET", {url:url, sync:!!sync, load:load});
		};
	}else{
		// TODOC: only works for dojo AMD loader
		if(require.getText){
			getText= require.getText;
		}else{
			console.error("dojo/text plugin failed to load because loader does not support getText");
		}
	}

	var
		theCache= {},

		strip= function(text){
			//Strips <?xml ...?> declarations so that external SVG and XML
			//documents can be added to a document without worry. Also, if the string
			//is an HTML document, only the part inside the body tag is returned.
			if(text){
				text= text.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
				var matches= text.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
				if(matches){
					text= matches[1];
				}
			}else{
				text = "";
			}
			return text;
		},

		notFound = {},

		pending = {},

		result= {
			dynamic:
				// the dojo/text caches it's own resources because of dojo.cache
				true,

			normalize:function(id, toAbsMid){
				// id is something like (path may be relative):
				//
				//	 "path/to/text.html"
				//	 "path/to/text.html!strip"
				var parts= id.split("!"),
					url= parts[0];
				return (/^\./.test(url) ? toAbsMid(url) : url) + (parts[1] ? "!" + parts[1] : "");
			},

			load:function(id, require, load){
				// id is something like (path is always absolute):
				//
				//	 "path/to/text.html"
				//	 "path/to/text.html!strip"
				var
					parts= id.split("!"),
					stripFlag= parts.length>1,
					absMid= parts[0],
					url = require.toUrl(parts[0]),
					text = notFound,
					finish = function(text){
						load(stripFlag ? strip(text) : text);
					};
				if(absMid in theCache){
					text = theCache[absMid];
				}else if(url in require.cache){
					text = require.cache[url];
				}else if(url in theCache){
					text = theCache[url];
				}
				if(text===notFound){
					if(pending[url]){
						pending[url].push(finish);
					}else{
						var pendingList = pending[url] = [finish];
						getText(url, !require.async, function(text){
							theCache[absMid]= theCache[url]= text;
							for(var i = 0; i<pendingList.length;){
								pendingList[i++](text);
							}
							delete pending[url];
						});
					}
				}else{
					finish(text);
				}
			}
		};

	dojo.cache= function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		//	 * (string string [value]) => (module, url, value)
		//	 * (object [value])        => (module, value), url defaults to ""
		//
		//	 * if module is an object, then it must be convertable to a string
		//	 * (module, url) module + (url ? ("/" + url) : "") must be a legal argument to require.toUrl
		//	 * value may be a string or an object; if an object then may have the properties "value" and/or "sanitize"
		var key;
		if(typeof module=="string"){
			if(/\//.test(module)){
				// module is a version 1.7+ resolved path
				key = module;
				value = url;
			}else{
				// module is a version 1.6- argument to dojo.moduleUrl
				key = require.toUrl(module.replace(/\./g, "/") + (url ? ("/" + url) : ""));
			}
		}else{
			key = module + "";
			value = url;
		}
		var
			val = (value != undefined && typeof value != "string") ? value.value : value,
			sanitize = value && value.sanitize;

		if(typeof val == "string"){
			//We have a string, set cache value
			theCache[key] = val;
			return sanitize ? strip(val) : val;
		}else if(val === null){
			//Remove cached value
			delete theCache[key];
			return null;
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in theCache)){
				getText(key, true, function(text){
					theCache[key]= text;
				});
			}
			return sanitize ? strip(theCache[key]) : theCache[key];
		}
	};

	return result;

/*=====
dojo.cache = function(module, url, value){
	// summary:
	//		A getter and setter for storing the string content associated with the
	//		module and url arguments.
	// description:
	//		If module is a string that contains slashes, then it is interpretted as a fully
	//		resolved path (typically a result returned by require.toUrl), and url should not be
	//		provided. This is the preferred signature. If module is a string that does not
	//		contain slashes, then url must also be provided and module and url are used to
	//		call `dojo.moduleUrl()` to generate a module URL. This signature is deprecated.
	//		If value is specified, the cache value for the moduleUrl will be set to
	//		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
	//		in its internal cache and return that cached value for the URL. To clear
	//		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
	//		the URL contents, only modules on the same domain of the page can use this capability.
	//		The build system can inline the cache values though, to allow for xdomain hosting.
	// module: String||Object
	//		If a String with slashes, a fully resolved path; if a String without slashes, the
	//		module name to use for the base part of the URL, similar to module argument
	//		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
	//		generates a valid path for the cache item. For example, a dojo._Url object.
	// url: String
	//		The rest of the path to append to the path derived from the module argument. If
	//		module is an object, then this second argument should be the "value" argument instead.
	// value: String||Object?
	//		If a String, the value to use in the cache for the module/url combination.
	//		If an Object, it can have two properties: value and sanitize. The value property
	//		should be the value to use in the cache, and sanitize can be set to true or false,
	//		to indicate if XML declarations should be removed from the value and if the HTML
	//		inside a body tag in the value should be extracted as the real value. The value argument
	//		or the value property on the value argument are usually only used by the build system
	//		as it inlines cache content.
	//	example:
	//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
	//		of call is used to avoid an issue with the build system erroneously trying to intern
	//		this example. To get the build system to intern your dojo.cache calls, use the
	//		"dojo.cache" style of call):
	//		| //If template.html contains "<h1>Hello</h1>" that will be
	//		| //the value for the text variable.
	//		| var text = dojo["cache"]("my.module", "template.html");
	//	example:
	//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
	//		 (the dojo["cache"] style of call is used to avoid an issue with the build system
	//		erroneously trying to intern this example. To get the build system to intern your
	//		dojo.cache calls, use the "dojo.cache" style of call):
	//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
	//		| //text variable will contain just "<h1>Hello</h1>".
	//		| var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
	//	example:
	//		Same example as previous, but demostrates how an object can be passed in as
	//		the first argument, then the value argument can then be the second argument.
	//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
	//		| //text variable will contain just "<h1>Hello</h1>".
	//		| var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});
	return val; //String
};
=====*/
});


},
'dijit/hccss':function(){
define([
	"require",			// require.toUrl
	"dojo/_base/config", // config.blankGif
	"dojo/dom-class", // domClass.add domConstruct.create domStyle.getComputedStyle
	"dojo/dom-construct", // domClass.add domConstruct.create domStyle.getComputedStyle
	"dojo/dom-style", // domClass.add domConstruct.create domStyle.getComputedStyle
	"dojo/ready", // ready
	"dojo/_base/sniff", // has("ie") has("mozilla")
	"dojo/_base/window" // win.body
], function(require, config, domClass, domConstruct, domStyle, ready, has, win){

	// module:
	//		dijit/hccss
	// summary:
	//		Test if computer is in high contrast mode, and sets dijit_a11y flag on <body> if it is.

	if(has("ie") || has("mozilla")){	// NOTE: checking in Safari messes things up
		// priority is 90 to run ahead of parser priority of 100
		ready(90, function(){
			// summary:
			//		Detects if we are in high-contrast mode or not

			// create div for testing if high contrast mode is on or images are turned off
			var div = domConstruct.create("div",{
				id: "a11yTestNode",
				style:{
					cssText:'border: 1px solid;'
						+ 'border-color:red green;'
						+ 'position: absolute;'
						+ 'height: 5px;'
						+ 'top: -999px;'
						+ 'background-image: url("' + (config.blankGif || require.toUrl("dojo/resources/blank.gif")) + '");'
				}
			}, win.body());

			// test it
			var cs = domStyle.getComputedStyle(div);
			if(cs){
				var bkImg = cs.backgroundImage;
				var needsA11y = (cs.borderTopColor == cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
				if(needsA11y){
					domClass.add(win.body(), "dijit_a11y");
				}
				if(has("ie")){
					div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
				}else{
					win.body().removeChild(div);
				}
			}
		});
	}
});

},
'dojo/dom-form':function(){
define(["./_base/lang", "./dom", "./io-query", "./json"], function(lang, dom, ioq, json){
	// module:
	//		dojo/dom-form
	// summary:
	//		This module defines form-processing functions.

	/*=====
	dojo.fieldToObject = function(inputNode){
		// summary:
		//		Serialize a form field to a JavaScript object.
		// description:
		//		Returns the value encoded in a form field as
		//		as a string or an array of strings. Disabled form elements
		//		and unchecked radio and checkboxes are skipped.	Multi-select
		//		elements are returned as an array of string values.
		// inputNode: DOMNode|String
		// returns: Object
	};
	=====*/

	/*=====
    dojo.formToObject = function(formNode){
        // summary:
        //		Serialize a form node to a JavaScript object.
        // description:
        //		Returns the values encoded in an HTML form as
        //		string properties in an object which it then returns. Disabled form
        //		elements, buttons, and other non-value form elements are skipped.
        //		Multi-select elements are returned as an array of string values.
		// formNode: DOMNode|String
		// returns: Object
        //
        // example:
        //		This form:
        //		|	<form id="test_form">
        //		|		<input type="text" name="blah" value="blah">
        //		|		<input type="text" name="no_value" value="blah" disabled>
        //		|		<input type="button" name="no_value2" value="blah">
        //		|		<select type="select" multiple name="multi" size="5">
        //		|			<option value="blah">blah</option>
        //		|			<option value="thud" selected>thud</option>
        //		|			<option value="thonk" selected>thonk</option>
        //		|		</select>
        //		|	</form>
        //
        //		yields this object structure as the result of a call to
        //		formToObject():
        //
        //		|	{
        //		|		blah: "blah",
        //		|		multi: [
        //		|			"thud",
        //		|			"thonk"
        //		|		]
        //		|	};
    };
	=====*/

	/*=====
    dojo.formToQuery = function(formNode){
        // summary:
        //		Returns a URL-encoded string representing the form passed as either a
        //		node or string ID identifying the form to serialize
		// formNode: DOMNode|String
		// returns: String
    };
	=====*/

	/*=====
    dojo.formToJson = function(formNode, prettyPrint){
        // summary:
        //		Create a serialized JSON string from a form node or string
        //		ID identifying the form to serialize
		// formNode: DOMNode|String
		// prettyPrint: Boolean?
		// returns: String
    };
	=====*/

    function setValue(/*Object*/obj, /*String*/name, /*String*/value){
        // summary:
        //		For the named property in object, set the value. If a value
        //		already exists and it is a string, convert the value to be an
        //		array of values.

        // Skip it if there is no value
        if(value === null){
            return;
        }

        var val = obj[name];
        if(typeof val == "string"){ // inline'd type check
            obj[name] = [val, value];
        }else if(lang.isArray(val)){
            val.push(value);
        }else{
            obj[name] = value;
        }
    }

	var exclude = "file|submit|image|reset|button";

	var form = {
		fieldToObject: function fieldToObject(/*DOMNode|String*/ inputNode){
			var ret = null;
			inputNode = dom.byId(inputNode);
			if(inputNode){
				var _in = inputNode.name, type = (inputNode.type || "").toLowerCase();
				if(_in && type && !inputNode.disabled){
					if(type == "radio" || type == "checkbox"){
						if(inputNode.checked){
							ret = inputNode.value;
						}
					}else if(inputNode.multiple){
						ret = [];
						var nodes = [inputNode.firstChild];
						while(nodes.length){
							for(var node = nodes.pop(); node; node = node.nextSibling){
								if(node.nodeType == 1 && node.tagName.toLowerCase() == "option"){
									if(node.selected){
										ret.push(node.value);
									}
								}else{
									if(node.nextSibling){
										nodes.push(node.nextSibling);
									}
									if(node.firstChild){
										nodes.push(node.firstChild);
									}
									break;
								}
							}
						}
					}else{
						ret = inputNode.value;
					}
				}
			}
			return ret; // Object
		},

		toObject: function formToObject(/*DOMNode|String*/ formNode){
			var ret = {}, elems = dom.byId(formNode).elements;
			for(var i = 0, l = elems.length; i < l; ++i){
				var item = elems[i], _in = item.name, type = (item.type || "").toLowerCase();
				if(_in && type && exclude.indexOf(type) < 0 && !item.disabled){
					setValue(ret, _in, form.fieldToObject(item));
					if(type == "image"){
						ret[_in + ".x"] = ret[_in + ".y"] = ret[_in].x = ret[_in].y = 0;
					}
				}
			}
			return ret; // Object
		},

		toQuery: function formToQuery(/*DOMNode|String*/ formNode){
			return ioq.objectToQuery(form.toObject(formNode)); // String
		},

		toJson: function formToJson(/*DOMNode|String*/ formNode, /*Boolean?*/prettyPrint){
			return json.stringify(form.toObject(formNode), null, prettyPrint ? 4 : 0); // String
		}
	};

    return form;
});

},
'dijit/_Contained':function(){
define([
	"dojo/_base/declare", // declare
	"./registry"	// registry.getEnclosingWidget(), registry.byNode()
], function(declare, registry){

	// module:
	//		dijit/_Contained
	// summary:
	//		Mixin for widgets that are children of a container widget

	return declare("dijit._Contained", null, {
		// summary:
		//		Mixin for widgets that are children of a container widget
		//
		// example:
		// | 	// make a basic custom widget that knows about it's parents
		// |	declare("my.customClass",[dijit._Widget,dijit._Contained],{});

		_getSibling: function(/*String*/ which){
			// summary:
			//      Returns next or previous sibling
			// which:
			//      Either "next" or "previous"
			// tags:
			//      private
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			return node && registry.byNode(node);	// dijit._Widget
		},

		getPreviousSibling: function(){
			// summary:
			//		Returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous"); // dijit._Widget
		},

		getNextSibling: function(){
			// summary:
			//		Returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next"); // dijit._Widget
		},

		getIndexInParent: function(){
			// summary:
			//		Returns the index of this widget within its container parent.
			//		It returns -1 if the parent does not exist, or if the parent
			//		is not a dijit._Container

			var p = this.getParent();
			if(!p || !p.getIndexOfChild){
				return -1; // int
			}
			return p.getIndexOfChild(this); // int
		}
	});
});

},
'com.ibm.bpm.coach/multiBindingListsHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all utilities that Coach runtime supports multiple lists binding

define(["dojo/_base/array", "dojo/dom-class", "./CONSTANTS", "./domHelper", "./dataBindingHelper"], function(array, domClass, CONSTANTS, domHelper, dataBindingHelper) {
				
	var multiBindingListsHelper = new Object();
	
	var _triedToLoadMessage = false;
	
	// returns an array of binding str that represent a list and is different from the binding of specified coachView
	multiBindingListsHelper.analyzeRelatedListBindingStrs = function(coachView) {
		var results = [];		
		if(multiBindingListsHelper.isBindingObjectList(coachView)) {
			var dataBinding = dataBindingHelper.rtDataBinding._get(coachView.context.element);
			var noBracket = dataBinding.substring(0, dataBinding.length -2);	
	    	for(var domId in coachView._domIdToBindingListMap) {
	    		var listStr = coachView._domIdToBindingListMap[domId];
	    		if(noBracket != listStr && array.indexOf(results, listStr) == -1) {
	    			results.push(listStr);
	    		}
	    	}		    	
		}
		
		return results;
	}
	
	multiBindingListsHelper.isBindingObjectList = function(coachView) {
	    if(!!coachView.context.binding) {
			var bindingObject = coachView.context.binding.get("value");
			if(bindingObject instanceof com.ibm.bpm.coach.List) {
				return true;
			}
    	}
    	
    	return false;
	}
	
	
	multiBindingListsHelper.processMonitoredLists = function(coachView, engine) {
		if(!coachView.context.binding) {
			return;
		}
    	var myBindingList = coachView.context.binding.get("value");
    	if(myBindingList instanceof com.ibm.bpm.coach.List) {
    		var innerListShorterThanOuterList = false;
    		var innerListLongerThanOuterList = false;
    		var lengthOfMyBindingList = myBindingList.length();
	    	var monitoredList;
	    	for(var bindingStr in coachView._bindingStrToListMap) {
	    		monitoredList = coachView._bindingStrToListMap[bindingStr];
	    		var monitoredListLength = monitoredList.length(); 
	    		if(monitoredListLength > lengthOfMyBindingList) {
	    			innerListLongerThanOuterList = true;
	    		} else if(monitoredListLength < lengthOfMyBindingList) {
	    			innerListShorterThanOuterList = true;
	    		}
	    	}
	    	
	    	multiBindingListsHelper.clearWarning(coachView);
	    			    	
	    	if(innerListShorterThanOuterList || innerListLongerThanOuterList) {
	    		 multiBindingListsHelper.showWarning(coachView, innerListShorterThanOuterList, innerListLongerThanOuterList, engine);
	    	}
    	}
    }
	    
    multiBindingListsHelper.clearWarning = function(coachView) {
    	var warningDiv = document.getElementById(multiBindingListsHelper.getWarningDivId(coachView));
    	if(!!warningDiv) {
    		warningDiv.parentNode.removeChild(warningDiv);
    	}
    }
    
    multiBindingListsHelper.getWarningDivId = function(coachView) {
    	return coachView.context.element.id + CONSTANTS.COACH_VIEW_WARNING;
    }
    
    multiBindingListsHelper.showWarning = function(coachView, innerListShorterThanOuterList, innerListLongerThanOuterList, engine) {
    	
    	if(!engine.localizedMessages.coachNG_warning_outterListLongerThanInnerList) {
    		if(!_triedToLoadMessage) {
    			engine.forceLocalizedMessages();
    			_triedToLoadMessage = true;
    		} 
    	}
    	var warningDiv = document.createElement("div");
		domClass.add(warningDiv, CONSTANTS.COACH_VIEW_WARNING);
		warningDiv.id = multiBindingListsHelper.getWarningDivId(coachView);
		
		if(innerListShorterThanOuterList) {
			var warningMssage1 = engine.localizedMessages.coachNG_warning_outterListLongerThanInnerList;
			multiBindingListsHelper.appendOneWarning(warningDiv, warningMssage1);
		}
		if(innerListLongerThanOuterList) {
			var warningMssage2 = engine.localizedMessages.coachNG_warning_outterListShorterThanInnerList;
			multiBindingListsHelper.appendOneWarning(warningDiv, warningMssage2);
		}
		
	    coachView.context.element.appendChild(warningDiv);
    }
    
    multiBindingListsHelper.appendOneWarning = function(warningDiv, warningMsg) {
    	var wapperDiv = document.createElement("div");
		var warningTextNode = document.createTextNode(warningMsg);
		wapperDiv.appendChild(warningTextNode);
		warningDiv.appendChild(wapperDiv);
    }
    
	return multiBindingListsHelper;
});
	    
},
'com.ibm.bpm.coach/disableBack':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// disable back/forward buttons and menu item in context menu

define(["dojo/_base/kernel", "dojo/back"], function(kernel, dojoBack) {
/* Looks we don't need this trick any more.
	// Fix IE issue: iframe_history.html checks if parent window has dojo and if dojo has back. unfortunately "back" is not in "dojo" somehow on IE. We know this dojo already has back module, so correct here if needed 
	if(!!window.dojo && !window.dojo.back) {
		window.dojo.back = dojoBack;
	}
*/
	
	var dojoVersion = kernel.version;
	
	// following content are shared between coach and social portal. Just need to copy them to social portal theme html file. 
	// NOTICE: If you change anything here, please sync the change to social portal.
	
	//*************** BEGIN ****************		
	var nextHashSuffix = "  ";
	
	_ApplicationState = function(stateData, bookmarkValue){
		this.stateData = stateData;
		this.changeUrl = bookmarkValue || false;
		
		this.back = function(){
			if(dojoVersion.major == 1 && dojoVersion.minor < 7) {
				// 1.6 or less
				setTimeout(function() {
					_generateHistory();
				}, 1);
			} else {
				// 1.7 or above
				_generateHistory();
			}
		};
		this.forward = function(){
			// do nothing for now. Since testing shows that forward button and menu item always disabled.
		}
	};
	
	function _generateHistory() {
		var baseHashValue = _getCurrentHash();
		var newHashValue = baseHashValue + nextHashSuffix;
		if(nextHashSuffix == " ") {
			nextHashSuffix = "  ";
		} else {
			nextHashSuffix = " ";
		}
		var appState = new _ApplicationState("Fake history", newHashValue);
		dojoBack.addToHistory(appState);
	}
	
	function _getCurrentHash() {
		var url = window.location;
		var hashValue = !!url.hash ? decodeURI(url.hash).substring(1) : "";
		
		var maxSpaces = "   "; // 3 spaces
		if(hashValue.length >= maxSpaces.length) {
			if(hashValue.substring(hashValue.length - maxSpaces.length) == maxSpaces) {
				// remove last few spaces
				hashValue = hashValue.substring(0, hashValue.length - maxSpaces.length);
			}
		} 
		return hashValue;
	}
	
	function initDisableBack() {
		var appState = new _ApplicationState("Initial state");				
		dojoBack.setInitialState(appState);
		
		// IE doesn't work properly if not to running following logic with a delay
		setTimeout(function() {
			_generateHistory();					
		}, 1000);
	}
	//*************** END ****************
			
	var disableBack = new Object();
	disableBack.init = initDisableBack;
	return disableBack;
});
	
},
'com.ibm.bpm.coach/Stateful':function(){
/*
Licensed Materials - Property of IBM
5725-C95
© Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach data stateful class js

define(["dojo/_base/declare",
        "dojo/_base/array",
        "dojo/_base/lang",
        "dojo/_base/json",
        "dojo/Stateful",
        "require",
        "dojox/date/hebrew/Date",
        "dojox/date/islamic/Date",
        "./commonHelper",
        "./dataBindingHelper",
        "./List",
        "./controller",
        "./engine",
        "./logger"
], function(declare, array, lang, json, stateful, require, hebrewDate, islamicDate, commonHelper, dataBindingHelper, list, controller, engine, logger) {
	//Extends dojo.Stateful.  Replaces the "get" method to handle wrapping embedded objects and lists to also make them
	//watchable.  It also caches the watchable objects so that subsequent get operations do not need to wrap new objects.
	return declare("com.ibm.bpm.coach.Stateful", [stateful], {
		//cache of all the child objects and lists
		childrenCache: null,
		_systemCallbackHandle: null,

		//constructor
		constructor: function(baseObject, path) {
			this.inherited(arguments);
			this.childrenCache = {};
			this._objectPath = path;
			this._systemCallbackHandle = null;
			this.enableSystemNotifications();
		},

		disableSystemNotifications: function () {
			this._systemCallbackHandle.unwatch()
		},
		
		enableSystemNotifications: function () {
			//set system call back, for notifications to controller
			this._systemCallbackHandle = this.registerCallback(null, lang.hitch(this, function(prop, oldVal, newVal) {
				    						controller.systemChangeHandler(this._objectPath, prop, newVal);
				    					})
			);			
		},
		
		//overide the super's get to add caching and wrapping of embeded objects and lists
		get: function (name) {
			logger.log("Stateful", "get", "Getting Property: " + this._objectPath + "." + name, logger.FINER);
			var result;
			
			var pathArray = name.split(".");
			if (pathArray.length > 1) {
				result = this.resolvePath(pathArray);
			} else {
				//first check the cache
				var openIndex = name.indexOf("["); 
				if (openIndex != -1) {
					//refers to an array
					var closeIndex = name.indexOf("]");
					logger.log("Stateful", "get", "Getting list index: " + name.substring(openIndex+1, closeIndex), logger.FINER);
					result = this.get(name.substring(0,openIndex)).get(name.substring(openIndex+1, closeIndex));
					return result;
				}
				var property = this.childrenCache[name];
	
				if (typeof(property) == "undefined") {
					var obj = this.inherited(arguments);
					if ((typeof obj == "undefined") || obj == null ){
						return obj;
					}
					//check if it is a simple type
					var dataTypes = ["number", "string", "boolean"];
					array.forEach(dataTypes, function(dataType) {
						if (typeof obj == dataType) {
							result = obj;
						}
					});
	
					if (typeof result == "undefined") {
						//handle a list
						if (Object.prototype.toString.call( obj ) === "[object Array]" || !!obj && !!obj.items && !!obj.listAllSelectedIndices
							//defect 121645 - lists returned from AJAX services use "selected" instead of listAllSelectedIndecies
						    || (!!obj && !!obj["@metadata"] && !!obj["items"] && !!obj["selected"])
						    ) {
							var objectPath = this._objectPath;
							require(["./List"], function(list) {
								result = new list(obj,  objectPath + "." + name);
							});
							//add result to the cache
							this.childrenCache[name] = result;
			        	} 
						else if (typeof obj == "object") {
				        			//handle an object
								if (obj == null || !obj.prototype) { // check that this is a literal
									result = obj;
								} else if (obj.constructor) {
									result = new obj.constructor();
								}
								
								//treat Date like  primitive type
								if (!(result instanceof Date)) {
									result = new com.ibm.bpm.coach.Stateful(result, this._objectPath + "." + name);
			
									//add result to the cache
									this.childrenCache[name] = result;
								}
						}
					} 
					else {
						//handle a simple type
						result = obj;
					}
				} else {
					//cache hit
					result = property;
				}
			}
			return result;
		}, //function get
		
		//Override the super's set to add path
		set: function (name, value, disableSystemNotification, triggeredByConstructor) {
			// dojo Statful calls set() during the execution of its constructor in version 1.8.3.
			// If first parameter (i.e. name) is an object, we know set() is called by the constructor of dojo Stateful,
			if (!!name){			
				// If an object is used, iterate through object
				if(typeof name === "object"){
					for(var x in name){
						if(name.hasOwnProperty(x) && x !="_watchCallbacks"){
							// we know set() is called by the constructor of dojo Stateful
							this.set(x, name[x], true, true);
						}
					}							
					return;
				}
			} else {
				return;
			}		
		
			logger.log("Stateful", "set", "Setting object: " + this._objectPath + "." + name + "= " + value, logger.FINE);
			
			// check if it makes sense to accept the set request
			var currentPlainValue = dataBindingHelper.getPlainValue(this, name);
			if(commonHelper.isValuesEqual(currentPlainValue, value) == commonHelper.YES) {
				// new value is equal to current value, ignore this set request
				logger.log("Stateful", "set", "Ignore this set request because new value is equal to current value. value = " + currentPlainValue, logger.FINE);
				return;
			}
			
			var pathArray = name.split(".");
			if (pathArray.length > 1) {
				name = pathArray.pop();
				var object = this.resolvePath(pathArray);
				if (object instanceof com.ibm.bpm.coach.Stateful) {
					object.set(name, value, disableSystemNotification, triggeredByConstructor);
				}
			} else {
				//defect where a stateful object is being set, need to serialize it and recreate it first
				if (value instanceof com.ibm.bpm.coach.Stateful || value instanceof com.ibm.bpm.coach.List) {
					value = json.fromJson(value.toJson());
				}
			    //test if this is a date and if so turn it into a date object
			    //dates will look like the following yy-mm-ddThh:mm:ss
			    if (/[0-9]{4}(\-[0-9]{2}){2}T([0-9]{2}\:){2}[0-9]{2}/.test(""+value))
			    {
			    	//it is a date
			    	this.set(name, this.stringToDate(value), disableSystemNotification, triggeredByConstructor);
			    }
			    else {
                    // support for Hebrew and Islamic dates
                    if (value instanceof islamicDate || value instanceof hebrewDate) {
                        value = value.toGregorian();
                    }
					
				    var oldWrappedValue = this.childrenCache[name];
				    
				    //check if the value has changed.  If it has not, do nothing. oldWrappedValue is undefined for simple type, the new value could be undefined
				    if ((value instanceof Date) || ((value != oldWrappedValue) || (typeof oldWrappedValue == "undefined")))
				    {				    
						delete this.childrenCache[name];
						if (oldWrappedValue instanceof com.ibm.bpm.coach.List){
							value = oldWrappedValue.duplicate(value);
						}
						if (disableSystemNotification && disableSystemNotification == true) {
							this.disableSystemNotifications();
							this.inherited(arguments);
							this.enableSystemNotifications();
						} else {
							this.inherited(arguments);
						}						
						if(!triggeredByConstructor) {
							// if set() is triggered by constructor, here is what happens:
							// when stateful object "local.a" is constructed, set() is called on "local.a.b" etc. If dataBindingHelper.updatePlainValueInStatefulChain("local.a", "b", valueB).
							// dataBindingHelper.updatePlainValueInStatefulChain() tries to get Stateful object "local.a" by calling get(). Since stateful object "local.a" doesn't exist yet (not fully constructed),
							// another Stateful object for "local.a" starts to construct. As the result, endless recursive call occurs.
							try {
								dataBindingHelper.updatePlainValueInStatefulChain(this._objectPath, name, this[name]);
							}catch(ex) {
	    						var errorMsg = "Stateful.set() caught Exception when calling dataBindingHelper.updatePlainValueInStatefulChain(). _objectPath is " + this._objectPath + ", name is " + name + ", Exception: " + ex;
	    						commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
							}
						}
				   }
			    }
			}
		},

		//walks an array of strings that reprsent an object graph of Stateful objects
		//returns the leaf or null if the path is not valid
		resolvePath: function(objectArray) {
			var result = null;
			if (objectArray) {
				result = this.resolveIndexedProperty(objectArray[0]);					
				
				for (var i=1; i<objectArray.length; i++) {
					if (result instanceof com.ibm.bpm.coach.Stateful) {
						result = result.resolveIndexedProperty(objectArray[i]);
					} else {
						result = null;
						break;
					}
				}
			}
			return result;
		},
		
		resolveIndexedProperty: function (property) {
			var result = null
			var openIndex = property.indexOf("["); 
			if (openIndex != -1) {
				//refers to an array
				var closeIndex = property.indexOf("]");
				result = this.get(property.substring(0,openIndex)).get(property.substring(openIndex+1, closeIndex));
			} else {
				result = this.get(property);					
			}
			return result;
		},

		//registers a callback for this object that will be invoked if this object is modified
		registerCallback: function (property, callback) {
			var handle;
			if (callback) {
				if (property) {
					//com.ibm.bpm.coach.GlobalAssets["watchHandles"].push(this.watch(property, callback));
					handle = this.watch(property, callback);
				} else {
					//com.ibm.bpm.coach.GlobalAssets["watchHandles"].push(this.watch(callback));
					handle =  this.watch(callback);
				}
				handle.unbind = function (){						
					this.unwatch();
				}
			}
			return handle;
		}, 
		
	    bind: function(property, callback, scope) {
	    	var handle;
	    	var obj = this;
	    	if (property) {
				var pathArray = property.split(".");
				if (pathArray.length > 1) {
					property = pathArray.pop();
					obj = this.resolvePath(pathArray);
				}
	    	}    	
			if (obj instanceof com.ibm.bpm.coach.Stateful) {
		    	handle = obj.registerCallback(property, function(object, oldVal, newVal) {
					//Change handler for Stateful objects, calls the subclass change handler
					var event = {property: object, oldVal: oldVal, newVal: newVal};
					try {
						callback.call(scope, event);
					} catch(ex) {
						commonHelper.consoleLog(ex, commonHelper.ERROR);
					}
				});
			}
			return handle;
	    },

	    bindAll: function(callback, scope) {
	    	return this.bind(null, callback, scope);
	    },
	    
		toJson: function() {
			logger.log("Stateful", "toJson", "Serializing object: " + this._objectPath, logger.FINER);
			//first get all of the stateful properties and do a toJson on them and 
			//then print the simple properties
			var excludeList = ["_inherited", "childrenCache", "_objectPath", "declaredClass", "_systemCallbackHandle"];

			var resultList = [];
			for (var x in this.childrenCache) {
                var jsonValue = this.get(x) != null ? this.get(x).toJson() : null;
				resultList.push("\"" + x + "\":" + jsonValue);
				excludeList.push(x);
			}
			for (var x in this) {
				if (typeof this[x] != 'function'){
					// Only need to handle the properties that are owned by this object, ignore the inherited properties.
					if(!this.hasOwnProperty(x)) {
						continue;
					}

					var skipVar = false;
					for (var j=0; j<excludeList.length; j++) {
						if (excludeList[j] == x) {
							skipVar = true;
							break;
						}
					}
					if (!skipVar) {
						if (typeof this[x] == 'object' ||
							Object.prototype.toString.call(this[x] ) === "[object Array]"
							) {
							if (this[x] instanceof Date) {
								var dateString = this.dateToString(this[x]);								
								resultList.push("\"" + x + "\":" + "\"" + dateString + "\"");
							} else {
                                var jsonValue = this.get(x) != null ? this.get(x).toJson() : null;
								resultList.push("\"" + x + "\":" + jsonValue);
							}
						} else {
							resultList.push("\"" + x + "\":" + json.toJson(this[x]));
						}
					}
				}
			}
			var result = "{" + resultList.join(",") + "}"; 
			logger.log("Stateful", "toJson", result, logger.FINER);
			return result;
		},
		
		stringToDate: function (dateStr) {
			//the date string will be of the form yyyy-mm-ddThh:mm:ss
			var datetime = dateStr.split("T");
			var date = datetime[0].split("-");
			var time = datetime[1].split(":");
			var epochValue = Date.UTC(parseInt(date[0]), parseInt(date[1],10)-1, parseInt(date[2],10), parseInt(time[0],10), parseInt(time[1],10), parseInt(time[2],10));
			var result = new Date(epochValue);
			return result;
			//return new Date(parseInt(date[0]), parseInt(date[1])-1, parseInt(date[2]), parseInt(time[0]), parseInt(time[1]), parseInt(time[2]));
		}, 
		
		dateToString: function (localDate) {
			var date = new Date(localDate.getUTCFullYear(), localDate.getUTCMonth(), localDate.getUTCDate(), localDate.getUTCHours(), localDate.getUTCMinutes(), localDate.getUTCSeconds(), localDate.getUTCMilliseconds());
            
            var yearString = null;
            var yearInt = parseInt(date.getFullYear());
            if (yearInt < 10) {
                yearString = "000" + yearInt
            } else if (yearInt < 100) {
                yearString = "00" + yearInt
            } else if (yearInt < 1000) {
                yearString = "0" + yearInt
            } else {
                yearString = "" + yearInt;
            }
            
			var monthString = null;
			var monthInt = parseInt(date.getMonth()) +1;
			(monthInt <10) ? monthString = "0" + monthInt : monthString = "" + monthInt;

            var dayString = null;
            var dayInt = parseInt(date.getDate());
            (dayInt <10) ? dayString = "0" + dayInt : dayString = "" + dayInt;

            var hourString = null;
            var hourInt = parseInt(date.getHours());
            (hourInt <10) ? hourString = "0" + hourInt : hourString = "" + hourInt;
                        
            var minString = null;
            var minInt = parseInt(date.getMinutes());
            (minInt <10) ? minString = "0" + minInt : minString = "" + minInt;

            var secString = null;
            var secInt = parseInt(date.getSeconds());
            (secInt <10) ? secString = "0" + secInt : secString = "" + secInt;
            
            var millisecString = null;
            var millisecInt = parseInt(date.getMilliseconds());
            (millisecInt < 10) ? 
                millisecString = "00" + millisecInt : 
                (millisecInt < 100) ? 
                    millisecString = "0" + millisecInt : 
                    millisecString = "" + millisecInt;
                                    
            var dateString = yearString + '-' + monthString + '-' + dayString;
            dateString += "T" + hourString + ':' + minString + ':' + secString + "." + millisecString;
            
            return dateString;      
        }

  	}); //com.ibm.bpm.coach.Stateful	
});


},
'dojo/store/util/SimpleQueryEngine':function(){
define(["../../_base/array"], function(arrayUtil) {
  //  module:
  //    dojo/store/util/SimpleQueryEngine
  //  summary:
  //    The module defines a simple filtering query engine for object stores. 

return function(query, options){
	// summary:
	//		Simple query engine that matches using filter functions, named filter
	//		functions or objects by name-value on a query object hash
	//
	// description:
	//		The SimpleQueryEngine provides a way of getting a QueryResults through
	//		the use of a simple object hash as a filter.  The hash will be used to
	//		match properties on data objects with the corresponding value given. In
	//		other words, only exact matches will be returned.
	//
	//		This function can be used as a template for more complex query engines;
	//		for example, an engine can be created that accepts an object hash that
	//		contains filtering functions, or a string that gets evaluated, etc.
	//
	//		When creating a new dojo.store, simply set the store's queryEngine
	//		field as a reference to this function.
	//
	// query: Object
	//		An object hash with fields that may match fields of items in the store.
	//		Values in the hash will be compared by normal == operator, but regular expressions
	//		or any object that provides a test() method are also supported and can be
	// 		used to match strings by more complex expressions
	// 		(and then the regex's or object's test() method will be used to match values).
	//
	// options: dojo.store.util.SimpleQueryEngine.__queryOptions?
	//		An object that contains optional information such as sort, start, and count.
	//
	// returns: Function
	//		A function that caches the passed query under the field "matches".  See any
	//		of the "query" methods on dojo.stores.
	//
	// example:
	//		Define a store with a reference to this engine, and set up a query method.
	//
	//	|	var myStore = function(options){
	//	|		//	...more properties here
	//	|		this.queryEngine = dojo.store.util.SimpleQueryEngine;
	//	|		//	define our query method
	//	|		this.query = function(query, options){
	//	|			return dojo.store.util.QueryResults(this.queryEngine(query, options)(this.data));
	//	|		};
	//	|	};

	// create our matching query function
	switch(typeof query){
		default:
			throw new Error("Can not query with a " + typeof query);
		case "object": case "undefined":
			var queryObject = query;
			query = function(object){
				for(var key in queryObject){
					var required = queryObject[key];
					if(required && required.test){
						if(!required.test(object[key])){
							return false;
						}
					}else if(required != object[key]){
						return false;
					}
				}
				return true;
			};
			break;
		case "string":
			// named query
			if(!this[query]){
				throw new Error("No filter function " + query + " was found in store");
			}
			query = this[query];
			// fall through
		case "function":
			// fall through
	}
	function execute(array){
		// execute the whole query, first we filter
		var results = arrayUtil.filter(array, query);
		// next we sort
		if(options && options.sort){
			results.sort(function(a, b){
				for(var sort, i=0; sort = options.sort[i]; i++){
					var aValue = a[sort.attribute];
					var bValue = b[sort.attribute];
					if (aValue != bValue) {
						return !!sort.descending == aValue > bValue ? -1 : 1;
					}
				}
				return 0;
			});
		}
		// now we paginate
		if(options && (options.start || options.count)){
			var total = results.length;
			results = results.slice(options.start || 0, (options.start || 0) + (options.count || Infinity));
			results.total = total;
		}
		return results;
	}
	execute.matches = query;
	return execute;
};
});

},
'dojo/parser':function(){
define(
	["./_base/kernel", "./_base/lang", "./_base/array", "./_base/config", "./_base/html", "./_base/window", "./_base/url",
	 	"./_base/json", "./aspect", "./date/stamp", "./has", "./query", "./on", "./ready"],
	function(dojo, dlang, darray, config, dhtml, dwindow, _Url, djson, aspect, dates, has, query, don, ready){

// module:
//		dojo/parser
// summary:
//		The Dom/Widget parsing package

new Date("X"); // workaround for #11279, new Date("") == NaN

if (1) {
	var form = document.createElement("form");
	// Test if DOMNode.attributes only lists the attributes the user specified, not attributes w/default values.
	has.add("dom-attributes-explicit", form.attributes.length == 0);

	// IE8 will erroneously list a few attributes that weren't specified,
	// but we know to skip them because they have a specified flag which is false
	has.add("dom-attributes-specified-flag", form.attributes.length < 40);

	// Otherwise, it's IE6-7 form.attributes will list hundreds of values, need to do outerHTML instead.
}

dojo.parser = new function(){
	// summary:
	//		The Dom/Widget parsing package

	var _nameMap = {
		// Map from widget name (ex: "dijit.form.Button") to structure mapping
		// lowercase version of attribute names to the version in the widget ex:
		//	{
		//		label: "label",
		//		onclick: "onClick"
		//	}
	};
	function getNameMap(proto){
		// summary:
		//		Returns map from lowercase name to attribute name in class, ex: {onclick: "onClick"}
		var map = {};
		for(var name in proto){
			if(name.charAt(0)=="_"){ continue; }	// skip internal properties
			map[name.toLowerCase()] = name;
		}
		return map;
	}
	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	aspect.after(dlang, "extend", function(){
		_nameMap = {};
	}, true);

	// Map from widget name (ex: "dijit.form.Button") to a map of { "list-of-mixins": ctor }
	// if "list-of-mixins" is "__type" this is the raw type without mixins
	var _ctorMap = {};


	function getCtor(type){
		var map = _ctorMap[type] || (_ctorMap[type] = {});
		return map["__type"] || (map["__type"] = (dlang.getObject(type) || require(type)));
	}

	this._functionFromScript = function(script, attrData){
		// summary:
		//		Convert a <script type="dojo/method" args="a, b, c"> ... </script>
		//		into a function
		// script: DOMNode
		//		The <script> DOMNode
		// attrData: String
		//		For HTML5 compliance, searches for attrData + "args" (typically
		//		"data-dojo-args") instead of "args"
		var preamble = "";
		var suffix = "";
		var argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args"));
		if(argsStr){
			darray.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			darray.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	};

	this.instantiate = /*====== dojo.parser.instantiate= ======*/ function(nodes, mixin, options) {
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of DOM nodes
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// options: Object?
		//		An object used to hold kwArgs for instantiation.
		//		See parse.options argument for details.

		mixin = mixin || {};
		options = options || {};

		var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (options.scope || dojo._scopeName) + "-",// typically "data-dojo-"
			dataDojoType = attrData + "type";						// typically "data-dojo-type"

		var list = [];
		darray.forEach(nodes, function(node){
			var type = dojoType in mixin ? mixin[dojoType] : node.getAttribute(dataDojoType) || node.getAttribute(dojoType);
			if(type){
				list.push({
					node: node,
					"type": type
				});
			}
		});

		// Instantiate the nodes and return the objects
		return this._instantiate(list, mixin, options);
	};

	this._instantiate = /*====== dojo.parser.instantiate= ======*/ function(nodes, mixin, options){
		// summary:
		//		Takes array of objects representing nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// options: Object
		//		An options object used to hold kwArgs for instantiation.
		//		See parse.options argument for details.

		var thelist = [];

		// Precompute names of special attributes we are looking for
		// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
		var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (options.scope || dojo._scopeName) + "-",// typically "data-dojo-"
			dataDojoType = attrData + "type",						// typically "data-dojo-type"
			dataDojoProps = attrData + "props",						// typically "data-dojo-props"
			dataDojoAttachPoint = attrData + "attach-point",
			dataDojoAttachEvent = attrData + "attach-event",
			dataDojoId = attrData + "id",
			dataDojoMixins = attrData + "mixins";

		// And make hash to quickly check if a given attribute is special, and to map the name to something friendly
		var specialAttrs = {};
		darray.forEach([dataDojoProps, dataDojoType, dojoType, dataDojoId, "jsId", dataDojoAttachPoint,
				dataDojoAttachEvent, "dojoAttachPoint", "dojoAttachEvent", "class", "style", dataDojoMixins], function(name){
			specialAttrs[name.toLowerCase()] = name.replace(options.scope, "dojo");
		});

		function extend(type, mixins){
			return type.createSubclass && type.createSubclass(mixins) || type.extend.apply(type, mixins);
		}

		darray.forEach(nodes, function(obj){
			if(!obj){ return; }

			var node = obj.node,
				type = obj.type,
				mixins = node.getAttribute(dataDojoMixins), ctor;

			if(mixins){
				var map = _ctorMap[type];
				// remove whitespaces
				mixins = mixins.replace(/ /g, "");
				ctor = map && map[mixins];
				if(!ctor){
					// first get ctor for raw type (& create _ctorMap[type] if needed (should not be))
					ctor = getCtor(type);
					// then do the mixin
					ctor = _ctorMap[type][mixins] = extend(ctor, darray.map(mixins.split(","), getCtor));
				}
			}else{
				ctor = getCtor(type);
			}

			var proto = ctor && ctor.prototype;

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};

			if(options.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				dlang.mixin(params, options.defaults);
			}
			if(obj.inherited){
				// settings from dir=rtl or lang=... on a node above this node
				dlang.mixin(params, obj.inherited);
			}

			// Get list of attributes explicitly listed in the markup
			var attributes;
			if(has("dom-attributes-explicit")){
				// Standard path to get list of user specified attributes
				attributes = node.attributes;
			}else if(has("dom-attributes-specified-flag")){
				// Special processing needed for IE8, to skip a few faux values in attributes[]
				attributes = darray.filter(node.attributes, function(a){ return a.specified;});
			}else{
				// Special path for IE6-7, avoid (sometimes >100) bogus entries in node.attributes
				var clone = /^input$|^img$/i.test(node.nodeName) ? node : node.cloneNode(false),
					attrs = clone.outerHTML.replace(/=[^\s"']+|="[^"]*"|='[^']*'/g, "").replace(/^\s*<[a-zA-Z0-9]*\s*/, "").replace(/\s*>.*$/, "");

				attributes = darray.map(attrs.split(/\s+/), function(name){
					var lcName = name.toLowerCase();
					return {
						name: name,
						// getAttribute() doesn't work for button.value, returns innerHTML of button.
						// but getAttributeNode().value doesn't work for the form.encType or li.value
						value: (node.nodeName == "LI" && name == "value") || lcName == "enctype" ?
								node.getAttribute(lcName) : node.getAttributeNode(lcName).value
					};
				});
			}

			// Read in attributes and process them, including data-dojo-props, data-dojo-type,
			// dojoAttachPoint, etc., as well as normal foo=bar attributes.
			var i=0, item;
			while(item = attributes[i++]){
				var name = item.name,
					lcName = name.toLowerCase(),
					value = item.value;

				if(lcName in specialAttrs){
					switch(specialAttrs[lcName]){

					// Data-dojo-props.   Save for later to make sure it overrides direct foo=bar settings
					case "data-dojo-props":
						var extra = value;
						break;

					// data-dojo-id or jsId. TODO: drop jsId in 2.0
					case "data-dojo-id":
					case "jsId":
						var jsname = value;
						break;

					// For the benefit of _Templated
					case "data-dojo-attach-point":
					case "dojoAttachPoint":
						params.dojoAttachPoint = value;
						break;
					case "data-dojo-attach-event":
					case "dojoAttachEvent":
						params.dojoAttachEvent = value;
						break;

					// Special parameter handling needed for IE
					case "class":
						params["class"] = node.className;
						break;
					case "style":
						params["style"] = node.style && node.style.cssText;
						break;
					}
				}else{
					// Normal attribute, ex: value="123"

					// Find attribute in widget corresponding to specified name.
					// May involve case conversion, ex: onclick --> onClick
					if(!(name in proto)){
						var map = (_nameMap[type] || (_nameMap[type] = getNameMap(proto)));
						name = map[lcName] || name;
					}

					// Set params[name] to value, doing type conversion
					if(name in proto){
						switch(typeof proto[name]){
						case "string":
							params[name] = value;
							break;
						case "number":
							params[name] = value.length ? Number(value) : NaN;
							break;
						case "boolean":
							// for checked/disabled value might be "" or "checked".	 interpret as true.
							params[name] = value.toLowerCase() != "false";
							break;
						case "function":
							if(value === "" || value.search(/[^\w\.]+/i) != -1){
								// The user has specified some text for a function like "return x+5"
								params[name] = new Function(value);
							}else{
								// The user has specified the name of a function like "myOnClick"
								// or a single word function "return"
								params[name] = dlang.getObject(value, false) || new Function(value);
							}
							break;
						default:
							var pVal = proto[name];
							params[name] =
								(pVal && "length" in pVal) ? (value ? value.split(/\s*,\s*/) : []) :	// array
									(pVal instanceof Date) ?
										(value == "" ? new Date("") :	// the NaN of dates
										value == "now" ? new Date() :	// current date
										dates.fromISOString(value)) :
								(pVal instanceof dojo._Url) ? (dojo.baseUrl + value) :
								djson.fromJson(value);
						}
					}else{
						params[name] = value;
					}
				}
			}

			// Mix things found in data-dojo-props into the params, overriding any direct settings
			if(extra){
				try{
					extra = djson.fromJson.call(options.propsThis, "{" + extra + "}");
					dlang.mixin(params, extra);
				}catch(e){
					// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
					throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
				}
			}

			// Any parameters specified in "mixin" override everything else.
			dlang.mixin(params, mixin);

			var scripts = obj.scripts || (ctor && (ctor._noScript || proto._noScript) ? [] :
						query("> script[type^='dojo/']", node));

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" data-dojo-event="foo"> tags are dojo.connected after instantiation
			// <script type="dojo/watch" data-dojo-prop="foo"> tags are dojo.watch after instantiation
			// <script type="dojo/on" data-dojo-event="foo"> tags are dojo.on after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var connects = [],	// functions to connect after instantiation
				calls = [],		// functions to call after instantiation
				watch = [],  //functions to watch after instantiation
				on = []; //functions to on after instantiation

			if(scripts){
				for(i=0; i<scripts.length; i++){
					var script = scripts[i];
					node.removeChild(script);
					// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
					var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
						prop = script.getAttribute(attrData + "prop"),
						scriptType = script.getAttribute("type"),
						nf = this._functionFromScript(script, attrData);
					if(event){
						if(scriptType == "dojo/connect"){
							connects.push({event: event, func: nf});
						}else if(scriptType == "dojo/on"){
							on.push({event: event, func: nf});
						}else{
							params[event] = nf;
						}
					}else if(scriptType == "dojo/watch"){
						watch.push({prop: prop, func: nf});
					}else{
						calls.push(nf);
					}
				}
			}

			// create the instance
			var markupFactory = ctor.markupFactory || proto.markupFactory;
			var instance = markupFactory ? markupFactory(params, node, ctor) : new ctor(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			if(jsname){
				dlang.setObject(jsname, instance);
			}

			// process connections and startup functions
			for(i=0; i<connects.length; i++){
				aspect.after(instance, connects[i].event, dojo.hitch(instance, connects[i].func), true);
			}
			for(i=0; i<calls.length; i++){
				calls[i].call(instance);
			}
			for(i=0; i<watch.length; i++){
				instance.watch(watch[i].prop, watch[i].func);
			}
			for(i=0; i<on.length; i++){
				don(instance, on[i].event, on[i].func);
			}
		}, this);

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			darray.forEach(thelist, function(instance){
				if( !options.noStart && instance  &&
					dlang.isFunction(instance.startup) &&
					!instance._started
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};

	this.scan = /*====== dojo.parser.scan= ======*/ function(root, options){
		// summary:
		//		Scan a DOM tree and return an array of objects representing the DOMNodes
		//		that need to be turned into widgets.
		// description:
		//		Search specified node (or document root node) recursively for class instances
		//		and return an array of objects that represent potential widgets to be
		//		instantiated. Searches for either data-dojo-type="MID" or dojoType="MID" where
		//		"MID" is a module ID like "dijit/form/Button" or a fully qualified Class name
		//		like "dijit.form.Button".
		//
		//		See parser.parse() for details of markup.
		// root: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `options`
		//		object can be passed in this place. If the `options` object has a
		//		`rootNode` member, that is used.
		// options: Object
		//		a kwArgs options object, see parse() for details

		// Output list
		var list = [];

		var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (options.scope || dojo._scopeName) + "-",	// typically "data-dojo-"
			dataDojoType = attrData + "type",						// typically "data-dojo-type"
			dataDojoTextDir = attrData + "textdir";					// typically "data-dojo-textdir"

		// Info on DOMNode currently being processed
		var node = root.firstChild;

		// Info on parent of DOMNode currently being processed
		//	- inherited: dir, lang, and textDir setting of parent, or inherited by parent
		//	- parent: pointer to identical structure for my parent (or null if no parent)
		//	- scripts: if specified, collects <script type="dojo/..."> type nodes from children
		var inherited = options.inherited;
		if(!inherited){
			function findAncestorAttr(node, attr){
				return (node.getAttribute && node.getAttribute(attr)) ||
					(node !== dwindow.doc && node !== dwindow.doc.documentElement && node.parentNode ? findAncestorAttr(node.parentNode, attr) : null);
			}
			inherited = {
				dir: findAncestorAttr(root, "dir"),
				lang: findAncestorAttr(root, "lang"),
				textDir: findAncestorAttr(root, dataDojoTextDir)
			};
			for(var key in inherited){
				if(!inherited[key]){ delete inherited[key]; }
			}
		}
		var parent = {
			inherited: inherited
		};

		// For collecting <script type="dojo/..."> type nodes (when null, we don't need to collect)
		var scripts;

		// when true, only look for <script type="dojo/..."> tags, and don't recurse to children
		var scriptsOnly;

		function getEffective(parent){
			// summary:
			//		Get effective dir, lang, textDir settings for specified obj
			//		(matching "parent" object structure above), and do caching.
			//		Take care not to return null entries.
			if(!parent.inherited){
				parent.inherited = {};
				var node = parent.node,
					grandparent = getEffective(parent.parent);
				var inherited  = {
					dir: node.getAttribute("dir") || grandparent.dir,
					lang: node.getAttribute("lang") || grandparent.lang,
					textDir: node.getAttribute(dataDojoTextDir) || grandparent.textDir
				};
				for(var key in inherited){
					if(inherited[key]){
						parent.inherited[key] = inherited[key];
					}
				}
			}
			return parent.inherited;
		}

		// DFS on DOM tree, collecting nodes with data-dojo-type specified.
		while(true){
			if(!node){
				// Finished this level, continue to parent's next sibling
				if(!parent || !parent.node){
					break;
				}
				node = parent.node.nextSibling;
				scripts = parent.scripts;
				scriptsOnly = false;
				parent = parent.parent;
				continue;
			}

			if(node.nodeType != 1){
				// Text or comment node, skip to next sibling
				node = node.nextSibling;
				continue;
			}

			if(scripts && node.nodeName.toLowerCase() == "script"){
				// Save <script type="dojo/..."> for parent, then continue to next sibling
				type = node.getAttribute("type");
				if(type && /^dojo\/\w/i.test(type)){
					scripts.push(node);
				}
				node = node.nextSibling;
				continue;
			}
			if(scriptsOnly){
				node = node.nextSibling;
				continue;
			}

			// Check for data-dojo-type attribute, fallback to backward compatible dojoType
			var type = node.getAttribute(dataDojoType) || node.getAttribute(dojoType);

			// Short circuit for leaf nodes containing nothing [but text]
			var firstChild = node.firstChild;
			if(!type && (!firstChild || (firstChild.nodeType == 3 && !firstChild.nextSibling))){
				node = node.nextSibling;
				continue;
			}

			// Setup data structure to save info on current node for when we return from processing descendant nodes
			var current = {
				node: node,
				scripts: scripts,
				parent: parent
			};

			// If dojoType/data-dojo-type specified, add to output array of nodes to instantiate
			// Note: won't find classes declared via dojo.Declaration, so use try/catch to avoid throw from require()
			// We don't care yet about mixins ctors, we check script stop only on main class
			var ctor;
			try{
				ctor = type && getCtor(type);
			}catch(e){
			}
			var childScripts = ctor && !ctor.prototype._noScript ? [] : null; // <script> nodes that are parent's children
			if(type){
				list.push({
					"type": type,
					node: node,
					scripts: childScripts,
					inherited: getEffective(current) // dir & lang settings for current node, explicit or inherited
				});
			}

			// Recurse, collecting <script type="dojo/..."> children, and also looking for
			// descendant nodes with dojoType specified (unless the widget has the stopParser flag).
			// When finished with children, go to my next sibling.
			node = firstChild;
			scripts = childScripts;
			scriptsOnly = ctor && ctor.prototype.stopParser && !(options.template);
			parent = current;
		}

		return list;
	};

	this.parse = /*====== dojo.parser.parse= ======*/ function(rootNode, options){
		// summary:
		//		Scan the DOM for class instances, and instantiate them.
		//
		// description:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them. Searches for either data-dojo-type="Class" or
		//		dojoType="Class" where "Class" is a a fully qualified class name,
		//		like `dijit.form.Button`
		//
		//		Using `data-dojo-type`:
		//		Attributes using can be mixed into the parameters used to instantiate the
		//		Class by using a `data-dojo-props` attribute on the node being converted.
		//		`data-dojo-props` should be a string attribute to be converted from JSON.
		//
		//		Using `dojoType`:
		//		Attributes are read from the original domNode and converted to appropriate
		//		types by looking up the Class prototype values. This is the default behavior
		//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
		//		go away in Dojo 2.0.
		//
		// rootNode: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `options`
		//		object can be passed in this place. If the `options` object has a
		//		`rootNode` member, that is used.
		//
		// options: Object?
		//		A hash of options.
		//
		//			* noStart: Boolean?
		//				when set will prevent the parser from calling .startup()
		//				when locating the nodes.
		//			* rootNode: DomNode?
		//				identical to the function's `rootNode` argument, though
		//				allowed to be passed in via this `options object.
		//			* template: Boolean
		//				If true, ignores ContentPane's stopParser flag and parses contents inside of
		//				a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
		//				nested inside the ContentPane to work.
		//			* inherited: Object
		//				Hash possibly containing dir and lang settings to be applied to
		//				parsed widgets, unless there's another setting on a sub-node that overrides
		//			* scope: String
		//				Root for attribute names to search for.   If scopeName is dojo,
		//				will search for data-dojo-type (or dojoType).   For backwards compatibility
		//				reasons defaults to dojo._scopeName (which is "dojo" except when
		//				multi-version support is used, when it will be something like dojo16, dojo20, etc.)
		//			* propsThis: Object
		//				If specified, "this" referenced from data-dojo-props will refer to propsThis.
		//				Intended for use from the widgets-in-template feature of `dijit._WidgetsInTemplateMixin`
		//
		// example:
		//		Parse all widgets on a page:
		//	|		dojo.parser.parse();
		//
		// example:
		//		Parse all classes within the node with id="foo"
		//	|		dojo.parser.parse(dojo.byId('foo'));
		//
		// example:
		//		Parse all classes in a page, but do not call .startup() on any
		//		child
		//	|		dojo.parser.parse({ noStart: true })
		//
		// example:
		//		Parse all classes in a node, but do not call .startup()
		//	|		dojo.parser.parse(someNode, { noStart:true });
		//	|		// or
		//	|		dojo.parser.parse({ noStart:true, rootNode: someNode });

		// determine the root node and options based on the passed arguments.
		var root;
		if(!options && rootNode && rootNode.rootNode){
			options = rootNode;
			root = options.rootNode;
		}else if(rootNode && dlang.isObject(rootNode) && !("nodeType" in rootNode)){
			options = rootNode;
		}else{
			root = rootNode;
		}
		root = root ? dhtml.byId(root) : dwindow.body();

		options = options || {};

		// List of all nodes on page w/dojoType specified
		var list = this.scan(root, options);

		// go build the object instances
		var mixin = options.template ? {template: true} : {};
		return this._instantiate(list, mixin, options); // Array
	};
}();


//Register the parser callback. It should be the first callback
//after the a11y test.
if(config.parseOnLoad){
	ready(100, dojo.parser, "parse");
}

return dojo.parser;
});

},
'dijit/form/_FormMixin':function(){
define([
	"dojo/_base/array", // array.every array.filter array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/window" // winUtils.scrollIntoView
], function(array, declare, kernel, lang, winUtils){

	// module:
	//		dijit/form/_FormMixin
	// summary:
	//		Mixin for containers of form widgets (i.e. widgets that represent a single value
	//		and can be children of a <form> node or dijit.form.Form widget)

	return declare("dijit.form._FormMixin", null, {
		// summary:
		//		Mixin for containers of form widgets (i.e. widgets that represent a single value
		//		and can be children of a <form> node or dijit.form.Form widget)
		// description:
		//		Can extract all the form widgets
		//		values and combine them into a single javascript object, or alternately
		//		take such an object and set the values for all the contained
		//		form widgets

	/*=====
		// value: Object
		//		Name/value hash for each child widget with a name and value.
		//		Child widgets without names are not part of the hash.
		//
		//		If there are multiple child widgets w/the same name, value is an array,
		//		unless they are radio buttons in which case value is a scalar (since only
		//		one radio button can be checked at a time).
		//
		//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
		//
		//		Example:
		//	|	{ name: "John Smith", interests: ["sports", "movies"] }
	=====*/

		// state: [readonly] String
		//		Will be "Error" if one or more of the child widgets has an invalid value,
		//		"Incomplete" if not all of the required child widgets are filled in.  Otherwise, "",
		//		which indicates that the form is ready to be submitted.
		state: "",

		//	TODO:
		//	* Repeater
		//	* better handling for arrays.  Often form elements have names with [] like
		//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])
		//
		//

		_getDescendantFormWidgets: function(/*dijit._WidgetBase[]?*/ children){
			// summary:
			//		Returns all form widget descendants, searching through non-form child widgets like BorderContainer
			var res = [];
			array.forEach(children || this.getChildren(), function(child){
				if("value" in child){
					res.push(child);
				}else{
					res = res.concat(this._getDescendantFormWidgets(child.getChildren()));
				}
			}, this);
			return res;
		},

		reset: function(){
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//		provides a few additional (ui-specific) features.
			//		1 - it will highlight any sub-widgets that are not
			//			valid
			//		2 - it will call focus() on the first invalid
			//			sub-widget
			var didFocus = false;
			return array.every(array.map(this._getDescendantFormWidgets(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					winUtils.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
	 			return valid;
	 		}), function(item){ return item; });
		},

		setValues: function(val){
			kernel.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*Object*/ obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by get('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = lang.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				if(!lang.isArray(values)){
					values = [ values ];
				}
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					array.forEach(widgets, function(w){
						w.set('value', array.indexOf(values, w.value) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					array.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			array.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call set('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							array.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						array.forEach(element.options, function(option){
							option.selected = array.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						array.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/

			// Note: no need to call this._set("value", ...) as the child updates will trigger onChange events
			// which I am monitoring.
		},

		getValues: function(){
			kernel.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			// 		Returns Object representing form values.   See description of `value` for details.
			// description:

			// The value is updated into this.value every time a child has an onChange event,
			// so in the common case this function could just return this.value.   However,
			// that wouldn't work when:
			//
			// 1. User presses return key to submit a form.  That doesn't fire an onchange event,
			// and even if it did it would come too late due to the setTimeout(..., 0) in _handleOnChange()
			//
			// 2. app for some reason calls this.get("value") while the user is typing into a
			// form field.   Not sure if that case needs to be supported or not.

			// get widget values
			var obj = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget)
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							lang.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = lang.getObject(name, false, obj);
							if(value === undefined){
								lang.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=lang.getObject(name, false, obj);
						if(!ary){
							ary=[];
							lang.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=lang.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(lang.isArray(prev)){
							prev.push(value);
						}else{
							lang.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						lang.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also domForm.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			array.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					}else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					}else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					}else{
						// can not set value when there is no name
					}
				}else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				}else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

	 	isValid: function(){
	 		// summary:
	 		//		Returns true if all of the widgets are valid.
			//		Deprecated, will be removed in 2.0.  Use get("state") instead.

			return this.state == "";
		},

		onValidStateChange: function(/*Boolean*/ /*===== isValid =====*/){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
			//
			//		Deprecated.  Will be removed in 2.0.  Use watch("state", ...) instead.
		},

		_getState: function(){
			// summary:
			//		Compute what this.state should be based on state of children
			var states = array.map(this._descendants, function(w){
				return w.get("state") || "";
			});

			return array.indexOf(states, "Error") >= 0 ? "Error" :
				array.indexOf(states, "Incomplete") >= 0 ? "Incomplete" : "";
		},

		disconnectChildren: function(){
			// summary:
			//		Remove connections to monitor changes to children's value, error state, and disabled state,
			//		in order to update Form.value and Form.state.
			array.forEach(this._childConnections || [], lang.hitch(this, "disconnect"));
			array.forEach(this._childWatches || [], function(w){ w.unwatch(); });
		},

		connectChildren: function(/*Boolean*/ inStartup){
			// summary:
			//		Setup connections to monitor changes to children's value, error state, and disabled state,
			//		in order to update Form.value and Form.state.
			//
			//		You can call this function directly, ex. in the event that you
			//		programmatically add a widget to the form *after* the form has been
			//		initialized.

			var _this = this;

			// Remove old connections, if any
			this.disconnectChildren();

			this._descendants = this._getDescendantFormWidgets();

			// (Re)set this.value and this.state.   Send watch() notifications but not on startup.
			var set = inStartup ? function(name, val){ _this[name] = val; } : lang.hitch(this, "_set");
			set("value", this.get("value"));
			set("state", this._getState());

			// Monitor changes to error state and disabled state in order to update
			// Form.state
			var conns = (this._childConnections = []),
				watches = (this._childWatches = []);
			array.forEach(array.filter(this._descendants,
				function(item){ return item.validate; }
			),
			function(widget){
				// We are interested in whenever the widget changes validity state - or
				// whenever the disabled attribute on that widget is changed.
				array.forEach(["state", "disabled"], function(attr){
					watches.push(widget.watch(attr, function(){
						_this.set("state", _this._getState());
					}));
				});
			});

			// And monitor calls to child.onChange so we can update this.value
			var onChange = function(){
				// summary:
				//		Called when child's value or disabled state changes

				// Use setTimeout() to collapse value changes in multiple children into a single
				// update to my value.   Multiple updates will occur on:
				//	1. Form.set()
				//	2. Form.reset()
				//	3. user selecting a radio button (which will de-select another radio button,
				//		 causing two onChange events)
				if(_this._onChangeDelayTimer){
					clearTimeout(_this._onChangeDelayTimer);
				}
				_this._onChangeDelayTimer = setTimeout(function(){
					delete _this._onChangeDelayTimer;
					_this._set("value", _this.get("value"));
				}, 10);
			};
			array.forEach(
				array.filter(this._descendants, function(item){ return item.onChange; } ),
				function(widget){
					// When a child widget's value changes,
					// the efficient thing to do is to just update that one attribute in this.value,
					// but that gets a little complicated when a checkbox is checked/unchecked
					// since this.value["checkboxName"] contains an array of all the checkboxes w/the same name.
					// Doing simple thing for now.
					conns.push(_this.connect(widget, "onChange", onChange));

					// Disabling/enabling a child widget should remove it's value from this.value.
					// Again, this code could be more efficient, doing simple thing for now.
					watches.push(widget.watch("disabled", onChange));
				}
			);
		},

		startup: function(){
			this.inherited(arguments);

			// Initialize value and valid/invalid state tracking.  Needs to be done in startup()
			// so that children are initialized.
			this.connectChildren(true);

			// Make state change call onValidStateChange(), will be removed in 2.0
			this.watch("state", function(attr, oldVal, newVal){ this.onValidStateChange(newVal == ""); });
		},

		destroy: function(){
			this.disconnectChildren();
			this.inherited(arguments);
		}

	});
});

},
'dojo/aspect':function(){
define([], function(){

// TODOC: after/before/around return object
// TODOC: after/before/around param types. 

/*=====
	dojo.aspect = {
		// summary: provides aspect oriented programming functionality, allowing for
		//		one to add before, around, or after advice on existing methods.
		//
		// example:
		//	|	define(["dojo/aspect"], function(aspect){
		//	|		var signal = aspect.after(targetObject, "methodName", function(someArgument){
		//	|			this will be called when targetObject.methodName() is called, after the original function is called
		//	|		});
		//
		// example:
		//	The returned signal object can be used to cancel the advice.
		//	|	signal.remove(); // this will stop the advice from being executed anymore
		//	|	aspect.before(targetObject, "methodName", function(someArgument){
		//	|		// this will be called when targetObject.methodName() is called, before the original function is called
		//	|	 });
		
		after: function(target, methodName, advice, receiveArguments){
			// summary: The "after" export of the aspect module is a function that can be used to attach
			//		"after" advice to a method. This function will be executed after the original method
			//		is executed. By default the function will be called with a single argument, the return
			//		value of the original method, or the the return value of the last executed advice (if a previous one exists).
			//		The fourth (optional) argument can be set to true to so the function receives the original
			//		arguments (from when the original method was called) rather than the return value.
			//		If there are multiple "after" advisors, they are executed in the order they were registered.
			// target: Object
			//		This is the target object
			// methodName: String
			//		This is the name of the method to attach to.
			// advice: Function
			//		This is function to be called after the original method
			// receiveArguments: Boolean?
			//		If this is set to true, the advice function receives the original arguments (from when the original mehtod
			//		was called) rather than the return value of the original/previous method.
			// returns:
			//		A signal object that can be used to cancel the advice. If remove() is called on this signal object, it will
			//		stop the advice function from being executed.
		},
		
		before: function(target, methodName, advice){
			// summary: The "before" export of the aspect module is a function that can be used to attach
			//		"before" advice to a method. This function will be executed before the original method
			//		is executed. This function will be called with the arguments used to call the method.
			//		This function may optionally return an array as the new arguments to use to call
			//		the original method (or the previous, next-to-execute before advice, if one exists).
			//		If the before method doesn't return anything (returns undefined) the original arguments
			//		will be preserved.
			//		If there are multiple "before" advisors, they are executed in the reverse order they were registered.
			//
			// target: Object
			//		This is the target object
			// methodName: String
			//		This is the name of the method to attach to.
			// advice: Function
			//		This is function to be called before the original method	 
		},

		around: function(target, methodName, advice){
			// summary: The "around" export of the aspect module is a function that can be used to attach
			//		"around" advice to a method. The advisor function is immediately executed when
			//		the around() is called, is passed a single argument that is a function that can be
			//		called to continue execution of the original method (or the next around advisor).
			//		The advisor function should return a function, and this function will be called whenever
			//		the method is called. It will be called with the arguments used to call the method.
			//		Whatever this function returns will be returned as the result of the method call (unless after advise changes it).
			//
			// example:
			//		If there are multiple "around" advisors, the most recent one is executed first,
			//		which can then delegate to the next one and so on. For example:
			//		|	around(obj, "foo", function(originalFoo){
			//		|		return function(){
			//		|			var start = new Date().getTime();
			//		|			var results = originalFoo.apply(this, arguments); // call the original
			//		|			var end = new Date().getTime();
			//		|			console.log("foo execution took " + (end - start) + " ms");
			//		|			return results;
			//		|		};
			//		|	});
			//
			// target: Object
			//		This is the target object
			// methodName: String
			//		This is the name of the method to attach to.
			// advice: Function
			//		This is function to be called around the original method
		}

	};
=====*/

	"use strict";
	var nextId = 0;
	function advise(dispatcher, type, advice, receiveArguments){
		var previous = dispatcher[type];
		var around = type == "around";
		var signal;
		if(around){
			var advised = advice(function(){
				return previous.advice(this, arguments);
			});
			signal = {
				remove: function(){
					if(advised){
						advised = dispatcher = advice = null;
					}
				},
				advice: function(target, args){
					return advised ?
						advised.apply(target, args) :  // called the advised function
						previous.advice(target, args); // cancelled, skip to next one
				}
			};
		}else{
			// create the remove handler
			signal = {
				remove: function(){
					if(signal.advice){
						var previous = signal.previous;
						var next = signal.next;
						if(!next && !previous){
							delete dispatcher[type];
						}else{
							if(previous){
								previous.next = next;
							}else{
								dispatcher[type] = next;
							}
							if(next){
								next.previous = previous;
							}
						}

						// remove the advice to signal that this signal has been removed
						dispatcher = advice = signal.advice = null;
					}
				},
				id: nextId++,
				advice: advice,
				receiveArguments: receiveArguments
			};
		}
		if(previous && !around){
			if(type == "after"){
				// add the listener to the end of the list
				// note that we had to change this loop a little bit to workaround a bizarre IE10 JIT bug
				while(previous.next && (previous = previous.next)){}
				previous.next = signal;
				signal.previous = previous;
			}else if(type == "before"){
				// add to beginning
				dispatcher[type] = signal;
				signal.next = previous;
				previous.previous = signal;
			}
		}else{
			// around or first one just replaces
			dispatcher[type] = signal;
		}
		return signal;
	}
	function aspect(type){
		return function(target, methodName, advice, receiveArguments){
			var existing = target[methodName], dispatcher;
			if(!existing || existing.target != target){
				// no dispatcher in place
				target[methodName] = dispatcher = function(){
					var executionId = nextId;
					// before advice
					var args = arguments;
					var before = dispatcher.before;
					while(before){
						args = before.advice.apply(this, args) || args;
						before = before.next;
					}
					// around advice
					if(dispatcher.around){
						var results = dispatcher.around.advice(this, args);
					}
					// after advice
					var after = dispatcher.after;
					while(after && after.id < executionId){
						results = after.receiveArguments ? after.advice.apply(this, args) || results :
								after.advice.call(this, results);
						after = after.next;
					}
					return results;
				};
				if(existing){
					dispatcher.around = {advice: function(target, args){
						return existing.apply(target, args);
					}};
				}
				dispatcher.target = target;
			}
			var results = advise((dispatcher || existing), type, advice, receiveArguments);
			advice = null;
			return results;
		};
	}
	return {
		before: aspect("before"),
		around: aspect("around"),
		after: aspect("after")
	};
});

},
'com.ibm.bpm.coach/collaborationBanner':function(){
require({cache:{
'url:com.ibm.bpm.coach/templates/CollaborationBanner.html':"<div class='collaboration-banner'>\n\t<table width=\"100%\">\n\t\t<tbody>\n\t\t\t<tr valign=\"center\" nowrap=\"nowrap\" height=\"30px\" width=\"100%\">\n\t\t\t\t<td nowrap=\"nowrap\">\n\t\t\t\t\t<span dojoattachpoint=\"title_area_icon\"></span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\">\t\t\n\t\t\t\t\t<span tabindex=\"0\" class='collaboration-title-area-icon-label' dojoattachpoint=\"title_area_label\"></span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\">\n\t\t\t\t\t<span role=\"wairole:alert\" tabindex=\"0\" dojoAttachPoint='participants'>${workingWithLabel} </span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\">\n\t\t\t\t\t<span class='collaboration-title-area-participants-avatar' dojoAttachPoint='partnerAvatar'>\t\t\t\t\t\n\t                   <div class=\"\">\t\t              \n\t\t                  <img class=\"collaboration-collaboratorImg\"  dojoattachpoint=\"partnerAvatarImage\" alt=\"User Avatar\" src=\"/portal/skins/Default/images/Avatar_16x16.png\">\t\t\n\t                   </div>\n                       <div class=\"collaboration-onlineAwarenessIcon\">\n                           <div class=\"collaboration-onlineAwarenessIconA11y\">&#9744</div>\n                       </div>  \n\t\t\t\t\t</span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\">\n\t\t\t\t\t<span role=\"wairole:alert\" tabindex=\"0\" class=\"collaboration-title-area-participants-fullName\" dojoattachpoint=\"partnerUserName\">${partnerFullName}</span>\n\t\t\t\t</td>\n\t\t\t\t<td nowrap=\"nowrap\" width=\"100%\">\n\t\t\t\t\t<div class=\"collaboration-controls collaboration-controls-link-style\" dojoattachpoint=\"controls\">\n\t\t\t\t\t\t<a tabindex=\"0\" id=\"makeEditorLink\" dojoattachpoint=\"makeEditorLink\" class=\"collaboration-controls-span collaboration-show\" href=\"javascript:\">${makeEditorLabel}</a>\n\t\t\t\t\t\t<a tabindex=\"0\" id=\"takeControlLink\" dojoattachpoint=\"takeControlLink\" class=\"collaboration-controls-span collaboration-hidden\" href=\"javascript:\">${takeControlLabel}</a>\n\t\t\t\t\t<div>\n\t\t\t\t</td>\n\t\t\t</tr>\n\t\t</tbody>\n\t</table>\n</div>"}});
/*
Licensed Materials - Property of IBM
5725-C95
© Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

define(['dojo/_base/kernel', 
		"dojo/_base/declare", // declare
		"dijit/_Widget",
		"dijit/_Templated",
		"dojo/string",
		'dojo/io/script', 
		"dojo/_base/lang", 
		"dojo/_base/xhr",
		"dojo/_base/fx",
		"dojo/_base/array",
		"dojo/_base/connect",
		"dojo/dom-style",
		"dojo/dom-class",
		"dojo/dom-construct",
		"./engine",
		'dojo/_base/unload',
		'require',
		// baader: the template includes the online awareness icon for the avatar, because per definition collaboration
		//         is only possible with an online user. So for performance reasons no extra check is integrated. When ever
		//         off-line collaboration is implemented, that needs to change and the online tracking needs to be integrated. 
		"dojo/text!./templates/CollaborationBanner.html",
		"./logger", "dojo/_base/config"],
		function(dojo, declare, _Widget, _Templated, dojoString, ioScript, lang, xhr, fx, array, connect, domStyle, domClass, domConstruct, engine, baseUnload, require, template, logger, dojoConfig) {

return declare(
	"com.ibm.bpm.coach.collaborationBanner",
	[_Widget, _Templated],
	{
	
    templateString: template,
	
	containerDivId : "socialPortalCollaborationBanner",
	
    message: null,

	eventParam: null,
		
	_container: null,
	
	fade: true,
	
	isEditor: false,
	
	isTaskOwner: false,
	
	partnerFullName : "Unknown User",
	
	partnerUsername : "unknownUser",
	
	makeEditorCallBack: null,
	
	takeControlCallBack: null,
	
	_collaboration_connections: [],
	
	userAvatar : undefined,
	
	// workingWithLabel:"You are working on the task with",
	// editModeLabel:"Edit Mode:",
	// viewModeLabel:"View Mode:",
	
	workingWithLabel:"You are working with",
	editModeLabel:"Edit Mode:",
	viewModeLabel:"View Mode:",
	makeEditorLabel:"Make Editor",
	takeControlLabel:"Task Over Control",
	
	constructor: function(params){
			
		var msgs = this.getEngineMessages(params.partnerFullName);
		
		this.workingWithLabel = msgs.workingWithLabel;
		this.editModeLabel = msgs.editModeLabel;
		this.viewModeLabel = msgs.viewModeLabel;
		this.makeEditorLabel = msgs.makeEditorLabel;
		this.takeControlLabel = msgs.takeControlLabel;
		
		this.createContainer();
			
	},
	
	createContainer: function(){
		
		if(window.parent["collaborationContainerDiv"]){
			this.container = window.parent["collaborationContainerDiv"];
			this.container.innerHTML = "";
		} else {
			logger.log("collaborationBanner", "createContainer", "Error: socialPortalCollaborationBanner Container not found.", logger.FINE);
		}
	},
	
    postCreate: function() {	
		this._collaboration_connections.push(connect.connect(this.makeEditorLink, 'onclick', this, "_makeEditor"));
		this._collaboration_connections.push(connect.connect(this.takeControlLink, 'onclick', this, "_takeControl"));

		
		this.userAvatar = this.getUserAvatar(this.partnerUsername);
		if(this.userAvatar)
			this.partnerAvatarImage.src = "data:image/" + this.userAvatar.imageFormat + ";base64," + this.userAvatar.userAvatarImage;	
		
		/*
		this.workingWithLabel= engine.collab_workingWithLabel;
		this.editModeLabel = engine.collab_editModeLabel;
		this.viewModeLabel = engine.collab_viewModeLabel;
		*/
		//attach events
		if(this.isTaskOwner){
			if(this.isEditor){
			
				//toggle controls
				this.makeEditorLink.className = "collaboration-controls-span collaboration-show";
				this.takeControlLink.className = "collaboration-controls-span collaboration-hidden";
				
				//Set title area
				this.title_area_icon.className = "collaboration-title-area-icon-edit";
				this.title_area_label.innerHTML= this.editModeLabel;
				
			}else{
				this.makeEditorLink.className = "collaboration-controls-span collaboration-hidden";
				this.takeControlLink.className = "collaboration-controls-span collaboration-show";
				
				this.title_area_icon.className = "collaboration-title-area-icon-readOnly";
				this.title_area_label.innerHTML= this.viewModeLabel;
			}		
		}else{		
			if(this.isEditor){
				this.makeEditorLink.className = "collaboration-controls-span collaboration-show";
				this.takeControlLink.className = "collaboration-controls-span collaboration-hidden";
				
				this.title_area_icon.className = "collaboration-title-area-icon-edit";
				this.title_area_label.innerHTML=this.editModeLabel;
			}else{
				this.makeEditorLink.className = "collaboration-controls-span collaboration-hidden";
				this.takeControlLink.className = "collaboration-controls-span collaboration-hidden";
				
				this.title_area_icon.className = "collaboration-title-area-icon-readOnly";
				this.title_area_label.innerHTML=this.viewModeLabel;
			}		
		
		}
		
		this.makeEditorLink.innerHTML = this.makeEditorLabel;
		domStyle.set(this.domNode, 'opacity', 0);
		domConstruct.place(this.domNode, this.container);
		fx.anim(this.domNode, {opacity: 1}, 250);
		
		try { 
			domClass.add(this.container.parentNode.parentNode, "bpm_c12n"); 
		} catch (e) { 
			//TODO If this fails in IE, we can try window.top.document.body
		};
	},
	
	getEngineMessages : function(partnerFullName){
		var msgs = {};
		var pFullName = partnerFullName;
		
		require(["./engine","dojo/string","dojo/_base/json"], function(engine, dojoString, json) {
			if(engine&&engine.localizedMessages){
                // forcing to load messages if they are not yet initialized
                engine.forceLocalizedMessages();

				msgs.workingWithLabel = engine.localizedMessages.collab_workingWithLabel;
				msgs.editModeLabel = engine.localizedMessages.collab_editModeLabel;
				msgs.viewModeLabel = engine.localizedMessages.collab_viewModeLabel;
				logger.log("collaborationBanner", "getEngineMessages", json.toJson(dojoString), logger.FINE);
				
				if(pFullName){
					msgs.makeEditorLabel = dojoString.substitute(engine.localizedMessages.collab_makeEditorLabel,[pFullName]);
					if(msgs.makeEditorLabel)
						msgs.makeEditorLabel = msgs.makeEditorLabel.replace("{0}",pFullName);
				}
				msgs.takeControlLabel = engine.localizedMessages.collab_takeControl;
			}
		});
		
		return msgs;
	
	},
	
	switchToViewerMode : function(flag){
		if(flag){
			this.makeEditorLink.className = "collaboration-controls-span collaboration-hidden";			
			this.title_area_icon.className = "collaboration-title-area-icon-readOnly";
			this.title_area_label.innerHTML = this.viewModeLabel;
		}else{
			this.makeEditorLink.className = "collaboration-controls-span collaboration-show";
			this.title_area_icon.className = "collaboration-title-area-icon-edit";
			this.title_area_label.innerHTML = this.editModeLabel;
		}
		if(this.participants)
			this.participants.focus();
	},
	
	hideTakeControlButton : function(flag){
		//enable all Buttons
		if(flag)
			this.takeControlLink.className = "collaboration-controls-span collaboration-hidden";
		else
			this.takeControlLink.className = "collaboration-controls-span collaboration-show";
		if(this.participants)
			this.participants.focus();
	},
	
	_makeEditor: function(e){
		/*implement by the parent*/
		makeEditorCallBack();
	},
	
	_takeControl: function(e){
		/*implement by the parent*/
		takeControlCallBack();
	},
	
	
	fireEvent:function(){
		//	parent defined
		logger.log("collaborationBanner", "fireEvent", "This function needs to be implemented by the parent.", logger.FINE);
	},

	_handleActionClick:function(event){		
		this.fireEvent(event.target.id,this.eventParam);
	},

	getUserAvatar : function(username){		
	  if (username == null) {
	    return null;
	  }

	 var avatar = this.userAvatar;
	  if (!this.userAvatar){
	    xhr.get({
	      url: dojoConfig.App._bpmContextRootMap.rest + "/v1/avatar/" + encodeURIComponent(username),
	      handleAs: "json",
	      sync: true,
	      headers: {
	        "Content-Type": "application/json",
	        "Accept": "application/json"
	      },
	      load: function(data, xhrArgs){
				avatar ={};
				avatar = data.data;
				avatar.inProgress = false;
	      },
	      error: function(data, xhrArgs){
	    	  switch(xhrArgs.xhr.status){
				// In the case of a 404, assume no avatar is defined for this user and
				// don't treat as an error since the default avatar will be displayed.
				case 404:
					avatar = null;	
					break;
				default:
					logger.log("collaborationBanner", "getUserAvatar", "REST call loading avatar image caused ERROR '" + error + "'", logger.FINE);
			}
	      }
	    });
	  }
	  return avatar;
	},

	cleanCollaborationBannerConnections:function(){
		array.forEach(this._collaboration_connections, connect.disconnect);
	},
	close: function(){
		fx.anim(this.domNode,{opacity: 0}, 500, null, lang.hitch(this, 'remove'));
	},
	
	remove: function(){
		try{ 
			domClass.remove(this.container.parentNode.parentNode, "bpm_c12n"); 
		} catch (e) {
			//TODO If this fails in IE, we can try window.top.document.body
		}
		
		this.cleanCollaborationBannerConnections();
		fx.anim(this.domNode,{height: 0, margin: 0}, 250, null,lang.partial(domConstruct.destroy, this.domNode));
		if(this.domNode){
			domConstruct.destroy(this.domNode);
		}
		//TODO Look at destroying container if last notification
	}
	
});


});

},
'dojo/topic':function(){
define(["./Evented"], function(Evented){
	// summary:
	//		The export of this module is a pubsub hub
	//		You can also use listen function itself as a pub/sub hub:
	//		| 	topic.subscribe("some/topic", function(event){
	//		|	... do something with event
	//		|	});
	//		|	topic.publish("some/topic", {name:"some event", ...});

	var hub = new Evented;
	return {
		publish: function(topic, event){
			// summary:
			//		Publishes a message to a topic on the pub/sub hub. All arguments after
			// 		the first will be passed to the subscribers, so any number of arguments
			// 		can be provided (not just event).
			// topic: String
			//		The name of the topic to publish to
			// event: Object
			//		An event to distribute to the topic listeners
			return hub.emit.apply(hub, arguments);
		},
		subscribe: function(topic, listener){
			// summary:
			//		Subcribes to a topic on the pub/sub hub
			// topic: String
			//		The topic to subscribe to
			//	listener: Function
			//		A function to call when a message is published to the given topic
			return hub.on.apply(hub, arguments);
		}
	}
});

},
'com.ibm.bpm.coach/engine':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach runtime engine main js

define(["dojo/parser", "dojo/_base/json", "dojo/_base/lang", "dojo/json", "dojo/_base/array", "dojo/query", "dojo/dom", "dojo/dom-class", "dojo/_base/xhr", "dojo/_base/connect", "./BPM_CONSTANTS", "./CONSTANTS", "./commonHelper", "./broadcastHelper", "./domHelper", "./dataBindingHelper", "./validationHelper", "./coachViewHelper", "./Stateful", "./Shimmer", "./logger", "./ViewContext", "./templateStore", "./bindingResolver"], function(parser, json, lang, json1, array, query, dom, domClass, xhr, connect, BPM_CONSTANTS, CONSTANTS, commonHelper, broadcastHelper, domHelper, dataBindingHelper, validationHelper, coachViewHelper, stateful, Shimmer, logger, ViewContext, templateStore, bindingResolver) {
	var engine = new Object();

	// Main modules for the framework starts
	
	// backward compatibility support
	engine.CONSTANTS = CONSTANTS;
	
	engine.GlobalAssets = {"topLevelViewScope": {}, "coach": null, "coachClass": null, "scopeClasses": {}, "watchHandles": [], "local" : {}, "epv": {},
					  "team": {"member": [], "manager": []}, "system": {}, "systemData": {}, "resource": {}, "options": {}, boundaryEventData: {} };
	
	engine.localizedMessages = {};
	
	engine._isCoachUnloaded = false;
	
	engine.isProcessingValidation = false;
	
	engine._internalUnloadHandler = [];
	
	engine.start = function(config) {
		// print out the dojo version, which may help view developer and support since the dojo version used by coach framework is not fixed per release.
		commonHelper.consoleLog("dojo version used by Coach is: " + commonHelper.getDojoVersion(), commonHelper.INFO);
		
		logger.init();
		
		engine.initLocalizedMessage();
		engine.initData.call(engine.GlobalAssets);
		templateStore.init(engine.GlobalAssets);
		bindingResolver.init(engine.GlobalAssets);
		engine.initConfig.call(engine.GlobalAssets);
		
		// reference to controller
		engine.controller = config.controller;
		engine.controller.init();
		
		// expose public APIs that are available globally
		engine.exposeGlobalAPIs();		
		engine.initViews.call(engine.GlobalAssets);
		engine.prepareView.call(engine.GlobalAssets);
		engine.addMessageListeners(config.whiteList);
		engine.completeInitialization();
	}
	
	engine.initLocalizedMessage = function() {
		commonHelper.timestamp("Enter engine.initLocalizedMessage()");
		logger.log("engine", "initLocalizedMessage", "is called", logger.INFO);
		var jsonResURL = "jsonResBundleLoader?bundle=JavaScriptMessages";
		
		xhr.get( {
		    url: jsonResURL,
		    handleAs: "json",
		    load: function(responseObject, ioArgs) {
		    	engine.localizedMessages = responseObject;		
				logger.log("engine", "initLocalizedMessage", "localizedMessages: " + engine.localizedMessages);
		    }
		});
	};

    // Loads localized messages in synchronous fashion if only they are not yet initialized.
    // This function is required to avoid timing issues in collaboration scenarios.
    engine.forceLocalizedMessages = function() {
        logger.log("engine", "forceLocalizedMessages", "is called", logger.INFO);
        var jsonResURL = "jsonResBundleLoader?bundle=JavaScriptMessages";

        var initialized = false;
        // check if messages are empty
        for (var prop in engine.localizedMessages) {
            if (engine.localizedMessages.hasOwnProperty(prop)) {
                initialized = true;
                break;
            }
        }
        // if messages are empty we need to pull them in a sync way
        if (initialized === false) {
            xhr.get( {
                url: jsonResURL,
                handleAs: "json",
                sync: true,
                load: function(responseObject, ioArgs) {
                    engine.localizedMessages = responseObject;
                    logger.log("engine", "forceLocalizedMessages", "localizedMessages: " + engine.localizedMessages);
                }
            });
        }
    }

    engine._getViewScope = function(parentViewScope, viewDomNode) {
		var viewDivId = viewDomNode.getAttribute("id");
		var viewId = viewDomNode.getAttribute(engine.CONSTANTS.VIEW_ID);
    	return parentViewScope.context.subview[viewId][viewDivId];
    }	    
    
    engine.exposeGlobalAPIs = function() {
    	// global object that provides access to functions that are globally available
    	com_ibm_bpm_global = {};
    	
    	/*
    		This property is available globally. It  holds the list of top level coach view instances in current window object.
    	*/
		com_ibm_bpm_global.topLevelCoachViews = engine.GlobalAssets.topLevelViewScope;
		
		// Note: this global API is not documented yet. Only let customer use it when it is really necessary.
		com_ibm_bpm_global.preference = {};
		com_ibm_bpm_global.preference.highlightUnresolvedBinding = true;
		
		com_ibm_bpm_global.coachView = {};
		
		/*
			This function is available globally. It returns the coach view instance that is described 
			by the input parameter "controlIds" which is a list of control ids.   
			The list of control ids should completely specify the view hierarchy, starting from the top level view, 
			all the way down to the coach view whose instance is expected to be returned.
			
			If  the list of control id doesn't describe the entire view hierarchy properly, this function returns null.
			
			If a coach view repeats, first instance of that view is chose to continue.
			
			For example, let's say we have following view hierarchy: 
				ViewA (Top Level View, control id is  id_ViewA)
				    ViewB  (control id is id_ViewB, binds to list)
				        ContentBox
				           ViewC (control id is id_ViewC)
			Then, 
			If controlIds is ["id_ViewA"], the return is the instance of ViewA.
			If controlIds is ["id_ViewA", "id_ViewB"], the return is the instance of ViewB.
			If controlIds is ["id_ViewA", "id_ViewB", "id_ViewC"], the return is the first instance of ViewC. 
				If ViewC doesn't have any instance under ViewB (e.g. the binding list of ViewB is empty), the return is null.	
			If controlIds is ["id_ViewB"], the return is null, because input parameter doesn't start with the control id of top level view.
			If controlIds is ["id_ViewA", "id_ViewC"]) returns null, because the control id of ViewB  is not on the input list.
		*/
		com_ibm_bpm_global.coachView.byControlId = function(controlIds) {
			return coachViewHelper.findFirstViewInstance(engine.GlobalAssets.topLevelViewScope, controlIds);
		}		
		
		/*
			This function is available globally. It returns the coach view instance that is uniquely described by the input parameter "domId". 
			If specified domId is not the id of a coach view instance's DOM node, this function returns null.
		*/
		com_ibm_bpm_global.coachView.byDomId = function(domId) {
			return coachViewHelper.findViewScope(domId, engine.GlobalAssets.topLevelViewScope);
		}
		
    }
	    
	
	engine.initViews = function() {
		commonHelper.timestamp("Enter engine.initViews()");
		logger.log("engine", "initViews", "is called", logger.INFO);
		
		// find out all top level views
		var topLevelViewList = query('#mainBody .' + engine.CONSTANTS.TOP_LEVEL);
		array.forEach(topLevelViewList, function(item, index) {
			var topLevelviewId = item.getAttribute(engine.CONSTANTS.VIEW_ID);
			commonHelper.timestamp("Call engine.initView() for top level view " + topLevelviewId);
			var oneTopLevelView = engine.initView.apply(this, [item, null]);
			commonHelper.timestamp("Finish engine.initView() for top level view " + topLevelviewId);
			this.topLevelViewScope[topLevelviewId] = oneTopLevelView;
		}, this);
		engine._isCoachUnloaded = false;
		
		// If no view has data binding, engine.GlobalAssets.local is not initialized yet.
		// So initialize it here accordingly
		if (engine.GlobalAssets.local == undefined || engine.GlobalAssets.local == null ||
		    !(engine.GlobalAssets.local instanceof com.ibm.bpm.coach.Stateful)) {
			engine.GlobalAssets.local = new stateful(engine.GlobalAssets.local, "local");
		}
		
	    var CoachClass = this.coachClass;
	    if(!!CoachClass) {
			try {			
				this.coach = new CoachClass();
		    } catch(ex) {
		    	var errorMsg = "engine.initViews() caught Exception when calling view constructor of Coach, Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
				return;
		    }
			try {			
				this.coach.load();
		    } catch(ex1) {
		    	var errorMsg1 = "engine.initViews() caught Exception when calling load() on Coach object, Exception: " + ex1;
		    	commonHelper.consoleLog(errorMsg1, commonHelper.ERROR);
				return;
		    }
	    }	    
		

		commonHelper.timestamp("Exit engine.initViews()");
	};

	 
	/*
	 * Start of configuration data handling
	 *
	 */
	 
	/**
	 * This function is used by code gen team to pass in the needed information to runtime 
	 * It is for the static value case, ie, user defined an option, the configuration value is static: type in by the user
	 */
	engine.resolveStaticOptionValue = function(configObjId, optionKey, optionValue){
	
	    //use the config object id and the option key to construct a unique key
		var optionValue = {staticOptionStr : "config." +  configObjId + "." + optionKey, value: optionValue };
		if(optionKey === "_metadata.visibility" || optionKey === "_metadata.labelVisibility" ){
			optionValue.isStandardValue = true;
		}
		return optionValue;
	}

	/**
	 * This function is used by code gen team to pass in the needed information to runtime 
	 * User defined an option, the configuration value for this option points to a bo or points to another option
	 * engine.resolveOptionValue.call(this, configId, "_metadata.visibility", "local.dataObject02"),
	 * engine.resolveOptionValue.call(this, configId, "_metadata.visibility", "config.config1._metadata.visibility"),
	 */
	engine.resolveOptionValue = function(configObjId, optionKey, optionBindingStr){
		var optionValue;
		if (!!optionBindingStr && optionBindingStr.indexOf("config.") == 0)
		{
			optionValue = {staticOptionStr : optionBindingStr};
		}
		else{
			//for bo case
			optionValue = {bindingStr : optionBindingStr};
		}
		return optionValue;
	}

	/**
	 * This function is used to get the value of an option
	 */
	engine.getOptionValue = function(configId, optionKey, isMetadata){
		var options = this.GlobalAssets.options[configId];
		if (!options){
			return;
		}
		//fix for defect 80359. we modified the template config object here so the change is limited to minor code path
		var optionValue, value;
		if (!!isMetadata){
			optionValue = options._metadata[optionKey];
		}
		else {
			optionValue = options[optionKey];
		}
		if (!!optionValue && (!!optionValue.staticOptionStr)){
			value = optionValue.value;
		}
		var binding = bindingResolver._resolveOptionBinding(options, optionKey, isMetadata);
		// for non-staticOptionStr cases, vlaue is still undefined
		if(!!value) {
			binding.value = value;
		}
		return !!binding ? binding.get("value") : null;
	}
	
	/* 
	 * This is for repeatting case
     * We first clone the config object
	 * If an option points to something under an array, then the index in binding will be adjusted
	 * If an option's value is a non bo object, we adjust the staticOptionStr to use the new config object id
	 * 
	 * static value      object                own: replace the configId portion of the staticOptionStr with the new configId
	 *                   function              same
	 * refrence to 
	 * 			local(non-array)         same
	 *          local(array)             points to the right index in array
	 *          
	 * reference to  static              same
	 *               static array        same array object
	 *               function            same
	 */
	engine.cloneAndAdjustDataConfig = function(viewElement, bindingIndex, maxNumOfUnresolvedRepeating, templateConfigIdToInstanceConfigIdMap){
		
		var tempViewCfgBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_CONFIG);
		if(!tempViewCfgBinding){
			return;
		}
	    var configObj = engine.GlobalAssets.options[tempViewCfgBinding];
		var clonedObjId = "config_" + viewElement.id;
		
		//clone the object and store it under GlobalAssets.options
		var clonedObj = {"_metadata" : {}, "templateConfigId" : tempViewCfgBinding, "configId": clonedObjId};
		
		// add info of newly cloned config to templateConfigIdToInstanceConfigIdMap
		engine._processOneConfig(templateConfigIdToInstanceConfigIdMap, clonedObj);
		
		var configValue;
		var bindingStr;
		var metaDataOptionValue;
		for(var configKey in configObj) {
			configValue = configObj[configKey];
		    
			if (configKey == "_metadata"){
			for(var metadataKey in configValue){				
				metaDataOptionValue = configObj[configKey][metadataKey];
				if (!metaDataOptionValue){
					continue;
				}
				//check if it points to an bo
				if (configValue[metadataKey].bindingStr){	
					bindingStr = configValue[metadataKey].bindingStr;
					//adjust index if needed
					bindingStr = dataBindingHelper.updateBindingStr(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating); 
					clonedObj[configKey][metadataKey] = {"bindingStr" : bindingStr };
					
				}
				//for static value case
				else if(!!configValue[metadataKey].staticOptionStr){
						// defect 96461
						var staticOptionStr1 = configValue[metadataKey].staticOptionStr;
						//adjust index if needed
						staticOptionStr1 = dataBindingHelper.updateBindingStr(staticOptionStr1, bindingIndex, maxNumOfUnresolvedRepeating); 

						// if metaDataOptionValue has been resolved, call its getter to get staticOptionValue, otherwise, use metaDataOptionValue.value as staticOptionValue
						var staticOptionValue = metaDataOptionValue.value;
						if(metaDataOptionValue.boundObject && metaDataOptionValue.get) {
							staticOptionValue = metaDataOptionValue.get("value");
						}
						clonedObj[configKey][metadataKey] =
                        {staticOptionStr: staticOptionStr1,
						 value: staticOptionValue
						}
						
						// following logic handles 2 scenarios: 
						// a) clonedObj[configKey][metadataKey].staticOptionStr is "config.configABC....". 
						//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ"), and we have added configABC --> configXYZ to templateConfigIdToInstanceConfigIdMap.
						//    Now it's time to update the config reference (replacing "configABC" with "configXYZ".
						// b) clonedObj[configKey][metadataKey].staticOptionStr is "config.configDEF....", 
						//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ").
						//    Now we need to check if configDEF has been cloned earlier, in the scope of from currentView up to the topLevelView. If so, replace "configDEF" with its cloned instance "configLMN". 
						clonedObj[configKey][metadataKey].staticOptionStr = engine._updateStaticOptionStr(staticOptionStr1, templateConfigIdToInstanceConfigIdMap);
				}
				else{
					clonedObj[configKey][metadataKey] = metaDataOptionValue;
				}
			}
			}
			else{
				if(!configValue){
					continue;
				}
				//check if it points to an object
				if (!!configValue.bindingStr){	
					bindingStr = configValue.bindingStr;
					//adjust index if needed
					bindingStr = dataBindingHelper.updateBindingStr(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating); 
					clonedObj[configKey] = {"bindingStr" : bindingStr };
				}
				//static value which maybe shared	
				else if (!!configValue.staticOptionStr) {
					var staticOptionStr1 = configValue.staticOptionStr;
					//adjust index if needed
					staticOptionStr1 = dataBindingHelper.updateBindingStr(staticOptionStr1, bindingIndex, maxNumOfUnresolvedRepeating); 
				
                   var optionValue = configValue.value;
                    if(configValue.boundObject && configValue.get) {
                        optionValue = configValue.get("value");
                    }
                    clonedObj[configKey] = {value: optionValue};
					
					// following logic handles 2 scenarios: 
					// a) staticOptionStr is "config.configABC....". 
					//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ"), and we have added configABC --> configXYZ to templateConfigIdToInstanceConfigIdMap.
					//    Now it's time to update the config reference (replacing "configABC" with "configXYZ".
					// b) staticOptionStr is "config.configDEF....", 
					//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ").
					//    Now we need to check if configDEF has been cloned earlier, in the scope of from currentView up to the topLevelView. If so, replace "configDEF" with its cloned instance "configLMN". 
					clonedObj[configKey].staticOptionStr = engine._updateStaticOptionStr(staticOptionStr1, templateConfigIdToInstanceConfigIdMap);
				}
				else{//simply copy over the value
					clonedObj[configKey] = 	configValue;			
				}
			}
		}	 
		
		engine.GlobalAssets.options[clonedObjId] = clonedObj;
		
		//update the view    
		viewElement.setAttribute(engine.CONSTANTS.DATA_CONFIG, clonedObjId);
	}
	
	engine._updateStaticOptionStr = function(staticOtionStr, templateConfigIdToInstanceConfigIdMap) {
		var pathArray = staticOtionStr.split(".");
		var referencedConfigId = pathArray[1];
		var newReferencedConfigId = templateConfigIdToInstanceConfigIdMap[referencedConfigId];
		if(!!newReferencedConfigId) {
			pathArray[1] = newReferencedConfigId;
			return pathArray.join(".");
		}
		
		return staticOtionStr;
	}
	
	// return the plain value represented by specified binding object (wrapper object that has boundObject, setter and getter functions)
	engine.getPlainValue = function(bindingObject) {
		if(!!bindingObject) {
			return dataBindingHelper.getPlainValue(bindingObject.boundObject, bindingObject.property);
		}
		return null;
	}
	
	engine.notifyDataConfigBindingChange = function(scopeObject){
		var dataConfig = scopeObject.context.options;
		var optionValue;
		if (!dataConfig){
			return;
		}
		
		var bindingForMetadata;
		for(var optionKey in dataConfig) {	
		    if (optionKey == "_metadata"){
				for (var metadataKey in dataConfig[optionKey]){
					bindingForMetadata = dataConfig[optionKey][metadataKey];
					// if config option binds to resource, system, epv etc, bindingForMetadata doesn't have boundObject at all. And this kind of binding is supposed to be readonly, no change event should be fired.
					if (!!bindingForMetadata.bindingStr && !!bindingForMetadata.changed && !!bindingForMetadata.boundObject){
						   //call the change method
						   var event = { type : "config", property: optionKey + "." + metadataKey, oldVal: null, newVal: engine.getPlainValue(bindingForMetadata)};
						   scopeObject._change(event);
                           // PMR 124679 Need to reset the .changed attribute
                           if ((typeof bindingForMetadata.changed != 'undefined') && (bindingForMetadata.changed == true)) {
                                scopeObject.context.options[optionKey][metadataKey].changed = false;
                           }
					}
				}
			}
			else{			
			   optionValue = dataConfig[optionKey];
				// if config option binds to resource, system, epv etc, optoinValue doesn't have boundObject at all. And this kind of binding is supposed to be readonly, no change event should be fired.
			   if (!!optionValue.bindingStr && !!optionValue.changed && !!optionValue.boundObject){
				   //call the change method
				   var event = { type : "config", property:optionKey, oldVal: null, newVal: engine.getPlainValue(optionValue)};
				   scopeObject._change(event);
                   // PMR 124679 Need to reset the .changed attribute
                   if ((typeof optionValue.changed != 'undefined') && (optionValue.changed == true)) {
                        scopeObject.context.options[optionKey].changed = false;
                   }
			   }
			}		
		}		
	}	
			 	
	//##Data Binding end
	
	engine.initView = function(viewElement, parentViewScope) {
		logger.log("engine", "initView ", "viewElement: " + viewElement.id);
		
		var tempViewDivId = viewElement.getAttribute("id");
		var tempViewId = viewElement.getAttribute(engine.CONSTANTS.VIEW_ID);
		var tempViewType = viewElement.getAttribute(engine.CONSTANTS.VIEW_DATA_TYPE);
		var tempViewDataBinding = engine.rtDataBinding._get(viewElement);
		var tempViewCfgBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_CONFIG);
				
	//##Data Binding start
		//data
		var dataBinding = bindingResolver.resolveBinding(tempViewDataBinding);

		//config
		var cfgBinding = this.options[tempViewCfgBinding]; 
		if (!!cfgBinding) {
			try {
				bindingResolver.resolveOptionsBinding(cfgBinding);
			} catch(ex) {
		    	var errorMsg = "engine.initView() caught Exception when calling bindingResolver.resolveOptionsBinding(), Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
		    	return;
			}
		}
		
		var subclass = this.scopeClasses[tempViewType];
		try {
			var viewScope = new subclass(new ViewContext(), viewElement, dataBinding, cfgBinding);
	    } catch(ex) {
	    	var errorMsg = "engine.initView() caught Exception when calling view constructor for " + tempViewDivId + ", Exception: " + ex;
	    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			return;
	    }
	    
		if(tempViewDataBinding) {
			viewScope._registerCallBackForDataBinding(tempViewDataBinding);
		}
		
		viewScope._registerCallBackForOptions();
	//##Data Binding end
		
		if (!!parentViewScope) {
			// Set the scope to the parent
			var viewId = viewElement.getAttribute(engine.CONSTANTS.VIEW_ID);
			var divId = viewElement.getAttribute("id");
			if(!parentViewScope.context.subview[viewId]) {
				parentViewScope.context.subview[viewId] = {};
			}
			parentViewScope.context.subview[viewId][divId] = viewScope;
			viewScope.context._parentView = parentViewScope;
		}
		
		// continue to build the children views
		viewScope._buildChildren();

		viewScope._preLoad();
		
		// Call the view onLoad lifecycle method
		engine.callViewLifeCycleAPI(viewScope, "load", CONSTANTS.ELAPSED_TIME_THRESHOLD_LOAD);
	    
	    viewScope._highlightBindingIssue();   
	    
	    // if the performance of following line is concern, we can collect bpmIds when the binding is resolved initially.
	    viewScope._bpmIds = dataBindingHelper.collectBpmIds(tempViewDataBinding);
	    
		return viewScope;
	};
	
	engine.callViewLifeCycleAPI = function(viewScope, apiName, elapsedTimeThreshold, parameter) {
		try {		
			var startTime = commonHelper.isDebugMode() ? (new Date()).getTime() : null;
			viewScope[apiName].call(viewScope, parameter);
			if(startTime) {
				var elapsedTime = (new Date()).getTime() - startTime;
				if(elapsedTime >= elapsedTimeThreshold) {
					var info = apiName + "() for " + viewScope.context.element.id + " used " + elapsedTime + "ms";
					commonHelper.timestamp(info);
				}
			}
	    } catch(ex) {
	    	try {
		    	var message = "Coach framework caught exception when calling Life cycle API " + apiName + "() on view " + viewScope.context.element.id;
		    	message +=  ", Exception: " + ex;
		    	commonHelper.consoleLog(message, commonHelper.ERROR);
	    	} catch(ex1) {
		    	commonHelper.consoleLog("Even failed to gracefully log this Exception: " + ex, commonHelper.ERROR);
	    	}
	    }
	}

	engine.prepareView = function() {
		commonHelper.timestamp("Enter engine.prepareView()");
		logger.log("engine", "prepareView", "is called", logger.INFO);
		for (var viewId in this.topLevelViewScope) {
			   var viewScope = this.topLevelViewScope[viewId];
			   engine.callView(viewScope);
		}
		
		if(!!this.coach) {
			try {			
				this.coach.view();
		    } catch(ex) {
		    	var errorMsg = "engine.prepareView() caught Exception when calling view() on Coach object, Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
		    }
		}
		
		commonHelper.timestamp("Exit engine.prepareView()");
	};

	engine.callView = function(viewScope) {
		// Call nested view view first
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					engine.callView(subViewScope);
				}
			}		
		}
		
	    // Call the view view lifecycle method		
		engine.callViewLifeCycleAPI(viewScope, "view", CONSTANTS.ELAPSED_TIME_THRESHOLD_VIEW);
	    
	    viewScope._monitorRelatedListsIfNeeded();
	    
	    // Display the content
		// viewScope.context.element.style.display = 'block';
		var viewClass = viewScope.context.element.getAttribute('class');
		if(!domClass.contains(viewScope.context.element, engine.CONSTANTS.COACH_VIEW_SHOW)) {
			domClass.add(viewScope.context.element, engine.CONSTANTS.COACH_VIEW_SHOW);
		}
	};

	//The function, adjustIdAndBinding, is called from 2 different places
	//one is from createView. The node( divNode )is cloned and passed into createView.
	//another is from _repeatContentBoxs, the node, divNode, is cloned in _repeatContentBoxs and passed into this function
	//I need to clone the data config object and adjust the binding str based on the index.
	//If clone the data before this function is called, then I need to do recursive traveral and clone 
	//Rather, do it here, so that it use the same traveral code
	engine.adjustIdAndBinding = function(divNode, parentViewScope, bindingIndex) {
		if(divNode.tagName != "DIV")
			return null;
		
		// var idSuffix = "_" + bindingIndex;  // bindingIndex is not enough as idSuffix due to the dynamic delete/add
		var idSuffix = "_r" + parentViewScope._counter;
		parentViewScope._counter++;

		var templateConfigIdToInstanceConfigIdMap = engine._prepareTemplateConfigIdToInstanceConfigIdMap(parentViewScope);
		if(engine.isContentBox(divNode)) {
			// this divNode is a contentBox
			engine._adjustIdAndBindingForContentBox(divNode, idSuffix, bindingIndex, 1, parentViewScope, templateConfigIdToInstanceConfigIdMap);
		} else {
			// this divNode is expected to be a view
			engine._adjustIdAndBindingForView(divNode, idSuffix, bindingIndex, 1, parentViewScope, templateConfigIdToInstanceConfigIdMap);
		}
	};
	
	engine._prepareTemplateConfigIdToInstanceConfigIdMap = function(viewScope) {
		var templateConfigIdToInstanceConfigIdMap = {};
		var oneView = viewScope;
		while(!!oneView) {
			var viewOptions = oneView.context.options; 
			engine._processOneConfig(templateConfigIdToInstanceConfigIdMap, viewOptions);
			oneView = oneView.context.parentView();
		} 
		return templateConfigIdToInstanceConfigIdMap;
	}
	
	engine._processOneConfig = function(templateConfigIdToInstanceConfigIdMap, aConfig) {
		if(!!aConfig && !!aConfig.templateConfigId) {
			templateConfigIdToInstanceConfigIdMap[aConfig.templateConfigId] = aConfig.configId;
		} 
	}
	
	engine._adjustIdAndBindingForContentBox = function(conentBoxDomNode, idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, templateConfigIdToInstanceConfigIdMap) {
		// adjust id
		conentBoxDomNode.setAttribute('id', conentBoxDomNode.id + idSuffix);
		
		// handle nested Views
		var immediateViewList = domHelper.shallowQueryByClass(conentBoxDomNode, engine.CONSTANTS.COACH_VIEW, false);
		for(var i=0; i<immediateViewList.length; i++) {
			var newMap = commonHelper.shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForView(immediateViewList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}
		
		// handle nested ContentBox
		var immediateContentBoxList = domHelper.shallowQueryByClass(conentBoxDomNode, engine.CONSTANTS.CONTENT_BOX, false);
		for(var i=0; i<immediateContentBoxList.length; i++) {
			var newMap = commonHelper.shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForContentBox(immediateContentBoxList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}			
	}
	
	
	engine._adjustIdAndBindingForView = function(viewDomNode, idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, templateConfigIdToInstanceConfigIdMap) {
		// adjust id
		viewDomNode.setAttribute('id', viewDomNode.id + idSuffix);

		var viewDataBinding = dataBindingHelper.rtDataBinding._get(viewDomNode);		
		
		// if view's data binding is a config option, need to make sure this config option is up-to-date (in case the config option has been cloned)
		if(viewDataBinding.indexOf("config") == 0) {
			var newConfigBindingStr = engine._updateStaticOptionStr(viewDataBinding, templateConfigIdToInstanceConfigIdMap);
			if(viewDataBinding != newConfigBindingStr) {
				dataBindingHelper.rtDataBinding._set(viewDomNode, newConfigBindingStr);
				viewDataBinding = newConfigBindingStr;
			}
		}

		// now adjust this view's databinding
		var numOfIndex = dataBindingHelper.getNumberOfIndex(viewDataBinding);
		if(numOfIndex >= maxNumOfUnresolvedRepeating) {
		
			if(maxNumOfUnresolvedRepeating == 1 && numOfIndex == 1) {
				// binding of viewDomNode recieves the index value (currentItem) from the binding list of its parent view
				// if the length of viewDomNode binding list could be differnt from that of parent view' binding list,
				// we need to manage this kind of situation.
				var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
				var involvedList = viewDataBinding.substring(0, viewDataBinding.indexOf(indexWithBracket));
				viewControlRepeating._addRelatedList(viewDomNode.id, involvedList);
			}				
		
			var newBinding = dataBindingHelper.updateBindingStr(viewDataBinding, bindingIndex, maxNumOfUnresolvedRepeating, numOfIndex);
			dataBindingHelper.rtDataBinding._set(viewDomNode, newBinding);
		} else {
			// now numOfIndex < maxNumOfUnresolvedRepeating, we will adjust the index when hande next repeat. 
		}
		
		// now handle the dataConfig
		engine.cloneAndAdjustDataConfig(viewDomNode, bindingIndex, maxNumOfUnresolvedRepeating, templateConfigIdToInstanceConfigIdMap);	
				
		// handle nested Views
		var immediateViewList = domHelper.shallowQueryByClass(viewDomNode, engine.CONSTANTS.COACH_VIEW, false);
		for(var i=0; i<immediateViewList.length; i++) {
			var newMap = commonHelper.shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForView(immediateViewList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}
		
		// handle nested ContentBox
		var immediateContentBoxList = domHelper.shallowQueryByClass(viewDomNode, engine.CONSTANTS.CONTENT_BOX, false);
		var newMaxNumOfUnresolvedRepeating = maxNumOfUnresolvedRepeating;
		var newViewControlRepeating = viewControlRepeating;
		if(dataBindingHelper.isBindingArray(viewDataBinding)) {
			newMaxNumOfUnresolvedRepeating++;
			// repeating of immediateContentBoxList[] is controlled by viewDomNode. but view object for viewDomNode is not
			// creaed yet. And newMaxNumOfUnresolvedRepeating won't be used in the called method because there is more than one
			// unresolved repeating at this moment.
			newViewControlRepeating = null;
		}
		for(var i=0; i<immediateContentBoxList.length; i++) {
			var newMap = commonHelper.shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForContentBox(immediateContentBoxList[i], idSuffix, bindingIndex, newMaxNumOfUnresolvedRepeating, newViewControlRepeating, newMap);
		}
		
	}

//************************* remove this part later, replace with the ones on dataBindingHelper	
	// rtDataBinding provides 3 helper method for resolving List related data-binding during runtime.
	engine.rtDataBinding = new Object();
	engine.rtDataBinding._get = function(viewElement) {
		var rtBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_BINDING_RT);
		if(!rtBinding) {
			return viewElement.getAttribute(engine.CONSTANTS.DATA_BINDING);
		} else {
			return rtBinding;
		}
	}
	engine.rtDataBinding._set = function(viewElement, bindingValue) {
		viewElement.setAttribute(engine.CONSTANTS.DATA_BINDING_RT, bindingValue);
	}
	engine.rtDataBinding._delete = function(viewElement) {	
		// viewElement.removeAttribute(engine.CONSTANTS.DATA_BINDING_RT);
	}
//************* end

	engine.isContentBox = function(node) {
		if(!!node) {
			return domClass.contains(node, engine.CONSTANTS.CONTENT_BOX);
		}
		return false;
	};
	
	// check if the provided contentBox is view managed
	engine.isViewManagedContentBox = function(contentBoxNode) {
		return contentBoxNode.getAttribute(engine.CONSTANTS.IS_VIEW_MANAGED) == "true";
	}
	
	// contentBoxDiv can be framework managed ContentBox or view managed ContentBox
	// newViewArray is optional. If provided, it stores the new views created by this method.
	engine.buildContentBox = function(parentView, contentBoxDiv, newViewArray) {
		var immediateViewList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.COACH_VIEW, true);
		var oneDirectView;
		array.forEach(immediateViewList, function(item, index) {
			oneDirectView = engine.initView.apply(engine.GlobalAssets, [item, parentView]);
			if(!!newViewArray) {
				newViewArray.push(oneDirectView);
			}
		});
		
		// find the direct ContentBox under this contentBox.
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.CONTENT_BOX, true);
		// only care about the framework managed ContentBox. view managed ContentBox will be built by its owner view.
		for(var k=0; k<immediateContentBoxList.length; k++) {
			if(!engine.isViewManagedContentBox(immediateContentBoxList[k])) {
				// Framework managed
				engine.buildContentBox(parentView, immediateContentBoxList[k], newViewArray);
			}
		}
	}
	
	// delete all subviews that reside in the specified contentBox, including 
	// 1) the views owned by specified contentBox directly
	// 2) the views under the framework or view managed contentBox that is owned by specified contentBox
	engine.deleteContentBox = function(contentBoxDiv, ownerView) {
		var directViewNodes = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.COACH_VIEW);
		array.forEach(directViewNodes, function(item, index) {
			engine.deleteView(item, ownerView);				
		}, ownerView);

		// find the direct ContentBox under this contentBox.
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.CONTENT_BOX, true);
		// delete all contentBox regardless it's framework managed or view managed.
		for(var k=0; k<immediateContentBoxList.length; k++) {
			engine.deleteContentBox(immediateContentBoxList[k], ownerView);
		}
	}
	
	engine.deleteView = function(subviewDivNode, ownerView) {
		var viewId = subviewDivNode.getAttribute(engine.CONSTANTS.VIEW_ID);
		var divId = subviewDivNode.getAttribute("id");
		var subviews = ownerView.context.subview[viewId];
		if(!!subviews && !!subviews[divId]) {
			engine.callUnload(subviews[divId]);
			delete subviews[divId];
		}
	}

	// Future consideration - engine holds a map between divId and viewScope object.
	// Pro: search vewScope object by div id (like following few methods) will perform better and looks smple.
	// Con: have to maitain the map properly when add/delete occurs
	// The performance of following methods (about finding viewScope) should be ok since it just traverse DOM tree straightly up/down.

	engine.findEndStateId = function(bdEventId) {
	    var systemData = engine.GlobalAssets.systemData;
	    if (systemData.boundaryEventInformation) {
	        for (var i=0; i<systemData.boundaryEventInformation.length; i++) {
	            if (systemData.boundaryEventInformation[i].boundaryEventID == bdEventId) {
	                return systemData.boundaryEventInformation[i].endStateID;
	            }
	        }
	    }
	    return "";
	}

    // helper function for retrieving boundary event information by the given boundary event id
    engine.getBoundaryEventInfo = function(bdEventId) {
        var systemData = engine.GlobalAssets.systemData;
        if (systemData.boundaryEventInformation) {
            for (var i=0; i<systemData.boundaryEventInformation.length; i++) {
                if (systemData.boundaryEventInformation[i].boundaryEventID == bdEventId) {
                    return {boundaryEventId: bdEventId,
                            boundaryEventEndStateId: systemData.boundaryEventInformation[i].endStateID,
                            fireValidation: systemData.boundaryEventInformation[i].fireValidation};
                }
            }
        }
        return {boundaryEventId: bdEventId, boundaryEventEndStateId: "", fireValidation: 0};
    }

    // helper function for retrieving validation boundary event id and end state id
    engine.getValidateBoundaryEventInfo = function() {
        var systemData = engine.GlobalAssets.systemData;
        if (!!systemData.validateBoundaryEventInformation) {
            // for now we assume that there may be only one validation boundary event per coach
            if (!!systemData.validateBoundaryEventInformation[0]) {
                return {validateBoundaryEventId: systemData.validateBoundaryEventInformation[0].boundaryEventID,
                        validateBoundaryEventEndStateId: systemData.validateBoundaryEventInformation[0].endStateID};
            }
        }
        return null;
    }

	// Helper API for handling debug response
	engine.handleDebugMode = function(nextURL) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);

		// redirect to the next URL
		window.location.assign(nextURL);
	}
	
	// Helper API for handling boundary event response
	engine.TaskCompleted = function(nextURL, nextTaskId) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			if (!!nextTaskId) {
				var event = {
					"name": "onCompleted",
					"taskID" : engine.GlobalAssets.systemData.zTaskId,
					"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
					"parameters": [nextTaskId]
				};
				broadcastHelper.broadcastMessage(event);
			}
			// Defect 70546.  We have to show task completed page in case of Process Admin Administrative service
			// if(!!nextTaskId) {
			if(!!nextURL) {
				//commenting this out for defect 79695, breaks auto-flow
				//if(window.parent.location.pathname == "/mum/enabler") {
				//      return;
				//}
				window.location.assign(nextURL);
			}
			// }
		} else {
			// redirect to the next URL in the non-embedded case
			if(!!nextURL) {
				if ((window.parent.location.pathname == "/mum/enabler") && (nextURL.indexOf("tm_process_finished.lsw") > -1)) {
					return;
				}
				window.location.assign(nextURL);
			}
		}
	}
	
	engine.TaskCompletedNotOwned = function(nextURL) {
		engine._taskCompletedOthers(nextURL);
	}

	engine.TaskCompletedAlreadyClosed = function(nextURL) {
		engine._taskCompletedOthers(nextURL);
	}
	
	engine._taskCompletedOthers = function(nextURL) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		if(!!nextURL) {
			window.location.assign(nextURL);
		}
	}	
	
	engine.TaskPostponed = function(nextURL) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onSuspended",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": []
			};
			broadcastHelper.broadcastMessage(event);
		} else {
			if(!!nextURL) {
				window.location.assign(nextURL);
			}
		}
	}


	engine.StateChange = function(nextURL) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		// redirect to the next URL
		window.location.assign(nextURL);
	}


	engine.ErrorBoundaryEvent = function(message) {
		// defect 129412
		// engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onError",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": [message]
			};
			broadcastHelper.broadcastMessage(event);
		}
		// Showing the coachNG error page.  This is necessary in particular in Process Admin Administrative service case.
		window.location.assign("html/coachNGError.jsp");
	}

	// Helper API for handling collab related event
	engine.handleFocusEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var outlineColor = !!event.outlineColor ? event.outlineColor : "red"; // now use "red" as default color. refine later
			var message = !!event.userName ? event.userName : engine.localizedMessages.coachNG_unknown;
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "brown"; // now use "brown" as default background color. refine later
			
		    var collaborationEvent = {"type": "focus", "domId": domId, "outlineColor": outlineColor, "color": color, "bgColor": bgColor, "message": message};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleBlurEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);	
					
		    var collaborationEvent = {"type": "blur", "domId": domId};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleMouseOnEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "red"; // now use "red" as default background color. refine later
			var message = !!event.message ? event.message : engine.localizedMessages.coachNG_unknown;			
		    
		    var collaborationEvent = {"type": "mouseOn", "domId": domId, "color": color, "bgColor": bgColor, "message": message};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleMouseOffEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);			
		    
		    var collaborationEvent = {"type": "mouseOff", "domId": domId};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleCollabDataChangeEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "red"; // now use "red" as default background color. refine later
			var message = !!event.message ? event.message : engine.localizedMessages.coachNG_unknown;
			var userName = !!event.userName ? event.userName : engine.localizedMessages.coachNG_unknown_user;
			var value = !!event.value ? event.value : engine.localizedMessages.coachNG_unknown_value;			

		    var collaborationEvent = {"type": "dataChange", "domId": domId, "color": color, "bgColor": bgColor, "message": message, "userName": userName, "value": value};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}
	
	engine.handleCustomCollaborationEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {			
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, event);
		}
	}
	
	// controller calls this internal API when it receives validation information of one boundary event
	engine.handleValidationEvent = function(rawEvent) {
		try {		
			var topLevelViews = engine.GlobalAssets.topLevelViewScope;
			// prepare validationHelper, make sure it's clean
			validationHelper.initialize();
			var hasValidationEvent = validationHelper.validateAllViews(rawEvent, topLevelViews);
			if(hasValidationEvent) {	
				engine.deliverValidationEvents();
				// broadcast that all validation events have been delivered.
				connect.publish("com.ibm.bpm.coach.validationCompleted", {});
			} else {
				validationHelper.clearTransientInfo();	
			}
		} catch(ex) {
			logger.log("engine", "engine.handleValidationEvent", "ex: " + ex);
		}
	}

	// controller calls this internal API right before it submit boundary event.
	// engine will ask all the views that have validation errors to clear themself
	engine.clearValidationEvent = function() {
		var treeNode = validationHelper.viewTree.rootNode.firstChild;
		if(!!treeNode) {
			engine._deliverValidationEvents(treeNode, validationHelper.viewDomIdToViewMap, true);

			// now we have delivered all the clear events, it's time for framework to clear all the memory of last validation cycle.
			validationHelper.initialize();

			// broadcast that all validation events (clear) have been delivered.
			connect.publish("com.ibm.bpm.coach.validationCleared", {});
		}
	}
	
	engine.deliverValidationEvents = function() {
		if(engine.isProcessingValidation) {
			// engine is already in the middle of delivering validation events. All the new validation events 
			// prepared by the caller will be handled by the existing delivery process. So need to do nothing for now.
			return;
		}
	
		engine.isProcessingValidation = true;
		
		// new validation events may occur as the result of processing of other validation event. The new event may
		// locate at the tree node which current round of delivery has gone through. So we have to go through
		// the entire tree multiple times to make sure no event is left. However, in order to avoid the possible
		// overflow, we only tranverse the entire tree up to 10 times.
		for(var k=0; k<10; k++) {
			var treeNode = validationHelper.viewTree.rootNode.firstChild;
			var foundValidationEvent = engine._deliverValidationEvents(treeNode, validationHelper.viewDomIdToViewMap);
			if(!foundValidationEvent) {
				break;
			}
		}
		// clear the transient data in validationHelper. Leave other data alone in validationHelper for future use.
		// we don't call clearTransientInfo() earlier (e.g. after validateAllViews() is executed and before deliverValidationEvents() is executed), 
		// the main reason is for performance - new validation event may occurs during the execution of deliverValidationEvents(), the calculation of new valdation event
		// need the help from the transient data in validationHelper. 
		validationHelper.clearTransientInfo();
		engine.isProcessingValidation = false;		
	}
	
	engine._deliverValidationEvents = function(treeNode, viewDomIdToViewMap, isToClear) {
		var foundValidationEvent = false;
		if(!treeNode) {
			return foundValidationEvent;
		}
		
		if(!!isToClear) {
			var clearEvent = {"type": BPM_CONSTANTS.VALIDATION_EVENT_TYPE_CLEAR};
			// send clear event to all the valid views who have received validation events
			if(!!treeNode.validationEvent) {
				var aView = engine._findViewInstance(treeNode, viewDomIdToViewMap);
				if(!!aView) {
					engine.callViewLifeCycleAPI(aView, "validate", CONSTANTS.ELAPSED_TIME_THRESHOLD_VALIDATE, clearEvent);
				}
				if(!foundValidationEvent) {
					foundValidationEvent = true;
				}
			}			
		} else {
			// deliver the validation events if exists
			if(!treeNode.isHandled) {
				// handle treeNode's validationEvent if exists
				if(!!treeNode.validationEvent) {
					var aView = engine._findViewInstance(treeNode, viewDomIdToViewMap);
					if(!!aView) {
						engine.callViewLifeCycleAPI(aView, "validate", CONSTANTS.ELAPSED_TIME_THRESHOLD_VALIDATE, treeNode.validationEvent);
					}
					if(!foundValidationEvent) {
						foundValidationEvent = true;
					}
				}		
				treeNode.isHandled = true;
			}
		}
		
		// handle treeNode's children
		if(!!treeNode.firstChild) {
			var childResult = engine._deliverValidationEvents(treeNode.firstChild, viewDomIdToViewMap, isToClear);
			if(!foundValidationEvent) {
				foundValidationEvent = childResult;
			}	
		}
		
		// hanlde treeNode's sibling
		if(!!treeNode.next) {
			var siblingResult = engine._deliverValidationEvents(treeNode.next, viewDomIdToViewMap, isToClear);
			if(!foundValidationEvent) {
				foundValidationEvent = siblingResult;
			}	
		}
		
		return foundValidationEvent;
	}
	
	engine._findViewInstance = function(treeNode, viewDomIdToViewMap) {
		var cachedView = viewDomIdToViewMap[treeNode.domId];
		if(!!cachedView) {
			if(cachedView._isReady()) {
				return cachedView;
			}			
		}
		
		var aView = null;

		if(!!treeNode.locationInfo) {
			// viewIdMetaData exists
			aView = coachViewHelper.findViewByLocation(treeNode.locationInfo, engine.GlobalAssets.topLevelViewScope);
		} else {
			aView = coachViewHelper.findViewScope(treeNode.domId, engine.GlobalAssets.topLevelViewScope);
		}	

		if(!!aView) {
			viewDomIdToViewMap[treeNode.viewDomId] = aView;
			return aView;
		}
		
		return null;
	}

	engine._findCollaboratingView = function(event) {
		if(!!event.viewIdMetaData) {
			// if viewIdMetaData exists, use it to figure out view's DOM id
			return coachViewHelper.findViewByLocation(event.viewIdMetaData, engine.GlobalAssets.topLevelViewScope);
		} else if(!!event.viewId) {
			// event.viewId means the dom id of the closest viewScope node
			return coachViewHelper.findViewScope(event.viewId, engine.GlobalAssets.topLevelViewScope);
		} else {
			return null;
		}	
	}
	
	engine._findCollaboratingDomId = function(collaboratingView, event) {
		var node = dom.byId(event.domId);
		if(!!node && dom.isDescendant(node, collaboratingView.context.element)) {
			return event.domId;
		} else {
			// fallback to dom id of collaboratingView
			return collaboratingView.context.element.id;
		}
	}
	
	engine.getViewIdMetaData = function(viewDomId) {
		var viewScope = coachViewHelper.findViewScope(viewDomId, engine.GlobalAssets.topLevelViewScope);
		return coachViewHelper.getViewIdMetaData(viewScope);
	}
	
	engine.callAjaxService = function(ajaxUrl, args) {
		var paramsKey = "&params="
		var params = null;
		if (!!args) {
			if (!!(args.params)) {
				if (lang.isString(args.params)) {
					params = encodeURIComponent(args.params);					
				} else {
					params = encodeURIComponent(json1.stringify(args.params));
				}
			}
		}
		var tempStr = (!!params)? (ajaxUrl+paramsKey+params) : ajaxUrl;
		var tempArray = tempStr.split("?");
		
		var xhrArgs = {
			url: tempArray[0],
			handleAs: "json",
		    headers: {"Content-Type": "application/x-www-form-urlencoded",
		    	"Accept": "application/json"},
			sync: false,
			postData: tempArray[1],
			load: function(data){
				if ((!!args) && (!!(args.load)) && (lang.isFunction(args.load))) {
					args.load(data.data.data);
				}
	        },
	        error: function(err) {
	        	if ((!!args) && (!!(args.error)) && (lang.isFunction(args.error))) {
	        		args.error(err);
	        	} else {
			    	var errorMsg = "engine.callAjaxService() caught Error when handling " + ajaxUrl + ", Error: " + err;
			    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
	        	}
	        }
		};
		xhr.post(xhrArgs);
	}
	
	
	engine.setReadonly = function(readonly) {
		if(readonly) {
			// if engine.coachShimmer exists, that means already in readonly mode. So current readonly mode request should be ingored
			if(!engine.coachShimmer) {
				engine.coachShimmer = new Shimmer(engine.CONSTANTS.COACH_SHIMMER_ID_BASE);
			}
		} else {
			// if engine.coachShimmer doesn't exist, that means already in edit mode. So current edit mode request should be ignored
			if(!!engine.coachShimmer) {
				engine.coachShimmer.dispose();
                engine.coachShimmer = null;
			}
		}
	}

					  
	engine.completeInitialization = function() {
		logger.log("engine", "completeInitialization", "is called", logger.INFO);
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onInitialized",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": [{"name": "URL", "value": window.document.location.href}]
			};
			broadcastHelper.broadcastMessage(event);
		} 
		if (window.addEventListener) {
			window.addEventListener('unload', engine.onWindowClose, false);
		} else if (window.attachEvent) {
			window.attachEvent('onunload', engine.onWindowClose);
		} else {
			window.onunload = engine.onWindowClose;
		}		
		commonHelper.timestamp("Exit engine.completeInitialization()");
	}
	
	//support for portlet enablement
	engine.addMessageListeners = function(whiteList) {
		logger.log("engine", "addMessageListeners", "is called", logger.FINER);
		
		// Nesting engine.checkPostMessageOrigin() inside engine.addMessageListeners to hide whiteList in the closure.
		// check if the origin is a trusted one
		engine.checkPostMessageOrigin = function(origin) {
			// firstly, check if provided origin is same as mine
			// host covers both hostname and port
			var myOrigin = window.location.protocol + "//" + window.location.host;
			if(origin == myOrigin) {
				return true;
			}
	
			// then, check if provided origin matches any one in the whiteList
			if(!!whiteList) {
				for(var k=0; k<whiteList.length; k++) {
					if(origin == whiteList[k]) {
						return true;
					}
				} 
			} 			
			return false;
		}		
		
		var receivePostMessage = function(event) {
			logger.log("engine", "receivePostMessage", ["is called with event:", event], logger.FINER);
			
			if(engine.checkPostMessageOrigin(event.origin, whiteList)) {
				var eventObj = json.fromJson(event.data);
				var returnTicket = {window: event.source, origin: event.origin};
				
				if (eventObj.name == 'broadcastBoundaryEvent') {
					// initializing external messaging for boundary events
					// engine.boundaryEventBroadcastTarget are only used for coach to broadcast boundaryEvent, e.g. to portlet
					engine.boundaryEventBroadcastTarget = returnTicket;
				} else if (eventObj.name == 'USER_PROFILE_CHANGED') {
					dojo.publish("/com/ibm/bpm/social/USER_PROFILE_CHANGED", [eventObj.username]);
				} else {
					// Once Coach completes this external request, send the response back to the window (represented by returnTicket.window) 
					// with correct origin (represented by returnTicket.origin)
					engine.controller.processExternalEvent(eventObj, returnTicket);
				}
			} else {
				// Coach framework received postMessage from untrusted origin.
				commonHelper.consoleLog("Coach ignores a postMessage received from untrusted origin " + event.origin, commonHelper.WARN);
			}
		}
		
		if(typeof window.addEventListener != 'undefined') {
			window.addEventListener("message", receivePostMessage, false);
		} else if(typeof window.attachEvent != 'undefined') {
			window.attachEvent("onmessage", receivePostMessage);
		}	
	}
	
	engine.onWindowClose = function() {
		logger.log("engine", "onWindowClose", "is called", logger.INFO);
		for (var i = 0; i < engine._internalUnloadHandler.length; i++) {
			try {
				engine._internalUnloadHandler[i]();
			} catch (ex) {
				// log the exception and ignore it
		    	var errorMsg = "engine.onWindowClose() caught Exception when invoking internal unload handler, Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			}
		}
		engine.onCoachUnload.call(engine.GlobalAssets);
	}
	
	engine._registerInternalUnloadHandler = function(callback) {
		engine._internalUnloadHandler.push(callback);
	}

	engine.onCoachUnload = function() {
		logger.log("engine", "onCoachUnload", "is called", logger.INFO);
		if (!(engine._isCoachUnloaded)) {
		
			if(!!this.coach) {
				try {			
					this.coach.unload();
			    } catch(ex) {
			    	var errorMsg = "engine.onCoachUnload() caught Exception when calling unload() on Coach object, Exception: " + ex;
			    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			    }
				delete this.coach;
			}
						
			var topLevelViewIds = new Array();
			for (var viewId in this.topLevelViewScope) {
				topLevelViewIds.push(viewId);
				var viewScope = this.topLevelViewScope[viewId];
				engine.callUnload(viewScope);
			}
			for (var k=0; k<topLevelViewIds.length; k++) {
				delete this.topLevelViewScope[topLevelViewIds[k]];
			}
			
			if(!!engine.controller) {
				engine.controller.destroy();
			}
			
			engine._isCoachUnloaded = true;
		}
	}

	engine.callUnload = function(viewScope) {
		if(!viewScope) return;
		
	    // Call the view unload lifecycle method
		viewScope._preUnload();

		engine.callViewLifeCycleAPI(viewScope, "unload", CONSTANTS.ELAPSED_TIME_THRESHOLD_UNLOAD);

		// Then call nested view unload
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {

				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					engine.callUnload(subViewScope);
				}
			}		
		}
		
	    // Finally call _postUnload() to dispose viewScope (make it ready for garbage collection)
		// viewScope.context.subview is set to null in viewScope.postUnload(), so have to get viewScope.context.subview before calling viewScope.postUnload()
		viewScope._postUnload();
	}	
	
	return engine;
});

},
'dojo/NodeList-dom':function(){
define(["./_base/kernel", "./query", "./_base/array", "./_base/lang", "./dom-class", "./dom-construct", "./dom-geometry", "./dom-attr", "./dom-style"], function(dojo, query, array, lang, domCls, domCtr, domGeom, domAttr, domStyle){
	/*===== var NodeList = dojo.NodeList; =====*/
	var magicGuard = function(a){
		// summary:
		//		the guard function for dojo.attr() and dojo.style()
		return a.length == 1 && (typeof a[0] == "string"); // inline'd type check
	};

	var orphan = function(node){
		// summary:
		//		function to orphan nodes
		var p = node.parentNode;
		if(p){
			p.removeChild(node);
		}
	};
	// FIXME: should we move orphan() to dojo.html?

	var NodeList = query.NodeList,
		awc = NodeList._adaptWithCondition,
		aafe = NodeList._adaptAsForEach,
		aam = NodeList._adaptAsMap;

	function getSet(module){
		return function(node, name, value){
			if(arguments.length == 2){
				return module[typeof name == "string" ? "get" : "set"](node, name);
			}
			// setter
			return module.set(node, name, value);
		};
	}

	lang.extend(NodeList, {
		_normalize: function(/*String||Element||Object||NodeList*/content, /*DOMNode?*/refNode){
			// summary:
			//		normalizes data to an array of items to insert.
			// description:
			//		If content is an object, it can have special properties "template" and
			//		"parse". If "template" is defined, then the template value is run through
			//		dojo.string.substitute (if dojo.string.substitute has been dojo.required elsewhere),
			//		or if templateFunc is a function on the content, that function will be used to
			//		transform the template into a final string to be used for for passing to dojo._toDom.
			//		If content.parse is true, then it is remembered for later, for when the content
			//		nodes are inserted into the DOM. At that point, the nodes will be parsed for widgets
			//		(if dojo.parser has been dojo.required elsewhere).

			//Wanted to just use a DocumentFragment, but for the array/NodeList
			//case that meant using cloneNode, but we may not want that.
			//Cloning should only happen if the node operations span
			//multiple refNodes. Also, need a real array, not a NodeList from the
			//DOM since the node movements could change those NodeLists.

			var parse = content.parse === true;

			//Do we have an object that needs to be run through a template?
			if(typeof content.template == "string"){
				var templateFunc = content.templateFunc || (dojo.string && dojo.string.substitute);
				content = templateFunc ? templateFunc(content.template, content) : content;
			}

			var type = (typeof content);
			if(type == "string" || type == "number"){
				content = domCtr.toDom(content, (refNode && refNode.ownerDocument));
				if(content.nodeType == 11){
					//DocumentFragment. It cannot handle cloneNode calls, so pull out the children.
					content = lang._toArray(content.childNodes);
				}else{
					content = [content];
				}
			}else if(!lang.isArrayLike(content)){
				content = [content];
			}else if(!lang.isArray(content)){
				//To get to this point, content is array-like, but
				//not an array, which likely means a DOM NodeList. Convert it now.
				content = lang._toArray(content);
			}

			//Pass around the parse info
			if(parse){
				content._runParse = true;
			}
			return content; //Array
		},

		_cloneNode: function(/*DOMNode*/ node){
			// summary:
			//		private utility to clone a node. Not very interesting in the vanilla
			//		dojo.NodeList case, but delegates could do interesting things like
			//		clone event handlers if that is derivable from the node.
			return node.cloneNode(true);
		},

		_place: function(/*Array*/ary, /*DOMNode*/refNode, /*String*/position, /*Boolean*/useClone){
			// summary:
			//		private utility to handle placing an array of nodes relative to another node.
			// description:
			//		Allows for cloning the nodes in the array, and for
			//		optionally parsing widgets, if ary._runParse is true.

			//Avoid a disallowed operation if trying to do an innerHTML on a non-element node.
			if(refNode.nodeType != 1 && position == "only"){
				return;
			}
			var rNode = refNode, tempNode;

			//Always cycle backwards in case the array is really a
			//DOM NodeList and the DOM operations take it out of the live collection.
			var length = ary.length;
			for(var i = length - 1; i >= 0; i--){
				var node = (useClone ? this._cloneNode(ary[i]) : ary[i]);

				//If need widget parsing, use a temp node, instead of waiting after inserting into
				//real DOM because we need to start widget parsing at one node up from current node,
				//which could cause some already parsed widgets to be parsed again.
				if(ary._runParse && dojo.parser && dojo.parser.parse){
					if(!tempNode){
						tempNode = rNode.ownerDocument.createElement("div");
					}
					tempNode.appendChild(node);
					dojo.parser.parse(tempNode);
					node = tempNode.firstChild;
					while(tempNode.firstChild){
						tempNode.removeChild(tempNode.firstChild);
					}
				}

				if(i == length - 1){
					domCtr.place(node, rNode, position);
				}else{
					rNode.parentNode.insertBefore(node, rNode);
				}
				rNode = node;
			}
		},

		/*=====
		position: function(){
			// summary:
			//		Returns border-box objects (x/y/w/h) of all elements in a node list
			//		as an Array (*not* a NodeList). Acts like `dojo.position`, though
			//		assumes the node passed is each node in this list.

			return dojo.map(this, dojo.position); // Array
		},

		attr: function(property, value){
			// summary:
			//		gets or sets the DOM attribute for every element in the
			//		NodeList. See also `dojo.attr`
			// property: String
			//		the attribute to get/set
			// value: String?
			//		optional. The value to set the property to
			// returns:
			//		if no value is passed, the result is an array of attribute values
			//		If a value is passed, the return is this NodeList
			// example:
			//		Make all nodes with a particular class focusable:
			//	|	dojo.query(".focusable").attr("tabIndex", -1);
			// example:
			//		Disable a group of buttons:
			//	|	dojo.query("button.group").attr("disabled", true);
			// example:
			//		innerHTML can be assigned or retrieved as well:
			//	|	// get the innerHTML (as an array) for each list item
			//	|	var ih = dojo.query("li.replaceable").attr("innerHTML");
			return; // dojo.NodeList
			return; // Array
		},

		style: function(property, value){
			// summary:
			//		gets or sets the CSS property for every element in the NodeList
			// property: String
			//		the CSS property to get/set, in JavaScript notation
			//		("lineHieght" instead of "line-height")
			// value: String?
			//		optional. The value to set the property to
			// returns:
			//		if no value is passed, the result is an array of strings.
			//		If a value is passed, the return is this NodeList
			return; // dojo.NodeList
			return; // Array
		},

		addClass: function(className){
			// summary:
			//		adds the specified class to every node in the list
			// className: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			return; // dojo.NodeList
		},

		removeClass: function(className){
			// summary:
			//		removes the specified class from every node in the list
			// className: String|Array?
			//		An optional String class name to remove, or several space-separated
			//		class names, or an array of class names. If omitted, all class names
			//		will be deleted.
			// returns:
			//		dojo.NodeList, this list
			return; // dojo.NodeList
		},

		toggleClass: function(className, condition){
			// summary:
			//		Adds a class to node if not present, or removes if present.
			//		Pass a boolean condition if you want to explicitly add or remove.
			// condition: Boolean?
			//		If passed, true means to add the class, false means to remove.
			// className: String
			//		the CSS class to add
			return; // dojo.NodeList
		},

		empty: function(){
			// summary:
			//		clears all content from each node in the list. Effectively
			//		equivalent to removing all child nodes from every item in
			//		the list.
			return this.forEach("item.innerHTML='';"); // dojo.NodeList
			// FIXME: should we be checking for and/or disposing of widgets below these nodes?
		},
		=====*/

		// useful html methods
		attr: awc(getSet(domAttr), magicGuard),
		style: awc(getSet(domStyle), magicGuard),

		addClass: aafe(domCls.add),
		removeClass: aafe(domCls.remove),
		replaceClass: aafe(domCls.replace),
		toggleClass: aafe(domCls.toggle),

		empty: aafe(domCtr.empty),
		removeAttr: aafe(domAttr.remove),

		position: aam(domGeom.position),
		marginBox: aam(domGeom.getMarginBox),

		// FIXME: connectPublisher()? connectRunOnce()?

		/*
		destroy: function(){
			// summary:
			//		destroys every item in the list.
			this.forEach(d.destroy);
			// FIXME: should we be checking for and/or disposing of widgets below these nodes?
		},
		*/

		place: function(/*String||Node*/ queryOrNode, /*String*/ position){
			// summary:
			//		places elements of this node list relative to the first element matched
			//		by queryOrNode. Returns the original NodeList. See: `dojo.place`
			// queryOrNode:
			//		may be a string representing any valid CSS3 selector or a DOM node.
			//		In the selector case, only the first matching element will be used
			//		for relative positioning.
			// position:
			//		can be one of:
			//		|	"last" (default)
			//		|	"first"
			//		|	"before"
			//		|	"after"
			//		|	"only"
			//		|	"replace"
			//		or an offset in the childNodes property
			var item = query(queryOrNode)[0];
			return this.forEach(function(node){ domCtr.place(node, item, position); }); // dojo.NodeList
		},

		orphan: function(/*String?*/ filter){
			// summary:
			//		removes elements in this list that match the filter
			//		from their parents and returns them as a new NodeList.
			// filter:
			//		CSS selector like ".foo" or "div > span"
			// returns:
			//		`dojo.NodeList` containing the orphaned elements
			return (filter ? query._filterResult(this, filter) : this).forEach(orphan); // dojo.NodeList
		},

		adopt: function(/*String||Array||DomNode*/ queryOrListOrNode, /*String?*/ position){
			// summary:
			//		places any/all elements in queryOrListOrNode at a
			//		position relative to the first element in this list.
			//		Returns a dojo.NodeList of the adopted elements.
			// queryOrListOrNode:
			//		a DOM node or a query string or a query result.
			//		Represents the nodes to be adopted relative to the
			//		first element of this NodeList.
			// position:
			//		can be one of:
			//		|	"last" (default)
			//		|	"first"
			//		|	"before"
			//		|	"after"
			//		|	"only"
			//		|	"replace"
			//		or an offset in the childNodes property
			return query(queryOrListOrNode).place(this[0], position)._stash(this);	// dojo.NodeList
		},

		// FIXME: do we need this?
		query: function(/*String*/ queryStr){
			// summary:
			//		Returns a new list whose members match the passed query,
			//		assuming elements of the current NodeList as the root for
			//		each search.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo">
			//	|		<p>
			//	|			bacon is tasty, <span>dontcha think?</span>
			//	|		</p>
			//	|	</div>
			//	|	<div id="bar">
			//	|		<p>great comedians may not be funny <span>in person</span></p>
			//	|	</div>
			//		If we are presented with the following definition for a NodeList:
			//	|	var l = new dojo.NodeList(dojo.byId("foo"), dojo.byId("bar"));
			//		it's possible to find all span elements under paragraphs
			//		contained by these elements with this sub-query:
			//	|	var spans = l.query("p span");

			// FIXME: probably slow
			if(!queryStr){ return this; }
			var ret = new NodeList;
			this.map(function(node){
				// FIXME: why would we ever get undefined here?
				query(queryStr, node).forEach(function(subNode){
					if(subNode !== undefined){
						ret.push(subNode);
					}
				});
			});
			return ret._stash(this);	// dojo.NodeList
		},

		filter: function(/*String|Function*/ filter){
			// summary:
			//		"masks" the built-in javascript filter() method (supported
			//		in Dojo via `dojo.filter`) to support passing a simple
			//		string filter in addition to supporting filtering function
			//		objects.
			// filter:
			//		If a string, a CSS rule like ".thinger" or "div > span".
			// example:
			//		"regular" JS filter syntax as exposed in dojo.filter:
			//		|	dojo.query("*").filter(function(item){
			//		|		// highlight every paragraph
			//		|		return (item.nodeName == "p");
			//		|	}).style("backgroundColor", "yellow");
			// example:
			//		the same filtering using a CSS selector
			//		|	dojo.query("*").filter("p").styles("backgroundColor", "yellow");

			var a = arguments, items = this, start = 0;
			if(typeof filter == "string"){ // inline'd type check
				items = query._filterResult(this, a[0]);
				if(a.length == 1){
					// if we only got a string query, pass back the filtered results
					return items._stash(this); // dojo.NodeList
				}
				// if we got a callback, run it over the filtered items
				start = 1;
			}
			return this._wrap(array.filter(items, a[start], a[start + 1]), this);	// dojo.NodeList
		},

		/*
		// FIXME: should this be "copyTo" and include parenting info?
		clone: function(){
			// summary:
			//		creates node clones of each element of this list
			//		and returns a new list containing the clones
		},
		*/

		addContent: function(/*String||DomNode||Object||dojo.NodeList*/ content, /*String||Integer?*/ position){
			// summary:
			//		add a node, NodeList or some HTML as a string to every item in the
			//		list.  Returns the original list.
			// description:
			//		a copy of the HTML content is added to each item in the
			//		list, with an optional position argument. If no position
			//		argument is provided, the content is appended to the end of
			//		each item.
			// content:
			//		DOM node, HTML in string format, a NodeList or an Object. If a DOM node or
			//		NodeList, the content will be cloned if the current NodeList has more than one
			//		element. Only the DOM nodes are cloned, no event handlers. If it is an Object,
			//		it should be an object with at "template" String property that has the HTML string
			//		to insert. If dojo.string has already been dojo.required, then dojo.string.substitute
			//		will be used on the "template" to generate the final HTML string. Other allowed
			//		properties on the object are: "parse" if the HTML
			//		string should be parsed for widgets (dojo.require("dojo.parser") to get that
			//		option to work), and "templateFunc" if a template function besides dojo.string.substitute
			//		should be used to transform the "template".
			// position:
			//		can be one of:
			//		|	"last"||"end" (default)
			//		|	"first||"start"
			//		|	"before"
			//		|	"after"
			//		|	"replace" (replaces nodes in this NodeList with new content)
			//		|	"only" (removes other children of the nodes so new content is the only child)
			//		or an offset in the childNodes property
			// example:
			//		appends content to the end if the position is omitted
			//	|	dojo.query("h3 > p").addContent("hey there!");
			// example:
			//		add something to the front of each element that has a
			//		"thinger" property:
			//	|	dojo.query("[thinger]").addContent("...", "first");
			// example:
			//		adds a header before each element of the list
			//	|	dojo.query(".note").addContent("<h4>NOTE:</h4>", "before");
			// example:
			//		add a clone of a DOM node to the end of every element in
			//		the list, removing it from its existing parent.
			//	|	dojo.query(".note").addContent(dojo.byId("foo"));
			// example:
			//		Append nodes from a templatized string.
			//		dojo.require("dojo.string");
			//		dojo.query(".note").addContent({
			//			template: '<b>${id}: </b><span>${name}</span>',
			//			id: "user332",
			//			name: "Mr. Anderson"
			//		});
			// example:
			//		Append nodes from a templatized string that also has widgets parsed.
			//		dojo.require("dojo.string");
			//		dojo.require("dojo.parser");
			//		var notes = dojo.query(".note").addContent({
			//			template: '<button dojoType="dijit.form.Button">${text}</button>',
			//			parse: true,
			//			text: "Send"
			//		});
			content = this._normalize(content, this[0]);
			for(var i = 0, node; (node = this[i]); i++){
				this._place(content, node, position, i > 0);
			}
			return this; //dojo.NodeList
		}
	});

	/*===== return dojo.NodeList; =====*/
	return NodeList;
});

},
'dojo/_base/connect':function(){
define(["./kernel", "../on", "../topic", "../aspect", "./event", "../mouse", "./sniff", "./lang", "../keys"], function(kernel, on, hub, aspect, eventModule, mouse, has, lang){
//  module:
//    dojo/_base/connect
//  summary:
//    This module defines the dojo.connect API.
//		This modules also provides keyboard event handling helpers.
//		This module exports an extension event for emulating Firefox's keypress handling.
//		However, this extension event exists primarily for backwards compatibility and
//		is not recommended. WebKit and IE uses an alternate keypress handling (only
//		firing for printable characters, to distinguish from keydown events), and most
//		consider the WebKit/IE behavior more desirable.
has.add("events-keypress-typed", function(){ // keypresses should only occur a printable character is hit
	var testKeyEvent = {charCode: 0};
	try{
		testKeyEvent = document.createEvent("KeyboardEvent");
		(testKeyEvent.initKeyboardEvent || testKeyEvent.initKeyEvent).call(testKeyEvent, "keypress", true, true, null, false, false, false, false, 9, 3);
	}catch(e){}
	return testKeyEvent.charCode == 0 && !has("opera");
});

function connect_(obj, event, context, method, dontFix){
	method = lang.hitch(context, method);
	if(!obj || !(obj.addEventListener || obj.attachEvent)){
		// it is a not a DOM node and we are using the dojo.connect style of treating a
		// method like an event, must go right to aspect
		return aspect.after(obj || kernel.global, event, method, true);
	}
	if(typeof event == "string" && event.substring(0, 2) == "on"){
		event = event.substring(2);
	}
	if(!obj){
		obj = kernel.global;
	}
	if(!dontFix){
		switch(event){
			// dojo.connect has special handling for these event types
			case "keypress":
				event = keypress;
				break;
			case "mouseenter":
				event = mouse.enter;
				break;
			case "mouseleave":
				event = mouse.leave;
				break;
		}
	}
	return on(obj, event, method, dontFix);
}

var _punctMap = {
	106:42,
	111:47,
	186:59,
	187:43,
	188:44,
	189:45,
	190:46,
	191:47,
	192:96,
	219:91,
	220:92,
	221:93,
	222:39,
	229:113
};
var evtCopyKey = has("mac") ? "metaKey" : "ctrlKey";


var _synthesizeEvent = function(evt, props){
	var faux = lang.mixin({}, evt, props);
	setKeyChar(faux);
	// FIXME: would prefer to use lang.hitch: lang.hitch(evt, evt.preventDefault);
	// but it throws an error when preventDefault is invoked on Safari
	// does Event.preventDefault not support "apply" on Safari?
	faux.preventDefault = function(){ evt.preventDefault(); };
	faux.stopPropagation = function(){ evt.stopPropagation(); };
	return faux;
};
function setKeyChar(evt){
	evt.keyChar = evt.charCode ? String.fromCharCode(evt.charCode) : '';
	evt.charOrCode = evt.keyChar || evt.keyCode;
}
var keypress;
if(has("events-keypress-typed")){
	// this emulates Firefox's keypress behavior where every keydown can correspond to a keypress
	var _trySetKeyCode = function(e, code){
		try{
			// squelch errors when keyCode is read-only
			// (e.g. if keyCode is ctrl or shift)
			return (e.keyCode = code);
		}catch(e){
			return 0;
		}
	};
	keypress = function(object, listener){
		var keydownSignal = on(object, "keydown", function(evt){
			// munge key/charCode
			var k=evt.keyCode;
			// These are Windows Virtual Key Codes
			// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/WinUI/WindowsUserInterface/UserInput/VirtualKeyCodes.asp
			var unprintable = (k!=13 || (has("ie") >= 9 && !has("quirks"))) && k!=32 && (k!=27||!has("ie")) && (k<48||k>90) && (k<96||k>111) && (k<186||k>192) && (k<219||k>222) && k!=229;
			// synthesize keypress for most unprintables and CTRL-keys
			if(unprintable||evt.ctrlKey){
				var c = unprintable ? 0 : k;
				if(evt.ctrlKey){
					if(k==3 || k==13){
						return listener.call(evt.currentTarget, evt); // IE will post CTRL-BREAK, CTRL-ENTER as keypress natively
					}else if(c>95 && c<106){
						c -= 48; // map CTRL-[numpad 0-9] to ASCII
					}else if((!evt.shiftKey)&&(c>=65&&c<=90)){
						c += 32; // map CTRL-[A-Z] to lowercase
					}else{
						c = _punctMap[c] || c; // map other problematic CTRL combinations to ASCII
					}
				}
				// simulate a keypress event
				var faux = _synthesizeEvent(evt, {type: 'keypress', faux: true, charCode: c});
				listener.call(evt.currentTarget, faux);
				if(has("ie")){
					_trySetKeyCode(evt, faux.keyCode);
				}
			}
		});
		var keypressSignal = on(object, "keypress", function(evt){
			var c = evt.charCode;
			c = c>=32 ? c : 0;
			evt = _synthesizeEvent(evt, {charCode: c, faux: true});
			return listener.call(this, evt);
		});
		return {
			remove: function(){
				keydownSignal.remove();
				keypressSignal.remove();
			}
		};
	};
}else{
	if(has("opera")){
		keypress = function(object, listener){
			return on(object, "keypress", function(evt){
				var c = evt.which;
				if(c==3){
					c=99; // Mozilla maps CTRL-BREAK to CTRL-c
				}
				// can't trap some keys at all, like INSERT and DELETE
				// there is no differentiating info between DELETE and ".", or INSERT and "-"
				c = c<32 && !evt.shiftKey ? 0 : c;
				if(evt.ctrlKey && !evt.shiftKey && c>=65 && c<=90){
					// lowercase CTRL-[A-Z] keys
					c += 32;
				}
				return listener.call(this, _synthesizeEvent(evt, { charCode: c }));
			});
		};
	}else{
		keypress = function(object, listener){
			return on(object, "keypress", function(evt){
				setKeyChar(evt);
				return listener.call(this, evt);
			});
		};
	}
}

var connect = {
	_keypress:keypress,

	connect:function(obj, event, context, method, dontFix){
		// normalize arguments
		var a=arguments, args=[], i=0;
		// if a[0] is a String, obj was omitted
		args.push(typeof a[0] == "string" ? null : a[i++], a[i++]);
		// if the arg-after-next is a String or Function, context was NOT omitted
		var a1 = a[i+1];
		args.push(typeof a1 == "string" || typeof a1 == "function" ? a[i++] : null, a[i++]);
		// absorb any additional arguments
		for(var l=a.length; i<l; i++){	args.push(a[i]); }
		return connect_.apply(this, args);
	},

	disconnect:function(handle){
		if(handle){
			handle.remove();
		}
	},

	subscribe:function(topic, context, method){
		return hub.subscribe(topic, lang.hitch(context, method));
	},

	publish:function(topic, args){
		return hub.publish.apply(hub, [topic].concat(args));
	},

	connectPublisher:function(topic, obj, event){
		var pf = function(){ connect.publish(topic, arguments); };
		return event ? connect.connect(obj, event, pf) : connect.connect(obj, pf); //Handle
	},

	isCopyKey: function(e){
		return e[evtCopyKey];	// Boolean
	}
};
connect.unsubscribe = connect.disconnect;

1 && lang.mixin(kernel, connect);
return connect;

/*=====
dojo.connect = function(obj, event, context, method, dontFix){
	// summary:
	//		`dojo.connect` is the core event handling and delegation method in
	//		Dojo. It allows one function to "listen in" on the execution of
	//		any other, triggering the second whenever the first is called. Many
	//		listeners may be attached to a function, and source functions may
	//		be either regular function calls or DOM events.
	//
	// description:
	//		Connects listeners to actions, so that after event fires, a
	//		listener is called with the same arguments passed to the original
	//		function.
	//
	//		Since `dojo.connect` allows the source of events to be either a
	//		"regular" JavaScript function or a DOM event, it provides a uniform
	//		interface for listening to all the types of events that an
	//		application is likely to deal with though a single, unified
	//		interface. DOM programmers may want to think of it as
	//		"addEventListener for everything and anything".
	//
	//		When setting up a connection, the `event` parameter must be a
	//		string that is the name of the method/event to be listened for. If
	//		`obj` is null, `kernel.global` is assumed, meaning that connections
	//		to global methods are supported but also that you may inadvertently
	//		connect to a global by passing an incorrect object name or invalid
	//		reference.
	//
	//		`dojo.connect` generally is forgiving. If you pass the name of a
	//		function or method that does not yet exist on `obj`, connect will
	//		not fail, but will instead set up a stub method. Similarly, null
	//		arguments may simply be omitted such that fewer than 4 arguments
	//		may be required to set up a connection See the examples for details.
	//
	//		The return value is a handle that is needed to
	//		remove this connection with `dojo.disconnect`.
	//
	// obj: Object|null:
	//		The source object for the event function.
	//		Defaults to `kernel.global` if null.
	//		If obj is a DOM node, the connection is delegated
	//		to the DOM event manager (unless dontFix is true).
	//
	// event: String:
	//		String name of the event function in obj.
	//		I.e. identifies a property `obj[event]`.
	//
	// context: Object|null
	//		The object that method will receive as "this".
	//
	//		If context is null and method is a function, then method
	//		inherits the context of event.
	//
	//		If method is a string then context must be the source
	//		object object for method (context[method]). If context is null,
	//		kernel.global is used.
	//
	// method: String|Function:
	//		A function reference, or name of a function in context.
	//		The function identified by method fires after event does.
	//		method receives the same arguments as the event.
	//		See context argument comments for information on method's scope.
	//
	// dontFix: Boolean?
	//		If obj is a DOM node, set dontFix to true to prevent delegation
	//		of this connection to the DOM event manager.
	//
	// example:
	//		When obj.onchange(), do ui.update():
	//	|	dojo.connect(obj, "onchange", ui, "update");
	//	|	dojo.connect(obj, "onchange", ui, ui.update); // same
	//
	// example:
	//		Using return value for disconnect:
	//	|	var link = dojo.connect(obj, "onchange", ui, "update");
	//	|	...
	//	|	dojo.disconnect(link);
	//
	// example:
	//		When onglobalevent executes, watcher.handler is invoked:
	//	|	dojo.connect(null, "onglobalevent", watcher, "handler");
	//
	// example:
	//		When ob.onCustomEvent executes, customEventHandler is invoked:
	//	|	dojo.connect(ob, "onCustomEvent", null, "customEventHandler");
	//	|	dojo.connect(ob, "onCustomEvent", "customEventHandler"); // same
	//
	// example:
	//		When ob.onCustomEvent executes, customEventHandler is invoked
	//		with the same scope (this):
	//	|	dojo.connect(ob, "onCustomEvent", null, customEventHandler);
	//	|	dojo.connect(ob, "onCustomEvent", customEventHandler); // same
	//
	// example:
	//		When globalEvent executes, globalHandler is invoked
	//		with the same scope (this):
	//	|	dojo.connect(null, "globalEvent", null, globalHandler);
	//	|	dojo.connect("globalEvent", globalHandler); // same
}
=====*/

/*=====
dojo.disconnect = function(handle){
	// summary:
	//		Remove a link created by dojo.connect.
	// description:
	//		Removes the connection between event and the method referenced by handle.
	// handle: Handle:
	//		the return value of the dojo.connect call that created the connection.
}
=====*/

/*=====
dojo.subscribe = function(topic, context, method){
	//	summary:
	//		Attach a listener to a named topic. The listener function is invoked whenever the
	//		named topic is published (see: dojo.publish).
	//		Returns a handle which is needed to unsubscribe this listener.
	//	topic: String:
	//		The topic to which to subscribe.
	//	context: Object|null:
	//		Scope in which method will be invoked, or null for default scope.
	//	method: String|Function:
	//		The name of a function in context, or a function reference. This is the function that
	//		is invoked when topic is published.
	//	example:
	//	|	dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); });
	//	|	dojo.publish("alerts", [ "read this", "hello world" ]);
}
=====*/

/*=====
dojo.unsubscribe = function(handle){
	//	summary:
	//		Remove a topic listener.
	//	handle: Handle
	//		The handle returned from a call to subscribe.
	//	example:
	//	|	var alerter = dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); };
	//	|	...
	//	|	dojo.unsubscribe(alerter);
}
=====*/

/*=====
dojo.publish = function(topic, args){
	//	summary:
	//		Invoke all listener method subscribed to topic.
	//	topic: String:
	//		The name of the topic to publish.
	//	args: Array?
	//		An array of arguments. The arguments will be applied
	//		to each topic subscriber (as first class parameters, via apply).
	//	example:
	//	|	dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); };
	//	|	dojo.publish("alerts", [ "read this", "hello world" ]);
}
=====*/

/*=====
dojo.connectPublisher = function(topic, obj, event){
	//	summary:
	//		Ensure that every time obj.event() is called, a message is published
	//		on the topic. Returns a handle which can be passed to
	//		dojo.disconnect() to disable subsequent automatic publication on
	//		the topic.
	//	topic: String:
	//		The name of the topic to publish.
	//	obj: Object|null:
	//		The source object for the event function. Defaults to kernel.global
	//		if null.
	//	event: String:
	//		The name of the event function in obj.
	//		I.e. identifies a property obj[event].
	//	example:
	//	|	dojo.connectPublisher("/ajax/start", dojo, "xhrGet");
}
=====*/

/*=====
dojo.isCopyKey = function(e){
	// summary:
	//		Checks an event for the copy key (meta on Mac, and ctrl anywhere else)
	// e: Event
	//		Event object to examine
}
=====*/

});



},
'dijit/_DialogMixin':function(){
define([
	"dojo/_base/declare", // declare
	"./a11y"	// _getTabNavigable
], function(declare, a11y){

	// module:
	//		dijit/_DialogMixin
	// summary:
	//		_DialogMixin provides functions useful to Dialog and TooltipDialog

	return declare("dijit._DialogMixin", null, {
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		execute: function(/*Object*/ /*===== formContents =====*/){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//	    Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//	    Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(){
			// summary:
			//		Finds focusable items in dialog,
			//		and sets this._firstFocusItem and this._lastFocusItem
			// tags:
			//		protected

			var elems = a11y._getTabNavigable(this.containerNode);
			this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
		}
	});
});

},
'dojo/_base/fx':function(){
define(["./kernel", "./lang", "../Evented", "./Color", "./connect", "./sniff", "../dom", "../dom-style"], function(dojo, lang, Evented, Color, connect, has, dom, style){
	// module:
	//		dojo/_base/fx
	// summary:
	//		This module defines the base dojo.fx implementation.
	// notes:
	//		Animation loosely package based on Dan Pupius' work, contributed under CLA; see
	//		http://pupius.co.uk/js/Toolkit.Drawing.js

	var _mixin = lang.mixin;

	dojo._Line = function(/*int*/ start, /*int*/ end){
		//	summary:
		//		dojo._Line is the object used to generate values from a start value
		//		to an end value
		//	start: int
		//		Beginning value for range
		//	end: int
		//		Ending value for range
		this.start = start;
		this.end = end;
	};

	dojo._Line.prototype.getValue = function(/*float*/ n){
		//	summary: Returns the point on the line
		//	n: a floating point number greater than 0 and less than 1
		return ((this.end - this.start) * n) + this.start; // Decimal
	};

	dojo.Animation = function(args){
		//	summary:
		//		A generic animation class that fires callbacks into its handlers
		//		object at various states.
		//	description:
		//		A generic animation class that fires callbacks into its handlers
		//		object at various states. Nearly all dojo animation functions
		//		return an instance of this method, usually without calling the
		//		.play() method beforehand. Therefore, you will likely need to
		//		call .play() on instances of `dojo.Animation` when one is
		//		returned.
		// args: Object
		//		The 'magic argument', mixing all the properties into this
		//		animation instance.

		_mixin(this, args);
		if(lang.isArray(this.curve)){
			this.curve = new dojo._Line(this.curve[0], this.curve[1]);
		}

	};
	dojo.Animation.prototype = new Evented();
	// Alias to drop come 2.0:
	dojo._Animation = dojo.Animation;

	lang.extend(dojo.Animation, {
		// duration: Integer
		//		The time in milliseonds the animation will take to run
		duration: 350,

	/*=====
		// curve: dojo._Line|Array
		//		A two element array of start and end values, or a `dojo._Line` instance to be
		//		used in the Animation.
		curve: null,

		// easing: Function?
		//		A Function to adjust the acceleration (or deceleration) of the progress
		//		across a dojo._Line
		easing: null,
	=====*/

		// repeat: Integer?
		//		The number of times to loop the animation
		repeat: 0,

		// rate: Integer?
		//		the time in milliseconds to wait before advancing to next frame
		//		(used as a fps timer: 1000/rate = fps)
		rate: 20 /* 50 fps */,

	/*=====
		// delay: Integer?
		//		The time in milliseconds to wait before starting animation after it
		//		has been .play()'ed
		delay: null,

		// beforeBegin: Event?
		//		Synthetic event fired before a dojo.Animation begins playing (synchronous)
		beforeBegin: null,

		// onBegin: Event?
		//		Synthetic event fired as a dojo.Animation begins playing (useful?)
		onBegin: null,

		// onAnimate: Event?
		//		Synthetic event fired at each interval of a `dojo.Animation`
		onAnimate: null,

		// onEnd: Event?
		//		Synthetic event fired after the final frame of a `dojo.Animation`
		onEnd: null,

		// onPlay: Event?
		//		Synthetic event fired any time a `dojo.Animation` is play()'ed
		onPlay: null,

		// onPause: Event?
		//		Synthetic event fired when a `dojo.Animation` is paused
		onPause: null,

		// onStop: Event
		//		Synthetic event fires when a `dojo.Animation` is stopped
		onStop: null,

	=====*/

		_percent: 0,
		_startRepeatCount: 0,

		_getStep: function(){
			var _p = this._percent,
				_e = this.easing
			;
			return _e ? _e(_p) : _p;
		},
		_fire: function(/*Event*/ evt, /*Array?*/ args){
			//	summary:
			//		Convenience function.  Fire event "evt" and pass it the
			//		arguments specified in "args".
			//	description:
			//		Convenience function.  Fire event "evt" and pass it the
			//		arguments specified in "args".
			//		Fires the callback in the scope of the `dojo.Animation`
			//		instance.
			//	evt:
			//		The event to fire.
			//	args:
			//		The arguments to pass to the event.
			var a = args||[];
			if(this[evt]){
				if(dojo.config.debugAtAllCosts){
					this[evt].apply(this, a);
				}else{
					try{
						this[evt].apply(this, a);
					}catch(e){
						// squelch and log because we shouldn't allow exceptions in
						// synthetic event handlers to cause the internal timer to run
						// amuck, potentially pegging the CPU. I'm not a fan of this
						// squelch, but hopefully logging will make it clear what's
						// going on
						console.error("exception in animation handler for:", evt);
						console.error(e);
					}
				}
			}
			return this; // dojo.Animation
		},

		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			// summary:
			//		Start the animation.
			// delay:
			//		How many milliseconds to delay before starting.
			// gotoStart:
			//		If true, starts the animation from the beginning; otherwise,
			//		starts it from its current position.
			// returns: dojo.Animation
			//		The instance to allow chaining.

			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			if(gotoStart){
				_t._stopTimer();
				_t._active = _t._paused = false;
				_t._percent = 0;
			}else if(_t._active && !_t._paused){
				return _t;
			}

			_t._fire("beforeBegin", [_t.node]);

			var de = delay || _t.delay,
				_p = lang.hitch(_t, "_play", gotoStart);

			if(de > 0){
				_t._delayTimer = setTimeout(_p, de);
				return _t;
			}
			_p();
			return _t;	// dojo.Animation
		},

		_play: function(gotoStart){
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			_t._startTime = new Date().valueOf();
			if(_t._paused){
				_t._startTime -= _t.duration * _t._percent;
			}

			_t._active = true;
			_t._paused = false;
			var value = _t.curve.getValue(_t._getStep());
			if(!_t._percent){
				if(!_t._startRepeatCount){
					_t._startRepeatCount = _t.repeat;
				}
				_t._fire("onBegin", [value]);
			}

			_t._fire("onPlay", [value]);

			_t._cycle();
			return _t; // dojo.Animation
		},

		pause: function(){
			// summary: Pauses a running animation.
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			_t._stopTimer();
			if(!_t._active){ return _t; /*dojo.Animation*/ }
			_t._paused = true;
			_t._fire("onPause", [_t.curve.getValue(_t._getStep())]);
			return _t; // dojo.Animation
		},

		gotoPercent: function(/*Decimal*/ percent, /*Boolean?*/ andPlay){
			//	summary:
			//		Sets the progress of the animation.
			//	percent:
			//		A percentage in decimal notation (between and including 0.0 and 1.0).
			//	andPlay:
			//		If true, play the animation after setting the progress.
			var _t = this;
			_t._stopTimer();
			_t._active = _t._paused = true;
			_t._percent = percent;
			if(andPlay){ _t.play(); }
			return _t; // dojo.Animation
		},

		stop: function(/*boolean?*/ gotoEnd){
			// summary: Stops a running animation.
			// gotoEnd: If true, the animation will end.
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			if(!_t._timer){ return _t; /* dojo.Animation */ }
			_t._stopTimer();
			if(gotoEnd){
				_t._percent = 1;
			}
			_t._fire("onStop", [_t.curve.getValue(_t._getStep())]);
			_t._active = _t._paused = false;
			return _t; // dojo.Animation
		},

		status: function(){
			// summary:
			//		Returns a string token representation of the status of
			//		the animation, one of: "paused", "playing", "stopped"
			if(this._active){
				return this._paused ? "paused" : "playing"; // String
			}
			return "stopped"; // String
		},

		_cycle: function(){
			var _t = this;
			if(_t._active){
				var curr = new Date().valueOf();
				var step = (curr - _t._startTime) / (_t.duration);

				if(step >= 1){
					step = 1;
				}
				_t._percent = step;

				// Perform easing
				if(_t.easing){
					step = _t.easing(step);
				}

				_t._fire("onAnimate", [_t.curve.getValue(step)]);

				if(_t._percent < 1){
					_t._startTimer();
				}else{
					_t._active = false;

					if(_t.repeat > 0){
						_t.repeat--;
						_t.play(null, true);
					}else if(_t.repeat == -1){
						_t.play(null, true);
					}else{
						if(_t._startRepeatCount){
							_t.repeat = _t._startRepeatCount;
							_t._startRepeatCount = 0;
						}
					}
					_t._percent = 0;
					_t._fire("onEnd", [_t.node]);
					!_t.repeat && _t._stopTimer();
				}
			}
			return _t; // dojo.Animation
		},

		_clearTimer: function(){
			// summary: Clear the play delay timer
			clearTimeout(this._delayTimer);
			delete this._delayTimer;
		}

	});

	// the local timer, stubbed into all Animation instances
	var ctr = 0,
		timer = null,
		runner = {
			run: function(){}
		};

	lang.extend(dojo.Animation, {

		_startTimer: function(){
			if(!this._timer){
				this._timer = connect.connect(runner, "run", this, "_cycle");
				ctr++;
			}
			if(!timer){
				timer = setInterval(lang.hitch(runner, "run"), this.rate);
			}
		},

		_stopTimer: function(){
			if(this._timer){
				connect.disconnect(this._timer);
				this._timer = null;
				ctr--;
			}
			if(ctr <= 0){
				clearInterval(timer);
				timer = null;
				ctr = 0;
			}
		}

	});

	var _makeFadeable =
				has("ie") ? function(node){
			// only set the zoom if the "tickle" value would be the same as the
			// default
			var ns = node.style;
			// don't set the width to auto if it didn't already cascade that way.
			// We don't want to f anyones designs
			if(!ns.width.length && style.get(node, "width") == "auto"){
				ns.width = "auto";
			}
		} :
				function(){};

	dojo._fade = function(/*Object*/ args){
		//	summary:
		//		Returns an animation that will fade the node defined by
		//		args.node from the start to end values passed (args.start
		//		args.end) (end is mandatory, start is optional)

		args.node = dom.byId(args.node);
		var fArgs = _mixin({ properties: {} }, args),
			props = (fArgs.properties.opacity = {});

		props.start = !("start" in fArgs) ?
			function(){
				return +style.get(fArgs.node, "opacity")||0;
			} : fArgs.start;
		props.end = fArgs.end;

		var anim = dojo.animateProperty(fArgs);
		connect.connect(anim, "beforeBegin", lang.partial(_makeFadeable, fArgs.node));

		return anim; // dojo.Animation
	};

	/*=====
	dojo.__FadeArgs = function(node, duration, easing){
		//	node: DOMNode|String
		//		The node referenced in the animation
		//	duration: Integer?
		//		Duration of the animation in milliseconds.
		//	easing: Function?
		//		An easing function.
		this.node = node;
		this.duration = duration;
		this.easing = easing;
	}
	=====*/

	dojo.fadeIn = function(/*dojo.__FadeArgs*/ args){
		// summary:
		//		Returns an animation that will fade node defined in 'args' from
		//		its current opacity to fully opaque.
		return dojo._fade(_mixin({ end: 1 }, args)); // dojo.Animation
	};

	dojo.fadeOut = function(/*dojo.__FadeArgs*/ args){
		// summary:
		//		Returns an animation that will fade node defined in 'args'
		//		from its current opacity to fully transparent.
		return dojo._fade(_mixin({ end: 0 }, args)); // dojo.Animation
	};

	dojo._defaultEasing = function(/*Decimal?*/ n){
		// summary: The default easing function for dojo.Animation(s)
		return 0.5 + ((Math.sin((n + 1.5) * Math.PI)) / 2);	// Decimal
	};

	var PropLine = function(properties){
		// PropLine is an internal class which is used to model the values of
		// an a group of CSS properties across an animation lifecycle. In
		// particular, the "getValue" function handles getting interpolated
		// values between start and end for a particular CSS value.
		this._properties = properties;
		for(var p in properties){
			var prop = properties[p];
			if(prop.start instanceof Color){
				// create a reusable temp color object to keep intermediate results
				prop.tempColor = new Color();
			}
		}
	};

	PropLine.prototype.getValue = function(r){
		var ret = {};
		for(var p in this._properties){
			var prop = this._properties[p],
				start = prop.start;
			if(start instanceof Color){
				ret[p] = Color.blendColors(start, prop.end, r, prop.tempColor).toCss();
			}else if(!lang.isArray(start)){
				ret[p] = ((prop.end - start) * r) + start + (p != "opacity" ? prop.units || "px" : 0);
			}
		}
		return ret;
	};

	/*=====
	dojo.declare("dojo.__AnimArgs", [dojo.__FadeArgs], {
		// Properties: Object?
		//	A hash map of style properties to Objects describing the transition,
		//	such as the properties of dojo._Line with an additional 'units' property
		properties: {}

		//TODOC: add event callbacks
	});
	=====*/

	dojo.animateProperty = function(/*dojo.__AnimArgs*/ args){
		// summary:
		//		Returns an animation that will transition the properties of
		//		node defined in `args` depending how they are defined in
		//		`args.properties`
		//
		// description:
		//		`dojo.animateProperty` is the foundation of most `dojo.fx`
		//		animations. It takes an object of "properties" corresponding to
		//		style properties, and animates them in parallel over a set
		//		duration.
		//
		// example:
		//		A simple animation that changes the width of the specified node.
		//	|	dojo.animateProperty({
		//	|		node: "nodeId",
		//	|		properties: { width: 400 },
		//	|	}).play();
		//		Dojo figures out the start value for the width and converts the
		//		integer specified for the width to the more expressive but
		//		verbose form `{ width: { end: '400', units: 'px' } }` which you
		//		can also specify directly. Defaults to 'px' if ommitted.
		//
		// example:
		//		Animate width, height, and padding over 2 seconds... the
		//		pedantic way:
		//	|	dojo.animateProperty({ node: node, duration:2000,
		//	|		properties: {
		//	|			width: { start: '200', end: '400', units:"px" },
		//	|			height: { start:'200', end: '400', units:"px" },
		//	|			paddingTop: { start:'5', end:'50', units:"px" }
		//	|		}
		//	|	}).play();
		//		Note 'paddingTop' is used over 'padding-top'. Multi-name CSS properties
		//		are written using "mixed case", as the hyphen is illegal as an object key.
		//
		// example:
		//		Plug in a different easing function and register a callback for
		//		when the animation ends. Easing functions accept values between
		//		zero and one and return a value on that basis. In this case, an
		//		exponential-in curve.
		//	|	dojo.animateProperty({
		//	|		node: "nodeId",
		//	|		// dojo figures out the start value
		//	|		properties: { width: { end: 400 } },
		//	|		easing: function(n){
		//	|			return (n==0) ? 0 : Math.pow(2, 10 * (n - 1));
		//	|		},
		//	|		onEnd: function(node){
		//	|			// called when the animation finishes. The animation
		//	|			// target is passed to this function
		//	|		}
		//	|	}).play(500); // delay playing half a second
		//
		// example:
		//		Like all `dojo.Animation`s, animateProperty returns a handle to the
		//		Animation instance, which fires the events common to Dojo FX. Use `dojo.connect`
		//		to access these events outside of the Animation definiton:
		//	|	var anim = dojo.animateProperty({
		//	|		node:"someId",
		//	|		properties:{
		//	|			width:400, height:500
		//	|		}
		//	|	});
		//	|	dojo.connect(anim,"onEnd", function(){
		//	|		console.log("animation ended");
		//	|	});
		//	|	// play the animation now:
		//	|	anim.play();
		//
		// example:
		//		Each property can be a function whose return value is substituted along.
		//		Additionally, each measurement (eg: start, end) can be a function. The node
		//		reference is passed direcly to callbacks.
		//	|	dojo.animateProperty({
		//	|		node:"mine",
		//	|		properties:{
		//	|			height:function(node){
		//	|				// shrink this node by 50%
		//	|				return dojo.position(node).h / 2
		//	|			},
		//	|			width:{
		//	|				start:function(node){ return 100; },
		//	|				end:function(node){ return 200; }
		//	|			}
		//	|		}
		//	|	}).play();
		//

		var n = args.node = dom.byId(args.node);
		if(!args.easing){ args.easing = dojo._defaultEasing; }

		var anim = new dojo.Animation(args);
		connect.connect(anim, "beforeBegin", anim, function(){
			var pm = {};
			for(var p in this.properties){
				// Make shallow copy of properties into pm because we overwrite
				// some values below. In particular if start/end are functions
				// we don't want to overwrite them or the functions won't be
				// called if the animation is reused.
				if(p == "width" || p == "height"){
					this.node.display = "block";
				}
				var prop = this.properties[p];
				if(lang.isFunction(prop)){
					prop = prop(n);
				}
				prop = pm[p] = _mixin({}, (lang.isObject(prop) ? prop: { end: prop }));

				if(lang.isFunction(prop.start)){
					prop.start = prop.start(n);
				}
				if(lang.isFunction(prop.end)){
					prop.end = prop.end(n);
				}
				var isColor = (p.toLowerCase().indexOf("color") >= 0);
				function getStyle(node, p){
					// dojo.style(node, "height") can return "auto" or "" on IE; this is more reliable:
					var v = { height: node.offsetHeight, width: node.offsetWidth }[p];
					if(v !== undefined){ return v; }
					v = style.get(node, p);
					return (p == "opacity") ? +v : (isColor ? v : parseFloat(v));
				}
				if(!("end" in prop)){
					prop.end = getStyle(n, p);
				}else if(!("start" in prop)){
					prop.start = getStyle(n, p);
				}

				if(isColor){
					prop.start = new Color(prop.start);
					prop.end = new Color(prop.end);
				}else{
					prop.start = (p == "opacity") ? +prop.start : parseFloat(prop.start);
				}
			}
			this.curve = new PropLine(pm);
		});
		connect.connect(anim, "onAnimate", lang.hitch(style, "set", anim.node));
		return anim; // dojo.Animation
	};

	dojo.anim = function(	/*DOMNode|String*/	node,
							/*Object*/			properties,
							/*Integer?*/		duration,
							/*Function?*/		easing,
							/*Function?*/		onEnd,
							/*Integer?*/		delay){
		//	summary:
		//		A simpler interface to `dojo.animateProperty()`, also returns
		//		an instance of `dojo.Animation` but begins the animation
		//		immediately, unlike nearly every other Dojo animation API.
		//	description:
		//		`dojo.anim` is a simpler (but somewhat less powerful) version
		//		of `dojo.animateProperty`.  It uses defaults for many basic properties
		//		and allows for positional parameters to be used in place of the
		//		packed "property bag" which is used for other Dojo animation
		//		methods.
		//
		//		The `dojo.Animation` object returned from `dojo.anim` will be
		//		already playing when it is returned from this function, so
		//		calling play() on it again is (usually) a no-op.
		//	node:
		//		a DOM node or the id of a node to animate CSS properties on
		//	duration:
		//		The number of milliseconds over which the animation
		//		should run. Defaults to the global animation default duration
		//		(350ms).
		//	easing:
		//		An easing function over which to calculate acceleration
		//		and deceleration of the animation through its duration.
		//		A default easing algorithm is provided, but you may
		//		plug in any you wish. A large selection of easing algorithms
		//		are available in `dojo.fx.easing`.
		//	onEnd:
		//		A function to be called when the animation finishes
		//		running.
		//	delay:
		//		The number of milliseconds to delay beginning the
		//		animation by. The default is 0.
		//	example:
		//		Fade out a node
		//	|	dojo.anim("id", { opacity: 0 });
		//	example:
		//		Fade out a node over a full second
		//	|	dojo.anim("id", { opacity: 0 }, 1000);
		return dojo.animateProperty({ // dojo.Animation
			node: node,
			duration: duration || dojo.Animation.prototype.duration,
			properties: properties,
			easing: easing,
			onEnd: onEnd
		}).play(delay || 0);
	};

	return {
		_Line: dojo._Line,
		Animation: dojo.Animation,
		_fade: dojo._fade,
		fadeIn: dojo.fadeIn,
		fadeOut: dojo.fadeOut,
		_defaultEasing: dojo._defaultEasing,
		animateProperty: dojo.animateProperty,
		anim: dojo.anim
	};
});

},
'dojo/_base/config':function(){
define(["../has", "require"], function(has, require){
	// module:
	//		dojo/_base/config
	// summary:
	//		This module defines the user configuration during bootstrap.
	// description:
	//		By defining user configuration as a module value, an entire configuration can be specified in a build,
    //		thereby eliminating the need for sniffing and or explicitly setting in the global variable dojoConfig.
    //		Also, when multiple instances of dojo exist in a single application, each will necessarily be located
	//		at an unique absolute module identifier as given by the package configuration. Implementing configuration
	//		as a module allows for specifying unique, per-instance configurations.
	// example:
	//		Create a second instance of dojo with a different, instance-uniqe configuration (assume the loader and
	//		dojo.js are already loaded).
	//		|	// specify a configuration that creates a new instance of dojo at the absolute module identifier "myDojo"
	//		|	require({
	//		|		packages:[{
	//		|			name:"myDojo",
	//		|			location:".", //assume baseUrl points to dojo.js
	//		|	    }]
	//		|	});
	//		|
	//		|	// specify a configuration for the myDojo instance
	//		|	define("myDojo/config", {
	//		|		// normal configuration variables go here, e.g.,
	//		|		locale:"fr-ca"
	//		|	});
	//		|
	//		|	// load and use the new instance of dojo
	//		|	require(["myDojo"], function(dojo) {
	//		|		// dojo is the new instance of dojo
	//		|		// use as required
	//		|	});

	var result = {};
	if(1){
		// must be the dojo loader; take a shallow copy of require.rawConfig
		var src = require.rawConfig, p;
		for(p in src){
			result[p] = src[p];
		}
	}else{
		var adviseHas = function(featureSet, prefix, booting){
			for(p in featureSet){
				p!="has" && has.add(prefix + p, featureSet[p], 0, booting);
			}
		};
		result = 1 ?
			// must be a built version of the dojo loader; all config stuffed in require.rawConfig
			require.rawConfig :
			// a foreign loader
			this.dojoConfig || this.djConfig || {};
		adviseHas(result, "config", 1);
		adviseHas(result.has, "", 1);
	}
	return result;

/*=====
// note:
//		'dojoConfig' does not exist under 'dojo.*' so that it can be set before the
//		'dojo' variable exists.
// note:
//		Setting any of these variables *after* the library has loaded does
//		nothing at all.

// FIXME: can we document these on dojo.config object and explain they must be set via djConfig/dojoConfig global prior to loading dojo.js

dojoConfig = {
	// summary:
	//		Application code can set the global 'dojoConfig' prior to loading
	//		the library to control certain global settings for how dojo works.
	//
	// isDebug: Boolean
	//		Defaults to `false`. If set to `true`, ensures that Dojo provides
	//		extended debugging feedback via Firebug. If Firebug is not available
	//		on your platform, setting `isDebug` to `true` will force Dojo to
	//		pull in (and display) the version of Firebug Lite which is
	//		integrated into the Dojo distribution, thereby always providing a
	//		debugging/logging console when `isDebug` is enabled. Note that
	//		Firebug's `console.*` methods are ALWAYS defined by Dojo. If
	//		`isDebug` is false and you are on a platform without Firebug, these
	//		methods will be defined as no-ops.
	isDebug: false,
	// locale: String
	//		The locale to assume for loading localized resources in this page,
	//		specified according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt).
	//		Must be specified entirely in lowercase, e.g. `en-us` and `zh-cn`.
	//		See the documentation for `dojo.i18n` and `dojo.requireLocalization`
	//		for details on loading localized resources. If no locale is specified,
	//		Dojo assumes the locale of the user agent, according to `navigator.userLanguage`
	//		or `navigator.language` properties.
	locale: undefined,
	// extraLocale: Array
	//		No default value. Specifies additional locales whose
	//		resources should also be loaded alongside the default locale when
	//		calls to `dojo.requireLocalization()` are processed.
	extraLocale: undefined,
	// baseUrl: String
	//		The directory in which `dojo.js` is located. Under normal
	//		conditions, Dojo auto-detects the correct location from which it
	//		was loaded. You may need to manually configure `baseUrl` in cases
	//		where you have renamed `dojo.js` or in which `<base>` tags confuse
	//		some browsers (e.g. IE 6). The variable `dojo.baseUrl` is assigned
	//		either the value of `djConfig.baseUrl` if one is provided or the
	//		auto-detected root if not. Other modules are located relative to
	//		this path. The path should end in a slash.
	baseUrl: undefined,
	// modulePaths: Object
	//		A map of module names to paths relative to `dojo.baseUrl`. The
	//		key/value pairs correspond directly to the arguments which
	//		`dojo.registerModulePath` accepts. Specifiying
	//		`djConfig.modulePaths = { "foo": "../../bar" }` is the equivalent
	//		of calling `dojo.registerModulePath("foo", "../../bar");`. Multiple
	//		modules may be configured via `djConfig.modulePaths`.
	modulePaths: {},
	// afterOnLoad: Boolean
	//		Indicates Dojo was added to the page after the page load. In this case
	//		Dojo will not wait for the page DOMContentLoad/load events and fire
	//		its dojo.addOnLoad callbacks after making sure all outstanding
	//		dojo.required modules have loaded. Only works with a built dojo.js,
	//		it does not work the dojo.js directly from source control.
	afterOnLoad: false,
	// addOnLoad: Function or Array
	//		Adds a callback via dojo.addOnLoad. Useful when Dojo is added after
	//		the page loads and djConfig.afterOnLoad is true. Supports the same
	//		arguments as dojo.addOnLoad. When using a function reference, use
	//		`djConfig.addOnLoad = function(){};`. For object with function name use
	//		`djConfig.addOnLoad = [myObject, "functionName"];` and for object with
	//		function reference use
	//		`djConfig.addOnLoad = [myObject, function(){}];`
	addOnLoad: null,
	// require: Array
	//		An array of module names to be loaded immediately after dojo.js has been included
	//		in a page.
	require: [],
	// defaultDuration: Array
	//		Default duration, in milliseconds, for wipe and fade animations within dijits.
	//		Assigned to dijit.defaultDuration.
	defaultDuration: 200,
	// dojoBlankHtmlUrl: String
	//		Used by some modules to configure an empty iframe. Used by dojo.io.iframe and
	//		dojo.back, and dijit popup support in IE where an iframe is needed to make sure native
	//		controls do not bleed through the popups. Normally this configuration variable
	//		does not need to be set, except when using cross-domain/CDN Dojo builds.
	//		Save dojo/resources/blank.html to your domain and set `djConfig.dojoBlankHtmlUrl`
	//		to the path on your domain your copy of blank.html.
	dojoBlankHtmlUrl: undefined,
	//	ioPublish: Boolean?
	//		Set this to true to enable publishing of topics for the different phases of
	//		IO operations. Publishing is done via dojo.publish. See dojo.__IoPublish for a list
	//		of topics that are published.
	ioPublish: false,
	//	useCustomLogger: Anything?
	//		If set to a value that evaluates to true such as a string or array and
	//		isDebug is true and Firebug is not available or running, then it bypasses
	//		the creation of Firebug Lite allowing you to define your own console object.
	useCustomLogger: undefined,
	// transparentColor: Array
	//		Array containing the r, g, b components used as transparent color in dojo.Color;
	//		if undefined, [255,255,255] (white) will be used.
	transparentColor: undefined,
	// skipIeDomLoaded: Boolean
	//		For IE only, skip the DOMContentLoaded hack used. Sometimes it can cause an Operation
	//		Aborted error if the rest of the page triggers script defers before the DOM is ready.
	//		If this is config value is set to true, then dojo.addOnLoad callbacks will not be
	//		triggered until the page load event, which is after images and iframes load. If you
	//		want to trigger the callbacks sooner, you can put a script block in the bottom of
	//		your HTML that calls dojo._loadInit();. If you are using multiversion support, change
	//		"dojo." to the appropriate scope name for dojo.
	skipIeDomLoaded: false
}
=====*/
});


},
'dijit/layout/utils':function(){
define([
	"dojo/_base/array", // array.filter array.forEach
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/lang", // lang.mixin
	".."	// for exporting symbols to dijit, remove in 2.0
], function(array, domClass, domGeometry, domStyle, lang, dijit){

	// module:
	//		dijit/layout/utils
	// summary:
	//		marginBox2contentBox() and layoutChildren()

	var layout = lang.getObject("layout", true, dijit);
	/*===== layout = dijit.layout =====*/

	layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
		// summary:
		//		Given the margin-box size of a node, return its content box size.
		//		Functions like domGeometry.contentBox() but is more reliable since it doesn't have
		//		to wait for the browser to compute sizes.
		var cs = domStyle.getComputedStyle(node);
		var me = domGeometry.getMarginExtents(node, cs);
		var pb = domGeometry.getPadBorderExtents(node, cs);
		return {
			l: domStyle.toPixelValue(node, cs.paddingLeft),
			t: domStyle.toPixelValue(node, cs.paddingTop),
			w: mb.w - (me.w + pb.w),
			h: mb.h - (me.h + pb.h)
		};
	};

	function capitalize(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	}

	function size(widget, dim){
		// size the child
		var newSize = widget.resize ? widget.resize(dim) : domGeometry.setMarginBox(widget.domNode, dim);

		// record child's size
		if(newSize){
			// if the child returned it's new size then use that
			lang.mixin(widget, newSize);
		}else{
			// otherwise, call getMarginBox(), but favor our own numbers when we have them.
			// the browser lies sometimes
			lang.mixin(widget, domGeometry.getMarginBox(widget.domNode));
			lang.mixin(widget, dim);
		}
	}

	layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Widget[]*/ children,
			/*String?*/ changedRegionId, /*Number?*/ changedRegionSize){
		// summary:
		//		Layout a bunch of child dom nodes within a parent dom node
		// container:
		//		parent node
		// dim:
		//		{l, t, w, h} object specifying dimensions of container into which to place children
		// children:
		//		an array of Widgets or at least objects containing:
		//			* domNode: pointer to DOM node to position
		//			* region or layoutAlign: position to place DOM node
		//			* resize(): (optional) method to set size of node
		//			* id: (optional) Id of widgets, referenced from resize object, below.
		// changedRegionId:
		//		If specified, the slider for the region with the specified id has been dragged, and thus
		//		the region's height or width should be adjusted according to changedRegionSize
		// changedRegionSize:
		//		See changedRegionId.

		// copy dim because we are going to modify it
		dim = lang.mixin({}, dim);

		domClass.add(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.    TODO: move these lines to LayoutContainer?   Unneeded other places I think.
		children = array.filter(children, function(item){ return item.region != "center" && item.layoutAlign != "client"; })
			.concat(array.filter(children, function(item){ return item.region == "center" || item.layoutAlign == "client"; }));

		// set positions/sizes
		array.forEach(children, function(child){
			var elm = child.domNode,
				pos = (child.region || child.layoutAlign);
			if(!pos){
				throw new Error("No region setting for " + child.id)
			}

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.position = "absolute";

			domClass.add(elm, "dijitAlign" + capitalize(pos));

			// Size adjustments to make to this child widget
			var sizeSetting = {};

			// Check for optional size adjustment due to splitter drag (height adjustment for top/bottom align
			// panes and width adjustment for left/right align panes.
			if(changedRegionId && changedRegionId == child.id){
				sizeSetting[child.region == "top" || child.region == "bottom" ? "h" : "w"] = changedRegionSize;
			}

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect its height.
			if(pos == "top" || pos == "bottom"){
				sizeSetting.w = dim.w;
				size(child, sizeSetting);
				dim.h -= child.h;
				if(pos == "top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos == "left" || pos == "right"){
				sizeSetting.h = dim.h;
				size(child, sizeSetting);
				dim.w -= child.w;
				if(pos == "left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos == "client" || pos == "center"){
				size(child, dim);
			}
		});
	};


	return {
		marginBox2contentBox: layout.marginBox2contentBox,
		layoutChildren: layout.layoutChildren
	};
});

},
'dojo/_base/unload':function(){
define(["./kernel", "./connect"], function(dojo, connect) {
	// module:
	//		dojo/unload
	// summary:
	//		This module contains the document and window unload detection API.

	var win = window;

	/*=====
		dojo.windowUnloaded = function(){
			// summary:
			//		signal fired by impending window destruction. You may use
			//		dojo.addOnWindowUnload() to register a listener for this
			//		event. NOTE: if you wish to dojo.connect() to this method
			//		to perform page/application cleanup, be aware that this
			//		event WILL NOT fire if no handler has been registered with
			//		dojo.addOnWindowUnload. This behavior started in Dojo 1.3.
			//		Previous versions always triggered dojo.windowUnloaded. See
			//		dojo.addOnWindowUnload for more info.
		};
	=====*/

	dojo.addOnWindowUnload = function(/*Object?|Function?*/obj, /*String|Function?*/functionName){
		// summary:
		//		registers a function to be triggered when window.onunload
		//		fires.
		//	description:
		//		The first time that addOnWindowUnload is called Dojo
		//		will register a page listener to trigger your unload
		//		handler with. Note that registering these handlers may
		//		destory "fastback" page caching in browsers that support
		//		it. Be careful trying to modify the DOM or access
		//		JavaScript properties during this phase of page unloading:
		//		they may not always be available. Consider
		//		dojo.addOnUnload() if you need to modify the DOM or do
		//		heavy JavaScript work since it fires at the eqivalent of
		//		the page's "onbeforeunload" event.
		// example:
		//	| dojo.addOnWindowUnload(functionPointer)
		//	| dojo.addOnWindowUnload(object, "functionName");
		//	| dojo.addOnWindowUnload(object, function(){ /* ... */});

		if (!dojo.windowUnloaded) {
			connect.connect(win, "unload", (dojo.windowUnloaded= function(){}));
		}
		connect.connect(win, "unload", obj, functionName);
	};

	dojo.addOnUnload = function(/*Object?|Function?*/obj, /*String|Function?*/functionName){
		// summary:
		//		registers a function to be triggered when the page unloads.
		//	description:
		//		The first time that addOnUnload is called Dojo will
		//		register a page listener to trigger your unload handler
		//		with.
		//
		//		In a browser enviroment, the functions will be triggered
		//		during the window.onbeforeunload event. Be careful of doing
		//		too much work in an unload handler. onbeforeunload can be
		//		triggered if a link to download a file is clicked, or if
		//		the link is a javascript: link. In these cases, the
		//		onbeforeunload event fires, but the document is not
		//		actually destroyed. So be careful about doing destructive
		//		operations in a dojo.addOnUnload callback.
		//
		//		Further note that calling dojo.addOnUnload will prevent
		//		browsers from using a "fast back" cache to make page
		//		loading via back button instantaneous.
		// example:
		//	| dojo.addOnUnload(functionPointer)
		//	| dojo.addOnUnload(object, "functionName")
		//	| dojo.addOnUnload(object, function(){ /* ... */});

		connect.connect(win, "beforeunload", obj, functionName);
	};

	return {
		addOnWindowUnload: dojo.addOnWindowUnload,
		addOnUnload: dojo.addOnUnload
	};
});

},
'dojo/selector/_loader':function(){
define(["../has", "require"],
		function(has, require){
// summary:
//		This module handles loading the appropriate selector engine for the given browser
"use strict";
var testDiv = document.createElement("div");
has.add("dom-qsa2.1", !!testDiv.querySelectorAll);
has.add("dom-qsa3", function(){
			// test to see if we have a reasonable native selector engine available
			try{
				testDiv.innerHTML = "<p class='TEST'></p>"; // test kind of from sizzle
				// Safari can't handle uppercase or unicode characters when
				// in quirks mode, IE8 can't handle pseudos like :empty
				return testDiv.querySelectorAll(".TEST:empty").length == 1;
			}catch(e){}
		});
var fullEngine;
var acme = "./acme", lite = "./lite";
return {
	load: function(id, parentRequire, loaded, config){
		var req = require;
		// here we implement the default logic for choosing a selector engine
		id = id == "default" ? has("config-selectorEngine") || "css3" : id;
		id = id == "css2" || id == "lite" ? lite :
				id == "css2.1" ? has("dom-qsa2.1") ? lite : acme :
				id == "css3" ? has("dom-qsa3") ? lite : acme :
				id == "acme" ? acme : (req = parentRequire) && id;
		if(id.charAt(id.length-1) == '?'){
			id = id.substring(0,id.length - 1);
			var optionalLoad = true;
		}
		// the query engine is optional, only load it if a native one is not available or existing one has not been loaded
		if(optionalLoad && (has("dom-compliant-qsa") || fullEngine)){
			return loaded(fullEngine);
		}
		// load the referenced selector engine
		req([id], function(engine){
			if(id != "./lite"){
				fullEngine = engine;
			}
			loaded(engine);
		});
	}
};
});

},
'dojox/date/hebrew/numerals':function(){
define(["dojo/_base/kernel", "dojo/_base/array"], function(dojo){
	dojo.getObject("date.hebrew.numerals", true, dojox);
	dojo.experimental("dojox.date.hebrew.numerals");

//Conversion from "Hindi" numerals to Hebrew numerals and vice versa

	var DIG="אבגדהוזחט";
	var	TEN="יכלמנסעפצ";
	var	HUN="קרשת";

	var transformChars = function(str, nogrsh){
		str = str.replace("יה", "טו").replace("יו", "טז");

		if(!nogrsh){
			var len = str.length;
			if(len > 1){
				str = str.substr(0, len - 1) + '"' + str.charAt(len - 1);
			}else{
				str += "\u05F3"; // 05F3:geresh
			}
		}
		return str; // String
	};
	 
	var parseStrToNumber = function(str){
		var num = 0;
		dojo.forEach(str, function(ch){
			var i;
			if((i = DIG.indexOf(ch)) != -1){
				num += ++i;
			}else if((i = TEN.indexOf(ch)) != -1){
				num += 10 * ++i;
			}else if((i = HUN.indexOf(ch)) != -1){
				num += 100 * ++i;
			}
		});
		return num; //Number
	};
	 
	var convertNumberToStr = function(num){
		var str  = "", n = 4, j = 9;
  		while(num){
			if(num >= n*100){
				str += HUN.charAt(n-1);
				num -= n*100;
				continue;
			}else if(n > 1){
				n--;
				continue;
			}else if(num >= j*10){
				str += TEN.charAt(j-1);
				num -= j*10;
			}else if(j > 1){
				j--;
				continue;
			}else if(num > 0){
				str += DIG.charAt(num-1);
				num = 0;
			}
		}
		return str; //String
	};

	dojox.date.hebrew.numerals.getYearHebrewLetters = function(/*Number */ year){
		// summary: converts the year from an integer to Hebrew numerals.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getYearHebrewLetters(date1.getFullYear());
		
		var rem = year % 1000;
		//FIXME: tests include dates outside this range and seem to pass.
		//	    	if((year - rem) / 1000 != 5){ throw new Error("Hebrew year "+year+" is not in range 5001-5999");}
		return transformChars(convertNumberToStr(rem)); // String
	};
	
	dojox.date.hebrew.numerals.parseYearHebrewLetters  = function(/*String hebrew year*/ year){
		// summary: converts the year written in Hebrew numerals to an integer
		//
		// example:
		// |		var date = new dojox.date.hebrew.Date();
		// |        	date.setFullYear(dojox.date.hebrew.numerals.parseYearHebrewLetters('\u05ea\u05e9\u05e1\u05f4\u05d7'));

		return parseStrToNumber(year) + 5000; // int
	};
	
	dojox.date.hebrew.numerals.getDayHebrewLetters =  function(day, /*boolean?*/ nogrsh){
		// summary: converts an integer to a String representing the number in Hebrew numerals.   Can be formatted with or without geresh &#x05f3;
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getDayHebrewLetters(date1.getDay());

		return transformChars(convertNumberToStr(day), nogrsh); // String
	};
	
	dojox.date.hebrew.numerals.parseDayHebrewLetters =  function(/*String hebrew*/ day){
		// summary: converts the string containing a Hebrew numeral to an integer
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		date1.setDate(dojox.date.hebrew.numerals.parseDayHebrewLetters('\u05d0')); // ALEPH
		return parseStrToNumber(day); // int
	};

	dojox.date.hebrew.numerals.getMonthHebrewLetters =  function(/*int*/month){
		// summary: converts an integer representing a  month to a String written in Hebrew numerals
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getMonthHebrewLetters(date1.getMonth());

		return transformChars(convertNumberToStr(month+1)); // String
	};

	dojox.date.hebrew.numerals.parseMonthHebrewLetters = function(/*String*/monthStr){
		// summary: converts a Hebrew numeral string representing
		//	a month to an integer.  The returned value
		//	is indexed in the month name array.  To use it for
		//	setMonth, do correction for leap year
		//
		// example:
		// |		var date = new dojox.date.hebrew.Date();
		// |            var number = dojox.date.hebrew.numerals.parseMonthHebrewLetters("\u05ea\u05de\u05d5\u05d6"); // Tammuz
		// |		date.setMonth(number);
			
		//month number from 0 to 12
		var monnum = dojox.date.hebrew.numerals.parseDayHebrewLetters(monthStr) - 1;

		if(monnum == -1 || monnum > 12){
			throw new Error("The month name is incorrect , month = " + monnum);
		}
		return monnum;
	};
	return dojox.date.hebrew.numerals;
});

},
'dojo/_base/declare':function(){
define(["./kernel", "../has", "./lang"], function(dojo, has, lang){
	// module:
	//		dojo/_base/declare
	// summary:
	//		This module defines dojo.declare.

	var mix = lang.mixin, op = Object.prototype, opts = op.toString,
		xtor = new Function, counter = 0, cname = "constructor";

	function err(msg, cls){ throw new Error("declare" + (cls ? " " + cls : "") + ": " + msg); }

	// C3 Method Resolution Order (see http://www.python.org/download/releases/2.3/mro/)
	function c3mro(bases, className){
		var result = [], roots = [{cls: 0, refs: []}], nameMap = {}, clsCount = 1,
			l = bases.length, i = 0, j, lin, base, top, proto, rec, name, refs;

		// build a list of bases naming them if needed
		for(; i < l; ++i){
			base = bases[i];
			if(!base){
				err("mixin #" + i + " is unknown. Did you use dojo.require to pull it in?", className);
			}else if(opts.call(base) != "[object Function]"){
				err("mixin #" + i + " is not a callable constructor.", className);
			}
			lin = base._meta ? base._meta.bases : [base];
			top = 0;
			// add bases to the name map
			for(j = lin.length - 1; j >= 0; --j){
				proto = lin[j].prototype;
				if(!proto.hasOwnProperty("declaredClass")){
					proto.declaredClass = "uniqName_" + (counter++);
				}
				name = proto.declaredClass;
				if(!nameMap.hasOwnProperty(name)){
					nameMap[name] = {count: 0, refs: [], cls: lin[j]};
					++clsCount;
				}
				rec = nameMap[name];
				if(top && top !== rec){
					rec.refs.push(top);
					++top.count;
				}
				top = rec;
			}
			++top.count;
			roots[0].refs.push(top);
		}

		// remove classes without external references recursively
		while(roots.length){
			top = roots.pop();
			result.push(top.cls);
			--clsCount;
			// optimization: follow a single-linked chain
			while(refs = top.refs, refs.length == 1){
				top = refs[0];
				if(!top || --top.count){
					// branch or end of chain => do not end to roots
					top = 0;
					break;
				}
				result.push(top.cls);
				--clsCount;
			}
			if(top){
				// branch
				for(i = 0, l = refs.length; i < l; ++i){
					top = refs[i];
					if(!--top.count){
						roots.push(top);
					}
				}
			}
		}
		if(clsCount){
			err("can't build consistent linearization", className);
		}

		// calculate the superclass offset
		base = bases[0];
		result[0] = base ?
			base._meta && base === result[result.length - base._meta.bases.length] ?
				base._meta.bases.length : 1 : 0;

		return result;
	}

	function inherited(args, a, f){
		var name, chains, bases, caller, meta, base, proto, opf, pos,
			cache = this._inherited = this._inherited || {};

		// crack arguments
		if(typeof args == "string"){
			name = args;
			args = a;
			a = f;
		}
		f = 0;

		caller = args.callee;
		name = name || caller.nom;
		if(!name){
			err("can't deduce a name to call inherited()", this.declaredClass);
		}

		meta = this.constructor._meta;
		bases = meta.bases;

		pos = cache.p;
		if(name != cname){
			// method
			if(cache.c !== caller){
				// cache bust
				pos = 0;
				base = bases[0];
				meta = base._meta;
				if(meta.hidden[name] !== caller){
					// error detection
					chains = meta.chains;
					if(chains && typeof chains[name] == "string"){
						err("calling chained method with inherited: " + name, this.declaredClass);
					}
					// find caller
					do{
						meta = base._meta;
						proto = base.prototype;
						if(meta && (proto[name] === caller && proto.hasOwnProperty(name) || meta.hidden[name] === caller)){
							break;
						}
					}while(base = bases[++pos]); // intentional assignment
					pos = base ? pos : -1;
				}
			}
			// find next
			base = bases[++pos];
			if(base){
				proto = base.prototype;
				if(base._meta && proto.hasOwnProperty(name)){
					f = proto[name];
				}else{
					opf = op[name];
					do{
						proto = base.prototype;
						f = proto[name];
						if(f && (base._meta ? proto.hasOwnProperty(name) : f !== opf)){
							break;
						}
					}while(base = bases[++pos]); // intentional assignment
				}
			}
			f = base && f || op[name];
		}else{
			// constructor
			if(cache.c !== caller){
				// cache bust
				pos = 0;
				meta = bases[0]._meta;
				if(meta && meta.ctor !== caller){
					// error detection
					chains = meta.chains;
					if(!chains || chains.constructor !== "manual"){
						err("calling chained constructor with inherited", this.declaredClass);
					}
					// find caller
					while(base = bases[++pos]){ // intentional assignment
						meta = base._meta;
						if(meta && meta.ctor === caller){
							break;
						}
					}
					pos = base ? pos : -1;
				}
			}
			// find next
			while(base = bases[++pos]){	// intentional assignment
				meta = base._meta;
				f = meta ? meta.ctor : base;
				if(f){
					break;
				}
			}
			f = base && f;
		}

		// cache the found super method
		cache.c = f;
		cache.p = pos;

		// now we have the result
		if(f){
			return a === true ? f : f.apply(this, a || args);
		}
		// intentionally no return if a super method was not found
	}

	function getInherited(name, args){
		if(typeof name == "string"){
			return this.__inherited(name, args, true);
		}
		return this.__inherited(name, true);
	}

	function inherited__debug(args, a1, a2){
		var f = this.getInherited(args, a1);
		if(f){ return f.apply(this, a2 || a1 || args); }
		// intentionally no return if a super method was not found
	}

	var inheritedImpl = dojo.config.isDebug ? inherited__debug : inherited;

	// emulation of "instanceof"
	function isInstanceOf(cls){
		var bases = this.constructor._meta.bases;
		for(var i = 0, l = bases.length; i < l; ++i){
			if(bases[i] === cls){
				return true;
			}
		}
		return this instanceof cls;
	}

	function mixOwn(target, source){
		// add props adding metadata for incoming functions skipping a constructor
		for(var name in source){
			if(name != cname && source.hasOwnProperty(name)){
				target[name] = source[name];
			}
		}
		if(has("bug-for-in-skips-shadowed")){
			for(var extraNames= lang._extraNames, i= extraNames.length; i;){
				name = extraNames[--i];
				if(name != cname && source.hasOwnProperty(name)){
					  target[name] = source[name];
				}
			}
		}
	}

	// implementation of safe mixin function
	function safeMixin(target, source){
		var name, t;
		// add props adding metadata for incoming functions skipping a constructor
		for(name in source){
			t = source[name];
			if((t !== op[name] || !(name in op)) && name != cname){
				if(opts.call(t) == "[object Function]"){
					// non-trivial function method => attach its name
					t.nom = name;
				}
				target[name] = t;
			}
		}
		if(has("bug-for-in-skips-shadowed")){
			for(var extraNames= lang._extraNames, i= extraNames.length; i;){
				name = extraNames[--i];
				t = source[name];
				if((t !== op[name] || !(name in op)) && name != cname){
					if(opts.call(t) == "[object Function]"){
						// non-trivial function method => attach its name
						  t.nom = name;
					}
					target[name] = t;
				}
			}
		}
		return target;
	}

	function extend(source){
		declare.safeMixin(this.prototype, source);
		return this;
	}

	function createSubclass(mixins){
		return declare([this].concat(mixins));
	}

	// chained constructor compatible with the legacy dojo.declare()
	function chainedConstructor(bases, ctorSpecial){
		return function(){
			var a = arguments, args = a, a0 = a[0], f, i, m,
				l = bases.length, preArgs;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original dojo.declare()
			// 1) call two types of the preamble
			if(ctorSpecial && (a0 && a0.preamble || this.preamble)){
				// full blown ritual
				preArgs = new Array(bases.length);
				// prepare parameters
				preArgs[0] = a;
				for(i = 0;;){
					// process the preamble of the 1st argument
					a0 = a[0];
					if(a0){
						f = a0.preamble;
						if(f){
							a = f.apply(this, a) || a;
						}
					}
					// process the preamble of this class
					f = bases[i].prototype;
					f = f.hasOwnProperty("preamble") && f.preamble;
					if(f){
						a = f.apply(this, a) || a;
					}
					// one peculiarity of the preamble:
					// it is called if it is not needed,
					// e.g., there is no constructor to call
					// let's watch for the last constructor
					// (see ticket #9795)
					if(++i == l){
						break;
					}
					preArgs[i] = a;
				}
			}
			// 2) call all non-trivial constructors using prepared arguments
			for(i = l - 1; i >= 0; --i){
				f = bases[i];
				m = f._meta;
				f = m ? m.ctor : f;
				if(f){
					f.apply(this, preArgs ? preArgs[i] : a);
				}
			}
			// 3) continue the original ritual: call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, args);
			}
		};
	}


	// chained constructor compatible with the legacy dojo.declare()
	function singleConstructor(ctor, ctorSpecial){
		return function(){
			var a = arguments, t = a, a0 = a[0], f;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original dojo.declare()
			// 1) call two types of the preamble
			if(ctorSpecial){
				// full blown ritual
				if(a0){
					// process the preamble of the 1st argument
					f = a0.preamble;
					if(f){
						t = f.apply(this, t) || t;
					}
				}
				f = this.preamble;
				if(f){
					// process the preamble of this class
					f.apply(this, t);
					// one peculiarity of the preamble:
					// it is called even if it is not needed,
					// e.g., there is no constructor to call
					// let's watch for the last constructor
					// (see ticket #9795)
				}
			}
			// 2) call a constructor
			if(ctor){
				ctor.apply(this, a);
			}
			// 3) continue the original ritual: call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, a);
			}
		};
	}

	// plain vanilla constructor (can use inherited() to call its base constructor)
	function simpleConstructor(bases){
		return function(){
			var a = arguments, i = 0, f, m;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original dojo.declare()
			// 1) do not call the preamble
			// 2) call the top constructor (it can use this.inherited())
			for(; f = bases[i]; ++i){ // intentional assignment
				m = f._meta;
				f = m ? m.ctor : f;
				if(f){
					f.apply(this, a);
					break;
				}
			}
			// 3) call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, a);
			}
		};
	}

	function chain(name, bases, reversed){
		return function(){
			var b, m, f, i = 0, step = 1;
			if(reversed){
				i = bases.length - 1;
				step = -1;
			}
			for(; b = bases[i]; i += step){ // intentional assignment
				m = b._meta;
				f = (m ? m.hidden : b.prototype)[name];
				if(f){
					f.apply(this, arguments);
				}
			}
		};
	}

	// forceNew(ctor)
	// return a new object that inherits from ctor.prototype but
	// without actually running ctor on the object.
	function forceNew(ctor){
		// create object with correct prototype using a do-nothing
		// constructor
		xtor.prototype = ctor.prototype;
		var t = new xtor;
		xtor.prototype = null;	// clean up
		return t;
	}

	// applyNew(args)
	// just like 'new ctor()' except that the constructor and its arguments come
	// from args, which must be an array or an arguments object
	function applyNew(args){
		// create an object with ctor's prototype but without
		// calling ctor on it.
		var ctor = args.callee, t = forceNew(ctor);
		// execute the real constructor on the new object
		ctor.apply(t, args);
		return t;
	}

	function declare(className, superclass, props){
		// crack parameters
		if(typeof className != "string"){
			props = superclass;
			superclass = className;
			className = "";
		}
		props = props || {};

		var proto, i, t, ctor, name, bases, chains, mixins = 1, parents = superclass;

		// build a prototype
		if(opts.call(superclass) == "[object Array]"){
			// C3 MRO
			bases = c3mro(superclass, className);
			t = bases[0];
			mixins = bases.length - t;
			superclass = bases[mixins];
		}else{
			bases = [0];
			if(superclass){
				if(opts.call(superclass) == "[object Function]"){
					t = superclass._meta;
					bases = bases.concat(t ? t.bases : superclass);
				}else{
					err("base class is not a callable constructor.", className);
				}
			}else if(superclass !== null){
				err("unknown base class. Did you use dojo.require to pull it in?", className);
			}
		}
		if(superclass){
			for(i = mixins - 1;; --i){
				proto = forceNew(superclass);
				if(!i){
					// stop if nothing to add (the last base)
					break;
				}
				// mix in properties
				t = bases[i];
				(t._meta ? mixOwn : mix)(proto, t.prototype);
				// chain in new constructor
				ctor = new Function;
				ctor.superclass = superclass;
				ctor.prototype = proto;
				superclass = proto.constructor = ctor;
			}
		}else{
			proto = {};
		}
		// add all properties
		declare.safeMixin(proto, props);
		// add constructor
		t = props.constructor;
		if(t !== op.constructor){
			t.nom = cname;
			proto.constructor = t;
		}

		// collect chains and flags
		for(i = mixins - 1; i; --i){ // intentional assignment
			t = bases[i]._meta;
			if(t && t.chains){
				chains = mix(chains || {}, t.chains);
			}
		}
		if(proto["-chains-"]){
			chains = mix(chains || {}, proto["-chains-"]);
		}

		// build ctor
		t = !chains || !chains.hasOwnProperty(cname);
		bases[0] = ctor = (chains && chains.constructor === "manual") ? simpleConstructor(bases) :
			(bases.length == 1 ? singleConstructor(props.constructor, t) : chainedConstructor(bases, t));

		// add meta information to the constructor
		ctor._meta  = {bases: bases, hidden: props, chains: chains,
			parents: parents, ctor: props.constructor};
		ctor.superclass = superclass && superclass.prototype;
		ctor.extend = extend;
		ctor.createSubclass = createSubclass;
		ctor.prototype = proto;
		proto.constructor = ctor;

		// add "standard" methods to the prototype
		proto.getInherited = getInherited;
		proto.isInstanceOf = isInstanceOf;
		proto.inherited    = inheritedImpl;
		proto.__inherited  = inherited;

		// add name if specified
		if(className){
			proto.declaredClass = className;
			lang.setObject(className, ctor);
		}

		// build chains and add them to the prototype
		if(chains){
			for(name in chains){
				if(proto[name] && typeof chains[name] == "string" && name != cname){
					t = proto[name] = chain(name, bases, chains[name] === "after");
					t.nom = name;
				}
			}
		}
		// chained methods do not return values
		// no need to chain "invisible" functions

		return ctor;	// Function
	}

	/*=====
	dojo.declare = function(className, superclass, props){
		//	summary:
		//		Create a feature-rich constructor from compact notation.
		//	className: String?:
		//		The optional name of the constructor (loosely, a "class")
		//		stored in the "declaredClass" property in the created prototype.
		//		It will be used as a global name for a created constructor.
		//	superclass: Function|Function[]:
		//		May be null, a Function, or an Array of Functions. This argument
		//		specifies a list of bases (the left-most one is the most deepest
		//		base).
		//	props: Object:
		//		An object whose properties are copied to the created prototype.
		//		Add an instance-initialization function by making it a property
		//		named "constructor".
		//	returns:
		//		New constructor function.
		//	description:
		//		Create a constructor using a compact notation for inheritance and
		//		prototype extension.
		//
		//		Mixin ancestors provide a type of multiple inheritance.
		//		Prototypes of mixin ancestors are copied to the new class:
		//		changes to mixin prototypes will not affect classes to which
		//		they have been mixed in.
		//
		//		Ancestors can be compound classes created by this version of
		//		dojo.declare. In complex cases all base classes are going to be
		//		linearized according to C3 MRO algorithm
		//		(see http://www.python.org/download/releases/2.3/mro/ for more
		//		details).
		//
		//		"className" is cached in "declaredClass" property of the new class,
		//		if it was supplied. The immediate super class will be cached in
		//		"superclass" property of the new class.
		//
		//		Methods in "props" will be copied and modified: "nom" property
		//		(the declared name of the method) will be added to all copied
		//		functions to help identify them for the internal machinery. Be
		//		very careful, while reusing methods: if you use the same
		//		function under different names, it can produce errors in some
		//		cases.
		//
		//		It is possible to use constructors created "manually" (without
		//		dojo.declare) as bases. They will be called as usual during the
		//		creation of an instance, their methods will be chained, and even
		//		called by "this.inherited()".
		//
		//		Special property "-chains-" governs how to chain methods. It is
		//		a dictionary, which uses method names as keys, and hint strings
		//		as values. If a hint string is "after", this method will be
		//		called after methods of its base classes. If a hint string is
		//		"before", this method will be called before methods of its base
		//		classes.
		//
		//		If "constructor" is not mentioned in "-chains-" property, it will
		//		be chained using the legacy mode: using "after" chaining,
		//		calling preamble() method before each constructor, if available,
		//		and calling postscript() after all constructors were executed.
		//		If the hint is "after", it is chained as a regular method, but
		//		postscript() will be called after the chain of constructors.
		//		"constructor" cannot be chained "before", but it allows
		//		a special hint string: "manual", which means that constructors
		//		are not going to be chained in any way, and programmer will call
		//		them manually using this.inherited(). In the latter case
		//		postscript() will be called after the construction.
		//
		//		All chaining hints are "inherited" from base classes and
		//		potentially can be overridden. Be very careful when overriding
		//		hints! Make sure that all chained methods can work in a proposed
		//		manner of chaining.
		//
		//		Once a method was chained, it is impossible to unchain it. The
		//		only exception is "constructor". You don't need to define a
		//		method in order to supply a chaining hint.
		//
		//		If a method is chained, it cannot use this.inherited() because
		//		all other methods in the hierarchy will be called automatically.
		//
		//		Usually constructors and initializers of any kind are chained
		//		using "after" and destructors of any kind are chained as
		//		"before". Note that chaining assumes that chained methods do not
		//		return any value: any returned value will be discarded.
		//
		//	example:
		//	|	dojo.declare("my.classes.bar", my.classes.foo, {
		//	|		// properties to be added to the class prototype
		//	|		someValue: 2,
		//	|		// initialization function
		//	|		constructor: function(){
		//	|			this.myComplicatedObject = new ReallyComplicatedObject();
		//	|		},
		//	|		// other functions
		//	|		someMethod: function(){
		//	|			doStuff();
		//	|		}
		//	|	});
		//
		//	example:
		//	|	var MyBase = dojo.declare(null, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyClass1 = dojo.declare(MyBase, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyClass2 = dojo.declare(MyBase, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyDiamond = dojo.declare([MyClass1, MyClass2], {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//
		//	example:
		//	|	var F = function(){ console.log("raw constructor"); };
		//	|	F.prototype.method = function(){
		//	|		console.log("raw method");
		//	|	};
		//	|	var A = dojo.declare(F, {
		//	|		constructor: function(){
		//	|			console.log("A.constructor");
		//	|		},
		//	|		method: function(){
		//	|			console.log("before calling F.method...");
		//	|			this.inherited(arguments);
		//	|			console.log("...back in A");
		//	|		}
		//	|	});
		//	|	new A().method();
		//	|	// will print:
		//	|	// raw constructor
		//	|	// A.constructor
		//	|	// before calling F.method...
		//	|	// raw method
		//	|	// ...back in A
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		"-chains-": {
		//	|			destroy: "before"
		//	|		}
		//	|	});
		//	|	var B = dojo.declare(A, {
		//	|		constructor: function(){
		//	|			console.log("B.constructor");
		//	|		},
		//	|		destroy: function(){
		//	|			console.log("B.destroy");
		//	|		}
		//	|	});
		//	|	var C = dojo.declare(B, {
		//	|		constructor: function(){
		//	|			console.log("C.constructor");
		//	|		},
		//	|		destroy: function(){
		//	|			console.log("C.destroy");
		//	|		}
		//	|	});
		//	|	new C().destroy();
		//	|	// prints:
		//	|	// B.constructor
		//	|	// C.constructor
		//	|	// C.destroy
		//	|	// B.destroy
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		"-chains-": {
		//	|			constructor: "manual"
		//	|		}
		//	|	});
		//	|	var B = dojo.declare(A, {
		//	|		constructor: function(){
		//	|			// ...
		//	|			// call the base constructor with new parameters
		//	|			this.inherited(arguments, [1, 2, 3]);
		//	|			// ...
		//	|		}
		//	|	});
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		"-chains-": {
		//	|			m1: "before"
		//	|		},
		//	|		m1: function(){
		//	|			console.log("A.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("A.m2");
		//	|		}
		//	|	});
		//	|	var B = dojo.declare(A, {
		//	|		"-chains-": {
		//	|			m2: "after"
		//	|		},
		//	|		m1: function(){
		//	|			console.log("B.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("B.m2");
		//	|		}
		//	|	});
		//	|	var x = new B();
		//	|	x.m1();
		//	|	// prints:
		//	|	// B.m1
		//	|	// A.m1
		//	|	x.m2();
		//	|	// prints:
		//	|	// A.m2
		//	|	// B.m2
		return new Function(); // Function
	};
	=====*/

	/*=====
	dojo.safeMixin = function(target, source){
		//	summary:
		//		Mix in properties skipping a constructor and decorating functions
		//		like it is done by dojo.declare.
		//	target: Object
		//		Target object to accept new properties.
		//	source: Object
		//		Source object for new properties.
		//	description:
		//		This function is used to mix in properties like lang.mixin does,
		//		but it skips a constructor property and decorates functions like
		//		dojo.declare does.
		//
		//		It is meant to be used with classes and objects produced with
		//		dojo.declare. Functions mixed in with dojo.safeMixin can use
		//		this.inherited() like normal methods.
		//
		//		This function is used to implement extend() method of a constructor
		//		produced with dojo.declare().
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		m1: function(){
		//	|			console.log("A.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("A.m2");
		//	|		}
		//	|	});
		//	|	var B = dojo.declare(A, {
		//	|		m1: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("B.m1");
		//	|		}
		//	|	});
		//	|	B.extend({
		//	|		m2: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("B.m2");
		//	|		}
		//	|	});
		//	|	var x = new B();
		//	|	dojo.safeMixin(x, {
		//	|		m1: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("X.m1");
		//	|		},
		//	|		m2: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("X.m2");
		//	|		}
		//	|	});
		//	|	x.m2();
		//	|	// prints:
		//	|	// A.m1
		//	|	// B.m1
		//	|	// X.m1
	};
	=====*/

	/*=====
	Object.inherited = function(name, args, newArgs){
		//	summary:
		//		Calls a super method.
		//	name: String?
		//		The optional method name. Should be the same as the caller's
		//		name. Usually "name" is specified in complex dynamic cases, when
		//		the calling method was dynamically added, undecorated by
		//		dojo.declare, and it cannot be determined.
		//	args: Arguments
		//		The caller supply this argument, which should be the original
		//		"arguments".
		//	newArgs: Object?
		//		If "true", the found function will be returned without
		//		executing it.
		//		If Array, it will be used to call a super method. Otherwise
		//		"args" will be used.
		//	returns:
		//		Whatever is returned by a super method, or a super method itself,
		//		if "true" was specified as newArgs.
		//	description:
		//		This method is used inside method of classes produced with
		//		dojo.declare to call a super method (next in the chain). It is
		//		used for manually controlled chaining. Consider using the regular
		//		chaining, because it is faster. Use "this.inherited()" only in
		//		complex cases.
		//
		//		This method cannot me called from automatically chained
		//		constructors including the case of a special (legacy)
		//		constructor chaining. It cannot be called from chained methods.
		//
		//		If "this.inherited()" cannot find the next-in-chain method, it
		//		does nothing and returns "undefined". The last method in chain
		//		can be a default method implemented in Object, which will be
		//		called last.
		//
		//		If "name" is specified, it is assumed that the method that
		//		received "args" is the parent method for this call. It is looked
		//		up in the chain list and if it is found the next-in-chain method
		//		is called. If it is not found, the first-in-chain method is
		//		called.
		//
		//		If "name" is not specified, it will be derived from the calling
		//		method (using a methoid property "nom").
		//
		//	example:
		//	|	var B = dojo.declare(A, {
		//	|		method1: function(a, b, c){
		//	|			this.inherited(arguments);
		//	|		},
		//	|		method2: function(a, b){
		//	|			return this.inherited(arguments, [a + b]);
		//	|		}
		//	|	});
		//	|	// next method is not in the chain list because it is added
		//	|	// manually after the class was created.
		//	|	B.prototype.method3 = function(){
		//	|		console.log("This is a dynamically-added method.");
		//	|		this.inherited("method3", arguments);
		//	|	};
		//	example:
		//	|	var B = dojo.declare(A, {
		//	|		method: function(a, b){
		//	|			var super = this.inherited(arguments, true);
		//	|			// ...
		//	|			if(!super){
		//	|				console.log("there is no super method");
		//	|				return 0;
		//	|			}
		//	|			return super.apply(this, arguments);
		//	|		}
		//	|	});
		return	{};	// Object
	}
	=====*/

	/*=====
	Object.getInherited = function(name, args){
		//	summary:
		//		Returns a super method.
		//	name: String?
		//		The optional method name. Should be the same as the caller's
		//		name. Usually "name" is specified in complex dynamic cases, when
		//		the calling method was dynamically added, undecorated by
		//		dojo.declare, and it cannot be determined.
		//	args: Arguments
		//		The caller supply this argument, which should be the original
		//		"arguments".
		//	returns:
		//		Returns a super method (Function) or "undefined".
		//	description:
		//		This method is a convenience method for "this.inherited()".
		//		It uses the same algorithm but instead of executing a super
		//		method, it returns it, or "undefined" if not found.
		//
		//	example:
		//	|	var B = dojo.declare(A, {
		//	|		method: function(a, b){
		//	|			var super = this.getInherited(arguments);
		//	|			// ...
		//	|			if(!super){
		//	|				console.log("there is no super method");
		//	|				return 0;
		//	|			}
		//	|			return super.apply(this, arguments);
		//	|		}
		//	|	});
		return	{};	// Object
	}
	=====*/

	/*=====
	Object.isInstanceOf = function(cls){
		//	summary:
		//		Checks the inheritance chain to see if it is inherited from this
		//		class.
		//	cls: Function
		//		Class constructor.
		//	returns:
		//		"true", if this object is inherited from this class, "false"
		//		otherwise.
		//	description:
		//		This method is used with instances of classes produced with
		//		dojo.declare to determine of they support a certain interface or
		//		not. It models "instanceof" operator.
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var B = dojo.declare(null, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var C = dojo.declare([A, B], {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var D = dojo.declare(A, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|
		//	|	var a = new A(), b = new B(), c = new C(), d = new D();
		//	|
		//	|	console.log(a.isInstanceOf(A)); // true
		//	|	console.log(b.isInstanceOf(A)); // false
		//	|	console.log(c.isInstanceOf(A)); // true
		//	|	console.log(d.isInstanceOf(A)); // true
		//	|
		//	|	console.log(a.isInstanceOf(B)); // false
		//	|	console.log(b.isInstanceOf(B)); // true
		//	|	console.log(c.isInstanceOf(B)); // true
		//	|	console.log(d.isInstanceOf(B)); // false
		//	|
		//	|	console.log(a.isInstanceOf(C)); // false
		//	|	console.log(b.isInstanceOf(C)); // false
		//	|	console.log(c.isInstanceOf(C)); // true
		//	|	console.log(d.isInstanceOf(C)); // false
		//	|
		//	|	console.log(a.isInstanceOf(D)); // false
		//	|	console.log(b.isInstanceOf(D)); // false
		//	|	console.log(c.isInstanceOf(D)); // false
		//	|	console.log(d.isInstanceOf(D)); // true
		return	{};	// Object
	}
	=====*/

	/*=====
	Object.extend = function(source){
		//	summary:
		//		Adds all properties and methods of source to constructor's
		//		prototype, making them available to all instances created with
		//		constructor. This method is specific to constructors created with
		//		dojo.declare.
		//	source: Object
		//		Source object which properties are going to be copied to the
		//		constructor's prototype.
		//	description:
		//		Adds source properties to the constructor's prototype. It can
		//		override existing properties.
		//
		//		This method is similar to dojo.extend function, but it is specific
		//		to constructors produced by dojo.declare. It is implemented
		//		using dojo.safeMixin, and it skips a constructor property,
		//		and properly decorates copied functions.
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		m1: function(){},
		//	|		s1: "Popokatepetl"
		//	|	});
		//	|	A.extend({
		//	|		m1: function(){},
		//	|		m2: function(){},
		//	|		f1: true,
		//	|		d1: 42
		//	|	});
	};
	=====*/

	dojo.safeMixin = declare.safeMixin = safeMixin;
	dojo.declare = declare;

	return declare;
});

},
'dijit/layout/ContentPane':function(){
define([
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin lang.delegate lang.hitch lang.isFunction lang.isObject
	"../_Widget",
	"./_ContentPaneResizeMixin",
	"dojo/string", // string.substitute
	"dojo/html", // html._ContentSetter html._emptyNode
	"dojo/i18n!../nls/loading",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.attr
	"dojo/_base/window", // win.body win.doc.createDocumentFragment
	"dojo/_base/xhr", // xhr.get
	"dojo/i18n" // i18n.getLocalization
], function(kernel, lang, _Widget, _ContentPaneResizeMixin, string, html, nlsLoading,
	array, declare, Deferred, dom, domAttr, win, xhr, i18n){

/*=====
	var _Widget = dijit._Widget;
	var _ContentPaneResizeMixin = dijit.layout._ContentPaneResizeMixin;
=====*/

// module:
//		dijit/layout/ContentPane
// summary:
//		A widget containing an HTML fragment, specified inline
//		or by uri.  Fragment may include widgets.


return declare("dijit.layout.ContentPane", [_Widget, _ContentPaneResizeMixin], {
	// summary:
	//		A widget containing an HTML fragment, specified inline
	//		or by uri.  Fragment may include widgets.
	//
	// description:
	//		This widget embeds a document fragment in the page, specified
	//		either by uri, javascript generated markup or DOM reference.
	//		Any widgets within this content are instantiated and managed,
	//		but laid out according to the HTML structure.  Unlike IFRAME,
	//		ContentPane embeds a document fragment as would be found
	//		inside the BODY tag of a full HTML document.  It should not
	//		contain the HTML, HEAD, or BODY tags.
	//		For more advanced functionality with scripts and
	//		stylesheets, see dojox.layout.ContentPane.  This widget may be
	//		used stand alone or as a base class for other widgets.
	//		ContentPane is useful as a child of other layout containers
	//		such as BorderContainer or TabContainer, but note that those
	//		widgets can contain any widget as a child.
	//
	// example:
	//		Some quick samples:
	//		To change the innerHTML: cp.set('content', '<b>new content</b>')
	//
	//		Or you can send it a NodeList: cp.set('content', dojo.query('div [class=selected]', userSelection))
	//
	//		To do an ajax update: cp.set('href', url)

	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; Use set('href', ...);
	href: "",

	// content: String || DomNode || NodeList || dijit._Widget
	//		The innerHTML of the ContentPane.
	//		Note that the initialization parameter / argument to set("content", ...)
	//		can be a String, DomNode, Nodelist, or _Widget.
	content: "",

	// extractContent: Boolean
	//		Extract visible content from inside of <body> .... </body>.
	//		I.e., strip <html> and <head> (and it's contents) from the href
	extractContent: false,

	// parseOnLoad: Boolean
	//		Parse content and create the widgets, if any.
	parseOnLoad: true,

	// parserScope: String
	//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
	//		will search for data-dojo-type (or dojoType).  For backwards compatibility
	//		reasons defaults to dojo._scopeName (which is "dojo" except when
	//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
	parserScope: kernel._scopeName,

	// preventCache: Boolean
	//		Prevent caching of data from href's by appending a timestamp to the href.
	preventCache: false,

	// preload: Boolean
	//		Force load of data on initialization even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//		Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'><span class='dijitInline dijitIconLoading'></span>${loadingState}</span>",

	// errorMessage: String
	//		Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'><span class='dijitInline dijitIconError'></span>${errorState}</span>",

	// isLoaded: [readonly] Boolean
	//		True if the ContentPane has data in it, either specified
	//		during initialization (via href or inline content), or set
	//		via set('content', ...) / set('href', ...)
	//
	//		False if it doesn't have any content, or if ContentPane is
	//		still in the process of downloading href.
	isLoaded: false,

	baseClass: "dijitContentPane",

	/*======
	// ioMethod: dojo.xhrGet|dojo.xhrPost
	//		Function that should grab the content specified via href.
	ioMethod: dojo.xhrGet,
	======*/

	// ioArgs: Object
	//		Parameters to pass to xhrGet() request, for example:
	// |	<div data-dojo-type="dijit.layout.ContentPane" data-dojo-props="href: './bar', ioArgs: {timeout: 500}">
	ioArgs: {},

	// onLoadDeferred: [readonly] dojo.Deferred
	//		This is the `dojo.Deferred` returned by set('href', ...) and refresh().
	//		Calling onLoadDeferred.addCallback() or addErrback() registers your
	//		callback to be called only once, when the prior set('href', ...) call or
	//		the initial href parameter to the constructor finishes loading.
	//
	//		This is different than an onLoad() handler which gets called any time any href
	//		or content is loaded.
	onLoadDeferred: null,

	// Cancel _WidgetBase's _setTitleAttr because we don't want the title attribute (used to specify
	// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
	// entire pane.
	_setTitleAttr: null,

	// Flag to parser that I'll parse my contents, so it shouldn't.
	stopParser: true,

	// template: [private] Boolean
	//		Flag from the parser that this ContentPane is inside a template
	//		so the contents are pre-parsed.
	// (TODO: this declaration can be commented out in 2.0)
	template: false,

	create: function(params, srcNodeRef){
		// Convert a srcNodeRef argument into a content parameter, so that the original contents are
		// processed in the same way as contents set via set("content", ...), calling the parser etc.
		// Avoid modifying original params object since that breaks NodeList instantiation, see #11906.
		if((!params || !params.template) && srcNodeRef && !("href" in params) && !("content" in params)){
			var df = win.doc.createDocumentFragment();
			srcNodeRef = dom.byId(srcNodeRef);
			while(srcNodeRef.firstChild){
				df.appendChild(srcNodeRef.firstChild);
			}
			params = lang.delegate(params, {content: df});
		}
		this.inherited(arguments, [params, srcNodeRef]);
	},

	postMixInProperties: function(){
		this.inherited(arguments);
		var messages = i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = string.substitute(this.loadingMessage, messages);
		this.errorMessage = string.substitute(this.errorMessage, messages);
	},

	buildRendering: function(){
		this.inherited(arguments);

		// Since we have no template we need to set this.containerNode ourselves, to make getChildren() work.
		// For subclasses of ContentPane that do have a template, does nothing.
		if(!this.containerNode){
			this.containerNode = this.domNode;
		}

		// remove the title attribute so it doesn't show up when hovering
		// over a node  (TODO: remove in 2.0, no longer needed after #11490)
		this.domNode.removeAttribute("title");

		if(!domAttr.get(this.domNode,"role")){
			this.domNode.setAttribute("role", "group");
		}
	},

	startup: function(){
		// summary:
		//		Call startup() on all children including non _Widget ones like dojo.dnd.Source objects

		// This starts all the widgets
		this.inherited(arguments);

		// And this catches stuff like dojo.dnd.Source
		if(this._contentSetter){
			array.forEach(this._contentSetter.parseResults, function(obj){
				if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			}, this);
		}
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Deprecated.   Use set('href', ...) instead.
		kernel.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
		return this.set("href", href);
	},
	_setHrefAttr: function(/*String|Uri*/ href){
		// summary:
		//		Hook so set("href", ...) works.
		// description:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false.
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage

		// Cancel any in-flight requests (a set('href', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		this.onLoadDeferred.addCallback(lang.hitch(this, "onLoad"));

		this._set("href", href);

		// _setHrefAttr() is called during creation and by the user, after creation.
		// Assuming preload == false, only in the second case do we actually load the URL;
		// otherwise it's done in startup(), and only if this widget is shown.
		if(this.preload || (this._created && this._isShown())){
			this._load();
		}else{
			// Set flag to indicate that href needs to be loaded the next time the
			// ContentPane is made visible
			this._hrefChanged = true;
		}

		return this.onLoadDeferred;		// Deferred
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Deprecated.   Use set('content', ...) instead.
		kernel.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
		this.set("content", data);
	},
	_setContentAttr: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Hook to make set("content", ...) work.
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we can't run refresh and clear content
		// refresh should only work if we downloaded the content
		this._set("href", "");

		// Cancel any in-flight requests (a set('content', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		// Even though user is just setting content directly, still need to define an onLoadDeferred
		// because the _onLoadHandler() handler is still getting called from setContent()
		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		if(this._created){
			// For back-compat reasons, call onLoad() for set('content', ...)
			// calls but not for content specified in srcNodeRef (ie: <div data-dojo-type=ContentPane>...</div>)
			// or as initialization parameter (ie: new ContentPane({content: ...})
			this.onLoadDeferred.addCallback(lang.hitch(this, "onLoad"));
		}

		this._setContent(data || "");

		this._isDownloaded = false; // mark that content is from a set('content') not a set('href')

		return this.onLoadDeferred; 	// Deferred
	},
	_getContentAttr: function(){
		// summary:
		//		Hook to make get("content") work
		return this.containerNode.innerHTML;
	},

	cancel: function(){
		// summary:
		//		Cancels an in-flight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect

		this.onLoadDeferred = null;
	},

	uninitialize: function(){
		if(this._beingDestroyed){
			this.cancel();
		}
		this.inherited(arguments);
	},

	destroyRecursive: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy the ContentPane and its contents

		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		this.inherited(arguments);
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does necessary processing, including href download and layout/resize of
		//		child widget(s)

		this.inherited(arguments);

		if(this.href){
			if(!this._xhrDfd && // if there's an href that isn't already being loaded
				(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
			){
				return this.refresh();	// If child has an href, promise that fires when the load is complete
			}
		}
	},

	refresh: function(){
		// summary:
		//		[Re]download contents of href and display
		// description:
		//		1. cancels any currently in-flight requests
		//		2. posts "loading..." message
		//		3. sends XHR to download new data

		// Cancel possible prior in-flight request
		this.cancel();

		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		this.onLoadDeferred.addCallback(lang.hitch(this, "onLoad"));
		this._load();
		return this.onLoadDeferred;		// If child has an href, promise that fires when refresh is complete
	},

	_load: function(){
		// summary:
		//		Load/reload the href specified in this.href

		// display loading message
		this._setContent(this.onDownloadStart(), true);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(lang.isObject(this.ioArgs)){
			lang.mixin(getArgs, this.ioArgs);
		}

		var hand = (this._xhrDfd = (this.ioMethod || xhr.get)(getArgs));

		hand.addCallback(function(html){
			try{
				self._isDownloaded = true;
				self._setContent(html, false);
				self.onDownloadEnd();
			}catch(err){
				self._onError('Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.canceled){
				// show error message in the pane
				self._onError('Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});

		// Remove flag saying that a load is needed
		delete this._hrefChanged;
	},

	_onLoadHandler: function(data){
		// summary:
		//		This is called whenever new content is being loaded
		this._set("isLoaded", true);
		try{
			this.onLoadDeferred.callback(data);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code: ' + e.message);
		}
	},

	_onUnloadHandler: function(){
		// summary:
		//		This is called whenever the content is being unloaded
		this._set("isLoaded", false);
		try{
			this.onUnload();
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code: ' + e.message);
		}
	},

	destroyDescendants: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy all the widgets inside the ContentPane and empty containerNode

		// Make sure we call onUnload (but only when the ContentPane has real content)
		if(this.isLoaded){
			this._onUnloadHandler();
		}

		// Even if this.isLoaded == false there might still be a "Loading..." message
		// to erase, so continue...

		// For historical reasons we need to delete all widgets under this.containerNode,
		// even ones that the user has created manually.
		var setter = this._contentSetter;
		array.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
		if(setter){
			// Most of the widgets in setter.parseResults have already been destroyed, but
			// things like Menu that have been moved to <body> haven't yet
			array.forEach(setter.parseResults, function(widget){
				if(widget.destroyRecursive && widget.domNode && widget.domNode.parentNode == win.body()){
					widget.destroyRecursive(preserveDom);
				}
			});
			delete setter.parseResults;
		}

		// And then clear away all the DOM nodes
		if(!preserveDom){
			html._emptyNode(this.containerNode);
		}

		// Delete any state information we have about current contents
		delete this._singleChild;
	},

	_setContent: function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
		// summary:
		//		Insert the content into the container node

		// first get rid of child widgets
		this.destroyDescendants();

		// html.set will take care of the rest of the details
		// we provide an override for the error handling to ensure the widget gets the errors
		// configure the setter instance with only the relevant widget instance properties
		// NOTE: unless we hook into attr, or provide property setters for each property,
		// we need to re-configure the ContentSetter with each use
		var setter = this._contentSetter;
		if(! (setter && setter instanceof html._ContentSetter)){
			setter = this._contentSetter = new html._ContentSetter({
				node: this.containerNode,
				_onError: lang.hitch(this, this._onError),
				onContentError: lang.hitch(this, function(e){
					// fires if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
					var errMess = this.onContentError(e);
					try{
						this.containerNode.innerHTML = errMess;
					}catch(e){
						console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
					}
				})/*,
				_onError */
			});
		}

		var setterParams = lang.mixin({
			cleanContent: this.cleanContent,
			extractContent: this.extractContent,
			parseContent: !cont.domNode && this.parseOnLoad,
			parserScope: this.parserScope,
			startup: false,
			dir: this.dir,
			lang: this.lang,
			textDir: this.textDir
		}, this._contentSetterParams || {});

		setter.set( (lang.isObject(cont) && cont.domNode) ? cont.domNode : cont, setterParams );

		// setter params must be pulled afresh from the ContentPane each time
		delete this._contentSetterParams;

		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(!isFakeContent){
			if(this._started){
				// Startup each top level child widget (and they will start their children, recursively)
				delete this._started;
				this.startup();

				// Call resize() on each of my child layout widgets,
				// or resize() on my single child layout widget...
				// either now (if I'm currently visible) or when I become visible
				this._scheduleLayout();
			}

			this._onLoadHandler(cont);
		}
	},

	_onError: function(type, err, consoleText){
		this.onLoadDeferred.errback(err);

		// shows user the string that is returned by on[type]Error
		// override on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent(errText, true);
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(/*===== data =====*/){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
		// tags:
		//		callback
	},

	onUnload: function(){
		// summary:
		//		Event hook, is called before old content is cleared
		// tags:
		//		callback
	},

	onDownloadStart: function(){
		// summary:
		//		Called before download starts.
		// description:
		//		The string returned by this function will be the html
		//		that tells the user we are loading something.
		//		Override with your own function if you want to change text.
		// tags:
		//		extension
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ /*===== error =====*/){
		// summary:
		//		Called on DOM faults, require faults etc. in content.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		By default (if this method is not overriden), it returns
		//		nothing, so the error message is just printed to the console.
		// tags:
		//		extension
	},

	onDownloadError: function(/*Error*/ /*===== error =====*/){
		// summary:
		//		Called when download error occurs.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		Default behavior (if this method is not overriden) is to display
		//		the error message inside the pane.
		// tags:
		//		extension
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		Called when download is finished.
		// tags:
		//		callback
	}
});

});

},
'dojo/json':function(){
define(["./has"], function(has){
	"use strict";
	var hasJSON = typeof JSON != "undefined";
	has.add("json-parse", hasJSON); // all the parsers work fine
		// Firefox 3.5/Gecko 1.9 fails to use replacer in stringify properly https://bugzilla.mozilla.org/show_bug.cgi?id=509184
	has.add("json-stringify", hasJSON && JSON.stringify({a:0}, function(k,v){return v||1;}) == '{"a":1}'); 
	if(has("json-stringify")){
		return JSON;
	}
	else{
		var escapeString = function(/*String*/str){
			//summary:
			//		Adds escape sequences for non-visual characters, double quote and
			//		backslash and surrounds with double quotes to form a valid string
			//		literal.
			return ('"' + str.replace(/(["\\])/g, '\\$1') + '"').
				replace(/[\f]/g, "\\f").replace(/[\b]/g, "\\b").replace(/[\n]/g, "\\n").
				replace(/[\t]/g, "\\t").replace(/[\r]/g, "\\r"); // string
		};
		return {
			parse: has("json-parse") ? JSON.parse : function(str, strict){
				// summary:
				// 		Parses a [JSON](http://json.org) string to return a JavaScript object.
				// description:
				//		This function follows [native JSON API](https://developer.mozilla.org/en/JSON)
				// 		Throws for invalid JSON strings. This delegates to eval() if native JSON
				// 		support is not available. By default this will evaluate any valid JS expression.
				//		With the strict parameter set to true, the parser will ensure that only
				//		valid JSON strings are parsed (otherwise throwing an error). Without the strict
				// 		parameter, the content passed to this method must come
				//		from a trusted source.
				// str:
				//		a string literal of a JSON item, for instance:
				//			`'{ "foo": [ "bar", 1, { "baz": "thud" } ] }'`
				//	strict: 
				//		When set to true, this will ensure that only valid, secure JSON is ever parsed.
				// 		Make sure this is set to true for untrusted content. Note that on browsers/engines
				//		without native JSON support, setting this to true will run slower.
				if(strict && !/^([\s\[\{]*(?:"(?:\\.|[^"])+"|-?\d[\d\.]*(?:[Ee][+-]?\d+)?|null|true|false|)[\s\]\}]*(?:,|:|$))+$/.test(str)){
					throw new SyntaxError("Invalid characters in JSON");
				}
				return eval('(' + str + ')');
			},
			stringify: function(value, replacer, spacer){
				//	summary:
				//		Returns a [JSON](http://json.org) serialization of an object.
				//	description:
				//		Returns a [JSON](http://json.org) serialization of an object.
				//		This function follows [native JSON API](https://developer.mozilla.org/en/JSON)
				//		Note that this doesn't check for infinite recursion, so don't do that!
				//	value:
				//		A value to be serialized. 
				//	replacer:
				//		A replacer function that is called for each value and can return a replacement
				//	spacer:
				//		A spacer string to be used for pretty printing of JSON
				//		
				//	example:
				//		simple serialization of a trivial object
				//		|	define(["dojo/json"], function(JSON){
				// 		|		var jsonStr = JSON.stringify({ howdy: "stranger!", isStrange: true });
				//		|		doh.is('{"howdy":"stranger!","isStrange":true}', jsonStr);
				var undef;
				if(typeof replacer == "string"){
					spacer = replacer;
					replacer = null;
				}
				function stringify(it, indent, key){
					if(replacer){
						it = replacer(key, it);
					}
					var val, objtype = typeof it;
					if(objtype == "number"){
						return isFinite(it) ? it + "" : "null";
					}
					if(objtype == "boolean"){
						return it + "";
					}
					if(it === null){
						return "null";
					}
					if(typeof it == "string"){
						return escapeString(it);
					}
					if(objtype == "function" || objtype == "undefined"){
						return undef; // undefined
					}
					// short-circuit for objects that support "json" serialization
					// if they return "self" then just pass-through...
					if(typeof it.toJSON == "function"){
						return stringify(it.toJSON(key), indent, key);
					}
					if(it instanceof Date){
						return '"{FullYear}-{Month+}-{Date}T{Hours}:{Minutes}:{Seconds}Z"'.replace(/\{(\w+)(\+)?\}/g, function(t, prop, plus){
							var num = it["getUTC" + prop]() + (plus ? 1 : 0);
							return num < 10 ? "0" + num : num;
						});
					}
					if(it.valueOf() !== it){
						// primitive wrapper, try again unwrapped:
						return stringify(it.valueOf(), indent, key);
					}
					var nextIndent= spacer ? (indent + spacer) : "";
					/* we used to test for DOM nodes and throw, but FF serializes them as {}, so cross-browser consistency is probably not efficiently attainable */ 
				
					var sep = spacer ? " " : "";
					var newLine = spacer ? "\n" : "";
				
					// array
					if(it instanceof Array){
						var itl = it.length, res = [];
						for(key = 0; key < itl; key++){
							var obj = it[key];
							val = stringify(obj, nextIndent, key);
							if(typeof val != "string"){
								val = "null";
							}
							res.push(newLine + nextIndent + val);
						}
						return "[" + res.join(",") + newLine + indent + "]";
					}
					// generic object code path
					var output = [];
					for(key in it){
						var keyStr;
						if(typeof key == "number"){
							keyStr = '"' + key + '"';
						}else if(typeof key == "string"){
							keyStr = escapeString(key);
						}else{
							// skip non-string or number keys
							continue;
						}
						val = stringify(it[key], nextIndent, key);
						if(typeof val != "string"){
							// skip non-serializable values
							continue;
						}
						// At this point, the most non-IE browsers don't get in this branch 
						// (they have native JSON), so push is definitely the way to
						output.push(newLine + nextIndent + keyStr + ":" + sep + val);
					}
					return "{" + output.join(",") + newLine + indent + "}"; // String
				}
				return stringify(value, "", "");
			}
		};
	}
});

},
'dojo/_base/json':function(){
define(["./kernel", "../json"], function(dojo, json){
  // module:
  //    dojo/_base/json
  // summary:
  //    This module defines the dojo JSON API.

dojo.fromJson = function(/*String*/ js){
	// summary:
	//		Parses a JavaScript expression and returns a JavaScript value.
	// description:
	//		Throws for invalid JavaScript expressions. It does not use a strict JSON parser. It
	//		always delegates to eval(). The content passed to this method must therefore come
	//		from a trusted source.
	//		It is recommend that you use dojo/json's parse function for an
	//		implementation uses the (faster) native JSON parse when available.
	// js:
	//		a string literal of a JavaScript expression, for instance:
	//			`'{ "foo": [ "bar", 1, { "baz": "thud" } ] }'`

	return eval("(" + js + ")"); // Object
};

/*=====
dojo._escapeString = function(){
	// summary:
	//		Adds escape sequences for non-visual characters, double quote and
	//		backslash and surrounds with double quotes to form a valid string
	//		literal.
};
=====*/
dojo._escapeString = json.stringify; // just delegate to json.stringify

dojo.toJsonIndentStr = "\t";
dojo.toJson = function(/*Object*/ it, /*Boolean?*/ prettyPrint){
	// summary:
	//		Returns a [JSON](http://json.org) serialization of an object.
	// description:
	//		Returns a [JSON](http://json.org) serialization of an object.
	//		Note that this doesn't check for infinite recursion, so don't do that!
	//		It is recommend that you use dojo/json's stringify function for an lighter
	//		and faster implementation that matches the native JSON API and uses the
	//		native JSON serializer when available.
	// it:
	//		an object to be serialized. Objects may define their own
	//		serialization via a special "__json__" or "json" function
	//		property. If a specialized serializer has been defined, it will
	//		be used as a fallback.
	//		Note that in 1.6, toJson would serialize undefined, but this no longer supported
	//		since it is not supported by native JSON serializer.
	// prettyPrint:
	//		if true, we indent objects and arrays to make the output prettier.
	//		The variable `dojo.toJsonIndentStr` is used as the indent string --
	//		to use something other than the default (tab), change that variable
	//		before calling dojo.toJson().
	//		Note that if native JSON support is available, it will be used for serialization,
	//		and native implementations vary on the exact spacing used in pretty printing.
	// returns:
	// 		A JSON string serialization of the passed-in object.
	// example:
	//		simple serialization of a trivial object
	//		|	var jsonStr = dojo.toJson({ howdy: "stranger!", isStrange: true });
	//		|	doh.is('{"howdy":"stranger!","isStrange":true}', jsonStr);
	// example:
	//		a custom serializer for an objects of a particular class:
	//		|	dojo.declare("Furby", null, {
	//		|		furbies: "are strange",
	//		|		furbyCount: 10,
	//		|		__json__: function(){
	//		|		},
	//		|	});

	// use dojo/json
	return json.stringify(it, function(key, value){
		if(value){
			var tf = value.__json__||value.json;
			if(typeof tf == "function"){
				return tf.call(value);
			}
		}
		return value;
	}, prettyPrint && dojo.toJsonIndentStr);	// String
};

return dojo;
});

},
'dojo/NodeList-traverse':function(){
define(["./query", "./_base/lang", "./_base/array"], function(dquery, lang, array) {
	// module:
	//		dojo/NodeList-traverse
	// summary:
	//		TODOC

var NodeList = dquery.NodeList;

/*=====
dojo["NodeList-traverse"] = {
	// summary: Adds a chainable methods to dojo.query() / Nodelist instances for traversing the DOM
};

// doc alias helpers:
NodeList = dojo.NodeList;
=====*/

lang.extend(NodeList, {
	_buildArrayFromCallback: function(/*Function*/callback){
		// summary:
		// 		builds a new array of possibly differing size based on the input list.
		// 		Since the returned array is likely of different size than the input array,
		// 		the array's map function cannot be used.
		var ary = [];
		for(var i = 0; i < this.length; i++){
			var items = callback.call(this[i], this[i], ary);
			if(items){
				ary = ary.concat(items);
			}
		}
		return ary;	//Array
	},

	_getUniqueAsNodeList: function(/*Array*/ nodes){
		// summary:
		// 		given a list of nodes, make sure only unique
		// 		elements are returned as our NodeList object.
		// 		Does not call _stash().
		var ary = [];
		//Using for loop for better speed.
		for(var i = 0, node; node = nodes[i]; i++){
			//Should be a faster way to do this. dojo.query has a private
			//_zip function that may be inspirational, but there are pathways
			//in query that force nozip?
			if(node.nodeType == 1 && array.indexOf(ary, node) == -1){
				ary.push(node);
			}
		}
		return this._wrap(ary, null, this._NodeListCtor);	 //dojo.NodeList
	},

	_getUniqueNodeListWithParent: function(/*Array*/ nodes, /*String*/ query){
		// summary:
		// 		gets unique element nodes, filters them further
		// 		with an optional query and then calls _stash to track parent NodeList.
		var ary = this._getUniqueAsNodeList(nodes);
		ary = (query ? dquery._filterResult(ary, query) : ary);
		return ary._stash(this);  //dojo.NodeList
	},

	_getRelatedUniqueNodes: function(/*String?*/ query, /*Function*/ callback){
		// summary:
		// 		cycles over all the nodes and calls a callback
		// 		to collect nodes for a possible inclusion in a result.
		// 		The callback will get two args: callback(node, ary),
		// 		where ary is the array being used to collect the nodes.
		return this._getUniqueNodeListWithParent(this._buildArrayFromCallback(callback), query);  //dojo.NodeList
	},

	children: function(/*String?*/ query){
		// summary:
		// 		Returns all immediate child elements for nodes in this dojo.NodeList.
		// 		Optionally takes a query to filter the child elements.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		// query:
		//		a CSS selector.
		// returns:
		//		dojo.NodeList, all immediate child elements for the nodes in this dojo.NodeList.
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".container").children();
		//		returns the four divs that are children of the container div.
		//		Running this code:
		//	|	dojo.query(".container").children(".red");
		//		returns the two divs that have the class "red".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			return lang._toArray(node.childNodes);
		}); //dojo.NodeList
	},

	closest: function(/*String*/ query, /*String|DOMNode?*/ root){
		// summary:
		// 		Returns closest parent that matches query, including current node in this
		// 		dojo.NodeList if it matches the query.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		// query:
		//		a CSS selector.
		// root:
		//		If specified, query is relative to "root" rather than document body.
		// returns:
		//		dojo.NodeList, the closest parent that matches the query, including the current
		//		node in this dojo.NodeList if it matches the query.
		// example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		//	|		<div class="red">Red One</div>
		//	|		Some Text
		//	|		<div class="blue">Blue One</div>
		//	|		<div class="red">Red Two</div>
		//	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".red").closest(".container");
		//		returns the div with class "container".
		return this._getRelatedUniqueNodes(null, function(node, ary){
			do{
				if(dquery._filterResult([node], query, root).length){
					return node;
				}
			}while(node != root && (node = node.parentNode) && node.nodeType == 1);
			return null; //To make rhino strict checking happy.
		}); //dojo.NodeList
	},

	parent: function(/*String?*/ query){
		// summary:
		// 		Returns immediate parent elements for nodes in this dojo.NodeList.
		// 		Optionally takes a query to filter the parent elements.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		//	query:
		//		a CSS selector.
		// returns:
		//		dojo.NodeList, immediate parent elements for nodes in this dojo.NodeList.
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		<div class="blue first"><span class="text">Blue One</span></div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue"><span class="text">Blue Two</span></div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".text").parent();
		//		returns the two divs with class "blue".
		//		Running this code:
		//	|	dojo.query(".text").parent(".first");
		//		returns the one div with class "blue" and "first".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			return node.parentNode;
		}); //dojo.NodeList
	},

	parents: function(/*String?*/ query){
		// summary:
		// 		Returns all parent elements for nodes in this dojo.NodeList.
		// 		Optionally takes a query to filter the child elements.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		//	query:
		//		a CSS selector.
		// returns:
		//		dojo.NodeList, all parent elements for nodes in this dojo.NodeList.
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		<div class="blue first"><span class="text">Blue One</span></div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue"><span class="text">Blue Two</span></div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".text").parents();
		//		returns the two divs with class "blue", the div with class "container",
		// 	|	the body element and the html element.
		//		Running this code:
		//	|	dojo.query(".text").parents(".container");
		//		returns the one div with class "container".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var pary = [];
			while(node.parentNode){
				node = node.parentNode;
				pary.push(node);
			}
			return pary;
		}); //dojo.NodeList
	},

	siblings: function(/*String?*/ query){
		// summary:
		// 		Returns all sibling elements for nodes in this dojo.NodeList.
		// 		Optionally takes a query to filter the sibling elements.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		//	query:
		//		a CSS selector.
		// returns:
		//		dojo.NodeList, all sibling elements for nodes in this dojo.NodeList.
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".first").siblings();
		//		returns the two divs with class "red" and the other div
		// 	|	with class "blue" that does not have "first".
		//		Running this code:
		//	|	dojo.query(".first").siblings(".red");
		//		returns the two div with class "red".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var pary = [];
			var nodes = (node.parentNode && node.parentNode.childNodes);
			for(var i = 0; i < nodes.length; i++){
				if(nodes[i] != node){
					pary.push(nodes[i]);
				}
			}
			return pary;
		}); //dojo.NodeList
	},

	next: function(/*String?*/ query){
		// summary:
		// 		Returns the next element for nodes in this dojo.NodeList.
		// 		Optionally takes a query to filter the next elements.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		//	query:
		//		a CSS selector.
		// returns:
		//		dojo.NodeList, the next element for nodes in this dojo.NodeList.
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue last">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".first").next();
		//		returns the div with class "red" and has innerHTML of "Red Two".
		//		Running this code:
		//	|	dojo.query(".last").next(".red");
		//		does not return any elements.
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var next = node.nextSibling;
			while(next && next.nodeType != 1){
				next = next.nextSibling;
			}
			return next;
		}); //dojo.NodeList
	},

	nextAll: function(/*String?*/ query){
		// summary:
		// 		Returns all sibling elements that come after the nodes in this dojo.NodeList.
		// 		Optionally takes a query to filter the sibling elements.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		//	query:
		//		a CSS selector.
		// returns:
		//		dojo.NodeList, all sibling elements that come after the nodes in this dojo.NodeList.
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red next">Red Two</div>
		// 	|		<div class="blue next">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".first").nextAll();
		//		returns the two divs with class of "next".
		//		Running this code:
		//	|	dojo.query(".first").nextAll(".red");
		//		returns the one div with class "red" and innerHTML "Red Two".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var pary = [];
			var next = node;
			while((next = next.nextSibling)){
				if(next.nodeType == 1){
					pary.push(next);
				}
			}
			return pary;
		}); //dojo.NodeList
	},

	prev: function(/*String?*/ query){
		// summary:
		// 		Returns the previous element for nodes in this dojo.NodeList.
		// 		Optionally takes a query to filter the previous elements.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		//	query:
		//		a CSS selector.
		// returns:
		//		dojo.NodeList, the previous element for nodes in this dojo.NodeList.
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".first").prev();
		//		returns the div with class "red" and has innerHTML of "Red One".
		//		Running this code:
		//	|	dojo.query(".first").prev(".blue");
		//		does not return any elements.
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var prev = node.previousSibling;
			while(prev && prev.nodeType != 1){
				prev = prev.previousSibling;
			}
			return prev;
		}); //dojo.NodeList
	},

	prevAll: function(/*String?*/ query){
		// summary:
		// 		Returns all sibling elements that come before the nodes in this dojo.NodeList.
		// 		Optionally takes a query to filter the sibling elements.
		// description:
		// 		The returned nodes will be in reverse DOM order -- the first node in the list will
		// 		be the node closest to the original node/NodeList.
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		//	query:
		//		a CSS selector.
		// returns:
		//		dojo.NodeList, all sibling elements that come before the nodes in this dojo.NodeList.
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red prev">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue prev">Blue One</div>
		// 	|		<div class="red second">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".second").prevAll();
		//		returns the two divs with class of "prev".
		//		Running this code:
		//	|	dojo.query(".first").prevAll(".red");
		//		returns the one div with class "red prev" and innerHTML "Red One".
		return this._getRelatedUniqueNodes(query, function(node, ary){
			var pary = [];
			var prev = node;
			while((prev = prev.previousSibling)){
				if(prev.nodeType == 1){
					pary.push(prev);
				}
			}
			return pary;
		}); //dojo.NodeList
	},

	andSelf: function(){
		// summary:
		// 		Adds the nodes from the previous dojo.NodeList to the current dojo.NodeList.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		// returns:
		//		dojo.NodeList
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red prev">Red One</div>
		// 	|		Some Text
		// 	|		<div class="blue prev">Blue One</div>
		// 	|		<div class="red second">Red Two</div>
		// 	|		<div class="blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".second").prevAll().andSelf();
		//		returns the two divs with class of "prev", as well as the div with class "second".
		return this.concat(this._parent);	//dojo.NodeList
	},

	//Alternate methods for the :first/:last/:even/:odd pseudos.
	first: function(){
		// summary:
		// 		Returns the first node in this dojo.NodeList as a dojo.NodeList.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		// returns:
		//		dojo.NodeList, with the first node in this dojo.NodeList
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue last">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".blue").first();
		//		returns the div with class "blue" and "first".
		return this._wrap(((this[0] && [this[0]]) || []), this); //dojo.NodeList
	},

	last: function(){
		// summary:
		// 		Returns the last node in this dojo.NodeList as a dojo.NodeList.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		// returns:
		//		dojo.NodeList, with the last node in this dojo.NodeList
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="red">Red One</div>
		// 	|		<div class="blue first">Blue One</div>
		// 	|		<div class="red">Red Two</div>
		// 	|		<div class="blue last">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".blue").last();
		//		returns the last div with class "blue",
		return this._wrap((this.length ? [this[this.length - 1]] : []), this); //dojo.NodeList
	},

	even: function(){
		// summary:
		// 		Returns the even nodes in this dojo.NodeList as a dojo.NodeList.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		// returns:
		//		dojo.NodeList, with the even nodes in this dojo.NodeList
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="interior red">Red One</div>
		// 	|		<div class="interior blue">Blue One</div>
		// 	|		<div class="interior red">Red Two</div>
		// 	|		<div class="interior blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".interior").even();
		//		returns the two divs with class "blue"
		return this.filter(function(item, i){
			return i % 2 != 0;
		}); //dojo.NodeList
	},

	odd: function(){
		// summary:
		// 		Returns the odd nodes in this dojo.NodeList as a dojo.NodeList.
		// description:
		// 		.end() can be used on the returned dojo.NodeList to get back to the
		// 		original dojo.NodeList.
		// returns:
		//		dojo.NodeList, with the odd nodes in this dojo.NodeList
		//	example:
		//		assume a DOM created by this markup:
		//	|	<div class="container">
		// 	|		<div class="interior red">Red One</div>
		// 	|		<div class="interior blue">Blue One</div>
		// 	|		<div class="interior red">Red Two</div>
		// 	|		<div class="interior blue">Blue Two</div>
		//	|	</div>
		//		Running this code:
		//	|	dojo.query(".interior").odd();
		//		returns the two divs with class "red"
		return this.filter(function(item, i){
			return i % 2 == 0;
		}); //dojo.NodeList
	}
});

return NodeList;
});

},
'com.ibm.bpm.coach/dataBindingHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all utilities that Coach runtime supports validation

define(["require", "dojo/_base/lang", "./CONSTANTS", "./bindingResolver"], function(require, lang, CONSTANTS, bindingResolver) {
				
	var dataBindingHelper = new Object();
	
	// rtDataBinding provides 3 helper method for resolving List related data-binding during runtime.
	dataBindingHelper.rtDataBinding = new Object();
	dataBindingHelper.rtDataBinding._get = function(viewElement) {
		var rtBinding = viewElement.getAttribute(CONSTANTS.DATA_BINDING_RT);
		if(!rtBinding) {
			return viewElement.getAttribute(CONSTANTS.DATA_BINDING);
		} else {
			return rtBinding;
		}
	}	
	dataBindingHelper.rtDataBinding._set = function(viewElement, bindingValue) {
		viewElement.setAttribute(CONSTANTS.DATA_BINDING_RT, bindingValue);
	}
	dataBindingHelper.rtDataBinding._delete = function(viewElement) {	
		// viewElement.removeAttribute(CONSTANTS.DATA_BINDING_RT);
	}
	
	dataBindingHelper.collectBpmIds = function(bindingStr) {
		var bpmIds = null;
		if(bindingStr.indexOf("[") == -1) {
			return bpmIds;
		}
		var pathArray = bindingStr.split(".");
		var tempBindingStr = null;
		for (var i=0; i<pathArray.length; i++) {
			var aPath = pathArray[i];
			var leftIndexPosition = aPath.indexOf("[");
			if (leftIndexPosition != -1) {
				var rightIndexPosition = aPath.indexOf("]");
				if(rightIndexPosition - leftIndexPosition > 1) {
					// encounters abc[n], need to find out bpm_id from the index (n)
					var indexValue = aPath.substring(leftIndexPosition+1, rightIndexPosition);
					var listShortName = aPath.substring(0, leftIndexPosition);
					var listBindingStr = !tempBindingStr ? listShortName : tempBindingStr + "." + listShortName;
					var resolvedBinding = bindingResolver.resolveBinding(listBindingStr);
					if(!resolvedBinding) {
						// can't resolve this binding. The rest of bindings won't be resolved for sure. So stop here.
						break;
					}
					var list = resolvedBinding.get("value");
					var bpmId = list.indexToId[indexValue];
					if(!bpmIds) {
						bpmIds = new Array();
					}
					bpmIds.push(bpmId);
				}
			}
			
			if(!tempBindingStr) {
				tempBindingStr = aPath;
			} else {
				tempBindingStr += "." + aPath;
			}
		}
		
		return bpmIds;
	}

	// update RTDataBinding for specified view and its all descendants views.
	dataBindingHelper.updateAllRtDataBinding = function(viewScope, bindingStrToListMap) {
    	if(!!viewScope._bpmIds) {
    		if(!bindingStrToListMap) {
    			bindingStrToListMap = {};
    		}
			var failToResolveIndex = false;
    		var updatedBindingStr = viewScope.context.element.getAttribute(CONSTANTS.DATA_BINDING);
			var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
    		for(var k=0; k<viewScope._bpmIds.length; k++) {
    			var position = updatedBindingStr.indexOf(indexWithBracket);
    			var listBindingStr = updatedBindingStr.substring(0, position);
				var list = bindingStrToListMap[listBindingStr];
				if(!list) {
					list = bindingResolver.resolveBinding(listBindingStr).get("value");
					bindingStrToListMap[listBindingStr] = list;
				}
				var currentIndex = list.idToIndex(viewScope._bpmIds[k]);
				if(currentIndex == null || typeof currentIndex == "undefined") {
					// example scenario: list item is deleted, but table row has not been removed. table cell (coach view) still remembers the bpmId that has no matching index in the list.
					failToResolveIndex = true;
					break;
				}
    			var replaceValue = "[" + currentIndex + "]";
    			updatedBindingStr = updatedBindingStr.replace(indexWithBracket, replaceValue);
    		}
    		
			if(!failToResolveIndex) {
				// let view instance know the new bindingStr
				viewScope._setRtDataBindingStr(updatedBindingStr);
			}
    	}
		
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					dataBindingHelper.updateAllRtDataBinding(subviews[divId], bindingStrToListMap);
				}
			}		
		}			
	}
	
	// this function is NOT called at all so far
	// make sure every RTDataBinding in current coach is up to date
	dataBindingHelper.updateRtDataBindingGlobally = function() {
		var bindingStrToListMap = new Object();
		var topLevelViews = bindingResolver.globalAssets.topLevelViewScope;
		for (var viewId in topLevelViews) {
			var viewScope = topLevelViews[viewId];
			dataBindingHelper.updateAllRtDataBinding(viewScope, bindingStrToListMap);
		}
	}

	// it's array if end with "[]", otherwise resolve the binding to check if it is type of com.ibm.bpm.coach.List
	dataBindingHelper.isBindingArray = function(bindingStr) {
		var refArray = bindingStr.split(".");
		if(refArray[0] == "local") {
			return (bindingStr.substring(bindingStr.length - 2) == "[]");
		} else {
			// could be config option
			var binding = bindingResolver.resolveBinding(bindingStr);
			if(!!binding && binding.get("value") instanceof com.ibm.bpm.coach.List) {
				return true;
			}
		}
			
		return false;
	}	
	
	dataBindingHelper.isNumber = function(stringValue) {
		var convertToNumber = Number(stringValue);
		return !isNaN(convertToNumber);
	}
	
	dataBindingHelper.getNumberOfIndex = function(bindingStr) {
		var pathArray = bindingStr.split(".");
		var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
		var numOfIndex = 0;
		for(var k=0; k<pathArray.length; k++) {
			if(pathArray[k].indexOf(indexWithBracket) != -1) {
				numOfIndex++;
			}			
		}		
		return numOfIndex;
	}
	
	dataBindingHelper.updateBindingStr = function(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating, numOfIndex) {
		if(!numOfIndex) {
			numOfIndex = dataBindingHelper.getNumberOfIndex(bindingStr);
		}
		
		if(numOfIndex >= maxNumOfUnresolvedRepeating) {
			var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
			var replaceValue = "[" + bindingIndex + "]";
			var newBinding;
			if(maxNumOfUnresolvedRepeating == 1 && numOfIndex > 1) {
				// assign all the outstanding "[index]" with current bindingIndex since this is the last repeating index
				newBinding = bindingStr;
				for(var k=0; k<numOfIndex; k++) {
					newBinding =newBinding.replace(indexWithBracket, replaceValue);
				}
			} else {
				newBinding = bindingStr.replace(indexWithBracket, replaceValue); // replace very first "[index]"
			}
			
			return newBinding; 
		} else {
			// now numOfIndex < maxNumOfUnresolvedRepeating. the unresolved "[index]" should be reserved for the following repeating
			return bindingStr;
		}				
	
	}
	
	// return the plain value (could be complex object) for the specified property on the Stateful or List object
	dataBindingHelper.getPlainValue = function(statefulOrListObject, property) {
		if(!!statefulOrListObject) {
			if (statefulOrListObject instanceof com.ibm.bpm.coach.List) {
				// List
				if(parseInt(property) >= 0) {
					// List item
					return statefulOrListObject.items[property];
				} else {
					// List property, e.g listSelected
					return statefulOrListObject[property];
				}
			} else {
				// Stateful
				return statefulOrListObject[property];
			}
		}
		return null;
	}
	
	// Example, bindingPath is "local.a.b.c", name is "d", value is "value of new d".
	// set("d", "value of new d") just occurred on the Stateful object represented by "local.a.b.c". So the plain value of d on this Stateful object is up to date.
	// However, the Stateful object representd by "local.a.b" has plain value of c which contains plain value of d. 
	// Dojo Stateful doesn't update the plain value c.d under "local.a.b", b.c.d under "local.a", and a.b.c.d under "local". This function updates those plain values.
	dataBindingHelper.updatePlainValueInStatefulChain = function(bindingPath, name, value) {
		var pathArray = dataBindingHelper._computePaths(bindingPath);
		// Example of pathArray: 
		// "local.a.b"							-> ["local", "a", "b"]
		// "local.a.b[1]"						-> ["local", "a", "b", "[1]"]
		// "local.a.b[1].c[2].d"				-> ["local", "a", "b", "[1]", "c", "[2]", "d"]
		// "config.config1.myOption"			-> ["config", "config1", "myOption"]
		// "config.config2._metadata.label"		-> ["config", "config2", "_metadata", "label"]
		// "config.config3.myOption.p1"			-> ["config", "config3", "myOption", "p1"]
		// "config.config3.myListOption[2].p1"	-> ["config", "config3", "myListOption", "[2]", "p1"]
		
		var statefulObject;
		// For pathArray ["local", "a", "b"], b represents a Stateful object. the plain value of "name" (e.g. "c")  on that Stateful object is already up to date.
		// So, last Stateful object that needs to update plain value "c" is "b" plain object stored in Stateful represented by a.
		var lastPathIndex = pathArray.length - 1;
		var firstPathIndex;
		
		if(pathArray[0] == "local") {
			firstPathIndex = 1;
			statefulObject = bindingResolver.globalAssets["local"];
		} else if(pathArray[0] == "config"){
			// first 2 items in pathArray don't represent Stateful object for non-metadata option, e.g. ["config", "config1", "myOption"]
			// first 3 items in pathArray don't represent Stateful object for metadata option, e.g. ["config", "config2", "_metadata", "label"]
			var pathForRootStateful = pathArray[0];
			if(pathArray[2] == "_metadata") {
				firstPathIndex = 4;
			} else {
				firstPathIndex = 3;
			}
			
			if(firstPathIndex >= pathArray.length) {
				// nothing needs to update
				statefulObject = null;
			} else {			
				for(var k=1; k<firstPathIndex; k++) {
					pathForRootStateful += "." + pathArray[k];
				}
				statefulObject = bindingResolver.resolveBinding(pathForRootStateful);
			}
		}
		
		if(statefulObject && firstPathIndex <= lastPathIndex) {
			var clonedPlainValue = lang.clone(value);
			for(var k=firstPathIndex; k<=lastPathIndex; k++) {
				var currentPath = pathArray[k];
				if(currentPath.indexOf("[") != -1) {
					// statefulObject is a List
					var index = currentPath.substring(1, currentPath.length-1);
					var plainObject = statefulObject["items"][index];
					statefulObject = statefulObject.get(index);
				} else {
					var plainObject = statefulObject[currentPath];
					statefulObject = statefulObject.get(currentPath);
				}
				dataBindingHelper._updatePlainValue(plainObject, pathArray, k+1, name, clonedPlainValue);
				if(!statefulObject) {
					return;
				}
			}
		}
	}
	
	dataBindingHelper._computePaths = function(bindingStr) {
		var sourcePathArray = bindingStr.split(".");
		var finalPathArray = [];

		for(var k=0; k<sourcePathArray.length; k++) {
			onePath = sourcePathArray[k];
			var positionOfLeftBracket = onePath.indexOf("[");
			if(positionOfLeftBracket != -1) {
				// name of the List
				finalPathArray.push(onePath.substring(0, positionOfLeftBracket));
				// index with bracket, e.g. [1]
				finalPathArray.push(onePath.substring(positionOfLeftBracket));				
			} else {
				finalPathArray.push(onePath);
			}
		}	
		return finalPathArray;
	}

	dataBindingHelper._updatePlainValue = function(plainObject, pathArray, index, name, value) {
		if(!plainObject) {
			return;
		}
		
		var target = plainObject;
		for(var k=index; k<pathArray.length; k++) {
			var currentPath = pathArray[k];
			if(currentPath.indexOf("[") != -1) {
				// target is a List
				var index = currentPath.substring(1, currentPath.length-1);
				target = target["items"][index];
			} else {
				target = target[currentPath];
			}
			if(!target) {
				return;
			}
		}
		target[name] = value;
	}
		
	return dataBindingHelper;
});
	
},
'dojo/i18n':function(){
define(["./_base/kernel", "require", "./has", "./_base/array", "./_base/config", "./_base/lang", "./_base/xhr", "./json"],
	function(dojo, require, has, array, config, lang, xhr, json) {
	// module:
	//		dojo/i18n
	// summary:
	//		This module implements the !dojo/i18n plugin and the v1.6- i18n API
	// description:
	//		We choose to include our own plugin to leverage functionality already contained in dojo
	//		and thereby reduce the size of the plugin compared to various loader implementations. Also, this
	//		allows foreign AMD loaders to be used without their plugins.


	has.add("dojo-preload-i18n-Api",
		// if true, define the preload localizations machinery
		1
	);

	true || has.add("dojo-v1x-i18n-Api",
		// if true, define the v1.x i18n functions
		1
	);

	var
		thisModule= dojo.i18n=
			// the dojo.i18n module
			{},

		nlsRe=
			// regexp for reconstructing the master bundle name from parts of the regexp match
			// nlsRe.exec("foo/bar/baz/nls/en-ca/foo") gives:
			// ["foo/bar/baz/nls/en-ca/foo", "foo/bar/baz/nls/", "/", "/", "en-ca", "foo"]
			// nlsRe.exec("foo/bar/baz/nls/foo") gives:
			// ["foo/bar/baz/nls/foo", "foo/bar/baz/nls/", "/", "/", "foo", ""]
			// so, if match[5] is blank, it means this is the top bundle definition.
			// courtesy of http://requirejs.org
			/(^.*(^|\/)nls)(\/|$)([^\/]*)\/?([^\/]*)/,

		getAvailableLocales= function(
			root,
			locale,
			bundlePath,
			bundleName
		){
			// return a vector of module ids containing all available locales with respect to the target locale
			// For example, assuming:
			//	 * the root bundle indicates specific bundles for "fr" and "fr-ca",
			//	 * bundlePath is "myPackage/nls"
			//	 * bundleName is "myBundle"
			// Then a locale argument of "fr-ca" would return
			//	 ["myPackage/nls/myBundle", "myPackage/nls/fr/myBundle", "myPackage/nls/fr-ca/myBundle"]
			// Notice that bundles are returned least-specific to most-specific, starting with the root.
			//
			// If root===false indicates we're working with a pre-AMD i18n bundle that doesn't tell about the available locales;
			// therefore, assume everything is available and get 404 errors that indicate a particular localization is not available
			//

			for(var result= [bundlePath + bundleName], localeParts= locale.split("-"), current= "", i= 0; i<localeParts.length; i++){
				current+= (current ? "-" : "") + localeParts[i];
				if(!root || root[current]){
					result.push(bundlePath + current + "/" + bundleName);
				}
			}
			return result;
		},

		cache= {},

		getL10nName= dojo.getL10nName = function(moduleName, bundleName, locale){
			locale = locale ? locale.toLowerCase() : dojo.locale;
			moduleName = "dojo/i18n!" + moduleName.replace(/\./g, "/");
			bundleName = bundleName.replace(/\./g, "/");
			return (/root/i.test(locale)) ?
				(moduleName + "/nls/" + bundleName) :
				(moduleName + "/nls/" + locale + "/" + bundleName);
		},

		doLoad = function(require, bundlePathAndName, bundlePath, bundleName, locale, load){
			// get the root bundle which instructs which other bundles are required to construct the localized bundle
			require([bundlePathAndName], function(root){
				var current= lang.clone(root.root),
					availableLocales= getAvailableLocales(!root._v1x && root, locale, bundlePath, bundleName);
				require(availableLocales, function(){
					for (var i= 1; i<availableLocales.length; i++){
						current= lang.mixin(lang.clone(current), arguments[i]);
					}
					// target may not have been resolve (e.g., maybe only "fr" exists when "fr-ca" was requested)
					var target= bundlePathAndName + "/" + locale;
					cache[target]= current;
					load();
				});
			});
		},

		normalize = function(id, toAbsMid){
			// id may be relative
			// preload has form *preload*<path>/nls/<module>*<flattened locales> and
			// therefore never looks like a relative
			return /^\./.test(id) ? toAbsMid(id) : id;
		},

		getLocalesToLoad = function(targetLocale){
			var list = config.extraLocale || [];
			list = lang.isArray(list) ? list : [list];
			list.push(targetLocale);
			return list;
		},

		load = function(id, require, load){
			//
			// id is in one of the following formats
			//
			//	1. <path>/nls/<bundle>
			//		=> load the bundle, localized to config.locale; load all bundles localized to
			//      config.extraLocale (if any); return the loaded bundle localized to config.locale.
			//
			//  2. <path>/nls/<locale>/<bundle>
			//		=> load then return the bundle localized to <locale>
			//
			//  3. *preload*<path>/nls/<module>*<JSON array of available locales>
			//		=> for config.locale and all config.extraLocale, load all bundles found
			//		   in the best-matching bundle rollup. A value of 1 is returned, which
			//         is meaningless other than to say the plugin is executing the requested
			//         preloads
			//
			// In cases 1 and 2, <path> is always normalized to an absolute module id upon entry; see
			// normalize. In case 3, it <path> is assumed to be absolue; this is arranged by the builder.
			//
			// To load a bundle means to insert the bundle into the plugin's cache and publish the bundle
			// value to the loader. Given <path>, <bundle>, and a particular <locale>, the cache key
			//
			//   <path>/nls/<bundle>/<locale>
			//
			// will hold the value. Similarly, then plugin will publish this value to the loader by
			//
			//   define("<path>/nls/<bundle>/<locale>", <bundle-value>);
			//
			// Given this algorithm, other machinery can provide fast load paths be preplacing
			// values in the plugin's cache, which is public. When a load is demanded the
			// cache is inspected before starting any loading. Explicitly placing values in the plugin
			// cache is an advanced/experimental feature that should not be needed; use at your own risk.
			//
			// For the normal AMD algorithm, the root bundle is loaded first, which instructs the
			// plugin what additional localized bundles are required for a particular locale. These
			// additional locales are loaded and a mix of the root and each progressively-specific
			// locale is returned. For example:
			//
			// 1. The client demands "dojo/i18n!some/path/nls/someBundle
			//
			// 2. The loader demands load(some/path/nls/someBundle)
			//
			// 3. This plugin require's "some/path/nls/someBundle", which is the root bundle.
			//
			// 4. Assuming config.locale is "ab-cd-ef" and the root bundle indicates that localizations
			//    are available for "ab" and "ab-cd-ef" (note the missing "ab-cd", then the plugin
			//    requires "some/path/nls/ab/someBundle" and "some/path/nls/ab-cd-ef/someBundle"
			//
			// 5. Upon receiving all required bundles, the plugin constructs the value of the bundle
			//    ab-cd-ef as...
			//
			//      mixin(mixin(mixin({}, require("some/path/nls/someBundle"),
			//        require("some/path/nls/ab/someBundle")),
			//          require("some/path/nls/ab-cd-ef/someBundle"));
			//
			//    This value is inserted into the cache and published to the loader at the
			//    key/module-id some/path/nls/someBundle/ab-cd-ef.
			//
			// The special preload signature (case 3) instructs the plugin to stop servicing all normal requests
			// (further preload requests will be serviced) until all ongoing preloading has completed.
			//
			// The preload signature instructs the plugin that a special rollup module is available that contains
			// one or more flattened, localized bundles. The JSON array of available locales indicates which locales
			// are available. Here is an example:
			//
			//   *preload*some/path/nls/someModule*["root", "ab", "ab-cd-ef"]
			//
			// This indicates the following rollup modules are available:
			//
			//   some/path/nls/someModule_ROOT
			//   some/path/nls/someModule_ab
			//   some/path/nls/someModule_ab-cd-ef
			//
			// Each of these modules is a normal AMD module that contains one or more flattened bundles in a hash.
			// For example, assume someModule contained the bundles some/bundle/path/someBundle and
			// some/bundle/path/someOtherBundle, then some/path/nls/someModule_ab would be expressed as folllows:
			//
			// define({
			//   some/bundle/path/someBundle:<value of someBundle, flattened with respect to locale ab>,
			//   some/bundle/path/someOtherBundle:<value of someOtherBundle, flattened with respect to locale ab>,
			// });
			//
			// E.g., given this design, preloading for locale=="ab" can execute the following algorithm:
			//
			// require(["some/path/nls/someModule_ab"], function(rollup){
			//   for(var p in rollup){
			//     var id = p + "/ab",
			//     cache[id] = rollup[p];
			//     define(id, rollup[p]);
			//   }
			// });
			//
			// Similarly, if "ab-cd" is requested, the algorithm can determine that "ab" is the best available and
			// load accordingly.
			//
			// The builder will write such rollups for every layer if a non-empty localeList  profile property is
			// provided. Further, the builder will include the following cache entry in the cache associated with
			// any layer.
			//
			//   "*now":function(r){r(['dojo/i18n!*preload*<path>/nls/<module>*<JSON array of available locales>']);}
			//
			// The *now special cache module instructs the loader to apply the provided function to context-require
			// with respect to the particular layer being defined. This causes the plugin to hold all normal service
			// requests until all preloading is complete.
			//
			// Notice that this algorithm is rarely better than the standard AMD load algorithm. Consider the normal case
			// where the target locale has a single segment and a layer depends on a single bundle:
			//
			// Without Preloads:
			//
			//   1. Layer loads root bundle.
			//   2. bundle is demanded; plugin loads single localized bundle.
			//
			// With Preloads:
			//
			//   1. Layer causes preloading of target bundle.
			//   2. bundle is demanded; service is delayed until preloading complete; bundle is returned.
			//
			// In each case a single transaction is required to load the target bundle. In cases where multiple bundles
			// are required and/or the locale has multiple segments, preloads still requires a single transaction whereas
			// the normal path requires an additional transaction for each additional bundle/locale-segment. However all
			// of these additional transactions can be done concurrently. Owing to this analysis, the entire preloading
			// algorithm can be discard during a build by setting the has feature dojo-preload-i18n-Api to false.
			//
			if(has("dojo-preload-i18n-Api")){
				var split = id.split("*"),
					preloadDemand = split[1]=="preload";
				if(preloadDemand){
					if(!cache[id]){
						// use cache[id] to prevent multiple preloads of the same preload; this shouldn't happen, but
						// who knows what over-aggressive human optimizers may attempt
						cache[id] = 1;
						preloadL10n(split[2], json.parse(split[3]), 1);
					}
					// don't stall the loader!
					load(1);
				}
				if(preloadDemand || waitForPreloads(id, require, load)){
					return;
				}
			}

			var match= nlsRe.exec(id),
				bundlePath= match[1] + "/",
				bundleName= match[5] || match[4],
				bundlePathAndName= bundlePath + bundleName,
				localeSpecified = (match[5] && match[4]),
				targetLocale=  localeSpecified || dojo.locale,
				loadTarget= bundlePathAndName + "/" + targetLocale,
				loadList = localeSpecified ? [targetLocale] : getLocalesToLoad(targetLocale),
				remaining = loadList.length,
				finish = function(){
					if(!--remaining){
						load(lang.delegate(cache[loadTarget]));
					}
				};
			array.forEach(loadList, function(locale){
				var target = bundlePathAndName + "/" + locale;
				if(has("dojo-preload-i18n-Api")){
					checkForLegacyModules(target);
				}
				if(!cache[target]){
					doLoad(require, bundlePathAndName, bundlePath, bundleName, locale, finish);
				}else{
					finish();
				}
			});
		};

	if(has("dojo-unit-tests")){
		var unitTests = thisModule.unitTests = [];
	}

	if(has("dojo-preload-i18n-Api") || 1){
		var normalizeLocale = thisModule.normalizeLocale= function(locale){
				var result = locale ? locale.toLowerCase() : dojo.locale;
				return result == "root" ? "ROOT" : result;
			},

			isXd = function(mid){
				return (1 && 1) ?
					require.isXdUrl(require.toUrl(mid + ".js")) :
					true;
			},

			preloading = 0,

			preloadWaitQueue = [],

			preloadL10n = thisModule._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated, /*boolean*/ guaranteedAmdFormat){
				//	summary:
				//		Load available flattened resource bundles associated with a particular module for dojo.locale and all dojo.config.extraLocale (if any)
				//
				//  descirption:
				//		Only called by built layer files. The entire locale hierarchy is loaded. For example,
				//		if locale=="ab-cd", then ROOT, "ab", and "ab-cd" are loaded. This is different than v1.6-
				//		in that the v1.6- would lonly load ab-cd...which was *always* flattened.
				//
				//		If guaranteedAmdFormat is true, then the module can be loaded with require thereby circumventing the detection algorithm
				//		and the extra possible extra transaction.
				//

				function forEachLocale(locale, func){
					// given locale= "ab-cd-ef", calls func on "ab-cd-ef", "ab-cd", "ab", "ROOT"; stops calling the first time func returns truthy
					var parts = locale.split("-");
					while(parts.length){
						if(func(parts.join("-"))){
							return true;
						}
						parts.pop();
					}
					return func("ROOT");
				}

				function preload(locale){
					locale = normalizeLocale(locale);
					forEachLocale(locale, function(loc){
						if(array.indexOf(localesGenerated, loc)>=0){
							var mid = bundlePrefix.replace(/\./g, "/")+"_"+loc;
							preloading++;
							(isXd(mid) || guaranteedAmdFormat ? require : syncRequire)([mid], function(rollup){
								for(var p in rollup){
									cache[p + "/" + loc] = rollup[p];
								}
								--preloading;
								while(!preloading && preloadWaitQueue.length){
									load.apply(null, preloadWaitQueue.shift());
								}
							});
							return true;
						}
						return false;
					});
				}

				preload();
				array.forEach(dojo.config.extraLocale, preload);
			},

			waitForPreloads = function(id, require, load){
				if(preloading){
					preloadWaitQueue.push([id, require, load]);
				}
				return preloading;
			};
	}

	if(1){
		// this code path assumes the dojo loader and won't work with a standard AMD loader
		var evalBundle=
				// use the function ctor to keep the minifiers away (also come close to global scope, but this is secondary)
				new Function(
					"__bundle",                // the bundle to evalutate
					"__checkForLegacyModules", // a function that checks if __bundle defined __mid in the global space
					"__mid",                   // the mid that __bundle is intended to define

					// returns one of:
					//		1 => the bundle was an AMD bundle
					//		a legacy bundle object that is the value of __mid
					//		instance of Error => could not figure out how to evaluate bundle

					  // used to detect when __bundle calls define
					  "var define = function(){define.called = 1;},"
					+ "    require = function(){define.called = 1;};"

					+ "try{"
					+		"define.called = 0;"
					+		"eval(__bundle);"
					+		"if(define.called==1)"
								// bundle called define; therefore signal it's an AMD bundle
					+			"return 1;"

					+		"if((__checkForLegacyModules = __checkForLegacyModules(__mid)))"
								// bundle was probably a v1.6- built NLS flattened NLS bundle that defined __mid in the global space
					+			"return __checkForLegacyModules;"

					+ "}catch(e){}"
					// evaulating the bundle was *neither* an AMD *nor* a legacy flattened bundle
					// either way, re-eval *after* surrounding with parentheses

					+ "try{"
					+ 		"return eval('('+__bundle+')');"
					+ "}catch(e){"
					+ 		"return e;"
					+ "}"
				),

			syncRequire= function(deps, callback){
				var results= [];
				array.forEach(deps, function(mid){
					var url= require.toUrl(mid + ".js");

					function load(text){
						var result = evalBundle(text, checkForLegacyModules, mid);
						if(result===1){
							// the bundle was an AMD module; re-inject it through the normal AMD path
							// we gotta do this since it could be an anonymous module and simply evaluating
							// the text here won't provide the loader with the context to know what
							// module is being defined()'d. With browser caching, this should be free; further
							// this entire code path can be circumvented by using the AMD format to begin with
							require([mid], function(bundle){
								results.push(cache[url]= bundle);
							});
						}else{
							if(result instanceof Error){
								console.error("failed to evaluate i18n bundle; url=" + url, result);
								result = {};
							}
							// nls/<locale>/<bundle-name> indicates not the root.
							results.push(cache[url] = (/nls\/[^\/]+\/[^\/]+$/.test(url) ? result : {root:result, _v1x:1}));
						}
					}

					if(cache[url]){
						results.push(cache[url]);
					}else{
						var bundle= require.syncLoadNls(mid);
						// don't need to check for legacy since syncLoadNls returns a module if the module
						// (1) was already loaded, or (2) was in the cache. In case 1, if syncRequire is called
						// from getLocalization --> load, then load will have called checkForLegacyModules() before
						// calling syncRequire; if syncRequire is called from preloadLocalizations, then we
						// don't care about checkForLegacyModules() because that will be done when a particular
						// bundle is actually demanded. In case 2, checkForLegacyModules() is never relevant
						// because cached modules are always v1.7+ built modules.
						if(bundle){
							results.push(bundle);
						}else{
							if(!xhr){
								try{
									require.getText(url, true, load);
								}catch(e){
									results.push(cache[url]= {});
								}
							}else{
								xhr.get({
									url:url,
									sync:true,
									load:load,
									error:function(){
										results.push(cache[url]= {});
									}
								});
							}
						}
					}
				});
				callback && callback.apply(null, results);
			},

			checkForLegacyModules = function(target){
				// legacy code may have already loaded [e.g] the raw bundle x/y/z at x.y.z; when true, push into the cache
				for(var result, names = target.split("/"), object = dojo.global[names[0]], i = 1; object && i<names.length-1; object = object[names[i++]]){}
				if(object){
					result = object[names[i]];
					if(!result){
						// fallback for incorrect bundle build of 1.6
						result = object[names[i].replace(/-/g,"_")];
					}
					if(result){
						cache[target] = result;
					}
				}
				return result;
			};

		thisModule.getLocalization= function(moduleName, bundleName, locale){
			var result,
				l10nName= getL10nName(moduleName, bundleName, locale).substring(10);
			load(l10nName, (!isXd(l10nName) ? syncRequire : require), function(result_){ result= result_; });
			return result;
		};

		if(has("dojo-unit-tests")){
			unitTests.push(function(doh){
				doh.register("tests.i18n.unit", function(t){
					var check;

					check = evalBundle("{prop:1}");
					t.is({prop:1}, check); t.is(undefined, check[1]);

					check = evalBundle("({prop:1})");
					t.is({prop:1}, check); t.is(undefined, check[1]);

					check = evalBundle("{'prop-x':1}");
					t.is({'prop-x':1}, check); t.is(undefined, check[1]);

					check = evalBundle("({'prop-x':1})");
					t.is({'prop-x':1}, check); t.is(undefined, check[1]);

					check = evalBundle("define({'prop-x':1})");
					t.is(1, check);

					check = evalBundle("this is total nonsense and should throw an error");
					t.is(check instanceof Error, true);
				});
			});
		}
	}

	return lang.mixin(thisModule, {
		dynamic:true,
		normalize:normalize,
		load:load,
		cache:cache
	});
});

},
'dojo/dom-construct':function(){
define(["exports", "./_base/kernel", "./_base/sniff", "./_base/window", "./dom", "./dom-attr", "./on"],
		function(exports, dojo, has, win, dom, attr, on){
	// module:
	//		dojo/dom-construct
	// summary:
	//		This module defines the core dojo DOM construction API.

	/*=====
	dojo.toDom = function(frag, doc){
		// summary:
		//		instantiates an HTML fragment returning the corresponding DOM.
		// frag: String
		//		the HTML fragment
		// doc: DocumentNode?
		//		optional document to use when creating DOM nodes, defaults to
		//		dojo.doc if not specified.
		// returns: DocumentFragment
		//
		// example:
		//		Create a table row:
		//	|	var tr = dojo.toDom("<tr><td>First!</td></tr>");
	};
	=====*/

	/*=====
	dojo._toDom = function(frag, doc){
		// summary:
		//		Existing alias for `dojo.toDom`. Deprecated, will be removed in 2.0.
	};
	=====*/

	/*=====
	dojo.place = function(node, refNode, position){
		// summary:
		//		Attempt to insert node into the DOM, choosing from various positioning options.
		//		Returns the first argument resolved to a DOM node.
		//
		// node: DOMNode|String
		//		id or node reference, or HTML fragment starting with "<" to place relative to refNode
		//
		// refNode: DOMNode|String
		//		id or node reference to use as basis for placement
		//
		// position: String|Number?
		//		string noting the position of node relative to refNode or a
		//		number indicating the location in the childNodes collection of refNode.
		//		Accepted string values are:
		//	|	* before
		//	|	* after
		//	|	* replace
		//	|	* only
		//	|	* first
		//	|	* last
		//		"first" and "last" indicate positions as children of refNode, "replace" replaces refNode,
		//		"only" replaces all children.  position defaults to "last" if not specified
		//
		// returns: DOMNode
		//		Returned values is the first argument resolved to a DOM node.
		//
		//		.place() is also a method of `dojo.NodeList`, allowing `dojo.query` node lookups.
		//
		// example:
		//		Place a node by string id as the last child of another node by string id:
		//	|	dojo.place("someNode", "anotherNode");
		//
		// example:
		//		Place a node by string id before another node by string id
		//	|	dojo.place("someNode", "anotherNode", "before");
		//
		// example:
		//		Create a Node, and place it in the body element (last child):
		//	|	dojo.place("<div></div>", dojo.body());
		//
		// example:
		//		Put a new LI as the first child of a list by id:
		//	|	dojo.place("<li></li>", "someUl", "first");
	};
	=====*/

	/*=====
	dojo.create = function(tag, attrs, refNode, pos){
		// summary:
		//		Create an element, allowing for optional attribute decoration
		//		and placement.
		//
		// description:
		//		A DOM Element creation function. A shorthand method for creating a node or
		//		a fragment, and allowing for a convenient optional attribute setting step,
		//		as well as an optional DOM placement reference.
		//|
		//		Attributes are set by passing the optional object through `dojo.setAttr`.
		//		See `dojo.setAttr` for noted caveats and nuances, and API if applicable.
		//|
		//		Placement is done via `dojo.place`, assuming the new node to be the action
		//		node, passing along the optional reference node and position.
		//
		// tag: DOMNode|String
		//		A string of the element to create (eg: "div", "a", "p", "li", "script", "br"),
		//		or an existing DOM node to process.
		//
		// attrs: Object
		//		An object-hash of attributes to set on the newly created node.
		//		Can be null, if you don't want to set any attributes/styles.
		//		See: `dojo.setAttr` for a description of available attributes.
		//
		// refNode: DOMNode?|String?
		//		Optional reference node. Used by `dojo.place` to place the newly created
		//		node somewhere in the dom relative to refNode. Can be a DomNode reference
		//		or String ID of a node.
		//
		// pos: String?
		//		Optional positional reference. Defaults to "last" by way of `dojo.place`,
		//		though can be set to "first","after","before","last", "replace" or "only"
		//		to further control the placement of the new node relative to the refNode.
		//		'refNode' is required if a 'pos' is specified.
		//
		// returns: DOMNode
		//
		// example:
		//		Create a DIV:
		//	|	var n = dojo.create("div");
		//
		// example:
		//		Create a DIV with content:
		//	|	var n = dojo.create("div", { innerHTML:"<p>hi</p>" });
		//
		// example:
		//		Place a new DIV in the BODY, with no attributes set
		//	|	var n = dojo.create("div", null, dojo.body());
		//
		// example:
		//		Create an UL, and populate it with LI's. Place the list as the first-child of a
		//		node with id="someId":
		//	|	var ul = dojo.create("ul", null, "someId", "first");
		//	|	var items = ["one", "two", "three", "four"];
		//	|	dojo.forEach(items, function(data){
		//	|		dojo.create("li", { innerHTML: data }, ul);
		//	|	});
		//
		// example:
		//		Create an anchor, with an href. Place in BODY:
		//	|	dojo.create("a", { href:"foo.html", title:"Goto FOO!" }, dojo.body());
		//
		// example:
		//		Create a `dojo.NodeList()` from a new element (for syntatic sugar):
		//	|	dojo.query(dojo.create('div'))
		//	|		.addClass("newDiv")
		//	|		.onclick(function(e){ console.log('clicked', e.target) })
		//	|		.place("#someNode"); // redundant, but cleaner.
	};
	=====*/

	/*=====
	dojo.empty = function(node){
			// summary:
			//		safely removes all children of the node.
			// node: DOMNode|String
			//		a reference to a DOM node or an id.
			// example:
			//		Destroy node's children byId:
			//	|	dojo.empty("someId");
			//
			// example:
			//		Destroy all nodes' children in a list by reference:
			//	|	dojo.query(".someNode").forEach(dojo.empty);
	}
	=====*/

	/*=====
	dojo.destroy = function(node){
		// summary:
		//		Removes a node from its parent, clobbering it and all of its
		//		children.
		//
		// description:
		//		Removes a node from its parent, clobbering it and all of its
		//		children. Function only works with DomNodes, and returns nothing.
		//
		// node: DOMNode|String
		//		A String ID or DomNode reference of the element to be destroyed
		//
		// example:
		//		Destroy a node byId:
		//	|	dojo.destroy("someId");
		//
		// example:
		//		Destroy all nodes in a list by reference:
		//	|	dojo.query(".someNode").forEach(dojo.destroy);
	};
	=====*/

	/*=====
	dojo._destroyElement = function(node){
		// summary:
		//		Existing alias for `dojo.destroy`. Deprecated, will be removed in 2.0.
	};
	=====*/

	// support stuff for dojo.toDom
	var tagWrap = {
			option: ["select"],
			tbody: ["table"],
			thead: ["table"],
			tfoot: ["table"],
			tr: ["table", "tbody"],
			td: ["table", "tbody", "tr"],
			th: ["table", "thead", "tr"],
			legend: ["fieldset"],
			caption: ["table"],
			colgroup: ["table"],
			col: ["table", "colgroup"],
			li: ["ul"]
		},
		reTag = /<\s*([\w\:]+)/,
		masterNode = {}, masterNum = 0,
		masterName = "__" + dojo._scopeName + "ToDomId";

	// generate start/end tag strings to use
	// for the injection for each special tag wrap case.
	for(var param in tagWrap){
		if(tagWrap.hasOwnProperty(param)){
			var tw = tagWrap[param];
			tw.pre = param == "option" ? '<select multiple="multiple">' : "<" + tw.join("><") + ">";
			tw.post = "</" + tw.reverse().join("></") + ">";
			// the last line is destructive: it reverses the array,
			// but we don't care at this point
		}
	}

	function _insertBefore(/*DomNode*/node, /*DomNode*/ref){
		var parent = ref.parentNode;
		if(parent){
			parent.insertBefore(node, ref);
		}
	}

	function _insertAfter(/*DomNode*/node, /*DomNode*/ref){
		// summary:
		//		Try to insert node after ref
		var parent = ref.parentNode;
		if(parent){
			if(parent.lastChild == ref){
				parent.appendChild(node);
			}else{
				parent.insertBefore(node, ref.nextSibling);
			}
		}
	}

	exports.toDom = function toDom(frag, doc){
		doc = doc || win.doc;
		var masterId = doc[masterName];
		if(!masterId){
			doc[masterName] = masterId = ++masterNum + "";
			masterNode[masterId] = doc.createElement("div");
		}

		// make sure the frag is a string.
		frag += "";

		// find the starting tag, and get node wrapper
		var match = frag.match(reTag),
			tag = match ? match[1].toLowerCase() : "",
			master = masterNode[masterId],
			wrap, i, fc, df;
		if(match && tagWrap[tag]){
			wrap = tagWrap[tag];
			master.innerHTML = wrap.pre + frag + wrap.post;
			for(i = wrap.length; i; --i){
				master = master.firstChild;
			}
		}else{
			master.innerHTML = frag;
		}

		// one node shortcut => return the node itself
		if(master.childNodes.length == 1){
			return master.removeChild(master.firstChild); // DOMNode
		}

		// return multiple nodes as a document fragment
		df = doc.createDocumentFragment();
		while((fc = master.firstChild)){ // intentional assignment
			df.appendChild(fc);
		}
		return df; // DOMNode
	};

	exports.place = function place(/*DOMNode|String*/node, /*DOMNode|String*/refNode, /*String|Number?*/position){
		refNode = dom.byId(refNode);
		if(typeof node == "string"){ // inline'd type check
			node = /^\s*</.test(node) ? exports.toDom(node, refNode.ownerDocument) : dom.byId(node);
		}
		if(typeof position == "number"){ // inline'd type check
			var cn = refNode.childNodes;
			if(!cn.length || cn.length <= position){
				refNode.appendChild(node);
			}else{
				_insertBefore(node, cn[position < 0 ? 0 : position]);
			}
		}else{
			switch(position){
				case "before":
					_insertBefore(node, refNode);
					break;
				case "after":
					_insertAfter(node, refNode);
					break;
				case "replace":
					refNode.parentNode.replaceChild(node, refNode);
					break;
				case "only":
					exports.empty(refNode);
					refNode.appendChild(node);
					break;
				case "first":
					if(refNode.firstChild){
						_insertBefore(node, refNode.firstChild);
						break;
					}
					// else fallthrough...
				default: // aka: last
					refNode.appendChild(node);
			}
		}
		return node; // DomNode
	};

	exports.create = function create(/*DOMNode|String*/tag, /*Object*/attrs, /*DOMNode?|String?*/refNode, /*String?*/pos){
		var doc = win.doc;
		if(refNode){
			refNode = dom.byId(refNode);
			doc = refNode.ownerDocument;
		}
		if(typeof tag == "string"){ // inline'd type check
			tag = doc.createElement(tag);
		}
		if(attrs){ attr.set(tag, attrs); }
		if(refNode){ exports.place(tag, refNode, pos); }
		return tag; // DomNode
	};

	function _empty(/*DomNode*/ node){
		if(node.canHaveChildren){
			try{
				// fast path
				node.innerHTML = "";
				return;
			}catch(e){
				// innerHTML is readOnly (e.g. TABLE (sub)elements in quirks mode)
				// Fall through (saves bytes)
			}
		}
		// SVG/strict elements don't support innerHTML/canHaveChildren, and OBJECT/APPLET elements in quirks node have canHaveChildren=false
		for(var c; c = node.lastChild;){ // intentional assignment
			_destroy(c, node); // destroy is better than removeChild so TABLE subelements are removed in proper order
		}
	}

	exports.empty = function empty(/*DOMNode|String*/ node){
		_empty(dom.byId(node));
	};


	function _destroy(/*DomNode*/ node, /*DomNode*/ parent){
		if(node.firstChild){
			_empty(node);
		}
		if(parent){
						// removeNode(false) doesn't leak in IE 6+, but removeChild() and removeNode(true) are known to leak under IE 8- while 9+ is TBD
			has("ie") && parent.canHaveChildren && 'removeNode' in node ? node.removeNode(false) :
							parent.removeChild(node);
		}
	}
	exports.destroy = function destroy(/*DOMNode|String*/ node){
		node = dom.byId(node);
		if(!node){ return; }
		_destroy(node, node.parentNode);
	};
});

},
'dijit/BackgroundIframe':function(){
define([
	"require",			// require.toUrl
	".",	// to export dijit.BackgroundIframe
	"dojo/_base/config",
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.set
	"dojo/_base/lang", // lang.extend lang.hitch
	"dojo/on",
	"dojo/_base/sniff", // has("ie"), has("mozilla"), has("quirks")
	"dojo/_base/window" // win.doc.createElement
], function(require, dijit, config, domConstruct, domStyle, lang, on, has, win){

	// module:
	//		dijit/BackgroundIFrame

	// Flag for whether to create background iframe behind popups like Menus and Dialog.
	// A background iframe is useful to prevent problems with popups appearing behind applets/pdf files,
	// and is also useful on older versions of IE (IE6 and IE7) to prevent the "bleed through select" problem.
	// TODO: For 2.0, make this false by default.  Also, possibly move definition to has.js so that this module can be
	// conditionally required via  dojo/has!bgIfame?dijit/BackgroundIframe
	has.add("bgIframe", has("ie") || has("mozilla"));
	// summary:
	//		new dijit.BackgroundIframe(node)
	//		Makes a background iframe as a child of node, that fills
	//		area (and position) of node

	// TODO: remove _frames, it isn't being used much, since popups never release their
	// iframes (see [22236])
	var _frames = new function(){
		// summary:
		//		cache of iframes

		var queue = [];

		this.pop = function(){
			var iframe;
			if(queue.length){
				iframe = queue.pop();
				iframe.style.display="";
			}else{
				if(has("ie") < 9){
					var burl = config["dojoBlankHtmlUrl"] || require.toUrl("dojo/resources/blank.html") || "javascript:\"\"";
					var html="<iframe src='" + burl + "' role='presentation'"
						+ " style='position: absolute; left: 0px; top: 0px;"
						+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
					iframe = win.doc.createElement(html);
				}else{
					iframe = domConstruct.create("iframe");
					iframe.src = 'javascript:""';
					iframe.className = "dijitBackgroundIframe";
					iframe.setAttribute("role", "presentation");
					domStyle.set(iframe, "opacity", 0.1);
				}
				iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			}
			return iframe;
		};

		this.push = function(iframe){
			iframe.style.display="none";
			queue.push(iframe);
		}
	}();


	dijit.BackgroundIframe = function(/*DomNode*/ node){
		// summary:
		//		For IE/FF z-index schenanigans. id attribute is required.
		//
		// description:
		//		new dijit.BackgroundIframe(node)
		//			Makes a background iframe as a child of node, that fills
		//			area (and position) of node

		if(!node.id){ throw new Error("no id"); }
		if(has("bgIframe")){
			var iframe = (this.iframe = _frames.pop());
			node.appendChild(iframe);
			if(has("ie")<7 || has("quirks")){
				this.resize(node);
				this._conn = on(node, 'resize', lang.hitch(this, function(){
					this.resize(node);
				}));
			}else{
				domStyle.set(iframe, {
					width: '100%',
					height: '100%'
				});
			}
		}
	};

	lang.extend(dijit.BackgroundIframe, {
		resize: function(node){
			// summary:
			// 		Resize the iframe so it's the same size as node.
			//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
			if(this.iframe){
				domStyle.set(this.iframe, {
					width: node.offsetWidth + 'px',
					height: node.offsetHeight + 'px'
				});
			}
		},
		destroy: function(){
			// summary:
			//		destroy the iframe
			if(this._conn){
				this._conn.remove();
				this._conn = null;
			}
			if(this.iframe){
				_frames.push(this.iframe);
				delete this.iframe;
			}
		}
	});

	return dijit.BackgroundIframe;
});

},
'dijit/_WidgetsInTemplateMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/parser", // parser.parse
	"dijit/registry"	// registry.findWidgets
], function(array, declare, parser, registry){

	// module:
	//		dijit/_WidgetsInTemplateMixin
	// summary:
	//		Mixin to supplement _TemplatedMixin when template contains widgets

	return declare("dijit._WidgetsInTemplateMixin", null, {
		// summary:
		//		Mixin to supplement _TemplatedMixin when template contains widgets

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  (Remove for 2.0 and assume true)
		widgetsInTemplate: true,

		_beforeFillContent: function(){
			if(this.widgetsInTemplate){
				// Before copying over content, instantiate widgets in template
				var node = this.domNode;

				var cw = (this._startupWidgets = parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang, textDir: this.textDir},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}));

				this._supportingWidgets = registry.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}
		},

		startup: function(){
			array.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		}
	});
});

},
'dojo/data/ObjectStore':function(){
define(["../_base/lang", "../Evented", "../_base/declare", "../_base/Deferred", "../_base/array", 
	"../_base/connect", "../regexp"
], function(lang, Evented, declare, Deferred, array, connect, regexp) {
	// module:
	//		dojo/data/ObjectStore
	// summary:
	//		TODOC


return declare("dojo.data.ObjectStore", [Evented],{
		objectStore: null,
		constructor: function(options){
			// summary:
			//		A Dojo Data implementation that wraps Dojo object stores for backwards
			//		compatibility.
			//	options:
			//		The configuration information to pass into the data store.
			//	options.objectStore:
			//		The object store to use as the source provider for this data store
			lang.mixin(this, options);
		},
		labelProperty: "label",

		getValue: function(/*Object*/ item, /*String*/property, /*value?*/defaultValue){
			// summary:
			//	Gets the value of an item's 'property'
			//
			//	item:
			//		The item to get the value from
			//	property:
			//		property to look up value for
			//	defaultValue:
			//		the default value

			return typeof item.get === "function" ? item.get(property) :
				property in item ?
					item[property] : defaultValue;
		},
		getValues: function(item, property){
			// summary:
			//		Gets the value of an item's 'property' and returns
			//		it. If this value is an array it is just returned,
			//		if not, the value is added to an array and that is returned.
			//
			//	item: /* object */
			//	property: /* string */
			//		property to look up value for

			var val = this.getValue(item,property);
			return val instanceof Array ? val : val === undefined ? [] : [val];
		},

		getAttributes: function(item){
			// summary:
			//	Gets the available attributes of an item's 'property' and returns
			//	it as an array.
			//
			//	item: /* object */

			var res = [];
			for(var i in item){
				if(item.hasOwnProperty(i) && !(i.charAt(0) == '_' && i.charAt(1) == '_')){
					res.push(i);
				}
			}
			return res;
		},

		hasAttribute: function(item,attribute){
			// summary:
			//		Checks to see if item has attribute
			//
			//	item: /* object */
			//	attribute: /* string */
			return attribute in item;
		},

		containsValue: function(item, attribute, value){
			// summary:
			//		Checks to see if 'item' has 'value' at 'attribute'
			//
			//	item: /* object */
			//	attribute: /* string */
			//	value: /* anything */
			return array.indexOf(this.getValues(item,attribute),value) > -1;
		},


		isItem: function(item){
			// summary:
			//		Checks to see if the argument is an item
			//
			//	item: /* object */
			//	attribute: /* string */

			// we have no way of determining if it belongs, we just have object returned from
			//	service queries
			return (typeof item == 'object') && item && !(item instanceof Date);
		},

		isItemLoaded: function(item){
			// summary:
			//		Checks to see if the item is loaded.
			//
			//		item: /* object */

			return item && typeof item.load !== "function";
		},

		loadItem: function(args){
			// summary:
			//		Loads an item and calls the callback handler. Note, that this will call the callback
			//		handler even if the item is loaded. Consequently, you can use loadItem to ensure
			//		that an item is loaded is situations when the item may or may not be loaded yet.
			//		If you access a value directly through property access, you can use this to load
			//		a lazy value as well (doesn't need to be an item).
			//
			//	example:
			//		store.loadItem({
			//			item: item, // this item may or may not be loaded
			//			onItem: function(item){
			//				// do something with the item
			//			}
			//		});

			var item;
			if(typeof args.item.load === "function"){
				Deferred.when(args.item.load(), function(result){
					item = result; // in synchronous mode this can allow loadItem to return the value
					var func = result instanceof Error ? args.onError : args.onItem;
					if(func){
						func.call(args.scope, result);
					}
				});
			}else if(args.onItem){
				// even if it is already loaded, we will use call the callback, this makes it easier to
				// use when it is not known if the item is loaded (you can always safely call loadItem).
				args.onItem.call(args.scope, args.item);
			}
			return item;
		},
		close: function(request){
			return request && request.abort && request.abort();
		},
		fetch: function(args){
			// summary:
			//		See dojo.data.api.Read.fetch
			//

			args = lang.delegate(args, args && args.queryOptions);
			var self = this;
			var scope = args.scope || self;
			var query = args.query;
			if(typeof query == "object"){ // can be null, but that is ignore by for-in
				query = lang.delegate(query); // don't modify the original
				for(var i in query){
					// find any strings and convert them to regular expressions for wildcard support
					var required = query[i];
					if(typeof required == "string"){
						query[i] = RegExp("^" + regexp.escapeString(required, "*?").replace(/\*/g, '.*').replace(/\?/g, '.') + "$", args.ignoreCase ? "mi" : "m");
						query[i].toString = (function(original){
							return function(){
								return original;
							}
						})(required);
					}
				}
			}

			var results = this.objectStore.query(query, args);
			Deferred.when(results.total, function(totalCount){
				Deferred.when(results, function(results){
					if(args.onBegin){
						args.onBegin.call(scope, totalCount || results.length, args);
					}
					if(args.onItem){
						for(var i=0; i<results.length;i++){
							args.onItem.call(scope, results[i], args);
						}
					}
					if(args.onComplete){
						args.onComplete.call(scope, args.onItem ? null : results, args);
					}
					return results;
				}, errorHandler);
			}, errorHandler);
			function errorHandler(error){
				if(args.onError){
					args.onError.call(scope, error, args);
				}
			}
			args.abort = function(){
				// abort the request
				if(results.cancel){
					results.cancel();
				}
			};
			if(results.observe){
				if(this.observing){
					// if we were previously observing, cancel the last time to avoid multiple notifications. Just the best we can do for the impedance mismatch between APIs
					this.observing.cancel();
				}
				this.observing = results.observe(function(object, removedFrom, insertedInto){
					if(array.indexOf(self._dirtyObjects, object) == -1){
						if(removedFrom == -1){
							self.onNew(object);
						}
						else if(insertedInto == -1){
							self.onDelete(object);
						}
						else{
							for(var i in object){
								if(i != self.objectStore.idProperty){
									self.onSet(object, i, null, object[i]);
								}
							}
						}
					}
				}, true);
			}
			this.onFetch(results);
			args.store = this;
			return args;
		},
		getFeatures: function(){
			// summary:
			//		return the store feature set

			return {
				"dojo.data.api.Read": !!this.objectStore.get,
				"dojo.data.api.Identity": true,
				"dojo.data.api.Write": !!this.objectStore.put,
				"dojo.data.api.Notification": true
			};
		},

		getLabel: function(/* item */ item){
			//	summary:
			//		See dojo.data.api.Read.getLabel()
			if(this.isItem(item)){
				return this.getValue(item,this.labelProperty); //String
			}
			return undefined; //undefined
		},

		getLabelAttributes: function(/* item */ item){
			//	summary:
			//		See dojo.data.api.Read.getLabelAttributes()
			return [this.labelProperty]; //array
		},

		//Identity API Support


		getIdentity: function(item){
			return this.objectStore.getIdentity ? this.objectStore.getIdentity(item) : item[this.objectStore.idProperty || "id"];
		},

		getIdentityAttributes: function(item){
			// summary:
			//		returns the attributes which are used to make up the
			//		identity of an item.	Basically returns this.objectStore.idProperty

			return [this.objectStore.idProperty];
		},

		fetchItemByIdentity: function(args){
			// summary:
			//		fetch an item by its identity, by looking in our index of what we have loaded
			var item;
			Deferred.when(this.objectStore.get(args.identity),
				function(result){
					item = result;
					args.onItem.call(args.scope, result);
				},
				function(error){
					args.onError.call(args.scope, error);
				}
			);
			return item;
		},

		newItem: function(data, parentInfo){
			// summary:
			//		adds a new item to the store at the specified point.
			//		Takes two parameters, data, and options.
			//
			//	data: Object
			//		The data to be added in as an item.
			
			// TODOC: parentInfo
			if(parentInfo){
				// get the previous value or any empty array
				var values = this.getValue(parentInfo.parent,parentInfo.attribute,[]);
				// set the new value
				values = values.concat([data]);
				data.__parent = values;
				this.setValue(parentInfo.parent, parentInfo.attribute, values);
			}
			this._dirtyObjects.push({object:data, save: true});
			this.onNew(data);
			return data;
		},
		deleteItem: function(item){
			// summary:
			//		deletes item and any references to that item from the store.
			//
			//	item:
			//		item to delete
			//

			//	If the desire is to delete only one reference, unsetAttribute or
			//	setValue is the way to go.
			this.changing(item, true);

			this.onDelete(item);
		},
		setValue: function(item, attribute, value){
			// summary:
			//		sets 'attribute' on 'item' to 'value'

			var old = item[attribute];
			this.changing(item);
			item[attribute]=value;
			this.onSet(item,attribute,old,value);
		},
		setValues: function(item, attribute, values){
			// summary:
			//	sets 'attribute' on 'item' to 'value' value
			//	must be an array.

			if(!lang.isArray(values)){
				throw new Error("setValues expects to be passed an Array object as its value");
			}
			this.setValue(item,attribute,values);
		},

		unsetAttribute: function(item, attribute){
			// summary:
			//		unsets 'attribute' on 'item'

			this.changing(item);
			var old = item[attribute];
			delete item[attribute];
			this.onSet(item,attribute,old,undefined);
		},

		_dirtyObjects: [],

		changing: function(object,_deleting){
			// summary:
			//		adds an object to the list of dirty objects.  This object
			//		contains a reference to the object itself as well as a
			//		cloned and trimmed version of old object for use with
			//		revert.
			object.__isDirty = true;
			//if an object is already in the list of dirty objects, don't add it again
			//or it will overwrite the premodification data set.
			for(var i=0; i<this._dirtyObjects.length; i++){
				var dirty = this._dirtyObjects[i];
				if(object==dirty.object){
					if(_deleting){
						// we are deleting, no object is an indicator of deletiong
						dirty.object = false;
						if(!this._saveNotNeeded){
							dirty.save = true;
						}
					}
					return;
				}
			}
			var old = object instanceof Array ? [] : {};
			for(i in object){
				if(object.hasOwnProperty(i)){
					old[i] = object[i];
				}
			}
			this._dirtyObjects.push({object: !_deleting && object, old: old, save: !this._saveNotNeeded});
		},

		save: function(kwArgs){
			// summary:
			//		Saves the dirty data using object store provider. See dojo.data.api.Write for API.
			//
			//	kwArgs.global:
			//		This will cause the save to commit the dirty data for all
			//		ObjectStores as a single transaction.
			//
			//	kwArgs.revertOnError
			//		This will cause the changes to be reverted if there is an
			//		error on the save. By default a revert is executed unless
			//		a value of false is provide for this parameter.

			// TODOC: kwArgs pseudo
			kwArgs = kwArgs || {};
			var result, actions = [];
			var savingObjects = [];
			var self = this;
			var dirtyObjects = this._dirtyObjects;
			var left = dirtyObjects.length;// this is how many changes are remaining to be received from the server
			try{
				connect.connect(kwArgs,"onError",function(){
					if(kwArgs.revertOnError !== false){
						var postCommitDirtyObjects = dirtyObjects;
						dirtyObjects = savingObjects;
						self.revert(); // revert if there was an error
						self._dirtyObjects = postCommitDirtyObjects;
					}
					else{
						self._dirtyObjects = dirtyObjects.concat(savingObjects);
					}
				});
				if(this.objectStore.transaction){
					var transaction = this.objectStore.transaction();
				}
				for(var i = 0; i < dirtyObjects.length; i++){
					var dirty = dirtyObjects[i];
					var object = dirty.object;
					var old = dirty.old;
					delete object.__isDirty;
					if(object){
						result = this.objectStore.put(object, {overwrite: !!old});
					}
					else if(typeof old != "undefined"){
						result = this.objectStore.remove(this.getIdentity(old));
					}
					savingObjects.push(dirty);
					dirtyObjects.splice(i--,1);
					Deferred.when(result, function(value){
						if(!(--left)){
							if(kwArgs.onComplete){
								kwArgs.onComplete.call(kwArgs.scope, actions);
							}
						}
					},function(value){

						// on an error we want to revert, first we want to separate any changes that were made since the commit
						left = -1; // first make sure that success isn't called
						kwArgs.onError.call(kwArgs.scope, value);
					});

				}
				if(transaction){
					transaction.commit();
				}
			}catch(e){
				kwArgs.onError.call(kwArgs.scope, value);
			}
		},

		revert: function(kwArgs){
			// summary:
			//		returns any modified data to its original state prior to a save();
			//
			var dirtyObjects = this._dirtyObjects;
			for(var i = dirtyObjects.length; i > 0;){
				i--;
				var dirty = dirtyObjects[i];
				var object = dirty.object;
				var old = dirty.old;
				if(object && old){
					// changed
					for(var j in old){
						if(old.hasOwnProperty(j) && object[j] !== old[j]){
							this.onSet(object, j, object[j], old[j]);
							object[j] = old[j];
						}
					}
					for(j in object){
						if(!old.hasOwnProperty(j)){
							this.onSet(object, j, object[j]);
							delete object[j];
						}
					}
				}else if(!old){
					// was an addition, remove it
					this.onDelete(object);
				}else{
					// was a deletion, we will add it back
					this.onNew(old);
				}
				delete (object || old).__isDirty;
				dirtyObjects.splice(i, 1);
			}

		},
		isDirty: function(item){
			// summary:
			//		returns true if the item is marked as dirty or true if there are any dirty items
			if(!item){
				return !!this._dirtyObjects.length;
			}
			return item.__isDirty;
		},
		//Notifcation Support

		onSet: function(){},
		onNew: function(){},
		onDelete:	function(){},
		// an extra to get result sets
		onFetch: function(results){}

	}
);
});

},
'dojo/_base/browser':function(){
if(require.has){
	require.has.add("config-selectorEngine", "acme");
}
define([
	"../ready",
	"./kernel",
	"./connect", // until we decide if connect is going back into non-browser environments
	"./unload",
	"./window",
	"./event",
	"./html",
	"./NodeList",
	"../query",
	"./xhr",
	"./fx"], function(dojo) {
	// module:
	//		dojo/_base/browser
	// summary:
	//		This module causes the browser-only base modules to be loaded.
	return dojo;
});

},
'com.ibm.bpm.coach/boundaryEventHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// boundary event utilities

define(["./BPM_CONSTANTS", "./commonHelper", "./logger"], function(BPM_CONSTANTS, commonHelper, logger) {
				
	var boundaryEventHelper = new Object();

	boundaryEventHelper.callbackForBoundaryEvent = function(callbackFun, alwaysInvokeCallback, boundaryEventStatus) {
		var isExecuted = (boundaryEventStatus == BPM_CONSTANTS.BOUNDARY_EVENT_STATUS_EXECUTED);
		if(alwaysInvokeCallback || isExecuted) {
			var response = {status: boundaryEventStatus};
			boundaryEventHelper._invokeCallbackFun(callbackFun, response);
		}
		
		if(!!callbackFun && !alwaysInvokeCallback && !isExecuted) {
			// callback is not invoked although callback function exist. log it here
			logger.log("boundaryEventHelper", "callbackForBoundaryEvent", "supplied callback function is not called for boundary event with status [" + boundaryEventStatus + "]");			
		}
	}
		
	boundaryEventHelper._invokeCallbackFun = function(callbackFun, response) {
		if(!!callbackFun) {
			try {
				callbackFun(response);
			} catch(ex) {
		    	var message = "Coach framework caught exception when calling boundary event's callback function " + callbackFun;
		    	message +=  ", Exception: " + ex;
		    	commonHelper.consoleLog(message, commonHelper.ERROR);
			}
		}
	}
		
	return boundaryEventHelper;
});

},
'com.ibm.bpm.coach/BPM_CONSTANTS':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all public constants

define([], function() {
				
	// Contains all the constants that are available for coach view (stock controls, customer views etc)
	var BPM_CONSTANTS = {
		
		// visibility
		VISIBILITY_DEFAULT:		"DEFAULT",
		VISIBILITY_REQUIRED:	"REQUIRED",
		VISIBILITY_EDITABLE:	"EDITABLE",
		VISIBILITY_READONLY:	"READONLY",
		VISIBILITY_NONE:		"NONE",
		VISIBILITY_HIDDEN:		"HIDDEN",

		VISIBILITY_SCRIPT_EVENT_TYPE_INIT:		"initialize",
		VISIBILITY_SCRIPT_EVENT_TYPE_CHANGE:	"change",

		// validation event
		VALIDATION_EVENT_TYPE_ERROR:	"error",
		VALIDATION_EVENT_TYPE_CLEAR:	"clear",
		
		// boundary event status
		BOUNDARY_EVENT_STATUS_EXECUTED:		"executed",
		BOUNDARY_EVENT_STATUS_UNQUALIFIED:	"unqualified",
		BOUNDARY_EVENT_STATUS_CANCELLED:		"cancelled",
		BOUNDARY_EVENT_STATUS_ABANDONED:		"abandoned"
		
	};
	
	return BPM_CONSTANTS;
});
	
},
'com.ibm.bpm.coach/Shimmer':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Shimmer to cover the coach area when it's needed to support read-only mode

define(["dojo/_base/connect", "dojo/_base/event", "dojo/dom-class", "dojo/keys", "./logger"], function(connect, dojoEvent, domClass, keys, logger) {

	return function (id) {
		// constants
		var	CSS_SHIMMER = "CoachView_shimmer";
		var	CSS_SHIMMER_ASSISTANT = "CoachView_shimmerAssistant";
		
		var shimmerId = id;
		var shimmerAssistantId = id + "_assistant";
	
		this.shimmer = document.getElementById(shimmerId);
		if(!!this.shimmer){
			document.body.removeChild(this.shimmer);
		}
		this.shimmer = document.createElement('div');
		this.shimmer.id = shimmerId;
		this.shimmer.tabIndex = 1;					
		domClass.add(this.shimmer, CSS_SHIMMER);
		document.body.insertBefore(this.shimmer, document.body.firstChild);
		
		this.shimmerAssistant = document.getElementById(shimmerAssistantId);
		if(!!this.shimmerAssistant){
			document.body.removeChild(this.shimmerAssistant);
		}
		// The following elements can have focus and are tab stops by default: 
		// a, BODY, button, frame, iframe, img, input, isIndex, object, select, textArea.
		this.shimmerAssistant = document.createElement('button');
		this.shimmerAssistant.id = shimmerAssistantId;
		domClass.add(this.shimmerAssistant, CSS_SHIMMER_ASSISTANT);
		document.body.appendChild(this.shimmerAssistant);
   	      
   	    // fix 121150
   	    // Portal encounters some issues in 8500 if this.shimmer grabs focus once Coach turns to readonly mode. 
   	    // Coach team has to change the design to help Portal team out:
   	    // If shimmer is in an iFrame, shimmer expects the parent window (e.g. Portal) to guarantee the focus is 
   	    // not in any field covered by shimmer. parent window can achieve it by simply grab the focus in its hand.
   	    // If shimmer is not in an iFrame, shimmer grabs the focus by itself to guarantee that the focus is 
   	    // not in any field covered by it.
   	    if(!window.parent || window.parent == window) {
   	    	// not in iFrame
			this.shimmer.focus();
		}
  
		var localShimmerAssistant = this.shimmerAssistant;
		var shimmerKeyHandle = connect.connect(this.shimmer, "onkeypress", function(e) {
			if(e.charOrCode == keys.TAB && e.shiftKey != true) {
			    dojoEvent.stop(e);			
				localShimmerAssistant.focus();
			}
		});	      
		var localShimmer = this.shimmer;
		var shimmerAssistantKeyHandle = connect.connect(this.shimmerAssistant, "onkeypress", function(e) {
			if(e.charOrCode == keys.TAB && e.shiftKey == true) {
			    dojoEvent.stop(e);
				localShimmer.focus();
			}
		});
	    
	    this.dispose = function() {
	        connect.disconnect(shimmerKeyHandle);
	        connect.disconnect(shimmerAssistantKeyHandle);
            if(!!this.shimmer) {
			    document.body.removeChild(this.shimmer);
			    this.shimmer = null;
            }
            if(!!this.shimmerAssistant) {
			    document.body.removeChild(this.shimmerAssistant);
			    this.shimmerAssistant = null;
            }
			logger.log("Shimmer", "dipsose", "Shimmer is disposed", logger.INFO);	    
	    };
	    
		logger.log("Shimmer", "constructor", "Shimmer is on", logger.INFO);	    
	};
});




},
'dojo/_base/event':function(){
define(["./kernel", "../on", "../has", "../dom-geometry"], function(dojo, on, has, dom){
  //  module:
  //    dojo/_base/event
  //  summary:
  //    This module defines dojo DOM event API.
	if(on._fixEvent){
		var fixEvent = on._fixEvent;
		on._fixEvent = function(evt, se){
			// add some additional normalization for back-compat, this isn't in on.js because it is somewhat more expensive
			evt = fixEvent(evt, se);
			if(evt){
				dom.normalizeEvent(evt);
			}
			return evt;
		};		
	}
	dojo.fixEvent = function(/*Event*/ evt, /*DOMNode*/ sender){
		// summary:
		//		normalizes properties on the event object including event
		//		bubbling methods, keystroke normalization, and x/y positions
		// evt: Event
		//		native event object
		// sender: DOMNode
		//		node to treat as "currentTarget"
		if(on._fixEvent){
			return on._fixEvent(evt, sender);
		}
		return evt;	// Event
	};
	
	dojo.stopEvent = function(/*Event*/ evt){
		// summary:
		//		prevents propagation and clobbers the default action of the
		//		passed event
		// evt: Event
		//		The event object. If omitted, window.event is used on IE.
		if(has("dom-addeventlistener") || (evt && evt.preventDefault)){
			evt.preventDefault();
			evt.stopPropagation();
		}else{
			evt = evt || window.event;
			evt.cancelBubble = true;
			on._preventDefault.call(evt);
		}
	};

	return {
		fix: dojo.fixEvent,
		stop: dojo.stopEvent
	};
});

},
'dojo/Stateful':function(){
define(["./_base/declare", "./_base/lang", "./_base/array"], function(declare, lang, array) {
	// module:
	//		dojo/Stateful
	// summary:
	//		TODOC

return declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	// example:
	//	|	var obj = new dojo.Stateful();
	//	|	obj.watch("foo", function(){
	//	|		console.log("foo changed to " + this.get("foo"));
	//	|	});
	//	|	obj.set("foo","bar");
	postscript: function(mixin){
		if(mixin){
			lang.mixin(this, mixin);
		}
	},

	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		//	name:
		//		The property to get.
		//	returns:
		//		The property value on this Stateful instance.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		// 		this just retrieves the object's property.
		// 		For example:
		//	|	stateful = new dojo.Stateful({foo: 3});
		//	|	stateful.get("foo") // returns 3
		//	|	stateful.foo // returns 3

		return this[name]; //Any
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		//	returns:
		//		The function returns this dojo.Stateful instance.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		// 		the property. A programmatic setter may be defined in subclasses.
		// 		For example:
		//	|	stateful = new dojo.Stateful();
		//	|	stateful.watch(function(name, oldValue, value){
		//	|		// this will be called on the set below
		//	|	}
		//	|	stateful.set(foo, 5);
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myObj.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
		if(typeof name === "object"){
			for(var x in name){
				if(name.hasOwnProperty(x) && x !="_watchCallbacks"){
					this.set(x, name[x]);
				}
			}
			return this;
		}
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this; //dojo.Stateful
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		//	name:
		//		Indicates the property to watch. This is optional (the callback may be the
		// 		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		// 		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		//	callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		// 		second argument as the old value and the third argument as the new value.

		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
						propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							propertyCallbacks[i].call(self, name, oldValue, value);
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);
		return {
			unwatch: function(){
				propertyCallbacks.splice(array.indexOf(propertyCallbacks, callback), 1);
			}
		}; //Object
	}

});

});

},
'dojo/store/util/QueryResults':function(){
define(["../../_base/array", "../../_base/lang", "../../_base/Deferred"
], function(array, lang, Deferred) {
  //  module:
  //    dojo/store/util/QueryResults
  //  summary:
  //    The module defines a query results wrapper

var util = lang.getObject("dojo.store.util", true);

util.QueryResults = function(results){
	// summary:
	//		A function that wraps the results of a store query with additional
	//		methods.
	//
	// description:
	//		QueryResults is a basic wrapper that allows for array-like iteration
	//		over any kind of returned data from a query.  While the simplest store
	//		will return a plain array of data, other stores may return deferreds or
	//		promises; this wrapper makes sure that *all* results can be treated
	//		the same.
	//
	//		Additional methods include `forEach`, `filter` and `map`.
	//
	// returns: Object
	//		An array-like object that can be used for iterating over.
	//
	// example:
	//		Query a store and iterate over the results.
	//
	//	|	store.query({ prime: true }).forEach(function(item){
	//	|		//	do something
	//	|	});

	if(!results){
		return results;
	}
	// if it is a promise it may be frozen
	if(results.then){
		results = lang.delegate(results);
	}
	function addIterativeMethod(method){
		if(!results[method]){
			results[method] = function(){
				var args = arguments;
				return Deferred.when(results, function(results){
					Array.prototype.unshift.call(args, results);
					return util.QueryResults(array[method].apply(array, args));
				});
			};
		}
	}
	addIterativeMethod("forEach");
	addIterativeMethod("filter");
	addIterativeMethod("map");
	if(!results.total){
		results.total = Deferred.when(results, function(results){
			return results.length;
		});
	}
	return results;
};

return util.QueryResults;
});

},
'url:dijit/templates/TooltipDialog.html':"<div role=\"presentation\" tabIndex=\"-1\">\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" data-dojo-attach-point=\"containerNode\" role=\"dialog\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\"></div>\n</div>\n",
'dojo/touch':function(){
define(["./_base/kernel", "./on", "./has", "./mouse"], function(dojo, on, has, mouse){
// module:
//		dojo/touch

/*=====
	dojo.touch = {
		// summary:
		//		This module provides unified touch event handlers by exporting
		//		press, move, release and cancel which can also run well on desktop.
		//		Based on http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html
		//
		// example:
		//		1. Used with dojo.connect()
		//		|	dojo.connect(node, dojo.touch.press, function(e){});
		//		|	dojo.connect(node, dojo.touch.move, function(e){});
		//		|	dojo.connect(node, dojo.touch.release, function(e){});
		//		|	dojo.connect(node, dojo.touch.cancel, function(e){});
		//
		//		2. Used with dojo.on
		//		|	define(["dojo/on", "dojo/touch"], function(on, touch){
		//		|		on(node, touch.press, function(e){});
		//		|		on(node, touch.move, function(e){});
		//		|		on(node, touch.release, function(e){});
		//		|		on(node, touch.cancel, function(e){});
		//
		//		3. Used with dojo.touch.* directly
		//		|	dojo.touch.press(node, function(e){});
		//		|	dojo.touch.move(node, function(e){});
		//		|	dojo.touch.release(node, function(e){});
		//		|	dojo.touch.cancel(node, function(e){});
		
		press: function(node, listener){
			// summary:
			//		Register a listener to 'touchstart'|'mousedown' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		move: function(node, listener){
			// summary:
			//		Register a listener to 'touchmove'|'mousemove' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		release: function(node, listener){
			// summary:
			//		Register a listener to 'touchend'|'mouseup' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		cancel: function(node, listener){
			// summary:
			//		Register a listener to 'touchcancel'|'mouseleave' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		}
	};
=====*/

	function _handle(/*String - press | move | release | cancel*/type){
		return function(node, listener){//called by on(), see dojo.on
			return on(node, type, listener);
		};
	}
	var touch = has("touch");
	//device neutral events - dojo.touch.press|move|release|cancel
	dojo.touch = {
		press: _handle(touch ? "touchstart": "mousedown"),
		move: _handle(touch ? "touchmove": "mousemove"),
		release: _handle(touch ? "touchend": "mouseup"),
		cancel: touch ? _handle("touchcancel") : mouse.leave
	};
	return dojo.touch;
});
},
'dojo/_base/url':function(){
define(["./kernel"], function(dojo) {
	// module:
	//		dojo/url
	// summary:
	//		This module contains dojo._Url

	var
		ore = new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$"),
		ire = new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$"),
		_Url = function(){
			var n = null,
				_a = arguments,
				uri = [_a[0]];
			// resolve uri components relative to each other
			for(var i = 1; i<_a.length; i++){
				if(!_a[i]){ continue; }

				// Safari doesn't support this.constructor so we have to be explicit
				// FIXME: Tracked (and fixed) in Webkit bug 3537.
				//		http://bugs.webkit.org/show_bug.cgi?id=3537
				var relobj = new _Url(_a[i]+""),
					uriobj = new _Url(uri[0]+"");

				if(
					relobj.path == "" &&
					!relobj.scheme &&
					!relobj.authority &&
					!relobj.query
				){
					if(relobj.fragment != n){
						uriobj.fragment = relobj.fragment;
					}
					relobj = uriobj;
				}else if(!relobj.scheme){
					relobj.scheme = uriobj.scheme;

					if(!relobj.authority){
						relobj.authority = uriobj.authority;

						if(relobj.path.charAt(0) != "/"){
							var path = uriobj.path.substring(0,
								uriobj.path.lastIndexOf("/") + 1) + relobj.path;

							var segs = path.split("/");
							for(var j = 0; j < segs.length; j++){
								if(segs[j] == "."){
									// flatten "./" references
									if(j == segs.length - 1){
										segs[j] = "";
									}else{
										segs.splice(j, 1);
										j--;
									}
								}else if(j > 0 && !(j == 1 && segs[0] == "") &&
									segs[j] == ".." && segs[j-1] != ".."){
									// flatten "../" references
									if(j == (segs.length - 1)){
										segs.splice(j, 1);
										segs[j - 1] = "";
									}else{
										segs.splice(j - 1, 2);
										j -= 2;
									}
								}
							}
							relobj.path = segs.join("/");
						}
					}
				}

				uri = [];
				if(relobj.scheme){
					uri.push(relobj.scheme, ":");
				}
				if(relobj.authority){
					uri.push("//", relobj.authority);
				}
				uri.push(relobj.path);
				if(relobj.query){
					uri.push("?", relobj.query);
				}
				if(relobj.fragment){
					uri.push("#", relobj.fragment);
				}
			}

			this.uri = uri.join("");

			// break the uri into its main components
			var r = this.uri.match(ore);

			this.scheme = r[2] || (r[1] ? "" : n);
			this.authority = r[4] || (r[3] ? "" : n);
			this.path = r[5]; // can never be undefined
			this.query = r[7] || (r[6] ? "" : n);
			this.fragment	 = r[9] || (r[8] ? "" : n);

			if(this.authority != n){
				// server based naming authority
				r = this.authority.match(ire);

				this.user = r[3] || n;
				this.password = r[4] || n;
				this.host = r[6] || r[7]; // ipv6 || ipv4
				this.port = r[9] || n;
			}
		};
	_Url.prototype.toString = function(){ return this.uri; };

	return dojo._Url = _Url;
});

},
'dojo/string':function(){
define(["./_base/kernel", "./_base/lang"], function(dojo, lang) {
	// module:
	//		dojo/string
	// summary:
	//		TODOC

lang.getObject("string", true, dojo);

/*=====
dojo.string = {
	// summary: String utilities for Dojo
};
=====*/

dojo.string.rep = function(/*String*/str, /*Integer*/num){
	// summary:
	//		Efficiently replicate a string `n` times.
	// str:
	//		the string to replicate
	// num:
	//		number of times to replicate the string

	if(num <= 0 || !str){ return ""; }

	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

dojo.string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	// text:
	//		the string to pad
	// size:
	//		length to provide padding
	// ch:
	//		character to pad, defaults to '0'
	// end:
	//		adds padding at the end if true, otherwise pads at start
	// example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	dojo.string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = dojo.string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

dojo.string.substitute = function(	/*String*/		template,
									/*Object|Array*/map,
									/*Function?*/	transform,
									/*Object?*/		thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// template:
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive.
	// map:
	//		hash to search for substitutions
	// transform:
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	// thisObject:
	//		where to look for optional format function; default to the global
	//		namespace
	// example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	dojo.string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	dojo.string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	// example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	dojo.string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	// example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	dojo.string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || dojo.global;
	transform = transform ?
		lang.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = lang.getObject(key, false, map);
			if(format){
				value = lang.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

/*=====
dojo.string.trim = function(str){
	// summary:
	//		Trims whitespace from both sides of the string
	// str: String
	//		String to be trimmed
	// returns: String
	//		Returns the trimmed string
	// description:
	//		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	//		The short yet performant version of this function is dojo.trim(),
	//		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	return "";	// String
}
=====*/

dojo.string.trim = String.prototype.trim ?
	lang.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

return dojo.string;
});

},
'dojo/domReady':function(){
define(['./has'], function(has){
	var global = this,
		doc = document,
		readyStates = { 'loaded': 1, 'complete': 1 },
		fixReadyState = typeof doc.readyState != "string",
		ready = !!readyStates[doc.readyState];

	// For FF <= 3.5
	if(fixReadyState){ doc.readyState = "loading"; }

	if(!ready){
		var readyQ = [], tests = [],
			detectReady = function(evt){
				evt = evt || global.event;
				if(ready || (evt.type == "readystatechange" && !readyStates[doc.readyState])){ return; }
				ready = 1;

				// For FF <= 3.5
				if(fixReadyState){ doc.readyState = "complete"; }

				while(readyQ.length){
					(readyQ.shift())();
				}
			},
			on = function(node, event){
				node.addEventListener(event, detectReady, false);
				readyQ.push(function(){ node.removeEventListener(event, detectReady, false); });
			};

		if(!has("dom-addeventlistener")){
			on = function(node, event){
				event = "on" + event;
				node.attachEvent(event, detectReady);
				readyQ.push(function(){ node.detachEvent(event, detectReady); });
			};

			var div = doc.createElement("div");
			try{
				if(div.doScroll && global.frameElement === null){
					// the doScroll test is only useful if we're in the top-most frame
					tests.push(function(){
						// Derived with permission from Diego Perini's IEContentLoaded
						// http://javascript.nwbox.com/IEContentLoaded/
						try{
							div.doScroll("left");
							return 1;
						}catch(e){}
					});
				}
			}catch(e){}
		}

		on(doc, "DOMContentLoaded");
		on(global, "load");

		if("onreadystatechange" in doc){
			on(doc, "readystatechange");
		}else if(!fixReadyState){
			// if the ready state property exists and there's
			// no readystatechange event, poll for the state
			// to change
			tests.push(function(){
				return readyStates[doc.readyState];
			});
		}

		if(tests.length){
			var poller = function(){
				if(ready){ return; }
				var i = tests.length;
				while(i--){
					if(tests[i]()){
						detectReady("poller");
						return;
					}
				}
				setTimeout(poller, 30);
			};
			poller();
		}
	}

	function domReady(callback){
		if(ready){
			callback(1);
		}else{
			readyQ.push(callback);
		}
	}
	domReady.load = function(id, req, load){
		domReady(load);
	};

	return domReady;
});

},
'dojo/dom-prop':function(){
define(["exports", "./_base/kernel", "./_base/sniff", "./_base/lang", "./dom", "./dom-style", "./dom-construct", "./_base/connect"],
		function(exports, dojo, has, lang, dom, style, ctr, conn){
	// module:
	//		dojo/dom-prop
	// summary:
	//		This module defines the core dojo DOM properties API.
	//      Indirectly depends on dojo.empty() and dojo.toDom().

	// =============================
	// Element properties Functions
	// =============================

	/*=====
	prop.get = function(node, name){
		// summary:
		//		Gets a property on an HTML element.
		// description:
		//		Handles normalized getting of properties on DOM nodes.
		//
		// node: DOMNode|String
		//		id or reference to the element to get the property on
		// name: String
		//		the name of the property to get.
		// returns:
		//		the value of the requested property or its default value
		//
		// example:
		//	|	// get the current value of the "foo" property on a node
		//	|	dojo.getProp(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.getProp("nodeId", "foo");
	};
	=====*/

	/*=====
	prop.set = function(node, name, value){
		// summary:
		//		Sets a property on an HTML element.
		// description:
		//		Handles normalized setting of properties on DOM nodes.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to set the property on
		// name: String|Object
		//		the name of the property to set, or a hash object to set
		//		multiple properties at once.
		// value: String?
		//		The value to set for the property
		// returns:
		//		the DOM node
		//
		// example:
		//	|	// use prop() to set the tab index
		//	|	dojo.setProp("nodeId", "tabIndex", 3);
		//	|
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.setProp("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.setProp("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.setProp("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);
	};
	=====*/

	// helper to connect events
	var _evtHdlrMap = {}, _ctr = 0, _attrId = dojo._scopeName + "attrid";

		// the next dictionary lists elements with read-only innerHTML on IE
	var _roInnerHtml = {col: 1, colgroup: 1,
			// frameset: 1, head: 1, html: 1, style: 1,
			table: 1, tbody: 1, tfoot: 1, thead: 1, tr: 1, title: 1};
	
	exports.names = {
		// properties renamed to avoid clashes with reserved words
		"class": "className",
		"for": "htmlFor",
		// properties written as camelCase
		tabindex: "tabIndex",
		readonly: "readOnly",
		colspan: "colSpan",
		frameborder: "frameBorder",
		rowspan: "rowSpan",
		valuetype: "valueType"
	};

	exports.get = function getProp(/*DOMNode|String*/node, /*String*/name){
		node = dom.byId(node);
		var lc = name.toLowerCase(), propName = exports.names[lc] || name;
		return node[propName];	// Anything
	};

	exports.set = function setProp(/*DOMNode|String*/node, /*String|Object*/name, /*String?*/value){
		node = dom.byId(node);
		var l = arguments.length;
		if(l == 2 && typeof name != "string"){ // inline'd type check
			// the object form of setter: the 2nd argument is a dictionary
			for(var x in name){
				exports.set(node, x, name[x]);
			}
			return node; // DomNode
		}
		var lc = name.toLowerCase(), propName = exports.names[lc] || name;
		if(propName == "style" && typeof value != "string"){ // inline'd type check
			// special case: setting a style
			style.style(node, value);
			return node; // DomNode
		}
		if(propName == "innerHTML"){
			// special case: assigning HTML
						if(has("ie") && node.tagName.toLowerCase() in _roInnerHtml){
				ctr.empty(node);
				node.appendChild(ctr.toDom(value, node.ownerDocument));
			}else{
							node[propName] = value;
						}
						return node; // DomNode
		}
		if(lang.isFunction(value)){
			// special case: assigning an event handler
			// clobber if we can
			var attrId = node[_attrId];
			if(!attrId){
				attrId = _ctr++;
				node[_attrId] = attrId;
			}
			if(!_evtHdlrMap[attrId]){
				_evtHdlrMap[attrId] = {};
			}
			var h = _evtHdlrMap[attrId][propName];
			if(h){
				//h.remove();
				conn.disconnect(h);
			}else{
				try{
					delete node[propName];
				}catch(e){}
			}
			// ensure that event objects are normalized, etc.
			if(value){
				//_evtHdlrMap[attrId][propName] = on(node, propName, value);
				_evtHdlrMap[attrId][propName] = conn.connect(node, propName, value);
			}else{
				node[propName] = null;
			}
			return node; // DomNode
		}
		node[propName] = value;
		return node;	// DomNode
	};
});

},
'com.ibm.bpm.coach/Coach':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

define(["dojo/_base/array", "./BPM_CONSTANTS", "./CONSTANTS", "./commonHelper", "./coachViewHelper", "./dynamicBindingHelper", "./engine"], function(dojoArray, BPM_CONSTANTS, CONSTANTS, commonHelper, coachViewHelper, dynamicBindingHelper, engine) {

	return function () {

		this._bindingHandlers = [];
		
		this.load = function() {
			commonHelper.timestamp("Enter Coach.load()");

			/* sample content of this._visibilityConfigs produced by code-gen
				[
					{
						viewInstancePath: ["Text2"],
						variablesToWatch: ["local.var1", "local.var3"],
						getVisibility: function(context, event, local) {
							return bpmConstants.VISIBILITY_DEFAULT;
						}
					}
				];	
			*/	

			if(!this._visibilityConfigs) {
				commonHelper.timestamp("Exit Coach.load(), _visibilityConfigs doesn't exist");
				return;
			}
			
			for(var i=0; i<this._visibilityConfigs.length; i++) {
				var aConfig = this._visibilityConfigs[i];
				// aConfig works as the scope object for bind(), when callback is invoked, need to access coach object
				aConfig._coach = this;
				// cache string version of viewInstancePath for later usage (mainly for comparison)
				aConfig.viewInstancePathStr = this._viewPathToString(aConfig.viewInstancePath);
				
				aConfig.variablesToWatch = this._preprocessWatchedVariables(aConfig.variablesToWatch);
				
				for(var j=0; j<aConfig.variablesToWatch.length; j++) {
					var callbackToManageVisibility = function(changeEvent) {
							// don't modify changeEvent which is supposed to be readonly
							// changeEvent coming from dynamicBindingHelper contains some internal information that shouldn't be exposed.
							var event = {
								type: BPM_CONSTANTS.VISIBILITY_SCRIPT_EVENT_TYPE_CHANGE,
								property: changeEvent.property,
								newVal: changeEvent.newVal,
								oldVal: changeEvent.oldVal,
								path:	changeEvent.path
							};
							if(typeof changeEvent._origin != "undefined") {
								event._origin = changeEvent._origin;
							};

							// "this" is visibilityConfig object
							this._coach._updateViewsVisibility(this, event);
						};					

					var bindingHandler = dynamicBindingHelper.bind(aConfig.variablesToWatch[j], callbackToManageVisibility, aConfig);
					this._bindingHandlers.push(bindingHandler);
				}
			}
			commonHelper.timestamp("Exit Coach.load()");
		}
		
		/*
		 	- if the watched variable is "local.abc[]", also watch "local.abc[index]"
		 	- if the watched variable is "local.abc[index]", also watch "local.abc[]"
		 	- filter out the duplicated variables in the array 
		*/
		this._preprocessWatchedVariables = function(watachedVariableArray) {
			var refinedArray = [];
			var oneVariable;
			for(var k=0; k<watachedVariableArray.length; k++) {
				oneVariable = watachedVariableArray[k];
				var pathArray = oneVariable.split(".");
				var lastPath = pathArray[pathArray.length - 1];
				var extraVariable = null;
				if(lastPath.indexOf(CONSTANTS.INDEX_WITH_BRACKET) != -1) {
					pathArray[pathArray.length - 1] = lastPath.replace(CONSTANTS.INDEX_WITH_BRACKET, "[]");
					extraVariable = pathArray.join(".");					
				} else if(lastPath.indexOf("[]") != -1) {
					pathArray[pathArray.length - 1] = lastPath.replace("[]", CONSTANTS.INDEX_WITH_BRACKET);
					extraVariable = pathArray.join(".");					
				}
								
				if(dojoArray.indexOf(refinedArray, oneVariable) == -1) {
					// new variable
					refinedArray.push(oneVariable);
				}
				if(!!extraVariable && dojoArray.indexOf(refinedArray, extraVariable) == -1) {
					// new variable
					refinedArray.push(extraVariable);
				}
			}
			return refinedArray;
		}
		
		this.unload = function() {
			// unregister what we have binded
			try {
				dynamicBindingHelper.unbindMultiple(this._bindingHandlers);
			} catch(ex) {
		    	commonHelper.consoleLog("Coach.unload(), failed to unbind the binding handler, Exception: " + ex, commonHelper.ERROR);
			}
		}

		this.view = function() {
			commonHelper.timestamp("Enter Coach.view()");
			if(!!this._visibilityConfigs) {
				for(var i=0; i<this._visibilityConfigs.length; i++) {
					var aConfig = this._visibilityConfigs[i];
					var event = {type: BPM_CONSTANTS.VISIBILITY_SCRIPT_EVENT_TYPE_INIT};
					this._updateViewsVisibility(aConfig, event);
				}			
			}
			commonHelper.timestamp("Exit Coach.view()");
		}
		
		this.updateVisibilityIfApplicable = function(coachView) {
			if(!this._visibilityConfigs) {
				// no coach level visibility configure, need to do nothing
				return;
			}
			
			// coachView._isVisibilityInitialized should be only set and get in this function
			// here is the reason to introduce coachView._isVisibilityInitialized: view managed ones and framework managed ones may mix in the view hierarchy.
			// as the result, this.updateVisibilityIfApplicable() might be called in one coachView more than once  
			if(coachView._isVisibilityInitialized) {
				// coachView has already been handled
				return;
			}
			
			// handle subviews firstly
			if (!!coachView.context.subview) {
				for (var viewId in coachView.context.subview) {
					var subviews = coachView.context.subview[viewId];
					for(var divId in subviews) {
						this.updateVisibilityIfApplicable( subviews[divId]);
					}
				}		
			}
			
			// now handle coachView itself
			var viewInstancePath = coachViewHelper.getViewPath(coachView);
			var viewInstancePathStr = this._viewPathToString(viewInstancePath);
			for(var i=0; i<this._visibilityConfigs.length; i++) {
				if(this._visibilityConfigs[i].viewInstancePathStr == viewInstancePathStr) {
					var event = {type: BPM_CONSTANTS.VISIBILITY_SCRIPT_EVENT_TYPE_INIT};
					var newVisibility = this._computeVisibility(this._visibilityConfigs[i], event);
					if(!!newVisibility) {
						this._setVisibility(coachView, newVisibility);
					}
					return;
				}
			}			

			// coachView._isVisibilityInitialized should be only set and get in this function
			coachView._isVisibilityInitialized = true;			
		}
		
		this._viewPathToString = function(viewPath) {
			return viewPath.join(".");
		}
		
		this._computeVisibility = function(aVisibilityConfig, event) {
			var getterFunc = aVisibilityConfig.getVisibility;
			// context
			var context = {bpm: {system: engine.GlobalAssets.system, team: engine.GlobalAssets.team}};	
			var parameters = [context, event, engine.GlobalAssets.local];

			try {		
				var startTime = commonHelper.isDebugMode() ? (new Date()).getTime() : null;
				var newVisibility = getterFunc.apply(null, parameters);
				if(startTime) {
					var elapsedTime = (new Date()).getTime() - startTime;
					if(elapsedTime >= CONSTANTS.ELAPSED_TIME_THRESHOLD_VISIBILITY_SCRIPT) {
						var info = this._getScriptDisplayName(aVisibilityConfig.viewInstancePath) + " used " + elapsedTime + "ms";
						commonHelper.timestamp(info);
					}
				}
				return newVisibility;
		    } catch(ex) {
		    	try {
			    	var message = "Coach framework caught exception when calling " + this._getScriptDisplayName(aVisibilityConfig.viewInstancePath);
			    	message +=  ", Exception: " + ex;
			    	commonHelper.consoleLog(message, commonHelper.ERROR);
		    	} catch(ex1) {
			    	commonHelper.consoleLog("Even failed to gracefully log this Exception: " + ex, commonHelper.ERROR);
		    	}
		    	
		    	// failed to get the visibility, make no sense to continue
		    	return null;
		    }
		}
		
		this._setVisibility = function(coachView, newVisibility) {
			if(coachView.context.options._metadata.visibility.get("value") != newVisibility) {
				coachView.context.options._metadata.visibility.set("value", newVisibility);
			}
		}
		
		this._isValidVisibility = function(visibility) {
			return (visibility && 
				(visibility == BPM_CONSTANTS.VISIBILITY_DEFAULT ||
				 visibility == BPM_CONSTANTS.VISIBILITY_REQUIRED ||
				 visibility == BPM_CONSTANTS.VISIBILITY_EDITABLE ||
				 visibility == BPM_CONSTANTS.VISIBILITY_READONLY ||
				 visibility == BPM_CONSTANTS.VISIBILITY_NONE ||
				 visibility == BPM_CONSTANTS.VISIBILITY_HIDDEN));
		}
		
		this._updateViewsVisibility = function(aVisibilityConfig, event) {
			var newVisibility = this._computeVisibility(aVisibilityConfig, event);
			if(!this._isValidVisibility(newVisibility)) {
				// not valid visibility, log and ignore it				
		    	var message = "Coach framework ignores an invalid visibility [" + newVisibility + "] after calling " + this._getScriptDisplayName(aVisibilityConfig.viewInstancePath);
		    	commonHelper.consoleLog(message, commonHelper.WARN);
				return;
			}
			
			// locates the viewInstances based on viewInstancePath
			var viewInstances = coachViewHelper.findAllViewInstances(engine.GlobalAssets.topLevelViewScope, aVisibilityConfig.viewInstancePath);		
			for(var	k=0; k<viewInstances.length; k++) {
				this._setVisibility(viewInstances[k], newVisibility);
			}
		}

		this._getScriptDisplayName = function(viewInstancePath) {
			return "Visibility script for view(s) represented by path [" + viewInstancePath.join() + "]";
		}
		
	};
});
},
'dojo/keys':function(){
define(["./_base/kernel", "./_base/sniff"], function(dojo, has) {
	// module:
	//		dojo/keys
	// summary:
	//		key constants
// Constants

// Public: client code should test
// keyCode against these named constants, as the
// actual codes can vary by browser.
return dojo.keys = {
	// summary:
	//		Definitions for common key values
	BACKSPACE: 8,
	TAB: 9,
	CLEAR: 12,
	ENTER: 13,
	SHIFT: 16,
	CTRL: 17,
	ALT: 18,
	META: has("safari") ? 91 : 224,		// the apple key on macs
	PAUSE: 19,
	CAPS_LOCK: 20,
	ESCAPE: 27,
	SPACE: 32,
	PAGE_UP: 33,
	PAGE_DOWN: 34,
	END: 35,
	HOME: 36,
	LEFT_ARROW: 37,
	UP_ARROW: 38,
	RIGHT_ARROW: 39,
	DOWN_ARROW: 40,
	INSERT: 45,
	DELETE: 46,
	HELP: 47,
	LEFT_WINDOW: 91,
	RIGHT_WINDOW: 92,
	SELECT: 93,
	NUMPAD_0: 96,
	NUMPAD_1: 97,
	NUMPAD_2: 98,
	NUMPAD_3: 99,
	NUMPAD_4: 100,
	NUMPAD_5: 101,
	NUMPAD_6: 102,
	NUMPAD_7: 103,
	NUMPAD_8: 104,
	NUMPAD_9: 105,
	NUMPAD_MULTIPLY: 106,
	NUMPAD_PLUS: 107,
	NUMPAD_ENTER: 108,
	NUMPAD_MINUS: 109,
	NUMPAD_PERIOD: 110,
	NUMPAD_DIVIDE: 111,
	F1: 112,
	F2: 113,
	F3: 114,
	F4: 115,
	F5: 116,
	F6: 117,
	F7: 118,
	F8: 119,
	F9: 120,
	F10: 121,
	F11: 122,
	F12: 123,
	F13: 124,
	F14: 125,
	F15: 126,
	NUM_LOCK: 144,
	SCROLL_LOCK: 145,
	UP_DPAD: 175,
	DOWN_DPAD: 176,
	LEFT_DPAD: 177,
	RIGHT_DPAD: 178,
	// virtual key mapping
	copyKey: has("mac") && !has("air") ? (has("safari") ? 91 : 224 ) : 17
};
});

},
'dojo/_base/lang':function(){
define(["./kernel", "../has", "./sniff"], function(dojo, has){
	//	module:
	//		dojo/_base/lang
	//	summary:
	//		This module defines Javascript language extensions.

	has.add("bug-for-in-skips-shadowed", function(){
		// if true, the for-in interator skips object properties that exist in Object's prototype (IE 6 - ?)
		for(var i in {toString: 1}){
			return 0;
		}
		return 1;
	});

	var _extraNames =
			has("bug-for-in-skips-shadowed") ?
				"hasOwnProperty.valueOf.isPrototypeOf.propertyIsEnumerable.toLocaleString.toString.constructor".split(".") : [],

		_extraLen = _extraNames.length,

		_mixin = function(dest, source, copyFunc){
			var name, s, i, empty = {};
			for(name in source){
				// the (!(name in empty) || empty[name] !== s) condition avoids copying properties in "source"
				// inherited from Object.prototype.	 For example, if dest has a custom toString() method,
				// don't overwrite it with the toString() method that source inherited from Object.prototype
				s = source[name];
				if(!(name in dest) || (dest[name] !== s && (!(name in empty) || empty[name] !== s))){
					dest[name] = copyFunc ? copyFunc(s) : s;
				}
			}

			if(has("bug-for-in-skips-shadowed")){
				if(source){
					for(i = 0; i < _extraLen; ++i){
						name = _extraNames[i];
						s = source[name];
						if(!(name in dest) || (dest[name] !== s && (!(name in empty) || empty[name] !== s))){
							dest[name] = copyFunc ? copyFunc(s) : s;
						}
					}
				}
			}

			return dest; // Object
		},

		mixin = function(dest, sources){
			if(!dest){ dest = {}; }
			for(var i = 1, l = arguments.length; i < l; i++){
				lang._mixin(dest, arguments[i]);
			}
			return dest; // Object
		},

		getProp = function(/*Array*/parts, /*Boolean*/create, /*Object*/context){
			var p, i = 0, dojoGlobal = dojo.global;
			if(!context){
				if(!parts.length){
					return dojoGlobal;
				}else{
					p = parts[i++];
					try{
						context = dojo.scopeMap[p] && dojo.scopeMap[p][1];
					}catch(e){}
					context = context || (p in dojoGlobal ? dojoGlobal[p] : (create ? dojoGlobal[p] = {} : undefined));
				}
			}
			while(context && (p = parts[i++])){
				context = (p in context ? context[p] : (create ? context[p] = {} : undefined));
			}
			return context; // mixed
		},

		setObject = function(name, value, context){
			var parts = name.split("."), p = parts.pop(), obj = getProp(parts, true, context);
			return obj && p ? (obj[p] = value) : undefined; // Object
		},

		getObject = function(name, create, context){
			return getProp(name.split("."), create, context); // Object
		},

		exists = function(name, obj){
			return lang.getObject(name, false, obj) !== undefined; // Boolean
		},

		opts = Object.prototype.toString,

		// Crockford (ish) functions

		isString = function(it){
			return (typeof it == "string" || it instanceof String); // Boolean
		},

		isArray = function(it){
			return it && (it instanceof Array || typeof it == "array"); // Boolean
		},

		isFunction = function(it){
			return opts.call(it) === "[object Function]";
		},

		isObject = function(it){
			return it !== undefined &&
				(it === null || typeof it == "object" || lang.isArray(it) || lang.isFunction(it)); // Boolean
		},

		isArrayLike = function(it){
			return it && it !== undefined && // Boolean
				// keep out built-in constructors (Number, String, ...) which have length
				// properties
				!lang.isString(it) && !lang.isFunction(it) &&
				!(it.tagName && it.tagName.toLowerCase() == 'form') &&
				(lang.isArray(it) || isFinite(it.length));
		},

		isAlien = function(it){
			return it && !lang.isFunction(it) && /\{\s*\[native code\]\s*\}/.test(String(it)); // Boolean
		},

		extend = function(constructor, props){
			for(var i=1, l=arguments.length; i<l; i++){
				lang._mixin(constructor.prototype, arguments[i]);
			}
			return constructor; // Object
		},

		_hitchArgs = function(scope, method){
			var pre = _toArray(arguments, 2);
			var named = lang.isString(method);
			return function(){
				// arrayify arguments
				var args = _toArray(arguments);
				// locate our method
				var f = named ? (scope||dojo.global)[method] : method;
				// invoke with collected args
				return f && f.apply(scope || this, pre.concat(args)); // mixed
			}; // Function
		},

		hitch = function(scope, method){
			if(arguments.length > 2){
				return lang._hitchArgs.apply(dojo, arguments); // Function
			}
			if(!method){
				method = scope;
				scope = null;
			}
			if(lang.isString(method)){
				scope = scope || dojo.global;
				if(!scope[method]){ throw(['dojo.hitch: scope["', method, '"] is null (scope="', scope, '")'].join('')); }
				return function(){ return scope[method].apply(scope, arguments || []); }; // Function
			}
			return !scope ? method : function(){ return method.apply(scope, arguments || []); }; // Function
		},

		delegate = (function(){
			// boodman/crockford delegation w/ cornford optimization
			function TMP(){}
			return function(obj, props){
				TMP.prototype = obj;
				var tmp = new TMP();
				TMP.prototype = null;
				if(props){
					lang._mixin(tmp, props);
				}
				return tmp; // Object
			};
		})(),

		efficient = function(obj, offset, startWith){
			return (startWith||[]).concat(Array.prototype.slice.call(obj, offset||0));
		},

		_toArray =
			has("ie") ?
				(function(){
					function slow(obj, offset, startWith){
						var arr = startWith||[];
						for(var x = offset || 0; x < obj.length; x++){
							arr.push(obj[x]);
						}
						return arr;
					}
					return function(obj){
						return ((obj.item) ? slow : efficient).apply(this, arguments);
					};
				})() : efficient,

		partial = function(/*Function|String*/method /*, ...*/){
			var arr = [ null ];
			return lang.hitch.apply(dojo, arr.concat(lang._toArray(arguments))); // Function
		},

		clone = function(/*anything*/ src){
			if(!src || typeof src != "object" || lang.isFunction(src)){
				// null, undefined, any non-object, or function
				return src;	// anything
			}
			if(src.nodeType && "cloneNode" in src){
				// DOM Node
				return src.cloneNode(true); // Node
			}
			if(src instanceof Date){
				// Date
				return new Date(src.getTime());	// Date
			}
			if(src instanceof RegExp){
				// RegExp
				return new RegExp(src);   // RegExp
			}
			var r, i, l;
			if(lang.isArray(src)){
				// array
				r = [];
				for(i = 0, l = src.length; i < l; ++i){
					if(i in src){
						r.push(clone(src[i]));
					}
				}
	// we don't clone functions for performance reasons
	//		}else if(d.isFunction(src)){
	//			// function
	//			r = function(){ return src.apply(this, arguments); };
			}else{
				// generic objects
				r = src.constructor ? new src.constructor() : {};
			}
			return lang._mixin(r, src, clone);
		},


		trim = String.prototype.trim ?
			function(str){ return str.trim(); } :
			function(str){ return str.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); },


		_pattern = /\{([^\}]+)\}/g,

		replace = function(tmpl, map, pattern){
			return tmpl.replace(pattern || _pattern, lang.isFunction(map) ?
				map : function(_, k){ return getObject(k, false, map); });
		},

		lang = {
			_extraNames:_extraNames,
			_mixin:_mixin,
			mixin:mixin,
			setObject:setObject,
			getObject:getObject,
			exists:exists,
			isString:isString,
			isArray:isArray,
			isFunction:isFunction,
			isObject:isObject,
			isArrayLike:isArrayLike,
			isAlien:isAlien,
			extend:extend,
			_hitchArgs:_hitchArgs,
			hitch:hitch,
			delegate:delegate,
			_toArray:_toArray,
			partial:partial,
			clone:clone,
			trim:trim,
			replace:replace
		};

	1 && mixin(dojo, lang);
	return lang;

	/*=====
	dojo._extraNames
	// summary:
	//		Array of strings. Lists property names that must be explicitly processed during for-in interation
	//		in environments that have has("bug-for-in-skips-shadowed") true.
	=====*/

	/*=====
	dojo._mixin = function(dest, source, copyFunc){
		//	summary:
		//		Copies/adds all properties of source to dest; returns dest.
		//	dest: Object:
		//		The object to which to copy/add all properties contained in source.
		//	source: Object:
		//		The object from which to draw all properties to copy into dest.
		//	copyFunc: Function?:
		//		The process used to copy/add a property in source; defaults to the Javascript assignment operator.
		//	returns:
		//		dest, as modified
		//	description:
		//		All properties, including functions (sometimes termed "methods"), excluding any non-standard extensions
		//		found in Object.prototype, are copied/added to dest. Copying/adding each particular property is
		//		delegated to copyFunc (if any); copyFunc defaults to the Javascript assignment operator if not provided.
		//		Notice that by default, _mixin executes a so-called "shallow copy" and aggregate types are copied/added by reference.
	}
	=====*/

	/*=====
	dojo.mixin = function(dest, sources){
	//	summary:
	//		Copies/adds all properties of one or more sources to dest; returns dest.
	//	dest: Object
	//		The object to which to copy/add all properties contained in source. If dest is falsy, then
	//		a new object is manufactured before copying/adding properties begins.
	//	sources: Object...
	//		One of more objects from which to draw all properties to copy into dest. sources are processed
	//		left-to-right and if more than one of these objects contain the same property name, the right-most
	//		value "wins".
	//	returns: Object
	//		dest, as modified
	//	description:
	//		All properties, including functions (sometimes termed "methods"), excluding any non-standard extensions
	//		found in Object.prototype, are copied/added from sources to dest. sources are processed left to right.
	//		The Javascript assignment operator is used to copy/add each property; therefore, by default, mixin
	//		executes a so-called "shallow copy" and aggregate types are copied/added by reference.
	//	example:
	//		make a shallow copy of an object
	//	| var copy = lang.mixin({}, source);
	//	example:
	//		many class constructors often take an object which specifies
	//		values to be configured on the object. In this case, it is
	//		often simplest to call `lang.mixin` on the `this` object:
	//	| dojo.declare("acme.Base", null, {
	//	|		constructor: function(properties){
	//	|			// property configuration:
	//	|			lang.mixin(this, properties);
	//	|
	//	|			console.log(this.quip);
	//	|			//	...
	//	|		},
	//	|		quip: "I wasn't born yesterday, you know - I've seen movies.",
	//	|		// ...
	//	| });
	//	|
	//	| // create an instance of the class and configure it
	//	| var b = new acme.Base({quip: "That's what it does!" });
	//	example:
	//		copy in properties from multiple objects
	//	| var flattened = lang.mixin(
	//	|		{
	//	|			name: "Frylock",
	//	|			braces: true
	//	|		},
	//	|		{
	//	|			name: "Carl Brutanananadilewski"
	//	|		}
	//	| );
	//	|
	//	| // will print "Carl Brutanananadilewski"
	//	| console.log(flattened.name);
	//	| // will print "true"
	//	| console.log(flattened.braces);
	}
	=====*/

	/*=====
	dojo.setObject = function(name, value, context){
	// summary:
	//		Set a property from a dot-separated string, such as "A.B.C"
	//	description:
	//		Useful for longer api chains where you have to test each object in
	//		the chain, or when you have an object reference in string format.
	//		Objects are created as needed along `path`. Returns the passed
	//		value if setting is successful or `undefined` if not.
	//	name: String
	//		Path to a property, in the form "A.B.C".
	//	value: anything
	//		value or object to place at location given by name
	//	context: Object?
	//		Optional. Object to use as root of path. Defaults to
	//		`dojo.global`.
	//	example:
	//		set the value of `foo.bar.baz`, regardless of whether
	//		intermediate objects already exist:
	//	| lang.setObject("foo.bar.baz", value);
	//	example:
	//		without `lang.setObject`, we often see code like this:
	//	| // ensure that intermediate objects are available
	//	| if(!obj["parent"]){ obj.parent = {}; }
	//	| if(!obj.parent["child"]){ obj.parent.child = {}; }
	//	| // now we can safely set the property
	//	| obj.parent.child.prop = "some value";
	//		whereas with `lang.setObject`, we can shorten that to:
	//	| lang.setObject("parent.child.prop", "some value", obj);
	}
	=====*/

	/*=====
	dojo.getObject = function(name, create, context){
	// summary:
	//		Get a property from a dot-separated string, such as "A.B.C"
	//	description:
	//		Useful for longer api chains where you have to test each object in
	//		the chain, or when you have an object reference in string format.
	//	name: String
	//		Path to an property, in the form "A.B.C".
	//	create: Boolean?
	//		Optional. Defaults to `false`. If `true`, Objects will be
	//		created at any point along the 'path' that is undefined.
	//	context: Object?
	//		Optional. Object to use as root of path. Defaults to
	//		'dojo.global'. Null may be passed.
	}
	=====*/

	/*=====
	dojo.exists = function(name, obj){
	//	summary:
	//		determine if an object supports a given method
	//	description:
	//		useful for longer api chains where you have to test each object in
	//		the chain. Useful for object and method detection.
	//	name: String
	//		Path to an object, in the form "A.B.C".
	//	obj: Object?
	//		Object to use as root of path. Defaults to
	//		'dojo.global'. Null may be passed.
	//	example:
	//	| // define an object
	//	| var foo = {
	//	|		bar: { }
	//	| };
	//	|
	//	| // search the global scope
	//	| lang.exists("foo.bar"); // true
	//	| lang.exists("foo.bar.baz"); // false
	//	|
	//	| // search from a particular scope
	//	| lang.exists("bar", foo); // true
	//	| lang.exists("bar.baz", foo); // false
	}
	=====*/

	/*=====
	dojo.isString = function(it){
	//	summary:
	//		Return true if it is a String
	//	it: anything
	//		Item to test.
	}
	=====*/

	/*=====
	dojo.isArray = function(it){
	//	summary:
	//		Return true if it is an Array.
	//		Does not work on Arrays created in other windows.
	//	it: anything
	//		Item to test.
	}
	=====*/

	/*=====
	dojo.isFunction = function(it){
	// summary:
	//		Return true if it is a Function
	//	it: anything
	//		Item to test.
	}
	=====*/

	/*=====
	dojo.isObject = function(it){
	// summary:
	//		Returns true if it is a JavaScript object (or an Array, a Function
	//		or null)
	//	it: anything
	//		Item to test.
	}
	=====*/

	/*=====
	dojo.isArrayLike = function(it){
	//	summary:
	//		similar to dojo.isArray() but more permissive
	//	it: anything
	//		Item to test.
	//	returns:
	//		If it walks like a duck and quacks like a duck, return `true`
	//	description:
	//		Doesn't strongly test for "arrayness".  Instead, settles for "isn't
	//		a string or number and has a length property". Arguments objects
	//		and DOM collections will return true when passed to
	//		dojo.isArrayLike(), but will return false when passed to
	//		dojo.isArray().
	}
	=====*/

	/*=====
	dojo.isAlien = function(it){
	// summary:
	//		Returns true if it is a built-in function or some other kind of
	//		oddball that *should* report as a function but doesn't
	}
	=====*/

	/*=====
	dojo.extend = function(constructor, props){
	// summary:
	//		Adds all properties and methods of props to constructor's
	//		prototype, making them available to all instances created with
	//		constructor.
	//	constructor: Object
	//		Target constructor to extend.
	//	props: Object...
	//		One or more objects to mix into constructor.prototype
	}
	=====*/

	/*=====
	dojo.hitch = function(scope, method){
	//	summary:
	//		Returns a function that will only ever execute in the a given scope.
	//		This allows for easy use of object member functions
	//		in callbacks and other places in which the "this" keyword may
	//		otherwise not reference the expected scope.
	//		Any number of default positional arguments may be passed as parameters
	//		beyond "method".
	//		Each of these values will be used to "placehold" (similar to curry)
	//		for the hitched function.
	//	scope: Object
	//		The scope to use when method executes. If method is a string,
	//		scope is also the object containing method.
	//	method: Function|String...
	//		A function to be hitched to scope, or the name of the method in
	//		scope to be hitched.
	//	example:
	//	|	dojo.hitch(foo, "bar")();
	//		runs foo.bar() in the scope of foo
	//	example:
	//	|	dojo.hitch(foo, myFunction);
	//		returns a function that runs myFunction in the scope of foo
	//	example:
	//		Expansion on the default positional arguments passed along from
	//		hitch. Passed args are mixed first, additional args after.
	//	|	var foo = { bar: function(a, b, c){ console.log(a, b, c); } };
	//	|	var fn = dojo.hitch(foo, "bar", 1, 2);
	//	|	fn(3); // logs "1, 2, 3"
	//	example:
	//	|	var foo = { bar: 2 };
	//	|	dojo.hitch(foo, function(){ this.bar = 10; })();
	//		execute an anonymous function in scope of foo
	}
	=====*/

	/*=====
	dojo.delegate = function(obj, props){
		//	summary:
		//		Returns a new object which "looks" to obj for properties which it
		//		does not have a value for. Optionally takes a bag of properties to
		//		seed the returned object with initially.
		//	description:
		//		This is a small implementaton of the Boodman/Crockford delegation
		//		pattern in JavaScript. An intermediate object constructor mediates
		//		the prototype chain for the returned object, using it to delegate
		//		down to obj for property lookup when object-local lookup fails.
		//		This can be thought of similarly to ES4's "wrap", save that it does
		//		not act on types but rather on pure objects.
		//	obj: Object
		//		The object to delegate to for properties not found directly on the
		//		return object or in props.
		//	props: Object...
		//		an object containing properties to assign to the returned object
		//	returns:
		//		an Object of anonymous type
		//	example:
		//	|	var foo = { bar: "baz" };
		//	|	var thinger = dojo.delegate(foo, { thud: "xyzzy"});
		//	|	thinger.bar == "baz"; // delegated to foo
		//	|	foo.thud == undefined; // by definition
		//	|	thinger.thud == "xyzzy"; // mixed in from props
		//	|	foo.bar = "thonk";
		//	|	thinger.bar == "thonk"; // still delegated to foo's bar
	}
	=====*/

	/*=====
	dojo.partial = function(method){
	//	summary:
	//		similar to hitch() except that the scope object is left to be
	//		whatever the execution context eventually becomes.
	//	method: Function|String
	//	description:
	//		Calling dojo.partial is the functional equivalent of calling:
	//		|	dojo.hitch(null, funcName, ...);
	}
	=====*/

	/*=====
	dojo.trim = function(str){
		//	summary:
		//		Trims whitespace from both sides of the string
		//	str: String
		//		String to be trimmed
		//	returns: String
		//		Returns the trimmed string
		//	description:
		//		This version of trim() was selected for inclusion into the base due
		//		to its compact size and relatively good performance
		//		(see [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript)
		//		Uses String.prototype.trim instead, if available.
		//		The fastest but longest version of this function is located at
		//		dojo.string.trim()
	}
	=====*/

	/*=====
	dojo.clone = function(src){
	// summary:
	//		Clones objects (including DOM nodes) and all children.
	//		Warning: do not clone cyclic structures.
	//	src:
	//		The object to clone
	}
	=====*/

	/*=====
	dojo._toArray = function(obj, offset, startWith){
		//	summary:
		//		Converts an array-like object (i.e. arguments, DOMCollection) to an
		//		array. Returns a new Array with the elements of obj.
		//	obj: Object
		//		the object to "arrayify". We expect the object to have, at a
		//		minimum, a length property which corresponds to integer-indexed
		//		properties.
		//	offset: Number?
		//		the location in obj to start iterating from. Defaults to 0.
		//		Optional.
		//	startWith: Array?
		//		An array to pack with the properties of obj. If provided,
		//		properties in obj are appended at the end of startWith and
		//		startWith is the returned array.
	}
	=====*/

	/*=====
	dojo.replace = function(tmpl, map, pattern){
		//	summary:
		//		Performs parameterized substitutions on a string. Throws an
		//		exception if any parameter is unmatched.
		//	tmpl: String
		//		String to be used as a template.
		//	map: Object|Function
		//		If an object, it is used as a dictionary to look up substitutions.
		//		If a function, it is called for every substitution with following
		//		parameters: a whole match, a name, an offset, and the whole template
		//		string (see https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/String/replace
		//		for more details).
		//	pattern: RegEx?
		//		Optional regular expression objects that overrides the default pattern.
		//		Must be global and match one item. The default is: /\{([^\}]+)\}/g,
		//		which matches patterns like that: "{xxx}", where "xxx" is any sequence
		//		of characters, which doesn't include "}".
		//	returns: String
		//		Returns the substituted string.
		//	example:
		//	|	// uses a dictionary for substitutions:
		//	|	dojo.replace("Hello, {name.first} {name.last} AKA {nick}!",
		//	|		{
		//	|			nick: "Bob",
		//	|			name: {
		//	|				first:	"Robert",
		//	|				middle: "X",
		//	|				last:		"Cringely"
		//	|			}
		//	|		});
		//	|	// returns: Hello, Robert Cringely AKA Bob!
		//	example:
		//	|	// uses an array for substitutions:
		//	|	dojo.replace("Hello, {0} {2}!",
		//	|		["Robert", "X", "Cringely"]);
		//	|	// returns: Hello, Robert Cringely!
		//	example:
		//	|	// uses a function for substitutions:
		//	|	function sum(a){
		//	|		var t = 0;
		//	|		dojo.forEach(a, function(x){ t += x; });
		//	|		return t;
		//	|	}
		//	|	dojo.replace(
		//	|		"{count} payments averaging {avg} USD per payment.",
		//	|		dojo.hitch(
		//	|			{ payments: [11, 16, 12] },
		//	|			function(_, key){
		//	|				switch(key){
		//	|					case "count": return this.payments.length;
		//	|					case "min":		return Math.min.apply(Math, this.payments);
		//	|					case "max":		return Math.max.apply(Math, this.payments);
		//	|					case "sum":		return sum(this.payments);
		//	|					case "avg":		return sum(this.payments) / this.payments.length;
		//	|				}
		//	|			}
		//	|		)
		//	|	);
		//	|	// prints: 3 payments averaging 13 USD per payment.
		//	example:
		//	|	// uses an alternative PHP-like pattern for substitutions:
		//	|	dojo.replace("Hello, ${0} ${2}!",
		//	|		["Robert", "X", "Cringely"], /\$\{([^\}]+)\}/g);
		//	|	// returns: Hello, Robert Cringely!
		return "";	// String
	}
	=====*/
});


},
'dijit/registry':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window", // win.body
	"."	// dijit._scopeName
], function(array, has, unload, win, dijit){

	// module:
	//		dijit/registry
	// summary:
	//		Registry of existing widget on page, plus some utility methods.
	//		Must be accessed through AMD api, ex:
	//		require(["dijit/registry"], function(registry){ registry.byId("foo"); })

	var _widgetTypeCtr = {}, hash = {};

	var registry =  {
		// summary:
		//		A set of widgets indexed by id

		length: 0,

		add: function(/*dijit._Widget*/ widget){
			// summary:
			//		Add a widget to the registry. If a duplicate ID is detected, a error is thrown.
			//
			// widget: dijit._Widget
			//		Any dijit._Widget subclass.
			if(hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from the registry. Does not destroy the widget; simply
			//		removes the reference.
			if(hash[id]){
				delete hash[id];
				this.length--;
			}
		},

		byId: function(/*String|Widget*/ id){
			// summary:
			//		Find a widget by it's id.
			//		If passed a widget then just returns the widget.
			return typeof id == "string" ? hash[id] : id;	// dijit._Widget
		},

		byNode: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget corresponding to the given DOMNode
			return hash[node.getAttribute("widgetId")]; // dijit._Widget
		},

		toArray: function(){
			// summary:
			//		Convert registry into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	array.map(dijit.registry.toArray(), function(w){ return w.domNode; });

			var ar = [];
			for(var id in hash){
				ar.push(hash[id]);
			}
			return ar;	// dijit._Widget[]
		},

		getUniqueId: function(/*String*/widgetType){
			// summary:
			//		Generates a unique id for a given widgetType

			var id;
			do{
				id = widgetType + "_" +
					(widgetType in _widgetTypeCtr ?
						++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
			}while(hash[id]);
			return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
		},

		findWidgets: function(/*DomNode*/ root){
			// summary:
			//		Search subtree under root returning widgets found.
			//		Doesn't search for nested widgets (ie, widgets inside other widgets).

			var outAry = [];

			function getChildrenHelper(root){
				for(var node = root.firstChild; node; node = node.nextSibling){
					if(node.nodeType == 1){
						var widgetId = node.getAttribute("widgetId");
						if(widgetId){
							var widget = hash[widgetId];
							if(widget){	// may be null on page w/multiple dojo's loaded
								outAry.push(widget);
							}
						}else{
							getChildrenHelper(node);
						}
					}
				}
			}

			getChildrenHelper(root);
			return outAry;
		},

		_destroyAll: function(){
			// summary:
			//		Code to destroy all widgets and do other cleanup on page unload

			// Clean up focus manager lingering references to widgets and nodes
			dijit._curFocus = null;
			dijit._prevFocus = null;
			dijit._activeStack = [];

			// Destroy all the widgets, top down
			array.forEach(registry.findWidgets(win.body()), function(widget){
				// Avoid double destroy of widgets like Menu that are attached to <body>
				// even though they are logically children of other widgets.
				if(!widget._destroyed){
					if(widget.destroyRecursive){
						widget.destroyRecursive();
					}else if(widget.destroy){
						widget.destroy();
					}
				}
			});
		},

		getEnclosingWidget: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
			//		the node is not contained within the DOM tree of any widget
			while(node){
				var id = node.getAttribute && node.getAttribute("widgetId");
				if(id){
					return hash[id];
				}
				node = node.parentNode;
			}
			return null;
		},

		// In case someone needs to access hash.
		// Actually, this is accessed from WidgetSet back-compatibility code
		_hash: hash
	};

	/*=====
	dijit.registry = {
		// summary:
		//		A list of widgets on a page.
	};
	=====*/
	dijit.registry = registry;

	return registry;
});

},
'dijit/_base/manager':function(){
define([
	"dojo/_base/array",
	"dojo/_base/config", // defaultDuration
	"../registry",
	".."	// for setting exports to dijit namespace
], function(array, config, registry, dijit){

	// module:
	//		dijit/_base/manager
	// summary:
	//		Shim to methods on registry, plus a few other declarations.
	//		New code should access dijit/registry directly when possible.

	/*=====
	dijit.byId = function(id){
		// summary:
		//		Returns a widget by it's id, or if passed a widget, no-op (like dom.byId())
		// id: String|dijit._Widget
		return registry.byId(id); // dijit._Widget
	};

	dijit.getUniqueId = function(widgetType){
		// summary:
		//		Generates a unique id for a given widgetType
		// widgetType: String
		return registry.getUniqueId(widgetType); // String
	};

	dijit.findWidgets = function(root){
		// summary:
		//		Search subtree under root returning widgets found.
		//		Doesn't search for nested widgets (ie, widgets inside other widgets).
		// root: DOMNode
		return registry.findWidgets(root);
	};

	dijit._destroyAll = function(){
		// summary:
		//		Code to destroy all widgets and do other cleanup on page unload

		return registry._destroyAll();
	};

	dijit.byNode = function(node){
		// summary:
		//		Returns the widget corresponding to the given DOMNode
		// node: DOMNode
		return registry.byNode(node); // dijit._Widget
	};

	dijit.getEnclosingWidget = function(node){
		// summary:
		//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
		//		the node is not contained within the DOM tree of any widget
		// node: DOMNode
		return registry.getEnclosingWidget(node);
	};
	=====*/
	array.forEach(["byId", "getUniqueId", "findWidgets", "_destroyAll", "byNode", "getEnclosingWidget"], function(name){
		dijit[name] = registry[name];
	});

	/*=====
	dojo.mixin(dijit, {
		// defaultDuration: Integer
		//		The default fx.animation speed (in ms) to use for all Dijit
		//		transitional fx.animations, unless otherwise specified
		//		on a per-instance basis. Defaults to 200, overrided by
		//		`djConfig.defaultDuration`
		defaultDuration: 200
	});
	=====*/
	dijit.defaultDuration = config["defaultDuration"] || 200;

	return dijit;
});

},
'dojo/_base/Color':function(){
define(["./kernel", "./lang", "./array", "./config"], function(dojo, lang, ArrayUtil, config){

	var Color = dojo.Color = function(/*Array|String|Object*/ color){
		// summary:
		//		Takes a named string, hex string, array of rgb or rgba values,
		//		an object with r, g, b, and a properties, or another `dojo.Color` object
		//		and creates a new Color instance to work from.
		//
		// example:
		//		Work with a Color instance:
		//	 | var c = new dojo.Color();
		//	 | c.setColor([0,0,0]); // black
		//	 | var hex = c.toHex(); // #000000
		//
		// example:
		//		Work with a node's color:
		//	 | var color = dojo.style("someNode", "backgroundColor");
		//	 | var n = new dojo.Color(color);
		//	 | // adjust the color some
		//	 | n.r *= .5;
		//	 | console.log(n.toString()); // rgb(128, 255, 255);
		if(color){ this.setColor(color); }
	};

	/*=====
	lang.mixin(dojo.Color,{
		named:{
			// summary: Dictionary list of all CSS named colors, by name. Values are 3-item arrays with corresponding RG and B values.
		}
	});
	=====*/

	// FIXME:
	// there's got to be a more space-efficient way to encode or discover
	// these!! Use hex?
	Color.named = {
		"black":  [0,0,0],
		"silver": [192,192,192],
		"gray":	  [128,128,128],
		"white":  [255,255,255],
		"maroon": [128,0,0],
		"red":	  [255,0,0],
		"purple": [128,0,128],
		"fuchsia":[255,0,255],
		"green":  [0,128,0],
		"lime":	  [0,255,0],
		"olive":  [128,128,0],
		"yellow": [255,255,0],
		"navy":	  [0,0,128],
		"blue":	  [0,0,255],
		"teal":	  [0,128,128],
		"aqua":	  [0,255,255],
		"transparent": config.transparentColor || [0,0,0,0]
	};

	lang.extend(Color, {
		r: 255, g: 255, b: 255, a: 1,
		_set: function(r, g, b, a){
			var t = this; t.r = r; t.g = g; t.b = b; t.a = a;
		},
		setColor: function(/*Array|String|Object*/ color){
			// summary:
			//		Takes a named string, hex string, array of rgb or rgba values,
			//		an object with r, g, b, and a properties, or another `dojo.Color` object
			//		and sets this color instance to that value.
			//
			// example:
			//	|	var c = new dojo.Color(); // no color
			//	|	c.setColor("#ededed"); // greyish
			if(lang.isString(color)){
				Color.fromString(color, this);
			}else if(lang.isArray(color)){
				Color.fromArray(color, this);
			}else{
				this._set(color.r, color.g, color.b, color.a);
				if(!(color instanceof Color)){ this.sanitize(); }
			}
			return this;	// dojo.Color
		},
		sanitize: function(){
			// summary:
			//		Ensures the object has correct attributes
			// description:
			//		the default implementation does nothing, include dojo.colors to
			//		augment it with real checks
			return this;	// dojo.Color
		},
		toRgb: function(){
			// summary:
			//		Returns 3 component array of rgb values
			// example:
			//	|	var c = new dojo.Color("#000000");
			//	|	console.log(c.toRgb()); // [0,0,0]
			var t = this;
			return [t.r, t.g, t.b]; // Array
		},
		toRgba: function(){
			// summary:
			//		Returns a 4 component array of rgba values from the color
			//		represented by this object.
			var t = this;
			return [t.r, t.g, t.b, t.a];	// Array
		},
		toHex: function(){
			// summary:
			//		Returns a CSS color string in hexadecimal representation
			// example:
			//	|	console.log(new dojo.Color([0,0,0]).toHex()); // #000000
			var arr = ArrayUtil.map(["r", "g", "b"], function(x){
				var s = this[x].toString(16);
				return s.length < 2 ? "0" + s : s;
			}, this);
			return "#" + arr.join("");	// String
		},
		toCss: function(/*Boolean?*/ includeAlpha){
			// summary:
			//		Returns a css color string in rgb(a) representation
			// example:
			//	|	var c = new dojo.Color("#FFF").toCss();
			//	|	console.log(c); // rgb('255','255','255')
			var t = this, rgb = t.r + ", " + t.g + ", " + t.b;
			return (includeAlpha ? "rgba(" + rgb + ", " + t.a : "rgb(" + rgb) + ")";	// String
		},
		toString: function(){
			// summary:
			//		Returns a visual representation of the color
			return this.toCss(true); // String
		}
	});

	Color.blendColors = dojo.blendColors = function(
		/*dojo.Color*/ start,
		/*dojo.Color*/ end,
		/*Number*/ weight,
		/*dojo.Color?*/ obj
	){
		// summary:
		//		Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,
		//		can reuse a previously allocated dojo.Color object for the result
		var t = obj || new Color();
		ArrayUtil.forEach(["r", "g", "b", "a"], function(x){
			t[x] = start[x] + (end[x] - start[x]) * weight;
			if(x != "a"){ t[x] = Math.round(t[x]); }
		});
		return t.sanitize();	// dojo.Color
	};

	Color.fromRgb = dojo.colorFromRgb = function(/*String*/ color, /*dojo.Color?*/ obj){
		// summary:
		//		Returns a `dojo.Color` instance from a string of the form
		//		"rgb(...)" or "rgba(...)". Optionally accepts a `dojo.Color`
		//		object to update with the parsed value and return instead of
		//		creating a new object.
		// returns:
		//		A dojo.Color object. If obj is passed, it will be the return value.
		var m = color.toLowerCase().match(/^rgba?\(([\s\.,0-9]+)\)/);
		return m && Color.fromArray(m[1].split(/\s*,\s*/), obj);	// dojo.Color
	};

	Color.fromHex = dojo.colorFromHex = function(/*String*/ color, /*dojo.Color?*/ obj){
		// summary:
		//		Converts a hex string with a '#' prefix to a color object.
		//		Supports 12-bit #rgb shorthand. Optionally accepts a
		//		`dojo.Color` object to update with the parsed value.
		//
		// returns:
		//		A dojo.Color object. If obj is passed, it will be the return value.
		//
		// example:
		//	 | var thing = dojo.colorFromHex("#ededed"); // grey, longhand
		//
		// example:
		//	| var thing = dojo.colorFromHex("#000"); // black, shorthand
		var t = obj || new Color(),
			bits = (color.length == 4) ? 4 : 8,
			mask = (1 << bits) - 1;
		color = Number("0x" + color.substr(1));
		if(isNaN(color)){
			return null; // dojo.Color
		}
		ArrayUtil.forEach(["b", "g", "r"], function(x){
			var c = color & mask;
			color >>= bits;
			t[x] = bits == 4 ? 17 * c : c;
		});
		t.a = 1;
		return t;	// dojo.Color
	};

	Color.fromArray = dojo.colorFromArray = function(/*Array*/ a, /*dojo.Color?*/ obj){
		// summary:
		//		Builds a `dojo.Color` from a 3 or 4 element array, mapping each
		//		element in sequence to the rgb(a) values of the color.
		// example:
		//		| var myColor = dojo.colorFromArray([237,237,237,0.5]); // grey, 50% alpha
		// returns:
		//		A dojo.Color object. If obj is passed, it will be the return value.
		var t = obj || new Color();
		t._set(Number(a[0]), Number(a[1]), Number(a[2]), Number(a[3]));
		if(isNaN(t.a)){ t.a = 1; }
		return t.sanitize();	// dojo.Color
	};

	Color.fromString = dojo.colorFromString = function(/*String*/ str, /*dojo.Color?*/ obj){
		// summary:
		//		Parses `str` for a color value. Accepts hex, rgb, and rgba
		//		style color values.
		// description:
		//		Acceptable input values for str may include arrays of any form
		//		accepted by dojo.colorFromArray, hex strings such as "#aaaaaa", or
		//		rgb or rgba strings such as "rgb(133, 200, 16)" or "rgba(10, 10,
		//		10, 50)"
		// returns:
		//		A dojo.Color object. If obj is passed, it will be the return value.
		var a = Color.named[str];
		return a && Color.fromArray(a, obj) || Color.fromRgb(str, obj) || Color.fromHex(str, obj);	// dojo.Color
	};

	return Color;
});

},
'dojo/_base/loader':function(){
define(["./kernel", "../has", "require", "module", "./json", "./lang", "./array"], function(dojo, has, require, thisModule, json, lang, array) {
	// module:
	//		dojo/_base/lader
	// summary:
	//		This module defines the v1.x synchronous loader API.

	// signal the loader in sync mode...
	//>>pure-amd

	if (!1){
		console.error("cannot load the Dojo v1.x loader with a foreign loader");
		return 0;
	}

	var makeErrorToken = function(id){
			return {src:thisModule.id, id:id};
		},

		slashName = function(name){
			return name.replace(/\./g, "/");
		},

		buildDetectRe = /\/\/>>built/,

		dojoRequireCallbacks = [],
		dojoRequireModuleStack = [],

		dojoRequirePlugin = function(mid, require, loaded){
			dojoRequireCallbacks.push(loaded);
			array.forEach(mid.split(","), function(mid){
				var module = getModule(mid, require.module);
				dojoRequireModuleStack.push(module);
				injectModule(module);
			});
			checkDojoRequirePlugin();
		},

		// checkDojoRequirePlugin inspects all of the modules demanded by a dojo/require!<module-list> dependency
		// to see if they have arrived. The loader does not release *any* of these modules to be instantiated
		// until *all* of these modules are on board, thereby preventing the evaluation of a module with dojo.require's
		// that reference modules that are not available.
		//
		// The algorithm works by traversing the dependency graphs (remember, there can be cycles so they are not trees)
		// of each module in the dojoRequireModuleStack array (which contains the list of modules demanded by dojo/require!).
		// The moment a single module is discovered that is missing, the algorithm gives up and indicates that not all
		// modules are on board. dojo/loadInit! and dojo/require! are ignored because there dependencies are inserted
		// directly in dojoRequireModuleStack. For example, if "your/module" module depends on "dojo/require!my/module", then
		// *both* "dojo/require!my/module" and "my/module" will be in dojoRequireModuleStack. Obviously, if "my/module"
		// is on board, then "dojo/require!my/module" is also satisfied, so the algorithm doesn't check for "dojo/require!my/module".
		//
		// Note: inserting a dojo/require!<some-module-list> dependency in the dojoRequireModuleStack achieves nothing
		// with the current algorithm; however, having such modules present makes it possible to optimize the algorithm
		//
		// Note: prior versions of this algorithm had an optimization that signaled loaded on dojo/require! dependencies
		// individually (rather than waiting for them all to be resolved). The implementation proved problematic with cycles
		// and plugins. However, it is possible to reattach that strategy in the future.

		// a set from module-id to {undefined | 1 | 0}, where...
		//   undefined => the module has not been inspected
		//   0 => the module or at least one of its dependencies has not arrived
		//   1 => the module is a loadInit! or require! plugin resource, or is currently being traversed (therefore, assume
		//        OK until proven otherwise), or has been completely traversed and all dependencies have arrived
		touched,

		traverse = function(m){
		    touched[m.mid] = 1;
			for(var t, module, deps = m.deps || [], i= 0; i<deps.length; i++){
				module = deps[i];
				if(!(t = touched[module.mid])){
					if(t===0 || !traverse(module)){
						touched[m.mid] = 0;
						return false;
					}
				}
			}
			return true;
		},

		checkDojoRequirePlugin = function(){
			// initialize the touched hash with easy-to-compute values that help short circuit recursive algorithm;
			// recall loadInit/require plugin modules are dependencies of modules in dojoRequireModuleStack...
			// which would cause a circular dependency chain that would never be resolved if checked here
			// notice all dependencies of any particular loadInit/require plugin module will already
			// be checked since those are pushed into dojoRequireModuleStack explicitly by the
			// plugin...so if a particular loadInitPlugin module's dependencies are not really
			// on board, that *will* be detected elsewhere in the traversal.
			var module, mid;
			touched = {};
			for(mid in modules){
				module = modules[mid];
				// this could be improved by remembering the result of the regex tests
				if(module.executed || module.noReqPluginCheck){
					touched[mid] = 1;
				}else{
					if(module.noReqPluginCheck!==0){
						// tag the module as either a loadInit or require plugin or not for future reference
						module.noReqPluginCheck = /loadInit\!/.test(mid) || /require\!/.test(mid) ? 1 : 0;
					}
					if(module.noReqPluginCheck){
						touched[mid] = 1;
					}else if(module.injected!==arrived){
						// not executed, has not arrived, and is not a loadInit or require plugin resource
						touched[mid] = 0;
					}// else, leave undefined and we'll traverse the dependencies
				}
			}

			for(var t, i = 0, end = dojoRequireModuleStack.length; i<end; i++){
				module = dojoRequireModuleStack[i];
				if(!(t = touched[module.mid])){
					if(t===0 || !traverse(module)){
						return;
					}
				}
			}
			loaderVars.holdIdle();
			var oldCallbacks = dojoRequireCallbacks;
			dojoRequireCallbacks = [];
			array.forEach(oldCallbacks, function(cb){cb(1);});
			loaderVars.releaseIdle();
		},

		dojoLoadInitPlugin = function(mid, require, loaded){
			// mid names a module that defines a "dojo load init" bundle, an object with two properties:
			//
			//   * names: a vector of module ids that give top-level names to define in the lexical scope of def
			//   * def: a function that contains some some legacy loader API applications
			//
			// The point of def is to possibly cause some modules to be loaded (but not executed) by dojo/require! where the module
			// ids are possibly-determined at runtime. For example, here is dojox.gfx from v1.6 expressed as an AMD module using the dojo/loadInit
			// and dojo/require plugins.
			//
			// // dojox/gfx:
			//
			//   define("*loadInit_12, {
			//     names:["dojo", "dijit", "dojox"],
			//     def: function(){
			//       dojo.loadInit(function(){
			//         var gfx = lang.getObject("dojox.gfx", true);
			//
			//         //
			//         // code required to set gfx properties ommitted...
			//         //
			//
			//         // now use the calculations to include the runtime-dependent module
			//         dojo.require("dojox.gfx." + gfx.renderer);
			//       });
			//	   }
			//   });
			//
			//   define(["dojo", "dojo/loadInit!" + id].concat("dojo/require!dojox/gfx/matric,dojox/gfx/_base"), function(dojo){
			//     // when this AMD factory function is executed, the following modules are guaranteed downloaded but not executed:
			//     //   "dojox.gfx." + gfx.renderer
			//     //   dojox.gfx.matrix
			//     //   dojox.gfx._base
			//     dojo.provide("dojo.gfx");
			//     dojo.require("dojox.gfx.matrix");
			//     dojo.require("dojox.gfx._base");
			//     dojo.require("dojox.gfx." + gfx.renderer);
			//     return lang.getObject("dojo.gfx");
			//   });
			//  })();
			//
			// The idea is to run the legacy loader API with global variables shadowed, which allows these variables to
			// be relocated. For example, dojox and dojo could be relocated to different names by giving a packageMap and the code above will
			// execute properly (because the plugin below resolves the load init bundle.names module with respect to the module that demanded
			// the plugin resource).
			//
			// Note that the relocation is specified in the runtime configuration; relocated names need not be set at build-time.
			//
			// Warning: this is not the best way to express dojox.gfx as and AMD module. In fact, the module has been properly converted in
			// v1.7. However, this technique allows the builder to convert legacy modules into AMD modules and guarantee the codepath is the
			// same in the converted AMD module.
			require([mid], function(bundle){
				// notice how names is resolved with respect to the module that demanded the plugin resource
				require(bundle.names, function(){
					// bring the bundle names into scope
					for(var scopeText = "", args= [], i = 0; i<arguments.length; i++){
						scopeText+= "var " + bundle.names[i] + "= arguments[" + i + "]; ";
						args.push(arguments[i]);
					}
					eval(scopeText);

					var callingModule = require.module,
						deps = [],
						hold = {},
						requireList = [],
						p,
						syncLoaderApi = {
							provide:function(moduleName){
								// mark modules that arrive consequent to multiple provides in this module as arrived since they can't be injected
								moduleName = slashName(moduleName);
								var providedModule = getModule(moduleName, callingModule);
								if(providedModule!==callingModule){
									setArrived(providedModule);
								}
							},
							require:function(moduleName, omitModuleCheck){
								moduleName = slashName(moduleName);
								omitModuleCheck && (getModule(moduleName, callingModule).result = nonmodule);
								requireList.push(moduleName);
							},
							requireLocalization:function(moduleName, bundleName, locale){
								// since we're going to need dojo/i8n, add it to deps if not already there
								deps.length || (deps = ["dojo/i18n"]);

								// figure out if the bundle is xdomain; if so, add it to the depsSet
								locale = (locale || dojo.locale).toLowerCase();
								moduleName = slashName(moduleName) + "/nls/" + (/root/i.test(locale) ? "" : locale + "/") + slashName(bundleName);
								if(getModule(moduleName, callingModule).isXd){
									deps.push("dojo/i18n!" + moduleName);
								}// else the bundle will be loaded synchronously when the module is evaluated
							},
							loadInit:function(f){
								f();
							}
						};

					// hijack the correct dojo and apply bundle.def
					try{
						for(p in syncLoaderApi){
							hold[p] = dojo[p];
							dojo[p] = syncLoaderApi[p];
						}
						bundle.def.apply(null, args);
					}catch(e){
						signal("error", [makeErrorToken("failedDojoLoadInit"), e]);
					}finally{
						for(p in syncLoaderApi){
							dojo[p] = hold[p];
						}
					}

					// requireList is the list of modules that need to be downloaded but not executed before the callingModule can be executed
					requireList.length && deps.push("dojo/require!" + requireList.join(","));

					dojoRequireCallbacks.push(loaded);
					array.forEach(requireList, function(mid){
						var module = getModule(mid, require.module);
						dojoRequireModuleStack.push(module);
						injectModule(module);
					});
					checkDojoRequirePlugin();
				});
			});
		},

		extractApplication = function(
			text,             // the text to search
			startSearch,      // the position in text to start looking for the closing paren
			startApplication  // the position in text where the function application expression starts
		){
			// find end of the call by finding the matching end paren
			// Warning: as usual, this will fail in the presense of unmatched right parans contained in strings, regexs, or unremoved comments
			var parenRe = /\(|\)/g,
				matchCount = 1,
				match;
			parenRe.lastIndex = startSearch;
			while((match = parenRe.exec(text))){
				if(match[0] == ")"){
					matchCount -= 1;
				}else{
					matchCount += 1;
				}
				if(matchCount == 0){
					break;
				}
			}

			if(matchCount != 0){
				throw "unmatched paren around character " + parenRe.lastIndex + " in: " + text;
			}

			//Put the master matching string in the results.
			return [dojo.trim(text.substring(startApplication, parenRe.lastIndex))+";\n", parenRe.lastIndex];
		},

		// the following regex is taken from 1.6. It is a very poor technique to remove comments and
		// will fail in some cases; for example, consider the code...
		//
		//    var message = "Category-1 */* Category-2";
		//
		// The regex that follows will see a /* comment and trash the code accordingly. In fact, there are all
		// kinds of cases like this with strings and regexs that will cause this design to fail miserably.
		//
		// Alternative regex designs exist that will result in less-likely failures, but will still fail in many cases.
		// The only solution guaranteed 100% correct is to parse the code and that seems overkill for this
		// backcompat/unbuilt-xdomain layer. In the end, since it's been this way for a while, we won't change it.
		// See the opening paragraphs of Chapter 7 or ECME-262 which describes the lexical abiguity further.
		removeCommentRe = /(\/\*([\s\S]*?)\*\/|\/\/(.*)$)/mg,

		syncLoaderApiRe = /(^|\s)dojo\.(loadInit|require|provide|requireLocalization|requireIf|requireAfterIf|platformRequire)\s*\(/mg,

		amdLoaderApiRe = /(^|\s)(require|define)\s*\(/m,

		extractLegacyApiApplications = function(text, noCommentText){
			// scan the noCommentText for any legacy loader API applications. Copy such applications into result (this is
			// used by the builder). Move dojo.loadInit applications to loadInitApplications string. Copy all other applications
			// to otherApplications string. If no applications were found, return 0, signalling an AMD module. Otherwise, return
			// loadInitApplications + otherApplications. Fixup text by replacing
			//
			//   dojo.loadInit(// etc...
			//
			// with
			//
			//   \n 0 && dojo.loadInit(// etc...
			//
			// Which results in the dojo.loadInit from *not* being applied. This design goes a long way towards protecting the
			// code from an over-agressive removeCommentRe. However...
			//
			// WARNING: the removeCommentRe will cause an error if a detected comment removes all or part of a legacy-loader application
			// that is not in a comment.

			var match, startSearch, startApplication, application,
				loadInitApplications = [],
				otherApplications = [],
				allApplications = [];

			// noCommentText may be provided by a build app with comments extracted by a better method than regex (hopefully)
			noCommentText = noCommentText || text.replace(removeCommentRe, function(match){
				// remove iff the detected comment has text that looks like a sync loader API application; this helps by
				// removing as little as possible, minimizing the changes the janky regex will kill the module
				syncLoaderApiRe.lastIndex = amdLoaderApiRe.lastIndex = 0;
				return (syncLoaderApiRe.test(match) || amdLoaderApiRe.test(match)) ? "" : match;
			});

			// find and extract all dojo.loadInit applications
			while((match = syncLoaderApiRe.exec(noCommentText))){
				startSearch = syncLoaderApiRe.lastIndex;
				startApplication = startSearch  - match[0].length;
				application = extractApplication(noCommentText, startSearch, startApplication);
				if(match[2]=="loadInit"){
					loadInitApplications.push(application[0]);
				}else{
					otherApplications.push(application[0]);
				}
				syncLoaderApiRe.lastIndex = application[1];
			}
			allApplications = loadInitApplications.concat(otherApplications);
			if(allApplications.length || !amdLoaderApiRe.test(noCommentText)){
				// either there were some legacy loader API applications or there were no AMD API applications
				return [text.replace(/(^|\s)dojo\.loadInit\s*\(/g, "\n0 && dojo.loadInit("), allApplications.join(""), allApplications];
			}else{
				// legacy loader API *was not* detected and AMD API *was* detected; therefore, assume it's an AMD module
				return 0;
			}
		},

		transformToAmd = function(module, text){
			// This is roughly the equivalent of dojo._xdCreateResource in 1.6-; however, it expresses a v1.6- dojo
			// module in terms of AMD define instead of creating the dojo proprietary xdomain module expression.
			// The module could have originated from several sources:
			//
			//   * amd require() a module, e.g., require(["my/module"])
			//   * amd require() a nonmodule, e.g., require(["my/resource.js"')
			//   * amd define() deps vector (always a module)
			//   * dojo.require() a module, e.g. dojo.require("my.module")
			//   * dojo.require() a nonmodule, e.g., dojo.require("my.module", true)
			//   * dojo.requireIf/requireAfterIf/platformRequire a module
			//
			// The module is scanned for legacy loader API applications; if none are found, then assume the module is an
			// AMD module and return 0. Otherwise, a synthetic dojo/loadInit plugin resource is created and the module text
			// is rewritten as an AMD module with the single dependency of this synthetic resource. When the dojo/loadInit
			// plugin loaded the synthetic resource, it will cause all dojo.loadInit's to be executed, find all dojo.require's
			// (either directly consequent to dojo.require or indirectly consequent to dojo.require[After]If or
			// dojo.platformRequire, and finally cause loading of all dojo.required modules with the dojo/require plugin. Thus,
			// when the dojo/loadInit plugin reports it has been loaded, all modules required by the given module are guaranteed
			// loaded (but not executed). This then allows the module to execute it's code path without interupts, thereby
			// following the synchronous code path.

			var extractResult, id, names = [], namesAsStrings = [];
			if(buildDetectRe.test(text) || !(extractResult = extractLegacyApiApplications(text))){
				// buildDetectRe.test(text) => a built module, always AMD
				// extractResult==0 => no sync API
				return 0;
			}

			// manufacture a synthetic module id that can never be a real mdule id (just like require does)
			id = module.mid + "-*loadInit";

			// construct the dojo/loadInit names vector which causes any relocated names to be defined as lexical variables under their not-relocated name
			// the dojo/loadInit plugin assumes the first name in names is "dojo"

			for(var p in getModule("dojo", module).result.scopeMap){
				names.push(p);
				namesAsStrings.push('"' + p + '"');
			}

			// rewrite the module as a synthetic dojo/loadInit plugin resource + the module expressed as an AMD module that depends on this synthetic resource
			return "// xdomain rewrite of " + module.path + "\n" +
				"define('" + id + "',{\n" +
				"\tnames:" + dojo.toJson(names) + ",\n" +
				"\tdef:function(" + names.join(",") + "){" + extractResult[1] + "}" +
				"});\n\n" +
			    "define(" + dojo.toJson(names.concat(["dojo/loadInit!"+id])) + ", function(" + names.join(",") + "){\n" + extractResult[0] + "});";
		},

		loaderVars = require.initSyncLoader(dojoRequirePlugin, checkDojoRequirePlugin, transformToAmd),

		sync =
			loaderVars.sync,

		xd =
			loaderVars.xd,

		arrived =
			loaderVars.arrived,

		nonmodule =
			loaderVars.nonmodule,

		executing =
			loaderVars.executing,

		executed =
			loaderVars.executed,

		syncExecStack =
			loaderVars.syncExecStack,

		modules =
			loaderVars.modules,

		execQ =
			loaderVars.execQ,

		getModule =
			loaderVars.getModule,

		injectModule =
			loaderVars.injectModule,

		setArrived =
			loaderVars.setArrived,

		signal =
			loaderVars.signal,

		finishExec =
			loaderVars.finishExec,

		execModule =
			loaderVars.execModule,

		getLegacyMode =
			loaderVars.getLegacyMode;

	dojo.provide = function(mid){
		var executingModule = syncExecStack[0],
			module = lang.mixin(getModule(slashName(mid), require.module), {
				executed:executing,
				result:lang.getObject(mid, true)
			});
		setArrived(module);
		if(executingModule){
			(executingModule.provides || (executingModule.provides = [])).push(function(){
				module.result = lang.getObject(mid);
				delete module.provides;
				module.executed!==executed && finishExec(module);
			});
		}// else dojo.provide called not consequent to loading; therefore, give up trying to publish module value to loader namespace
		return module.result;
	};

	has.add("config-publishRequireResult", 1, 0, 0);

	dojo.require = function(moduleName, omitModuleCheck) {
		//	summary:
		//		loads a Javascript module from the appropriate URI
		//
		//	moduleName: String
		//		module name to load, using periods for separators,
		//		 e.g. "dojo.date.locale".  Module paths are de-referenced by dojo's
		//		internal mapping of locations to names and are disambiguated by
		//		longest prefix. See `dojo.registerModulePath()` for details on
		//		registering new modules.
		//
		//	omitModuleCheck: Boolean?
		//		if `true`, omitModuleCheck skips the step of ensuring that the
		//		loaded file actually defines the symbol it is referenced by.
		//		For example if it called as `dojo.require("a.b.c")` and the
		//		file located at `a/b/c.js` does not define an object `a.b.c`,
		//		and exception will be throws whereas no exception is raised
		//		when called as `dojo.require("a.b.c", true)`
		//
		//	description:
		// 		Modules are loaded via dojo.require by using one of two loaders: the normal loader
		// 		and the xdomain loader. The xdomain loader is used when dojo was built with a
		// 		custom build that specified loader=xdomain and the module lives on a modulePath
		// 		that is a whole URL, with protocol and a domain. The versions of Dojo that are on
		// 		the Google and AOL CDNs use the xdomain loader.
		//
		// 		If the module is loaded via the xdomain loader, it is an asynchronous load, since
		// 		the module is added via a dynamically created script tag. This
		// 		means that dojo.require() can return before the module has loaded. However, this
		// 		should only happen in the case where you do dojo.require calls in the top-level
		// 		HTML page, or if you purposely avoid the loader checking for dojo.require
		// 		dependencies in your module by using a syntax like dojo["require"] to load the module.
		//
		// 		Sometimes it is useful to not have the loader detect the dojo.require calls in the
		// 		module so that you can dynamically load the modules as a result of an action on the
		// 		page, instead of right at module load time.
		//
		// 		Also, for script blocks in an HTML page, the loader does not pre-process them, so
		// 		it does not know to download the modules before the dojo.require calls occur.
		//
		// 		So, in those two cases, when you want on-the-fly module loading or for script blocks
		// 		in the HTML page, special care must be taken if the dojo.required code is loaded
		// 		asynchronously. To make sure you can execute code that depends on the dojo.required
		// 		modules, be sure to add the code that depends on the modules in a dojo.addOnLoad()
		// 		callback. dojo.addOnLoad waits for all outstanding modules to finish loading before
		// 		executing.
		//
		// 		This type of syntax works with both xdomain and normal loaders, so it is good
		// 		practice to always use this idiom for on-the-fly code loading and in HTML script
		// 		blocks. If at some point you change loaders and where the code is loaded from,
		// 		it will all still work.
		//
		// 		More on how dojo.require
		//		`dojo.require("A.B")` first checks to see if symbol A.B is
		//		defined. If it is, it is simply returned (nothing to do).
		//
		//		If it is not defined, it will look for `A/B.js` in the script root
		//		directory.
		//
		//		`dojo.require` throws an exception if it cannot find a file
		//		to load, or if the symbol `A.B` is not defined after loading.
		//
		//		It returns the object `A.B`, but note the caveats above about on-the-fly loading and
		// 		HTML script blocks when the xdomain loader is loading a module.
		//
		//		`dojo.require()` does nothing about importing symbols into
		//		the current namespace.  It is presumed that the caller will
		//		take care of that.
		//
		// 	example:
		// 		To use dojo.require in conjunction with dojo.ready:
		//
		//		|	dojo.require("foo");
		//		|	dojo.require("bar");
		//	   	|	dojo.addOnLoad(function(){
		//	   	|		//you can now safely do something with foo and bar
		//	   	|	});
		//
		//	example:
		//		For example, to import all symbols into a local block, you might write:
		//
		//		|	with (dojo.require("A.B")) {
		//		|		...
		//		|	}
		//
		//		And to import just the leaf symbol to a local variable:
		//
		//		|	var B = dojo.require("A.B");
		//	   	|	...
		//
		//	returns:
		//		the required namespace object
		function doRequire(mid, omitModuleCheck){
			var module = getModule(slashName(mid), require.module);
			if(syncExecStack.length && syncExecStack[0].finish){
				// switched to async loading in the middle of evaluating a legacy module; stop
				// applying dojo.require so the remaining dojo.requires are applied in order
				syncExecStack[0].finish.push(mid);
				return undefined;
			}

			// recall module.executed has values {0, executing, executed}; therefore, truthy indicates executing or executed
			if(module.executed){
				return module.result;
			}
			omitModuleCheck && (module.result = nonmodule);

			var currentMode = getLegacyMode();

			// recall, in sync mode to inject is to *eval* the module text
			// if the module is a legacy module, this is the same as executing
			// but if the module is an AMD module, this means defining, not executing
			injectModule(module);
			// the inject may have changed the mode
			currentMode = getLegacyMode();

			// in sync mode to dojo.require is to execute
			if(module.executed!==executed && module.injected===arrived){
				// the module was already here before injectModule was called probably finishing up a xdomain
				// load, but maybe a module given to the loader directly rather than having the loader retrieve it
				loaderVars.holdIdle();
				execModule(module);
				loaderVars.releaseIdle();
			}
			if(module.executed){
				return module.result;
			}

			if(currentMode==sync){
				// the only way to get here is in sync mode and dojo.required a module that
				//   * was loaded async in the injectModule application a few lines up
				//   * was an AMD module that had deps that are being loaded async and therefore couldn't execute
				if(module.cjs){
					// the module was an AMD module; unshift, not push, which causes the current traversal to be reattempted from the top
					execQ.unshift(module);
				}else{
					// the module was a legacy module
					syncExecStack.length && (syncExecStack[0].finish= [mid]);
				}
			}else{
				// the loader wasn't in sync mode on entry; probably async mode; therefore, no expectation of getting
				// the module value synchronously; make sure it gets executed though
				execQ.push(module);
			}
			return undefined;
		}

		var result = doRequire(moduleName, omitModuleCheck);
		if(has("config-publishRequireResult") && !lang.exists(moduleName) && result!==undefined){
			lang.setObject(moduleName, result);
		}
		return result;
	};

	dojo.loadInit = function(f) {
		f();
	};

	dojo.registerModulePath = function(/*String*/moduleName, /*String*/prefix){
		//	summary:
		//		Maps a module name to a path
		//	description:
		//		An unregistered module is given the default path of ../[module],
		//		relative to Dojo root. For example, module acme is mapped to
		//		../acme.  If you want to use a different module name, use
		//		dojo.registerModulePath.
		//	example:
		//		If your dojo.js is located at this location in the web root:
		//	|	/myapp/js/dojo/dojo/dojo.js
		//		and your modules are located at:
		//	|	/myapp/js/foo/bar.js
		//	|	/myapp/js/foo/baz.js
		//	|	/myapp/js/foo/thud/xyzzy.js
		//		Your application can tell Dojo to locate the "foo" namespace by calling:
		//	|	dojo.registerModulePath("foo", "../../foo");
		//		At which point you can then use dojo.require() to load the
		//		modules (assuming they provide() the same things which are
		//		required). The full code might be:
		//	|	<script type="text/javascript"
		//	|		src="/myapp/js/dojo/dojo/dojo.js"></script>
		//	|	<script type="text/javascript">
		//	|		dojo.registerModulePath("foo", "../../foo");
		//	|		dojo.require("foo.bar");
		//	|		dojo.require("foo.baz");
		//	|		dojo.require("foo.thud.xyzzy");
		//	|	</script>

		var paths = {};
		paths[moduleName.replace(/\./g, "/")] = prefix;
		require({paths:paths});
	};

	dojo.platformRequire = function(/*Object*/modMap){
		//	summary:
		//		require one or more modules based on which host environment
		//		Dojo is currently operating in
		//	description:
		//		This method takes a "map" of arrays which one can use to
		//		optionally load dojo modules. The map is indexed by the
		//		possible dojo.name_ values, with two additional values:
		//		"default" and "common". The items in the "default" array will
		//		be loaded if none of the other items have been choosen based on
		//		dojo.name_, set by your host environment. The items in the
		//		"common" array will *always* be loaded, regardless of which
		//		list is chosen.
		//	example:
 		//		|	dojo.platformRequire({
		//		|		browser: [
		//		|			"foo.sample", // simple module
		//		|			"foo.test",
		//		|			["foo.bar.baz", true] // skip object check in _loadModule (dojo.require)
		//		|		],
		//		|		default: [ "foo.sample._base" ],
		//		|		common: [ "important.module.common" ]
		//		|	});

		var result = (modMap.common || []).concat(modMap[dojo._name] || modMap["default"] || []),
			temp;
		while(result.length){
			if(lang.isArray(temp = result.shift())){
				dojo.require.apply(dojo, temp);
			}else{
				dojo.require(temp);
			}
		}
	};

	dojo.requireIf = dojo.requireAfterIf = function(/*Boolean*/ condition, /*String*/ moduleName, /*Boolean?*/omitModuleCheck){
		// summary:
		//		If the condition is true then call `dojo.require()` for the specified
		//		resource
		//
		// example:
		//	|	dojo.requireIf(dojo.isBrowser, "my.special.Module");

		if(condition){
			dojo.require(moduleName, omitModuleCheck);
		}
	};

	dojo.requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale){
		require(["../i18n"], function(i18n){
			i18n.getLocalization(moduleName, bundleName, locale);
		});
	};

	return {
		extractLegacyApiApplications:extractLegacyApiApplications,
		require:loaderVars.dojoRequirePlugin,
		loadInit:dojoLoadInitPlugin
	};
});

},
'com.ibm.bpm.coach/dynamicBindingHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// do more test on unbind when we plan to expose this dynamic binding capability as Coach public API
define(["require", "dojo/_base/lang", "dojo/_base/array", "./CONSTANTS", "./commonHelper", "./dataBindingHelper", "./bindingResolver"], function(require, lang, array, CONSTANTS, commonHelper, dataBindingHelper, bindingResolver) {
	
	var dynamicBindingHelper = new Object();
	
	/*	
	 Dynamic bind
	 example of bindingStr: 
	 "local.name", "local.contactBook.owner", 
	 "local.contactBook.phones", "local.contactBook.contacts[index]", "local.contactBook.contacts[index].name", 
	 "local.contactBook.contacts[index].phones[]", "local.contactBook.contacts[index].phones[index].number",
	 "local.contactBook.contacts.listSelected", "local.contactBook.contacts.listSelected.phones.listSelected.name", "local.contactBook.contacts.listSelected.phones[index].name",
	 "config.config1.myOption", "config.config2._metadata.label", "config.config3.myOption.p1",
	 "config.config3.myListOption[index]", "config.config3.myListOption[index].p1",
	 "config.config3.myListOption.listSelected", "config.config3.myListOption.listSelected.p2"
	 
	 "local.contactBook.phones[]" is acceptable, it is interpreted as the recommended notation "local.contactBook.phones". Those 2 notations mean that the change on list itself.
	 "local.contactBook.phones[index]" means the change (add/delete/replace) at List item level.

	  When callback is invoked, input parameter event object is expected to be readonly, should not be modified by callback function.
	 */
	dynamicBindingHelper.bind = function(bindingStr, callback, scope) {
		if(!bindingStr || !callback) {
			return;
		}
			
		if(!!scope) {
			callback = lang.hitch(scope, callback); 
		}
		
		var sourcePathArray = bindingStr.split(".");
		var lastPath = sourcePathArray[sourcePathArray.length - 1];

		// lastPath that ends with "[]" (e.g. "local.contactBook.phones[]") is translated as bare list (e.g. "local.contactBook.phones")
		if(lastPath.indexOf("[]") != -1) {
			sourcePathArray[sourcePathArray.length - 1] = lastPath.replace("[]", "");
			bindingStr = sourcePathArray.join(".");
		}
		
		var handle = {
			"_type": CONSTANTS.HANDLE_TYPE_DYNAMIC,
			"bindingStr": bindingStr,
			"callback": callback, 
			"unbind": function() {
				dynamicBindingHelper.unbindOne(handle);
			}
		};
		
		dynamicBindingHelper._addCallback(bindingStr, callback);

		var bindingPathArray = dynamicBindingHelper._computePaths(bindingStr);
		// bindingStr should always starts with "local"
		dynamicBindingHelper._processBindingPaths(dynamicBindingHelper.rootNode, bindingStr, bindingPathArray, 1);
		
		return handle;		
	}
	
	// perform better than dynamicBindingHelper.unbindOne() in general
	dynamicBindingHelper.unbindMultiple = function(handleArray) {
		if(!!handleArray) {
			var uselessBindingStrArray = [];
			for(var k=0; k<handleArray.length; k++) {
				var uselessBindingStr = dynamicBindingHelper._removeCallback(handleArray[k].bindingStr, handleArray[k].callback);
				if(uselessBindingStr) {
					uselessBindingStrArray.push(uselessBindingStr);
				}
			}
			
			if(uselessBindingStrArray.length > 0) {
				dynamicBindingHelper.rootNode.cleanup(uselessBindingStrArray);
			}
		}
	}
	
	dynamicBindingHelper.unbindOne = function(handle) {
		var uselessBindingStr = dynamicBindingHelper._removeCallback(handle.bindingStr, handle.callback);
		if(uselessBindingStr) {
			dynamicBindingHelper.rootNode.cleanup([uselessBindingStr]);
		}
	}

	// sensitive to the internal tree structure
	// example 1
	// input parameter is: "local.a.b[index].c". output array is: ["root", "local", "a", "b", "$LIST_ELEMENTS$", "b[index]", "c"]
	// example 2
	// input parameter is: 	"config.config1.myOption". output array is: ["root", "config", "config1", "myOption", "value"]
	// example 3
	// input parameter is: 	"config.config2._metadata.label". output array is: ["root", "config", "config2", "_metadata", "label", "value"]
	// example 4
	// input parameter is: 	"config.config3.myOption.p1". output array is: ["root", "config", "config3", "myOption", "value", "p1"]
	// example 5
	// input parameter is: 	"config.config3.myListOption[index].p1". output array is: ["root", "config", "config3", "myListOption", "value", "$LIST_ELEMENTS$", "value[index], "p1"]
	// 					["root", "config", "config3", "myListOption", "$LIST_ELEMENTS$", "myListOption[index], "p1"]
	
	dynamicBindingHelper._computePaths = function(bindingStr) {
		var sourcePathArray = bindingStr.split(".");
		var finalPathArray = [];
		finalPathArray.push(dynamicBindingHelper.VIRTUAL_PATH_ROOT);
		for(var k=0; k<sourcePathArray.length; k++) {
			onePath = sourcePathArray[k];
			
			if(onePath.indexOf(dynamicBindingHelper.INDEX_WITH_BRACKET) != -1) {
				// e.g. a[currentItem]
				// add 2 additional paths to form the internal tree structure
				var virtualPathForList = onePath.replace(dynamicBindingHelper.INDEX_WITH_BRACKET, "");
				finalPathArray.push(virtualPathForList);
				var virtualPathForListContent = dynamicBindingHelper.VIRTUAL_PATH_LIST_CONTENT;				
				finalPathArray.push(virtualPathForListContent);			
			}
			
			finalPathArray.push(onePath);
		}
		
		return finalPathArray;
	}
	
	// provided event is native event received as the result of bind() on Stateful object or bindAll() on List 	
	dynamicBindingHelper._invokeCallback = function(treeNode, event) {
		var callbackList = dynamicBindingHelper.bindingStrToCallbackListMap[treeNode.bindingStr];
		if(!!callbackList) {
			var rtBindingStr = treeNode.computeRtBindingStr();
			if(treeNode.getType() == dynamicBindingHelper.NODE_TYPE_LIST_CONTENT) {
				// change occurs at NODE_TYPE_LIST_CONTENT node, it must be add/delete/set action on list item. need to figure out the index
				if(event.insertedInto != -1) {
					rtBindingStr += "[" + event.insertedInto + "]";
				} else if(event.removedFrom != -1) {
					rtBindingStr += "[" + event.removedFrom + "]";
				}
			}
			for(var k=0; k<callbackList.length; k++) {			
				try {
					var clonedEvent = commonHelper.shallowCloneMap(event);
					clonedEvent.path = rtBindingStr;
					// if this kind of binding capability is exposed as public API in the future, need to monitor the performance of custom script					
					callbackList[k].call(null, clonedEvent);
				} catch(ex) {
					commonHelper.consoleLog(ex, commonHelper.ERROR);
				}
			}
		}
	}
	
	// event objects inside bindingStrToEventGroupMap are artificial events created by dynamicBindingHelper based on the native event
	dynamicBindingHelper._invokeCallback2 = function(bindingStrToEventGroupMap) {
		var bindingStrArray = [];
		for(var bindingStr in bindingStrToEventGroupMap) {
			bindingStrArray.push(bindingStr);
		}
		
		if(bindingStrArray.length == 0) {
			// no event to deliver
			return;
		}
		
		bindingStrArray.sort();
		// depth first
		bindingStrArray.reverse();
		
		for(var i=0; i<bindingStrArray.length; i++) {
			var bindingStr = bindingStrArray[i];
			var callbackList = dynamicBindingHelper.bindingStrToCallbackListMap[bindingStr];
			var rtBindingStrToEventMap = bindingStrToEventGroupMap[bindingStr];
			for(var rtBindingStr in rtBindingStrToEventMap) {
				var event = rtBindingStrToEventMap[rtBindingStr];
				var isBoundObjectSame = dynamicBindingHelper._isBoundObjectSame(event._oldResolvedVar, event._newResolvedVar);
				if(!isBoundObjectSame) {
					event._isResolvedVarChanged = true;
				}
				if(commonHelper.isValuesEqual(event.oldVal, event.newVal) != commonHelper.YES || !isBoundObjectSame) {
					for(var k=0; k<callbackList.length; k++) {			
						try {
							// if this kind of binding capability is exposed as public API in the future, need to monitor the performance of custom script					
							callbackList[k].call(null, event);
						} catch(ex) {
							commonHelper.consoleLog(ex, commonHelper.ERROR);
						}
					}
				}
			}
		}
	}
	
	dynamicBindingHelper._isBoundObjectSame = function(resolvedVar1, resolvedVar2) {
		// if neither resolvedVar1 nor resolvedVar2 exists, treat as they are same
		if(!resolvedVar1 && !resolvedVar2) {
			return true;
		}
		
		if(resolvedVar1 && resolvedVar2) {
			return resolvedVar1.boundObject == resolvedVar2.boundObject;
		}
		return false;
	}

	// sensitive to the internal tree structure
	// example of pathArray: 
	// 			["root", "local", "a", "b", "$LIST_ELEMENTS$", "b[index]", "c"]
	// Note: no need to watch b[index] since its parent node calls bindAll() to take care of its children
	dynamicBindingHelper._processBindingPaths = function(parentTreeNode, bindingStr, pathArray, index) {
		if(index >= pathArray.length) {
			return;
		}

		// _processBindingPaths() is called from many places, e.g. bind(), rebuildAllChildren(), listChanged()
		// we need to make sure that there are solid ground for _processBindingPaths() to execute.
		// For example, when parentTreeNode is NODE_TYPE_LIST_SELECTED and actually no selection yet, _processBindingPaths() should stop asap.
		if(parentTreeNode.getType() == dynamicBindingHelper.NODE_TYPE_LIST_SELECTED) {
			var resolvedVariable =  parentTreeNode.getResolvedBinding();		
			var bindingValue = resolvedVariable.get("value");
			if(!bindingValue) {
				// no selection on the list yet. can't continue. Once "listSelected" is available, we will continue to build this branch
				return;
			}
		}
		
		var onePath = pathArray[index];
			
		if(onePath == dynamicBindingHelper.VIRTUAL_PATH_LIST_CONTENT) {
			// This is the virtual layer that is under list node and holds all list elements
						
			var matchedNode = parentTreeNode.getChildNodeByPath(onePath);
			if(!matchedNode) {
				matchedNode = new TreeNode(parentTreeNode, onePath);
				matchedNode.setType(dynamicBindingHelper.NODE_TYPE_LIST_CONTENT);
			}

			// current path is VIRTUAL_PATH_LIST_CONTENT which is always followed by path "xyz[index]".
			if(index == (pathArray.length - 2)) {
				// VIRTUAL_PATH_LIST_CONTENT node is responsible for the changes on List item level (add/delete/replace)
				matchedNode.watch(bindingStr);
				// we have already handled VIRTUAL_PATH_LIST_CONTENT level, and level "xyz[index]" is the last path. 
				// No need to create node for "xyz[index]" level because VIRTUAL_PATH_LIST_CONTENT is already responsible for the changes at List item level
				return;
			}
			
			// VIRTUAL_PATH_LIST_CONTENT node is responsible for the changes on List item level (add/delete/replace), so need to watch this node
			matchedNode.watch();
			
			if(!matchedNode.firstChild) {
				// need to try to expend the List, because there are other path after "xyz[index]"
				var resolvedBinding = parentTreeNode.getResolvedBinding();
				// resolvedBinding is undefined for following scenario:
				// bindingStr is "local.outerList[index].nameValuePairList[index].name". "local.outerList" contains null as one of its items (let's say index is 1).
				// So the variable instance like "local.outerList[1].nameValuePairList" is not resolvable, as the result, resolvedBinding is undefined.
				if(resolvedBinding) {
					var bindingList = resolvedBinding.get("value");
					// bindingList is undefined for following scenario:
					// bindingStr is "local.outerList[index].nameValuePairList[index].name". "local.outerList" contains empty object {} as one of its items (let's say index is 1). Which is not correct, but it happens.
					// So the variable instance like "local.outerList[1].nameValuePairList" doesn't have value, as the result, bindingList is undefined.
					if(bindingList) {
						var pathOfListItemNode = parentTreeNode.path + dynamicBindingHelper.INDEX_WITH_BRACKET;
						for(var k=0; k<bindingList.length(); k++) {
							new TreeNode(matchedNode, pathOfListItemNode);
						}
					}
				}
			}
			
			var oneNode = matchedNode.firstChild; // oneNode represents list item node
			// matchedNode's child nodes (list items) have been created, and no need to watch on those nodes because 
			// VIRTUAL_PATH_LIST_CONTENT node already takes care of the changes(replace) on list item. so jump over one level. 
			// This is important, otherwise may end up duplicated binding				
			var newIndex = index + 2;
			while(!!oneNode) {
				dynamicBindingHelper._processBindingPaths(oneNode, bindingStr, pathArray, newIndex);
				oneNode = oneNode.next;
			}
		} else {
			var matchedNode = parentTreeNode.getChildNodeByPath(onePath);
			if(!matchedNode) {
				matchedNode = new TreeNode(parentTreeNode, onePath);
			}
			
			if(onePath == "listSelected") {
				// make sure it is "listSelected" of a real list.
				var parentResolvedVariable =  parentTreeNode.getResolvedBinding();		
				var parentBindingValue = parentResolvedVariable.get("value");
				if(parentBindingValue instanceof com.ibm.bpm.coach.List) {
					// yes, "listSelected" of a list
					matchedNode.setType(dynamicBindingHelper.NODE_TYPE_LIST_SELECTED);
										
					var resolvedVariable = parentTreeNode.getResolvedBinding();
					// cache the plainValue of dynamicBindingHelper.NODE_TYPE_LIST_SELECTED node
					matchedNode.plainValue = resolvedVariable.get("value")[matchedNode.path];
				}
			}
			
			if(index == (pathArray.length - 1)) {
				// Last path
				matchedNode.watch(bindingStr);
				return;
			}

			// we still need to watch this node in order to automatically rebind the children nodes when this node is changed
			// scenario: bidingStr is a.b.c, so c is watched explicitly.  Need to watch a and b internally, as the result, when a or b is changed, we have chance to rebind c
			// don't watch following nodes: root, root.local, root.config, root.config.configXYZ,  root.config.configXYZ._metadata
			if(index > 1) {
				// already filtered out root, root.local, root.config
				if(pathArray[1] != "config") {
					matchedNode.watch();
				} else {
					if(index > 2 && onePath != "_metadata") {
						matchedNode.watch();
					}
				}
			}			
			
			dynamicBindingHelper._processBindingPaths(matchedNode, bindingStr, pathArray, index+1);
		}			
	}
	
	dynamicBindingHelper._addCallback = function(bindingStr, callback) {
		var callbackList = dynamicBindingHelper.bindingStrToCallbackListMap[bindingStr];
		if(!callbackList) {
			callbackList = [];
			dynamicBindingHelper.bindingStrToCallbackListMap[bindingStr] = callbackList;
		}
		callbackList.push(callback);
	}
	
	// returns specified bindingStr if that bindingStr doesn't have any callback
	dynamicBindingHelper._removeCallback = function(bindingStr, callback) {
		var isBindingStrUseless = false;
		var callbackArray = dynamicBindingHelper.bindingStrToCallbackListMap[bindingStr];
		if(!!callbackArray) {
			var index = array.indexOf(callbackArray, callback);
			if(index != -1) {
				callbackArray.splice(index, 1);
				if(callbackArray.length == 0) {
					isBindingStrUseless = true;
					dynamicBindingHelper.bindingStrToCallbackListMap[bindingStr] = null;
					delete dynamicBindingHelper.bindingStrToCallbackListMap[bindingStr];
				}
			}
		}
		
		if(isBindingStrUseless) {
			return bindingStr;
		} else {
			return null;
		}
	}


	/*
		treeNode structure:
		{
			path:			"abc",
			bindingStr:		"local.xyz.abc",
			resolvedVar:	resolvedVariable,
			nativeBindingHandle: bindingHandle,
			parent:			parentTreeNode,
			next:			nextSibling,
			previous:		previousSibling,
			firstChild:		firstChildTreeNode,
			lastChild:		lastChildTreeNode,
		}
	*/
	// index is optional
	var TreeNode = function(parentTreeNode, path, index) {
		this.parent = parentTreeNode;
		this.path = path;
		
		if(parentTreeNode) {			
			if(!parentTreeNode.firstChild) {
				// this has to be the first child, even if index is specified with a non-zero value
				parentTreeNode.firstChild = this;
				parentTreeNode.lastChild = this;
				this.previous = null;
				this.next = null;
			} else {
				if(index == 0) {
					var myNext = parentTreeNode.firstChild;
					parentTreeNode.firstChild = this;
					this.previous = null;
					this.next = myNext;
				} else if(!!index){
					// try to put the new node at the desired index
					var nodeAtIndex = parentTreeNode.firstChild;
					for(var k=0; k<index; k++) {
						nodeAtIndex = nodeAtIndex.next;
						if(!nodeAtIndex) {
							break;
						}
					}
					
					if(!!nodeAtIndex) {
						var previousNode = nodeAtIndex.previous;
						previousNode.next = this;
						this.next = nodeAtIndex;
						nodeAtIndex.previous = this;
					} else {
						// specified index is out of bound. add this as the last child of parent node
						parentTreeNode.lastChild.next = this;
						this.previous = parentTreeNode.lastChild;
						parentTreeNode.lastChild = this;
						this.next = null;						
					}
					
				} else {
					// index is not specified, add this as the last child of parent node
					parentTreeNode.lastChild.next = this;
					this.previous = parentTreeNode.lastChild;
					parentTreeNode.lastChild = this;
					this.next = null;
				}
			}
		}		
	}
	
	// dispose this object
	TreeNode.prototype.dispose = function() {
		this.parent = null;
		this.previous = null;
		this.next = null;
		this.firstChild = null;
		this.lastChild = null;

		this.unwatch();
		this.resolvedVar = null;
		this.path = null;
		this.bindingStr = null;
	}
	
	// get resolved binding.
	TreeNode.prototype.getResolvedBinding = function() {
		if(!this.resolvedVar) {
			var rtBindingStr = this.computeRtBindingStr();
			this.resolvedVar =  bindingResolver.resolveBinding(rtBindingStr);
		}
		return this.resolvedVar;
	}
	
	// add native binding listener
	// if bindingStr is specified, means that the node is watched for the user of dynamicBindingHelper.bind()
	// if bindingStr is not specified, means that the node is watched for internal purpose, 
	// e.g. rebind (a.b.c, a is replaced, need to know that change in order to rebind on c), bind (e.g. new item is added to list)
	TreeNode.prototype.watch = function(bindingStr) {
		if(!this.bindingStr && !!bindingStr) {
			this.bindingStr = bindingStr;
		}
		
		if(!this.nativeBindingHandle) {
			var resolvedVariable = this.getResolvedBinding();		
			// bind on it if exists
			if(!!resolvedVariable) {
				if(this.getType() == dynamicBindingHelper.NODE_TYPE_LIST_CONTENT) {
					// only NODE_TYPE_LIST_CONTENT can use bindAll to minitor the change at list item level
					var bindingValue = resolvedVariable.get("value");
					if(bindingValue instanceof com.ibm.bpm.coach.List) {
						this.nativeBindingHandle = bindingValue.bindAll(this.listChange, this);
					}				
				} else {
					if(!!resolvedVariable.property) {
						this.nativeBindingHandle = resolvedVariable.boundObject.bind(resolvedVariable.property, this.propertyChange, this);
					}
				}
			}		
		}
	}
	
	// remove native binding listener
	TreeNode.prototype.unwatch = function() {
		this.bindingStr = null;
		
		if(!!this.nativeBindingHandle) {
			try {
				this.nativeBindingHandle.unbind();
			} catch(ex) {
		    	commonHelper.consoleLog("dynamicBindingHelper.TreeNode.unwatch(), failed to unbind the binding handler, Exception: " + ex, commonHelper.ERROR);
			}
		}
		this.nativeBindingHandle = null;
	}
	
	// sensitive to the internal tree structure
	TreeNode.prototype.computeRtBindingStr = function(index) {
		var pathArray = [];
		var oneNode = this;
		var onePath;
		
		if(!oneNode.parent) {
			// this is root node which has no binding
			return null;
		}
		
		do {
			if(oneNode.getType() == dynamicBindingHelper.NODE_TYPE_LIST_CONTENT) {
				// I am artificial node that groups the list items
				pathArray.push(oneNode.parent.path);
				// skip one level above: list node
				oneNode = oneNode.parent;
			} else if(!!oneNode.path) {			
				if(oneNode.path.indexOf(dynamicBindingHelper.INDEX_WITH_BRACKET) != -1) {
					// e.g. a[index]
					// figure out my position in the list
					var index = oneNode.getIndex();
					var rtPath = oneNode.path.replace(dynamicBindingHelper.INDEX_WITH_BRACKET, "[" + index + "]");
					pathArray.push(rtPath);
					// skip 2 levels above: VIRTUAL_PATH_LIST_CONTENT node and List node
					oneNode = oneNode.parent.parent;	
				} else {
					pathArray.push(oneNode.path);
				}
			}
			oneNode = oneNode.parent;
		} while(!!oneNode.parent);
		
		return pathArray.reverse().join(".");
	}
	
	// sensitive to the internal tree structure
	TreeNode.prototype.computeBindingStr = function() {
		var pathArray = [];
		var oneNode = this;
		var onePath;
		
		if(!oneNode.parent) {
			// this is root node which has no binding
			return null;
		}
		
		do {
			if(oneNode.getType() == dynamicBindingHelper.NODE_TYPE_LIST_CONTENT) {
				// I am artificial node that groups the list items
				pathArray.push(oneNode.parent.path + dynamicBindingHelper.INDEX_WITH_BRACKET);
				// skip one level above: list node
				oneNode = oneNode.parent;
			} else if(!!oneNode.path) {			
				pathArray.push(oneNode.path);
				if(oneNode.path.indexOf(dynamicBindingHelper.INDEX_WITH_BRACKET) != -1) {
					// e.g. a[index]
					// skip 2 levels above: VIRTUAL_PATH_LIST_CONTENT node and List node
					oneNode = oneNode.parent.parent;	
				}
			}
			oneNode = oneNode.parent;
		} while(!!oneNode.parent);
		
		return pathArray.reverse().join(".");
	}
	
	
	// return the position of this node among its siblings
	TreeNode.prototype.getIndex = function() {
		var index = 0;
		var tempNode = this;
		while(!!tempNode.previous) {
			index++;
			tempNode = tempNode.previous;
		}

		return index;
	}

	TreeNode.prototype.getChildNodeByIndex = function(index) {
		var childNode = this.firstChild;
		for(var k=0; k<index; k++) {
			if(!!childNode) {
				childNode = childNode.next;
			} else {
				return null;
			}
		}
		
		return childNode;
	}

	// get the first child node that matches specified path
	TreeNode.prototype.getChildNodeByPath = function(path) {
		var oneNode = this.firstChild;
		while(!!oneNode) {
			if(oneNode.path == path) {
				return oneNode;
			}
			oneNode = oneNode.next;
		}
		return null;
	}
	
	// remove one child node
	TreeNode.prototype.removeChild = function(childNode) {
		//
		childNode.removeAllChildren();
		
		var leftSide = childNode.previous;
		var rightSide = childNode.next;
		
		if(!!leftSide) {
			leftSide.next = rightSide;
		} else {
			// childNode is the first child
			this.firstChild = rightSide;
		}
		
		if(!!rightSide) {
			rightSide.previous = leftSide;
		} else {
			// childNode is the last child
			this.lastChild = leftSide;
		}
		
		// now, no node links to childNode
		childNode.dispose();
	}

	// remove all children nodes
	TreeNode.prototype.removeAllChildren = function() {
		var childToRemove = this.firstChild;
		
		while(!!childToRemove) {
			childToRemove.removeAllChildren();
			var nextOne = childToRemove.next;
			childToRemove.dispose();
						
			childToRemove = nextOne;
		}
		
		this.firstChild = null;
		this.lastChild = null;
	}
	
	// Since some bindingStr are not used any more (as the result of unbind action), clean up the branch start from myself
	TreeNode.prototype.cleanup = function(uselessBindingStrArray) {
		// firstly check if my branch needs to take any action
		myPath = this.path;
		var bindingStrToCheck = this.bindingStr ? this.bindingStr : this.computeBindingStr();
		
		var worthToContinue = false;
		if(this == dynamicBindingHelper.rootNode) {
			// I am root node which is just an artificial node and doesn't match to any binding path. So need to go head to check my child nodes.
			worthToContinue = true;
		} else {
			for(var k=0; k<uselessBindingStrArray.length; k++) {
				if(commonHelper.startsWithPath(uselessBindingStrArray[k], bindingStrToCheck, true)) {
					worthToContinue = true;
					break;
				}
			}
		}
		
		if(!worthToContinue) {
			return;
		}
		
		// cleanup children node 
		var childNode = this.firstChild;		
		while(!!childNode) {
			var temp = childNode.next;
			childNode.cleanup(uselessBindingStrArray);
			childNode = temp;
		}
		
		// cleanup myself
		
		// // check if need to remove this.bindingStr. the node may still be watched after removing this.bindingStr. for example, in order to support rebind
		if(this.bindingStr && array.indexOf(uselessBindingStrArray, this.bindingStr) != -1) {
			this.bindingStr = null;
		}
		
		if(this.firstChild) {
			// I still have child node, so I have reason to stay alive
			return;
		}
		
		if(this.bindingStr) {
			// I still have this.bindingStr, means that some real callback relies on me. So stay alive
			return;
		}
				
		if(this.getType == dynamicBindingHelper.NODE_TYPE_LIST_CONTENT) {
			// I am artificial node that groups List items.
			// I don't have child, don't have this.bindingStr, but I should be alive as long as somebody still need me to watch the change (add/delete/replace) on list item level.
			for(var subscribedBindingStr in dynamicBindingHelper.bindingStrToCallbackListMap) {
				if(commonHelper.startsWithPath(subscribedBindingStr, bindingStrToCheck, true)) {
					return;
				}
			}
		}
		
		// I have no value any more, ask parent to remove me
		if(this.parent) {
			this.parent.removeChild(this); // removeChild() calls unwatch() implicitly
		}
	}
	
	TreeNode.prototype.setType = function(type) {
		this.type = type;
	}
	TreeNode.prototype.getType = function() {
		return this.type;
	}
	
	TreeNode.prototype.getLevel = function() {
		var myLevel = 0;
		var temp = this;
		while(!!temp.parent) {
			myLevel++;
			temp = temp.parent;
		}
		return myLevel;
	}
	
	// sensitive to the internal tree structure
	/*
	 skipNode: if a node is skipped, plainValue is passed down to its children nodes directly.
	           node is skipped for 2 scenarios - a) starting node, because it has native event already. b) dynamicBindingHelper.VIRTUAL_PATH_LIST_CONTENT node because nobody could bind on it
	 plainValue: is used to find the value for event.oldVal or event.newVal 
	 bindingStrToEventGroupMap: a map between bindingStr and an object which is the map between rtBindingStr and changeEvent.
	 isForOldVar: if it is true, collect old value for the change event. Otherwise, collect new value for the change event.
	*/
	TreeNode.prototype.collectInfoForChangeEvents = function(skipNode, plainValue, bindingStrToEventGroupMap, isForOldVar, indexIfListItem) {
		// handle this node firstly
		if(!skipNode) {
			if(this.bindingStr && this.path != dynamicBindingHelper.VIRTUAL_PATH_LIST_CONTENT) {
				// work out a property change event
				var rtBindingStr = this.computeRtBindingStr();
				if(!bindingStrToEventGroupMap[this.bindingStr]) {
					bindingStrToEventGroupMap[this.bindingStr] = {};
				}
				if(!bindingStrToEventGroupMap[this.bindingStr][rtBindingStr]) {
					bindingStrToEventGroupMap[this.bindingStr][rtBindingStr] = {path: rtBindingStr, oldVal: null, newVal: null, property: this.path, _origin: "system", _oldResolvedVar: null, _newResolvedVar: null};
				}
				
				if(isForOldVar) {
					bindingStrToEventGroupMap[this.bindingStr][rtBindingStr].oldVal = plainValue;
					bindingStrToEventGroupMap[this.bindingStr][rtBindingStr]._oldResolvedVar = this.resolvedVar;
				} else {
					bindingStrToEventGroupMap[this.bindingStr][rtBindingStr].newVal = plainValue;
					bindingStrToEventGroupMap[this.bindingStr][rtBindingStr]._newResolvedVar = this.resolvedVar;
				}
			} else if((this.path.indexOf(dynamicBindingHelper.INDEX_WITH_BRACKET) != -1) && this.parent.bindingStr) {
				// work out a list item change event (add/delete/replace list item)
				var bindingStr = this.parent.bindingStr;
				var rtBindingStr = this.computeRtBindingStr();
				if(!bindingStrToEventGroupMap[bindingStr]) {
					bindingStrToEventGroupMap[bindingStr] = {};
				}
				if(!bindingStrToEventGroupMap[bindingStr][rtBindingStr]) {
					bindingStrToEventGroupMap[bindingStr][rtBindingStr] = {path: rtBindingStr, oldVal: null, newVal: null, insertedInto: -1, removedFrom: -1, _origin: "system"};
				}
				
				if(isForOldVar) {
					bindingStrToEventGroupMap[bindingStr][rtBindingStr].oldVal = plainValue;
					bindingStrToEventGroupMap[bindingStr][rtBindingStr].removedFrom = indexIfListItem;
					bindingStrToEventGroupMap[bindingStr][rtBindingStr]._oldResolvedVar = this.resolvedVar;
				} else {
					bindingStrToEventGroupMap[bindingStr][rtBindingStr].newVal = plainValue;
					bindingStrToEventGroupMap[bindingStr][rtBindingStr].insertedInto = indexIfListItem;
					bindingStrToEventGroupMap[bindingStr][rtBindingStr]._newResolvedVar = this.resolvedVar;
				}
			}
		}
		
		// handle the children nodes
		var child = this.firstChild;
		
		while(!!child) {	
			var isListContentNode = (child.path == dynamicBindingHelper.VIRTUAL_PATH_LIST_CONTENT); 
			var childPlainValue = null;
			var childIndex = null;
			if(!!plainValue) {
				if(isListContentNode) {
					childPlainValue = plainValue.items;
				} else if(child.path.indexOf(dynamicBindingHelper.INDEX_WITH_BRACKET) != -1) {
					// e.g. a[index]
					// figure out my position in the list
					childIndex = child.getIndex();
					// plainValue is expected to be an Array
					childPlainValue = plainValue[childIndex];
				} else {
					// property of complex object
					childPlainValue = plainValue[child.path];
				}
			}
			
			if(child.getType() == dynamicBindingHelper.NODE_TYPE_LIST_SELECTED && !childPlainValue) {
				// native change event doesn't contain the value for listSelected. To work around it, dynamicBindingHelper.NODE_TYPE_LIST_SELECTED tree node caches the plain value by itself
				childPlainValue	= child.plainValue;
			}
			
			child.collectInfoForChangeEvents(isListContentNode, childPlainValue, bindingStrToEventGroupMap, isForOldVar, childIndex);
			child = child.next;
		}
	}
	
	TreeNode.prototype.rebuildAllChidren = function() {
		this.removeAllChildren();

		var bindingStr = !!this.bindingStr ? this.bindingStr : this.computeBindingStr();
		
		var affectedBindingStrArray = [];
		for(var subscribedBindingStr in dynamicBindingHelper.bindingStrToCallbackListMap) {
			if(commonHelper.startsWithPath(subscribedBindingStr, bindingStr, true)) {
				affectedBindingStrArray.push(subscribedBindingStr);
			}
		}
		if(affectedBindingStrArray.length > 0) {
			var nextLevel = this.getLevel() + 1;
			for(var k=0; k<affectedBindingStrArray.length; k++) {
				var bindingPathArray = dynamicBindingHelper._computePaths(affectedBindingStrArray[k]);
				dynamicBindingHelper._processBindingPaths(this, affectedBindingStrArray[k], bindingPathArray, nextLevel);
			}
		}
	}
	
	// sensitive to the internal tree structure
	TreeNode.prototype.propertyChange = function(event) {
		if(event.property == "value" && event.property != this.path) {
			// sometimes (e.g. binding is on config) event.property is "value" although what has been changed really has a meaningful name.
			event.property = this.path;
		}
		
		// before removing the children node, walk though entire branch to collect the information (mainly event.oldVal) for the possible change event
		var bindingStrToEventGroupMap = {};
		this.collectInfoForChangeEvents(true, event.oldVal, bindingStrToEventGroupMap, true);
	
		// this node is changed. If this node has chidren nodes, the native listener on the children nodes need to be refreshed.
		this.rebuildAllChidren();
					
		// now the new branch is well established. walk though entire branch to collect the information (mainly event.NewVar) for the possible change event
		this.collectInfoForChangeEvents(true, event.newVal, bindingStrToEventGroupMap, false);
		
		// fire the change events per this structure change
		dynamicBindingHelper._invokeCallback2(bindingStrToEventGroupMap);

		if(this.bindingStr) {
			dynamicBindingHelper._invokeCallback(this, event);
		}
		
		if(this.getType() == dynamicBindingHelper.NODE_TYPE_LIST_SELECTED) {
			// cache the plainValue for dynamicBindingHelper.NODE_TYPE_LIST_SELECTED node
			this.plainValue = event.newVal;
		}
	}
	
	// sensitive to the internal tree structure
	// listener for List change, "this" means a VIRTUAL_PATH_LIST_CONTENT node
	TreeNode.prototype.listChange = function(changeEvent) {
		// bindingStrToEventGroupMap is responsible for storing event information
		var bindingStrToEventGroupMap = {};
		if(changeEvent.removedFrom >= 0 && changeEvent.insertedInto >= 0 && changeEvent.removedFrom == changeEvent.insertedInto) {
			// set action on List item
			var nodeToUpdate = this.getChildNodeByIndex(changeEvent.removedFrom);
			
			if(nodeToUpdate) {
				// before removing the children, walk though entire branch to collect the information (mainly event.oldVal) for the possible change event
				nodeToUpdate.collectInfoForChangeEvents(true, changeEvent.oldVal, bindingStrToEventGroupMap, true);
	
				// nodeToUpdate is changed. If it has chidren nodes, the native listener on the children nodes need to be refreshed.
				nodeToUpdate.rebuildAllChidren();
				
				// now the new branch is well established. walk though entire branch to collect the information (mainly event.NewVar) for the possible change event
				nodeToUpdate.collectInfoForChangeEvents(true, changeEvent.newVal, bindingStrToEventGroupMap, false);
			}
		} else {
			if(changeEvent.removedFrom >= 0) { 
				// one item has been removed from the list.								
				var nodeToRemove = this.getChildNodeByIndex(changeEvent.removedFrom);
				
				if(nodeToRemove) {
					// before removing the child, walk though entire branch headed by this child node to collect the information for the possible change event
					nodeToRemove.collectInfoForChangeEvents(true, changeEvent.oldVal, bindingStrToEventGroupMap, true);
	
					this.removeChild(nodeToRemove);
				}
			}
			
			if(changeEvent.insertedInto >= 0) {
				// one item has been added to the list.	
				// need to check if the future change on some node of this new branch needs to be watched
				
				var listNode = this.parent;
				// the path for the newly added list item
				var path = listNode.path + dynamicBindingHelper.INDEX_WITH_BRACKET;
				var grandparentNodeBindingStr = !!listNode.parent.bindingStr ? listNode.parent.bindingStr : listNode.parent.computeBindingStr();
				// the prefix of bindingStr for the newly added list item
				var prefix = grandparentNodeBindingStr + "." + path;
				
				var affectedBindingStrArray = [];
				for(var subscribedBindingStr in dynamicBindingHelper.bindingStrToCallbackListMap) {
					// prefix is something like local.a.b[index] which is already monitored by VIRTUAL_PATH_LIST_CONTENT node.
					// so, if subscribedBindingStr is same as prefix, don't need to handle it since "this" (VIRTUAL_PATH_LIST_CONTENT node) is responsible for this kind of binding
					if(commonHelper.startsWithPath(subscribedBindingStr, prefix)) {
						affectedBindingStrArray.push(subscribedBindingStr);
					}
				}
				
				var newNode = null;
				if(affectedBindingStrArray.length > 0) {
					// need to handle the new branch
					// the parent node for the new node is myself, VIRTUAL_PATH_LIST_CONTENT node
					var parantForNewNode = this;
					newNode = new TreeNode(parantForNewNode, path, changeEvent.insertedInto);
					var nextLevel = newNode.getLevel() + 1;
					for(var k=0; k<affectedBindingStrArray.length; k++) {
						var bindingPathArray = dynamicBindingHelper._computePaths(affectedBindingStrArray[k]);
						dynamicBindingHelper._processBindingPaths(newNode, affectedBindingStrArray[k], bindingPathArray, nextLevel);
					}
				}
				if(newNode) {
					// now the new branch is well established. walk though entire branch to collect the information (mainly event.NewVar) for the possible change event
					newNode.collectInfoForChangeEvents(true, changeEvent.newVal, bindingStrToEventGroupMap, false);
				}			
			}
		}

		// fire the change events per this structure change
		dynamicBindingHelper._invokeCallback2(bindingStrToEventGroupMap);

		if(this.bindingStr) {
			// if this.bindingStr exists, means there is at least one callback
			dynamicBindingHelper._invokeCallback(this, changeEvent);
		}				
	}
		
	dynamicBindingHelper.NODE_TYPE_LIST_SELECTED = "TYPE_LIST_SELECTED";
	dynamicBindingHelper.NODE_TYPE_LIST_CONTENT = "TYPE_LIST_CONTENT";

	dynamicBindingHelper.INDEX_WITH_BRACKET = CONSTANTS.INDEX_WITH_BRACKET;
	dynamicBindingHelper.VIRTUAL_PATH_LIST_CONTENT = "$LIST_ELEMENTS$";
	dynamicBindingHelper.VIRTUAL_PATH_ROOT = "$ROOT$";
	
	dynamicBindingHelper.rootNode = new TreeNode(null, dynamicBindingHelper.VIRTUAL_PATH_ROOT);
	
	dynamicBindingHelper.bindingStrToCallbackListMap = {};
	
	return dynamicBindingHelper;
});

},
'dojo/mouse':function(){
define(["./_base/kernel", "./on", "./has", "./dom", "./_base/window"], function(dojo, on, has, dom, win){

	/*=====
	dojo.mouse = {
	// summary:
	//		This module provide mouse event handling utility functions and exports
	//		mouseenter and mouseleave event emulation.
	// enter: Synthetic Event
	//		This is an extension event for the mouseenter that IE provides, emulating the
	//		behavior on other browsers.
	// leave: Synthetic Event
	//		This is an extension event for the mouseleave that IE provides, emulating the
	//		behavior on other browsers.
	// isLeft: Function
	//		Test an event object (from a mousedown event) to see if the left button was pressed.
	// isMiddle: Function
	//		Test an event object (from a mousedown event) to see if the middle button was pressed.
	// isRight: Function
	//		Test an event object (from a mousedown event) to see if the right button was pressed.
	// example:
	//		To use these events, you register a mouseenter like this:
	//		|	define(["dojo/on", dojo/mouse"], function(on, mouse){
	//		|		on(targetNode, mouse.enter, function(event){
	//		|			dojo.addClass(targetNode, "highlighted");
	//		|		});
	//		|		on(targetNode, mouse.leave, function(event){
	//		|			dojo.removeClass(targetNode, "highlighted");
	//		|		});
	};
	======*/

    has.add("dom-quirks", win.doc && win.doc.compatMode == "BackCompat");
 	has.add("events-mouseenter", win.doc && "onmouseenter" in win.doc.createElement("div"));

	var mouseButtons;
	if((has("dom-quirks") && has("ie")) || !has("dom-addeventlistener")){
		mouseButtons = {
			LEFT:   1,
			MIDDLE: 4,
			RIGHT:  2,
			// helper functions
			isButton: function(e, button){ return e.button & button; },
			isLeft:   function(e){ return e.button & 1; },
			isMiddle: function(e){ return e.button & 4; },
			isRight:  function(e){ return e.button & 2; }
		};
	}else{
		mouseButtons = {
			LEFT:   0,
			MIDDLE: 1,
			RIGHT:  2,
			// helper functions
			isButton: function(e, button){ return e.button == button; },
			isLeft:   function(e){ return e.button == 0; },
			isMiddle: function(e){ return e.button == 1; },
			isRight:  function(e){ return e.button == 2; }
		};
	}
	dojo.mouseButtons = mouseButtons;

/*=====
	dojo.mouseButtons = {
		// LEFT: Number
		//		Numeric value of the left mouse button for the platform.
		LEFT:   0,
		// MIDDLE: Number
		//		Numeric value of the middle mouse button for the platform.
		MIDDLE: 1,
		// RIGHT: Number
		//		Numeric value of the right mouse button for the platform.
		RIGHT:  2,

		isButton: function(e, button){
			// summary:
			//		Checks an event object for a pressed button
			// e: Event
			//		Event object to examine
			// button: Number
			//		The button value (example: dojo.mouseButton.LEFT)
			return e.button == button; // Boolean
		},
		isLeft: function(e){
			// summary:
			//		Checks an event object for the pressed left button
			// e: Event
			//		Event object to examine
			return e.button == 0; // Boolean
		},
		isMiddle: function(e){
			// summary:
			//		Checks an event object for the pressed middle button
			// e: Event
			//		Event object to examine
			return e.button == 1; // Boolean
		},
		isRight: function(e){
			// summary:
			//		Checks an event object for the pressed right button
			// e: Event
			//		Event object to examine
			return e.button == 2; // Boolean
		}
	};
=====*/

	function eventHandler(type, mustBubble){
		// emulation of mouseenter/leave with mouseover/out using descendant checking
		var handler = function(node, listener){
			return on(node, type, function(evt){
				if(!dom.isDescendant(evt.relatedTarget, mustBubble ? evt.target : node)){
					return listener.call(this, evt);
				}
			});
		};
		if(!mustBubble){
			handler.bubble = eventHandler(type, true);
		}
		return handler;
	}
	return {
		enter: eventHandler("mouseover"),
		leave: eventHandler("mouseout"),
		isLeft: mouseButtons.isLeft,
		isMiddle: mouseButtons.isMiddle,
		isRight: mouseButtons.isRight
	};
});

},
'dijit/a11y':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/config", // defaultDuration
	"dojo/_base/declare", // declare
	"dojo/dom",			// dom.byId
	"dojo/dom-attr", // domAttr.attr domAttr.has
	"dojo/dom-style", // style.style
	"dojo/_base/sniff", // has("ie")
	"./_base/manager",	// manager._isElementShown
	"."	// for exporting methods to dijit namespace
], function(array, config, declare, dom, domAttr, domStyle, has, manager, dijit){

	// module:
	//		dijit/a11y
	// summary:
	//		Accessibility utility functions (keyboard, tab stops, etc.)

	var shown = (dijit._isElementShown = function(/*Element*/ elem){
		var s = domStyle.get(elem);
		return (s.visibility != "hidden")
			&& (s.visibility != "collapsed")
			&& (s.display != "none")
			&& (domAttr.get(elem, "type") != "hidden");
	});

	dijit.hasDefaultTabStop = function(/*Element*/ elem){
		// summary:
		//		Tests if element is tab-navigable even without an explicit tabIndex setting

		// No explicit tabIndex setting, need to investigate node type
		switch(elem.nodeName.toLowerCase()){
			case "a":
				// An <a> w/out a tabindex is only navigable if it has an href
				return domAttr.has(elem, "href");
			case "area":
			case "button":
			case "input":
			case "object":
			case "select":
			case "textarea":
				// These are navigable by default
				return true;
			case "iframe":
				// If it's an editor <iframe> then it's tab navigable.
				var body;
				try{
					// non-IE
					var contentDocument = elem.contentDocument;
					if("designMode" in contentDocument && contentDocument.designMode == "on"){
						return true;
					}
					body = contentDocument.body;
				}catch(e1){
					// contentWindow.document isn't accessible within IE7/8
					// if the iframe.src points to a foreign url and this
					// page contains an element, that could get focus
					try{
						body = elem.contentWindow.document.body;
					}catch(e2){
						return false;
					}
				}
				return body && (body.contentEditable == 'true' ||
					(body.firstChild && body.firstChild.contentEditable == 'true'));
			default:
				return elem.contentEditable == 'true';
		}
	};

	var isTabNavigable = (dijit.isTabNavigable = function(/*Element*/ elem){
		// summary:
		//		Tests if an element is tab-navigable

		// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
		if(domAttr.get(elem, "disabled")){
			return false;
		}else if(domAttr.has(elem, "tabIndex")){
			// Explicit tab index setting
			return domAttr.get(elem, "tabIndex") >= 0; // boolean
		}else{
			// No explicit tabIndex setting, so depends on node type
			return dijit.hasDefaultTabStop(elem);
		}
	});

	dijit._getTabNavigable = function(/*DOMNode*/ root){
		// summary:
		//		Finds descendants of the specified root node.
		//
		// description:
		//		Finds the following descendants of the specified root node:
		//		* the first tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the last tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the first element in document order with the lowest
		//		  positive tabIndex value
		//		* the last element in document order with the highest
		//		  positive tabIndex value
		var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};

		function radioName(node){
			// If this element is part of a radio button group, return the name for that group.
			return node && node.tagName.toLowerCase() == "input" &&
				node.type && node.type.toLowerCase() == "radio" &&
				node.name && node.name.toLowerCase();
		}

		var walkTree = function(/*DOMNode*/parent){
			for(var child = parent.firstChild; child; child = child.nextSibling){
				// Skip text elements, hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
				// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
				if(child.nodeType != 1 || (has("ie") <= 9 && child.scopeName !== "HTML") || !shown(child)){
					continue;
				}

				if(isTabNavigable(child)){
					var tabindex = domAttr.get(child, "tabIndex");
					if(!domAttr.has(child, "tabIndex") || tabindex == 0){
						if(!first){
							first = child;
						}
						last = child;
					}else if(tabindex > 0){
						if(!lowest || tabindex < lowestTabindex){
							lowestTabindex = tabindex;
							lowest = child;
						}
						if(!highest || tabindex >= highestTabindex){
							highestTabindex = tabindex;
							highest = child;
						}
					}
					var rn = radioName(child);
					if(domAttr.get(child, "checked") && rn){
						radioSelected[rn] = child;
					}
				}
				if(child.nodeName.toUpperCase() != 'SELECT'){
					walkTree(child);
				}
			}
		};
		if(shown(root)){
			walkTree(root);
		}
		function rs(node){
			// substitute checked radio button for unchecked one, if there is a checked one with the same name.
			return radioSelected[radioName(node)] || node;
		}

		return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
	};
	dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is first in the tabbing order
		var elems = dijit._getTabNavigable(dom.byId(root));
		return elems.lowest ? elems.lowest : elems.first; // DomNode
	};

	dijit.getLastInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is last in the tabbing order
		var elems = dijit._getTabNavigable(dom.byId(root));
		return elems.last ? elems.last : elems.highest; // DomNode
	};

	return {
		hasDefaultTabStop: dijit.hasDefaultTabStop,
		isTabNavigable: dijit.isTabNavigable,
		_getTabNavigable: dijit._getTabNavigable,
		getFirstInTabbingOrder: dijit.getFirstInTabbingOrder,
		getLastInTabbingOrder: dijit.getLastInTabbingOrder
	};
});

},
'dojo/regexp':function(){
define(["./_base/kernel", "./_base/lang"], function(dojo, lang) {
	// module:
	//		dojo/regexp
	// summary:
	//		TODOC

lang.getObject("regexp", true, dojo);

/*=====
dojo.regexp = {
	// summary: Regular expressions and Builder resources
};
=====*/

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
};

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression.
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
};

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression.
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
};

return dojo.regexp;
});

},
'com.ibm.bpm.coach/bindingResolver':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// 

define(["dojo/_base/lang", "./commonHelper"], function(lang, commonHelper) {
				
	var bindingResolver = {};
	
	bindingResolver.init = function(globalAssets) {
		bindingResolver.globalAssets = globalAssets;
	}

	//##Data Binding start
	bindingResolver.resolveBinding = function(bindingString) {
		//Resolves a binding string in the form of BO.property

		if (bindingString == null || bindingString == "") return;

		var result;
		var refArray = bindingString.split(".");
		if (refArray[0] == "local") {
			if (!(bindingResolver.globalAssets["local"] instanceof com.ibm.bpm.coach.Stateful)) {
				bindingResolver.globalAssets["local"] = new com.ibm.bpm.coach.Stateful(bindingResolver.globalAssets["local"], "local");
			}
			refArray.splice(0, 1);
			result = bindingResolver._resolveBindingObject(bindingResolver.globalAssets["local"] , refArray);			
		}
		else if (bindingString.indexOf("config") == 0){
			//bind to parent's options, resolve it
			result = bindingResolver._resolveStaticOptionBinding(bindingString);
		}	
		else {
			//handle system, resource, epv
			//add a setter, as required for consistency, but no callback is registered
			//if we want to notify change, we will make them as stateful so we do not need to maintain the callback functions
			var get = function (propName) {return (propName=="value")?this.value:null};
			var set = function (propName, val) {
				var errorMsg = "bindingResolver.resolveBinding(), set method is not supported on: " + bindingString;
				commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			};

			var bindingString1 = bindingString.substring(refArray[0].length);
			var value = eval("this.globalAssets['"+refArray[0]+"']" + bindingString1);
			result = {value: value, get: get, set: set};
		}
		return result;
	}

	bindingResolver.resolveOptionsBinding = function(options) {
		for(var optionKey in options) {	
		    if (optionKey == "_metadata"){
				for (var metadataKey in options[optionKey]){
					bindingResolver._resolveOptionBinding(options, metadataKey, true);
				}
			}
			else{			
			  bindingResolver._resolveOptionBinding(options, optionKey, false);
			}		
		}	
	}	
		
	bindingResolver._resolveBindingObject = function (objectRoot, refArray){
		var parentObject = objectRoot;
		var boundObject = objectRoot;
		var prop = null;
		for (var i=0; i<refArray.length; i++) {
			parentObject = boundObject;
			if (!boundObject){
				//for binding like  local.myList.listSelected.pro1, if the list is empty, listSelected will be undefined
		//		logger.log("bindingResolver", "_resolveBindingObject ", "boundObject is undefined. property:" + refArray[i-1]);
				break;
			}
			if (refArray[i].indexOf("[]") != -1) {
				refArray[i] = refArray[i].substring(0, refArray[i].indexOf("[]"));
			}
			var subMatch = refArray[i].match(/\[.*\]/);
			var index = null;
			if (subMatch != null) {
				index = subMatch[0].substring(1, subMatch[0].length-1);
				refArray[i] = refArray[i].substring(0, subMatch.index);
			}
			prop = refArray[i];
			if(index == null) {
				boundObject = boundObject.get(refArray[i]);
			} else {
				parentObject = boundObject.get(refArray[i]);
				boundObject = boundObject.get(refArray[i]).get(index);
				prop = index;
			}
		}
		var set;
		var get;
		if ( parentObject instanceof com.ibm.bpm.coach.List && (parseInt(prop) >= 0)) {
			var id = parentObject.indexToId[prop];
			set = function(propName, val) {(propName=="value")? this.boundObject.set(id, val):0};
			get = function (propName) {return (propName=="value")? this.boundObject.getById(id):null};
		} else {
			set = function(propName, val) {(propName=="value")? this.boundObject.set(this.property, val):0};
			get = function (propName) {return (propName=="value")? this.boundObject.get(this.property):null};
		}
		
		var result = !!parentObject ? {boundObject: parentObject, property: prop, get: get, set: set} : undefined;
		return result;
	}

	/*
		resolve one configure option.
		this function is splited from original engine._resolveAndRegisterOptionBindingCallBack()
	*/
	bindingResolver._resolveOptionBinding = function(options, shortOptionKey, isMetadata, update, objectPath){

	        var optionValue;
			var binding;
			var optionKey = shortOptionKey;
			optionValue = options[optionKey];	
			
			if (!!isMetadata){
				optionKey = "_metadata." + shortOptionKey;
				optionValue = options["_metadata"][shortOptionKey];	
			}
			//fix a bug: the user may not give the option a value, 
			//for update case, nothing need to be done for non bo case
			if(!optionValue || !!update && !optionValue.bindingStr)
				return;
				
			//resolved previously, and not for update case
			if(!!optionValue.boundObject && !update){
				binding = optionValue;
			}
				
	         //resolve the object if it is ref option to local, epv, resource
			else {
			if(!!optionValue.bindingStr){
				var affected = (!update) || !objectPath || commonHelper.startsWithPath(optionValue.bindingStr, objectPath);
				if(affected){					
					binding = bindingResolver.resolveBinding(optionValue.bindingStr);					
					// fix JR45548
					if(!!binding) {
					//This is useful later when check the binding string of this option
					binding.bindingStr = optionValue.bindingStr;
					if(!!update){
						binding.changed = true;
					}					
					}
				}
			}
			
			else if (!!optionValue.staticOptionStr){
				//static value	
				binding = bindingResolver._resolveStaticOptionBinding (optionValue.staticOptionStr, optionValue.value);
				binding.staticOptionStr = optionValue.staticOptionStr; //to be used for clone case
				//consider the case:  option for config 2 actually pointing to option in config 1 with staticOptionStr  options.config1.showAddButton
				//the binding is stored in _resolveStaticOptionBinding. set it here too
			}
			if (!!binding){
				if((optionKey === "_metadata.visibility" || optionKey === "_metadata.labelVisibility") && !optionValue.isStandardValue){			
					binding.get = function (propName) {
						var result = (propName=="value") ? this.boundObject.get(this.property) : null;
						result = (!!result? result.toUpperCase().replace(/[^A-Z]/g,'') : result);
						return result;
					};					
				}
				//store the value
				if (!!isMetadata){
					options["_metadata"][shortOptionKey] = binding;
				}
				else
					options[shortOptionKey] = binding;
			}
			}
			
			// when the binding path is not resolvable, the object under context.options (e.g. this.context.options.myOption1, this.context.options._metadata.visibility) 
			// still hold the original plain object that contains the binding path. The plain object doesn't have get() and set() functions. 
			// However, the current practice is that stock controls (customer may copy the code from stock control) assume that this.context.options.myOption1.get function
			// exists as long as this.context.options.myOption1 exist.
			// To fix above issue, we add the dummy get() and set() function here. If the binding path becomes resolvable later, the whole wrapper object will be replaced.
			// New wrapper object will contain a functional get() and set().
			// If wrapper object is a function, do not add dummy get() and set().
			var wrapperObject = !!isMetadata ? options["_metadata"][shortOptionKey] : options[shortOptionKey];
			if (!!wrapperObject && !wrapperObject.get && !lang.isFunction(wrapperObject)){
				// dummy get() and set()
				wrapperObject.get = function(propName) {
					return;
				}
				wrapperObject.set = function(propName, val) {
					return;
				}
			}
			
			return binding;
	}
	
	bindingResolver._resolveStaticOptionBinding = function(staticOptionStr, value){
				var result;
				
				var pathArray = staticOptionStr.split(".");
				var target = pathArray[0] == "config" ? bindingResolver.globalAssets : bindingResolver.globalAssets.options;
				var parent;
				var optionKey;
				
				//the data structure property name is options( globalAssets.options;), but the object path start with config
				if (pathArray[0] == "config")
				  pathArray[0] = "options";				  
				  
				/* Path can be 
				 * config.config2.option1.prop1.name,   config.config2.option1, config.config2.option1.prop1.name, 
				 * In any case, we use .notition until we get option1, and then make option1 stateful if it is not yet
				 */
				var bindToProperties = false;
				var upTo = 3;
				if (staticOptionStr.indexOf("_metadata") > -1){
					upTo = 4;
				}

				// handle the scenario: config.config1.option1[3].name, or config.config1.option1[3]
				// option1 will be the stateful object, we need to reserve the index (e.g. 3) for further resolving.
				var planedStatefulObjectStr = pathArray[upTo - 1];
				var leftBracketPosition = planedStatefulObjectStr.indexOf("[");
				if(leftBracketPosition != -1) {
					var rightBracketPosition = planedStatefulObjectStr.indexOf("]");
					var numericValue = planedStatefulObjectStr.substring(leftBracketPosition+1, rightBracketPosition);					
					if(numericValue != "") {
						pathArray.splice(upTo, 0, numericValue);
					}
					pathArray[upTo - 1] = planedStatefulObjectStr.substring(0, leftBracketPosition);
				}
				
				bindToProperties = pathArray.length > upTo;
				
				for (var i=0; i<upTo; i++){
					parent = target;
					optionKey = pathArray[i];
					target = target[optionKey];
				}
				
				//if the option is a service, nothing need to do here. This will cover the following cases
				//1. option value is a service: code gen will generate a function
				//2. option value refer to another option whose value is a function: config.config2.myServiceOption
				if (lang.isFunction(target)){
					result = target;//it is not a real binding, just the function it self
					parent[optionKey] = result;
				}
				else{
					var binding;
					if (!!target.boundObject){
						binding = target;
					}
					else{
					//create the binding object for the option with getters and setters, so that the view can get it using this.context.options.option1.get('value')
						var boundObject = new com.ibm.bpm.coach.Stateful({value : target.value}, staticOptionStr);
						binding = bindingResolver._createBindingObject(boundObject, "value");
						
						// remember the staticOptionStr (if exists) of parent[optionKey] before assigning new object to parent[optionKey]
						var possibleStaticOptionStr = !!parent[optionKey] ? parent[optionKey].staticOptionStr : null;
						parent[optionKey] = binding;
						// if parent[optionKey].staticOptionStr exists before we assign a new object to parent[optionKey], we need to keep this staticOptionStr in new object because clone needs it.
						// when _resolveStaticOptionBinding() is called by _resolveAndRegisterOptionBindingCallBack(), staticOptionStr is set back. 
						// But _resolveStaticOptionBinding has other callers, that's why we handle here.
						if(!!possibleStaticOptionStr) {
							parent[optionKey].staticOptionStr = possibleStaticOptionStr;
						}
						
						//for system change handler: Nothing needs to be done in the Stateful object, instead
						//the controller should be able to understand it by the object path
					}
					if (bindToProperties){
						//options.config1.option1.name
						pathArray.splice(0, upTo - 1);
						pathArray[0] = "value";
						result = this._resolveBindingObject(binding.boundObject, pathArray );
					}
					else {
						result = binding;
					}
					//if the binding is to some property in the option value, ie, option value is a complex object, then we need to continue to resolve the binding
					
				}
				
			return result;
	}

	bindingResolver._createBindingObject = function(boundObject, property){
		var get = function (propName) {return ((propName=="value") ? this.boundObject.get(this.property) : null)};
		var set = function(propName, newVal) {
			if(propName =="value") {
				this.boundObject.set("value", newVal);
			}
		};
		
		//when initView ->resolvebinding, register callback on binding, if the property is null, it register on all property changes 
		var binding = {get: get, set: set, property : property, boundObject : boundObject};
	    return binding;	
	}
	
	
	return bindingResolver;
});

},
'dojo/_base/NodeList':function(){
define(["./kernel", "../query", "./array", "./html", "../NodeList-dom"], function(dojo, query, array){
  //  module:
  //    dojo/_base/NodeList
  //  summary:
  //    This module defines dojo.NodeList.
 
var NodeList = query.NodeList;

	/*=====
	dojo.extend(dojo.NodeList, {
		connect: function(methodName, objOrFunc, funcName){
			// summary:
			//		attach event handlers to every item of the NodeList. Uses dojo.connect()
			//		so event properties are normalized
			// methodName: String
			//		the name of the method to attach to. For DOM events, this should be
			//		the lower-case name of the event
			// objOrFunc: Object|Function|String
			//		if 2 arguments are passed (methodName, objOrFunc), objOrFunc should
			//		reference a function or be the name of the function in the global
			//		namespace to attach. If 3 arguments are provided
			//		(methodName, objOrFunc, funcName), objOrFunc must be the scope to
			//		locate the bound function in
			// funcName: String?
			//		optional. A string naming the function in objOrFunc to bind to the
			//		event. May also be a function reference.
			// example:
			//		add an onclick handler to every button on the page
			//		|	dojo.query("div:nth-child(odd)").connect("onclick", function(e){
			//		|		console.log("clicked!");
			//		|	});
			// example:
			//		attach foo.bar() to every odd div's onmouseover
			//		|	dojo.query("div:nth-child(odd)").connect("onmouseover", foo, "bar");
		},
		coords: function(){
			// summary:
			//		Deprecated: Use position() for border-box x/y/w/h
			//		or marginBox() for margin-box w/h/l/t.
			//		Returns the box objects of all elements in a node list as
			//		an Array (*not* a NodeList). Acts like `dojo.coords`, though assumes
			//		the node passed is each node in this list.

			return dojo.map(this, dojo.coords); // Array
		}
	 });

	 var NodeList = dojo.NodeList;
	=====*/
	var nlp = NodeList.prototype;

	// don't bind early to dojo.connect since we no longer explicitly depend on it
	nlp.connect = NodeList._adaptAsForEach(function(){
		return dojo.connect.apply(this, arguments);
	});
	nlp.coords = NodeList._adaptAsMap(dojo.coords);

	NodeList.events = [
		// summary:
		//		list of all DOM events used in NodeList
		"blur", "focus", "change", "click", "error", "keydown", "keypress",
		"keyup", "load", "mousedown", "mouseenter", "mouseleave", "mousemove",
		"mouseout", "mouseover", "mouseup", "submit"
	];

	// FIXME: pseudo-doc the above automatically generated on-event functions

	// syntactic sugar for DOM events
	array.forEach(NodeList.events, function(evt){
			var _oe = "on" + evt;
			nlp[_oe] = function(a, b){
				return this.connect(_oe, a, b);
			};
				// FIXME: should these events trigger publishes?
				/*
				return (a ? this.connect(_oe, a, b) :
							this.forEach(function(n){
								// FIXME:
								//		listeners get buried by
								//		addEventListener and can't be dug back
								//		out to be triggered externally.
								// see:
								//		http://developer.mozilla.org/en/docs/DOM:element

								console.log(n, evt, _oe);

								// FIXME: need synthetic event support!
								var _e = { target: n, faux: true, type: evt };
								// dojo._event_listener._synthesizeEvent({}, { target: n, faux: true, type: evt });
								try{ n[evt](_e); }catch(e){ console.log(e); }
								try{ n[_oe](_e); }catch(e){ console.log(e); }
							})
				);
				*/
		}
	);

	dojo.NodeList = NodeList;
	return dojo.NodeList;
});

},
'dijit/place':function(){
define([
	"dojo/_base/array", // array.forEach array.map array.some
	"dojo/dom-geometry", // domGeometry.getMarginBox domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.getBox
	"."	// dijit (defining dijit.place to match API doc)
], function(array, domGeometry, domStyle, kernel, win, winUtils, dijit){

	// module:
	//		dijit/place
	// summary:
	//		Code to place a popup relative to another node


	function _place(/*DomNode*/ node, choices, layoutNode, aroundNodeCoords){
		// summary:
		//		Given a list of spots to put node, put it at the first spot where it fits,
		//		of if it doesn't fit anywhere then the place with the least overflow
		// choices: Array
		//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
		//		Above example says to put the top-left corner of the node at (10,20)
		// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
		//		for things like tooltip, they are displayed differently (and have different dimensions)
		//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
		//		It also passes in the available size for the popup, which is useful for tooltips to
		//		tell them that their width is limited to a certain amount.	 layoutNode() may return a value expressing
		//		how much the popup had to be modified to fit into the available space.	 This is used to determine
		//		what the best placement is.
		// aroundNodeCoords: Object
		//		Size of aroundNode, ex: {w: 200, h: 50}

		// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
		// viewport over document
		var view = winUtils.getBox();

		// This won't work if the node is inside a <div style="position: relative">,
		// so reattach it to win.doc.body.	 (Otherwise, the positioning will be wrong
		// and also it might get cutoff)
		if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
			win.body().appendChild(node);
		}

		var best = null;
		array.some(choices, function(choice){
			var corner = choice.corner;
			var pos = choice.pos;
			var overflow = 0;

			// calculate amount of space available given specified position of node
			var spaceAvailable = {
				w: {
					'L': view.l + view.w - pos.x,
					'R': pos.x - view.l,
					'M': view.w
				   }[corner.charAt(1)],
				h: {
					'T': view.t + view.h - pos.y,
					'B': pos.y - view.t,
					'M': view.h
				   }[corner.charAt(0)]
			};

			// configure node to be displayed in given position relative to button
			// (need to do this in order to get an accurate size for the node, because
			// a tooltip's size changes based on position, due to triangle)
			if(layoutNode){
				var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
				overflow = typeof res == "undefined" ? 0 : res;
			}

			// get node's size
			var style = node.style;
			var oldDisplay = style.display;
			var oldVis = style.visibility;
			if(style.display == "none"){
				style.visibility = "hidden";
				style.display = "";
			}
			var mb = domGeometry. getMarginBox(node);
			style.display = oldDisplay;
			style.visibility = oldVis;

			// coordinates and size of node with specified corner placed at pos,
			// and clipped by viewport
			var
				startXpos = {
					'L': pos.x,
					'R': pos.x - mb.w,
					'M': Math.max(view.l, Math.min(view.l + view.w, pos.x + (mb.w >> 1)) - mb.w) // M orientation is more flexible
				}[corner.charAt(1)],
				startYpos = {
					'T': pos.y,
					'B': pos.y - mb.h,
					'M': Math.max(view.t, Math.min(view.t + view.h, pos.y + (mb.h >> 1)) - mb.h)
				}[corner.charAt(0)],
				startX = Math.max(view.l, startXpos),
				startY = Math.max(view.t, startYpos),
				endX = Math.min(view.l + view.w, startXpos + mb.w),
				endY = Math.min(view.t + view.h, startYpos + mb.h),
				width = endX - startX,
				height = endY - startY;

			overflow += (mb.w - width) + (mb.h - height);

			if(best == null || overflow < best.overflow){
				best = {
					corner: corner,
					aroundCorner: choice.aroundCorner,
					x: startX,
					y: startY,
					w: width,
					h: height,
					overflow: overflow,
					spaceAvailable: spaceAvailable
				};
			}

			return !overflow;
		});

		// In case the best position is not the last one we checked, need to call
		// layoutNode() again.
		if(best.overflow && layoutNode){
			layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
		}

		// And then position the node.  Do this last, after the layoutNode() above
		// has sized the node, due to browser quirks when the viewport is scrolled
		// (specifically that a Tooltip will shrink to fit as though the window was
		// scrolled to the left).
		//
		// In RTL mode, set style.right rather than style.left so in the common case,
		// window resizes move the popup along with the aroundNode.
		var l = domGeometry.isBodyLtr(),
			s = node.style;
		s.top = best.y + "px";
		s[l ? "left" : "right"] = (l ? best.x : view.w - best.x - best.w) + "px";
		s[l ? "right" : "left"] = "auto";	// needed for FF or else tooltip goes to far left

		return best;
	}

	/*=====
	dijit.place.__Position = function(){
		// x: Integer
		//		horizontal coordinate in pixels, relative to document body
		// y: Integer
		//		vertical coordinate in pixels, relative to document body

		this.x = x;
		this.y = y;
	};
	=====*/

	/*=====
	dijit.place.__Rectangle = function(){
		// x: Integer
		//		horizontal offset in pixels, relative to document body
		// y: Integer
		//		vertical offset in pixels, relative to document body
		// w: Integer
		//		width in pixels.   Can also be specified as "width" for backwards-compatibility.
		// h: Integer
		//		height in pixels.   Can also be specified as "height" from backwards-compatibility.

		this.x = x;
		this.y = y;
		this.w = w;
		this.h = h;
	};
	=====*/

	return (dijit.place = {
		// summary:
		//		Code to place a DOMNode relative to another DOMNode.
		//		Load using require(["dijit/place"], function(place){ ... }).

		at: function(node, pos, corners, padding){
			// summary:
			//		Positions one of the node's corners at specified position
			//		such that node is fully visible in viewport.
			// description:
			//		NOTE: node is assumed to be absolutely or relatively positioned.
			// node: DOMNode
			//		The node to position
			// pos: dijit.place.__Position
			//		Object like {x: 10, y: 20}
			// corners: String[]
			//		Array of Strings representing order to try corners in, like ["TR", "BL"].
			//		Possible values are:
			//			* "BL" - bottom left
			//			* "BR" - bottom right
			//			* "TL" - top left
			//			* "TR" - top right
			// padding: dijit.place.__Position?
			//		optional param to set padding, to put some buffer around the element you want to position.
			// example:
			//		Try to place node's top right corner at (10,20).
			//		If that makes node go (partially) off screen, then try placing
			//		bottom left corner at (10,20).
			//	|	place(node, {x: 10, y: 20}, ["TR", "BL"])
			var choices = array.map(corners, function(corner){
				var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
				if(padding){
					c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
					c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
				}
				return c;
			});

			return _place(node, choices);
		},

		around: function(
			/*DomNode*/		node,
			/*DomNode || dijit.place.__Rectangle*/ anchor,
			/*String[]*/	positions,
			/*Boolean*/		leftToRight,
			/*Function?*/	layoutNode){

			// summary:
			//		Position node adjacent or kitty-corner to anchor
			//		such that it's fully visible in viewport.
			//
			// description:
			//		Place node such that corner of node touches a corner of
			//		aroundNode, and that node is fully visible.
			//
			// anchor:
			//		Either a DOMNode or a __Rectangle (object with x, y, width, height).
			//
			// positions:
			//		Ordered list of positions to try matching up.
			//			* before: places drop down to the left of the anchor node/widget, or to the right in the case
			//				of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//				with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//			* after: places drop down to the right of the anchor node/widget, or to the left in the case
			//				of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//				with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//			* before-centered: centers drop down to the left of the anchor node/widget, or to the right
			//				 in the case of RTL scripts like Hebrew and Arabic
			//			* after-centered: centers drop down to the right of the anchor node/widget, or to the left
			//				 in the case of RTL scripts like Hebrew and Arabic
			//			* above-centered: drop down is centered above anchor node
			//			* above: drop down goes above anchor node, left sides aligned
			//			* above-alt: drop down goes above anchor node, right sides aligned
			//			* below-centered: drop down is centered above anchor node
			//			* below: drop down goes below anchor node
			//			* below-alt: drop down goes below anchor node, right sides aligned
			//
			// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
			//		For things like tooltip, they are displayed differently (and have different dimensions)
			//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
			//
			// leftToRight:
			//		True if widget is LTR, false if widget is RTL.   Affects the behavior of "above" and "below"
			//		positions slightly.
			//
			// example:
			//	|	placeAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
			//		This will try to position node such that node's top-left corner is at the same position
			//		as the bottom left corner of the aroundNode (ie, put node below
			//		aroundNode, with left edges aligned).	If that fails it will try to put
			// 		the bottom-right corner of node where the top right corner of aroundNode is
			//		(ie, put node above aroundNode, with right edges aligned)
			//

			// if around is a DOMNode (or DOMNode id), convert to coordinates
			var aroundNodePos = (typeof anchor == "string" || "offsetWidth" in anchor)
				? domGeometry.position(anchor, true)
				: anchor;

			// Compute position and size of visible part of anchor (it may be partially hidden by ancestor nodes w/scrollbars)
			if(anchor.parentNode){
				// ignore nodes between position:relative and position:absolute
				var sawPosAbsolute = domStyle.getComputedStyle(anchor).position == "absolute";
				var parent = anchor.parentNode;
				while(parent && parent.nodeType == 1 && parent.nodeName != "BODY"){  //ignoring the body will help performance
					var parentPos = domGeometry.position(parent, true),
						pcs = domStyle.getComputedStyle(parent);
					if(/relative|absolute/.test(pcs.position)){
						sawPosAbsolute = false;
					}
					if(!sawPosAbsolute && /hidden|auto|scroll/.test(pcs.overflow)){
						var bottomYCoord = Math.min(aroundNodePos.y + aroundNodePos.h, parentPos.y + parentPos.h);
						var rightXCoord = Math.min(aroundNodePos.x + aroundNodePos.w, parentPos.x + parentPos.w);
						aroundNodePos.x = Math.max(aroundNodePos.x, parentPos.x);
						aroundNodePos.y = Math.max(aroundNodePos.y, parentPos.y);
						aroundNodePos.h = bottomYCoord - aroundNodePos.y;
						aroundNodePos.w = rightXCoord - aroundNodePos.x;
					}
					if(pcs.position == "absolute"){
						sawPosAbsolute = true;
					}
					parent = parent.parentNode;
				}
			}


			var x = aroundNodePos.x,
				y = aroundNodePos.y,
				width = "w" in aroundNodePos ? aroundNodePos.w : (aroundNodePos.w = aroundNodePos.width),
				height = "h" in aroundNodePos ? aroundNodePos.h : (kernel.deprecated("place.around: dijit.place.__Rectangle: { x:"+x+", y:"+y+", height:"+aroundNodePos.height+", width:"+width+" } has been deprecated.  Please use { x:"+x+", y:"+y+", h:"+aroundNodePos.height+", w:"+width+" }", "", "2.0"), aroundNodePos.h = aroundNodePos.height);

			// Convert positions arguments into choices argument for _place()
			var choices = [];
			function push(aroundCorner, corner){
				choices.push({
					aroundCorner: aroundCorner,
					corner: corner,
					pos: {
						x: {
							'L': x,
							'R': x + width,
							'M': x + (width >> 1)
						   }[aroundCorner.charAt(1)],
						y: {
							'T': y,
							'B': y + height,
							'M': y + (height >> 1)
						   }[aroundCorner.charAt(0)]
					}
				})
			}
			array.forEach(positions, function(pos){
				var ltr =  leftToRight;
				switch(pos){
					case "above-centered":
						push("TM", "BM");
						break;
					case "below-centered":
						push("BM", "TM");
						break;
					case "after-centered":
						ltr = !ltr;
						// fall through
					case "before-centered":
						push(ltr ? "ML" : "MR", ltr ? "MR" : "ML");
						break;
					case "after":
						ltr = !ltr;
						// fall through
					case "before":
						push(ltr ? "TL" : "TR", ltr ? "TR" : "TL");
						push(ltr ? "BL" : "BR", ltr ? "BR" : "BL");
						break;
					case "below-alt":
						ltr = !ltr;
						// fall through
					case "below":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "BL" : "BR", ltr ? "TL" : "TR");
						push(ltr ? "BR" : "BL", ltr ? "TR" : "TL");
						break;
					case "above-alt":
						ltr = !ltr;
						// fall through
					case "above":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "TL" : "TR", ltr ? "BL" : "BR");
						push(ltr ? "TR" : "TL", ltr ? "BR" : "BL");
						break;
					default:
						// To assist dijit/_base/place, accept arguments of type {aroundCorner: "BL", corner: "TL"}.
						// Not meant to be used directly.
						push(pos.aroundCorner, pos.corner);
				}
			});

			var position = _place(node, choices, layoutNode, {w: width, h: height});
			position.aroundNodePos = aroundNodePos;

			return position;
		}
	});
});

},
'dojo/_base/kernel':function(){
define(["../has", "./config", "require", "module"], function(has, config, require, module){
	// module:
	//		dojo/_base/kernel
	// summary:
	//		This module is the foundational module of the dojo boot sequence; it defines the dojo object.
	var
		// loop variables for this module
		i, p,

		// create dojo, dijit, and dojox
		// FIXME: in 2.0 remove dijit, dojox being created by dojo
		dijit = {},
		dojox = {},
		dojo = {
			// notice dojo takes ownership of the value of the config module
			config:config,
			global:this,
			dijit:dijit,
			dojox:dojox
		};


	// Configure the scope map. For a 100% AMD application, the scope map is not needed other than to provide
	// a _scopeName property for the dojo, dijit, and dojox root object so those packages can create
	// unique names in the global space.
	//
	// Built, legacy modules use the scope map to allow those modules to be expressed as if dojo, dijit, and dojox,
	// where global when in fact they are either global under different names or not global at all. In v1.6-, the
	// config variable "scopeMap" was used to map names as used within a module to global names. This has been
	// subsumed by the dojo packageMap configuration variable which relocates packages to different names. See
	// http://livedocs.dojotoolkit.org/developer/design/loader#legacy-cross-domain-mode for details.
	//
	// The following computations contort the packageMap for this dojo instance into a scopeMap.
	var scopeMap =
			// a map from a name used in a legacy module to the (global variable name, object addressed by that name)
			// always map dojo, dijit, and dojox
			{
				dojo:["dojo", dojo],
				dijit:["dijit", dijit],
				dojox:["dojox", dojox]
			},

		packageMap =
			// the package map for this dojo instance; note, a foreign loader or no pacakgeMap results in the above default config
			(require.packs && require.packs[module.id.match(/[^\/]+/)[0]].packageMap) || {},

		item;

	// process all mapped top-level names for this instance of dojo
	for(p in packageMap){
		if(scopeMap[p]){
			// mapped dojo, dijit, or dojox
			scopeMap[p][0] = packageMap[p];
		}else{
			// some other top-level name
			scopeMap[p] = [packageMap[p], {}];
		}
	}

	// publish those names to _scopeName and, optionally, the global namespace
	for(p in scopeMap){
		item = scopeMap[p];
		item[1]._scopeName = item[0];
		if(!config.noGlobals){
			this[item[0]] = item[1];
		}
	}
	dojo.scopeMap = scopeMap;

	// FIXME: dojo.baseUrl and dojo.config.baseUrl should be deprecated
	dojo.baseUrl = dojo.config.baseUrl = require.baseUrl;
	dojo.isAsync = !1 || require.async;
	dojo.locale = config.locale;

	/*=====
		dojo.version = function(){
			// summary:
			//		Version number of the Dojo Toolkit
			// major: Integer
			//		Major version. If total version is "1.2.0beta1", will be 1
			// minor: Integer
			//		Minor version. If total version is "1.2.0beta1", will be 2
			// patch: Integer
			//		Patch version. If total version is "1.2.0beta1", will be 0
			// flag: String
			//		Descriptor flag. If total version is "1.2.0beta1", will be "beta1"
			// revision: Number
			//		The SVN rev from which dojo was pulled
			this.major = 0;
			this.minor = 0;
			this.patch = 0;
			this.flag = "";
			this.revision = 0;
		}
	=====*/
	var rev = "$Rev: 27793 $".match(/\d+/);
	dojo.version = {
		major: 1, minor: 7, patch: 4, flag: "",
		revision: rev ? +rev[0] : NaN,
		toString: function(){
			var v = dojo.version;
			return v.major + "." + v.minor + "." + v.patch + v.flag + " (" + v.revision + ")";	// String
		}
	};


	// If 1 is truthy, then as a dojo module is defined it should push it's definitions
	// into the dojo object, and conversely. In 2.0, it will likely be unusual to augment another object
	// as a result of defining a module. This has feature gives a way to force 2.0 behavior as the code
	// is migrated. Absent specific advice otherwise, set extend-dojo to truthy.
	true || has.add("extend-dojo", 1);


	dojo.eval = function(scriptText){
		//	summary:
		//		A legacy method created for use exclusively by internal Dojo methods. Do not use this method
		//		directly unless you understand its possibly-different implications on the platforms your are targeting.
		//	description:
		//		Makes an attempt to evaluate scriptText in the global scope. The function works correctly for browsers
		//		that support indirect eval.
		//
		//		As usual, IE does not. On IE, the only way to implement global eval is to
		//		use execScript. Unfortunately, execScript does not return a value and breaks some current usages of dojo.eval.
		//		This implementation uses the technique of executing eval in the scope of a function that is a single scope
		//		frame below the global scope; thereby coming close to the global scope. Note carefully that
		//
		//		dojo.eval("var pi = 3.14;");
		//
		//		will define global pi in non-IE environments, but define pi only in a temporary local scope for IE. If you want
		//		to define a global variable using dojo.eval, write something like
		//
		//		dojo.eval("window.pi = 3.14;")
		//	scriptText:
		//		The text to evaluation.
		//	returns:
		//		The result of the evaluation. Often `undefined`
	};

	(Function("d", "d.eval = function(){return d.global.eval ? d.global.eval(arguments[0]) : eval(arguments[0]);}"))(dojo);


	if(0){
		dojo.exit = function(exitcode){
			quit(exitcode);
		};
	} else{
		dojo.exit = function(){
		};
	}

	true || has.add("dojo-guarantee-console",
		// ensure that console.log, console.warn, etc. are defined
		1
	);
	if(1){
		typeof console != "undefined" || (console = {});
		//	Be careful to leave 'log' always at the end
		var cn = [
			"assert", "count", "debug", "dir", "dirxml", "error", "group",
			"groupEnd", "info", "profile", "profileEnd", "time", "timeEnd",
			"trace", "warn", "log"
		];
		var tn;
		i = 0;
		while((tn = cn[i++])){
			if(!console[tn]){
				(function(){
					var tcn = tn + "";
					console[tcn] = ('log' in console) ? function(){
						var a = Array.apply({}, arguments);
						a.unshift(tcn + ":");
						console["log"](a.join(" "));
					} : function(){};
					console[tcn]._fake = true;
				})();
			}
		}
	}

	has.add("dojo-debug-messages",
		// include dojo.deprecated/dojo.experimental implementations
		!!config.isDebug
	);
	if(has("dojo-debug-messages")){
		dojo.deprecated = function(/*String*/ behaviour, /*String?*/ extra, /*String?*/ removal){
			//	summary:
			//		Log a debug message to indicate that a behavior has been
			//		deprecated.
			//	behaviour: String
			//		The API or behavior being deprecated. Usually in the form
			//		of "myApp.someFunction()".
			//	extra: String?
			//		Text to append to the message. Often provides advice on a
			//		new function or facility to achieve the same goal during
			//		the deprecation period.
			//	removal: String?
			//		Text to indicate when in the future the behavior will be
			//		removed. Usually a version number.
			//	example:
			//	| dojo.deprecated("myApp.getTemp()", "use myApp.getLocaleTemp() instead", "1.0");

			var message = "DEPRECATED: " + behaviour;
			if(extra){ message += " " + extra; }
			if(removal){ message += " -- will be removed in version: " + removal; }
			console.warn(message);
		};

		dojo.experimental = function(/* String */ moduleName, /* String? */ extra){
			//	summary: Marks code as experimental.
			//	description:
			//		This can be used to mark a function, file, or module as
			//		experimental.	 Experimental code is not ready to be used, and the
			//		APIs are subject to change without notice.	Experimental code may be
			//		completed deleted without going through the normal deprecation
			//		process.
			//	moduleName: String
			//		The name of a module, or the name of a module file or a specific
			//		function
			//	extra: String?
			//		some additional message for the user
			//	example:
			//	| dojo.experimental("dojo.data.Result");
			//	example:
			//	| dojo.experimental("dojo.weather.toKelvin()", "PENDING approval from NOAA");

			var message = "EXPERIMENTAL: " + moduleName + " -- APIs subject to change without notice.";
			if(extra){ message += " " + extra; }
			console.warn(message);
		};
	}else{
		dojo.deprecated = dojo.experimental =  function(){};
	}

	true || has.add("dojo-modulePaths",
		// consume dojo.modulePaths processing
		1
	);
	if(1){
		// notice that modulePaths won't be applied to any require's before the dojo/_base/kernel factory is run;
		// this is the v1.6- behavior.
		if(config.modulePaths){
			dojo.deprecated("dojo.modulePaths", "use paths configuration");
			var paths = {};
			for(p in config.modulePaths){
				paths[p.replace(/\./g, "/")] = config.modulePaths[p];
			}
			require({paths:paths});
		}
	}

	true || has.add("dojo-moduleUrl",
		// include dojo.moduleUrl
		1
	);
	if(1){
		dojo.moduleUrl = function(/*String*/module, /*String?*/url){
			//	summary:
			//		Returns a URL relative to a module.
			//	example:
			//	|	var pngPath = dojo.moduleUrl("acme","images/small.png");
			//	|	console.dir(pngPath); // list the object properties
			//	|	// create an image and set it's source to pngPath's value:
			//	|	var img = document.createElement("img");
			//	|	img.src = pngPath;
			//	|	// add our image to the document
			//	|	dojo.body().appendChild(img);
			//	example:
			//		you may de-reference as far as you like down the package
			//		hierarchy.  This is sometimes handy to avoid lenghty relative
			//		urls or for building portable sub-packages. In this example,
			//		the `acme.widget` and `acme.util` directories may be located
			//		under different roots (see `dojo.registerModulePath`) but the
			//		the modules which reference them can be unaware of their
			//		relative locations on the filesystem:
			//	|	// somewhere in a configuration block
			//	|	dojo.registerModulePath("acme.widget", "../../acme/widget");
			//	|	dojo.registerModulePath("acme.util", "../../util");
			//	|
			//	|	// ...
			//	|
			//	|	// code in a module using acme resources
			//	|	var tmpltPath = dojo.moduleUrl("acme.widget","templates/template.html");
			//	|	var dataPath = dojo.moduleUrl("acme.util","resources/data.json");

			dojo.deprecated("dojo.moduleUrl()", "use require.toUrl", "2.0");

			// require.toUrl requires a filetype; therefore, just append the suffix "/*.*" to guarantee a filetype, then
			// remove the suffix from the result. This way clients can request a url w/out a filetype. This should be
			// rare, but it maintains backcompat for the v1.x line (note: dojo.moduleUrl will be removed in v2.0).
			// Notice * is an illegal filename so it won't conflict with any real path map that may exist the paths config.
			var result = null;
			if(module){
				result = require.toUrl(module.replace(/\./g, "/") + (url ? ("/" + url) : "") + "/*.*").replace(/\/\*\.\*/, "") + (url ? "" : "/");
			}
			return result;
		};
	}

	dojo._hasResource = {}; // for backward compatibility with layers built with 1.6 tooling

	return dojo;
});

},
'dojo/date':function(){
define(["./_base/kernel", "./_base/lang"], function(dojo, lang) {
	// module:
	//		dojo/date
	// summary:
	//		TODOC

lang.getObject("date", true, dojo);

/*=====
dojo.date = {
	// summary: Date manipulation utilities
}
=====*/

dojo.date.getDaysInMonth = function(/*Date*/dateObject){
	//	summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && dojo.date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
};

dojo.date.isLeapYear = function(/*Date*/dateObject){
	//	summary:
	//		Determines if the year of the dateObject is a leap year
	//	description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
};

// FIXME: This is not localized
dojo.date.getTimezoneName = function(/*Date*/dateObject){
	//	summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	//	description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string,
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
};

// Utility methods to do arithmetic calculations with Dates

dojo.date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	//	summary:
	//		Compare two date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(+date1);
	date2 = new Date(+(date2 || new Date()));

	if(portion == "date"){
		// Ignore times and compare dates.
		date1.setHours(0, 0, 0, 0);
		date2.setHours(0, 0, 0, 0);
	}else if(portion == "time"){
		// Ignore dates and compare times.
		date1.setFullYear(0, 0, 0);
		date2.setFullYear(0, 0, 0);
	}

	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

dojo.date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var sum = new Date(+date); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but this is not always true.  see dojo.cldr.supplemental

			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var days, weeks;
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			var adj = 0;
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = (7 * weeks) + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
//		case "hour":
//		case "minute":
//		case "second":
//		case "millisecond":
		default:
			property = "UTC"+interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

dojo.date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	//	summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(dojo.date.difference(date1, date2, "day"));
			var weeks = parseInt(dojo.date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojo.date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

return dojo.date;
});

},
'com.ibm.bpm.coach/collaborationPopup':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// manage the popup for collaboration

define(["dojo/query", "dijit/popup", "dijit/TooltipDialog", "dojo/dom-geometry", "dojo/window", "dojo/has", "dojo/_base/sniff"], function(query, popup, TooltipDialog, domGeometry, dojoWindow, has) {

	var popupManager = new Object();
	
	popupManager.CONSTANTS = {
		// css class names
		COLLABORATOR_INFO:	"CoachView_collaboratorInfo",
		COLLABORATOR:		"CoachView_collaborator",
		DATA_CHANGED:		"CoachView_dataChanged"
	};
	
	function scrollIntoView(node, viewNode) {
	
		_scrollIntoView(node);
		
		if(domGeometry.position(node).y == 0 && viewNode.id != node.id) {			
			// if node's y position is still 0 after we try to scrollIntoView, node could be invisible.
			// For those cases, we try on the node of coachView instead
			_scrollIntoView(viewNode);
		}	     		
	}
	
	function _scrollIntoView(node) {
/*	
	     var vs = dojoWindow.getBox();
	     console.log("node.id is " + node.id);
	     console.log("vs.h is " + vs.h);
	     console.log("before doing anything, domGeometry.position(node) is " + domGeometry.position(node).y);
*/
		if(has("ie")) {
			node.scrollIntoView(false);
	     	// console.log("after node.scrollIntoView(false), domGeometry.position(node) is " + domGeometry.position(node).y);
			
			// HTML's scrollIntoView may cut the left edge of node on ie
			// dojo's scrollIntoView has problem to handle the scrollbar (horizontal scrollbar covers part of node after scrollIntoView)
			// As the result, we have to call both for ie
			dojoWindow.scrollIntoView(node);	
		    // console.log("after dojoWindow.scrollIntoView(node), domGeometry.position(node) is " + domGeometry.position(node).y);
		} else {
			node.scrollIntoView(false);
	     	// console.log("after node.scrollIntoView(false), domGeometry.position(node) is " + domGeometry.position(node).y);
		}
	}
		
	popupManager.showCollaboratorInfo = function(node, color, bgColor, message, viewNode) {
	    // make sure node is visible in current viewport
		scrollIntoView(node, viewNode);
		
		var newContent = "<div class='" + popupManager.CONSTANTS.COLLABORATOR_INFO + "' style=" + "'color: " + color + ";background-color: " + bgColor + ";'><label>"  + message + "</label></div>";

	    var tooltipDialog = new TooltipDialog({
	        content: newContent 
	    });
	    
	    // have to open and close the TooltipDialog to let it fully populated then we re-style it
		popup.open({ 
	        popup: tooltipDialog, 
	        around: node, 
	        orient: ["above-alt", "above", "below-alt", "below"]
	        // orient: {TR: 'BR', BL: 'BR'} // this orient works on dojo 1.6                           
	    }); 	    
	    popup.close(tooltipDialog);
	    
	    var customClass = popupManager.CONSTANTS.COLLABORATOR;
	    var domNodeOfPopup = findRootDivOfPopup(tooltipDialog);
	    if (!!domNodeOfPopup) {
	    	    var queryNode = query('.dijitTooltipContainer', domNodeOfPopup);
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    	    queryNode = query('.dijitTooltipFocusNode', domNodeOfPopup); 
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    	    queryNode = query('.dijitTooltipAbove', domNodeOfPopup); 
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    	    queryNode = query('.dijitTooltipBelow', domNodeOfPopup); 
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    	    queryNode = query('.dijitTooltipConnector', domNodeOfPopup);
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    }
	
		popup.open({ 
	        popup: tooltipDialog, 
	        around: node, 
	        orient: ["above-alt", "above", "below-alt", "below"]
	    }); 

		require(["dojo/_base/fx"], function(baseFx){				
			  baseFx.fadeOut({
				   node: domNodeOfPopup,
				   duration: 5000,
				   onEnd: function() {
				       popup.close(tooltipDialog);
				       domNodeOfPopup.parentNode.removeChild(domNodeOfPopup);
				       domNodeOfPopup = null;
				   }
			   }).play();
		});
	}
	
	popupManager.hideCollaboratorInfo = function(node) {
		// popup goes away automatically. do need to do nothing at this moment
	}
	
	popupManager.showDataChange = function(node, message, viewNode) {

	    // make sure node is visible in current viewport
    	scrollIntoView(node, viewNode)

		var newContent = "<label>" + message + "</label>";				

	    var tooltipDialog = new TooltipDialog({
	        content: newContent 
	    });
	    
		popup.open({ 
	        popup: tooltipDialog, 
	        around: node, 
	        orient: ["above-alt", "above", "below-alt", "below"]
	    }); 
	    
	    var customClass = popupManager.CONSTANTS.DATA_CHANGED;
	    var domNodeOfPopup = findRootDivOfPopup(tooltipDialog);
	    if (!!domNodeOfPopup) {
		var queryNode = query('.dijitTooltipContainer', domNodeOfPopup);
		if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
		var queryNode = query('.dijitTooltipConnector', domNodeOfPopup);
		if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    }
			    
		require(["dojo/_base/fx", "dojo/fx"], function(baseFx, fx){			
			var outlineColor = !!node.style.outline ? node.style.outlineColor : null;
			var fadeOutPopup = baseFx.fadeOut({
				   node: domNodeOfPopup,
				   duration: 1000,
				   onEnd: function() {
				       popup.close(tooltipDialog);
				       domNodeOfPopup.parentNode.removeChild(domNodeOfPopup);
				       domNodeOfPopup = null;
				       if(!!node.style.outline) {
				           node.style.outlineColor = outlineColor;
				       }
				   }
				 });
			var fadeOutOutline = baseFx.animateProperty({
				   node:node,
				   duration: 1000,
				   properties: {
				     outlineColor: {end: "null"},
				   }
				});
			var fadeInOutline = baseFx.animateProperty({
				   node:node,
				   duration: 1000,
				   properties: {
				     outlineColor: {end: outlineColor},
				   }
				});
					 
			if(!!node.style.outline) {
				fx.chain([fadeOutOutline, fadeInOutline, fadeOutOutline, fadeOutPopup]).play();			
			} else {
				fadeOutPopup.play();
			}			
		});
	}	

	function findRootDivOfPopup(tooltipDialog) {
		// update accordingly if the DOM node of tooltipDialog changes in the future dojo release
		return tooltipDialog.domNode.parentNode;
	/*
		var bubbleDiv;	
	    var dijitTooltipDialogPopups = query('.dijitTooltipDialogPopup');
	    if(dijitTooltipDialogPopups.length == 1) {
	    	bubbleDiv = dijitTooltipDialogPopups[0];
	    } else {
	    	for(var k=0; k<dijitTooltipDialogPopups.length; k++) {
	    		if(query('.bubbleNode', dijitTooltipDialogPopups[k]).length > 0) {
	    			bubbleDiv = dijitTooltipDialogPopups[k];
	    			break;
	    		};
	    	};
	    }	    
	    return bubbleDiv;
	*/
	}                             

		
	return popupManager;

});

},
'dojo/cldr/nls/en/gregorian':function(){
define(
//begin v1.x content
{
	"dateFormatItem-yM": "M/y",
	"field-dayperiod": "AM/PM",
	"dateFormatItem-yQ": "Q y",
	"dayPeriods-format-wide-pm": "PM",
	"field-minute": "Minute",
	"eraNames": [
		"Before Christ",
		"Anno Domini"
	],
	"dateFormatItem-MMMEd": "E, MMM d",
	"dateTimeFormat-full": "{1} {0}",
	"field-day-relative+-1": "Yesterday",
	"field-weekday": "Day of the Week",
	"dateFormatItem-hms": "h:mm:ss a",
	"dateFormatItem-yQQQ": "QQQ y",
	"days-standAlone-wide": [
		"Sunday",
		"Monday",
		"Tuesday",
		"Wednesday",
		"Thursday",
		"Friday",
		"Saturday"
	],
	"dateFormatItem-MMM": "LLL",
	"months-standAlone-narrow": [
		"J",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"dateTimeFormat-short": "{1} {0}",
	"field-era": "Era",
	"field-hour": "Hour",
	"dayPeriods-format-wide-am": "AM",
	"dateTimeFormat-medium": "{1} {0}",
	"dateFormatItem-y": "y",
	"timeFormat-full": "h:mm:ss a zzzz",
	"months-standAlone-abbr": [
		"Jan",
		"Feb",
		"Mar",
		"Apr",
		"May",
		"Jun",
		"Jul",
		"Aug",
		"Sep",
		"Oct",
		"Nov",
		"Dec"
	],
	"dateFormatItem-yMMM": "MMM y",
	"field-day-relative+0": "Today",
	"field-day-relative+1": "Tomorrow",
	"days-standAlone-narrow": [
		"S",
		"M",
		"T",
		"W",
		"T",
		"F",
		"S"
	],
	"eraAbbr": [
		"BC",
		"AD"
	],
	"dateFormat-long": "MMMM d, y",
	"timeFormat-medium": "h:mm:ss a",
	"dateFormatItem-EEEd": "d EEE",
	"field-zone": "Zone",
	"dateFormatItem-Hm": "HH:mm",
	"dateFormat-medium": "MMM d, y",
	"dateFormatItem-Hms": "HH:mm:ss",
	"quarters-standAlone-wide": [
		"1st quarter",
		"2nd quarter",
		"3rd quarter",
		"4th quarter"
	],
	"dateFormatItem-ms": "mm:ss",
	"field-year": "Year",
	"quarters-standAlone-narrow": [
		"1",
		"2",
		"3",
		"4"
	],
	"dateTimeFormat-long": "{1} {0}",
	"field-week": "Week",
	"months-standAlone-wide": [
		"January",
		"February",
		"March",
		"April",
		"May",
		"June",
		"July",
		"August",
		"September",
		"October",
		"November",
		"December"
	],
	"dateFormatItem-MMMd": "MMM d",
	"timeFormat-long": "h:mm:ss a z",
	"months-format-abbr": [
		"Jan",
		"Feb",
		"Mar",
		"Apr",
		"May",
		"Jun",
		"Jul",
		"Aug",
		"Sep",
		"Oct",
		"Nov",
		"Dec"
	],
	"dayPeriods-format-wide-noon": "noon",
	"timeFormat-short": "h:mm a",
	"field-month": "Month",
	"quarters-format-abbr": [
		"Q1",
		"Q2",
		"Q3",
		"Q4"
	],
	"days-format-abbr": [
		"Sun",
		"Mon",
		"Tue",
		"Wed",
		"Thu",
		"Fri",
		"Sat"
	],
	"dateFormatItem-M": "L",
	"days-format-narrow": [
		"S",
		"M",
		"T",
		"W",
		"T",
		"F",
		"S"
	],
	"field-second": "Second",
	"field-day": "Day",
	"dateFormatItem-MEd": "E, M/d",
	"months-format-narrow": [
		"J",
		"F",
		"M",
		"A",
		"M",
		"J",
		"J",
		"A",
		"S",
		"O",
		"N",
		"D"
	],
	"dateFormatItem-hm": "h:mm a",
	"days-standAlone-abbr": [
		"Sun",
		"Mon",
		"Tue",
		"Wed",
		"Thu",
		"Fri",
		"Sat"
	],
	"dateFormat-short": "M/d/yy",
	"dateFormatItem-yMMMEd": "EEE, MMM d, y",
	"dateFormat-full": "EEEE, MMMM d, y",
	"dateFormatItem-Md": "M/d",
	"dateFormatItem-yMEd": "EEE, M/d/y",
	"months-format-wide": [
		"January",
		"February",
		"March",
		"April",
		"May",
		"June",
		"July",
		"August",
		"September",
		"October",
		"November",
		"December"
	],
	"dateFormatItem-d": "d",
	"quarters-format-wide": [
		"1st quarter",
		"2nd quarter",
		"3rd quarter",
		"4th quarter"
	],
	"days-format-wide": [
		"Sunday",
		"Monday",
		"Tuesday",
		"Wednesday",
		"Thursday",
		"Friday",
		"Saturday"
	],
	"eraNarrow": [
		"B",
		"A"
	]
}
//end v1.x content
);
},
'dojo/store/Observable':function(){
define(["../_base/kernel", "../_base/lang", "../_base/Deferred", "../_base/array"
], function(kernel, lang, Deferred, array) {
	// module:
	//		dojo/store/Observable
	// summary:
	//		TODOC

var ds = lang.getObject("dojo.store", true);

return ds.Observable = function(store){
	// summary:
	//		The Observable store wrapper takes a store and sets an observe method on query()
	//		results that can be used to monitor results for changes.
	//
	// description:
	//		Observable wraps an existing store so that notifications can be made when a query
	//		is performed.
	//
	// example:
	//		Create a Memory store that returns an observable query, and then log some
	//		information about that query.
	//
	//	|	var store = dojo.store.Observable(new dojo.store.Memory({
	//	|		data: [
	//	|			{id: 1, name: "one", prime: false},
	//	|			{id: 2, name: "two", even: true, prime: true},
	//	|			{id: 3, name: "three", prime: true},
	//	|			{id: 4, name: "four", even: true, prime: false},
	//	|			{id: 5, name: "five", prime: true}
	//	|		]
	//	|	}));
	//	|	var changes = [], results = store.query({ prime: true });
	//	|	var observer = results.observe(function(object, previousIndex, newIndex){
	//	|		changes.push({previousIndex:previousIndex, newIndex:newIndex, object:object});
	//	|	});
	//
	//		See the Observable tests for more information.

	var undef, queryUpdaters = [], revision = 0;
	// a Comet driven store could directly call notify to notify observers when data has
	// changed on the backend
	store.notify = function(object, existingId){
		revision++;
		var updaters = queryUpdaters.slice();
		for(var i = 0, l = updaters.length; i < l; i++){
			updaters[i](object, existingId);
		}
	};
	var originalQuery = store.query;
	store.query = function(query, options){
		options = options || {};
		var results = originalQuery.apply(this, arguments);
		if(results && results.forEach){
			var nonPagedOptions = lang.mixin({}, options);
			delete nonPagedOptions.start;
			delete nonPagedOptions.count;

			var queryExecutor = store.queryEngine && store.queryEngine(query, nonPagedOptions);
			var queryRevision = revision;
			var listeners = [], queryUpdater;
			results.observe = function(listener, includeObjectUpdates){
				if(listeners.push(listener) == 1){
					// first listener was added, create the query checker and updater
					queryUpdaters.push(queryUpdater = function(changed, existingId){
						Deferred.when(results, function(resultsArray){
							var atEnd = resultsArray.length != options.count;
							var i, l, listener;
							if(++queryRevision != revision){
								throw new Error("Query is out of date, you must observe() the query prior to any data modifications");
							}
							var removedObject, removedFrom = -1, insertedInto = -1;
							if(existingId !== undef){
								// remove the old one
								for(i = 0, l = resultsArray.length; i < l; i++){
									var object = resultsArray[i];
									if(store.getIdentity(object) == existingId){
										removedObject = object;
										removedFrom = i;
										if(queryExecutor || !changed){// if it was changed and we don't have a queryExecutor, we shouldn't remove it because updated objects would be eliminated
											resultsArray.splice(i, 1);
										}
										break;
									}
								}
							}
							if(queryExecutor){
								// add the new one
								if(changed &&
										// if a matches function exists, use that (probably more efficient)
										(queryExecutor.matches ? queryExecutor.matches(changed) : queryExecutor([changed]).length)){

									var firstInsertedInto = removedFrom > -1 ? 
										removedFrom : // put back in the original slot so it doesn't move unless it needs to (relying on a stable sort below)
										resultsArray.length;
									resultsArray.splice(firstInsertedInto, 0, changed); // add the new item
									insertedInto = array.indexOf(queryExecutor(resultsArray), changed); // sort it
									// we now need to push the chagne back into the original results array
									resultsArray.splice(firstInsertedInto, 1); // remove the inserted item from the previous index
									
									if((options.start && insertedInto == 0) ||
										(!atEnd && insertedInto == resultsArray.length)){
										// if it is at the end of the page, assume it goes into the prev or next page
										insertedInto = -1;
									}else{
										resultsArray.splice(insertedInto, 0, changed); // and insert into the results array with the correct index
									}
								}
							}else if(changed && !options.start){
								// we don't have a queryEngine, so we can't provide any information
								// about where it was inserted, but we can at least indicate a new object
								insertedInto = removedFrom >= 0 ? removedFrom : (store.defaultIndex || 0);
							}
							if((removedFrom > -1 || insertedInto > -1) &&
									(includeObjectUpdates || !queryExecutor || (removedFrom != insertedInto))){
								var copyListeners = listeners.slice();
								for(i = 0;listener = copyListeners[i]; i++){
									listener(changed || removedObject, removedFrom, insertedInto);
								}
							}
						});
					});
				}
				return {
					cancel: function(){
						// remove this listener
						var index = array.indexOf(listeners, listener);
						if(index > -1){ // check to make sure we haven't already called cancel
							listeners.splice(index, 1);
							if(!listeners.length){
								// no more listeners, remove the query updater too
								queryUpdaters.splice(array.indexOf(queryUpdaters, queryUpdater), 1);
							}
						}						
					}
				};
			};
		}
		return results;
	};
	var inMethod;
	function whenFinished(method, action){
		var original = store[method];
		if(original){
			store[method] = function(value){
				if(inMethod){
					// if one method calls another (like add() calling put()) we don't want two events
					return original.apply(this, arguments);
				}
				inMethod = true;
				try{
					var results = original.apply(this, arguments);
					Deferred.when(results, function(results){
						action((typeof results == "object" && results) || value);
					});
					return results;
				}finally{
					inMethod = false;
				}
			};
		}
	}
	// monitor for updates by listening to these methods
	whenFinished("put", function(object){
		store.notify(object, store.getIdentity(object));
	});
	whenFinished("add", function(object){
		store.notify(object);
	});
	whenFinished("remove", function(id){
		store.notify(undefined, id);
	});

	return store;
};
});

},
'dojo/main':function(){
define([
	"./_base/kernel",
	"./has",
	"require",
	"./_base/sniff",
	"./_base/lang",
	"./_base/array",
	"./ready",
	"./_base/declare",
	"./_base/connect",
	"./_base/Deferred",
	"./_base/json",
	"./_base/Color",
	"./has!dojo-firebug?./_firebug/firebug",
	"./_base/browser",
	"./_base/loader"], function(dojo, has, require, sniff, lang, array, ready){
	// module:
	//		dojo/main
	// summary:
	//		This is the package main module for the dojo package; it loads dojo base appropriate for the execution environment.

	// the preferred way to load the dojo firebug console is by setting has("dojo-firebug") true in dojoConfig
	// the isDebug config switch is for backcompat and will work fine in sync loading mode; it works in
	// async mode too, but there's no guarantee when the module is loaded; therefore, if you need a firebug
	// console guarnanteed at a particular spot in an app, either set config.has["dojo-firebug"] true before
	// loading dojo.js or explicitly include dojo/_firebug/firebug in a dependency list.
	if(dojo.config.isDebug){
		require(["./_firebug/firebug"]);
	}

	// dojoConfig.require is deprecated; use the loader configuration property deps
	true || has.add("dojo-config-require", 1);
	if(1){
		var deps= dojo.config.require;
		if(deps){
			// dojo.config.require may be dot notation
			deps= array.map(lang.isArray(deps) ? deps : [deps], function(item){ return item.replace(/\./g, "/"); });
			if(dojo.isAsync){
				require(deps);
			}else{
				// this is a bit janky; in 1.6- dojo is defined before these requires are applied; but in 1.7+
				// dojo isn't defined until returning from this module; this is only a problem in sync mode
				// since we're in sync mode, we know we've got our loader with its priority ready queue
				ready(1, function(){require(deps);});
			}
		}
	}

	return dojo;
});

},
'dojo/on':function(){
define(["./has!dom-addeventlistener?:./aspect", "./_base/kernel", "./has"], function(aspect, dojo, has){
	// summary:
	//		The export of this module is a function that provides core event listening functionality. With this function
	//		you can provide a target, event type, and listener to be notified of
	//		future matching events that are fired.
	// target: Element|Object
	//		This is the target object or DOM element that to receive events from
	// type: String|Function
	// 		This is the name of the event to listen for or an extension event type.
	// listener: Function
	// 		This is the function that should be called when the event fires.
	// returns: Object
	// 		An object with a remove() method that can be used to stop listening for this
	// 		event.
	// description:
	// 		To listen for "click" events on a button node, we can do:
	// 		|	define(["dojo/on"], function(listen){
	// 		|		on(button, "click", clickHandler);
	//		|		...
	//  	Evented JavaScript objects can also have their own events.
	// 		|	var obj = new Evented;
	//		|	on(obj, "foo", fooHandler);
	//		And then we could publish a "foo" event:
	//		|	on.emit(obj, "foo", {key: "value"});
	//		We can use extension events as well. For example, you could listen for a tap gesture:
	// 		|	define(["dojo/on", "dojo/gesture/tap", function(listen, tap){
	// 		|		on(button, tap, tapHandler);
	//		|		...
	//		which would trigger fooHandler. Note that for a simple object this is equivalent to calling:
	//		|	obj.onfoo({key:"value"});
	//		If you use on.emit on a DOM node, it will use native event dispatching when possible.

 	"use strict";
	if(1){ // check to make sure we are in a browser, this module should work anywhere
		var major = window.ScriptEngineMajorVersion;
		has.add("jscript", major && (major() + ScriptEngineMinorVersion() / 10));
		has.add("event-orientationchange", has("touch") && !has("android")); // TODO: how do we detect this?
	}
	var on = function(target, type, listener, dontFix){
		if(typeof target.on == "function" && typeof type != "function" && !target.nodeType){
			// delegate to the target's on() method, so it can handle it's own listening if it wants
			return target.on(type, listener);
		}
		// delegate to main listener code
		return on.parse(target, type, listener, addListener, dontFix, this);
	};
	on.pausable =  function(target, type, listener, dontFix){
		// summary:
		//		This function acts the same as on(), but with pausable functionality. The
		// 		returned signal object has pause() and resume() functions. Calling the
		//		pause() method will cause the listener to not be called for future events. Calling the
		//		resume() method will cause the listener to again be called for future events.
		var paused;
		var signal = on(target, type, function(){
			if(!paused){
				return listener.apply(this, arguments);
			}
		}, dontFix);
		signal.pause = function(){
			paused = true;
		};
		signal.resume = function(){
			paused = false;
		};
		return signal;
	};
	on.once = function(target, type, listener, dontFix){
		// summary:
		//		This function acts the same as on(), but will only call the listener once. The 
		// 		listener will be called for the first
		//		event that takes place and then listener will automatically be removed.
		var signal = on(target, type, function(){
			// remove this listener
			signal.remove();
			// proceed to call the listener
			return listener.apply(this, arguments);
		});
		return signal;
	};
	on.parse = function(target, type, listener, addListener, dontFix, matchesTarget){
		if(type.call){
			// event handler function
			// on(node, dojo.touch.press, touchListener);
			return type.call(matchesTarget, target, listener);
		}

		if(type.indexOf(",") > -1){
			// we allow comma delimited event names, so you can register for multiple events at once
			var events = type.split(/\s*,\s*/);
			var handles = [];
			var i = 0;
			var eventName;
			while(eventName = events[i++]){
				handles.push(addListener(target, eventName, listener, dontFix, matchesTarget));
			}
			handles.remove = function(){
				for(var i = 0; i < handles.length; i++){
					handles[i].remove();
				}
			};
			return handles;
		}
		return addListener(target, type, listener, dontFix, matchesTarget)
	};
	var touchEvents = /^touch/;
	function addListener(target, type, listener, dontFix, matchesTarget){		
		// event delegation:
		var selector = type.match(/(.*):(.*)/);
		// if we have a selector:event, the last one is interpreted as an event, and we use event delegation
		if(selector){
			type = selector[2];
			selector = selector[1];
			// create the extension event for selectors and directly call it
			return on.selector(selector, type).call(matchesTarget, target, listener);
		}
		// test to see if it a touch event right now, so we don't have to do it every time it fires
		if(has("touch")){
			if(touchEvents.test(type)){
				// touch event, fix it
				listener = fixTouchListener(listener);
			}
			if(!has("event-orientationchange") && (type == "orientationchange")){
				//"orientationchange" not supported <= Android 2.1, 
				//but works through "resize" on window
				type = "resize"; 
				target = window;
				listener = fixTouchListener(listener);
			} 
		}
		// normal path, the target is |this|
		if(target.addEventListener){
			// the target has addEventListener, which should be used if available (might or might not be a node, non-nodes can implement this method as well)
			// check for capture conversions
			var capture = type in captures;
			target.addEventListener(capture ? captures[type] : type, listener, capture);
			// create and return the signal
			return {
				remove: function(){
					target.removeEventListener(type, listener, capture);
				}
			};
		}
		type = "on" + type;
		if(fixAttach && target.attachEvent){
			return fixAttach(target, type, listener);
		}
	 	throw new Error("Target must be an event emitter");
	}

	on.selector = function(selector, eventType, children){
		// summary:
		//		Creates a new extension event with event delegation. This is based on
		// 		the provided event type (can be extension event) that
		// 		only calls the listener when the CSS selector matches the target of the event.
		//	selector:
		//		The CSS selector to use for filter events and determine the |this| of the event listener.
		//	eventType:
		//		The event to listen for
		// children:
		//		Indicates if children elements of the selector should be allowed. This defaults to 
		// 		true (except in the case of normally non-bubbling events like mouse.enter, in which case it defaults to false).
		//	example:
		//		define(["dojo/on", "dojo/mouse"], function(listen, mouse){
		//			on(node, on.selector(".my-class", mouse.enter), handlerForMyHover);
		return function(target, listener){
			var matchesTarget = this;
			var bubble = eventType.bubble;
			if(bubble){
				// the event type doesn't naturally bubble, but has a bubbling form, use that
				eventType = bubble;
			}else if(children !== false){
				// for normal bubbling events we default to allowing children of the selector
				children = true;
			}
			return on(target, eventType, function(event){
				var eventTarget = event.target;
				// see if we have a valid matchesTarget or default to dojo.query
				matchesTarget = matchesTarget && matchesTarget.matches ? matchesTarget : dojo.query;
				// there is a selector, so make sure it matches
				while(!matchesTarget.matches(eventTarget, selector, target)){
					if(eventTarget == target || !children || !(eventTarget = eventTarget.parentNode)){ // intentional assignment
						return;
					}
				}
				return listener.call(eventTarget, event);
			});
		};
	};

	function syntheticPreventDefault(){
		this.cancelable = false;
	}
	function syntheticStopPropagation(){
		this.bubbles = false;
	}
	var slice = [].slice,
		syntheticDispatch = on.emit = function(target, type, event){
		// summary:
		//		Fires an event on the target object.
		//	target:
		//		The target object to fire the event on. This can be a DOM element or a plain 
		// 		JS object. If the target is a DOM element, native event emiting mechanisms
		//		are used when possible.
		//	type:
		//		The event type name. You can emulate standard native events like "click" and 
		// 		"mouseover" or create custom events like "open" or "finish".
		//	event:
		//		An object that provides the properties for the event. See https://developer.mozilla.org/en/DOM/event.initEvent 
		// 		for some of the properties. These properties are copied to the event object.
		//		Of particular importance are the cancelable and bubbles properties. The
		//		cancelable property indicates whether or not the event has a default action
		// 		that can be cancelled. The event is cancelled by calling preventDefault() on
		// 		the event object. The bubbles property indicates whether or not the
		//		event will bubble up the DOM tree. If bubbles is true, the event will be called
		//		on the target and then each parent successively until the top of the tree
		//		is reached or stopPropagation() is called. Both bubbles and cancelable 
		// 		default to false.
		//	returns:
		//		If the event is cancelable and the event is not cancelled,
		// 		emit will return true. If the event is cancelable and the event is cancelled,
		// 		emit will return false.
		//	details:
		//		Note that this is designed to emit events for listeners registered through
		//		dojo/on. It should actually work with any event listener except those
		// 		added through IE's attachEvent (IE8 and below's non-W3C event emiting
		// 		doesn't support custom event types). It should work with all events registered
		// 		through dojo/on. Also note that the emit method does do any default
		// 		action, it only returns a value to indicate if the default action should take
		// 		place. For example, emiting a keypress event would not cause a character
		// 		to appear in a textbox.
		//	example:
		//		To fire our own click event
		//	|	on.emit(dojo.byId("button"), "click", {
		//	|		cancelable: true,
		//	|		bubbles: true,
		//	|		screenX: 33,
		//	|		screenY: 44
		//	|	});
		//		We can also fire our own custom events:
		//	|	on.emit(dojo.byId("slider"), "slide", {
		//	|		cancelable: true,
		//	|		bubbles: true,
		//	|		direction: "left-to-right"
		//	|	});
		var args = slice.call(arguments, 2);
		var method = "on" + type;
		if("parentNode" in target){
			// node (or node-like), create event controller methods
			var newEvent = args[0] = {};
			for(var i in event){
				newEvent[i] = event[i];
			}
			newEvent.preventDefault = syntheticPreventDefault;
			newEvent.stopPropagation = syntheticStopPropagation;
			newEvent.target = target;
			newEvent.type = type;
			event = newEvent;
		}
		do{
			// call any node which has a handler (note that ideally we would try/catch to simulate normal event propagation but that causes too much pain for debugging)
			target[method] && target[method].apply(target, args);
			// and then continue up the parent node chain if it is still bubbling (if started as bubbles and stopPropagation hasn't been called)
		}while(event && event.bubbles && (target = target.parentNode));
		return event && event.cancelable && event; // if it is still true (was cancelable and was cancelled), return the event to indicate default action should happen
	};
	var captures = {}; 
	if(has("dom-addeventlistener")){
		// normalize focusin and focusout
		captures = {
			focusin: "focus",
			focusout: "blur"
		};
		if(has("opera")){
			captures.keydown = "keypress"; // this one needs to be transformed because Opera doesn't support repeating keys on keydown (and keypress works because it incorrectly fires on all keydown events)
		}

		// emiter that works with native event handling
		on.emit = function(target, type, event){
			if(target.dispatchEvent && document.createEvent){
				// use the native event emiting mechanism if it is available on the target object
				// create a generic event				
				// we could create branch into the different types of event constructors, but 
				// that would be a lot of extra code, with little benefit that I can see, seems 
				// best to use the generic constructor and copy properties over, making it 
				// easy to have events look like the ones created with specific initializers
				var nativeEvent = document.createEvent("HTMLEvents");
				nativeEvent.initEvent(type, !!event.bubbles, !!event.cancelable);
				// and copy all our properties over
				for(var i in event){
					var value = event[i];
					if(!(i in nativeEvent)){
						nativeEvent[i] = event[i];
					}
				}
				return target.dispatchEvent(nativeEvent) && nativeEvent;
			}
			return syntheticDispatch.apply(on, arguments); // emit for a non-node
		};
	}else{
		// no addEventListener, basically old IE event normalization
		on._fixEvent = function(evt, sender){
			// summary:
			//		normalizes properties on the event object including event
			//		bubbling methods, keystroke normalization, and x/y positions
			// evt:
			//		native event object
			// sender:
			//		node to treat as "currentTarget"
			if(!evt){
				var w = sender && (sender.ownerDocument || sender.document || sender).parentWindow || window;
				evt = w.event;
			}
			if(!evt){return(evt);}
			if(!evt.target){ // check to see if it has been fixed yet
				evt.target = evt.srcElement;
				evt.currentTarget = (sender || evt.srcElement);
				if(evt.type == "mouseover"){
					evt.relatedTarget = evt.fromElement;
				}
				if(evt.type == "mouseout"){
					evt.relatedTarget = evt.toElement;
				}
				if(!evt.stopPropagation){
					evt.stopPropagation = stopPropagation;
					evt.preventDefault = preventDefault;
				}
				switch(evt.type){
					case "keypress":
						var c = ("charCode" in evt ? evt.charCode : evt.keyCode);
						if (c==10){
							// CTRL-ENTER is CTRL-ASCII(10) on IE, but CTRL-ENTER on Mozilla
							c=0;
							evt.keyCode = 13;
						}else if(c==13||c==27){
							c=0; // Mozilla considers ENTER and ESC non-printable
						}else if(c==3){
							c=99; // Mozilla maps CTRL-BREAK to CTRL-c
						}
						// Mozilla sets keyCode to 0 when there is a charCode
						// but that stops the event on IE.
						evt.charCode = c;
						_setKeyChar(evt);
						break;
				}
			}
			return evt;
		};
		var IESignal = function(handle){
			this.handle = handle;
		};
		IESignal.prototype.remove = function(){
			delete _dojoIEListeners_[this.handle];
		};
		var fixListener = function(listener){
			// this is a minimal function for closing on the previous listener with as few as variables as possible
			return function(evt){
				evt = on._fixEvent(evt, this);
				return listener.call(this, evt);
			}
		}
		var fixAttach = function(target, type, listener){
			listener = fixListener(listener);
			if(((target.ownerDocument ? target.ownerDocument.parentWindow : target.parentWindow || target.window || window) != top || 
						has("jscript") < 5.8) && 
					!has("config-_allow_leaks")){
				// IE will leak memory on certain handlers in frames (IE8 and earlier) and in unattached DOM nodes for JScript 5.7 and below.
				// Here we use global redirection to solve the memory leaks
				if(typeof _dojoIEListeners_ == "undefined"){
					_dojoIEListeners_ = [];
				}
				var emiter = target[type];
				if(!emiter || !emiter.listeners){
					var oldListener = emiter;
					target[type] = emiter = Function('event', 'var callee = arguments.callee; for(var i = 0; i<callee.listeners.length; i++){var listener = _dojoIEListeners_[callee.listeners[i]]; if(listener){listener.call(this,event);}}');
					emiter.listeners = [];
					emiter.global = this;
					if(oldListener){
						emiter.listeners.push(_dojoIEListeners_.push(oldListener) - 1);
					}
				}
				var handle;
				emiter.listeners.push(handle = (emiter.global._dojoIEListeners_.push(listener) - 1));
				return new IESignal(handle);
			}
			return aspect.after(target, type, listener, true);
		};

		var _setKeyChar = function(evt){
			evt.keyChar = evt.charCode ? String.fromCharCode(evt.charCode) : '';
			evt.charOrCode = evt.keyChar || evt.keyCode;
		};
		// Called in Event scope
		var stopPropagation = function(){
			this.cancelBubble = true;
		};
		var preventDefault = on._preventDefault = function(){
			// Setting keyCode to 0 is the only way to prevent certain keypresses (namely
			// ctrl-combinations that correspond to menu accelerator keys).
			// Otoh, it prevents upstream listeners from getting this information
			// Try to split the difference here by clobbering keyCode only for ctrl
			// combinations. If you still need to access the key upstream, bubbledKeyCode is
			// provided as a workaround.
			this.bubbledKeyCode = this.keyCode;
			if(this.ctrlKey){
				try{
					// squelch errors when keyCode is read-only
					// (e.g. if keyCode is ctrl or shift)
					this.keyCode = 0;
				}catch(e){
				}
			}
			this.returnValue = false;
		};
	}
	if(has("touch")){ 
		var Event = function (){};
		var windowOrientation = window.orientation; 
		var fixTouchListener = function(listener){ 
			return function(originalEvent){ 
				//Event normalization(for ontouchxxx and resize): 
				//1.incorrect e.pageX|pageY in iOS 
				//2.there are no "e.rotation", "e.scale" and "onorientationchange" in Andriod
				//3.More TBD e.g. force | screenX | screenX | clientX | clientY | radiusX | radiusY

				// see if it has already been corrected
				var event = originalEvent.corrected;
				if(!event){
					var type = originalEvent.type;
					try{
						delete originalEvent.type; // on some JS engines (android), deleting properties make them mutable
					}catch(e){} 
					if(originalEvent.type){
						// deleting properties doesn't work (older iOS), have to use delegation
						Event.prototype = originalEvent;
						var event = new Event;
						// have to delegate methods to make them work
						event.preventDefault = function(){
							originalEvent.preventDefault();
						};
						event.stopPropagation = function(){
							originalEvent.stopPropagation();
						};
					}else{
						// deletion worked, use property as is
						event = originalEvent;
						event.type = type;
					}
					originalEvent.corrected = event;
					if(type == 'resize'){
						if(windowOrientation == window.orientation){ 
							return null;//double tap causes an unexpected 'resize' in Andriod 
						} 
						windowOrientation = window.orientation;
						event.type = "orientationchange"; 
						return listener.call(this, event);
					}
					// We use the original event and augment, rather than doing an expensive mixin operation
					if(!("rotation" in event)){ // test to see if it has rotation
						event.rotation = 0; 
						event.scale = 1;
					}
					//use event.changedTouches[0].pageX|pageY|screenX|screenY|clientX|clientY|target
					var firstChangeTouch = event.changedTouches[0];
					for(var i in firstChangeTouch){ // use for-in, we don't need to have dependency on dojo/_base/lang here
						delete event[i]; // delete it first to make it mutable
						event[i] = firstChangeTouch[i];
					}
				}
				return listener.call(this, event); 
			}; 
		}; 
	}
	return on;
});

},
'dijit/_Widget':function(){
define([
	"dojo/aspect",	// aspect.around
	"dojo/_base/config",	// config.isDebug
	"dojo/_base/connect",	// connect.connect
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/query",
	"dojo/ready",
	"./registry",	// registry.byNode
	"./_WidgetBase",
	"./_OnDijitClickMixin",
	"./_FocusMixin",
	"dojo/uacss",		// browser sniffing (included for back-compat; subclasses may be using)
	"./hccss"		// high contrast mode sniffing (included to set CSS classes on <body>, module ret value unused)
], function(aspect, config, connect, declare, kernel, lang, query, ready,
			registry, _WidgetBase, _OnDijitClickMixin, _FocusMixin){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _OnDijitClickMixin = dijit._OnDijitClickMixin;
	var _FocusMixin = dijit._FocusMixin;
=====*/


// module:
//		dijit/_Widget
// summary:
//		Old base for widgets.   New widgets should extend _WidgetBase instead


function connectToDomNode(){
	// summary:
	//		If user connects to a widget method === this function, then they will
	//		instead actually be connecting the equivalent event on this.domNode
}

// Trap dojo.connect() calls to connectToDomNode methods, and redirect to _Widget.on()
function aroundAdvice(originalConnect){
	return function(obj, event, scope, method){
		if(obj && typeof event == "string" && obj[event] == connectToDomNode){
			return obj.on(event.substring(2).toLowerCase(), lang.hitch(scope, method));
		}
		return originalConnect.apply(connect, arguments);
	};
}
aspect.around(connect, "connect", aroundAdvice);
if(kernel.connect){
	aspect.around(kernel, "connect", aroundAdvice);
}

var _Widget = declare("dijit._Widget", [_WidgetBase, _OnDijitClickMixin, _FocusMixin], {
	// summary:
	//		Base class for all Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//			- declaratively/programatically specifying widget initialization parameters like
	//				onMouseMove="foo" that call foo when this.domNode gets a mousemove event
	//			- ondijitclick
	//				Support new data-dojo-attach-event="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//			- focus related functions
	//				In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//				dijit/_base/focus.js.
	//			- deprecated methods
	//			- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//			- browser sniffing (putting browser id like .dj_ie on <html> node)
	//			- high contrast mode sniffing (add .dijit_a11y class to <body> if machine is in high contrast mode)


	////////////////// DEFERRED CONNECTS ///////////////////

	onClick: connectToDomNode,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: connectToDomNode,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: connectToDomNode,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: connectToDomNode,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: connectToDomNode,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: connectToDomNode,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: connectToDomNode,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: connectToDomNode,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: connectToDomNode,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: connectToDomNode,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: connectToDomNode,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: connectToDomNode,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	constructor: function(params){
		// extract parameters like onMouseMove that should connect directly to this.domNode
		this._toConnect = {};
		for(var name in params){
			if(this[name] === connectToDomNode){
				this._toConnect[name.replace(/^on/, "").toLowerCase()] = params[name];
				delete params[name];
			}
		}
	},

	postCreate: function(){
		this.inherited(arguments);

		// perform connection from this.domNode to user specified handlers (ex: onMouseMove)
		for(var name in this._toConnect){
			this.on(name, this._toConnect[name]);
		}
		delete this._toConnect;
	},

	on: function(/*String*/ type, /*Function*/ func){
		if(this[this._onMap(type)] === connectToDomNode){
			// Use connect.connect() rather than on() to get handling for "onmouseenter" on non-IE, etc.
			// Also, need to specify context as "this" rather than the default context of the DOMNode
			return connect.connect(this.domNode, type.toLowerCase(), this, func);
		}
		return this.inherited(arguments);
	},

	_setFocusedAttr: function(val){
		// Remove this method in 2.0 (or sooner), just here to set _focused == focused, for back compat
		// (but since it's a private variable we aren't required to keep supporting it).
		this._focused = val;
		this._set("focused", val);
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		kernel.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				kernel.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		kernel.deprecated(this.declaredClass+"::getDescendants() is deprecated. Use getChildren() instead.", "", "2.0");
		return this.containerNode ? query('[widgetId]', this.containerNode).map(registry.byNode) : []; // dijit._Widget[]
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
		//		`dijit.layout.AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
			//		Called when another widget becomes the selected pane in a
			//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
			//		`dijit.layout.AccordionContainer`, etc.
			//
			//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
			// tags:
			//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit.layout.StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

// For back-compat, remove in 2.0.
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dijit/_base"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}
return _Widget;
});

},
'dijit/_OnDijitClickMixin':function(){
define([
	"dojo/on",
	"dojo/_base/array", // array.forEach
	"dojo/keys", // keys.ENTER keys.SPACE
	"dojo/_base/declare", // declare
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window" // win.doc.addEventListener win.doc.attachEvent win.doc.detachEvent
], function(on, array, keys, declare, has, unload, win){

	// module:
	//		dijit/_OnDijitClickMixin
	// summary:
	//		Mixin so you can pass "ondijitclick" to this.connect() method,
	//		as a way to handle clicks by mouse, or by keyboard (SPACE/ENTER key)


	// Keep track of where the last keydown event was, to help avoid generating
	// spurious ondijitclick events when:
	// 1. focus is on a <button> or <a>
	// 2. user presses then releases the ENTER key
	// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
	// 4. onkeyup event fires, causing the ondijitclick handler to fire
	var lastKeyDownNode = null;
	if(has("ie")){
		(function(){
			var keydownCallback = function(evt){
				lastKeyDownNode = evt.srcElement;
			};
			win.doc.attachEvent('onkeydown', keydownCallback);
			unload.addOnWindowUnload(function(){
				win.doc.detachEvent('onkeydown', keydownCallback);
			});
		})();
	}else{
		win.doc.addEventListener('keydown', function(evt){
			lastKeyDownNode = evt.target;
		}, true);
	}

	// Custom a11yclick (a.k.a. ondijitclick) event
	var a11yclick = function(node, listener){
		if(/input|button/i.test(node.nodeName)){
			// pass through, the browser already generates click event on SPACE/ENTER key
			return on(node, "click", listener);
		}else{
			// Don't fire the click event unless both the keydown and keyup occur on this node.
			// Avoids problems where focus shifted to this node or away from the node on keydown,
			// either causing this node to process a stray keyup event, or causing another node
			// to get a stray keyup event.

			function clickKey(/*Event*/ e){
				return (e.keyCode == keys.ENTER || e.keyCode == keys.SPACE) &&
						!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey;
			}
			var handles = [
				on(node, "keypress", function(e){
					//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
					if(clickKey(e)){
						// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
						lastKeyDownNode = e.target;

						// Prevent viewport scrolling on space key in IE<9.
						// (Reproducible on test_Button.html on any of the first dijit.form.Button examples)
						// Do this onkeypress rather than onkeydown because onkeydown.preventDefault() will
						// suppress the onkeypress event, breaking _HasDropDown
						e.preventDefault();
					}
				}),

				on(node, "keyup", function(e){
					//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
					if(clickKey(e) && e.target == lastKeyDownNode){	// === breaks greasemonkey
						//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
						lastKeyDownNode = null;
						listener.call(this, e);
					}
				}),

				on(node, "click", function(e){
					// and connect for mouse clicks too (or touch-clicks on mobile)
					listener.call(this, e);
				})
			];

			return {
				remove: function(){
					array.forEach(handles, function(h){ h.remove(); });
				}
			};
		}
	};

	return declare("dijit._OnDijitClickMixin", null, {
		connect: function(
				/*Object|null*/ obj,
				/*String|Function*/ event,
				/*String|Function*/ method){
			// summary:
			//		Connects specified obj/event to specified method of this object
			//		and registers for disconnect() on widget destroy.
			// description:
			//		Provide widget-specific analog to connect.connect, except with the
			//		implicit use of this widget as the target object.
			//		This version of connect also provides a special "ondijitclick"
			//		event which triggers on a click or space or enter keyup.
			//		Events connected with `this.connect` are disconnected upon
			//		destruction.
			// returns:
			//		A handle that can be passed to `disconnect` in order to disconnect before
			//		the widget is destroyed.
			// example:
			//	|	var btn = new dijit.form.Button();
			//	|	// when foo.bar() is called, call the listener we're going to
			//	|	// provide in the scope of btn
			//	|	btn.connect(foo, "bar", function(){
			//	|		console.debug(this.toString());
			//	|	});
			// tags:
			//		protected

			return this.inherited(arguments, [obj, event == "ondijitclick" ? a11yclick : event, method]);
		}
	});
});

},
'dijit/_FocusMixin':function(){
define([
	"./focus",
	"./_WidgetBase",
	"dojo/_base/declare", // declare
	"dojo/_base/lang" // lang.extend
], function(focus, _WidgetBase, declare, lang){

/*=====
	var _WidgetBase = dijit._WidgetBase;
=====*/

	// module:
	//		dijit/_FocusMixin
	// summary:
	//		Mixin to widget to provide _onFocus() and _onBlur() methods that
	//		fire when a widget or it's descendants get/lose focus

	// We don't know where _FocusMixin will occur in the inheritance chain, but we need the _onFocus()/_onBlur() below
	// to be last in the inheritance chain, so mixin to _WidgetBase.
	lang.extend(_WidgetBase, {
		// focused: [readonly] Boolean
		//		This widget or a widget it contains has focus, or is "active" because
		//		it was recently clicked.
		focused: false,

		onFocus: function(){
			// summary:
			//		Called when the widget becomes "active" because
			//		it or a widget inside of it either has focus, or has recently
			//		been clicked.
			// tags:
			//		callback
		},

		onBlur: function(){
			// summary:
			//		Called when the widget stops being "active" because
			//		focus moved to something outside of it, or the user
			//		clicked somewhere outside of it, or the widget was
			//		hidden.
			// tags:
			//		callback
		},

		_onFocus: function(){
			// summary:
			//		This is where widgets do processing for when they are active,
			//		such as changing CSS classes.  See onFocus() for more details.
			// tags:
			//		protected
			this.onFocus();
		},

		_onBlur: function(){
			// summary:
			//		This is where widgets do processing for when they stop being active,
			//		such as changing CSS classes.  See onBlur() for more details.
			// tags:
			//		protected
			this.onBlur();
		}
	});

	return declare("dijit._FocusMixin", null, {
		// summary:
		//		Mixin to widget to provide _onFocus() and _onBlur() methods that
		//		fire when a widget or it's descendants get/lose focus

		// flag that I want _onFocus()/_onBlur() notifications from focus manager
		_focusManager: focus
	});

});

},
'dojo/cache':function(){
define(["./_base/kernel", "./text"], function(dojo, text){
	// module:
	//		dojo/cache
	// summary:
	//		The module defines dojo.cache by loading dojo/text.

	//dojo.cache is defined in dojo/text
	return dojo.cache;
});

},
'dojox/date/islamic/Date':function(){
define(["dojo/_base/kernel", "dojo/_base/declare", "dojo/date"], function(dojo, declare, dd){

dojo.getObject("date.buddhist.Date", true, dojox);
dojo.experimental("dojox.date.buddhist.Date");

dojo.declare("dojox.date.islamic.Date", null, {
	// summary: The component defines the Islamic (Hijri) Calendar Object
	//
	// description:
	//	This module is similar to the Date() object provided by JavaScript
	//
	// example:
	// |	dojo.require("dojox.date.islamic.Date");
	// |
	// |	var date = new dojox.date.islamic.Date();
	// |	document.writeln(date.getFullYear()+'\'+date.getMonth()+'\'+date.getDate());


	_date: 0,
	_month: 0,
	_year: 0,
	_hours: 0,
	_minutes: 0,
	_seconds: 0,
	_milliseconds: 0,
	_day: 0,
	_GREGORIAN_EPOCH : 1721425.5,
	_ISLAMIC_EPOCH : 1948439.5,

	constructor: function(){
		// summary: This is the constructor
		// description:
		//	This function initialize the date object values
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		var date2 = new dojox.date.islamic.Date("12\2\1429");
		// |
		// |		var date3 = new dojox.date.islamic.Date(date2);
		// |
		// |		var date4 = new dojox.date.islamic.Date(1429,2,12);

		var len = arguments.length;
		if(!len){// use the current date value, added "" to the similarity to date
			this.fromGregorian(new Date());
		}else if(len == 1){
			var arg0 = arguments[0];
			if(typeof arg0 == "number"){ // this is time "valueof"
				arg0 = new Date(arg0);
			}

			if(arg0 instanceof Date){
				this.fromGregorian(arg0);
			}else if(arg0 == ""){
				// date should be invalid.  Dijit relies on this behavior.
				this._date = new Date(""); //TODO: should this be NaN?  _date is not a Date object
			}else{  // this is Islamic.Date object
				this._year = arg0._year;
				this._month =  arg0._month;
				this._date = arg0._date;
				this._hours = arg0._hours;
				this._minutes = arg0._minutes;
				this._seconds = arg0._seconds;
				this._milliseconds = arg0._milliseconds;
			}
		}else if(len >=3){
			// YYYY MM DD arguments passed, month is from 0-12
			this._year += arguments[0];
			this._month += arguments[1];
			this._date += arguments[2];
			this._hours += arguments[3] || 0;
			this._minutes += arguments[4] || 0;
			this._seconds += arguments[5] || 0;
			this._milliseconds += arguments[6] || 0;
		}
	},

	getDate:function(){
		// summary: This function returns the date value (1 - 30)
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getDate);
		return this._date;
	},
	
	getMonth:function(){
		// summary: This function return the month value ( 0 - 11 )
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getMonth()+1);

		return this._month;
	},

	getFullYear:function(){
		// summary: This function return the Year value
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getFullYear());

		return this._year;
	},
		
	getDay:function(){
		// summary: This function return Week Day value ( 0 - 6 )
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getDay());

		return this.toGregorian().getDay();
	},
		
	getHours:function(){
		//summary: returns the Hour value
		return this._hours;
	},
	
	getMinutes:function(){
		//summary: returns the Minuites value
		return this._minutes;
	},

	getSeconds:function(){
		//summary: returns the seconde value
		return this._seconds;
	},

	getMilliseconds:function(){
		//summary: returns the Milliseconds value
		return this._milliseconds;
	},

	setDate: function(/*number*/date){
		// summary: This function sets the Date
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setDate(2);

		date = parseInt(date);

		if(date > 0 && date <= this.getDaysInIslamicMonth(this._month, this._year)){
			this._date = date;
		}else{
			var mdays;
			if(date>0){
				for(mdays = this.getDaysInIslamicMonth(this._month, this._year);
					date > mdays;
						date -= mdays,mdays =this.getDaysInIslamicMonth(this._month, this._year)){
					this._month++;
					if(this._month >= 12){this._year++; this._month -= 12;}
				}

				this._date = date;
			}else{
				for(mdays = this.getDaysInIslamicMonth((this._month-1)>=0 ?(this._month-1) :11 ,((this._month-1)>=0)? this._year: this._year-1);
						date <= 0;
							mdays = this.getDaysInIslamicMonth((this._month-1)>=0 ? (this._month-1) :11,((this._month-1)>=0)? this._year: this._year-1)){
					this._month--;
					if(this._month < 0){this._year--; this._month += 12;}

					date+=mdays;
				}
				this._date = date;
			}
		}
		return this;
	},

	setFullYear:function(/*number*/year){
		// summary: This function set Year
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setYear(1429);

		this._year = +year;
	},

	setMonth: function(/*number*/month) {
		// summary: This function set Month
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setMonth(2);

		this._year += Math.floor(month / 12);
		if(month > 0){
			this._month = Math.floor(month % 12);
		}else{
			this._month = Math.floor(((month % 12) + 12) % 12);
		}
	},

	setHours:function(){
		//summary: set the Hours
		var hours_arg_no = arguments.length;
		var hours = 0;
		if(hours_arg_no >= 1){
			hours = parseInt(arguments[0]);
		}

		if(hours_arg_no >= 2){
			this._minutes = parseInt(arguments[1]);
		}

		if(hours_arg_no >= 3){
			this._seconds = parseInt(arguments[2]);
		}

		if(hours_arg_no == 4){
			this._milliseconds = parseInt(arguments[3]);
		}

		while(hours >= 24){
			this._date++;
			var mdays = this.getDaysInIslamicMonth(this._month, this._year);
			if(this._date > mdays){
					this._month ++;
					if(this._month >= 12){this._year++; this._month -= 12;}
					this._date -= mdays;
			}
			hours -= 24;
		}
		this._hours = hours;
	},

	_addMinutes: function(/*Number*/minutes){
		minutes += this._minutes;
		this.setMinutes(minutes);
		this.setHours(this._hours + parseInt(minutes / 60));
		return this;
	},

	_addSeconds: function(/*Number*/seconds){
		seconds += this._seconds;
		this.setSeconds(seconds);
		this._addMinutes(parseInt(seconds / 60));
		return this;
	},

	_addMilliseconds: function(/*Number*/milliseconds){
		milliseconds += this._milliseconds;
		this.setMilliseconds(milliseconds);
		this._addSeconds(parseInt(milliseconds / 1000));
		return this;
	},

	setMinutes: function(/*Number*/minutes){
		//summary: sets the minutes (0-59) only.
		this._minutes = minutes % 60;
		return this;
	},

	setSeconds: function(/*Number*/seconds){
		//summary: sets the seconds (0-59) only.
		this._seconds = seconds % 60;
		return this;
	},

	setMilliseconds: function(/*Number*/milliseconds){
		this._milliseconds = milliseconds % 1000;
		return this;
	},
		
	toString:function(){
		// summary: This returns a string representation of the date in "DDDD MMMM DD YYYY HH:MM:SS" format
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		document.writeln(date1.toString());

		//FIXME: TZ/DST issues?
		var x = new Date();
		x.setHours(this._hours);
		x.setMinutes(this._minutes);
		x.setSeconds(this._seconds);
		x.setMilliseconds(this._milliseconds);
		return this._month+" "+ this._date + " " + this._year + " " + x.toTimeString();
	},
		
		
	toGregorian:function(){
		// summary: This returns the equevalent Grogorian date value in Date object
		// example:
		// |		var dateIslamic = new dojox.date.islamic.Date(1429,11,20);
		// |		var dateGregorian = dateIslamic.toGregorian();

		var hYear = this._year;
		var hMonth = this._month;
		var hDate = this._date;
		var julianDay = hDate + Math.ceil(29.5 * hMonth) + (hYear - 1) * 354
						+ Math.floor((3 + (11 * hYear)) / 30) + this._ISLAMIC_EPOCH - 1;

		var wjd = Math.floor(julianDay - 0.5) + 0.5,
			depoch = wjd - this._GREGORIAN_EPOCH,
			quadricent = Math.floor(depoch / 146097),
			dqc = this._mod(depoch, 146097),
			cent = Math.floor(dqc / 36524),
			dcent = this._mod(dqc, 36524),
			quad = Math.floor(dcent / 1461),
			dquad = this._mod(dcent, 1461),
			yindex = Math.floor(dquad / 365),
			year = (quadricent * 400) + (cent * 100) + (quad * 4) + yindex;
		if(!(cent == 4 || yindex == 4)){
			year++;
		}
		
		var gYearStart = this._GREGORIAN_EPOCH + (365 * (year - 1)) + Math.floor((year - 1) / 4)
						- ( Math.floor((year - 1) / 100)) + Math.floor((year - 1) / 400);
						
		var yearday = wjd - gYearStart;
		
		var tjd = (this._GREGORIAN_EPOCH - 1) + (365 * (year - 1)) + Math.floor((year - 1) / 4)
				-( Math.floor((year - 1) / 100)) + Math.floor((year - 1) / 400) + Math.floor( (739 / 12)
				+ ( (dd.isLeapYear(new Date(year,3,1)) ? -1 : -2)) + 1);
			
		var leapadj = ((wjd < tjd ) ? 0 : (dd.isLeapYear(new Date(year,3,1)) ? 1 : 2));
					
		var month = Math.floor((((yearday + leapadj) * 12) + 373) / 367);
		var tjd2 = (this._GREGORIAN_EPOCH - 1) + (365 * (year - 1))
					+ Math.floor((year - 1) / 4) - (Math.floor((year - 1) / 100))
					+ Math.floor((year - 1) / 400) + Math.floor((((367 * month) - 362) / 12)
					+ ((month <= 2) ? 0 : (dd.isLeapYear(new Date(year,month,1)) ? -1 : -2)) + 1);
					
		var day = (wjd - tjd2) + 1;

		var gdate = new Date(year, (month - 1), day, this._hours, this._minutes, this._seconds, this._milliseconds);

		return gdate;
	},

	//TODO: would it make more sense to make this a constructor option? or a static?
	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	fromGregorian:function(/*Date*/gdate){
		// summary: This function returns the equivalent Islamic Date value for the Gregorian Date
		// example:
		// |		var dateIslamic = new dojox.date.islamic.Date();
		// |		var dateGregorian = new Date(2008,10,12);
		// |		dateIslamic.fromGregorian(dateGregorian);

		var date = new Date(gdate);
		var gYear = date.getFullYear(),
			gMonth = date.getMonth(),
			gDay = date.getDate();
		
		var julianDay = (this._GREGORIAN_EPOCH - 1) + (365 * (gYear - 1)) + Math.floor((gYear - 1) / 4)
					+ (-Math.floor((gYear - 1) / 100)) + Math.floor((gYear - 1) / 400)
					+ Math.floor((((367 * (gMonth+1)) - 362) / 12)
					+ (((gMonth+1) <= 2) ? 0 : (dd.isLeapYear(date) ? -1 : -2)) + gDay);
		julianDay = Math.floor(julianDay) + 0.5;

		var days = julianDay - this._ISLAMIC_EPOCH;
		var hYear  = Math.floor( (30 * days + 10646) / 10631.0 );
		var hMonth = Math.ceil((days - 29 - this._yearStart(hYear)) / 29.5 );
		hMonth = Math.min(hMonth, 11);
		var hDay = Math.ceil(days - this._monthStart(hYear, hMonth)) + 1;

		this._date = hDay;
		this._month = hMonth;
		this._year = hYear;
		this._hours = date.getHours();
		this._minutes = date.getMinutes();
		this._seconds = date.getSeconds();
		this._milliseconds = date.getMilliseconds();
		this._day = date.getDay();
		return this;
	},
	
	valueOf:function(){
		// summary: This function returns The stored time value in milliseconds
		// since midnight, January 1, 1970 UTC

		return this.toGregorian().valueOf();
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_yearStart:function(/*Number*/year){
		//summary: return start of Islamic year
		return (year-1)*354 + Math.floor((3+11*year)/30.0);
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_monthStart:function(/*Number*/year, /*Number*/month){
		//summary: return the start of Islamic Month
		return Math.ceil(29.5*month) +
			(year-1)*354 + Math.floor((3+11*year)/30.0);
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_civilLeapYear:function(/*Number*/year){
		//summary: return Boolean value if Islamic leap year
		return (14 + 11 * year) % 30 < 11;
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	getDaysInIslamicMonth:function(/*Number*/month, /*Number*/ year){
		//summary: returns the number of days in the given Islamic Month
		var length = 0;
		length = 29 + ((month+1) % 2);
		if(month == 11 && this._civilLeapYear(year)){
			length++;
		}
		return length;
	},

	_mod:function(a, b){
		return a - (b * Math.floor(a / b));
	}
});

//TODOC
dojox.date.islamic.Date.getDaysInIslamicMonth = function(/*dojox.date.islamic.Date*/month){
	return new dojox.date.islamic.Date().getDaysInIslamicMonth(month.getMonth(),month.getFullYear()); // dojox.date.islamic.Date
};
return dojox.date.islamic.Date;
});

},
'dijit/_Templated':function(){
define([
	"./_WidgetBase",
	"./_TemplatedMixin",
	"./_WidgetsInTemplateMixin",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.extend lang.isArray
	"dojo/_base/kernel" // kernel.deprecated
], function(_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, array, declare, lang, kernel){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _WidgetsInTemplateMixin = dijit._WidgetsInTemplateMixin;
=====*/

	// module:
	//		dijit/_Templated
	// summary:
	//		Deprecated mixin for widgets that are instantiated from a template.

	// These arguments can be specified for widgets which are used in templates.
	// Since any widget can be specified as sub widgets in template, mix it
	// into the base widget class.  (This is a hack, but it's effective.)
	lang.extend(_WidgetBase, {
		waiRole: "",
		waiState:""
	});

	return declare("dijit._Templated", [_TemplatedMixin, _WidgetsInTemplateMixin], {
		// summary:
		//		Deprecated mixin for widgets that are instantiated from a template.
		//		Widgets should use _TemplatedMixin plus if necessary _WidgetsInTemplateMixin instead.

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  False by default.
		widgetsInTemplate: false,

		constructor: function(){
			kernel.deprecated(this.declaredClass + ": dijit._Templated deprecated, use dijit._TemplatedMixin and if necessary dijit._WidgetsInTemplateMixin", "", "2.0");
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){

			this.inherited(arguments);

			// Do deprecated waiRole and waiState
			var nodes = lang.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = lang.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];

				// waiRole, waiState
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					baseNode.setAttribute("role", role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					array.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							baseNode.setAttribute("aria-"+pair[0], pair[1]);
						}
					});
				}
			}
		}
	});
});

},
'com.ibm.bpm.coach/commonHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all common utilities

define(["dojo/_base/kernel","./CONSTANTS"], function(dojoKernel, CONSTANTS) {
				
	var commonHelper = new Object();
	
	commonHelper.INFO = "info";
	commonHelper.WARN = "warn";
	commonHelper.ERROR = "error";
	commonHelper.LOG = "log";
	
	// this API write the message directly to browser's console as long as browser's console exists
	commonHelper.consoleLog = function(message, type) {
		if(!console) {
			return;
		}
		
		if(commonHelper.INFO === type && !!console.info) {
			console.info(message);
		} else if(commonHelper.WARN === type && !!console.warn) {
			console.warn(message);
		} else if(commonHelper.ERROR === type && !!console.error) {
			console.error(message);
		} else if(!!console.log){
			console.log(message);
		}
	}
	
	commonHelper.timestamp = function(action) {
		if(commonHelper.isDebugMode()) {
			commonHelper.consoleLog("Performance: " + (new Date()).getTime() + " -- " + action, commonHelper.INFO);
		}
	}
	
	commonHelper.isDebugMode = function() {
		if(!!dojoKernel.config) {
			return dojoKernel.config.isDebug;
		}
		
		return false;
	}
	
	commonHelper.getDojoVersion = function() {
		return dojoKernel.version;
	}	
	
	commonHelper.shallowCloneMap = function(aMap) {
		var clonedMap = {};
		for(var key in aMap) {
			clonedMap[key] = aMap[key];
		}
		return clonedMap;
	}
	
	commonHelper.isMapEmpty = function(map) {
	   for(var key in map) {
		  if (map.hasOwnProperty(key)) {
			 return false;
		  }
	   }
	   return true;
	}
	
	// return value of commonHelper.isValuesEqual()
	commonHelper.YES = 1;
	commonHelper.UNKNOWN = 0;
	commonHelper.NO = -1;
	
	/*
	 Compare two provided values.
	 return -
	 	commonHelper.YES: value1 is equal to value2
	 	commonHelper.NO: value1 is not equal to value2.
	 	commonHelper.UNKNOWN: the brief check can not determine if value1 and value2 are equal
	*/
	commonHelper.isValuesEqual = function(value1, value2) {
		// only compare when value1 and value2 are primitive types.
		// if value1 and value2 points to same object, this function returns commonHelper.UNKNOWN instead of commonHelper.YES 
		// in the sake of backward compatibility (some stock control didn't use coach API properly in release 8000 and 8010)
		var typeOfValue = typeof(value1);
		if(typeOfValue == 'string' || typeOfValue == 'number' || typeOfValue == 'boolean' || typeOfValue == 'undefined' || typeOfValue == 'null') {
			if(value1 === value2) {
				return commonHelper.YES;
			} else {
				return commonHelper.NO;
			}
		}
		return commonHelper.UNKNOWN;
	}
	
	/*
	 Check if bindingStrA starts with bindingStrB in term of path.
	 If the data represented by bindingStrA is sub-object of the data represented by bindingStrB, return true. 
	 If bindingStrA is equal to bindingStrB, return true if includeEqualCase is true, Otherwise return false.
	 Return false for all other cases.
	 
	 Example:
	 	bindingStrA			bindingStrB		return
	 	-----------------------------------------------
	 	local.abc			local.abc		true(if includeEqualCase == true), false otherwise
	 	*					null/undefined	false;
	 	null/undefined		*				false;
	 	local.def			local.abc		false
	 	local.abcdef		local.abc		false
	 	local.abc.def		local.abc		true
	 	local.abc.d.e		local.abc		true
	 	local.abc[2]		local.abc		true
	*/
	commonHelper.startsWithPath = function(bindingStrA, bindingStrB, includeEqualCase) {
		if(bindingStrA == bindingStrB) {
			if(includeEqualCase) {
				return true;
			}
		} else if(!!bindingStrA && !!bindingStrB && (bindingStrA.indexOf(bindingStrB) == 0)) {
			// still need fine check, because ("local.abcdef", "local.abc") also passes above test
			var difference = bindingStrA.replace(bindingStrB, "");
			// difference is not blank string because bindingStrA != bindingStrB
			var firstCharacter = difference.substring(0, 1);
			
			if(firstCharacter == "." || firstCharacter == "[") {
				//("local.abc.def", "local.abc"), or ("local.abc[2]", "local.abc")
				return true;
			}
		}
		
		return false;
	}		
	
	commonHelper.timestamp("commonHelper is created");
	
	return commonHelper;
});

},
'dojo/query':function(){
define(["./_base/kernel", "./has", "./dom", "./on", "./_base/array", "./_base/lang", "./selector/_loader", "./selector/_loader!default"],
	function(dojo, has, dom, on, array, lang, loader, defaultEngine){
"use strict";

	has.add("array-extensible", function(){
		// test to see if we can extend an array (not supported in old IE)
		return lang.delegate([], {length: 1}).length == 1 && !has("bug-for-in-skips-shadowed");
	});
	
	var ap = Array.prototype, aps = ap.slice, apc = ap.concat, forEach = array.forEach;

	var tnl = function(/*Array*/ a, /*dojo.NodeList?*/ parent, /*Function?*/ NodeListCtor){
		// summary:
		//		decorate an array to make it look like a `dojo.NodeList`.
		// a:
		//		Array of nodes to decorate.
		// parent:
		//		An optional parent NodeList that generated the current
		//		list of nodes. Used to call _stash() so the parent NodeList
		//		can be accessed via end() later.
		// NodeListCtor:
		//		An optional constructor function to use for any
		//		new NodeList calls. This allows a certain chain of
		//		NodeList calls to use a different object than dojo.NodeList.
		var nodeList = new (NodeListCtor || this._NodeListCtor || nl)(a);
		return parent ? nodeList._stash(parent) : nodeList;
	};

	var loopBody = function(f, a, o){
		a = [0].concat(aps.call(a, 0));
		o = o || dojo.global;
		return function(node){
			a[0] = node;
			return f.apply(o, a);
		};
	};

	// adapters

	var adaptAsForEach = function(f, o){
		// summary:
		//		adapts a single node function to be used in the forEach-type
		//		actions. The initial object is returned from the specialized
		//		function.
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			this.forEach(loopBody(f, arguments, o));
			return this;	// Object
		};
	};

	var adaptAsMap = function(f, o){
		// summary:
		//		adapts a single node function to be used in the map-type
		//		actions. The return is a new array of values, as via `dojo.map`
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			return this.map(loopBody(f, arguments, o));
		};
	};

	var adaptAsFilter = function(f, o){
		// summary:
		//		adapts a single node function to be used in the filter-type actions
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			return this.filter(loopBody(f, arguments, o));
		};
	};

	var adaptWithCondition = function(f, g, o){
		// summary:
		//		adapts a single node function to be used in the map-type
		//		actions, behaves like forEach() or map() depending on arguments
		// f: Function
		//		a function to adapt
		// g: Function
		//		a condition function, if true runs as map(), otherwise runs as forEach()
		// o: Object?
		//		an optional context for f and g
		return function(){
			var a = arguments, body = loopBody(f, a, o);
			if(g.call(o || dojo.global, a)){
				return this.map(body);	// self
			}
			this.forEach(body);
			return this;	// self
		};
	};

	var NodeList = function(array){
		// summary:
		//		dojo.NodeList is an of Array-like object which adds syntactic
		//		sugar for chaining, common iteration operations, animation, and
		//		node manipulation. NodeLists are most often returned as the
		//		result of dojo.query() calls.
		// description:
		//		dojo.NodeList instances provide many utilities that reflect
		//		core Dojo APIs for Array iteration and manipulation, DOM
		//		manipulation, and event handling. Instead of needing to dig up
		//		functions in the dojo.* namespace, NodeLists generally make the
		//		full power of Dojo available for DOM manipulation tasks in a
		//		simple, chainable way.
		// example:
		//		create a node list from a node
		//		|	new dojo.NodeList(dojo.byId("foo"));
		// example:
		//		get a NodeList from a CSS query and iterate on it
		//		|	var l = dojo.query(".thinger");
		//		|	l.forEach(function(node, index, nodeList){
		//		|		console.log(index, node.innerHTML);
		//		|	});
		// example:
		//		use native and Dojo-provided array methods to manipulate a
		//		NodeList without needing to use dojo.* functions explicitly:
		//		|	var l = dojo.query(".thinger");
		//		|	// since NodeLists are real arrays, they have a length
		//		|	// property that is both readable and writable and
		//		|	// push/pop/shift/unshift methods
		//		|	console.log(l.length);
		//		|	l.push(dojo.create("span"));
		//		|
		//		|	// dojo's normalized array methods work too:
		//		|	console.log( l.indexOf(dojo.byId("foo")) );
		//		|	// ...including the special "function as string" shorthand
		//		|	console.log( l.every("item.nodeType == 1") );
		//		|
		//		|	// NodeLists can be [..] indexed, or you can use the at()
		//		|	// function to get specific items wrapped in a new NodeList:
		//		|	var node = l[3]; // the 4th element
		//		|	var newList = l.at(1, 3); // the 2nd and 4th elements
		// example:
		//		the style functions you expect are all there too:
		//		|	// style() as a getter...
		//		|	var borders = dojo.query(".thinger").style("border");
		//		|	// ...and as a setter:
		//		|	dojo.query(".thinger").style("border", "1px solid black");
		//		|	// class manipulation
		//		|	dojo.query("li:nth-child(even)").addClass("even");
		//		|	// even getting the coordinates of all the items
		//		|	var coords = dojo.query(".thinger").coords();
		// example:
		//		DOM manipulation functions from the dojo.* namespace area also
		//		available:
		//		|	// remove all of the elements in the list from their
		//		|	// parents (akin to "deleting" them from the document)
		//		|	dojo.query(".thinger").orphan();
		//		|	// place all elements in the list at the front of #foo
		//		|	dojo.query(".thinger").place("foo", "first");
		// example:
		//		Event handling couldn't be easier. `dojo.connect` is mapped in,
		//		and shortcut handlers are provided for most DOM events:
		//		|	// like dojo.connect(), but with implicit scope
		//		|	dojo.query("li").connect("onclick", console, "log");
		//		|
		//		|	// many common event handlers are already available directly:
		//		|	dojo.query("li").onclick(console, "log");
		//		|	var toggleHovered = dojo.hitch(dojo, "toggleClass", "hovered");
		//		|	dojo.query("p")
		//		|		.onmouseenter(toggleHovered)
		//		|		.onmouseleave(toggleHovered);
		// example:
		//		chainability is a key advantage of NodeLists:
		//		|	dojo.query(".thinger")
		//		|		.onclick(function(e){ /* ... */ })
		//		|		.at(1, 3, 8) // get a subset
		//		|			.style("padding", "5px")
		//		|			.forEach(console.log);
		var isNew = this instanceof nl && has("array-extensible");
		if(typeof array == "number"){
			array = Array(array);
		}
		var nodeArray = (array && "length" in array) ? array : arguments;
		if(isNew || !nodeArray.sort){
			// make sure it's a real array before we pass it on to be wrapped 
			var target = isNew ? this : [],
				l = target.length = nodeArray.length;
			for(var i = 0; i < l; i++){
				target[i] = nodeArray[i];
			}
			if(isNew){
				// called with new operator, this means we are going to use this instance and push
				// the nodes on to it. This is usually much faster since the NodeList properties
				//	don't need to be copied (unless the list of nodes is extremely large).
				return target;
			}
			nodeArray = target;
		}
		// called without new operator, use a real array and copy prototype properties,
		// this is slower and exists for back-compat. Should be removed in 2.0.
		lang._mixin(nodeArray, nlp);
		nodeArray._NodeListCtor = function(array){
			// call without new operator to preserve back-compat behavior
			return nl(array);
		};
		return nodeArray;
	};
	
	var nl = NodeList, nlp = nl.prototype = 
		has("array-extensible") ? [] : {};// extend an array if it is extensible

	// expose adapters and the wrapper as private functions

	nl._wrap = nlp._wrap = tnl;
	nl._adaptAsMap = adaptAsMap;
	nl._adaptAsForEach = adaptAsForEach;
	nl._adaptAsFilter  = adaptAsFilter;
	nl._adaptWithCondition = adaptWithCondition;

	// mass assignment

	// add array redirectors
	forEach(["slice", "splice"], function(name){
		var f = ap[name];
		//Use a copy of the this array via this.slice() to allow .end() to work right in the splice case.
		// CANNOT apply ._stash()/end() to splice since it currently modifies
		// the existing this array -- it would break backward compatibility if we copy the array before
		// the splice so that we can use .end(). So only doing the stash option to this._wrap for slice.
		nlp[name] = function(){ return this._wrap(f.apply(this, arguments), name == "slice" ? this : null); };
	});
	// concat should be here but some browsers with native NodeList have problems with it

	// add array.js redirectors
	forEach(["indexOf", "lastIndexOf", "every", "some"], function(name){
		var f = array[name];
		nlp[name] = function(){ return f.apply(dojo, [this].concat(aps.call(arguments, 0))); };
	});

	/*===== var NodeList = dojo.NodeList; =====*/
	lang.extend(NodeList, {
		// copy the constructors
		constructor: nl,
		_NodeListCtor: nl,
		toString: function(){
			// Array.prototype.toString can't be applied to objects, so we use join
			return this.join(",");
		},
		_stash: function(parent){
			// summary:
			//		private function to hold to a parent NodeList. end() to return the parent NodeList.
			//
			// example:
			// How to make a `dojo.NodeList` method that only returns the third node in
			// the dojo.NodeList but allows access to the original NodeList by using this._stash:
			//	|	dojo.extend(dojo.NodeList, {
			//	|		third: function(){
			//	|			var newNodeList = dojo.NodeList(this[2]);
			//	|			return newNodeList._stash(this);
			//	|		}
			//	|	});
			//	|	// then see how _stash applies a sub-list, to be .end()'ed out of
			//	|	dojo.query(".foo")
			//	|		.third()
			//	|			.addClass("thirdFoo")
			//	|		.end()
			//	|		// access to the orig .foo list
			//	|		.removeClass("foo")
			//	|
			//
			this._parent = parent;
			return this; //dojo.NodeList
		},

		on: function(eventName, listener){
			// summary:
			//		Listen for events on the nodes in the NodeList. Basic usage is:
			//		| query(".my-class").on("click", listener);
			// 		This supports event delegation by using selectors as the first argument with the event names as
			//		pseudo selectors. For example:
			//		| dojo.query("#my-list").on("li:click", listener);
			//		This will listen for click events within <li> elements that are inside the #my-list element.
			//		Because on supports CSS selector syntax, we can use comma-delimited events as well:
			//		| dojo.query("#my-list").on("li button:mouseover, li:click", listener);
			var handles = this.map(function(node){
				return on(node, eventName, listener); // TODO: apply to the NodeList so the same selector engine is used for matches
			});
			handles.remove = function(){
				for(var i = 0; i < handles.length; i++){
					handles[i].remove();
				}
			};
			return handles;
		},

		end: function(){
			// summary:
			//		Ends use of the current `dojo.NodeList` by returning the previous dojo.NodeList
			//		that generated the current dojo.NodeList.
			// description:
			//		Returns the `dojo.NodeList` that generated the current `dojo.NodeList`. If there
			//		is no parent dojo.NodeList, an empty dojo.NodeList is returned.
			// example:
			//	|	dojo.query("a")
			//	|		.filter(".disabled")
			//	|			// operate on the anchors that only have a disabled class
			//	|			.style("color", "grey")
			//	|		.end()
			//	|		// jump back to the list of anchors
			//	|		.style(...)
			//
			if(this._parent){
				return this._parent;
			}else{
				//Just return empty list.
				return new this._NodeListCtor(0);
			}
		},

		// http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array#Methods

		// FIXME: handle return values for #3244
		//		http://trac.dojotoolkit.org/ticket/3244

		// FIXME:
		//		need to wrap or implement:
		//			join (perhaps w/ innerHTML/outerHTML overload for toString() of items?)
		//			reduce
		//			reduceRight

		/*=====
		slice: function(begin, end){
			// summary:
			//		Returns a new NodeList, maintaining this one in place
			// description:
			//		This method behaves exactly like the Array.slice method
			//		with the caveat that it returns a dojo.NodeList and not a
			//		raw Array. For more details, see Mozilla's (slice
			//		documentation)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:slice]
			// begin: Integer
			//		Can be a positive or negative integer, with positive
			//		integers noting the offset to begin at, and negative
			//		integers denoting an offset from the end (i.e., to the left
			//		of the end)
			// end: Integer?
			//		Optional parameter to describe what position relative to
			//		the NodeList's zero index to end the slice at. Like begin,
			//		can be positive or negative.
			return this._wrap(a.slice.apply(this, arguments));
		},

		splice: function(index, howmany, item){
			// summary:
			//		Returns a new NodeList, manipulating this NodeList based on
			//		the arguments passed, potentially splicing in new elements
			//		at an offset, optionally deleting elements
			// description:
			//		This method behaves exactly like the Array.splice method
			//		with the caveat that it returns a dojo.NodeList and not a
			//		raw Array. For more details, see Mozilla's (splice
			//		documentation)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:splice]
			//		For backwards compatibility, calling .end() on the spliced NodeList
			//		does not return the original NodeList -- splice alters the NodeList in place.
			// index: Integer
			//		begin can be a positive or negative integer, with positive
			//		integers noting the offset to begin at, and negative
			//		integers denoting an offset from the end (i.e., to the left
			//		of the end)
			// howmany: Integer?
			//		Optional parameter to describe what position relative to
			//		the NodeList's zero index to end the slice at. Like begin,
			//		can be positive or negative.
			// item: Object...?
			//		Any number of optional parameters may be passed in to be
			//		spliced into the NodeList
			// returns:
			//		dojo.NodeList
			return this._wrap(a.splice.apply(this, arguments));
		},

		indexOf: function(value, fromIndex){
			// summary:
			//		see dojo.indexOf(). The primary difference is that the acted-on
			//		array is implicitly this NodeList
			// value: Object:
			//		The value to search for.
			// fromIndex: Integer?:
			//		The location to start searching from. Optional. Defaults to 0.
			// description:
			//		For more details on the behavior of indexOf, see Mozilla's
			//		(indexOf
			//		docs)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:indexOf]
			// returns:
			//		Positive Integer or 0 for a match, -1 of not found.
			return d.indexOf(this, value, fromIndex); // Integer
		},

		lastIndexOf: function(value, fromIndex){
			// summary:
			//		see dojo.lastIndexOf(). The primary difference is that the
			//		acted-on array is implicitly this NodeList
			// description:
			//		For more details on the behavior of lastIndexOf, see
			//		Mozilla's (lastIndexOf
			//		docs)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:lastIndexOf]
			// value: Object
			//		The value to search for.
			// fromIndex: Integer?
			//		The location to start searching from. Optional. Defaults to 0.
			// returns:
			//		Positive Integer or 0 for a match, -1 of not found.
			return d.lastIndexOf(this, value, fromIndex); // Integer
		},

		every: function(callback, thisObject){
			// summary:
			//		see `dojo.every()` and the (Array.every
			//		docs)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:every].
			//		Takes the same structure of arguments and returns as
			//		dojo.every() with the caveat that the passed array is
			//		implicitly this NodeList
			// callback: Function: the callback
			// thisObject: Object?: the context
			return d.every(this, callback, thisObject); // Boolean
		},

		some: function(callback, thisObject){
			// summary:
			//		Takes the same structure of arguments and returns as
			//		`dojo.some()` with the caveat that the passed array is
			//		implicitly this NodeList.  See `dojo.some()` and Mozilla's
			//		(Array.some
			//		documentation)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:some].
			// callback: Function: the callback
			// thisObject: Object?: the context
			return d.some(this, callback, thisObject); // Boolean
		},
		=====*/

		concat: function(item){
			// summary:
			//		Returns a new NodeList comprised of items in this NodeList
			//		as well as items passed in as parameters
			// description:
			//		This method behaves exactly like the Array.concat method
			//		with the caveat that it returns a `dojo.NodeList` and not a
			//		raw Array. For more details, see the (Array.concat
			//		docs)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:concat]
			// item: Object?
			//		Any number of optional parameters may be passed in to be
			//		spliced into the NodeList
			// returns:
			//		dojo.NodeList

			//return this._wrap(apc.apply(this, arguments));
			// the line above won't work for the native NodeList :-(

			// implementation notes:
			// 1) Native NodeList is not an array, and cannot be used directly
			// in concat() --- the latter doesn't recognize it as an array, and
			// does not inline it, but append as a single entity.
			// 2) On some browsers (e.g., Safari) the "constructor" property is
			// read-only and cannot be changed. So we have to test for both
			// native NodeList and dojo.NodeList in this property to recognize
			// the node list.

			var t = lang.isArray(this) ? this : aps.call(this, 0),
				m = array.map(arguments, function(a){
					return a && !lang.isArray(a) &&
						(typeof NodeList != "undefined" && a.constructor === NodeList || a.constructor === this._NodeListCtor) ?
							aps.call(a, 0) : a;
				});
			return this._wrap(apc.apply(t, m), this);	// dojo.NodeList
		},

		map: function(/*Function*/ func, /*Function?*/ obj){
			// summary:
			//		see dojo.map(). The primary difference is that the acted-on
			//		array is implicitly this NodeList and the return is a
			//		dojo.NodeList (a subclass of Array)
			///return d.map(this, func, obj, d.NodeList); // dojo.NodeList
			return this._wrap(array.map(this, func, obj), this); // dojo.NodeList
		},

		forEach: function(callback, thisObj){
			// summary:
			//		see `dojo.forEach()`. The primary difference is that the acted-on
			//		array is implicitly this NodeList. If you want the option to break out
			//		of the forEach loop, use every() or some() instead.
			forEach(this, callback, thisObj);
			// non-standard return to allow easier chaining
			return this; // dojo.NodeList
		},
		filter: function(/*String|Function*/ filter){
			// summary:
			//		"masks" the built-in javascript filter() method (supported
			//		in Dojo via `dojo.filter`) to support passing a simple
			//		string filter in addition to supporting filtering function
			//		objects.
			// filter:
			//		If a string, a CSS rule like ".thinger" or "div > span".
			// example:
			//		"regular" JS filter syntax as exposed in dojo.filter:
			//		|	dojo.query("*").filter(function(item){
			//		|		// highlight every paragraph
			//		|		return (item.nodeName == "p");
			//		|	}).style("backgroundColor", "yellow");
			// example:
			//		the same filtering using a CSS selector
			//		|	dojo.query("*").filter("p").styles("backgroundColor", "yellow");

			var a = arguments, items = this, start = 0;
			if(typeof filter == "string"){ // inline'd type check
				items = query._filterResult(this, a[0]);
				if(a.length == 1){
					// if we only got a string query, pass back the filtered results
					return items._stash(this); // dojo.NodeList
				}
				// if we got a callback, run it over the filtered items
				start = 1;
			}
			return this._wrap(array.filter(items, a[start], a[start + 1]), this);	// dojo.NodeList
		},
		instantiate: function(/*String|Object*/ declaredClass, /*Object?*/ properties){
			// summary:
			//		Create a new instance of a specified class, using the
			//		specified properties and each node in the nodeList as a
			//		srcNodeRef.
			// example:
			//		Grabs all buttons in the page and converts them to diji.form.Buttons.
			//	|	var buttons = dojo.query("button").instantiate("dijit.form.Button", {showLabel: true});
			var c = lang.isFunction(declaredClass) ? declaredClass : lang.getObject(declaredClass);
			properties = properties || {};
			return this.forEach(function(node){
				new c(properties, node);
			});	// dojo.NodeList
		},
		at: function(/*===== index =====*/){
			// summary:
			//		Returns a new NodeList comprised of items in this NodeList
			//		at the given index or indices.
			//
			// index: Integer...
			//		One or more 0-based indices of items in the current
			//		NodeList. A negative index will start at the end of the
			//		list and go backwards.
			//
			// example:
			//	Shorten the list to the first, second, and third elements
			//	|	dojo.query("a").at(0, 1, 2).forEach(fn);
			//
			// example:
			//	Retrieve the first and last elements of a unordered list:
			//	|	dojo.query("ul > li").at(0, -1).forEach(cb);
			//
			// example:
			//	Do something for the first element only, but end() out back to
			//	the original list and continue chaining:
			//	|	dojo.query("a").at(0).onclick(fn).end().forEach(function(n){
			//	|		console.log(n); // all anchors on the page.
			//	|	})
			//
			// returns:
			//		dojo.NodeList
			var t = new this._NodeListCtor(0);
			forEach(arguments, function(i){
				if(i < 0){ i = this.length + i; }
				if(this[i]){ t.push(this[i]); }
			}, this);
			return t._stash(this); // dojo.NodeList
		}
	});


/*===== 
dojo.query = function(selector, context){
	// summary:
	//		This modules provides DOM querying functionality. The module export is a function
	//		that can be used to query for DOM nodes by CSS selector and returns a dojo.NodeList
	//		representing the matching nodes.
	//
	// selector: String
	//		A CSS selector to search for.
	// context: String|DomNode?
	//		An optional context to limit the searching scope. Only nodes under `context` will be 
	//		scanned. 
	// 
	//	example:
	//		add an onclick handler to every submit button in the document
	//		which causes the form to be sent via Ajax instead:
	//	|	define(["dojo/query"], function(query){
	// 	|	query("input[type='submit']").on("click", function(e){
	//	|		dojo.stopEvent(e); // prevent sending the form
	//	|		var btn = e.target;
	//	|		dojo.xhrPost({
	//	|			form: btn.form,
	//	|			load: function(data){
	//	|				// replace the form with the response
	//	|				var div = dojo.doc.createElement("div");
	//	|				dojo.place(div, btn.form, "after");
	//	|				div.innerHTML = data;
	//	|				dojo.style(btn.form, "display", "none");
	//	|			}
	//	|		});
	//	|	}); 
	//
	// description:
	//		dojo/query is responsible for loading the appropriate query engine and wrapping 
	//		its results with a `dojo.NodeList`. You can use dojo/query with a specific selector engine
	//		by using it as a plugin. For example, if you installed the sizzle package, you could
	//		use it as the selector engine with:
	//		|	define("dojo/query!sizzle", function(query){
	//		|		query("div")...
	//
	//		The id after the ! can be a module id of the selector engine or one of the following values:
	//		|	+ acme: This is the default engine used by Dojo base, and will ensure that the full
	//		|	Acme engine is always loaded. 
	//		|	
	//		|	+ css2: If the browser has a native selector engine, this will be used, otherwise a
	//		|	very minimal lightweight selector engine will be loaded that can do simple CSS2 selectors
	//		|	(by #id, .class, tag, and [name=value] attributes, with standard child or descendant (>)
	//		|	operators) and nothing more.
	//		|
	//		|	+ css2.1: If the browser has a native selector engine, this will be used, otherwise the
	//		|	full Acme engine will be loaded. 
	//		|	
	//		|	+ css3: If the browser has a native selector engine with support for CSS3 pseudo
	//		|	selectors (most modern browsers except IE8), this will be used, otherwise the
	//		|	full Acme engine will be loaded.
	//		|	
	//		|	+ Or the module id of a selector engine can be used to explicitly choose the selector engine
	//		
	//		For example, if you are using CSS3 pseudo selectors in module, you can specify that
	//		you will need support them with:
	//		|	define("dojo/query!css3", function(query){
	//		|		query('#t > h3:nth-child(odd)')...
	//
	//		You can also choose the selector engine/load configuration by setting the <FIXME:what is the configuration setting?>.
	//		For example:
	//		|	<script data-dojo-config="query-selector:'css3'" src="dojo.js"></script>
	//		
	return new dojo.NodeList(); // dojo.NodeList
};
=====*/

function queryForEngine(engine, NodeList){
	var query = function(/*String*/ query, /*String|DOMNode?*/ root){
		//	summary:
		//		Returns nodes which match the given CSS selector, searching the
		//		entire document by default but optionally taking a node to scope
		//		the search by. Returns an instance of dojo.NodeList.
		if(typeof root == "string"){
			root = dom.byId(root);
			if(!root){
				return new NodeList([]);
			}
		}
		var results = typeof query == "string" ? engine(query, root) : query ? query.orphan ? query : [query] : [];
		if(results.orphan){
			// already wrapped
			return results; 
		}
		return new NodeList(results);
	};
	query.matches = engine.match || function(node, selector, root){
		// summary:
		//		Test to see if a node matches a selector
		return query.filter([node], selector, root).length > 0;
	};
	// the engine provides a filtering function, use it to for matching
	query.filter = engine.filter || function(nodes, selector, root){
		// summary:
		//		Filters an array of nodes. Note that this does not guarantee to return a dojo.NodeList, just an array.
		return query(selector, root).filter(function(node){
			return array.indexOf(nodes, node) > -1;
		});
	};
	if(typeof engine != "function"){
		var search = engine.search;
		engine = function(selector, root){
			// Slick does it backwards (or everyone else does it backwards, probably the latter)
			return search(root || document, selector);
		};
	}
	return query;
}
var query = queryForEngine(defaultEngine, NodeList);
// the query that is returned from this module is slightly different than dojo.query,
// because dojo.query has to maintain backwards compatibility with returning a
// true array which has performance problems. The query returned from the module
// does not use true arrays, but rather inherits from Array, making it much faster to
// instantiate.
dojo.query = queryForEngine(defaultEngine, function(array){
	// call it without the new operator to invoke the back-compat behavior that returns a true array
	return NodeList(array);
});

query.load = /*===== dojo.query.load= ======*/ function(id, parentRequire, loaded, config){
	// summary: can be used as AMD plugin to conditionally load new query engine
	// example:
	//	|	define(["dojo/query!custom"], function(qsa){ 
	//	|		// loaded selector/custom.js as engine
	//	|		qsa("#foobar").forEach(...);
	//	|	});
	loader.load(id, parentRequire, function(engine){
		loaded(queryForEngine(engine, NodeList));
	});
};

dojo._filterQueryResult = query._filterResult = function(nodes, selector, root){
	return new NodeList(query.filter(nodes, selector, root));
};
dojo.NodeList = query.NodeList = NodeList;
return query;
});

},
'dojo/dom-class':function(){
define(["./_base/lang", "./_base/array", "./dom"], function(lang, array, dom){
	// module:
	//		dojo/dom-class
	// summary:
	//		This module defines the core dojo DOM class API.

	var className = "className";

	/* Part I of classList-based implementation is preserved here for posterity
	var classList = "classList";
	has.add("dom-classList", function(){
		return classList in document.createElement("p");
	});
	*/

	// =============================
	// (CSS) Class Functions
	// =============================

	/*=====
	dojo.hasClass = function(node, classStr){
		// summary:
		//		Returns whether or not the specified classes are a portion of the
		//		class list currently applied to the node.
		//
		// node: String|DOMNode
		//		String ID or DomNode reference to check the class for.
		//
		// classStr: String
		//		A string class name to look for.
		//
		// returns: Boolean
		//
		// example:
		//		Do something if a node with id="someNode" has class="aSillyClassName" present
		//	|	if(dojo.hasClass("someNode","aSillyClassName")){ ... }
	};
	=====*/

	/*=====
	dojo.addClass = function(node, classStr){
		// summary:
		//		Adds the specified classes to the end of the class list on the
		//		passed node. Will not re-apply duplicate classes.
		//
		// node: String|DOMNode
		//		String ID or DomNode reference to add a class string too
		//
		// classStr: String|Array
		//		A String class name to add, or several space-separated class names,
		//		or an array of class names.
		//
		// example:
		//		Add a class to some node:
		//	|	dojo.addClass("someNode", "anewClass");
		//
		// example:
		//		Add two classes at once:
		//	|	dojo.addClass("someNode", "firstClass secondClass");
		//
		// example:
		//		Add two classes at once (using array):
		//	|	dojo.addClass("someNode", ["firstClass", "secondClass"]);
		//
		// example:
		//		Available in `dojo.NodeList` for multiple additions
		//	|	dojo.query("ul > li").addClass("firstLevel");
	};
	=====*/

	/*=====
	dojo.removeClass = function(node, classStr){
		// summary:
		//		Removes the specified classes from node. No `dojo.hasClass`
		//		check is required.
		//
		// node: String|DOMNode
		//		String ID or DomNode reference to remove the class from.
		//
		// classStr: String|Array
		//		An optional String class name to remove, or several space-separated
		//		class names, or an array of class names. If omitted, all class names
		//		will be deleted.
		//
		// example:
		//		Remove a class from some node:
		//	|	dojo.removeClass("someNode", "firstClass");
		//
		// example:
		//		Remove two classes from some node:
		//	|	dojo.removeClass("someNode", "firstClass secondClass");
		//
		// example:
		//		Remove two classes from some node (using array):
		//	|	dojo.removeClass("someNode", ["firstClass", "secondClass"]);
		//
		// example:
		//		Remove all classes from some node:
		//	|	dojo.removeClass("someNode");
		//
		// example:
		//		Available in `dojo.NodeList()` for multiple removal
		//	|	dojo.query(".foo").removeClass("foo");
	};
	=====*/

	/*=====
	dojo.replaceClass = function(node, addClassStr, removeClassStr){
		// summary:
		//		Replaces one or more classes on a node if not present.
		//		Operates more quickly than calling dojo.removeClass and dojo.addClass
		//
		// node: String|DOMNode
		//		String ID or DomNode reference to remove the class from.
		//
		// addClassStr: String|Array
		//		A String class name to add, or several space-separated class names,
		//		or an array of class names.
		//
		// removeClassStr: String|Array?
		//		A String class name to remove, or several space-separated class names,
		//		or an array of class names.
		//
		// example:
		//	|	dojo.replaceClass("someNode", "add1 add2", "remove1 remove2");
		//
		// example:
		//	Replace all classes with addMe
		//	|	dojo.replaceClass("someNode", "addMe");
		//
		// example:
		//	Available in `dojo.NodeList()` for multiple toggles
		//	|	dojo.query(".findMe").replaceClass("addMe", "removeMe");
	};
	=====*/

	/*=====
	dojo.toggleClass = function(node, classStr, condition){
		// summary:
		//		Adds a class to node if not present, or removes if present.
		//		Pass a boolean condition if you want to explicitly add or remove.
		//      Returns the condition that was specified directly or indirectly.
		//
		// node: String|DOMNode
		//		String ID or DomNode reference to toggle a class string
		//
		// classStr: String|Array
		//		A String class name to toggle, or several space-separated class names,
		//		or an array of class names.
		//
		// condition:
		//		If passed, true means to add the class, false means to remove.
		//      Otherwise dojo.hasClass(node, classStr) is used to detect the class presence.
		//
		// example:
		//	|	dojo.toggleClass("someNode", "hovered");
		//
		// example:
		//		Forcefully add a class
		//	|	dojo.toggleClass("someNode", "hovered", true);
		//
		// example:
		//		Available in `dojo.NodeList()` for multiple toggles
		//	|	dojo.query(".toggleMe").toggleClass("toggleMe");
	};
	=====*/

	var cls, // exports object
		spaces = /\s+/, a1 = [""];

	function str2array(s){
		if(typeof s == "string" || s instanceof String){
			if(s && !spaces.test(s)){
				a1[0] = s;
				return a1;
			}
			var a = s.split(spaces);
			if(a.length && !a[0]){
				a.shift();
			}
			if(a.length && !a[a.length - 1]){
				a.pop();
			}
			return a;
		}
		// assumed to be an array
		if(!s){
			return [];
		}
		return array.filter(s, function(x){ return x; });
	}

	/* Part II of classList-based implementation is preserved here for posterity
	if(has("dom-classList")){
		// new classList version
		cls = {
			contains: function containsClass(node, classStr){
				var clslst = classStr && dom.byId(node)[classList];
				return clslst && clslst.contains(classStr); // Boolean
			},

			add: function addClass(node, classStr){
				node = dom.byId(node);
				classStr = str2array(classStr);
				for(var i = 0, len = classStr.length; i < len; ++i){
					node[classList].add(classStr[i]);
				}
			},

			remove: function removeClass(node, classStr){
				node = dom.byId(node);
				if(classStr === undefined){
					node[className] = "";
				}else{
					classStr = str2array(classStr);
					for(var i = 0, len = classStr.length; i < len; ++i){
						node[classList].remove(classStr[i]);
					}
				}
			},

			replace: function replaceClass(node, addClassStr, removeClassStr){
				node = dom.byId(node);
				if(removeClassStr === undefined){
					node[className] = "";
				}else{
					removeClassStr = str2array(removeClassStr);
					for(var i = 0, len = removeClassStr.length; i < len; ++i){
						node[classList].remove(removeClassStr[i]);
					}
				}
				addClassStr = str2array(addClassStr);
				for(i = 0, len = addClassStr.length; i < len; ++i){
					node[classList].add(addClassStr[i]);
				}
			},

			toggle: function toggleClass(node, classStr, condition){
				node = dom.byId(node);
				if(condition === undefined){
					classStr = str2array(classStr);
					for(var i = 0, len = classStr.length; i < len; ++i){
						node[classList].toggle(classStr[i]);
					}
				}else{
					cls[condition ? "add" : "remove"](node, classStr);
				}
				return condition;   // Boolean
			}
		}
	}
	*/

	// regular DOM version
	var fakeNode = {};  // for effective replacement
	cls = {
		contains: function containsClass(/*DomNode|String*/node, /*String*/classStr){
			return ((" " + dom.byId(node)[className] + " ").indexOf(" " + classStr + " ") >= 0); // Boolean
		},

		add: function addClass(/*DomNode|String*/node, /*String|Array*/classStr){
			node = dom.byId(node);
			classStr = str2array(classStr);
			var cls = node[className], oldLen;
			cls = cls ? " " + cls + " " : " ";
			oldLen = cls.length;
			for(var i = 0, len = classStr.length, c; i < len; ++i){
				c = classStr[i];
				if(c && cls.indexOf(" " + c + " ") < 0){
					cls += c + " ";
				}
			}
			if(oldLen < cls.length){
				node[className] = cls.substr(1, cls.length - 2);
			}
		},

		remove: function removeClass(/*DomNode|String*/node, /*String|Array?*/classStr){
			node = dom.byId(node);
			var cls;
			if(classStr !== undefined){
				classStr = str2array(classStr);
				cls = " " + node[className] + " ";
				for(var i = 0, len = classStr.length; i < len; ++i){
					cls = cls.replace(" " + classStr[i] + " ", " ");
				}
				cls = lang.trim(cls);
			}else{
				cls = "";
			}
			if(node[className] != cls){ node[className] = cls; }
		},

		replace: function replaceClass(/*DomNode|String*/node, /*String|Array*/addClassStr, /*String|Array?*/removeClassStr){
			node = dom.byId(node);
			fakeNode[className] = node[className];
			cls.remove(fakeNode, removeClassStr);
			cls.add(fakeNode, addClassStr);
			if(node[className] !== fakeNode[className]){
				node[className] = fakeNode[className];
			}
		},

		toggle: function toggleClass(/*DomNode|String*/node, /*String|Array*/classStr, /*Boolean?*/condition){
			node = dom.byId(node);
			if(condition === undefined){
				classStr = str2array(classStr);
				for(var i = 0, len = classStr.length, c; i < len; ++i){
					c = classStr[i];
					cls[cls.contains(node, c) ? "remove" : "add"](node, c);
				}
			}else{
				cls[condition ? "add" : "remove"](node, classStr);
			}
			return condition;   // Boolean
		}
	};

	return cls;
});

},
'dijit/focus':function(){
define([
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/dom", // domAttr.get dom.isDescendant
	"dojo/dom-attr", // domAttr.get dom.isDescendant
	"dojo/dom-construct", // connect to domConstruct.empty, domConstruct.destroy
	"dojo/Evented",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/_base/sniff", // has("ie")
	"dojo/Stateful",
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.get
	"./a11y",	// a11y.isTabNavigable
	"./registry",	// registry.byId
	"."		// to set dijit.focus
], function(aspect, declare, dom, domAttr, domConstruct, Evented, lang, on, ready, has, Stateful, unload, win, winUtils,
			a11y, registry, dijit){

	// module:
	//		dijit/focus
	// summary:
	//		Returns a singleton that tracks the currently focused node, and which widgets are currently "active".

/*=====
	dijit.focus = {
		// summary:
		//		Tracks the currently focused node, and which widgets are currently "active".
		//		Access via require(["dijit/focus"], function(focus){ ... }).
		//
		//		A widget is considered active if it or a descendant widget has focus,
		//		or if a non-focusable node of this widget or a descendant was recently clicked.
		//
		//		Call focus.watch("curNode", callback) to track the current focused DOMNode,
		//		or focus.watch("activeStack", callback) to track the currently focused stack of widgets.
		//
		//		Call focus.on("widget-blur", func) or focus.on("widget-focus", ...) to monitor when
		//		when widgets become active/inactive
		//
		//		Finally, focus(node) will focus a node, suppressing errors if the node doesn't exist.

		// curNode: DomNode
		//		Currently focused item on screen
		curNode: null,

		// activeStack: dijit._Widget[]
		//		List of currently active widgets (focused widget and it's ancestors)
		activeStack: [],

		registerIframe: function(iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the <iframe> itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle with remove() method to deregister.
		},

		registerWin: function(targetWindow, effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow: Window?
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode: DOMNode?
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle with remove() method to deregister.
		}
	};
=====*/

	var FocusManager = declare([Stateful, Evented], {
		// curNode: DomNode
		//		Currently focused item on screen
		curNode: null,

		// activeStack: dijit._Widget[]
		//		List of currently active widgets (focused widget and it's ancestors)
		activeStack: [],

		constructor: function(){
			// Don't leave curNode/prevNode pointing to bogus elements
			var check = lang.hitch(this, function(node){
				if(dom.isDescendant(this.curNode, node)){
					this.set("curNode", null);
				}
				if(dom.isDescendant(this.prevNode, node)){
					this.set("prevNode", null);
				}
			});
			aspect.before(domConstruct, "empty", check);
			aspect.before(domConstruct, "destroy", check);
		},

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the <iframe> itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle with remove() method to deregister.
			return this.registerWin(iframe.contentWindow, iframe);
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle with remove() method to deregister.

			// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

			var _this = this;
			var mousedownListener = function(evt){
				_this._justMouseDowned = true;
				setTimeout(function(){ _this._justMouseDowned = false; }, 0);

				// workaround weird IE bug where the click is on an orphaned node
				// (first time clicking a Select/DropDownButton inside a TooltipDialog)
				if(has("ie") && evt && evt.srcElement && evt.srcElement.parentNode == null){
					return;
				}

				_this._onTouchNode(effectiveNode || evt.target || evt.srcElement, "mouse");
			};

			// Listen for blur and focus events on targetWindow's document.
			// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
			// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
			// fire.
			// Connect to <html> (rather than document) on IE to avoid memory leaks, but document on other browsers because
			// (at least for FF) the focus event doesn't fire on <html> or <body>.
			var doc = has("ie") ? targetWindow.document.documentElement : targetWindow.document;
			if(doc){
				if(has("ie")){
					targetWindow.document.body.attachEvent('onmousedown', mousedownListener);
					var activateListener = function(evt){
						// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
						// ignore those events
						var tag = evt.srcElement.tagName.toLowerCase();
						if(tag == "#document" || tag == "body"){ return; }

						// Previous code called _onTouchNode() for any activate event on a non-focusable node.   Can
						// probably just ignore such an event as it will be handled by onmousedown handler above, but
						// leaving the code for now.
						if(a11y.isTabNavigable(evt.srcElement)){
							_this._onFocusNode(effectiveNode || evt.srcElement);
						}else{
							_this._onTouchNode(effectiveNode || evt.srcElement);
						}
					};
					doc.attachEvent('onactivate', activateListener);
					var deactivateListener =  function(evt){
						_this._onBlurNode(effectiveNode || evt.srcElement);
					};
					doc.attachEvent('ondeactivate', deactivateListener);

					return {
						remove: function(){
							targetWindow.document.detachEvent('onmousedown', mousedownListener);
							doc.detachEvent('onactivate', activateListener);
							doc.detachEvent('ondeactivate', deactivateListener);
							doc = null;	// prevent memory leak (apparent circular reference via closure)
						}
					};
				}else{
					doc.body.addEventListener('mousedown', mousedownListener, true);
					doc.body.addEventListener('touchstart', mousedownListener, true);
					var focusListener = function(evt){
						_this._onFocusNode(effectiveNode || evt.target);
					};
					doc.addEventListener('focus', focusListener, true);
					var blurListener = function(evt){
						_this._onBlurNode(effectiveNode || evt.target);
					};
					doc.addEventListener('blur', blurListener, true);

					return {
						remove: function(){
							doc.body.removeEventListener('mousedown', mousedownListener, true);
							doc.body.removeEventListener('touchstart', mousedownListener, true);
							doc.removeEventListener('focus', focusListener, true);
							doc.removeEventListener('blur', blurListener, true);
							doc = null;	// prevent memory leak (apparent circular reference via closure)
						}
					};
				}
			}
		},

		_onBlurNode: function(/*DomNode*/ /*===== node =====*/){
			// summary:
			// 		Called when focus leaves a node.
			//		Usually ignored, _unless_ it *isn't* followed by touching another node,
			//		which indicates that we tabbed off the last field on the page,
			//		in which case every widget is marked inactive
			this.set("prevNode", this.curNode);
			this.set("curNode", null);

			if(this._justMouseDowned){
				// the mouse down caused a new widget to be marked as active; this blur event
				// is coming late, so ignore it.
				return;
			}

			// if the blur event isn't followed by a focus event then mark all widgets as inactive.
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
			}
			this._clearActiveWidgetsTimer = setTimeout(lang.hitch(this, function(){
				delete this._clearActiveWidgetsTimer;
				this._setStack([]);
				this.prevNode = null;
			}), 100);
		},

		_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
			// summary:
			//		Callback when node is focused or mouse-downed
			// node:
			//		The node that was touched.
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			// ignore the recent blurNode event
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
				delete this._clearActiveWidgetsTimer;
			}

			// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
			var newStack=[];
			try{
				while(node){
					var popupParent = domAttr.get(node, "dijitPopupParent");
					if(popupParent){
						node=registry.byId(popupParent).domNode;
					}else if(node.tagName && node.tagName.toLowerCase() == "body"){
						// is this the root of the document or just the root of an iframe?
						if(node === win.body()){
							// node is the root of the main document
							break;
						}
						// otherwise, find the iframe this node refers to (can't access it via parentNode,
						// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
						node=winUtils.get(node.ownerDocument).frameElement;
					}else{
						// if this node is the root node of a widget, then add widget id to stack,
						// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
						// to support MenuItem)
						var id = node.getAttribute && node.getAttribute("widgetId"),
							widget = id && registry.byId(id);
						if(widget && !(by == "mouse" && widget.get("disabled"))){
							newStack.unshift(id);
						}
						node=node.parentNode;
					}
				}
			}catch(e){ /* squelch */ }

			this._setStack(newStack, by);
		},

		_onFocusNode: function(/*DomNode*/ node){
			// summary:
			//		Callback when node is focused

			if(!node){
				return;
			}

			if(node.nodeType == 9){
				// Ignore focus events on the document itself.  This is here so that
				// (for example) clicking the up/down arrows of a spinner
				// (which don't get focus) won't cause that widget to blur. (FF issue)
				return;
			}

			this._onTouchNode(node);

			if(node == this.curNode){ return; }
			this.set("curNode", node);
		},

		_setStack: function(/*String[]*/ newStack, /*String*/ by){
			// summary:
			//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
			// newStack:
			//		array of widget id's, starting from the top (outermost) widget
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			var oldStack = this.activeStack;
			this.set("activeStack", newStack);

			// compare old stack to new stack to see how many elements they have in common
			for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
				if(oldStack[nCommon] != newStack[nCommon]){
					break;
				}
			}

			var widget;
			// for all elements that have gone out of focus, set focused=false
			for(var i=oldStack.length-1; i>=nCommon; i--){
				widget = registry.byId(oldStack[i]);
				if(widget){
					widget._hasBeenBlurred = true;		// TODO: used by form widgets, should be moved there
					widget.set("focused", false);
					if(widget._focusManager == this){
						widget._onBlur(by);
					}
					this.emit("widget-blur", widget, by);
				}
			}

			// for all element that have come into focus, set focused=true
			for(i=nCommon; i<newStack.length; i++){
				widget = registry.byId(newStack[i]);
				if(widget){
					widget.set("focused", true);
					if(widget._focusManager == this){
						widget._onFocus(by);
					}
					this.emit("widget-focus", widget, by);
				}
			}
		},

		focus: function(node){
			// summary:
			//		Focus the specified node, suppressing errors if they occur
			if(node){
				try{ node.focus(); }catch(e){/*quiet*/}
			}
		}
	});

	var singleton = new FocusManager();

	// register top window and all the iframes it contains
	ready(function(){
		var handle = singleton.registerWin(win.doc.parentWindow || win.doc.defaultView);
		if(has("ie")){
			unload.addOnWindowUnload(function(){
				handle.remove();
				handle = null;
			})
		}
	});

	// Setup dijit.focus as a pointer to the singleton but also (for backwards compatibility)
	// as a function to set focus.
	dijit.focus = function(node){
		singleton.focus(node);	// indirection here allows dijit/_base/focus.js to override behavior
	};
	for(var attr in singleton){
		if(!/^_/.test(attr)){
			dijit.focus[attr] = typeof singleton[attr] == "function" ? lang.hitch(singleton, attr) : singleton[attr];
		}
	}
	singleton.watch(function(attr, oldVal, newVal){
		dijit.focus[attr] = newVal;
	});

	return singleton;
});

},
'dojo/dom-attr':function(){
define(["exports", "./_base/sniff", "./_base/lang", "./dom", "./dom-style", "./dom-prop"],
		function(exports, has, lang, dom, style, prop){
	// module:
	//		dojo/dom-attr
	// summary:
	//		This module defines the core dojo DOM attributes API.

	// =============================
	// Element attribute Functions
	// =============================

	// This module will be obsolete soon. Use dojo.prop instead.

	// dojo.attr() should conform to http://www.w3.org/TR/DOM-Level-2-Core/

	// attribute-related functions (to be obsolete soon)

	/*=====
	dojo.hasAttr = function(node, name){
		// summary:
		//		Returns true if the requested attribute is specified on the
		//		given element, and false otherwise.
		// node: DOMNode|String
		//		id or reference to the element to check
		// name: String
		//		the name of the attribute
		// returns: Boolean
		//		true if the requested attribute is specified on the
		//		given element, and false otherwise
	};
	=====*/

	/*=====
	dojo.getAttr = function(node, name){
		// summary:
		//		Gets an attribute on an HTML element.
		// description:
		//		Handles normalized getting of attributes on DOM Nodes.
		// node: DOMNode|String
		//		id or reference to the element to get the attribute on
		// name: String
		//		the name of the attribute to get.
		// returns:
		//		the value of the requested attribute or null if that attribute does not have a specified or
		//		default value;
		//
		// example:
		//	|	// get the current value of the "foo" attribute on a node
		//	|	dojo.getAttr(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.getAttr("nodeId", "foo");
	};
	=====*/

	/*=====
	dojo.setAttr = function(node, name, value){
		// summary:
		//		Sets an attribute on an HTML element.
		// description:
		//		Handles normalized setting of attributes on DOM Nodes.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to set the attribute on
		// name: String|Object
		//		the name of the attribute to set, or a hash of key-value pairs to set.
		// value: String?
		//		the value to set for the attribute, if the name is a string.
		// returns:
		//		the DOM node
		//
		// example:
		//	|	// use attr() to set the tab index
		//	|	dojo.setAttr("nodeId", "tabIndex", 3);
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.setAttr("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.setAttr("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.setAttr("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.setStyle("someNode", obj);
	};
	=====*/

	/*=====
	dojo.removeAttr = function(node, name){
		// summary:
		//		Removes an attribute from an HTML element.
		// node: DOMNode|String
		//		id or reference to the element to remove the attribute from
		// name: String
		//		the name of the attribute to remove
	};
	=====*/

	/*=====
	dojo.getNodeProp = function(node, name){
		// summary:
		//		Returns an effective value of a property or an attribute.
		// node: DOMNode|String
		//		id or reference to the element to remove the attribute from
		// name: String
		//		the name of the attribute
		// returns:
		//      the value of the attribute
	};
	=====*/

	var forcePropNames = {
			innerHTML:	1,
			className:	1,
			htmlFor:	has("ie"),
			value:		1
		},
		attrNames = {
			// original attribute names
			classname: "class",
			htmlfor: "for",
			// for IE
			tabindex: "tabIndex",
			readonly: "readOnly"
		};

	function _hasAttr(node, name){
		var attr = node.getAttributeNode && node.getAttributeNode(name);
		return attr && attr.specified; // Boolean
	}

	// There is a difference in the presence of certain properties and their default values
	// between browsers. For example, on IE "disabled" is present on all elements,
	// but it is value is "false"; "tabIndex" of <div> returns 0 by default on IE, yet other browsers
	// can return -1.

	exports.has = function hasAttr(/*DOMNode|String*/node, /*String*/name){
		var lc = name.toLowerCase();
		return forcePropNames[prop.names[lc] || name] || _hasAttr(dom.byId(node), attrNames[lc] || name);	// Boolean
	};

	exports.get = function getAttr(/*DOMNode|String*/node, /*String*/name){
		node = dom.byId(node);
		var lc = name.toLowerCase(),
			propName = prop.names[lc] || name,
			forceProp = forcePropNames[propName];
		// should we access this attribute via a property or via getAttribute()?
		value = node[propName];
		if(forceProp && typeof value != "undefined"){
			// node's property
			return value;	// Anything
		}
		if(propName != "href" && (typeof value == "boolean" || lang.isFunction(value))){
			// node's property
			return value;	// Anything
		}
		// node's attribute
		// we need _hasAttr() here to guard against IE returning a default value
		var attrName = attrNames[lc] || name;
		return _hasAttr(node, attrName) ? node.getAttribute(attrName) : null; // Anything
	};

	exports.set = function setAttr(/*DOMNode|String*/node, /*String|Object*/name, /*String?*/value){
		node = dom.byId(node);
		if(arguments.length == 2){ // inline'd type check
			// the object form of setter: the 2nd argument is a dictionary
			for(var x in name){
				exports.set(node, x, name[x]);
			}
			return node; // DomNode
		}
		var lc = name.toLowerCase(),
			propName = prop.names[lc] || name,
			forceProp = forcePropNames[propName];
		if(propName == "style" && typeof value != "string"){ // inline'd type check
			// special case: setting a style
			style.set(node, value);
			return node; // DomNode
		}
		if(forceProp || typeof value == "boolean" || lang.isFunction(value)){
			return prop.set(node, name, value)
		}
		// node's attribute
		node.setAttribute(attrNames[lc] || name, value);
		return node; // DomNode
	};

	exports.remove = function removeAttr(/*DOMNode|String*/ node, /*String*/ name){
		dom.byId(node).removeAttribute(attrNames[name.toLowerCase()] || name);
	};

	exports.getNodeProp = function getNodeProp(/*DomNode|String*/ node, /*String*/ name){
		node = dom.byId(node);
		var lc = name.toLowerCase(), propName = prop.names[lc] || name;
		if((propName in node) && propName != "href"){
			// node's property
			return node[propName];	// Anything
		}
		// node's attribute
		var attrName = attrNames[lc] || name;
		return _hasAttr(node, attrName) ? node.getAttribute(attrName) : null; // Anything
	};
});

},
'dojo/selector/acme':function(){
define(["../_base/kernel", "../has", "../dom", "../_base/sniff", "../_base/array", "../_base/lang", "../_base/window"], function(dojo, has, dom){
  //  module:
  //	dojo/selector/acme
  //  summary:
  //	This module defines the Acme selector engine

/*
	acme architectural overview:

		acme is a relatively full-featured CSS3 query library. It is
		designed to take any valid CSS3 selector and return the nodes matching
		the selector. To do this quickly, it processes queries in several
		steps, applying caching where profitable.

		The steps (roughly in reverse order of the way they appear in the code):
			1.) check to see if we already have a "query dispatcher"
				- if so, use that with the given parameterization. Skip to step 4.
			2.) attempt to determine which branch to dispatch the query to:
				- JS (optimized DOM iteration)
				- native (FF3.1+, Safari 3.1+, IE 8+)
			3.) tokenize and convert to executable "query dispatcher"
				- this is where the lion's share of the complexity in the
					system lies. In the DOM version, the query dispatcher is
					assembled as a chain of "yes/no" test functions pertaining to
					a section of a simple query statement (".blah:nth-child(odd)"
					but not "div div", which is 2 simple statements). Individual
					statement dispatchers are cached (to prevent re-definition)
					as are entire dispatch chains (to make re-execution of the
					same query fast)
			4.) the resulting query dispatcher is called in the passed scope
					(by default the top-level document)
				- for DOM queries, this results in a recursive, top-down
					evaluation of nodes based on each simple query section
				- for native implementations, this may mean working around spec
					bugs. So be it.
			5.) matched nodes are pruned to ensure they are unique (if necessary)
*/


	////////////////////////////////////////////////////////////////////////
	// Toolkit aliases
	////////////////////////////////////////////////////////////////////////

	// if you are extracting acme for use in your own system, you will
	// need to provide these methods and properties. No other porting should be
	// necessary, save for configuring the system to use a class other than
	// dojo.NodeList as the return instance instantiator
	var trim = 			dojo.trim;
	var each = 			dojo.forEach;
	// 					d.isIE; // float
	// 					d.isSafari; // float
	// 					d.isOpera; // float
	// 					d.isWebKit; // float
	// 					d.doc ; // document element

	var getDoc = function(){ return dojo.doc; };
	// NOTE(alex): the spec is idiotic. CSS queries should ALWAYS be case-sensitive, but nooooooo
	var cssCaseBug = (getDoc().compatMode) == "BackCompat";

	////////////////////////////////////////////////////////////////////////
	// Global utilities
	////////////////////////////////////////////////////////////////////////


	var specials = ">~+";

	// global thunk to determine whether we should treat the current query as
	// case sensitive or not. This switch is flipped by the query evaluator
	// based on the document passed as the context to search.
	var caseSensitive = false;

	// how high?
	var yesman = function(){ return true; };

	////////////////////////////////////////////////////////////////////////
	// Tokenizer
	////////////////////////////////////////////////////////////////////////

	var getQueryParts = function(query){
		//	summary:
		//		state machine for query tokenization
		//	description:
		//		instead of using a brittle and slow regex-based CSS parser,
		//		acme implements an AST-style query representation. This
		//		representation is only generated once per query. For example,
		//		the same query run multiple times or under different root nodes
		//		does not re-parse the selector expression but instead uses the
		//		cached data structure. The state machine implemented here
		//		terminates on the last " " (space) character and returns an
		//		ordered array of query component structures (or "parts"). Each
		//		part represents an operator or a simple CSS filtering
		//		expression. The structure for parts is documented in the code
		//		below.


		// NOTE:
		//		this code is designed to run fast and compress well. Sacrifices
		//		to readability and maintainability have been made.  Your best
		//		bet when hacking the tokenizer is to put The Donnas on *really*
		//		loud (may we recommend their "Spend The Night" release?) and
		//		just assume you're gonna make mistakes. Keep the unit tests
		//		open and run them frequently. Knowing is half the battle ;-)
		if(specials.indexOf(query.slice(-1)) >= 0){
			// if we end with a ">", "+", or "~", that means we're implicitly
			// searching all children, so make it explicit
			query += " * "
		}else{
			// if you have not provided a terminator, one will be provided for
			// you...
			query += " ";
		}

		var ts = function(/*Integer*/ s, /*Integer*/ e){
			// trim and slice.

			// take an index to start a string slice from and an end position
			// and return a trimmed copy of that sub-string
			return trim(query.slice(s, e));
		};

		// the overall data graph of the full query, as represented by queryPart objects
		var queryParts = [];


		// state keeping vars
		var inBrackets = -1, inParens = -1, inMatchFor = -1,
			inPseudo = -1, inClass = -1, inId = -1, inTag = -1, currentQuoteChar,
			lc = "", cc = "", pStart;

		// iteration vars
		var x = 0, // index in the query
			ql = query.length,
			currentPart = null, // data structure representing the entire clause
			_cp = null; // the current pseudo or attr matcher

		// several temporary variables are assigned to this structure during a
		// potential sub-expression match:
		//		attr:
		//			a string representing the current full attribute match in a
		//			bracket expression
		//		type:
		//			if there's an operator in a bracket expression, this is
		//			used to keep track of it
		//		value:
		//			the internals of parenthetical expression for a pseudo. for
		//			:nth-child(2n+1), value might be "2n+1"

		var endTag = function(){
			// called when the tokenizer hits the end of a particular tag name.
			// Re-sets state variables for tag matching and sets up the matcher
			// to handle the next type of token (tag or operator).
			if(inTag >= 0){
				var tv = (inTag == x) ? null : ts(inTag, x); // .toLowerCase();
				currentPart[ (specials.indexOf(tv) < 0) ? "tag" : "oper" ] = tv;
				inTag = -1;
			}
		};

		var endId = function(){
			// called when the tokenizer might be at the end of an ID portion of a match
			if(inId >= 0){
				currentPart.id = ts(inId, x).replace(/\\/g, "");
				inId = -1;
			}
		};

		var endClass = function(){
			// called when the tokenizer might be at the end of a class name
			// match. CSS allows for multiple classes, so we augment the
			// current item with another class in its list
			if(inClass >= 0){
				currentPart.classes.push(ts(inClass + 1, x).replace(/\\/g, ""));
				inClass = -1;
			}
		};

		var endAll = function(){
			// at the end of a simple fragment, so wall off the matches
			endId();
			endTag();
			endClass();
		};

		var endPart = function(){
			endAll();
			if(inPseudo >= 0){
				currentPart.pseudos.push({ name: ts(inPseudo + 1, x) });
			}
			// hint to the selector engine to tell it whether or not it
			// needs to do any iteration. Many simple selectors don't, and
			// we can avoid significant construction-time work by advising
			// the system to skip them
			currentPart.loops = (
					currentPart.pseudos.length ||
					currentPart.attrs.length ||
					currentPart.classes.length	);

			currentPart.oquery = currentPart.query = ts(pStart, x); // save the full expression as a string


			// otag/tag are hints to suggest to the system whether or not
			// it's an operator or a tag. We save a copy of otag since the
			// tag name is cast to upper-case in regular HTML matches. The
			// system has a global switch to figure out if the current
			// expression needs to be case sensitive or not and it will use
			// otag or tag accordingly
			currentPart.otag = currentPart.tag = (currentPart["oper"]) ? null : (currentPart.tag || "*");

			if(currentPart.tag){
				// if we're in a case-insensitive HTML doc, we likely want
				// the toUpperCase when matching on element.tagName. If we
				// do it here, we can skip the string op per node
				// comparison
				currentPart.tag = currentPart.tag.toUpperCase();
			}

			// add the part to the list
			if(queryParts.length && (queryParts[queryParts.length-1].oper)){
				// operators are always infix, so we remove them from the
				// list and attach them to the next match. The evaluator is
				// responsible for sorting out how to handle them.
				currentPart.infixOper = queryParts.pop();
				currentPart.query = currentPart.infixOper.query + " " + currentPart.query;
				/*
				console.debug(	"swapping out the infix",
								currentPart.infixOper,
								"and attaching it to",
								currentPart);
				*/
			}
			queryParts.push(currentPart);

			currentPart = null;
		};

		// iterate over the query, character by character, building up a
		// list of query part objects
		for(; lc=cc, cc=query.charAt(x), x < ql; x++){
			//		cc: the current character in the match
			//		lc: the last character (if any)

			// someone is trying to escape something, so don't try to match any
			// fragments. We assume we're inside a literal.
			if(lc == "\\"){ continue; }
			if(!currentPart){ // a part was just ended or none has yet been created
				// NOTE: I hate all this alloc, but it's shorter than writing tons of if's
				pStart = x;
				//	rules describe full CSS sub-expressions, like:
				//		#someId
				//		.className:first-child
				//	but not:
				//		thinger > div.howdy[type=thinger]
				//	the indidual components of the previous query would be
				//	split into 3 parts that would be represented a structure
				//	like:
				//		[
				//			{
				//				query: "thinger",
				//				tag: "thinger",
				//			},
				//			{
				//				query: "div.howdy[type=thinger]",
				//				classes: ["howdy"],
				//				infixOper: {
				//					query: ">",
				//					oper: ">",
				//				}
				//			},
				//		]
				currentPart = {
					query: null, // the full text of the part's rule
					pseudos: [], // CSS supports multiple pseud-class matches in a single rule
					attrs: [],	// CSS supports multi-attribute match, so we need an array
					classes: [], // class matches may be additive, e.g.: .thinger.blah.howdy
					tag: null,	// only one tag...
					oper: null, // ...or operator per component. Note that these wind up being exclusive.
					id: null,	// the id component of a rule
					getTag: function(){
						return caseSensitive ? this.otag : this.tag;
					}
				};

				// if we don't have a part, we assume we're going to start at
				// the beginning of a match, which should be a tag name. This
				// might fault a little later on, but we detect that and this
				// iteration will still be fine.
				inTag = x;
			}

			// Skip processing all quoted characters.
			// If we are inside quoted text then currentQuoteChar stores the character that began the quote,
			// thus that character that will end it.
			if(currentQuoteChar){
				if(cc == currentQuoteChar){
					currentQuoteChar = null;
				}
				continue;
			}else if (cc == "'" || cc == '"'){
				currentQuoteChar = cc;
				continue;
			}

			if(inBrackets >= 0){
				// look for a the close first
				if(cc == "]"){ // if we're in a [...] clause and we end, do assignment
					if(!_cp.attr){
						// no attribute match was previously begun, so we
						// assume this is an attribute existence match in the
						// form of [someAttributeName]
						_cp.attr = ts(inBrackets+1, x);
					}else{
						// we had an attribute already, so we know that we're
						// matching some sort of value, as in [attrName=howdy]
						_cp.matchFor = ts((inMatchFor||inBrackets+1), x);
					}
					var cmf = _cp.matchFor;
					if(cmf){
						// try to strip quotes from the matchFor value. We want
						// [attrName=howdy] to match the same
						//	as [attrName = 'howdy' ]
						if(	(cmf.charAt(0) == '"') || (cmf.charAt(0) == "'") ){
							_cp.matchFor = cmf.slice(1, -1);
						}
					}
					// remove backslash escapes from an attribute match, since DOM
					// querying will get attribute values without backslashes
					if(_cp.matchFor){
						_cp.matchFor = _cp.matchFor.replace(/\\/g, "");
					}

					// end the attribute by adding it to the list of attributes.
					currentPart.attrs.push(_cp);
					_cp = null; // necessary?
					inBrackets = inMatchFor = -1;
				}else if(cc == "="){
					// if the last char was an operator prefix, make sure we
					// record it along with the "=" operator.
					var addToCc = ("|~^$*".indexOf(lc) >=0 ) ? lc : "";
					_cp.type = addToCc+cc;
					_cp.attr = ts(inBrackets+1, x-addToCc.length);
					inMatchFor = x+1;
				}
				// now look for other clause parts
			}else if(inParens >= 0){
				// if we're in a parenthetical expression, we need to figure
				// out if it's attached to a pseudo-selector rule like
				// :nth-child(1)
				if(cc == ")"){
					if(inPseudo >= 0){
						_cp.value = ts(inParens+1, x);
					}
					inPseudo = inParens = -1;
				}
			}else if(cc == "#"){
				// start of an ID match
				endAll();
				inId = x+1;
			}else if(cc == "."){
				// start of a class match
				endAll();
				inClass = x;
			}else if(cc == ":"){
				// start of a pseudo-selector match
				endAll();
				inPseudo = x;
			}else if(cc == "["){
				// start of an attribute match.
				endAll();
				inBrackets = x;
				// provide a new structure for the attribute match to fill-in
				_cp = {
					/*=====
					attr: null, type: null, matchFor: null
					=====*/
				};
			}else if(cc == "("){
				// we really only care if we've entered a parenthetical
				// expression if we're already inside a pseudo-selector match
				if(inPseudo >= 0){
					// provide a new structure for the pseudo match to fill-in
					_cp = {
						name: ts(inPseudo+1, x),
						value: null
					};
					currentPart.pseudos.push(_cp);
				}
				inParens = x;
			}else if(
				(cc == " ") &&
				// if it's a space char and the last char is too, consume the
				// current one without doing more work
				(lc != cc)
			){
				endPart();
			}
		}
		return queryParts;
	};


	////////////////////////////////////////////////////////////////////////
	// DOM query infrastructure
	////////////////////////////////////////////////////////////////////////

	var agree = function(first, second){
		// the basic building block of the yes/no chaining system. agree(f1,
		// f2) generates a new function which returns the boolean results of
		// both of the passed functions to a single logical-anded result. If
		// either are not passed, the other is used exclusively.
		if(!first){ return second; }
		if(!second){ return first; }

		return function(){
			return first.apply(window, arguments) && second.apply(window, arguments);
		}
	};

	var getArr = function(i, arr){
		// helps us avoid array alloc when we don't need it
		var r = arr||[]; // FIXME: should this be 'new d._NodeListCtor()' ?
		if(i){ r.push(i); }
		return r;
	};

	var _isElement = function(n){ return (1 == n.nodeType); };

	// FIXME: need to coalesce _getAttr with defaultGetter
	var blank = "";
	var _getAttr = function(elem, attr){
		if(!elem){ return blank; }
		if(attr == "class"){
			return elem.className || blank;
		}
		if(attr == "for"){
			return elem.htmlFor || blank;
		}
		if(attr == "style"){
			return elem.style.cssText || blank;
		}
		return (caseSensitive ? elem.getAttribute(attr) : elem.getAttribute(attr, 2)) || blank;
	};

	var attrs = {
		"*=": function(attr, value){
			return function(elem){
				// E[foo*="bar"]
				//		an E element whose "foo" attribute value contains
				//		the substring "bar"
				return (_getAttr(elem, attr).indexOf(value)>=0);
			}
		},
		"^=": function(attr, value){
			// E[foo^="bar"]
			//		an E element whose "foo" attribute value begins exactly
			//		with the string "bar"
			return function(elem){
				return (_getAttr(elem, attr).indexOf(value)==0);
			}
		},
		"$=": function(attr, value){
			// E[foo$="bar"]
			//		an E element whose "foo" attribute value ends exactly
			//		with the string "bar"
			return function(elem){
				var ea = " "+_getAttr(elem, attr);
				var lastIndex = ea.lastIndexOf(value);
				return lastIndex > -1 && (lastIndex==(ea.length-value.length));
			}
		},
		"~=": function(attr, value){
			// E[foo~="bar"]
			//		an E element whose "foo" attribute value is a list of
			//		space-separated values, one of which is exactly equal
			//		to "bar"

			// return "[contains(concat(' ',@"+attr+",' '), ' "+ value +" ')]";
			var tval = " "+value+" ";
			return function(elem){
				var ea = " "+_getAttr(elem, attr)+" ";
				return (ea.indexOf(tval)>=0);
			}
		},
		"|=": function(attr, value){
			// E[hreflang|="en"]
			//		an E element whose "hreflang" attribute has a
			//		hyphen-separated list of values beginning (from the
			//		left) with "en"
			var valueDash = value+"-";
			return function(elem){
				var ea = _getAttr(elem, attr);
				return (
					(ea == value) ||
					(ea.indexOf(valueDash)==0)
				);
			}
		},
		"=": function(attr, value){
			return function(elem){
				return (_getAttr(elem, attr) == value);
			}
		}
	};

	// avoid testing for node type if we can. Defining this in the negative
	// here to avoid negation in the fast path.
	var _noNES = (typeof getDoc().firstChild.nextElementSibling == "undefined");
	var _ns = !_noNES ? "nextElementSibling" : "nextSibling";
	var _ps = !_noNES ? "previousElementSibling" : "previousSibling";
	var _simpleNodeTest = (_noNES ? _isElement : yesman);

	var _lookLeft = function(node){
		// look left
		while(node = node[_ps]){
			if(_simpleNodeTest(node)){ return false; }
		}
		return true;
	};

	var _lookRight = function(node){
		// look right
		while(node = node[_ns]){
			if(_simpleNodeTest(node)){ return false; }
		}
		return true;
	};

	var getNodeIndex = function(node){
		var root = node.parentNode;
		var i = 0,
			tret = root.children || root.childNodes,
			ci = (node["_i"]||-1),
			cl = (root["_l"]||-1);

		if(!tret){ return -1; }
		var l = tret.length;

		// we calculate the parent length as a cheap way to invalidate the
		// cache. It's not 100% accurate, but it's much more honest than what
		// other libraries do
		if( cl == l && ci >= 0 && cl >= 0 ){
			// if it's legit, tag and release
			return ci;
		}

		// else re-key things
		root["_l"] = l;
		ci = -1;
		for(var te = root["firstElementChild"]||root["firstChild"]; te; te = te[_ns]){
			if(_simpleNodeTest(te)){
				te["_i"] = ++i;
				if(node === te){
					// NOTE:
					//	shortcutting the return at this step in indexing works
					//	very well for benchmarking but we avoid it here since
					//	it leads to potential O(n^2) behavior in sequential
					//	getNodexIndex operations on a previously un-indexed
					//	parent. We may revisit this at a later time, but for
					//	now we just want to get the right answer more often
					//	than not.
					ci = i;
				}
			}
		}
		return ci;
	};

	var isEven = function(elem){
		return !((getNodeIndex(elem)) % 2);
	};

	var isOdd = function(elem){
		return ((getNodeIndex(elem)) % 2);
	};

	var pseudos = {
		"checked": function(name, condition){
			return function(elem){
				return !!("checked" in elem ? elem.checked : elem.selected);
			}
		},
		"first-child": function(){ return _lookLeft; },
		"last-child": function(){ return _lookRight; },
		"only-child": function(name, condition){
			return function(node){
				return _lookLeft(node) && _lookRight(node);
			};
		},
		"empty": function(name, condition){
			return function(elem){
				// DomQuery and jQuery get this wrong, oddly enough.
				// The CSS 3 selectors spec is pretty explicit about it, too.
				var cn = elem.childNodes;
				var cnl = elem.childNodes.length;
				// if(!cnl){ return true; }
				for(var x=cnl-1; x >= 0; x--){
					var nt = cn[x].nodeType;
					if((nt === 1)||(nt == 3)){ return false; }
				}
				return true;
			};
		},
		"disabled": function(name, condition){
			return function(elem){
				return elem.disabled;
			};
		},
		"enabled": function(name, condition){
			return function(elem){
				return !elem.disabled;
			};
		},
		"contains": function(name, condition){
			var cz = condition.charAt(0);
			if( cz == '"' || cz == "'" ){ //remove quote
				condition = condition.slice(1, -1);
			}
			return function(elem){
				return (elem.innerHTML.indexOf(condition) >= 0);
			}
		},
		"not": function(name, condition){
			var p = getQueryParts(condition)[0];
			var ignores = { el: 1 };
			if(p.tag != "*"){
				ignores.tag = 1;
			}
			if(!p.classes.length){
				ignores.classes = 1;
			}
			var ntf = getSimpleFilterFunc(p, ignores);
			return function(elem){
				return (!ntf(elem));
			}
		},
		"nth-child": function(name, condition){
			var pi = parseInt;
			// avoid re-defining function objects if we can
			if(condition == "odd"){
				return isOdd;
			}else if(condition == "even"){
				return isEven;
			}
			// FIXME: can we shorten this?
			if(condition.indexOf("n") != -1){
				var tparts = condition.split("n", 2);
				var pred = tparts[0] ? ((tparts[0] == '-') ? -1 : pi(tparts[0])) : 1;
				var idx = tparts[1] ? pi(tparts[1]) : 0;
				var lb = 0, ub = -1;
				if(pred > 0){
					if(idx < 0){
						idx = (idx % pred) && (pred + (idx % pred));
					}else if(idx>0){
						if(idx >= pred){
							lb = idx - idx % pred;
						}
						idx = idx % pred;
					}
				}else if(pred<0){
					pred *= -1;
					// idx has to be greater than 0 when pred is negative;
					// shall we throw an error here?
					if(idx > 0){
						ub = idx;
						idx = idx % pred;
					}
				}
				if(pred > 0){
					return function(elem){
						var i = getNodeIndex(elem);
						return (i>=lb) && (ub<0 || i<=ub) && ((i % pred) == idx);
					}
				}else{
					condition = idx;
				}
			}
			var ncount = pi(condition);
			return function(elem){
				return (getNodeIndex(elem) == ncount);
			}
		}
	};

	var defaultGetter = (dojo.isIE < 9 || dojo.isIE == 9 && dojo.isQuirks) ? function(cond){
		var clc = cond.toLowerCase();
		if(clc == "class"){ cond = "className"; }
		return function(elem){
			return (caseSensitive ? elem.getAttribute(cond) : elem[cond]||elem[clc]);
		}
	} : function(cond){
		return function(elem){
			return (elem && elem.getAttribute && elem.hasAttribute(cond));
		}
	};

	var getSimpleFilterFunc = function(query, ignores){
		// generates a node tester function based on the passed query part. The
		// query part is one of the structures generated by the query parser
		// when it creates the query AST. The "ignores" object specifies which
		// (if any) tests to skip, allowing the system to avoid duplicating
		// work where it may have already been taken into account by other
		// factors such as how the nodes to test were fetched in the first
		// place
		if(!query){ return yesman; }
		ignores = ignores||{};

		var ff = null;

		if(!("el" in ignores)){
			ff = agree(ff, _isElement);
		}

		if(!("tag" in ignores)){
			if(query.tag != "*"){
				ff = agree(ff, function(elem){
					return (elem && (elem.tagName == query.getTag()));
				});
			}
		}

		if(!("classes" in ignores)){
			each(query.classes, function(cname, idx, arr){
				// get the class name
				/*
				var isWildcard = cname.charAt(cname.length-1) == "*";
				if(isWildcard){
					cname = cname.substr(0, cname.length-1);
				}
				// I dislike the regex thing, even if memoized in a cache, but it's VERY short
				var re = new RegExp("(?:^|\\s)" + cname + (isWildcard ? ".*" : "") + "(?:\\s|$)");
				*/
				var re = new RegExp("(?:^|\\s)" + cname + "(?:\\s|$)");
				ff = agree(ff, function(elem){
					return re.test(elem.className);
				});
				ff.count = idx;
			});
		}

		if(!("pseudos" in ignores)){
			each(query.pseudos, function(pseudo){
				var pn = pseudo.name;
				if(pseudos[pn]){
					ff = agree(ff, pseudos[pn](pn, pseudo.value));
				}
			});
		}

		if(!("attrs" in ignores)){
			each(query.attrs, function(attr){
				var matcher;
				var a = attr.attr;
				// type, attr, matchFor
				if(attr.type && attrs[attr.type]){
					matcher = attrs[attr.type](a, attr.matchFor);
				}else if(a.length){
					matcher = defaultGetter(a);
				}
				if(matcher){
					ff = agree(ff, matcher);
				}
			});
		}

		if(!("id" in ignores)){
			if(query.id){
				ff = agree(ff, function(elem){
					return (!!elem && (elem.id == query.id));
				});
			}
		}

		if(!ff){
			if(!("default" in ignores)){
				ff = yesman;
			}
		}
		return ff;
	};

	var _nextSibling = function(filterFunc){
		return function(node, ret, bag){
			while(node = node[_ns]){
				if(_noNES && (!_isElement(node))){ continue; }
				if(
					(!bag || _isUnique(node, bag)) &&
					filterFunc(node)
				){
					ret.push(node);
				}
				break;
			}
			return ret;
		}
	};

	var _nextSiblings = function(filterFunc){
		return function(root, ret, bag){
			var te = root[_ns];
			while(te){
				if(_simpleNodeTest(te)){
					if(bag && !_isUnique(te, bag)){
						break;
					}
					if(filterFunc(te)){
						ret.push(te);
					}
				}
				te = te[_ns];
			}
			return ret;
		}
	};

	// get an array of child *elements*, skipping text and comment nodes
	var _childElements = function(filterFunc){
		filterFunc = filterFunc||yesman;
		return function(root, ret, bag){
			// get an array of child elements, skipping text and comment nodes
			var te, x = 0, tret = root.children || root.childNodes;
			while(te = tret[x++]){
				if(
					_simpleNodeTest(te) &&
					(!bag || _isUnique(te, bag)) &&
					(filterFunc(te, x))
				){
					ret.push(te);
				}
			}
			return ret;
		};
	};

	/*
	// thanks, Dean!
	var itemIsAfterRoot = d.isIE ? function(item, root){
		return (item.sourceIndex > root.sourceIndex);
	} : function(item, root){
		return (item.compareDocumentPosition(root) == 2);
	};
	*/

	// test to see if node is below root
	var _isDescendant = function(node, root){
		var pn = node.parentNode;
		while(pn){
			if(pn == root){
				break;
			}
			pn = pn.parentNode;
		}
		return !!pn;
	};

	var _getElementsFuncCache = {};

	var getElementsFunc = function(query){
		var retFunc = _getElementsFuncCache[query.query];
		// if we've got a cached dispatcher, just use that
		if(retFunc){ return retFunc; }
		// else, generate a new on

		// NOTE:
		//		this function returns a function that searches for nodes and
		//		filters them.  The search may be specialized by infix operators
		//		(">", "~", or "+") else it will default to searching all
		//		descendants (the " " selector). Once a group of children is
		//		found, a test function is applied to weed out the ones we
		//		don't want. Many common cases can be fast-pathed. We spend a
		//		lot of cycles to create a dispatcher that doesn't do more work
		//		than necessary at any point since, unlike this function, the
		//		dispatchers will be called every time. The logic of generating
		//		efficient dispatchers looks like this in pseudo code:
		//
		//		# if it's a purely descendant query (no ">", "+", or "~" modifiers)
		//		if infixOperator == " ":
		//			if only(id):
		//				return def(root):
		//					return d.byId(id, root);
		//
		//			elif id:
		//				return def(root):
		//					return filter(d.byId(id, root));
		//
		//			elif cssClass && getElementsByClassName:
		//				return def(root):
		//					return filter(root.getElementsByClassName(cssClass));
		//
		//			elif only(tag):
		//				return def(root):
		//					return root.getElementsByTagName(tagName);
		//
		//			else:
		//				# search by tag name, then filter
		//				return def(root):
		//					return filter(root.getElementsByTagName(tagName||"*"));
		//
		//		elif infixOperator == ">":
		//			# search direct children
		//			return def(root):
		//				return filter(root.children);
		//
		//		elif infixOperator == "+":
		//			# search next sibling
		//			return def(root):
		//				return filter(root.nextElementSibling);
		//
		//		elif infixOperator == "~":
		//			# search rightward siblings
		//			return def(root):
		//				return filter(nextSiblings(root));

		var io = query.infixOper;
		var oper = (io ? io.oper : "");
		// the default filter func which tests for all conditions in the query
		// part. This is potentially inefficient, so some optimized paths may
		// re-define it to test fewer things.
		var filterFunc = getSimpleFilterFunc(query, { el: 1 });
		var qt = query.tag;
		var wildcardTag = ("*" == qt);
		var ecs = getDoc()["getElementsByClassName"];

		if(!oper){
			// if there's no infix operator, then it's a descendant query. ID
			// and "elements by class name" variants can be accelerated so we
			// call them out explicitly:
			if(query.id){
				// testing shows that the overhead of yesman() is acceptable
				// and can save us some bytes vs. re-defining the function
				// everywhere.
				filterFunc = (!query.loops && wildcardTag) ?
					yesman :
					getSimpleFilterFunc(query, { el: 1, id: 1 });

				retFunc = function(root, arr){
					var te = dom.byId(query.id, (root.ownerDocument||root));
					if(!te || !filterFunc(te)){ return; }
					if(9 == root.nodeType){ // if root's a doc, we just return directly
						return getArr(te, arr);
					}else{ // otherwise check ancestry
						if(_isDescendant(te, root)){
							return getArr(te, arr);
						}
					}
				}
			}else if(
				ecs &&
				// isAlien check. Workaround for Prototype.js being totally evil/dumb.
				/\{\s*\[native code\]\s*\}/.test(String(ecs)) &&
				query.classes.length &&
				!cssCaseBug
			){
				// it's a class-based query and we've got a fast way to run it.

				// ignore class and ID filters since we will have handled both
				filterFunc = getSimpleFilterFunc(query, { el: 1, classes: 1, id: 1 });
				var classesString = query.classes.join(" ");
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					var tret = root.getElementsByClassName(classesString);
					while((te = tret[x++])){
						if(filterFunc(te, root) && _isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};

			}else if(!wildcardTag && !query.loops){
				// it's tag only. Fast-path it.
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					var tret = root.getElementsByTagName(query.getTag());
					while((te = tret[x++])){
						if(_isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};
			}else{
				// the common case:
				//		a descendant selector without a fast path. By now it's got
				//		to have a tag selector, even if it's just "*" so we query
				//		by that and filter
				filterFunc = getSimpleFilterFunc(query, { el: 1, tag: 1, id: 1 });
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					// we use getTag() to avoid case sensitivity issues
					var tret = root.getElementsByTagName(query.getTag());
					while((te = tret[x++])){
						if(filterFunc(te, root) && _isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};
			}
		}else{
			// the query is scoped in some way. Instead of querying by tag we
			// use some other collection to find candidate nodes
			var skipFilters = { el: 1 };
			if(wildcardTag){
				skipFilters.tag = 1;
			}
			filterFunc = getSimpleFilterFunc(query, skipFilters);
			if("+" == oper){
				retFunc = _nextSibling(filterFunc);
			}else if("~" == oper){
				retFunc = _nextSiblings(filterFunc);
			}else if(">" == oper){
				retFunc = _childElements(filterFunc);
			}
		}
		// cache it and return
		return _getElementsFuncCache[query.query] = retFunc;
	};

	var filterDown = function(root, queryParts){
		// NOTE:
		//		this is the guts of the DOM query system. It takes a list of
		//		parsed query parts and a root and finds children which match
		//		the selector represented by the parts
		var candidates = getArr(root), qp, x, te, qpl = queryParts.length, bag, ret;

		for(var i = 0; i < qpl; i++){
			ret = [];
			qp = queryParts[i];
			x = candidates.length - 1;
			if(x > 0){
				// if we have more than one root at this level, provide a new
				// hash to use for checking group membership but tell the
				// system not to post-filter us since we will already have been
				// gauranteed to be unique
				bag = {};
				ret.nozip = true;
			}
			var gef = getElementsFunc(qp);
			for(var j = 0; (te = candidates[j]); j++){
				// for every root, get the elements that match the descendant
				// selector, adding them to the "ret" array and filtering them
				// via membership in this level's bag. If there are more query
				// parts, then this level's return will be used as the next
				// level's candidates
				gef(te, ret, bag);
			}
			if(!ret.length){ break; }
			candidates = ret;
		}
		return ret;
	};

	////////////////////////////////////////////////////////////////////////
	// the query runner
	////////////////////////////////////////////////////////////////////////

	// these are the primary caches for full-query results. The query
	// dispatcher functions are generated then stored here for hash lookup in
	// the future
	var _queryFuncCacheDOM = {},
		_queryFuncCacheQSA = {};

	// this is the second level of spliting, from full-length queries (e.g.,
	// "div.foo .bar") into simple query expressions (e.g., ["div.foo",
	// ".bar"])
	var getStepQueryFunc = function(query){
		var qparts = getQueryParts(trim(query));

		// if it's trivial, avoid iteration and zipping costs
		if(qparts.length == 1){
			// we optimize this case here to prevent dispatch further down the
			// chain, potentially slowing things down. We could more elegantly
			// handle this in filterDown(), but it's slower for simple things
			// that need to be fast (e.g., "#someId").
			var tef = getElementsFunc(qparts[0]);
			return function(root){
				var r = tef(root, []);
				if(r){ r.nozip = true; }
				return r;
			}
		}

		// otherwise, break it up and return a runner that iterates over the parts recursively
		return function(root){
			return filterDown(root, qparts);
		}
	};

	// NOTES:
	//	* we can't trust QSA for anything but document-rooted queries, so
	//	  caching is split into DOM query evaluators and QSA query evaluators
	//	* caching query results is dirty and leak-prone (or, at a minimum,
	//	  prone to unbounded growth). Other toolkits may go this route, but
	//	  they totally destroy their own ability to manage their memory
	//	  footprint. If we implement it, it should only ever be with a fixed
	//	  total element reference # limit and an LRU-style algorithm since JS
	//	  has no weakref support. Caching compiled query evaluators is also
	//	  potentially problematic, but even on large documents the size of the
	//	  query evaluators is often < 100 function objects per evaluator (and
	//	  LRU can be applied if it's ever shown to be an issue).
	//	* since IE's QSA support is currently only for HTML documents and even
	//	  then only in IE 8's "standards mode", we have to detect our dispatch
	//	  route at query time and keep 2 separate caches. Ugg.

	// we need to determine if we think we can run a given query via
	// querySelectorAll or if we'll need to fall back on DOM queries to get
	// there. We need a lot of information about the environment and the query
	// to make the determiniation (e.g. does it support QSA, does the query in
	// question work in the native QSA impl, etc.).
	var nua = navigator.userAgent;
	// some versions of Safari provided QSA, but it was buggy and crash-prone.
	// We need te detect the right "internal" webkit version to make this work.
	var wk = "WebKit/";
	var is525 = (
		dojo.isWebKit &&
		(nua.indexOf(wk) > 0) &&
		(parseFloat(nua.split(wk)[1]) > 528)
	);

	// IE QSA queries may incorrectly include comment nodes, so we throw the
	// zipping function into "remove" comments mode instead of the normal "skip
	// it" which every other QSA-clued browser enjoys
	var noZip = dojo.isIE ? "commentStrip" : "nozip";

	var qsa = "querySelectorAll";
	var qsaAvail = (
		!!getDoc()[qsa] &&
		// see #5832
		(!dojo.isSafari || (dojo.isSafari > 3.1) || is525 )
	);

	//Don't bother with n+3 type of matches, IE complains if we modify those.
	var infixSpaceRe = /\\[>~+]|n\+\d|([^ \\])?([>~+])([^ =])?/g;
	var infixSpaceFunc = function(match, pre, ch, post){
		return ch ? (pre ? pre + " " : "") + ch + (post ? " " + post : "") : /*n+3*/ match;
	};
	
	//Don't apply the infixSpaceRe to attribute value selectors
	var attRe = /([^[]*)([^\]]*])?/g;
	var attFunc = function(match, nonAtt, att) {
		return nonAtt.replace(infixSpaceRe, infixSpaceFunc) + (att||"");
	};
	var getQueryFunc = function(query, forceDOM){
		//Normalize query. The CSS3 selectors spec allows for omitting spaces around
		//infix operators, >, ~ and +
		//Do the work here since detection for spaces is used as a simple "not use QSA"
		//test below.
		query = query.replace(attRe, attFunc);

		if(qsaAvail){
			// if we've got a cached variant and we think we can do it, run it!
			var qsaCached = _queryFuncCacheQSA[query];
			if(qsaCached && !forceDOM){ return qsaCached; }
		}

		// else if we've got a DOM cached variant, assume that we already know
		// all we need to and use it
		var domCached = _queryFuncCacheDOM[query];
		if(domCached){ return domCached; }

		// TODO:
		//		today we're caching DOM and QSA branches separately so we
		//		recalc useQSA every time. If we had a way to tag root+query
		//		efficiently, we'd be in good shape to do a global cache.

		var qcz = query.charAt(0);
		var nospace = (-1 == query.indexOf(" "));

		// byId searches are wicked fast compared to QSA, even when filtering
		// is required
		if( (query.indexOf("#") >= 0) && (nospace) ){
			forceDOM = true;
		}

		var useQSA = (
			qsaAvail && (!forceDOM) &&
			// as per CSS 3, we can't currently start w/ combinator:
			//		http://www.w3.org/TR/css3-selectors/#w3cselgrammar
			(specials.indexOf(qcz) == -1) &&
			// IE's QSA impl sucks on pseudos
			(!dojo.isIE || (query.indexOf(":") == -1)) &&

			(!(cssCaseBug && (query.indexOf(".") >= 0))) &&

			// FIXME:
			//		need to tighten up browser rules on ":contains" and "|=" to
			//		figure out which aren't good
			//		Latest webkit (around 531.21.8) does not seem to do well with :checked on option
			//		elements, even though according to spec, selected options should
			//		match :checked. So go nonQSA for it:
			//		http://bugs.dojotoolkit.org/ticket/5179
			(query.indexOf(":contains") == -1) && (query.indexOf(":checked") == -1) &&
			(query.indexOf("|=") == -1) // some browsers don't grok it
		);

		// TODO:
		//		if we've got a descendant query (e.g., "> .thinger" instead of
		//		just ".thinger") in a QSA-able doc, but are passed a child as a
		//		root, it should be possible to give the item a synthetic ID and
		//		trivially rewrite the query to the form "#synid > .thinger" to
		//		use the QSA branch


		if(useQSA){
			var tq = (specials.indexOf(query.charAt(query.length-1)) >= 0) ?
						(query + " *") : query;
			return _queryFuncCacheQSA[query] = function(root){
				try{
					// the QSA system contains an egregious spec bug which
					// limits us, effectively, to only running QSA queries over
					// entire documents.  See:
					//		http://ejohn.org/blog/thoughts-on-queryselectorall/
					//	despite this, we can also handle QSA runs on simple
					//	selectors, but we don't want detection to be expensive
					//	so we're just checking for the presence of a space char
					//	right now. Not elegant, but it's cheaper than running
					//	the query parser when we might not need to
					if(!((9 == root.nodeType) || nospace)){ throw ""; }
					var r = root[qsa](tq);
					// skip expensive duplication checks and just wrap in a NodeList
					r[noZip] = true;
					return r;
				}catch(e){
					// else run the DOM branch on this query, ensuring that we
					// default that way in the future
					return getQueryFunc(query, true)(root);
				}
			}
		}else{
			// DOM branch
			var parts = query.match(/([^\s,](?:"(?:\\.|[^"])+"|'(?:\\.|[^'])+'|[^,])*)/g);
			return _queryFuncCacheDOM[query] = ((parts.length < 2) ?
				// if not a compound query (e.g., ".foo, .bar"), cache and return a dispatcher
				getStepQueryFunc(query) :
				// if it *is* a complex query, break it up into its
				// constituent parts and return a dispatcher that will
				// merge the parts when run
				function(root){
					var pindex = 0, // avoid array alloc for every invocation
						ret = [],
						tp;
					while((tp = parts[pindex++])){
						ret = ret.concat(getStepQueryFunc(tp)(root));
					}
					return ret;
				}
			);
		}
	};

	var _zipIdx = 0;

	// NOTE:
	//		this function is Moo inspired, but our own impl to deal correctly
	//		with XML in IE
	var _nodeUID = dojo.isIE ? function(node){
		if(caseSensitive){
			// XML docs don't have uniqueID on their nodes
			return (node.getAttribute("_uid") || node.setAttribute("_uid", ++_zipIdx) || _zipIdx);

		}else{
			return node.uniqueID;
		}
	} :
	function(node){
		return (node._uid || (node._uid = ++_zipIdx));
	};

	// determine if a node in is unique in a "bag". In this case we don't want
	// to flatten a list of unique items, but rather just tell if the item in
	// question is already in the bag. Normally we'd just use hash lookup to do
	// this for us but IE's DOM is busted so we can't really count on that. On
	// the upside, it gives us a built in unique ID function.
	var _isUnique = function(node, bag){
		if(!bag){ return 1; }
		var id = _nodeUID(node);
		if(!bag[id]){ return bag[id] = 1; }
		return 0;
	};

	// attempt to efficiently determine if an item in a list is a dupe,
	// returning a list of "uniques", hopefully in doucment order
	var _zipIdxName = "_zipIdx";
	var _zip = function(arr){
		if(arr && arr.nozip){
			return arr;
		}
		var ret = [];
		if(!arr || !arr.length){ return ret; }
		if(arr[0]){
			ret.push(arr[0]);
		}
		if(arr.length < 2){ return ret; }

		_zipIdx++;

		// we have to fork here for IE and XML docs because we can't set
		// expandos on their nodes (apparently). *sigh*
		if(dojo.isIE && caseSensitive){
			var szidx = _zipIdx+"";
			arr[0].setAttribute(_zipIdxName, szidx);
			for(var x = 1, te; te = arr[x]; x++){
				if(arr[x].getAttribute(_zipIdxName) != szidx){
					ret.push(te);
				}
				te.setAttribute(_zipIdxName, szidx);
			}
		}else if(dojo.isIE && arr.commentStrip){
			try{
				for(var x = 1, te; te = arr[x]; x++){
					if(_isElement(te)){
						ret.push(te);
					}
				}
			}catch(e){ /* squelch */ }
		}else{
			if(arr[0]){ arr[0][_zipIdxName] = _zipIdx; }
			for(var x = 1, te; te = arr[x]; x++){
				if(arr[x][_zipIdxName] != _zipIdx){
					ret.push(te);
				}
				te[_zipIdxName] = _zipIdx;
			}
		}
		return ret;
	};

	// the main executor
	var query = function(/*String*/ query, /*String|DOMNode?*/ root){
		//	summary:
		//		Returns nodes which match the given CSS3 selector, searching the
		//		entire document by default but optionally taking a node to scope
		//		the search by. Returns an array.
		//	description:
		//		dojo.query() is the swiss army knife of DOM node manipulation in
		//		Dojo. Much like Prototype's "$$" (bling-bling) function or JQuery's
		//		"$" function, dojo.query provides robust, high-performance
		//		CSS-based node selector support with the option of scoping searches
		//		to a particular sub-tree of a document.
		//
		//		Supported Selectors:
		//		--------------------
		//
		//		acme supports a rich set of CSS3 selectors, including:
		//
		//			* class selectors (e.g., `.foo`)
		//			* node type selectors like `span`
		//			* ` ` descendant selectors
		//			* `>` child element selectors
		//			* `#foo` style ID selectors
		//			* `*` universal selector
		//			* `~`, the preceded-by sibling selector
		//			* `+`, the immediately preceded-by sibling selector
		//			* attribute queries:
		//			|	* `[foo]` attribute presence selector
		//			|	* `[foo='bar']` attribute value exact match
		//			|	* `[foo~='bar']` attribute value list item match
		//			|	* `[foo^='bar']` attribute start match
		//			|	* `[foo$='bar']` attribute end match
		//			|	* `[foo*='bar']` attribute substring match
		//			* `:first-child`, `:last-child`, and `:only-child` positional selectors
		//			* `:empty` content emtpy selector
		//			* `:checked` pseudo selector
		//			* `:nth-child(n)`, `:nth-child(2n+1)` style positional calculations
		//			* `:nth-child(even)`, `:nth-child(odd)` positional selectors
		//			* `:not(...)` negation pseudo selectors
		//
		//		Any legal combination of these selectors will work with
		//		`dojo.query()`, including compound selectors ("," delimited).
		//		Very complex and useful searches can be constructed with this
		//		palette of selectors and when combined with functions for
		//		manipulation presented by dojo.NodeList, many types of DOM
		//		manipulation operations become very straightforward.
		//
		//		Unsupported Selectors:
		//		----------------------
		//
		//		While dojo.query handles many CSS3 selectors, some fall outside of
		//		what's reasonable for a programmatic node querying engine to
		//		handle. Currently unsupported selectors include:
		//
		//			* namespace-differentiated selectors of any form
		//			* all `::` pseduo-element selectors
		//			* certain pseduo-selectors which don't get a lot of day-to-day use:
		//			|	* `:root`, `:lang()`, `:target`, `:focus`
		//			* all visual and state selectors:
		//			|	* `:root`, `:active`, `:hover`, `:visisted`, `:link`
		//			* `:*-of-type` pseudo selectors
		//
		//		dojo.query and XML Documents:
		//		-----------------------------
		//
		//		`dojo.query` (as of dojo 1.2) supports searching XML documents
		//		in a case-sensitive manner. If an HTML document is served with
		//		a doctype that forces case-sensitivity (e.g., XHTML 1.1
		//		Strict), dojo.query() will detect this and "do the right
		//		thing". Case sensitivity is dependent upon the document being
		//		searched and not the query used. It is therefore possible to
		//		use case-sensitive queries on strict sub-documents (iframes,
		//		etc.) or XML documents while still assuming case-insensitivity
		//		for a host/root document.
		//
		//		Non-selector Queries:
		//		---------------------
		//
		//		If something other than a String is passed for the query,
		//		`dojo.query` will return a new `dojo.NodeList` instance
		//		constructed from that parameter alone and all further
		//		processing will stop. This means that if you have a reference
		//		to a node or NodeList, you can quickly construct a new NodeList
		//		from the original by calling `dojo.query(node)` or
		//		`dojo.query(list)`.
		//
		//	query:
		//		The CSS3 expression to match against. For details on the syntax of
		//		CSS3 selectors, see <http://www.w3.org/TR/css3-selectors/#selectors>
		//	root:
		//		A DOMNode (or node id) to scope the search from. Optional.
		//	returns: Array
		//	example:
		//		search the entire document for elements with the class "foo":
		//	|	dojo.query(".foo");
		//		these elements will match:
		//	|	<span class="foo"></span>
		//	|	<span class="foo bar"></span>
		//	|	<p class="thud foo"></p>
		//	example:
		//		search the entire document for elements with the classes "foo" *and* "bar":
		//	|	dojo.query(".foo.bar");
		//		these elements will match:
		//	|	<span class="foo bar"></span>
		//		while these will not:
		//	|	<span class="foo"></span>
		//	|	<p class="thud foo"></p>
		//	example:
		//		find `<span>` elements which are descendants of paragraphs and
		//		which have a "highlighted" class:
		//	|	dojo.query("p span.highlighted");
		//		the innermost span in this fragment matches:
		//	|	<p class="foo">
		//	|		<span>...
		//	|			<span class="highlighted foo bar">...</span>
		//	|		</span>
		//	|	</p>
		//	example:
		//		set an "odd" class on all odd table rows inside of the table
		//		`#tabular_data`, using the `>` (direct child) selector to avoid
		//		affecting any nested tables:
		//	|	dojo.query("#tabular_data > tbody > tr:nth-child(odd)").addClass("odd");
		//	example:
		//		remove all elements with the class "error" from the document
		//		and store them in a list:
		//	|	var errors = dojo.query(".error").orphan();
		//	example:
		//		add an onclick handler to every submit button in the document
		//		which causes the form to be sent via Ajax instead:
		//	|	dojo.query("input[type='submit']").onclick(function(e){
		//	|		dojo.stopEvent(e); // prevent sending the form
		//	|		var btn = e.target;
		//	|		dojo.xhrPost({
		//	|			form: btn.form,
		//	|			load: function(data){
		//	|				// replace the form with the response
		//	|				var div = dojo.doc.createElement("div");
		//	|				dojo.place(div, btn.form, "after");
		//	|				div.innerHTML = data;
		//	|				dojo.style(btn.form, "display", "none");
		//	|			}
		//	|		});
		//	|	});

		root = root || getDoc();

		// throw the big case sensitivity switch

		var od = root.ownerDocument || root;	// root is either Document or a node inside the document
		caseSensitive = (od.createElement("div").tagName === "div");

		// NOTE:
		//		adding "true" as the 2nd argument to getQueryFunc is useful for
		//		testing the DOM branch without worrying about the
		//		behavior/performance of the QSA branch.
		var r = getQueryFunc(query)(root);

		// FIXME:
		//		need to investigate this branch WRT #8074 and #8075
		if(r && r.nozip){
			return r;
		}
		return _zip(r); // dojo.NodeList
	};
	query.filter = function(/*Node[]*/ nodeList, /*String*/ filter, /*String|DOMNode?*/ root){
		// summary:
		// 		function for filtering a NodeList based on a selector, optimized for simple selectors
		var tmpNodeList = [],
			parts = getQueryParts(filter),
			filterFunc =
				(parts.length == 1 && !/[^\w#\.]/.test(filter)) ?
				getSimpleFilterFunc(parts[0]) :
				function(node){
					return dojo.query(filter, root).indexOf(node) != -1;
				};
		for(var x = 0, te; te = nodeList[x]; x++){
			if(filterFunc(te)){ tmpNodeList.push(te); }
		}
		return tmpNodeList;
	};
	return query;
});//end defineQuery

},
'com.ibm.bpm.coach/collaboration':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/**********************************COLLABORATION UTILS START*********************************************/
define(['require', 'dojo/_base/kernel', 'dojo/io/script', "dojo/_base/lang", "dojo/_base/xhr", "dojo/has", "dojo/_base/event", "./engine", 'dojo/_base/json', './collaborationBanner',"./logger", "dojo/_base/config"], 
		function(require, dojo, ioScript, lang, xhr, has, event, engine, json, baseCollaborationBanner, logger, dojoConfig) {
	
var collab = new Object();


_collaborationChannelNamePrefix = "/bpm/ProcessPortal/notifications/collaboration/";
collab.collaboratonChannelNames = {fullSync:"/DATA_FULL_SYNC/",
					dataChange:"/DATA_CHANGE",
					userJoin:"/USER_JOIN_SESSION",
					userLeave:"/USER_LEAVE_SESSION",
					rosterChange:"/EDITOR_CHANGE"};
					
collab.taskSessionID = null;
collab.taskOwnerID = null;
collab.sessionID = null;
collab.sessionRoster = null;
collab.currentUserForCollaboration =null;
collab.connectedToBayeuxServer = false;
collab.connectionEstablished = false;
collab.cometdForCollab = null;
collab.skipUserLeavePublish = false; 
collab.unsubAllChannelsOnExit = false;
_collaboration_timeout= 2000;
_collaboration_timer= null;
_RosterChangeEvent_timeout= 1000;
_RosterChangeEvent_timer= null;
_collaboration_dataChange_subscription=null;
_collaboration_subscriptions=[];
collaborationBanner = null;
	

//dojo.registerModulePath("org", "/bpm/notification/org");
//dojo.registerModulePath("dojox.cometd", "/bpm/notification/dojox/cometd");

collab.RegisterForCollaboration = function(sessionID, TaskOwnerId, fullSynchReceivedCallbackMethod, 
								  publishFullSynchMethod, IncomingChangesetCallbackMethod,
								  rosterChangeCallbackMethod,setToReadOnly){

		if(collaborationBanner) {
			collab.exitCollaboration(true);
		}
	
		if(!collab.connectionEstablished){
			logger.log("collaboration", "RegisterForCollaboration", "cometd connection already exisit start registering...", logger.FINE);
		}
		
		if(!collab.currentUserForCollaboration) {
			collab.currentUserForCollaboration = getUserInfo();
		}
		
		collab.taskSessionID = sessionID;
		collab.taskOwnerID = TaskOwnerId;
		collab.sessionID = sessionID.task_id+"/"+sessionID.coach_session_id;
		
		//update the rosterlist
		
		if(!collab.sessionRoster){//no session cached check with server
			logger.log("collaboration", "RegisterForCollaboration", "no session cached check with server", logger.FINE);
			var taskID = sessionID.task_id.split("t")[1];//variable has tNNN format
			collab.sessionRoster = getRosterListByTaskID(taskID);

			if(!collab.sessionRoster){//no previous collaboration exists on this task
				logger.log("collaboration", "RegisterForCollaboration", "Server returned 0 participants, no previous collaboration exists on this task...", logger.FINE);
				collab.sessionRoster = {
					"taskOwnerID":TaskOwnerId,
					"editorID":TaskOwnerId,
                    "numCollabUsers":0,
					"allParticipants":[{
						"userName":collab.currentUserForCollaboration.userName,
						"fullName":collab.currentUserForCollaboration.fullName
					}]
				};
			}
		}
		
		//publishFullSynchMethod should return the data set to be passed to the joiner
		coachsidePublishFullSynch = publishFullSynchMethod;
		coachsideFullSynchReceivedCallbackMethod = fullSynchReceivedCallbackMethod;
		coachsideIncomingChangesetCallbackMethod = IncomingChangesetCallbackMethod;
		coachsideRosterChangeCallbackMethod = rosterChangeCallbackMethod;
		coachsideSetToReadOnly=setToReadOnly;
		var collabInitialized = collab.initCollaboration();
		
		if(collabInitialized) {
			return collab.sessionRoster;
		}
};

collab.publishData = function(data){
	logger.log("collaboration", "publishData", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, logger.FINE);
	collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, 
						  data);
};


collab.setSkipUserLeavePublish = function (flag){
	collab.skipUserLeavePublish = flag;
};

collab.setUnsubscribeAll = function(flag) {
	collab.unsubAllChannelsOnExit = flag;
};

collab.unsubscribeDataChangeset = function (){
	logger.log("collaboration", "_unsubscribeDataChangeset", "Unsubscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, logger.FINE);
	if(_collaboration_dataChange_subscription){
		collab.cometdForCollab.unsubscribe(_collaboration_dataChange_subscription);
		_collaboration_dataChange_subscription=null;
	}
	
};

function getUserInfo(userID){
	//var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();//TO-DO need to use Kam's code
	var user = null;
	var rest_url = dojoConfig.App._bpmContextRootMap.rest + "/v1/user/current?parts=all";
	if(userID) rest_url =  dojoConfig.App._bpmContextRootMap.rest + "/v1/user/userID?parts=all";
	
		xhr.get({
			url: rest_url,
			handleAs: 'json',
			headers: {
				"Content-Type": "application/json", 
				"Accept": "application/json"
			},
			sync: true,
			load:  function(data, xhrArgs){
				userData = data.data;
			},
			error: lang.hitch(this, handleFailedRestConnection)
		});
	return userData;
};

function getRosterListByTaskID(task_id){
	var servlet_url = dojoConfig.App._bpmContextRootMap.socialBusWeb +"/roster/task/"+task_id;
	var roster;
	xhr.get({
			url: servlet_url,
			handleAs: 'json',
			headers: {
				"Content-Type": "application/json", 
				"Accept": "application/json"
			},
			sync: true,
			preventCache: true,
			load:  function(data, xhrArgs){
				if(data.taskEditorID&&
					data.taskEditorID!=""&&
					data.taskOwnerID&&
					data.taskOwnerID!=""){
						roster={};
						roster.editorID = data.taskEditorID;
						roster.taskOwnerID = data.taskOwnerID;
						roster.numCollabUsers = data.numCollabUsers;
						if(data.participants&&data.participants.length&&data.participants.length>0){
							
							roster.allParticipants = [];
							
							for(var i in data.participants){
								roster.allParticipants.push(data.participants[i]);
							}
						}
				}				
			},
			error: lang.hitch(this, handleFailedRestConnection)
		});
	if(roster){
		if(!roster.allParticipants){
			roster.allParticipants =[{"userName":collab.currentUserForCollaboration.userName,
									  "fullName":collab.currentUserForCollaboration.fullName}];
		}
	}
	return roster;
};

function handleFailedRestConnection(message){
	logger.log("collaboration", "handleFailedRestConnection", message, logger.FINE);
}

function coachsidePublishFullSynch(){};
function coachsideFullSynchReceivedCallbackMethod(){};
function coachsideIncomingChangesetCallbackMethod(){};
function coachsideRosterChangeCallbackMethod(){};
function coachsideSetToReadOnly(){};

collab.initCollaboration = function() {
	//var hostSameFlag = _isHostSame();
	if(!_isCollaborationSupported()){
		logger.log("collaboration", "initCollaboration", "collaborationContainerDiv not found, this is not social portal env. Exit registerForCollaboration.", logger.FINE);
		return false;
	}
	var cometDConnectExist = window.parent["BPMCometDConnectionEstablished"];
	collab.cometdForCollab = window.parent["cometDForCollab"];

	if(cometDConnectExist && collab.cometdForCollab){
		logger.log("collaboration", "initCollaboration", "cometd channel already established ... ", logger.FINE);
		_publishAndSubscribeToChannels();

		if(collab.sessionRoster.allParticipants.length>1){
			startCollaboration();
		}
	}else{
		logger.log("collaboration", "initCollaboration", "cometd channel is not established yet, wait for 2 seconds ... ", logger.FINE);
		_collaboration_timer = setTimeout(lang.hitch(this, 'initCollaboration'), _collaboration_timeout);	
	}
	
	return true;
};

function _publishAndSubscribeToChannels(){
	//TO-DO: should also handle meta connect leaves
	_subscribeFullSynch(collab.currentUserForCollaboration.userName);
	_subscribeUserLeave();
	_subscribeRosterRoleChange();
	_publishUserJoined(	collab.sessionRoster.taskOwnerID,
						collab.sessionRoster.editorID,
						collab.currentUserForCollaboration.userName, 
						collab.currentUserForCollaboration.fullName);
	_subscribeUserJoin();
};
function handleConnectionBroken(message) {
	collab.connectionEstablished = false;
	logger.log("collaboration", "handleConnectionBroken", message, logger.FINE);
};


function startCollaboration(){
	   logger.log("collaboration", "startCollaboration", "Should start banner for collaboration now, editor is:"+ collab.sessionRoster.editorID, logger.FINE);
		if(!collaborationBanner){
			var currentUserID = collab.currentUserForCollaboration.userName;
			var participants = collab.sessionRoster.allParticipants;
			var partner;
			for(var i in participants){
				if(participants[i].userName!=currentUserID)
					partner = participants[i];//only support 1 partner
			}
			if(partner){
				var bannerParams = {};
				bannerParams.isTaskOwner = (collab.sessionRoster.taskOwnerID==currentUserID)? true:false;		
				bannerParams.isEditor = (collab.sessionRoster.editorID==currentUserID)? true:false;
				bannerParams._makeEditor = lang.hitch(this,setEditor);
				bannerParams._takeControl = lang.hitch(this,takeControl);
			
				bannerParams.partnerFullName = partner.fullName;
				bannerParams.partnerUsername = partner.userName;
			
				collaborationBanner = new com.ibm.bpm.coach.collaborationBanner(bannerParams);
			}
		}
};


collab.exitCollaboration = function(beforeUnloadObject, isForceExitByNewClient){
	if(beforeUnloadObject){
		_unsubscribeAllChannels();	
		collab.unsubscribeDataChangeset();
	}

	if(collab.sessionRoster || collaborationBanner) {
		if(isForceExitByNewClient){
			//set coach view to read only 
			coachsideSetToReadOnly(); 
		} else {
			//handle role change
			if(collab.currentUserForCollaboration.userName == collab.sessionRoster.taskOwnerID){
				if(collab.currentUserForCollaboration.userName != collab.sessionRoster.editorID){
					_publishRosterRoleChange(collab.sessionRoster.editorID);
					coachsideRosterChangeCallbackMethod(collab.sessionRoster);
				}
			}
		}

		if(collaborationBanner)	{
			collaborationBanner.remove();
			collaborationBanner = null;
		}
	}
	
	if(collab.currentUserForCollaboration && _checkForUserInCollaboration(collab.currentUserForCollaboration.userName)) {		
		if(isForceExitByNewClient) {
			collab.sessionRoster = null;
		}
		else if(collab.skipUserLeavePublish) {//do not publish user leave if going between coaches
			collab.skipUserLeavePublish = false;
		} else {
			_publishUserLeave(collab.currentUserForCollaboration.userName);
			collab.sessionRoster = null;
		}
	}

	if(collab.unsubAllChannelsOnExit) {
		collab.unsubAllChannelsOnExit = false;
		_unsubscribeAllChannels();
	}
};

function setEditor(evt){
	event.stop(evt);
	//Set the viewer Editor
	var currentUserID = collab.currentUserForCollaboration.userName;
	var participants = collab.sessionRoster.allParticipants;
	var viewer;
	
	for(var i in participants){
		if(participants[i].userName!=currentUserID)
			viewer = participants[i];//only support 1 partner
	}
	
	//collab.sessionRoster.editorID = viewer.userName;
	//change local roster and broadcast change
	if(viewer)
		_assignEditorAccess(viewer.userName);
	//call Callback function
	coachsideRosterChangeCallbackMethod(collab.sessionRoster);	
	_subscribeDataChangeset();
	//switch collaboration bar view
	collaborationBanner.switchToViewerMode(true);
	
	if(collab.sessionRoster.taskOwnerID == currentUserID)
		collaborationBanner.hideTakeControlButton(false);
	
};

function takeControl(evt){
	event.stop(evt);
	//Only task owner can take control	
	var currentUserID = collab.currentUserForCollaboration.userName;	
	var taskOwner = collab.sessionRoster.taskOwnerID;
	if(currentUserID==taskOwner){
		//broadcast change
		_assignEditorAccess(currentUserID);		
		logger.log("collaboration", "takeControl", "wait for 1 second before unsubscribing to the datachange channel ... ", logger.FINE);
		_RosterChangeEvent_timer = setTimeout(lang.hitch(collab, 'unsubscribeDataChangeset'), _RosterChangeEvent_timeout);	
		//call Callback function
		coachsideRosterChangeCallbackMethod(collab.sessionRoster);		
		//switch collaboration bar view
		collaborationBanner.switchToViewerMode(false);
		collaborationBanner.hideTakeControlButton(true);
	}
};

function publishFullSynchData(receiverName, data){
	logger.log("collaboration", "publishFullSynchData", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.fullSync + receiverName, logger.FINE);
	var computedData = {newRoster:collab.sessionRoster,dataSet:data};
	collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.fullSync + receiverName, 
						  computedData);
};

function _assignEditorAccess(newEditorID){
	logger.log("collaboration", "_assignEditorAccess", "started", logger.FINE);
	if(collab.sessionRoster&&(collab.sessionRoster.editorID != newEditorID)){		
		collab.sessionRoster.editorID = newEditorID;
		_publishRosterRoleChange(newEditorID);		
	}
};


function _subscribeDataChangeset(){
	logger.log("collaboration", "_subscribeDataChangeset", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, logger.FINE);
	if(!_collaboration_dataChange_subscription){
		_collaboration_dataChange_subscription = collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.dataChange, 
								coachsideIncomingChangesetCallbackMethod);
		//_collaboration_subscriptions.push({"channelName":collab.collaboratonChannelNames.dataChange, "subscription":_collaboration_dataChange_subscription});
	}
	};



function _unsubscribeAllChannels(){
	if (collab.cometdForCollab && collab.cometdForCollab.batch) {
		collab.cometdForCollab.batch(function(){
			for (var i in _collaboration_subscriptions){
				collab.cometdForCollab.unsubscribe(_collaboration_subscriptions[i]["subscription"]);
			}
		});
	}
};

function _unsubscribeChannel(channelName){
	for (var i in _collaboration_subscriptions){
		if(_collaboration_subscriptions[i]["channelName"] == channelName)
			collab.cometdForCollab.unsubscribe(_collaboration_subscriptions[i]["subscription"]);
	}
};

function _subscribeFullSynch(userName){
	logger.log("collaboration", "_subscribeFullSynch", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.fullSync + userName, logger.FINE);
	_collaboration_subscriptions.push({"channelName":collab.collaboratonChannelNames.fullSync,
										"subscription":collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.fullSync + userName, 
							_handleFullSynchReceived)});
};

function _subscribeUserLeave(){
	logger.log("collaboration", "_subscribeUserLeave", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userLeave, logger.FINE);
	_collaboration_subscriptions.push({"channelName":collab.collaboratonChannelNames.userLeave, 
										"subscription":collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userLeave, 
							_handleUserLeave)});
};

function _publishUserLeave(userName){
	if(collab.cometdForCollab&&collab.cometdForCollab.publish&&userName){
		logger.log("collaboration", "_publishUserLeave", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userLeave+" : "+userName, logger.FINE);
		collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userLeave, 
							  {"userName":userName});
	}
};

function _subscribeUserJoin(){
	logger.log("collaboration", "_subscribeUserJoin", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userJoin, logger.FINE);
	_collaboration_subscriptions.push({"channelName":collab.collaboratonChannelNames.userJoin, 
										"subscription":collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userJoin, 
							_handleUserJoin)});
};

function _publishUserJoined(taskOwnerID, editorID,userName, fullName){
	logger.log("collaboration", "_publishUserJoined", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userJoin+" : "+userName, logger.FINE);
	collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.userJoin, 
						  {"taskOwnerID":taskOwnerID, "editorID":editorID,"userName":userName,"fullName":fullName});
};

function _publishRosterRoleChange(newEditorID){
	logger.log("collaboration", "_publishRosterRoleChange", "Publishing to channel " + _collaborationChannelNamePrefix + collab.sessionID+collab.collaboratonChannelNames.rosterChange, logger.FINE);
	collab.cometdForCollab.publish(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.rosterChange, 
						  {"editorID":newEditorID});
};

function _subscribeRosterRoleChange(){
	logger.log("collaboration", "_subscribeRosterRoleChange", "Subscribing to channel " + _collaborationChannelNamePrefix + collab.sessionID+collab.collaboratonChannelNames.rosterChange, logger.FINE);
	_collaboration_subscriptions.push({"channelName":collab.sessionID+collab.collaboratonChannelNames.rosterChange, 
									"subscription":collab.cometdForCollab.subscribe(_collaborationChannelNamePrefix + collab.sessionID + collab.collaboratonChannelNames.rosterChange, 
						  _handleRosterChange)});
};

function _handleRosterChange(msg){
	var newEditorID = msg.data.editorID;
	if(collab.sessionRoster){
		collab.sessionRoster.editorID = newEditorID;
		var currentUserID = collab.currentUserForCollaboration.userName;
		if(currentUserID == newEditorID){		
			//switch collaboration bar view
			logger.log("collaboration", "_handleRosterChange", "wait for 1 second before unsubscribing to the datachange channel ... ", logger.FINE);
			_RosterChangeEvent_timer = setTimeout(lang.hitch(collab, 'unsubscribeDataChangeset'), _RosterChangeEvent_timeout);	
			if (collaborationBanner) {
			   collaborationBanner.switchToViewerMode(false);
			   collaborationBanner.hideTakeControlButton(true);
			}
		}else{
			_subscribeDataChangeset();
			if (collaborationBanner) {
			   collaborationBanner.switchToViewerMode(true);
			   if(currentUserID==collab.sessionRoster.taskOwnerID) collaborationBanner.hideTakeControlButton(false);		
			}
		}
		coachsideRosterChangeCallbackMethod(collab.sessionRoster);
		
	}else{
		logger.log("collaboration", "_handleRosterChange", "sessionRoster does not exist", logger.FINE);
	}
};
function _handleUserLeave(incomingData){

	logger.log("collaboration", "_handleUserLeave", json.toJson(incomingData), logger.FINE);
	userName = incomingData.data.userName;
	logger.log("collaboration", "_handleUserLeave", "user: " + userName, logger.FINE);
	if(collab.sessionRoster&&collab.sessionRoster.allParticipants){
		for(var i in collab.sessionRoster.allParticipants){
			if(collab.sessionRoster.allParticipants[i].userName == userName){
				collab.sessionRoster.allParticipants.splice(i, 1);
				collab.sessionRoster.numCollabUsers--;
			}
		}
	}
	coachsideRosterChangeCallbackMethod(collab.sessionRoster);
	var numberOfParticipants = 0;
	if(collab.sessionRoster&&collab.sessionRoster.allParticipants){
		for( var j in collab.sessionRoster.allParticipants){
			numberOfParticipants++;
		}
	}
	if(numberOfParticipants<2){
		collab.setSkipUserLeavePublish(true);
		collab.exitCollaboration();
	}
};


function _handleUserJoin(userInfo){
	logger.log("collaboration", "_handleUserJoin", "userInfo: " + json.toJson(userInfo), logger.FINE);
	logger.log("collaboration", "_handleUserJoin", " session roster: " + json.toJson(collab.sessionRoster), logger.FINE);
	var userName = userInfo.data.userName;
	var fullName = userInfo.data.fullName;
	
	var taskId = collab.taskSessionID.task_id.split("t")[1];
	collab.sessionRoster = getRosterListByTaskID(taskId);
	
	if(!collab.sessionRoster){//new session
		logger.log("collaboration", "_handleUserJoin", "sessionRoster does not exist start new sessionRoster...", logger.FINE);
		
		if(userName!=collab.currentUserForCollaboration.userName){//someone joined, let's start collaboration
			collab.sessionRoster = {
				"taskOwnerID": collab.taskOwnerID,
				"editorID": collab.taskOwnerID,
				"numCollabUsers": 1, 
				"allParticipants": [{
										"userName":collab.currentUserForCollaboration.userName,
										"fullName":collab.currentUserForCollaboration.fullName
									},{
										"userName":userName,
										"fullName":fullName
									}]
			};
			logger.log("collaboration", "_handleUserJoin", "new sessionRoster created...", logger.FINE);
			publishFullSynchData(userName,coachsidePublishFullSynch());
			logger.log("collaboration", "_handleUserJoin", "coachsidePublishFullSynch called...", logger.FINE);
		}
	} else {//collaboration is already started
		logger.log("collaboration", "_handleUserJoin", "Collaboration already started with the roster:", logger.FINE);
		logger.log("collaboration", "_handleUserJoin", json.toJson(collab.sessionRoster), logger.FINE);

		var userAlreadyExistInSession = _checkForUserInCollaboration(userName);
		var numOfParticipants = 0;

		for(var i in collab.sessionRoster.allParticipants){
			numOfParticipants++;
		}

		if(!userAlreadyExistInSession){ //add to roster if not already in
				collab.sessionRoster.allParticipants.push({
					"userName":userName,
					"fullName":fullName
				});

				collab.sessionRoster.numCollabUsers++;
				numOfParticipants++;
				//coachsideRosterChangeCallbackMethod(collab.sessionRoster);//shouldnt need this
		}

		if(collab.sessionRoster.editorID == collab.currentUserForCollaboration.userName ||
			numOfParticipants >= 1 || userAlreadyExistInSession) {
				//I am the editor, send out full synch
				publishFullSynchData(userName, coachsidePublishFullSynch());
			}
		
		if(collab.sessionRoster.editorID && collab.sessionRoster.editorID != "" &&
			collab.sessionRoster.editorID != collab.currentUserForCollaboration.userName) {
			_subscribeDataChangeset();
		} else {
			collab.unsubscribeDataChangeset();
		}
	}

	if(collab.sessionRoster && collab.sessionRoster.allParticipants.length > 1) {
		startCollaboration();	//do no subscribe for data change since you are the editor
	}
	
	coachsideRosterChangeCallbackMethod(collab.sessionRoster);
};

function _handleFullSynchReceived(message){
	
	collab.sessionRoster.taskOwnerID = message.data.newRoster.taskOwnerID;
	collab.sessionRoster.editorID = message.data.newRoster.editorID;
	collab.sessionRoster.allParticipants = [];
	
	for(var i in message.data.newRoster.allParticipants){
		collab.sessionRoster.allParticipants.push(message.data.newRoster.allParticipants[i]);
	}
	
	logger.log("collaboration", "_handleFullSynchReceived", "session roster: " + json.toJson(collab.sessionRoster), logger.FINE);

	if(collab.sessionRoster.allParticipants && collab.sessionRoster.allParticipants.length > 1) {
		startCollaboration();	
	}

	if(collab.sessionRoster.editorID&&collab.sessionRoster.editorID != collab.currentUserForCollaboration.userName)
		_subscribeDataChangeset();
	else
		collab.unsubscribeDataChangeset();
	var data = message.data.dataSet;
	coachsideRosterChangeCallbackMethod(collab.sessionRoster);
	coachsideFullSynchReceivedCallbackMethod(data);//subscribe for data change since the session already started before you joined		
};

function _checkForUserInCollaboration(userName){
	if(collab.sessionRoster && collab.sessionRoster.allParticipants){
		for(var i in collab.sessionRoster.allParticipants){
			if(collab.sessionRoster.allParticipants[i].userName == userName)
				return true;
		}
	}
	return false;
};

function _isCollaborationSupported(){
	try{
		var windowLocation = window.location.href;
		var parentLocation = parent.location.href;		
	
		if(windowLocation&&parentLocation){
			var parentParts = parentLocation.split("/");
			var winParts = windowLocation.split("/");
			if(parentParts && winParts&& 
				parentParts[0]==winParts[0]&&//this is the protocal part should be http: or https:
				parentParts[2]==winParts[2]){//this is the hostname+port part should be <hostname>:<port>
				logger.log("collaboration", "_isCollaborationSupported", "Check for hostname and port, succeeded... Proceed...", logger.FINE);
				if(window.parent&&window.parent["collaborationContainerDiv"]){
					logger.log("collaboration", "_isCollaborationSupported", "Check for window.parent and container div succeeded... Proceed with register for collaboration...", logger.FINE);
					return true;
				}else{
					logger.log("collaboration", "_isCollaborationSupported", "Check for window.parent and container div failed... Abort collaboration...", logger.FINE);
					return false;
				}
			}
			else{
				logger.log("collaboration", "_isCollaborationSupported", "Check for hostname and port, failed... Abort collaboration...", logger.FINE);				
				return false;
			}
		}
	}catch(err){//not supported
		logger.log("collaboration", "_isCollaborationSupported", "Not able to access parent object... Collaboration not supported....", logger.FINE);
		return false;
	}
	
	// if not able to get parent location info we should not proceed with collaboration.
	return false;
};

require(["./engine"], function(engine) {
	engine._registerInternalUnloadHandler(collab.exitCollaboration);
});

window["exitCollaboration"]= collab.exitCollaboration;
return collab;
});

},
'dojo/store/Memory':function(){
define(["../_base/declare", "./util/QueryResults", "./util/SimpleQueryEngine"], function(declare, QueryResults, SimpleQueryEngine) {
  //  module:
  //    dojo/store/Memory
  //  summary:
  //    The module defines an in-memory object store.


return declare("dojo.store.Memory", null, {
	// summary:
	//		This is a basic in-memory object store. It implements dojo.store.api.Store.
	constructor: function(/*dojo.store.Memory*/ options){
		// summary:
		//		Creates a memory object store.
		// options:
		//		This provides any configuration information that will be mixed into the store.
		// 		This should generally include the data property to provide the starting set of data.
		for(var i in options){
			this[i] = options[i];
		}
		this.setData(this.data || []);
	},
	// data: Array
	//		The array of all the objects in the memory store
	data:null,

	// idProperty: String
	//		Indicates the property to use as the identity property. The values of this
	//		property should be unique.
	idProperty: "id",

	// index: Object
	//		An index of data indices into the data array by id
	index:null,

	// queryEngine: Function
	//		Defines the query engine to use for querying the data store
	queryEngine: SimpleQueryEngine,
	get: function(id){
		//	summary:
		//		Retrieves an object by its identity
		//	id: Number
		//		The identity to use to lookup the object
		//	returns: Object
		//		The object in the store that matches the given id.
		return this.data[this.index[id]];
	},
	getIdentity: function(object){
		// 	summary:
		//		Returns an object's identity
		// 	object: Object
		//		The object to get the identity from
		//	returns: Number
		return object[this.idProperty];
	},
	put: function(object, options){
		// 	summary:
		//		Stores an object
		// 	object: Object
		//		The object to store.
		// 	options: dojo.store.api.Store.PutDirectives??
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		//	returns: Number
		var data = this.data,
			index = this.index,
			idProperty = this.idProperty;
		var id = (options && "id" in options) ? options.id : idProperty in object ? object[idProperty] : Math.random();
		if(id in index){
			// object exists
			if(options && options.overwrite === false){
				throw new Error("Object already exists");
			}
			// replace the entry in data
			data[index[id]] = object;
		}else{
			// add the new object
			index[id] = data.push(object) - 1;
		}
		return id;
	},
	add: function(object, options){
		// 	summary:
		//		Creates an object, throws an error if the object already exists
		// 	object: Object
		//		The object to store.
		// 	options: dojo.store.api.Store.PutDirectives??
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		//	returns: Number
		(options = options || {}).overwrite = false;
		// call put with overwrite being false
		return this.put(object, options);
	},
	remove: function(id){
		// 	summary:
		//		Deletes an object by its identity
		// 	id: Number
		//		The identity to use to delete the object
		// returns: Boolean
		// 		Returns true if an object was removed, falsy (undefined) if no object matched the id
		var index = this.index;
		var data = this.data;
		if(id in index){
			data.splice(index[id], 1);
			// now we have to reindex
			this.setData(data);
			return true;
		}
	},
	query: function(query, options){
		// 	summary:
		//		Queries the store for objects.
		// 	query: Object
		//		The query to use for retrieving objects from the store.
		//	options: dojo.store.api.Store.QueryOptions?
		//		The optional arguments to apply to the resultset.
		//	returns: dojo.store.api.Store.QueryResults
		//		The results of the query, extended with iterative methods.
		//
		// 	example:
		// 		Given the following store:
		//
		// 	|	var store = new dojo.store.Memory({
		// 	|		data: [
		// 	|			{id: 1, name: "one", prime: false },
		//	|			{id: 2, name: "two", even: true, prime: true},
		//	|			{id: 3, name: "three", prime: true},
		//	|			{id: 4, name: "four", even: true, prime: false},
		//	|			{id: 5, name: "five", prime: true}
		//	|		]
		//	|	});
		//
		//	...find all items where "prime" is true:
		//
		//	|	var results = store.query({ prime: true });
		//
		//	...or find all items where "even" is true:
		//
		//	|	var results = store.query({ even: true });
		return QueryResults(this.queryEngine(query, options)(this.data));
	},
	setData: function(data){
		// 	summary:
		//		Sets the given data as the source for this store, and indexes it
		//	data: Object[]
		//		An array of objects to use as the source of data.
		if(data.items){
			// just for convenience with the data format IFRS expects
			this.idProperty = data.identifier;
			data = this.data = data.items;
		}else{
			this.data = data;
		}
		this.index = {};
		for(var i = 0, l = data.length; i < l; i++){
			this.index[data[i][this.idProperty]] = i;
		}
	}
});

});

},
'dojo/date/stamp':function(){
define(["../_base/kernel", "../_base/lang", "../_base/array"], function(dojo, lang, array) {
	// module:
	//		dojo/date/stamp
	// summary:
	//		TODOC

lang.getObject("date.stamp", true, dojo);

// Methods to convert dates to or from a wire (string) format using well-known conventions

dojo.date.stamp.fromISOString = function(/*String*/formattedString, /*Number?*/defaultTime){
	//	summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	//	description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//			* dates only
	//			|	* yyyy
	//			|	* yyyy-MM
	//			|	* yyyy-MM-dd
	// 			* times only, with an optional time zone appended
	//			|	* THH:mm
	//			|	* THH:mm:ss
	//			|	* THH:mm:ss.SSS
	// 			* and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	// 		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
	//
  	//	formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	//
	//	defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!dojo.date.stamp._isoRegExp){
		dojo.date.stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = dojo.date.stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			array.forEach(array.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
};

/*=====
	dojo.date.stamp.__Options = function(){
		//	selector: String
		//		"date" or "time" for partial formatting of the Date object.
		//		Both date and time will be formatted by default.
		//	zulu: Boolean
		//		if true, UTC/GMT is used for a timezone
		//	milliseconds: Boolean
		//		if true, output milliseconds
		this.selector = selector;
		this.zulu = zulu;
		this.milliseconds = milliseconds;
	}
=====*/

dojo.date.stamp.toISOString = function(/*Date*/dateObject, /*dojo.date.stamp.__Options?*/options){
	//	summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	//	description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	//	dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") +
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
};

return dojo.date.stamp;
});

},
'dijit/main':function(){
define([
	"dojo/_base/kernel"
], function(dojo){
	// module:
	//		dijit
	// summary:
	//		The dijit package main module

	return dojo.dijit;
});

},
'com.ibm.bpm.coach/domHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains a few utilities coach framework uses to manipulate DOM

define(["dojo/_base/array", "dojo/dom-class", "dojo/query", "./CONSTANTS"], function(array, domClass, query, CONSTANTS) {
				
	var domHelper = new Object();

	// only expect input className to be either "ContentBox" or "view"
	// once find a node whose class satisfies, won't go deeper on that node. So the nodes in the return array don't have direct or indirect containership.
	domHelper.shallowQueryByClass = function(node, className, sortResult) {
		var unsortedResult = new Array();
		var deepQueryResults = query(' .' + className, node);
		array.forEach(deepQueryResults, function(oneNode) {
			var parentNode = oneNode.parentNode;
			while(parentNode != node) {
				if(domClass.contains(parentNode, CONSTANTS.CONTENT_BOX) || domClass.contains(parentNode, CONSTANTS.COACH_VIEW)) {
					// if parentNode is view or ContentBox, the node is not what we want
					return;
				}
				parentNode = parentNode.parentNode;
			}
			unsortedResult.push(oneNode);
		});
		
		if(sortResult) {
			return domHelper.sortDomNodes(unsortedResult);
		} else {
			return unsortedResult;
		}		
	};
	
	// sort the nodes to match their orders in the DOM tree
	domHelper.sortDomNodes = function(nodeList) {		
		if(!!nodeList && nodeList.length > 1) {
			var testNode = nodeList[0];
			if(!!testNode.sourceIndex) { // IE
				nodeList.sort(function(nodeA, nodeB) {
					return nodeA.sourceIndex - nodeB.sourceIndex;
				});
			}else if(!!testNode.compareDocumentPosition) {
				nodeList.sort(function(nodeA, nodeB) {
					return 3 - (nodeA.compareDocumentPosition(nodeB) & 6);
				});
			}		
		}
		
		return nodeList;
	}


	return domHelper;
});

},
'com.ibm.bpm.coach/ViewContext':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

define(["require", "dojo/_base/array", "dojo/_base/connect", "dojo/dom-class", "./logger", "./validationHelper", "./coachViewHelper", "./BPM_CONSTANTS", "./CONSTANTS", "./boundaryEventHelper", "./broadcastHelper", "./commonHelper", "./bindingResolver", "dojo/_base/config"], function(require, array, connect, domClass, logger, validationHelper, coachViewHelper, BPM_CONSTANTS, CONSTANTS, boundaryEventHelper, broadcastHelper, commonHelper, bindingResolver, dojoConfig) {

	var ViewContext = function () {	
		this._init = function(ownerView, system, team, element, binding, option, readonly) {
			// Establish the context object
			this.element = element;
			this.binding = binding;
			this.options = option;
			this.readonly = readonly;
			this.subview = {};
			this.bpm = {"system": system, "team": team};
    		this.viewid = this.element.getAttribute(CONSTANTS.VIEW_ID);
			this._parent = ownerView;
			this._parentView = null;
			this._validationSubscribed = false;
			this.contextRootMap = dojoConfig.App._bpmContextRootMap;
		};
	}		
	
	// do we really need this? investigate later
	ViewContext.prototype.constructor = ViewContext;

	/**
	 * summary:
	 *		escape the text to clean xss attacks
	 *		using .createTextNode to utilize the browser escaping capabilities
	 * txt: the string to escape
	 */
	ViewContext.prototype.htmlEscape = function (txt) {
		var element = document.createElement('span');
		var content = document.createTextNode(txt);
		element.appendChild(content);
		return element.innerHTML;
	};
	
	ViewContext.prototype.parentView = function(){
		return this._parentView;
	};
	
	ViewContext.prototype.triggerCollaboration = function(payload) {
        var element = this.element;
        require(["./controller"], function(controller) {
            controller.customCollaborationHandler(element.getAttribute("id"), payload);
        });
	};
	
	ViewContext.prototype.getInheritedVisibility = function(){
		//find its parent and get parent's visibility
		var visibility = "EDITABLE";
		var parentScope = this.parentView();
		
		if (!parentScope || !parentScope.context || !parentScope.context.options || !parentScope.context.options._metadata)
		      return visibility;
			  
		//if it is null, goes up
		visibility = parentScope.context.options._metadata.visibility.get("value");
		if (visibility == "DEFAULT"){
			return parentScope.context.getInheritedVisibility();
		}
		return visibility;
	};
	
	ViewContext.prototype.bindingType = function() {
		return this.element.getAttribute("data-bindingType");
	};
	
	/*
	Fires a named boundary event. When the boundary event is handled, the supplied callback is called. 
	The callback function is optional.
	callback function has a response object as single parameter - {status: boundaryEventStatus}
	boundaryEventStatus in above response object has one of following 4 values:
		"executed" 		- boundary event has been successfully executed.
		"unqualified"	- context.element doesn't have attribute 'data-eventid', boundary event can not be executed.
		"cancelled"		- boundary event is canceled as the result of calling context.cancelBoundaryEvents() API.
		"abandoned"		- boundary event is abandoned by coach framework when coach framework can not process this boundary event, e.g. failed to communicate with server. 	
	
	options is optional, it contains following properties:
		callbackForAll	- boolean value (true/false) to indicate if the supplied callback should be invoked. The default value is false.
						  If the value is true, callback function (if supplied) is invoked regardless the status is "executed", "unqualified", "cancelled" or "abandoned".
						  If the value is false (or "options" parameter does not have property "callbackForAll", or "options" is not provided), 
						  callback function (if supplied) is invoked only when the status is "executed".
	*/
	ViewContext.prototype.trigger = function(callbackfn, options) {
		var _this = this;
		var alwaysInvokeCallback = false;
		if(!!options && options.callbackForAll === true) {
			alwaysInvokeCallback = true;
		} 
		setTimeout(function() {
			var bdEventId = _this.element.getAttribute('data-eventid');
			if (!!bdEventId) {
				var engine = _this._getEngine();
				var boundaryEventInfo = engine.getBoundaryEventInfo(bdEventId);
				var validateBoundaryEventInfo = engine.getValidateBoundaryEventInfo();
				connect.publish("com.ibm.bpm.coach.BoundaryEventTriggered", [{
					boundaryEventInfo: boundaryEventInfo, validateBoundaryEventInfo: validateBoundaryEventInfo, callback:callbackfn, alwaysInvokeCallback: alwaysInvokeCallback
				 }]);
			} else {
				// 'data-eventid' doesn't exist, so no boundary event should be handled. Anyway, need to let caller know
				boundaryEventHelper.callbackForBoundaryEvent(callbackfn, alwaysInvokeCallback, BPM_CONSTANTS.BOUNDARY_EVENT_STATUS_UNQUALIFIED);
			}
		}, 1);
	};
	
	/*
	 broadcast the supplied message. Input parameter message is an object which is expected to have an attribute "name".
	 This API is not supposed to broadcast sensitive or confidential message.
	*/
	ViewContext.prototype.broadcastMessage = function(message) {
		broadcastHelper.broadcastMessage(message);
	};	
	
	/*
	Return turn if origin matches where Coach is loaded from or one of the sites listed on whitelist, 
	Otherwise false.
	Typical scenario - Coach view receives an postMessage event , coach view can check if the event comes 
	from a trusted site by calling this API with event.origin as parameter.	

	origin:	the site to be checked. Should include protocol, host and port.
			Example value of origin: "https://bpm1.ibm.com:9080", "https://bpm2.ibm.com:9443", "http://bpm3.ibm.com".
	*/
	ViewContext.prototype.isTrustedSite = function(origin) {
		var engine = this._getEngine();
		return engine.checkPostMessageOrigin(origin);
	};	
		
	/*
	Cancel all the boundary events that have not been sent to server.
	Once the boundary event is canceled, the callback function (if supplied) is called to notify the status of boundary event.
	See context.trigger(callback) API for detailed information.
	*/
	ViewContext.prototype.cancelBoundaryEvents = function() {
        require(["./controller"], function(controller) {
            controller.clearBoundaryEventQueue(true);
        });
	};
	
	
	// Public API for view developer.
	/*
	If domNode is the node of any view, a single instance of view will be created and returned.
	If domNode is the node of contextBox, framework creates view instances for all the views owned 
	by this contentBox, and recursively create views for all the framework managed contentBoxs 
	that are owned by this contentBox. An array of view instance will be returned.
	*/
	ViewContext.prototype.createView = function(divNode, index, parentView) {
		logger.log("CoachView", "context.createView", "divNode: " + divNode.id + ", index: " + index + ", parentView: " + parentView);

		/* parentViewScope may not be this._parent, for example,
				viewX
					viewB
						ViewA
							CB_1 (fwk managed)
								ViewC
								CB_2 (view managed, constructed by ViewB)
									ViewD
								CB_3 (fwk managed)
									ViewE
		
		CB_2 is constructed by ViewB, but viewB is not the parentViewScope of the subview (ViewD) in CB_2
		Parent of viewD is View A.

        Parent-child relationship is based on dom structure. ie, your most immediate outer view scope is your parent.
		vice versa: your immediate inner view scope is your child
		
		But at creation time, the view scope thats call viewscope.context.createView  may not be the parent of the view to be created
        This happens to view managed content box		
		
		*/	
		
		var engine = this._getEngine();
		var parentViewScope = (!!(parentView)) ? parentView : coachViewHelper.findParentViewScope(this._parent, divNode);
		
		if(!!index || index == 0) {
			engine.adjustIdAndBinding(divNode, parentViewScope, index);
		}
		
		if(engine.isContentBox(divNode)) {
			var newViewList = new Array();			
			engine.buildContentBox(parentViewScope, divNode, newViewList);
			
			for(var k=0; k<newViewList.length; k++) {
				engine.callView(newViewList[k]);
			}
			
			// check if those newly created view instances have special visibility setting
			if(!!engine.GlobalAssets.coach) {
				for(var i=0; i<newViewList.length; i++) {
					engine.GlobalAssets.coach.updateVisibilityIfApplicable(newViewList[i]);
				}
			}
						
			// check if it's needed to deliver the validation event to the newly created views
			var foundNewValidationEvent = false;
			for(var k=0; k<newViewList.length; k++) {
				foundNewValidationEvent = foundNewValidationEvent || validationHelper.validateView(newViewList[k]);
			}
			if(foundNewValidationEvent) {
				// the new view or its descendant views own validation event
				// just tell engine to deliver the new validation event. It's up to engine to decide when to deliver
				engine.deliverValidationEvents();
			}

			return newViewList;			
		} else {
			// this divNode is expected to be a view
	    	var viewScope = engine.initView.apply(engine.GlobalAssets, [divNode, parentViewScope]);
			engine.callView(viewScope);
			
			// check if this newly created view instance has special visibility setting
			if(!!engine.GlobalAssets.coach) {
				engine.GlobalAssets.coach.updateVisibilityIfApplicable(viewScope);
			}

			// check if it's needed to deliver the validation event to the newly created views
			var foundNewValidationEvent = validationHelper.validateView(viewScope);
			if(foundNewValidationEvent) {
				// the new view or its descendant views own validation event
				// just tell engine to deliver the new validation event. It's up to engine to decide when to deliver
				engine.deliverValidationEvents();
			}
			
			return viewScope;
		}				
	};
	
	// Public API for view developer. 		
	ViewContext.prototype.deleteView = function(divNode) {
		logger.log("CoachView", "context.deleteView", "divNode: " + divNode.id);
		if(!divNode) 	return;
		
		var engine = this._getEngine();
		if(engine.isContentBox(divNode)) {
			engine.deleteContentBox(divNode, this._parent);				
		} else {
			// this divNode is expected to be a view
			engine.deleteView(divNode, this._parent);				
		}
	};
	
	ViewContext.prototype.setDisplay = function(isDisplay, domNode) {
		logger.log("CoachView", "context.setDisplay", "domNode: " + domNode + ", isDisplay: " + isDisplay);
		var engine = this._getEngine();
		var domNode1 = (!!domNode)?domNode:this.element;
    	if (!isDisplay) {
    		domClass.add(domNode1, engine.CONSTANTS.COACH_VIEW_HIDDEN);
    	} else {
    		domClass.remove(domNode1, engine.CONSTANTS.COACH_VIEW_HIDDEN);
    	}
	};
	
	ViewContext.prototype.setVisibility = function(isVisible, domNode) {
		logger.log("CoachView", "context.setVisibility", "domNode: " + domNode + ", isVisible: " + isVisible);
		var engine = this._getEngine();
		var domNode1 = (!!domNode)?domNode:this.element;
    	if (!isVisible) {
    		domClass.add(domNode1, engine.CONSTANTS.COACH_VIEW_INVISIBLE);
    	} else {
    		domClass.remove(domNode1, engine.CONSTANTS.COACH_VIEW_INVISIBLE);
    	}
	};

	ViewContext.prototype.log = function(method, message) {
		var engine = this._getEngine();
		var viewName = engine.CONSTANTS.COACH_VIEW;
		var dataType = this.element.getAttribute(engine.CONSTANTS.VIEW_DATA_TYPE);
		if (!!dataType) {
			dataType = dataType.split(".");
			if (!!dataType) {
				viewName = dataType.pop();
			}
		}
		logger.log(viewName, method, message);
	};		
	
	// sortResult is optional, this API returns unsorted array unless sortResult is set to true
	ViewContext.prototype.getSubview = function(viewId, sortResult) {
		logger.log("CoachView", "context.getSubview", "viewId: " + viewId + ", sortResult: " + sortResult);
		var subviewsArray = new Array();
		var subviewsObject = this.subview[viewId];
		for(var divId in subviewsObject) {
			if(!!subviewsObject[divId]) { // there might be "hole" in subviewsObject as the result of deleteView
				subviewsArray.push(subviewsObject[divId]);
			}
		}
		
		if(sortResult && subviewsArray.length >= 2) {
			var testNode = subviewsArray[0].context.element;
			if(!!testNode.sourceIndex) {
				subviewsArray.sort(function (viewScopeA, viewScopeB) {
					return viewScopeA.context.element.sourceIndex - viewScopeB.context.element.sourceIndex;
				});
			}else if(!!testNode.compareDocumentPosition) {
				subviewsArray.sort(function (viewScopeA, viewScopeB) {
					var nodeA = viewScopeA.context.element;
					var nodeB = viewScopeB.context.element;
					return 3 - (nodeA.compareDocumentPosition(nodeB) & 6);
				});
			}				
		}
		
		return subviewsArray;
	}

	ViewContext.prototype.refreshView = function(){
		logger.log("CoachView", "context.refreshView", "is called");
		this._parent._refreshView(true);
	};
	
	/**
	Calls this API to indicate that it is interested in any validation errors that happen on its descendant views.
	When validation error occurs, validate() method of this view is called prior to that of descendant views.
	
	Views that have data binding are titled to validation event implicitly, namely their validate() method 
	are called when validation event occurs (either "clear" type or "error" type). 
	For the views that don't have data binding, e.g. Tab view, if those views are interested in 
	any validation events that happen on its descendant views, those views have to call this.context.subscribeValidation() 
	to register explicitly.
		
	A view who has data binding can also call this.context.subscribeValidation() to receive the validation events of 
	its descendant views, beside of its own validation event. In this case, after the view calls 
	this.context.unsubscribeValidation(), it can still receive its own validation event.	
	*/
	ViewContext.prototype.subscribeValidation = function(){
		logger.log("CoachView", "context.subscribeValidation", "is called");
		this._validationSubscribed = true;
	};
		
	/**
	Calls this API to unsubscribe.
	Nothing happens if subscribeValidation() was not called previously.
	*/
	ViewContext.prototype.unsubscribeValidation = function(){
		logger.log("CoachView", "context.unsubscribeValidation", "is called");
		this._validationSubscribed = false;
	};
	
	/**
	 * Return true if the view has chance to fall into the troubled situation about different lengths of lists, 
	 * otherwise return false.
	 * For example, table view calls context.containDiffLoopingContext() to determine if table needs to disable its 
	 * add/delete capability during the runtime even if customer has specified to enable add/delete through configuration.
	 * 
	 * Example scenario of different length of lists: Let's say we have following View structure -
	 * 			   TableView (bind to ListA[])
	 * 			       ContentBox
	 * 			           TextView (bind to ListB.currentItem)
	 * 
	 * Case 1: When the length of ListA is shorter than that of ListB, some items in ListB won't have chance to be displayed. 
	 * Case 2: When the length of ListA is longer than that of ListB, some instances of TextView won't have valid data binding. 
	 * Case 3: when he length of ListA and ListB are different, TableView has difficulty to support add/delete. 
	 * 
	 * Above 3 cases are what we mean by "troubled situation about different lengths of lists".
	 * Coach framework handles case 1 and case 2 automatically by showing warning message once the length of involved lists 
	 * are different.
	 * 
	 * TableView can call context.containDiffLoopingContext() to handle case 3. Based on TableView's structure and 
	 * binding information, the return of containDiffLoopingContext() is true. So TableView can disable its 
	 * add and delete capacities accordingly.
	 */
	ViewContext.prototype.containDiffLoopingContext = function(){
		logger.log("CoachView", "context.containDiffLoopingContext", "is called");
		return this._parent._checkIfImpactedByMultipleLists();
	};	
	
	/**
	Get the options object defined in the given viewDomNode even if the view represented by viewDomNode has not been constructed yet. 
	Typically this API is used by the view to have a peek at the configure options (e.g. label, visibility etc) of its children views 
	before any of the children views are created.
	
	viewDomNode - The node of any view.
	
	Following is an example:
		var childOptions = this.context.getOptions(childViewDomNode);
		var childLabel = childOptions._metadata.label.get("value");
		childOptions._metadata.visibility.set("value", "READONLY");
	*/
	ViewContext.prototype.getOptions = function(viewDomNode) {
		logger.log("CoachView", "context.getOptions", "is called for view with DOM ID " + viewDomNode.id);
		var tempViewCfgBinding = viewDomNode.getAttribute(CONSTANTS.DATA_CONFIG);

		var cfgBinding = bindingResolver.globalAssets.options[tempViewCfgBinding]; 
		if (!!cfgBinding) {
			try {
				bindingResolver.resolveOptionsBinding(cfgBinding);
			} catch(ex) {
		    	var errorMsg = "ViewContext.getOptions() caught Exception when calling bindingResolver.resolveOptionsBinding(), Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
		    	return null;
			}
		}
		return cfgBinding;
	}
	
	// private API
	// get the value of an option
	ViewContext.prototype._getOptionValue = function(configId, optionKey, isMetadata) {
		logger.log("CoachView", "context._getOptionValue", "is called");
		var engine = this._getEngine();
		return engine.getOptionValue(configId, optionKey, isMetadata);
	}

	ViewContext.prototype._getEngine = function(){
		// we know engine module has been loaded.
		return require("./engine");
	};

	/**
	* This API will set data-ibmbpm-uniqueViewId attribute onto the targetNode.  
	* If targetNode is not specified, the attribute will be set to the 
	* this.context.element dom node.
	*/	
	ViewContext.prototype.setUniqueViewId = function(uniqueViewId, targetNode){
		var viewDOM = (!!targetNode)?targetNode:this.element;
		viewDOM.setAttribute("data-ibmbpm-uniqueViewId", uniqueViewId);
	};

	/** 
	* This API will locate and return the first match of the dom node that has the matched data-ibmbpm-uniqueViewId attribute with the input uniqueViewId.  
	* Null will be returned if no match found.  Search will start from startNode, and by default (without the optional optParam.startNode pass 
	* in) startNode is parent DOM node of this.context.element.  Also, search is by default only limited to the scope of the immediate children of the 
	* startNode unless optional optParam.deepSearch = true is passed in. 
	*/	
	ViewContext.prototype.getDomNode = function(uniqueViewId, optParam){
		var startNode = (!!optParam)?optParam['startNode']:null;
		var deepSearch = (!!optParam)?optParam['deepSearch']:null;
		var startingDom = (!!startNode)?startNode:this.element.parentElement;
    	var selector = "* [data-ibmbpm-uniqueViewId='" + uniqueViewId + "']";
    	var targetDOMs = startingDom.querySelectorAll(selector);
    	var targetDOM = null;
		if (!deepSearch) {
			if (!!targetDOMs) {
				for (var i=0; i<targetDOMs.length; i++) {
					if (targetDOMs[i].parentElement == this.element.parentElement) {
						targetDOM = targetDOMs[i];
						break;
					}
				}
			}
		} else {
			if ((!!targetDOMs) && (targetDOMs.length > 0)) {
				targetDOM = targetDOMs[0];
			}
		}
		return targetDOM;
	};
				
	return ViewContext;

});
},
'com.ibm.bpm.coach/logger':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// logger js
define(["require", "dojo/_base/lang", "dojo/has"], function(require, lang, has) { 
	// logger start
	var logger = new Object();
	
	logger.debugComp = {};
	logger.components = null;
	
	logger.INFO = "INFO";
	logger.FINE = "FINE"; 
	logger.FINER = "FINER";

	logger.init = function() {
		//get log details from session
		logger.components = new RegExp("coachDebugTrace=([^&#]*)").exec(window.location.href);
		if (logger.components != null && logger.components != "") {
			logger.components = unescape(logger.components[1]);
			var compArray = logger.components.split(";");
			if (compArray != null && compArray.length != 0) {
				for (var i=0; i<compArray.length; i++) {
					var level = compArray[i].split(":");
					if (level != null && level.length >1) {
						logger.debugComp[level[0]] = level[1];
					} else {
						logger.debugComp[compArray[i]] = logger.FINE;
					}
				}
			}
		} else {
			logger.components = "";
		}
		
	}

	logger.log = function (comp, funct, message, level) {
	
		if (level == undefined) {
			level = logger.FINE;
		}
		var allComponents = !!logger.debugComp["*"];
		
		if (allComponents && logger.debugComp[comp] == undefined) {
			//"*" selected so turn on all tracing
		    if (logger.debugComp["*"] == logger.FINER ||
		    	(level == logger.FINE && (logger.debugComp["*"].indexOf(logger.FINE) != -1)) ||
		    	(level == logger.INFO && ((logger.debugComp["*"] == logger.INFO) || (logger.debugComp["*"].indexOf(logger.FINE) != -1))))  {
		    	if (has("ie") <=8 || has("safari") > 0) {
					console.log(comp + "." + funct + "():" + message);
		    	} else if (lang.isArray(message)) {
		    		message.unshift(comp + "." + funct + "(): ");
		    		console.log.apply(this,message);
		    	}else {
					console.log(comp + "." + funct + "():", message);		    		
		    	}
		    }
		} else if (!!logger.debugComp[comp]) {
			//check specific component
		    if (logger.debugComp[comp] == logger.FINER  ||
		    	(level == logger.FINE && (logger.debugComp[comp].indexOf(logger.FINE) != -1)) ||
		    	(level == logger.INFO && ((logger.debugComp[comp] == logger.INFO) || (logger.debugComp[comp].indexOf(logger.FINE) != -1))))  {
                if (has("ie") <=8 || has("safari") > 0) {
					console.log(comp + "." + funct + "():" + message);
                }else if (lang.isArray(message)) {
		    		message.unshift(comp + "." + funct + "(): ");
		    		console.log.apply(this,message);
		    	}else {
					console.log(comp + "." + funct + "():", message);		    		
		    	}
		    }
		}
	}

	logger.destroy = function() {
	}

	// logger end
	
	return logger;
});

},
'com.ibm.bpm.coach/templateStore':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// 

define(["dojo/_base/lang"], function(lang) {
				
	var templateStore = {typeToResolvedTemplateMap: {}, typeToCycledTemplateMap: {}, typeToRawTemplateMap: {}, pathToTypeMap: {}};

	templateStore.init = function(globalAssets) {
		// dataStructures.defaults typically has content like following:
		// defaults:{S:"",I:0,D:0.0,DT:new Date(),B:false}
		if(globalAssets.dataStructures) {
			templateStore.typeToResolvedTemplateMap = globalAssets.dataStructures.defaults;
			templateStore.typeToRawTemplateMap = globalAssets.dataStructures.templates;
			// delete the reference in globalAssets
			delete globalAssets.dataStructures;
		}
	}
	
	templateStore.getByType = function(type) {
		var resolvedTemplate = templateStore.typeToResolvedTemplateMap[type];
		// Note: "" and 0 are valid template
		if(resolvedTemplate === undefined) {
			resolvedTemplate = templateStore.typeToCycledTemplateMap[type];
			if(resolvedTemplate === undefined) {
				// the resolved template for this type will be saved into templateStore.typeToResolvedTemplateMap or templateStore.typeToCycledTemplateMap
				// during the execution of templateStore._assembleTemplate.
				var assembleResult = templateStore._assembleTemplate(type, true, {});
				resolvedTemplate = assembleResult.template; 
			}
		}
		return resolvedTemplate;
	}
	
	templateStore.getTypeByPath = function(path) {
		var refinedPath = templateStore._generalizePath(path);
		return templateStore._getTypeByPath(refinedPath);
	}
	
	// input parameter path should be generalized one
	templateStore._getTypeByPath = function(path, pathArray) {
		if(templateStore.pathToTypeMap[path]) {
			return templateStore.pathToTypeMap[path];
		} else {
			// try to figure it out my type from my path information
			if(!pathArray) {
				pathArray = path.split(".");
			}
			if(pathArray.length > 1) {
				var lastPath = pathArray[pathArray.length - 1];
				var newPathArray = pathArray.slice(0, (pathArray.length - 1));
				var newPath = newPathArray.join(".");
				var parentType = templateStore._getTypeByPath(newPath, newPathArray);
				if(parentType) {
					var parentRawTemplate = templateStore.typeToRawTemplateMap[parentType];
					if(parentRawTemplate && parentRawTemplate[lastPath]) {
						var type = parentRawTemplate[lastPath].replace("[]", "");
						templateStore.pathToTypeMap[path] = type;
						return type;
					}
				}
			}
		}
		return null;		
	}
	
	templateStore.save = function(path, type) {
		var refinedPath = templateStore._generalizePath(path);
		if(!templateStore.pathToTypeMap[refinedPath]) {
			templateStore.pathToTypeMap[refinedPath] = type;
		}		
	}
	
	templateStore._generalizePath = function(path) {
		return path.replace(/\[.*\]/g, "");
	}
					
	// assemble template for specified type.
	// Note: templateStore.typeToResolvedTemplateMap and templateStore.typeToCycledTemplateMacan only be edited in this function after they are initialized
	templateStore._assembleTemplate = function(type, isStartPoint, typeToInterimTemplateMap) {
		var result = {template: {}, isCycled: false};
		var rawTemplate = templateStore.typeToRawTemplateMap[type];
		
		if(!rawTemplate) {
			// if rawTemplate doesn't exist, we assume it was deleted when it was visited previously. Now it's visited again, so cycled reference occurs
			result.isCycled = true;
			typeToInterimTemplateMap[type] = result.template; // value is blank object {}
		} else {
			// delete templateStore.typeToRawTemplateMap[type] to avoid endless loop of type referencing.
			// e.g. initially templateStore.typeToRawTemplateMap is 
			// {t3: {p1: "S", p2: "t2", p3: "t2"}, t2: {p4: "I[]", p5: "t1", p6: "t2"}, t1: {p7: "DT", p8: "t3", p9: "t2"}}
			// based on current algorithm (depth firstly traverse), possible endless loop are on following properties in order: p8, p9, p6
			// However, since templateStore.typeToRawTemplateMap["t3"] has been deleted, p8 gets {} as its template. So are p9 and p6
			// As to p3, when it is visited, templateStore.typeToResolvedTemplateMap["t2"] has been created thanks to p2, so no issue.
			delete templateStore.typeToRawTemplateMap[type];
			
			for(var property in rawTemplate) {
				var subType = rawTemplate[property];
				if(subType.indexOf("[]") != -1) {
					// List type, create an empty List with proper te (template)
					result.template[property] = {items: [], listAllSelectedIndices: [], te: subType.replace("[]", "")};
				} else {
					var subTemplate = templateStore.typeToResolvedTemplateMap[subType];
					// Note: "" and 0 are valid template
					if(subTemplate === undefined) {
						subTemplate = typeToInterimTemplateMap[subType];
						if(subTemplate === undefined) {
							var subAssembledResult = templateStore._assembleTemplate(subType, false, typeToInterimTemplateMap);
							// subAssembledResult.template exists for sure because the worst case is that templateStore._assembleTemplate() provides a blank object
							subTemplate = subAssembledResult.template;
							if(subAssembledResult.isCycled) {
								result.isCycled = true;
							}
						} else {
							// subTemplate comes from typeToInterimTemplateMap, all the templates in typeToInterimTemplateMap has cycled reference
							result.isCycled = true;
						}
					}
					result.template[property] = lang.clone(subTemplate);
				}
			}
			
			if(result.isCycled) {
				// the resolved template is in favour of the starting type. so we only save the starting type in templateStore.typeToCycledTemplateMap, 
				// put others in typeToInterimTemplateMap.
				// e.g. {t1: {p1: t2, p2: t2}, t2: {p3: t1}}, if starting type is t1, the resolved result would be
				// t1 -- {p1: {p3: {}}, p2: {p3: {}}}
				// t2 -- {p3: {}}
				// t1 gets its best version, the template goes to templateStore.typeToCycledTemplateMap. 
				// t2's current template is in favour of t1, so it goes to typeToInterimTemplateMap which only exists when t1 is assembled.
				// when t2 becomes starting type, it will get its final version, i.e. t2  -- {p3: {p1: {}, p2: {}}}				
				if(isStartPoint) {
					templateStore.typeToCycledTemplateMap[type] = result.template;
				} else {
					// don't check if typeToInterimTemplateMap[type] already exists, simply update the interim template even if it exists. 
					// See following example for the reason:
					// {t1: {p1: t2, p2: t1, p3: t2}, t2: {p4: t2, p5: t1}}
					// first interim template for t2 is blank object {}, created when p4 is visited. interim template for t1 is also blank object {} 
					// when p5 is visited.
					// when p1 is completed, new interim template for t2 is worked out as {p4: {}, p5{}}. p2 still use {} as interim template for t1.
					// when p3 is visited, t2's new interim template is used.
					// final template for t1 is {p1: {p4: {}, p5{}}, p2: {}, p3: {p4: {}, p5{}}}				
					typeToInterimTemplateMap[type] = result.template;
				}
					
				// templateStore.typeToRawTemplateMap[type] was deleted. However, raw template of this type needs to be restored when cycle is detected.
				// 1. If this type is not the startPoint, the resolved template for this type is not the final version. 
				//    Raw template is needed to work out this type's final version of template when this type becomes the start point in the future.
				// 2. If this type is the startPoint, the resolved template for this type is the final version. However,
				//    We may still need this type's raw template when other type's template is resolved.
				//    Example, {t1: {p1: t2}, t2: {p2: t1}}. When start point is t1, t1's resolved template is {p1: {p2: {}}. 
				//    When t2 becomes the start point later, we need the help from t1's raw template to work out {p2: {p1: {}} as t2's resolved template.
				//    If we use t1's resolved template, we will end up with {p2: {p1: {p2: {}}} as t2's resolved template, which is not good.
				
				// In the example of {t1: {p1: t2, p2: t1, p3: t2}, t2: {p4: t2, p5: t1}}, t2's raw template is restored when p1 is completed. But it would be used when p3 is handled.
				// Following order must be followed when it's time to resolve template: templateStore.typeToResolvedTemplateMap, typeToInterimTemplateMap, rawTemplate
				// templateStore.typeToCycledTemplateMap is not used, because the template there is in favour of its owner type.
				templateStore.typeToRawTemplateMap[type] = rawTemplate;
			} else {
				// no cycle. If isStartPoint is true, we get the template we plan to resolve. If isStartPoint is false, we get a great by-product
				templateStore.typeToResolvedTemplateMap[type] = result.template;
			}
		}
		
		return result;
	}
	
	return templateStore;
});

},
'com.ibm.bpm.coach/CoachView':function(){
/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach view super class js

define(["dojo/_base/lang", "dojo/_base/array", "dojo/dom", "dojo/dom-class", "./CONSTANTS", "./commonHelper", "./domHelper", "./engine", "./collaborationPopup","./logger", "./dataBindingHelper", "./multiBindingListsHelper", "./validationHelper", "./coachViewHelper", "./bindingResolver", "./specificBindingHelper"], function(lang, array, dom, domClass, CONSTANTS, commonHelper, domHelper, engine, collaborationPopup, logger, dataBindingHelper, multiBindingListsHelper, validationHelper, coachViewHelper, bindingResolver, specificBindingHelper) {
	var CoachView = function (context, element, binding, option, readonly) {
		
		if(!!context) {
			this.context = context;
			this.context._init(this, engine.GlobalAssets.system, engine.GlobalAssets.team, element, binding, option, readonly);

			// if the view binds with a list and is framework managed, use following templates to handle the dynamic adding (e.g. list has been inserted a new item)
			this._contentBoxTemplates = [];
			// use this counter to work out the unique id (div id) for the content in the repeating ContentBox
			this._counter = 0;
					
			// hold all the handles of binding
			this._bindingHandles = {};
			
			this._impactedByMultipleLists = undefined;
			this._domIdToBindingListMap = {};
			// we monitor those list objects
			this._bindingStrToListMap = {};	

		    this._changeObject = lang.hitch(this, function(property, oldVal, newVal) {
		    	//Change handler for Stateful objects, calls the subclass change handler
		    	if(this._isReady()) {
			    	var event = {property: property, oldVal: oldVal, newVal: newVal, removedFrom: null, insertedInto: null};
			    	this._change(event);
		    	}
		    });
    
		}		
	}
	
	CoachView.prototype.constructor = CoachView;
	
    CoachView.prototype.view = function() {
    	// Super-class lifeCycle view method
    	var visibility = (!!this.context.options  &&  (!!this.context.options._metadata) && !!(this.context.options._metadata.visibility)) ? this.context.options._metadata.visibility.get("value"): "DEFAULT";
    	if (visibility == "DEFAULT") {
    		visibility = this.context.getInheritedVisibility();
    	}
    	if (visibility == "NONE") {
    		this.context.setDisplay(false);
    	} else {
    		this.context.setDisplay(true);
    	}

    };
    
    // Internal method
    CoachView.prototype._preLoad = function () {
    }
    
    CoachView.prototype.load = function() {
    	// Super-class lifeCycle load method
    };
    
    // Internal method
    CoachView.prototype._preUnload = function() {
    	for(var k in this._bindingHandles) {
			if(!!this._bindingHandles[k]) {
				if(!!this._bindingHandles[k].unbind){
					this._bindingHandles[k].unbind();
				}
				else if(!!this._bindingHandles[k].cancel){
					this._bindingHandles[k].cancel();
				}
			}
    	}
    	this._bindingHandles = null;
    }

    CoachView.prototype.unload = function() {
    	// Super-class lifeCycle unload method
    	// console.debug("unload() is called on viewid " + this.context.viewid + ", Class is " + this.context.element.getAttribute("Class"));
    };
    
    // Internal method to make the CoachView object ready for garbage collection
    CoachView.prototype._postUnload = function() {
    	// considering this.context could be referenced by something, set the content of context to null firstly then set context to null
    	this.context.element = null;
    	this.context.binding = null;
    	this.context.options = null;
    	this.context.subview = null;
    	this.context.bpm = null;
    	this.context._parent = null;
    	this.context._parentView = null;	    	
    	this.context = null;	    	
    }

	CoachView.prototype._change = function(rawEvent, eventType, eventProperty){
		var event = this._formalizeEvent(rawEvent, eventType, eventProperty);
		
		engine.callViewLifeCycleAPI(this, "change", CONSTANTS.ELAPSED_TIME_THRESHOLD_CHANGE, event);
		
		//post change,
		//one thing needs to do is to check if we need to update the visibility of children
		if (event.type == "config" && event.property == "_metadata.visibility"){
			var visibility;
			//go through its subview, if the value is default, then..
			var subviewScope;
			var subviewScopes;
			for (var subviewId in this.context.subview){
				subviewScopes = this.context.subview[subviewId];
				for (var realViewId in subviewScopes){
					subviewScope = subviewScopes[realViewId];
					if (subviewScope.context.options && subviewScope.context.options._metadata && subviewScope.context.options._metadata.visibility.get('value') == "DEFAULT"){						
						subviewScope._change(event);
					}			
				}
			}				
		}		
	}
			
	
	CoachView.prototype._formalizeEvent = function(rawEvent, eventType, eventProperty) {
		// every change event contains following properties
		var event = {property: null, oldVal: rawEvent.oldVal, newVal: rawEvent.newVal};
		
		if(typeof eventProperty != "undefined") {
			event.property = eventProperty;
		}else if(typeof rawEvent.property != "undefined") {
			event.property = rawEvent.property;
		}
		
		if(typeof eventType != "undefined") {
			event.type = eventType;
		}else if(typeof rawEvent.type != "undefined") {
			event.type = rawEvent.type;
		}

		if(typeof rawEvent.insertedInto != "undefined") {
			event.insertedInto = rawEvent.insertedInto;
		}
		if(typeof rawEvent.removedFrom != "undefined") {
			event.removedFrom = rawEvent.removedFrom;
		}
		if(typeof rawEvent.path != "undefined") {
			event._path = rawEvent.path;
		}
		return event;
	}
	
	// general comment - there are 3 levels of change for dataBinding:
	// 1. The value of resolved binding object is changed. Namely, same resolved binding object is still used. 
	//    When it happens, _bindingChanged() is invoked, event._isResolvedVarChanged doesn't exist.
	// 2. The resolved binding object itself is replaced. As by-product, the value of resolved binding object is also determined as changed.
	//    When it happens, _bindingChanged() is invoked, event._isResolvedVarChanged references to the new resolved object.
	// 3. A new binding String is assigned on this view instance. e.g. old one is "local.listA[2].p", new one is "local.listA[1].p", because "local.listA[0]" is deleted.
	//    This scenario is handled in _setRtDataBindingStr(). Basically rebind for "_bindingChanged" (via specificBindingHelper), 
	//    get a new resolved binding object (main purpose is to make sure _objectPath in resolved object is up to date).
	//    Since the only difference between new/old binding string is the index, no need to rebind for "_listUpdated" and "_watchRtDataBinding".	
	CoachView.prototype._bindingChanged = function(event, calledByRefreshView){
		if(event._isResolvedVarChanged) {
			// resolved binding object is completely replace. example: this view binds to local.a.b, when resolved object of local.a gets a new value, the resolved object of local.a.b becomes completely new.
			this.context.binding = event._newResolvedVar;
		}
						
		if(this.context.binding && this.context.binding.get("value") instanceof com.ibm.bpm.coach.List){
			// the change is on the list. as the result, this.context.binding.get("value") might point to different object.
			// because delete this.childrenCache[name] is called in set(name, value) in Stateful.js
			// so need to rebind "_listUpdated" and "_watchRtDataBinding" if they exist.
			var bindingHandle = this._bindingHandles["_listUpdated"];
			if (bindingHandle) {
				// if bindingHandle exists, there is fwk-managed contentBox (see _buildChildren() for detail).
				// need to re-bind and adjust the instances of contextBox
				if(bindingHandle.unbind){
					bindingHandle.unbind();
				}
				// bind on new object
				this._bindingHandles["_listUpdated"] = this.context.binding.get("value").bindAll(lang.hitch(this, this._listUpdated));
			}
			
			bindingHandle = this._bindingHandles["_watchRtDataBinding"];
			if (bindingHandle) {
				if(bindingHandle.unbind){
					bindingHandle.unbind();
				}
				// bind on new object
				this._bindingHandles["_watchRtDataBinding"] = this.context.binding.get("value").bindAll(lang.hitch(this, this._watchRtDataBinding));
			}
			
			// when _refreshView() calls _bindingChanged(), no need to execute following block because _refreshView() has updated runtime data binding strings and rebuild the repeated contentBoxes.
			if(!calledByRefreshView) {
				// If old list and new list have same length, need to do nothing.
				// Because the view instances that bind to list element (e.g local.listA[0], local.listB[1].p) will receive their notification and update accordingly -
				// resolved binding object, rebind (e.g. for _listUpdated callback) etc.
				// If the length is not different, need to delete all the repeated contentBox and re-create them.			
				if(event.oldVal && event.oldVal.items && event.newVal && event.newVal.items && event.oldVal.items.length == event.newVal.items.length) {
				} else {
					this._rebuildRepeatedContentBoxs();
				}
			}			
		}
		
		this._highlightBindingIssue();
		
		// event object should not be modified.
		this._change(event);
	}
	
	// predefined option
	CoachView.prototype._metadataChanged = function(event, metadataKey){
		if(event._isResolvedVarChanged) {
			this.context.options._metadata[metadataKey] = event._newResolvedVar;
		}

		// event object should not be modified.
		var eventType = "config";
		// event.property already exists. However, event.property means what really has changed. 
		// It could be the property of local variable, or property of config variable. The event we are preparing here
		// is for the change event handler. If a coach view instance's _metadata.visibility binds to local.varA,
		// event's property should be "_metadata.visibility" instead of "varA" when it's delivered via change event handler.
		var refinedEventProperty = "_metadata." + metadataKey;

		this._change(event, eventType, refinedEventProperty);
	}
	
	// normal option
	CoachView.prototype._optionChanged = function(event, optionKey){
		if(event._isResolvedVarChanged) {
			this.context.options[optionKey] = event._newResolvedVar;
		}
		// event object should not be modified.
		this._change(event, "config", optionKey);
	}
	
	CoachView.prototype._setRtDataBindingStr = function(newStr) {
		var oldRtBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
		if(oldRtBindingStr != newStr) {
			dataBindingHelper.rtDataBinding._set(this.context.element, newStr);
			
			// re-resolve the bindingStr to make sure this.context.binding is up to date
			this.context.binding = newStr ? bindingResolver.resolveBinding(newStr) : null;
			this._registerCallBackForDataBinding(newStr);
			this._highlightBindingIssue();
		}
	}
	
	CoachView.prototype._registerCallBackForDataBinding = function(bindingStr) {
		this._registerCallBack(bindingStr, this._bindingChanged, this, "_bindingChanged");
	}
	
	CoachView.prototype._registerCallBackForOptions = function() {
	    var options = this.context.options;
		for(var optionKey in options) {	
		    if (optionKey == "_metadata"){
				for (var metadataKey in options[optionKey]){
					var scopeObject = {viewInstance: this, metadataKey: metadataKey};
					var callbackFunc = function(event) {
						// "this" is scopeObject defined above
						this.viewInstance._metadataChanged(event, this.metadataKey);
					}
					var optionUniqueName = "_metadata." + metadataKey;
					var staticOptionStr = options[optionKey][metadataKey].staticOptionStr;
					var bindingStr = options[optionKey][metadataKey].bindingStr;
					if(staticOptionStr) {
						this._registerCallBack(staticOptionStr, callbackFunc, scopeObject, optionUniqueName);
					} else if(bindingStr) {
						this._registerCallBack(bindingStr, callbackFunc, scopeObject, optionUniqueName);
					}
				}
			}
			else{			
				var scopeObject = {viewInstance: this, optionKey: optionKey};
				var callbackFunc = function(event) {
					this.viewInstance._optionChanged(event, this.optionKey);
				}
				var staticOptionStr = options[optionKey].staticOptionStr;
				var bindingStr = options[optionKey].bindingStr;
				if(staticOptionStr) {
					this._registerCallBack(staticOptionStr, callbackFunc, scopeObject, optionKey);
				} else if(bindingStr) {
					this._registerCallBack(bindingStr, callbackFunc, scopeObject, optionKey);
				}
			}
		}
	}

	CoachView.prototype._registerCallBack = function(bindingStr, callbackFunc, scopeObject, handleName) {
		// unbind old handler
		var oldBindingHandler = this._bindingHandles[handleName];
		if(oldBindingHandler) {
			if(oldBindingHandler.unbind){
				try{
					oldBindingHandler.unbind();
				}catch(ex) {
					var errorMsg = "CoachView._registerCallBack() caught Exception when unbind " + bindingStr + ", Exception: " + ex;
					commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
				}
			}
		}
		
		if(bindingStr) {
			this._bindingHandles[handleName] = specificBindingHelper.bind(bindingStr, callbackFunc, scopeObject);
		}
	}
	
    CoachView.prototype.change = function(event) {
		// Default behavior.  Call view method if subview do not have any change logic
    	this.view();
    };	
	
    CoachView.prototype.collaboration = function(event) {
        
		var node = dom.byId(event.domId);
		
		/* now engine guarantee that event.domId is the right one. So we don't need to have following logic here any more. 
		if(!node || !dom.isDescendant(node, this.context.element)) {
			// notice that sometimes event.domId (detected in editor side) is the id generated by dojo, that id may not exist in the dom node of this coachView in viewer side.
			// For those cases, use coachView's dom node instead.
			node = this.context.element;
		}
		*/
		
    	if(event.type == "focus") {
			if(!this.originalOutline && !!node.style.outline && node.style.outline != "") {
				this.originalOutline = node.style.outline;
			}
			// node.style.border = "solid " + event.outlineColor;
			node.style.outline = "1px solid " + event.outlineColor;
			
			collaborationPopup.showCollaboratorInfo(node, event.color, event.bgColor, event.message, this.context.element);				
    	} else if(event.type == "blur") {
 			if(!!this.originalOutline) {
				node.style.outline = this.originalOutline;
				delete this.originalOutline;
			} else {
   				node.style.outline = "none";
			}
    		
    	} else if(event.type == "mouseOn") {
			collaborationPopup.showCollaboratorInfo(node, event.color, event.bgColor, event.message, this.context.element);
    	} else if(event.type == "mouseOff") {
			collaborationPopup.hideCollaboratorInfo(node);	    		
    	} else if(event.type == "dataChange") {
			collaborationPopup.showDataChange(node, event.message, this.context.element);
    	}    	
    };
    
	/**
	The default implementation provided by framework does nothing, it's each view's responsibility 
	to decide what to do when it receives the validation event.
	
	If validation errors occur during the process of boundary event, coach framework calls validate(event) 
	on all the involved views. When both parent and children views are involved, parent view is called 
	prior to children views. Framework guarantees that involved views are only called once per boundary event.
	
	Details about the input parameter event:		
	event.type can be either "error" or "clear".
	
	If event.type is "clear", event doesn't have any other property.		
	If event.type is "error", event has 2 more properties as following:
	
	- event.errors: list of errors. This view receives those errors because its data binding or part of data binding 
	  has validation errors. Each error object has 3 properties: binding, message and view_dom_ids.		
		a) binding is the path relatives to the data binding of current view.
		b) message is the localized message.
		c) view_dom_ids is the list of DOM id of the views that are either current view or the descendant views of current view and also receive the error message.		
	- errors_subcription: list of errors. This view receives those errors because it has called subscribeValidation() 
	  and some of its descendant views have validation errors on them. If the error of one view has been described 
	  in event.errors, it won't show in errors_subcription. Each error object has 2 properties: view_dom_id and messages.		
		a) view_dom_id is the DOM id of the view that contains the error.
		b) messages is a list of the localized messages.
	
	event example: 
	{		
	   type: "error",
	   errors: 
	           [
	            {
					binding: "ownerBirthday.year"
					message: "2030 is after current year of 2012",
					view_dom_ids: ["domId_PhoneBookView", "domId_yearView"]
	            }
	          ]
	   errors_subcription: 
	          [
	            {
					view_dom_id: "domId_advertisementView",
					messages: ["something wrong"]
	            }
	          ]
	}
	
	Since coach framework calls validate() on all involved views synchronously, the execution of this method 
	is expected to complete shortly.
	
	*/
    CoachView.prototype.validate = function(event) {
    }
    
	CoachView.prototype._buildChildren = function() {
		var currentViewScope = this;
		var currentDiv = currentViewScope.context.element;
	
		var binding = currentViewScope.context.binding;
		var bindToArray = (!!binding && binding.get("value") instanceof com.ibm.bpm.coach.List);			

		// view could hold subviews directly, those direct subview won't repeat even if parent view binds with a list
		var directViewList = domHelper.shallowQueryByClass(currentDiv, engine.CONSTANTS.COACH_VIEW, true);
		var oneDirectView;
		array.forEach(directViewList, function(item, index) {
			oneDirectView = engine.initView.apply(engine.GlobalAssets, [item, currentViewScope]);
		});			
	
		// find the direct ContentBox under this view
		var immediateContentBoxList = domHelper.shallowQueryByClass(currentDiv, engine.CONSTANTS.CONTENT_BOX, true);
		
		if(bindToArray) {
			if(!!binding && binding.get("value").length() == 0) {
				// view binds an empty list, no repeat will occurs, so no subview will call _addRelatedList() to report.
				// we have to simulate to collection information for the multiple binding list support here.
				this._simulateRepeatingToDetectMultiLists(immediateContentBoxList);
			}

			// Register listener for List changes to watch if rtDataBinding is dirty. Do it for both view managed and framework managed. 
			var oneHandler = binding.get("value").bindAll(lang.hitch(currentViewScope, currentViewScope._watchRtDataBinding));
			this._bindingHandles["_watchRtDataBinding"] = oneHandler;
		}			
		
		// only care about the framework managed ContentBox. view managed ContentBox will be built by its parent view.
		var fwkManagedContentBoxs = new Array();
		for(var k=0; k<immediateContentBoxList.length; k++) {
			if(!engine.isViewManagedContentBox(immediateContentBoxList[k])) {
				// Framework managed
				fwkManagedContentBoxs[fwkManagedContentBoxs.length] = immediateContentBoxList[k];
			}
		}
		
		if(fwkManagedContentBoxs.length == 0) {
			// no Framework managed contentBox, so job is done
			return;
		}
		
		if(bindToArray) {
			// save the template in the viewScope
			this._contentBoxTemplates = fwkManagedContentBoxs;
			
			// keep the templates in the dom since we need them to indicate where to insert 
			// when dynamically adding from nothing (the list was empty before adding).			
			// Hide those templates by changing its css class from "ContentBox" to "ContentBoxTemplate" so they don't take the space
			for(var i=0; i<this._contentBoxTemplates.length; i++) {
				domClass.replace(this._contentBoxTemplates[i], engine.CONSTANTS.CONTENTBOX_TEMPLATE, engine.CONSTANTS.CONTENT_BOX);
			}
			if(!!binding) {
				// Register listener for List changes
				var bindingHandle = binding.get("value").bindAll(lang.hitch(currentViewScope, currentViewScope._listUpdated));
				this._bindingHandles["_listUpdated"] = bindingHandle;
				
				// need to repeat the framework managed contentBox based on the data-binding						
				for(var k=0; k<binding.get("value").length(); k++) {
					this._repeatContentBoxs(k, false);
				}
			}
		} else {
			for(var i=0; i<fwkManagedContentBoxs.length; i++) {
				engine.buildContentBox(currentViewScope, fwkManagedContentBoxs[i]);
			}
		}	
	}
	
	// when binding list is empty, or ViewContext.containDiffLoopingContext() is called before view is fully initialized,
	// have to simulate to collect enough information to decide if this view involves multiple lists
	// this._impactedByMultipleLists is assigned in the function
	CoachView.prototype._simulateRepeatingToDetectMultiLists = function(immediateContentBoxList) {
		this._impactedByMultipleLists = false;

		if(multiBindingListsHelper.isBindingObjectList(this)) {
			if(!immediateContentBoxList) {
				immediateContentBoxList = domHelper.shallowQueryByClass(this.context.element, engine.CONSTANTS.CONTENT_BOX, true);			
			}
			
			for(var k=0; k<immediateContentBoxList.length; k++) {
				var oneDiv = immediateContentBoxList[k].cloneNode(true);						
				engine.adjustIdAndBinding(oneDiv, this, 0);		
				// this.context.element.parentNode.appendChild(oneDiv); // only for debug purpose
			}

	    	var listBindingStrArray = multiBindingListsHelper.analyzeRelatedListBindingStrs(this);
	    	this._impactedByMultipleLists = (listBindingStrArray.length > 0);
		}
	}
	
	// Repeat all the contentBox templates (of framework managed contentBoxs) for the specified bindingIndex.
	CoachView.prototype._repeatContentBoxs = function(bindingIndex, callView) {
		for(var i=0; i<this._contentBoxTemplates.length; i++) {
			var templateNode = this._contentBoxTemplates[i];
			var oneDiv = templateNode.cloneNode(true);
			// ContentBox template's css class is "ContentBoxTemplate", change it to "ContentBox"						
			domClass.replace(oneDiv, engine.CONSTANTS.CONTENT_BOX, engine.CONSTANTS.CONTENTBOX_TEMPLATE);
			oneDiv.setAttribute(engine.CONSTANTS.TEMPLATE_ID, templateNode.id);

			// content boxes are not always directly under their CoachView's element
			templateNode.parentNode.insertBefore(oneDiv, templateNode);

			engine.adjustIdAndBinding(oneDiv, this, bindingIndex);
			var newViewList = new Array();
			engine.buildContentBox(this, oneDiv, newViewList);
			
			if(callView == true) {
				for(var k=0; k<newViewList.length; k++) {
					engine.callView(newViewList[k]);
				}
				
				// check if those newly created view instance has special visibility setting
				if(!!engine.GlobalAssets.coach) {
					for(var i=0; i<newViewList.length; i++) {
						engine.GlobalAssets.coach.updateVisibilityIfApplicable(newViewList[i]);
					}
				}

				// check if it's needed to deliver the validation event to the newly created views
				var foundNewValidationEvent = false;
				for(var j=0; j<newViewList.length; j++) {
					foundNewValidationEvent = foundNewValidationEvent || validationHelper.validateView(newViewList[j]);
				}
				if(foundNewValidationEvent) {
					// the new view or its descendant views own validation event
					// just tell engine to deliver the new validation event. It's up to engine to decide when to deliver
					engine.deliverValidationEvents();
				}
			}
		}
	}
	
	// If bindingIndex is specified, delete the framework managed contentBox instances according to the binding index
	// If bindingIndex is not specified, delete all framework managed contentBox instances
	CoachView.prototype._deleteContentBoxs = function(bindingIndex) {
		var contentBoxGroup = this._findContentBoxGroup(bindingIndex);
		for(var k=0; k<contentBoxGroup.length; k++) {
			// delete the contentBox 
			engine.deleteContentBox(contentBoxGroup[k], this);
			// content boxes are not always directly under their CoachView's element
			contentBoxGroup[k].parentNode.removeChild(contentBoxGroup[k]);
		}
	}
	
	CoachView.prototype._watchRtDataBinding = function(changeEvent) {			
		if(changeEvent.removedFrom >= 0 && changeEvent.insertedInto >= 0 && changeEvent.removedFrom == changeEvent.insertedInto) {
			// event is "set", we don't care about it
			return;
		} else {
			if(changeEvent.removedFrom >= 0) { // one item has been removed from the list.	
				// notify validationHelper
				var dataBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
				
				validationHelper.listItemRemoved(dataBindingStr, changeEvent.removedFrom, changeEvent.oldVal);
				
				// do 2 things:
				// 1. mark templates dirty. The reason to only mark templates dirty instead of correcting them right way is performance, 
				//    dirty templates may not be used at all. Let the dirty templates accumulate, correct them when those templates are needed.
				// 2. update the runtime binding path immediately.
				var contentBoxGroup = this._findContentBoxGroup(changeEvent.removedFrom, true);
				var affectedSubviewArray = [];
				for(var k=0; k<contentBoxGroup.length; k++) {
					coachViewHelper.findSubviews(affectedSubviewArray, this, contentBoxGroup[k]);
				}
				
				var bindingStrToListMap = {};
				for(k=0; k<affectedSubviewArray.length; k++) {
					// mark template dirty
					affectedSubviewArray[k]._markTemplatesDirty();
					// correct the runtime binding path if needed
					dataBindingHelper.updateAllRtDataBinding(affectedSubviewArray[k], bindingStrToListMap);					
				}
			}
			
			if(changeEvent.insertedInto >= 0) {
				// when we support insert in the middle later, we need to do similar thing like delete
			}	
		}	
    };
	

    CoachView.prototype._listUpdated = function(changeEvent) {
    	/*
		console.debug("list change detected, object: ", changeEvent.object, 
		"removedFrom: ",changeEvent.removedFrom, 
		"insertedInto: ",changeEvent.insertedInto);
		*/
		
		if(changeEvent.removedFrom >= 0 && changeEvent.insertedInto >= 0 && changeEvent.removedFrom == changeEvent.insertedInto) {
			// don't need to do anything. All the impacted view instances will be updated automatically: replace the binding object if needed, fire change event.
		} else {
			// if both changeEvent.removedFrom and changeEvent.insertedInto exist, execute both
			if(changeEvent.removedFrom >= 0) { // one item has been removed from the list.	
				this._deleteContentBoxs(changeEvent.removedFrom);
			}
			
			if(changeEvent.insertedInto >= 0 && this._contentBoxTemplates.length != 0) {
				if(this._isTemplatesDirty) {
					// templates are dirty. Because some list item has been deleted e.g. my binding is local.listA[2].listB, but local.listA[1] was deleted.
					// As the result, my templates are dirty. Meanwhile, my relative that binds to local.listA[0].listB is still fine.
					// hard to correct the dirty templates. The meaningful way is to delete the branch that contains the dirty templates, force the re-build.
					// Once the related views are rebuilt, the newly inserted item is handled automatically.
					// Find a parent view that has non-dirty templates, then rebuild the contentBoxs from there.
					var parentView = this.context.parentView();
					while(parentView) {
						if(parentView._contentBoxTemplates.length != 0 && !parentView._isTemplatesDirty) {
							// find the healthy view
							break;
						}
						parentView = parentView.context.parentView();
					}
					if(parentView) {
						parentView._rebuildRepeatedContentBoxs();
					} else {
						// this branch should not be reached. Anyway, have it to be safe. If it does happen, refresh entire branch starting from the topLevel view.
						var warningMsg = "CoachView._listUpdated(), Rare scenario: The templates of view instance (DOM Id " + this.context.element.id 
							+ ") is dirty. However, coach framework failed to find a view that has healthy templates in its parental chain. Coach framework will call refreshView() on the topLevel view.";
						commonHelper.consoleLog(warningMsg, commonHelper.WARN);
						this._getTopLevelView()._refreshView(true);
					}
				} else {
					// now we only support to add to the end of the list, figure out the proper position to insert when list supports to add item at any position.
					this._repeatContentBoxs(changeEvent.insertedInto, true);
				}
			}	
		}	
    };
    
    // If bindingIndex is specified and continueToEnd is not true, find one contentBox instance for each template based on the bindingIndex.
	// If bindingIndex is specified and continueToEnd is true, for each template, find the contentBox instances whose position is equal or greater than bindingIndex.
    // If bindingIndex is NOT specified, find all the contentBox instances for all the contentBox templates owned by this CoachView
    CoachView.prototype._findContentBoxGroup = function(bindingIndex, continueToEnd) {
		var contentBoxGroup = new Array();
    	var rootNode = this.context.element;
		for(var i=0; i<this._contentBoxTemplates.length; i++) {
			// find the contentBox who was created using current template
			var contentBoxList = rootNode.querySelectorAll(' .' + engine.CONSTANTS.CONTENT_BOX + '[' + engine.CONSTANTS.TEMPLATE_ID + '=' + this._contentBoxTemplates[i].id + ']'); 
			if(bindingIndex == undefined) {
				// all instances qualify
				for(var k=0; k<contentBoxList.length; k++) {
					contentBoxGroup.push(contentBoxList[k]);
				}
			} else if(contentBoxList.length > bindingIndex) {
				if(continueToEnd) {
					for(var i=bindingIndex; i<contentBoxList.length; i++) {
						contentBoxGroup.push(contentBoxList[i]);
					}
				} else {
					contentBoxGroup.push(contentBoxList[bindingIndex]);
				}
			}
		}
		return contentBoxGroup;
    }
    
    CoachView.prototype._addRelatedList = function(viewDomId, relatedList) {
    	this._domIdToBindingListMap[viewDomId] = relatedList;
    }
    
    CoachView.prototype._monitorRelatedListsIfNeeded = function() {
    	this._impactedByMultipleLists = false;
    	
		if(!multiBindingListsHelper.isBindingObjectList(this)) {
    		return;
		}

		var myDataBindingObject = this.context.binding.get("value");			
    	var listBindingStrArray = multiBindingListsHelper.analyzeRelatedListBindingStrs(this);
    	if(listBindingStrArray.length > 0) {
    		this._impactedByMultipleLists = true;
    		var listStr;
			for(var k=0; k<listBindingStrArray.length; k++) {
				listStr = listBindingStrArray[k];
    			if(!this._bindingStrToListMap[listStr]) {
    				var resolvedList = bindingResolver.resolveBinding(listStr).get("value");
    				this._bindingStrToListMap[listStr] = resolvedList;
    				
					var bindingHandle = resolvedList.bindAll(lang.hitch(this, this._monitorListsLengthChange));
					this._bindingHandles["_monitorLengthOf_" + listStr] = bindingHandle;
    			}
			}
			this._bindingHandles["_monitorLengthOf_myOwnBinding"] = myDataBindingObject.bindAll(lang.hitch(this, this._monitorListsLengthChange));		    		
    		multiBindingListsHelper.processMonitoredLists(this, engine);
    	}
    }
    
    CoachView.prototype._monitorListsLengthChange = function(changeEvent) {
    	// we only care the length change, so either add or delete, not set
    	if(changeEvent.removedFrom != changeEvent.insertedInto) {
    		multiBindingListsHelper.processMonitoredLists(this, engine);
    	}
    }
    
    CoachView.prototype._checkIfImpactedByMultipleLists = function() {
		if(this._impactedByMultipleLists == undefined) {
		
			// this._impactedByMultipleLists is assigned in _simulateRepeatingToDetectMultiLists() or _monitorRelatedListsIfNeeded()
			// this._impactedByMultipleLists is still undefined, means that _monitorRelatedListsIfNeeded() is not called. So this view is not fully rendered yet.
			// As the result, maybe this view has some List binding that has not be discovered yet. 
			// So, have to trigger framework to evaluate the contentBox templates of this view to discover all the related List bindings.
			
			// have to simulate to collection information for the multiple binding list support here.
			this._simulateRepeatingToDetectMultiLists();
		}
		
		return this._impactedByMultipleLists;
    }
    
    CoachView.prototype._isReady = function() {
    	return !!this.context;
    }
    
	CoachView.prototype._getTopLevelView = function() {
		var aView = this;
		while(!!aView.context.parentView()) {
			aView = aView.context.parentView();
		}
		return aView;
	}
    
	// recursively check this view and its subviews, if there is templates for framework managed contentbox, make them dirty.
	CoachView.prototype._markTemplatesDirty = function() {
		if(this._contentBoxTemplates.length > 0) {
			// owns templates for framework managed contentBox
			this._isTemplatesDirty = true;
		}
		
		for (var subviewId in this.context.subview){
			for (var subviewDomId in this.context.subview[subviewId]){
				this.context.subview[subviewId][subviewDomId]._markTemplatesDirty();
			}
		}				
	}

	CoachView.prototype._rebuildRepeatedContentBoxs = function() {
		// only care about the templates of framework managed contentBox.
		if(this._contentBoxTemplates.length != 0) {
			//delete all framework managed content boxes				
			this._deleteContentBoxs();
			
			//repeat all framework managed content boxes: 		
			for(var k=0; k<this.context.binding.get("value").length(); k++) {
				this._repeatContentBoxs(k, true);
			}
		}
	}
	
	CoachView.prototype._refreshView = function(updateRtDataBindingStr) {
		if(!this._isReady()) {
			// this view instance is supposed to be recycled.
			return;
		}
	
		// step 1: delete all framework managed content boxes (owner view binds to list, so doesn't include the non-repeating contentBox)
		if(this._contentBoxTemplates.length != 0) {
			//delete all framework managed content boxes				
			this._deleteContentBoxs();
		}
		
		// step 2: update runtime Data Binding string
		if(updateRtDataBindingStr) {
			var topLevelParentView = this._getTopLevelView();
			// call updateAllRtDataBinding() on topLevelParentView. Actually all runtime binding path should be up-to-date. Do it just to be safe.
			dataBindingHelper.updateAllRtDataBinding(topLevelParentView);
		}
		
		// step 3: re-resolve dataBinding, re-bind on it
		var oldPlainBindingValue = engine.getPlainValue(this.context.binding);
		var propertyName = this.context.binding ? this.context.binding.property: null;
		var rtBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
		this.context.binding = rtBindingStr ? bindingResolver.resolveBinding(rtBindingStr) : null;
		this._registerCallBackForDataBinding(rtBindingStr);
		
		// step 4: re-resolve configure options, re-bind on them
		bindingResolver.resolveOptionsBinding(this.context.options);
		this._registerCallBackForOptions();
		
		// step 5: re-create framework managed content boxes (owner view binds to list)
		if(this._contentBoxTemplates.length != 0) {
			//repeat all framework managed content boxes: 		
			for(var k=0; k<this.context.binding.get("value").length(); k++) {
				this._repeatContentBoxs(k, true);
			}
		}

		// step 6: recursively refresh subviews
		var childView;
		for (var viewId in this.context.subview) {		    
			var subviews = this.context.subview[viewId];
			for(var divId in subviews) {
				childView = subviews[divId];
				if(childView && childView._isReady()) {
					childView._refreshView(false);
				}
			}
		}
		
		// step 7: trigger change() for dataBinding
		var newPlainBindingValue = engine.getPlainValue(this.context.binding);
		// when both newPlainBindingValue and oldPlainBindingValue are null (likely because the bindingStr is ""), commonHelper.isValuesEqual() returns commonHelper.UNKNOWN (in the sake of backward compatibility).
		// however, make no sense to fire change event for this scenario.
		if((newPlainBindingValue != null || oldPlainBindingValue != null) && commonHelper.isValuesEqual(newPlainBindingValue, oldPlainBindingValue) != commonHelper.YES) {
			// if failed to get propertyName from old binding, try to see if new binding has it
			if(!propertyName) {
				propertyName = this.context.binding ? this.context.binding.property: null;
			}
			var event = {property: propertyName, oldVal: oldPlainBindingValue, newVal: newPlainBindingValue, path: rtBindingStr};
			this._bindingChanged(event, true);
		}
		
		// step 8: trigger change() for configure options
		engine.notifyDataConfigBindingChange(this);
	}	
	
	CoachView.prototype._highlightBindingIssue = function() {
		if(!com_ibm_bpm_global.preference.highlightUnresolvedBinding) {
			return;
		}
		var dataBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
	    if(!!dataBindingStr) {
			var dataBinding = this.context.binding;
	    	// if dataBinding doesn't exist,
	    	// or dataBinding.boundObject is list, but can't get value from it (index is out of bound). 
	    	if(!dataBinding ||(dataBinding.boundObject instanceof com.ibm.bpm.coach.List && (dataBinding.get("value") == null || typeof dataBinding.get("value") == "undefined"))) {
		    	// we failed to resolve binding for this view. so highlight it here to let user be aware 
		    	this.context.element.style.outline = "1px dotted red";
		    } else {
				// remove the highlight which indicates binding issue if the issue is gone.
				if(this.context.element.style.outline == "1px dotted red") {
					this.context.element.style.outline = "none";
				}
			}
	    }
	}	

	CoachView.prototype._getBindingStrings = function() {
		var bindingStrArray = new Array();
		// make sure rtDataBinding is up-to-date before we call it	
		var dataBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
		bindingStrArray.push(dataBindingStr);
				
		return bindingStrArray;
	}

    return CoachView;
	
});


},
'dojo/io-query':function(){
define(["./_base/lang"], function(lang){
	// module:
	//		dojo/io-query
	// summary:
	//		This module defines query string processing functions.

    var backstop = {};

    function objectToQuery(/*Object*/ map){
        // summary:
        //		takes a name/value mapping object and returns a string representing
        //		a URL-encoded version of that object.
        // example:
        //		this object:
        //
        //	|	{
        //	|		blah: "blah",
        //	|		multi: [
        //	|			"thud",
        //	|			"thonk"
        //	|		]
        //	|	};
        //
        // yields the following query string:
        //
        //	|	"blah=blah&multi=thud&multi=thonk"

        // FIXME: need to implement encodeAscii!!
        var enc = encodeURIComponent, pairs = [];
        for(var name in map){
            var value = map[name];
            if(value != backstop[name]){
                var assign = enc(name) + "=";
                if(lang.isArray(value)){
                    for(var i = 0, l = value.length; i < l; ++i){
                        pairs.push(assign + enc(value[i]));
                    }
                }else{
                    pairs.push(assign + enc(value));
                }
            }
        }
        return pairs.join("&"); // String
    }

    function queryToObject(/*String*/ str){
        // summary:
        //		Create an object representing a de-serialized query section of a
        //		URL. Query keys with multiple values are returned in an array.
        //
        // example:
        //		This string:
        //
        //	|		"foo=bar&foo=baz&thinger=%20spaces%20=blah&zonk=blarg&"
        //
        //		results in this object structure:
        //
        //	|		{
        //	|			foo: [ "bar", "baz" ],
        //	|			thinger: " spaces =blah",
        //	|			zonk: "blarg"
        //	|		}
        //
        //		Note that spaces and other urlencoded entities are correctly
        //		handled.

        // FIXME: should we grab the URL string if we're not passed one?
        var dec = decodeURIComponent, qp = str.split("&"), ret = {}, name, val;
        for(var i = 0, l = qp.length, item; i < l; ++i){
            item = qp[i];
            if(item.length){
                var s = item.indexOf("=");
                if(s < 0){
                    name = dec(item);
                    val = "";
                }else{
                    name = dec(item.slice(0, s));
                    val  = dec(item.slice(s + 1));
                }
                if(typeof ret[name] == "string"){ // inline'd type check
                    ret[name] = [ret[name]];
                }

                if(lang.isArray(ret[name])){
                    ret[name].push(val);
                }else{
                    ret[name] = val;
                }
            }
        }
        return ret; // Object
    }

    return {
        objectToQuery: objectToQuery,
        queryToObject: queryToObject
    };
});
},
'com.ibm.bpm.coach/CONSTANTS':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

define([], function() {
				
	// holds all the constants used by framework, those constants are visible in the generated html
	var CONSTANTS = {
		// css class names
		CONTENT_BOX:		"ContentBox",
		COACH_VIEW:			"CoachView",
		COACH_VIEW_SHOW:	"CoachView_show",
		COACH_VIEW_HIDDEN:	"CoachView_hidden",
		COACH_VIEW_INVISIBLE:	"CoachView_invisible",
		TOP_LEVEL:			"topLevel",
		CONTENTBOX_TEMPLATE:"ContentBoxTemplate",
		COACH_VIEW_WARNING:	"CoachView_warning",
	
		// attribute names
		VIEW_ID:			"data-viewId",
		VIEW_DATA_TYPE:		"data-type",
		DATA_BINDING:		"data-binding",
		DATA_BINDING_RT:	"data-bindingRT",
		DATA_CONFIG:		"data-config",
		IS_VIEW_MANAGED:	"data-view-managed",
		TEMPLATE_ID:		"templateId",
		
		// internal constants
		COACH_SHIMMER_ID_BASE:	"coachViewShimmer",
		TYPE_COACH:				"type_coach",
		TYPE_VIEW:				"type_view",
		TYPE_FWK_CONTENTBOX:	"type_fwk_contentBox",
		TYPE_VIEW_CONTENTBOX:	"type_view_contentBox",
		
		//
		CURRENT_ITEM:		"index",
		INDEX_WITH_BRACKET:	"[index]",		
		
		// binding handle types
		HANDLE_TYPE_NATIVE:		"handle_type_native",
		HANDLE_TYPE_DYNAMIC:	"handle_type_dynamic",
		HANDLE_TYPE_SPECIFIC:	"handle_type_specific",
		
		// millisecond
		ELAPSED_TIME_THRESHOLD_LOAD: 100,
		ELAPSED_TIME_THRESHOLD_UNLOAD: 50,
		ELAPSED_TIME_THRESHOLD_VIEW: 50,
		ELAPSED_TIME_THRESHOLD_CHANGE: 50,
		ELAPSED_TIME_THRESHOLD_COLLABORATION: 50,
		ELAPSED_TIME_THRESHOLD_VALIDATE: 50,

		ELAPSED_TIME_THRESHOLD_VISIBILITY_SCRIPT: 100,
			
		// following 4 are used to specify the location of view instance in the DOM tree
		TOPLEVEL_VIEW_ID:	"topLevelViewId",
		VIEW_PATH:			"viewPath",
		SUBVIEW_ID:			"subviewId",
		SUBVIEW_INDEX:		"subviewIndex"
		
	};
	
	
	return CONSTANTS;
});
	
},
'dijit/TooltipDialog':function(){
require({cache:{
'url:dijit/templates/TooltipDialog.html':"<div role=\"presentation\" tabIndex=\"-1\">\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" data-dojo-attach-point=\"containerNode\" role=\"dialog\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\"></div>\n</div>\n"}});
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.replace
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.hitch
	"./focus",
	"./layout/ContentPane",
	"./_DialogMixin",
	"./form/_FormMixin",
	"./_TemplatedMixin",
	"dojo/text!./templates/TooltipDialog.html",
	"."		// exports methods to dijit global
], function(declare, domClass, event, keys, lang,
			focus, ContentPane, _DialogMixin, _FormMixin, _TemplatedMixin, template, dijit){

/*=====
	var ContentPane = dijit.layout.ContentPane;
	var _DialogMixin = dijit._DialogMixin;
	var _FormMixin = dijit.form._FormMixin;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

	// module:
	//		dijit/TooltipDialog
	// summary:
	//		Pops up a dialog that appears like a Tooltip


	return declare("dijit.TooltipDialog",
		[ContentPane, _TemplatedMixin, _FormMixin, _DialogMixin], {
		// summary:
		//		Pops up a dialog that appears like a Tooltip

		// title: String
		// 		Description of tooltip dialog (required for a11y)
		title: "",

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for TooltipDialog, since TooltipDialog
		//		is never a child of a layout container, nor can you specify the size of
		//		TooltipDialog in order to control the size of an inner widget.
		doLayout: false,

		// autofocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// baseClass: [protected] String
		//		The root className to use for the various states of this widget
		baseClass: "dijitTooltipDialog",

		// _firstFocusItem: [private] [readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_firstFocusItem: null,

		// _lastFocusItem: [private] [readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_lastFocusItem: null,

		templateString: template,

		_setTitleAttr: function(/*String*/ title){
			this.containerNode.title = title;
			this._set("title", title)
		},

		postCreate: function(){
			this.inherited(arguments);
			this.connect(this.containerNode, "onkeypress", "_onKey");
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner){
			// summary:
			//		Configure widget to be displayed in given position relative to the button.
			//		This is called from the dijit.popup code, and should not be called
			//		directly.
			// tags:
			//		protected
			var newC = "dijitTooltipAB" + (corner.charAt(1) == 'L' ? "Left" : "Right")
					+ " dijitTooltip"
					+ (corner.charAt(0) == 'T' ? "Below" : "Above");

			domClass.replace(this.domNode, newC, this._currentOrientClass || "");
			this._currentOrientClass = newC;
		},

		focus: function(){
			// summary:
			//		Focus on first field
			this._getFocusItems(this.containerNode);
			focus.focus(this._firstFocusItem);
		},

		onOpen: function(/*Object*/ pos){
			// summary:
			//		Called when dialog is displayed.
			//		This is called from the dijit.popup code, and should not be called directly.
			// tags:
			//		protected

			this.orient(this.domNode,pos.aroundCorner, pos.corner);
			this._onShow(); // lazy load trigger
		},

		onClose: function(){
			// summary:
			//		Called when dialog is hidden.
			//		This is called from the dijit.popup code, and should not be called directly.
			// tags:
			//		protected
			this.onHide();
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handler for keyboard events
			// description:
			//		Keep keyboard focus in dialog; close dialog on escape key
			// tags:
			//		private

			var node = evt.target;
			if(evt.charOrCode === keys.TAB){
				this._getFocusItems(this.containerNode);
			}
			var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
			if(evt.charOrCode == keys.ESCAPE){
				// Use setTimeout to avoid crash on IE, see #10396.
				setTimeout(lang.hitch(this, "onCancel"), 0);
				event.stop(evt);
			}else if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === keys.TAB){
				if(!singleFocusItem){
					focus.focus(this._lastFocusItem); // send focus to last item in dialog
				}
				event.stop(evt);
			}else if(node == this._lastFocusItem && evt.charOrCode === keys.TAB && !evt.shiftKey){
				if(!singleFocusItem){
					focus.focus(this._firstFocusItem); // send focus to first item in dialog
				}
				event.stop(evt);
			}else if(evt.charOrCode === keys.TAB){
				// we want the browser's default tab handling to move focus
				// but we don't want the tab to propagate upwards
				evt.stopPropagation();
			}
		}
	});
});

},
'dojox/date/hebrew/Date':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"./numerals"
], function(dojo, declare, numerals){

dojo.getObject("date.hebrew.Date", true, dojox);
dojo.experimental("dojox.date.hebrew.Date");

dojo.declare("dojox.date.hebrew.Date", null, {
	// summary: A Date-like object which implements the Hebrew calendar
	//
	// description:
	//	A Date-like object which implements the Hebrew Calendar.  Because this object
	//	implements many of the same methods as the native JavaScript Date object, which
	//	implements the Gregorian calendar, it can often be used its place.  Note that
	//	this object does not extend Date or use its prototype.
	//
	// example:
	// |	dojo.require("dojox.date.hebrew.Date");
	// |
	// |	var date = new dojox.date.hebrew.Date();
	// |	console.log(date.getFullYear()+'\'+date.getMonth()+'\'+date.getDate());

	// Hebrew date calculations are performed in terms of days, hours, and
	// "parts" (or halakim), which are 1/1080 of an hour, or 3 1/3 seconds.
	//_HOUR_PARTS: 1080,
	//_DAY_PARTS: 24*1080,
   
	// An approximate value for the length of a lunar month.
	// It is used to calculate the approximate year and month of a given
	// absolute date.
	//_MONTH_FRACT: 12*1080 + 793,
	//_MONTH_PARTS: 29*24*1080 + 12*1080 + 793,
	    
	// The time of the new moon (in parts) on 1 Tishri, year 1 (the epoch)
	// counting from noon on the day before.  BAHARAD is an abbreviation of
	// Bet (Monday), Hey (5 hours from sunset), Resh-Daled (204).
	//_BAHARAD: 11*1080 + 204,

	// The Julian day of the Gregorian epoch, that is, January 1, 1 on the
	// Gregorian calendar.
	//_JAN_1_1_JULIAN_DAY: 1721426,

	/**
	* The lengths of the Hebrew months.  This is complicated, because there
	* are three different types of years, or six if you count leap years.
	* Due to the rules for postponing the start of the year to avoid having
	* certain holidays fall on the sabbath, the year can end up being three
	* different lengths, called "deficient", "normal", and "complete".
	*/
	
	//"Absolute" indexes of  months:  Tishri -  0, Heshvan - 1, Kislev - 2, Tevet - 3, Shevat -  4, Adar I (leap years only) - 5, Adar - 6, Nisan - 7, Iyar - 8, Sivan - 9,   Tammuz-10, Av - 11, Elul - 12.

	_MONTH_LENGTH:  [
		// Deficient  Normal     Complete
		[   30,	    30,	    30	],		 //Tishri    0
		[   29,	    29,	    30	],		 //Heshvan   1
		[   29,	    30,	    30	],		 //Kislev   2
		[   29,	    29,	    29	],		 //Tevet  3
		[   30,	    30,	    30	],		 //Shevat  4
		[   30,	    30,	    30	],		 //Adar I (leap years only)  5
		[   29,	    29,	    29	],		 //Adar  6
		[   30,	    30,	    30	],		 //Nisan  7
		[   29,	    29,	    29	],		 //Iyar  8
		[   30,	    30,	    30	],		 //Sivan  9
		[   29,	    29,	    29	],		 //Tammuz  10
		[   30,	    30,	    30	],		 //Av  11
		[   29,	    29,	    29	]		 //Elul  12
	],

	/**
	* The cumulative # of days to the end of each month in a non-leap year
	* Although this can be calculated from the MONTH_LENGTH table,
	* keeping it around separately makes some calculations a lot faster
	*/
	_MONTH_START:  [
		// Deficient  Normal	Complete
		[    0,		0,		0  ],		// (placeholder)
		[   30,	    30,	    30  ],		// Tishri
		[   59,	    59,	    60  ],		// Heshvan
		[   88,	    89,	    90  ],		// Kislev
		[  117,	   118,	   119  ],		// Tevet
		[  147,	   148,	   149  ],		// Shevat
		[  147,	   148,	   149  ],		// (Adar I placeholder)
		[  176,	   177,	   178  ],		// Adar
		[  206,	   207,	   208  ],		// Nisan
		[  235,	   236,	   237  ],		// Iyar
		[  265,	   266,	   267  ],		// Sivan
		[  294,	   295,	   296  ],		// Tammuz
		[  324,	   325,	   326  ],		// Av
		[  353,	   354,	   355  ]		// Elul
	],

	/**
	* The cumulative # of days to the end of each month in a leap year
	*/
	_LEAP_MONTH_START:  [
		// Deficient  Normal	Complete
		[    0,		0,		0  ],		// (placeholder)
		[   30,	    30,	    30  ],		// Tishri
		[   59,	    59,	    60  ],		// Heshvan
		[   88,	    89,	    90  ],		// Kislev
		[  117,	   118,	   119  ],		// Tevet
		[  147,	   148,	   149  ],		// Shevat
		[  177,	   178,	   179  ],		// Adar I
		[  206,	   207,	   208  ],		// Adar II
		[  236,	   237,	   238  ],		// Nisan
		[  265,	   266,	   267  ],		// Iyar
		[  295,	   296,	   297  ],		// Sivan
		[  324,	   325,	   326  ],		// Tammuz
		[  354,	   355,	   356  ],		// Av
		[  383,	   384,	   385  ]		// Elul
	],
	
	_GREGORIAN_MONTH_COUNT:  [
		//len len2   st  st2
		[  31,  31,   0,   0 ], // Jan
		[  28,  29,  31,  31 ], // Feb
		[  31,  31,  59,  60 ], // Mar
		[  30,  30,  90,  91 ], // Apr
		[  31,  31, 120, 121 ], // May
		[  30,  30, 151, 152 ], // Jun
		[  31,  31, 181, 182 ], // Jul
		[  31,  31, 212, 213 ], // Aug
		[  30,  30, 243, 244 ], // Sep
		[  31,  31, 273, 274 ], // Oct
		[  30,  30, 304, 305 ], // Nov
		[  31,  31, 334, 335 ] // Dec
		// len  length of month
		// len2 length of month in a leap year
		// st   days in year before start of month
		// st2  days in year before month in leap year
	],

    _date: 0,
	_month: 0,
	_year: 0,
	_hours: 0,
	_minutes: 0,
	_seconds: 0,
	_milliseconds: 0,
	_day: 0,

 	constructor: function(){
		// summary: initialize the date object value
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		var date2 = new dojox.date.hebrew.Date(date1);
		// |
		// |		var date3 = new dojox.date.hebrew.Date(5768,2,12);

		var len = arguments.length;
		if(!len){// use the current date value, added "" to the similarity to date
			this.fromGregorian(new Date());
		}else if(len == 1){
			var arg0 = arguments[0];
			if(typeof arg0 == "number"){ // this is time "valueof"
				arg0 = new Date(arg0);
			}

			if(arg0 instanceof Date){
				this.fromGregorian(arg0);
			}else if(arg0 == ""){
				// date should be invalid.  Dijit relies on this behavior.
				this._year = this._month = this._date = this._hours = this._minutes = this._seconds = this._milliseconds = NaN;
			}else{  // this is hebrew.Date object
				this._year = arg0._year;
				this._month =  arg0._month;
				this._date = arg0._date;
				this._hours = arg0._hours;
				this._minutes = arg0._minutes;
				this._seconds = arg0._seconds;
				this._milliseconds = arg0._milliseconds;
			}
		}else if(len >= 3){
			// YYYY, MM, DD arguments passed, month is from 0-12,  "absolute" index of month
			this._year += arguments[0];
			this._month += arguments[1];
			this._date += arguments[2];
			
			if(this._month > 12){
				console.warn("the month is incorrect , set 0  " + this._month + "   " + this._year );
				this._month = 0;
			}
			this._hours += arguments[3] || 0;
			this._minutes += arguments[4] || 0;
			this._seconds += arguments[5] || 0;
			this._milliseconds += arguments[6] || 0;
		}
  
		this._setDay();
	},
	
	getDate: function(){
		// summary: returns the date value (1 - 30)
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDate());

		return this._date; // int
	},

	getDateLocalized: function(/*String?*/locale){
		// summary: returns the date value as hebrew numerals for the Hebrew locale,
		//		a number for all others.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDate());

		return (locale || dojo.locale).match(/^he(?:-.+)?$/) ?
			numerals.getDayHebrewLetters(this._date) : this.getDate();
	},

	getMonth: function(){
		// summary: returns the month value (0 - 12)
		//
		// description: the result is the index in the month array:
		//	0. Tishri
		//	1. Heshvan
		//	2. Kislev
		//	3. Tevet
		//	4. Shevat
		//	5. Adar I (leap years only)
		//	6. Adar
		//	7. Nisan
		//	8. Iyar
		//	9. Sivan
		//	10. Tammuz
		//	11.	Av
		//	12. Elul - 12
		// For non leap years, for months after Shevat, the actual position of
		// the month in the year (used for short format) is less than
		// the "absolute" index by 1.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |
		// |		console.log(date1.getMonth()+1);
		// |		>> 7

		return this._month;
	},

	getFullYear: function(){
		// summary: returns the Year value
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |
		// |		console.log(date1.getFullYear());
		// |		>> 5769
		return this._year;
	},
			
	getHours: function(){
 		//summary: returns the hour value
		return this._hours;
	},
		
	getMinutes: function(){
		//summary: returns the minutes value

		return this._minutes;
	},

	getSeconds: function(){
		//summary: returns the seconds value
		return this._seconds;
	},

	getMilliseconds: function(){
		//summary: returns the milliseconds value

		return this._milliseconds;
	},

	setDate: function(/*number*/date){
		// summary: sets the date number for a given month
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |		date1.setDate(2);

		date = +date;
		var mdays;
		if(date>0){
			while (date > (mdays = this.getDaysInHebrewMonth(this._month, this._year))){
				date -= mdays;
				this._month++;
				if(this._month >= 13){this._year++; this._month -= 13;}
			}
		}else{
			while(date<=0){
				mdays = this.getDaysInHebrewMonth((this._month-1)>=0 ? (this._month-1) : 12, ((this._month-1)>=0)? this._year : this._year-1);
				this._month--;
				if(this._month < 0){this._year--; this._month += 13;}
				date += mdays;
			}
		}
		this._date = date;
		this._setDay();
		return this;
	},
	

	setFullYear: function(/*number*/year, /*number?*/month, /*number?*/ date){
		// summary: set the year
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setFullYear(5768);
		// |		date1.setFullYear(5768, 1, 1);
		
		this._year = year = +year;
		if(!this.isLeapYear(year) && this._month==5){  //incorrect month number for non leap year
			this._month++;
		}
		
		if(month !== undefined){this.setMonth(month);}
		if(date !== undefined){this.setDate(date);}
		
		var dnum = this.getDaysInHebrewMonth(this._month, this._year);
		if(dnum < this._date){
			this._date = dnum;
		} // if the date in this month more than number of the days in this month
		
		this._setDay();
		return this;
	},

	setMonth: function(/*number*/month){
		// summary: sets the month.  You should use "absolute" index in the month array:
		//	0. Tishri
		//	1. Heshvan
		//	2. Kislev
		//	3. Tevet
		//	4. Shevat
		//	5. Adar I (leap years only)
		//	6. Adar
		//	7. Nisan
		//	8. Iyar
		//	9. Sivan
		//	10. Tammuz
		//	11.	Av
		//	12. Elul - 12
		//  For non leap years, for months after Shevat, the actual position of
		//	the month in the year (used for short format) is less than
		//	the "absolute" index by 1.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setMonth(0); //first month

		month = +month; // coerce to a Number
		if(!this.isLeapYear(this._year) && month == 5){month++;}
	
		if(month>=0){
			while(month >12){
				this._year++;
				month -= 13;
				if (!this.isLeapYear(this._year) && month >= 5){month++;}
			}
		}else{
			while(month<0){
				this._year--;
				month += (!this.isLeapYear(this._year)  &&  month < -7) ? 12 : 13;
			}
		}
		
		this._month = month;

		var dnum = this.getDaysInHebrewMonth(this._month, this._year);
		if(dnum < this._date){
			this._date = dnum;
		} // if the date in this month more than number of the days in this month
		
		this._setDay();
		return this;
	},

	setHours: function(){
		//	summary: sets the hour
		//
		//	description: Sets the hour and optionally minutes, seconds, milliseconds also.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setHours(12, 30, 0, 0);

		var hours_arg_no = arguments.length;
		var hours = 0;
		if(hours_arg_no >= 1){
			hours += +arguments[0];
		}

		if(hours_arg_no >= 2){
			this._minutes += +arguments[1];
		}

		if(hours_arg_no >= 3){
			this._seconds += +arguments[2];
		}

		if(hours_arg_no == 4){
			this._milliseconds += +arguments[3];
		}

		while(hours >= 24){
			this._date++;
			var mdays = this.getDaysInHebrewMonth(this._month, this._year);
			if(this._date > mdays)
			{
				this._month++;
				if(!this.isLeapYear(this._year) && this._month==5){ this._month++; }
				if(this._month >= 13){this._year++; this._month -= 13;}
				this._date -= mdays;
			}
			hours -= 24;
		}
		this._hours = hours;
		this._setDay();
		return this;
	},

	_addMinutes: function(/*Number*/minutes){
		minutes += this._minutes;
		this.setMinutes(minutes);
		this.setHours(this._hours + parseInt(minutes / 60));
		return this;
	},

	_addSeconds: function(/*Number*/seconds){
		seconds += this._seconds;
		this.setSeconds(seconds);
		this._addMinutes(parseInt(seconds / 60));
		return this;
	},

	_addMilliseconds: function(/*Number*/milliseconds){
		milliseconds += this._milliseconds;
		this.setMilliseconds(milliseconds);
		this._addSeconds(parseInt(milliseconds / 1000));
		return this;
	},

	setMinutes: function(/*Number*/minutes){
		//summary: sets the minutes (0-59) only.
		this._minutes = minutes % 60;
		return this;
	},

	setSeconds: function(/*Number*/seconds){
		//summary: sets the seconds (0-59) only.
		this._seconds = seconds % 60;
		return this;
	},

	setMilliseconds: function(/*Number*/milliseconds){
		this._milliseconds = milliseconds % 1000;
		return this;
	},

	_setDay: function(){
		var day = this._startOfYear(this._year);
		if(this._month != 0){
			day += (this.isLeapYear(this._year) ? this._LEAP_MONTH_START : this._MONTH_START)[this._month || 0][this._yearType(this._year)];
		}
		day += this._date - 1;
		this._day = (day+1) % 7;
	},

	toString: function(){
		// summary: returns a string representation of the date in "dd, MM, yyyy HH:mm:ss" format
		//
		// description: returns a string representation of the date in "dd, MM, yyyy HH:mm:ss" format (all numeric)
		//	For user presentation, use dojox.date.hebrew.locale.format which will present in the appropriate language
		//  and format.  toString() language- and culturally-specific conventions to keep this module free of
		//	dependencies on dojox.date.locale and dojo.cldr.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |		console.log(date1.toString());
		// |		>>> "1, 6, 5769 0:0:0"
		return this._date + ", " + this._month + ", " + this._year + "  " + this._hours + ":" + this._minutes + ":" + this._seconds; // String
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	getDaysInHebrewMonth: function(/*Number*/month, /*Number*/ year){
		// summary: returns the number of days in the given month and year

		// Aside from the leap month, these two months can vary: 1=HESHVAN, 2=KISLEV
		// The rest are a fixed length
		var yearType = (month == 1 || month == 2) ? this._yearType(year) : 0;
		return (!this.isLeapYear(this._year) && month == 5) ? 0 : this._MONTH_LENGTH[month][yearType];
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_yearType: function(/*Number*/year){
		var yearLength = this._handleGetYearLength(Number(year));
		if(yearLength > 380){
			yearLength -= 30;        // Subtract length of leap month.
		}

		var yearType = yearLength - 353;
		if (yearType < 0 || yearType > 2){
			throw new Error("Illegal year length " + yearLength + " in year " + year);
		}
		return yearType;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_handleGetYearLength: function(/*number*/eyear){
		return this._startOfYear(eyear+1) - this._startOfYear(eyear);
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_startOfYear: function(/*number*/year){
		var months = Math.floor((235 * year - 234) / 19),	// # of months before year
			frac = months * (12*1080 + 793) + 11*1080 + 204/*BAHARAD*/,	// Fractional part of day #
			day  = months * 29 + Math.floor(frac / (24*1080));	// Whole # part of calculation
		frac %= 24*1080;	// Time of day

		var wd = day % 7;	// Day of week (0 == Monday)

		if(wd == 2 || wd == 4 || wd == 6){
			// If the 1st is on Sun, Wed, or Fri, postpone to the next day
			day += 1;
			wd = day % 7;
		}
		if(wd == 1 && frac > 15 * 1080 + 204 && !this.isLeapYear(year)){
			// If the new moon falls after 3:11:20am (15h204p from the previous noon)
			// on a Tuesday and it is not a leap year, postpone by 2 days.
			// This prevents 356-day years.
			day += 2;
		}else if(wd == 0 && frac > 21 * 1080 + 589 && this.isLeapYear(year-1)){
			// If the new moon falls after 9:32:43 1/3am (21h589p from yesterday noon)
			// on a Monday and *last* year was a leap year, postpone by 1 day.
			// Prevents 382-day years.
			day += 1;
		}

		return day;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	isLeapYear: function(/*Number*/year){
	//	summary:
	//		Determines if the year (argument) is a leap year
	//	description: The Leap year contains additional month adar sheni
	//
		//return (year * 12 + 17) % 19 >= 12;
		var x = (year*12 + 17) % 19;
		return x >= ((x < 0) ? -7 : 12);
	},

	
	fromGregorian: function(/*Date*/gdate){
		// summary: This function sets this Date to the Hebrew Date corresponding to the Gregorian Date
		// example:
		// |		var dateHebrew = new dojox.date.hebrew.Date();
		// |		var dateGregorian = new Date(2008,10,12);
		// |		dateHebrew.fromGregorian(dateGregorian);

		var result = (!isNaN(gdate)) ? this._computeHebrewFields(gdate) : NaN;
		this._year = (!isNaN(gdate)) ? result[0] : NaN;
		this._month = (!isNaN(gdate))? result[1] : NaN;
		this._date = (!isNaN(gdate)) ? result[2] : NaN;
		this._hours = gdate.getHours();
		this._milliseconds = gdate.getMilliseconds();
		this._minutes = gdate.getMinutes();
		this._seconds = gdate.getSeconds();
		if (!isNaN(gdate)) this._setDay();
		return this;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar.handleComputeFields from ICU4J v3.6.1 at http://www.icu-project.org/
	_computeHebrewFields: function(/*Date*/gdate){
		var julianDay = this._getJulianDayFromGregorianDate(gdate),
			d = julianDay - 347997,
			m = Math.floor((d * 24*1080) / (29*24*1080 + 12*1080 + 793)),       // Months (approx)
			year = Math.floor((19 * m + 234) / 235) + 1,  // Years (approx)
			ys  = this._startOfYear(year),                 // 1st day of year
			dayOfYear = (d - ys);
		// Because of the postponement rules, it's possible to guess wrong.  Fix it.
		while(dayOfYear < 1){
			year--;
			ys  = this._startOfYear(year);
			dayOfYear = d - ys;
		}

		// Now figure out which month we're in, and the date within that month

		var typeofYear = this._yearType(year),
			monthStart = this.isLeapYear(year) ? this._LEAP_MONTH_START : this._MONTH_START,
			month = 0;

		while(dayOfYear > monthStart[month][typeofYear]){
			month++;
		}
		month--;
		var dayOfMonth = dayOfYear - monthStart[month][typeofYear];
		return [year, month, dayOfMonth];
	},

	// ported from the Java class com.ibm.icu.util.Calendar.computeGregorianFields from ICU4J v3.6.1 at http://www.icu-project.org/
	toGregorian: function(){
		// summary: returns the equivalent Grogorian date value as a native Date object
		// example:
		// |		var dateHebrew = new dojox.date.hebrew.Date(5768,11,20);
		// |		var dateGregorian = dateHebrew.toGregorian();
		
		var hYear = this._year || 0,
			hMonth = this._month || 0,
			hDate = this._date || 0,
			day = this._startOfYear(hYear);

		if(hMonth != 0){
			day += (this.isLeapYear(hYear) ? this._LEAP_MONTH_START : this._MONTH_START)[hMonth][this._yearType(hYear)];
		}

		var julianDay =  (hDate + day + 347997),
		// The Gregorian epoch day is zero for Monday January 1, year 1.
			gregorianEpochDay = julianDay - 1721426;

		// Here we convert from the day number to the multiple radix
		// representation.  We use 400-year, 100-year, and 4-year cycles.
		// For example, the 4-year cycle has 4 years + 1 leap day; giving
		// 1461 == 365*4 + 1 days.
		var rem = [];
		var n400 = this._floorDivide(gregorianEpochDay , 146097, rem), // 400-year cycle length
			n100 = this._floorDivide(rem[0] , 36524, rem), // 100-year cycle length
			n4 = this._floorDivide(rem[0] , 1461, rem), // 4-year cycle length
			n1 = this._floorDivide(rem[0] , 365, rem),
			year = 400*n400 + 100*n100 + 4*n4 + n1,
			dayOfYear = rem[0]; // zero-based day of year

		if(n100 == 4 || n1 == 4){
			dayOfYear = 365; // Dec 31 at end of 4- or 400-yr cycle
		}else{
			++year;
		}

		var isLeap = !(year%4) && // equiv. to (year%4 == 0)
				(year%100 || !(year%400)),
			correction = 0,
			march1 = isLeap ? 60 : 59; // zero-based DOY for March 1
		if(dayOfYear >= march1){ correction = isLeap ? 1 : 2; }
		var month = Math.floor((12 * (dayOfYear + correction) + 6) / 367); // zero-based month
		var dayOfMonth = dayOfYear -
				this._GREGORIAN_MONTH_COUNT[month][isLeap ? 3 : 2] + 1; // one-based DOM

		return new Date(year, month, dayOfMonth, this._hours, this._minutes, this._seconds, this._milliseconds); // Date
	},
	_floorDivide: function(numerator, denominator, remainder){
		if(numerator >= 0){
			remainder[0] = (numerator % denominator);
			return Math.floor(numerator / denominator);
		}
		var quotient = Math.floor(numerator / denominator);
		remainder[0] = numerator - (quotient * denominator);
		return quotient;
	},

	getDay: function(){
		// summary: returns weekday value (0 - 6)
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDay());

		var hYear = this._year,
			hMonth = this._month,
			hDate = this._date,
			day = this._startOfYear(hYear);

		if(hMonth != 0){
			day += (this.isLeapYear(hYear) ? this._LEAP_MONTH_START : this._MONTH_START)[hMonth][this._yearType(hYear)];
		}

		day += hDate - 1;
		return (day+1) % 7;
	},

	// ported from the Java class com.ibm.icu.util.Calendar.computeGregorianMonthStart from ICU4J v3.6.1 at http://www.icu-project.org/
	_getJulianDayFromGregorianDate: function(gdate){
		//summary: returns the Julian day of a Gregorian date

		var year = gdate.getFullYear(),
			month = gdate.getMonth(),
			d = gdate.getDate(),
			isLeap = !(year%4) && (year%100 || !(year%400)), //TODO: dup
			y = year - 1;
		// This computation is actually ... + (_JAN_1_1_JULIAN_DAY - 3) + 2.
		// Add 2 because Gregorian calendar starts 2 days after Julian
		// calendar.
		var julianDay = 365*y + Math.floor(y/4) - Math.floor(y/100) +
			Math.floor(y/400) + 1721426 - 1;
		// At this point julianDay indicates the day BEFORE the first day
		// of January 1, <eyear> of the Gregorian calendar.
		if(month > 0) {
			julianDay += this._GREGORIAN_MONTH_COUNT[month][isLeap ? 3 : 2];
		}
		
		julianDay += d;
		return julianDay;
	}
});

dojox.date.hebrew.Date.prototype.valueOf = function(){
	return this.toGregorian().valueOf();
};
return dojox.date.hebrew.Date;
});

},
'dojo/ready':function(){
define(["./_base/kernel", "./has", "require", "./domReady", "./_base/lang"], function(dojo, has, require, domReady, lang) {
	// module:
	//		dojo/ready
	// summary:
	//		This module defines the dojo.ready API.
	//
	// note:
	//		This module should be unnecessary in dojo 2.0
	var
		// truthy if DOMContentLoaded or better (e.g., window.onload fired) has been achieved
		isDomReady = 0,

		// a function to call to cause onLoad to be called when all requested modules have been loaded
		requestCompleteSignal,

		// The queue of functions waiting to execute as soon as dojo.ready conditions satisfied
		loadQ = [],

		// prevent recursion in onLoad
		onLoadRecursiveGuard = 0,

		handleDomReady = function(){
			isDomReady = 1;
			dojo._postLoad = dojo.config.afterOnLoad = true;
			if(loadQ.length){
				requestCompleteSignal(onLoad);
			}
		},

		// run the next function queued with dojo.ready
		onLoad = function(){
			if(isDomReady && !onLoadRecursiveGuard && loadQ.length){
				//guard against recursions into this function
				onLoadRecursiveGuard = 1;
				var f = loadQ.shift();
					try{
						f();
					}
						// FIXME: signal the error via require.on
					finally{
						onLoadRecursiveGuard = 0;
					}
				onLoadRecursiveGuard = 0;
				if(loadQ.length){
					requestCompleteSignal(onLoad);
				}
			}
		};

	// define requireCompleteSignal; impl depends on loader
	if(1){
		require.on("idle", onLoad);
		requestCompleteSignal = function(){
			if(require.idle()){
				onLoad();
			} // else do nothing, onLoad will be called with the next idle signal
		};
	}else{
		// RequireJS or similar
		requestCompleteSignal = function(){
			// the next function call will fail if you don't have a loader with require.ready
			// in that case, either fix your loader, use dojo's loader, or don't call dojo.ready;
			require.ready(onLoad);
		};
	}

	var ready = dojo.ready = dojo.addOnLoad = function(priority, context, callback){
		// summary: Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.
		// priority: Integer?
		//		The order in which to exec this callback relative to other callbacks, defaults to 1000
		// context: Object?|Function
		//		The context in which to run execute callback, or a callback if not using context
		// callback: Function?
		//		The function to execute.
		//
		// example:
		//	Simple DOM and Modules ready syntax
		//	|	dojo.ready(function(){ alert("Dom ready!"); });
		//
		// example:
		//	Using a priority
		//	|	dojo.ready(2, function(){ alert("low priority ready!"); })
		//
		// example:
		//	Using context
		//	|	dojo.ready(foo, function(){
		//	|		// in here, this == foo
		//	|	})
		//
		// example:
		//	Using dojo.hitch style args:
		//	|	var foo = { dojoReady: function(){ console.warn(this, "dojo dom and modules ready."); } };
		//	|	dojo.ready(foo, "dojoReady");

		var hitchArgs = lang._toArray(arguments);
		if(typeof priority != "number"){
			callback = context;
			context = priority;
			priority = 1000;
		}else{
			hitchArgs.shift();
		}
		callback = callback ?
			lang.hitch.apply(dojo, hitchArgs) :
			function(){
				context();
			};
		callback.priority = priority;
		for(var i = 0; i < loadQ.length && priority >= loadQ[i].priority; i++){}
		loadQ.splice(i, 0, callback);
		requestCompleteSignal();
	};

	true || has.add("dojo-config-addOnLoad", 1);
	if(1){
		var dca = dojo.config.addOnLoad;
		if(dca){
			ready[(lang.isArray(dca) ? "apply" : "call")](dojo, dca);
		}
	}

	if(1 && dojo.config.parseOnLoad && !dojo.isAsync){
		ready(99, function(){
			if(!dojo.parser){
				dojo.deprecated("Add explicit require(['dojo/parser']);", "", "2.0");
				require(["dojo/parser"]);
			}
		});
	}

	if(1){
		domReady(handleDomReady);
	}else{
		handleDomReady();
	}

	return ready;
});

},
'dijit/_TemplatedMixin':function(){
define([
	"dojo/_base/lang", // lang.getObject
	"dojo/touch",
	"./_WidgetBase",
	"dojo/string", // string.substitute string.trim
	"dojo/cache",	// dojo.cache
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.destroy, domConstruct.toDom
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window" // win.doc
], function(lang, touch, _WidgetBase, string, cache, array, declare, domConstruct, has, unload, win) {

/*=====
	var _WidgetBase = dijit._WidgetBase;
=====*/

	// module:
	//		dijit/_TemplatedMixin
	// summary:
	//		Mixin for widgets that are instantiated from a template

	var _TemplatedMixin = declare("dijit._TemplatedMixin", null, {
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template.
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with require([... "dojo/text!..."], ...) instead
		templatePath: null,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template nodes poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with data-dojo-attach-point=... in the
		//		template, ex: ["containerNode", "labelNode"]
 		_attachPoints: [],
 =====*/

/*=====
		// _attachEvents: [private] Handle[]
		//		List of connections associated with data-dojo-attach-event=... in the
		//		template
 		_attachEvents: [],
 =====*/

		constructor: function(){
			this._attachPoints = [];
			this._attachEvents = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = lang.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			if(!this.templateString){
				this.templateString = cache(this.templatePath, {sanitize: true});
			}

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = _TemplatedMixin.getCachedTemplate(this.templateString, this._skipNodeCache);

			var node;
			if(lang.isString(cached)){
				node = domConstruct.toDom(this._stringRepl(cached));
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// Call down to _Widget.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to _setBaseClassAttr
			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node, function(n,p){ return n.getAttribute(p); });

			this._beforeFillContent();		// hook for _WidgetsInTemplateMixin

			this._fillContent(this.srcNodeRef);
		},

		_beforeFillContent: function(){
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			//		Alternately, if rootNode is an array of widgets, then will process data-dojo-attach-point
			//		etc. for those widgets.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint/data-dojo-attach-point
			//			* dojoAttachEvent/data-dojo-attach-event
			// rootNode: DomNode|Widget[]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			var nodes = lang.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = lang.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && (getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type"))){
					continue;
				}
				// Process data-dojo-attach-point
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(lang.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process data-dojo-attach-event
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = lang.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							// Map "press", "move" and "release" to keys.touch, keys.move, keys.release
							this._attachEvents.push(this.connect(baseNode, touch[event] || event, thisFunc));
						}
					}
				}
			}
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			array.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			// And same for event handlers
			array.forEach(this._attachEvents, this.disconnect, this);
			this._attachEvents = [];

			this.inherited(arguments);
		}
	});

	// key is templateString; object is either string or DOM tree
	_TemplatedMixin._templateCache = {};

	_TemplatedMixin.getCachedTemplate = function(templateString, alwaysUseString){
		// summary:
		//		Static method to get a template based on the templatePath or
		//		templateString key
		// templateString: String
		//		The template
		// alwaysUseString: Boolean
		//		Don't cache the DOM tree for this template, even if it doesn't have any variables
		// returns: Mixed
		//		Either string (if there are ${} variables that need to be replaced) or just
		//		a DOM tree (if the node can be cloned directly)

		// is it already cached?
		var tmplts = _TemplatedMixin._templateCache;
		var key = templateString;
		var cached = tmplts[key];
		if(cached){
			try{
				// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the current document, then use the current cached value
				if(!cached.ownerDocument || cached.ownerDocument == win.doc){
					// string or node of the same document
					return cached;
				}
			}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
			domConstruct.destroy(cached);
		}

		templateString = string.trim(templateString);

		if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
			// there are variables in the template so all we can do is cache the string
			return (tmplts[key] = templateString); //String
		}else{
			// there are no variables in the template so we can cache the DOM tree
			var node = domConstruct.toDom(templateString);
			if(node.nodeType != 1){
				throw new Error("Invalid template: " + templateString);
			}
			return (tmplts[key] = node); //Node
		}
	};

	if(has("ie")){
		unload.addOnWindowUnload(function(){
			var cache = _TemplatedMixin._templateCache;
			for(var key in cache){
				var value = cache[key];
				if(typeof value == "object"){ // value is either a string or a DOM node template
					domConstruct.destroy(value);
				}
				delete cache[key];
			}
		});
	}

	// These arguments can be specified for widgets which are used in templates.
	// Since any widget can be specified as sub widgets in template, mix it
	// into the base widget class.  (This is a hack, but it's effective.)
	lang.extend(_WidgetBase,{
		dojoAttachEvent: "",
		dojoAttachPoint: ""
	});

	return _TemplatedMixin;
});

},
'com.ibm.bpm.coach/coachViewHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// utilities around CoachView

define(["dojo/dom-class", "./CONSTANTS", "./domHelper"], function(domClass, CONSTANTS, domHelper) {
				
	var coachViewHelper = new Object();

	coachViewHelper.getViewIdMetaData = function(viewScope) {
		if(!!viewScope) {
			var metaData = new Object();
			var botomUpViewLevels = new Array();
			var currentView = viewScope;
			while(!!currentView) {
				var viewId = currentView.context.viewid;
				var parentView = currentView.context.parentView();
				if(!parentView) {
					// currentView is already topLevelView
					metaData[CONSTANTS.TOPLEVEL_VIEW_ID] = viewId;
				} else {
					var oneLevel = new Object();
					oneLevel[CONSTANTS.SUBVIEW_ID] = viewId;
					var siblings = parentView.context.getSubview(viewId, true);
					for(var k=0; k<siblings.length; k++) {
						if(siblings[k] === currentView) {
							oneLevel[CONSTANTS.SUBVIEW_INDEX] = k;
							break;
						}
					}
					botomUpViewLevels.push(oneLevel);					
				}
				currentView = parentView;
			}
			if(botomUpViewLevels.length > 0) {
				metaData[CONSTANTS.VIEW_PATH] = botomUpViewLevels.reverse();
			}
			
			return metaData;
		}
		
		return null;
	}
	
	coachViewHelper.hasSameStructure = function(viewIdMetaData1, viewIdMetaData2) {
		if(!!viewIdMetaData1 && !!viewIdMetaData2) {
			if(viewIdMetaData1[CONSTANTS.TOPLEVEL_VIEW_ID] === viewIdMetaData2[CONSTANTS.TOPLEVEL_VIEW_ID]
				&& !!viewIdMetaData1[CONSTANTS.VIEW_PATH] && !!viewIdMetaData2[CONSTANTS.VIEW_PATH]
				&& viewIdMetaData1[CONSTANTS.VIEW_PATH].length == viewIdMetaData2[CONSTANTS.VIEW_PATH].length) {
				
				for(var k=0; k<viewIdMetaData1[CONSTANTS.VIEW_PATH].length; k++) {
					if(viewIdMetaData1[CONSTANTS.VIEW_PATH][k][CONSTANTS.SUBVIEW_ID] != viewIdMetaData2[CONSTANTS.VIEW_PATH][k][CONSTANTS.SUBVIEW_ID]) {
						return false;
					}
				}
				// now we know the structures are same, although CONSTANTS.SUBVIEW_INDEX might be different 
				return true;
			}			
		}
		
		return false;
	}
	
	
	coachViewHelper.findViewByLocation = function(viewIdMetaData, topLevelViewScope) {
		var topLevelViewId = viewIdMetaData[CONSTANTS.TOPLEVEL_VIEW_ID];
		var topLevelView = topLevelViewScope[topLevelViewId];
		var viewPath = viewIdMetaData[CONSTANTS.VIEW_PATH];
		if(!!viewPath) {
			var currentView = topLevelView;
			for(var k=0; k<viewPath.length; k++) {
				var oneLevel = viewPath[k];
				var subviews = currentView.context.getSubview(oneLevel[CONSTANTS.SUBVIEW_ID], true);
				var index = oneLevel[CONSTANTS.SUBVIEW_INDEX];
				if(subviews.length <= index) {
					// something wrong. no way to continue
					return null;
				} else {
					currentView = subviews[index];
				}
			}
			return currentView;
		} else {
			// topLevelView is what we are looking for
			return topLevelView;
		}
	}
	
	
	coachViewHelper.findViewScope = function(domId, topLevelViewScope) {

		var domNode = document.getElementById(domId);
		
		var nodeChain = new Array();
		var classAttribute;
		while(!!domNode && domNode.tagName != "BODY") {
			if(domClass.contains(domNode, CONSTANTS.TOP_LEVEL)) {
				// now domNode is the topLevel view
				var topLevelViewId = domNode.getAttribute(CONSTANTS.VIEW_ID);
				var topViewScope = topLevelViewScope[topLevelViewId];			
				return coachViewHelper._findViewScopeByChain(topViewScope, nodeChain);
			}
			nodeChain[nodeChain.length] = domNode;
			domNode = domNode.parentNode;
		}
		
		return null;	
	}

	// find the direct parent view for the viewDivNode
	coachViewHelper.findParentViewScope = function(rootViewScope, viewDivNode) {
		// we intend not to have robust checking on the while loops in ths method. If exception occurs, we will find some scenarios that we haven't thought of. 
		
		var parentViewScopeNode = null;
		var rootDivNode = rootViewScope.context.element;
		var parentNode = viewDivNode.parentNode;
		while(!parentViewScopeNode) {
			if(parentNode == rootDivNode) {
				// well, rootViewScope is the direct owner
				return rootViewScope;
			}
			if(domClass.contains(parentNode, CONSTANTS.COACH_VIEW)) {
				parentViewScopeNode = parentNode;
				break;
			}
			parentNode = parentNode.parentNode;
		}
		
		// try to find the ViewScope object of parentViewScopeNode with the help of rootViewScope		
		var nodeChain = new Array();
		nodeChain[0] = parentViewScopeNode;
		parentNode = parentViewScopeNode.parentNode;
		while(parentNode != rootDivNode) {
			nodeChain[nodeChain.length] = parentNode;
			parentNode = parentNode.parentNode;			
		}
		
		return coachViewHelper._findViewScopeByChain(rootViewScope, nodeChain);
	}

	coachViewHelper._findViewScopeByChain = function(rootViewScope, nodeChain) {
		// we intend not to have robust checking on the for loop in ths method. If exception occurs, we will find some scenarios that we haven't thought of. 

		if(!rootViewScope || !nodeChain) {
			return null;
		}
		
		var tempViewScope = rootViewScope;
		for(var k=nodeChain.length -1; k>=0; k--) {
			var viewId = nodeChain[k].getAttribute(CONSTANTS.VIEW_ID);
			var divId = nodeChain[k].getAttribute("id");
			if(!!viewId && !!divId) {
				if(!tempViewScope.context.subview[viewId]) {
					// console.debug("*****ERROR: Developer, viewId [" + viewId + "] may not the right one");
					return null;
				} else if(!tempViewScope.context.subview[viewId][divId]) {
					// console.debug("*****ERROR: Developer, divId [" + divId + "] may not the right one");
					return null;
				} 
				tempViewScope = tempViewScope.context.subview[viewId][divId];
				if(!tempViewScope) {
					return null;
				}
			}
		}
		return tempViewScope;
	}
	
	coachViewHelper.findAllViewInstances = function(topLevelViews, viewInstancePath) {
		var viewInstances = [];
		if(!!viewInstancePath && viewInstancePath.length > 0) {
			var topLevelView = topLevelViews[viewInstancePath[0]];
			coachViewHelper._collectViewInstances(viewInstances, topLevelView, viewInstancePath, 1);
		}
		return viewInstances;
	}
	
	coachViewHelper._collectViewInstances = function(viewInstances, currentViewInstance, viewInstancePath, index) {
		if(!currentViewInstance) {
			return;
		}
		
		if(index >= viewInstancePath.length) {
			// finish all path
			viewInstances.push(currentViewInstance);
		} else {
			var viewsWithSameViewId = currentViewInstance.context.getSubview(viewInstancePath[index]);
			var newIndex = index + 1;
			for(var k=0; k<viewsWithSameViewId.length; k++) {
				coachViewHelper._collectViewInstances(viewInstances, viewsWithSameViewId[k], viewInstancePath, newIndex);
			}
		}
	}
	
	/*	
		This function returns the coach view instance that is described by the input parameter "viewInstancePath" which is a list of control ids.   
		The list of control ids should completely specify the view hierarchy, starting from the top level view, 
		all the way down to the coach view whose instance is expected to be returned.
		
		If  the list of control id doesn't describe the entire view hierarchy properly, this function returns null.			
		If a coach view repeats, first instance of that view is chose to continue.
	*/
	coachViewHelper.findFirstViewInstance = function(topLevelViews, viewInstancePath) {
		try {
			if(!!topLevelViews && !!viewInstancePath && viewInstancePath.length > 0) {
				var topLevelView = topLevelViews[viewInstancePath[0]];
				return coachViewHelper._findFirstViewInstance(topLevelView, viewInstancePath, 1);
			}
		} catch(ex) {
			// need to do nothing
		}
		
		return null;
	}
	
	coachViewHelper._findFirstViewInstance = function(currentViewInstance, viewInstancePath, index) {
		if(!currentViewInstance) {
			return null;
		}
		
		if(index >= viewInstancePath.length) {
			// finish all path
			return currentViewInstance;
		} else {
			var viewsWithSameViewId = currentViewInstance.context.getSubview(viewInstancePath[index]);
			if(viewsWithSameViewId.length > 0) {
				var newIndex = index + 1;
				return coachViewHelper._findFirstViewInstance(viewsWithSameViewId[0], viewInstancePath, newIndex);
			} else {
				// no subview
				return null;
			}
		}
	}
	
	coachViewHelper.getViewPath = function(viewInstance) {
		var path = [];
		var aView = viewInstance;
		while(!!aView) {
			path.push(aView.context.viewid);
			aView = aView.context.parentView();
		}
		return path.reverse();
	}
	
	// Find the subviews of specified ownerView, those subviews should reside in specified contentBox (contentBoxDomNode) 
	// directly or indirectly (e.g. subview is in another contentBox which is in specified contentBox)
	// As the return, subviews are put in the provided subviewArray
	coachViewHelper.findSubviews = function(subviewArray, ownerView, contentBoxDomNode) {
		// handle nested Views
		var immediateViewList = domHelper.shallowQueryByClass(contentBoxDomNode, CONSTANTS.COACH_VIEW, false);
		var viewId, domId;
		for(var i=0; i<immediateViewList.length; i++) {
			var domId = immediateViewList[i].getAttribute("id");
			var viewId = immediateViewList[i].getAttribute(CONSTANTS.VIEW_ID);
			if(ownerView.context.subview[viewId] && ownerView.context.subview[viewId][domId]) {
				subviewArray.push(ownerView.context.subview[viewId][domId]);
			}
		}
		
		// handle nested ContentBox
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDomNode, CONSTANTS.CONTENT_BOX, false);
		for(var i=0; i<immediateContentBoxList.length; i++) {
			coachViewHelper.findSubviews(subviewArray, ownerView, immediateContentBoxList[i]);
		}			
	}	
	
	return coachViewHelper;
});

},
'dojo/Evented':function(){
define(["./aspect", "./on"], function(aspect, on){
	// summary:
	//		The export of this module is a class that can be used as a mixin or base class, 
	// 		to add on() and emit() methods to a class
	// 		for listening for events and emiting events:
	// 		|define(["dojo/Evented"], function(Evented){
	// 		|	var EventedWidget = dojo.declare([Evented, dijit._Widget], {...});
	//		|	widget = new EventedWidget();
	//		|	widget.on("open", function(event){
	//		|	... do something with event
	//		|	 });
	//		|
	//		|	widget.emit("open", {name:"some event", ...});

 	"use strict";
 	var after = aspect.after;
	function Evented(){
	}
	Evented.prototype = {
		on: function(type, listener){
			return on.parse(this, type, listener, function(target, type){
				return after(target, 'on' + type, listener, true);
			});
		},
		emit: function(type, event){
			var args = [this];
			args.push.apply(args, arguments);
			return on.emit.apply(on, args);
		}
	};
	return Evented;
});

},
'dojo/window':function(){
define(["./_base/lang", "./_base/sniff", "./_base/window", "./dom", "./dom-geometry", "./dom-style", "./dom-construct"],
	function(lang, has, baseWindow, dom, geom, style, domConstruct) {

	// feature detection
	/* not needed but included here for future reference
	has.add("rtl-innerVerticalScrollBar-on-left", function(win, doc){
		var	body = baseWindow.body(doc),
			scrollable = domConstruct.create('div', {
				style: {overflow:'scroll', overflowX:'hidden', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', width:'64px', height:'64px'}
			}, body, "last"),
			center = domConstruct.create('center', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last"),
			inner = domConstruct.create('div', {
				style: {overflow:'visible', display:'inline' }
			}, center, "last");
		inner.innerHTML="&nbsp;";
		var midPoint = Math.max(inner.offsetLeft, geom.position(inner).x);
		var ret = midPoint >= 32;
		center.removeChild(inner);
		scrollable.removeChild(center);
		body.removeChild(scrollable);
		return ret;
	});
	*/
	has.add("rtl-adjust-position-for-verticalScrollBar", function(win, doc){
		var	body = baseWindow.body(doc),
			scrollable = domConstruct.create('div', {
				style: {overflow:'scroll', overflowX:'visible', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', top:'0', width:'64px', height:'64px'}
			}, body, "last"),
			div = domConstruct.create('div', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last"),
			ret = geom.position(div).x != 0;
		scrollable.removeChild(div);
		body.removeChild(scrollable);
		return ret;
	});

	has.add("position-fixed-support", function(win, doc){
		// IE6, IE7+quirks, and some older mobile browsers don't support position:fixed
		var	body = baseWindow.body(doc),
			outer = domConstruct.create('span', {
				style: {visibility:'hidden', position:'fixed', left:'1px', top:'1px'}
			}, body, "last"),
			inner = domConstruct.create('span', {
				style: {position:'fixed', left:'0', top:'0'}
			}, outer, "last"),
			ret = geom.position(inner).x != geom.position(outer).x;
		outer.removeChild(inner);
		body.removeChild(outer);
		return ret;
	});

// module:
//		dojo/window
// summary:
//		TODOC

var window = lang.getObject("dojo.window", true);

/*=====
dojo.window = {
	// summary:
	//		TODO
};
window = dojo.window;
=====*/

window.getBox = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var
		scrollRoot = (baseWindow.doc.compatMode == 'BackCompat') ? baseWindow.body() : baseWindow.doc.documentElement,
		// get scroll position
		scroll = geom.docScroll(), // scrollRoot.scrollTop/Left should work
		w, h;

	if(has("touch")){ // if(scrollbars not supported)
		var uiWindow = baseWindow.doc.parentWindow || baseWindow.doc.defaultView;   // use UI window, not dojo.global window. baseWindow.doc.parentWindow probably not needed since it's not defined for webkit
		// on mobile, scrollRoot.clientHeight <= uiWindow.innerHeight <= scrollRoot.offsetHeight, return uiWindow.innerHeight
		w = uiWindow.innerWidth || scrollRoot.clientWidth; // || scrollRoot.clientXXX probably never evaluated
		h = uiWindow.innerHeight || scrollRoot.clientHeight;
	}else{
		// on desktops, scrollRoot.clientHeight <= scrollRoot.offsetHeight <= uiWindow.innerHeight, return scrollRoot.clientHeight
		// uiWindow.innerWidth/Height includes the scrollbar and cannot be used
		w = scrollRoot.clientWidth;
		h = scrollRoot.clientHeight;
	}
	return {
		l: scroll.x,
		t: scroll.y,
		w: w,
		h: h
	};
};

window.get = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(has("ie") && window !== document.parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc.parentWindow || doc.defaultView;	//	Window
};

window.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view using minimal movement, if it is not already.

	// Don't rely on node.scrollIntoView working just because the function is there since
	// it forces the node to the page's bottom or top (and left or right in IE) without consideration for the minimal movement.
	// WebKit's node.scrollIntoViewIfNeeded doesn't work either for inner scrollbars in right-to-left mode
	// and when there's a fixed position scrollable element

	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
		node = dom.byId(node);
		var	doc = node.ownerDocument || baseWindow.doc,	// TODO: why baseWindow.doc?  Isn't node.ownerDocument always defined?
			body = baseWindow.body(doc),
			html = doc.documentElement || body.parentNode,
			isIE = has("ie"),
			isWK = has("webkit");
		// if an untested browser, then use the native method
		if(node == body || node == html){ return; }
		if(!(has("mozilla") || isIE || isWK || has("opera")) && ("scrollIntoView" in node)){
			node.scrollIntoView(false); // short-circuit to native if possible
			return;
		}
		var	backCompat = doc.compatMode == 'BackCompat',
			rootWidth = Math.min(body.clientWidth || html.clientWidth, html.clientWidth || body.clientWidth),
			rootHeight = Math.min(body.clientHeight || html.clientHeight, html.clientHeight || body.clientHeight),
			scrollRoot = (isWK || backCompat) ? body : html,
			nodePos = pos || geom.position(node),
			el = node.parentNode,
			isFixed = function(el){
				return (isIE <= 6 || (isIE == 7 && backCompat))
					? false
					: (has("position-fixed-support") && (style.get(el, 'position').toLowerCase() == "fixed"));
			};
		if(isFixed(node)){ return; } // nothing to do
		while(el){
			if(el == body){ el = scrollRoot; }
			var	elPos = geom.position(el),
				fixedPos = isFixed(el),
				rtl = style.getComputedStyle(el).direction.toLowerCase() == "rtl";

			if(el == scrollRoot){
				elPos.w = rootWidth; elPos.h = rootHeight;
				if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
				if(elPos.x < 0 || !isIE || isIE >= 9){ elPos.x = 0; } // older IE can have values > 0
				if(elPos.y < 0 || !isIE || isIE >= 9){ elPos.y = 0; }
			}else{
				var pb = geom.getPadBorderExtents(el);
				elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
				var clientSize = el.clientWidth,
					scrollBarSize = elPos.w - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					if(rtl && has("rtl-adjust-position-for-verticalScrollBar")){
						elPos.x += scrollBarSize;
					}
					elPos.w = clientSize;
				}
				clientSize = el.clientHeight;
				scrollBarSize = elPos.h - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.h = clientSize;
				}
			}
			if(fixedPos){ // bounded by viewport, not parents
				if(elPos.y < 0){
					elPos.h += elPos.y; elPos.y = 0;
				}
				if(elPos.x < 0){
					elPos.w += elPos.x; elPos.x = 0;
				}
				if(elPos.y + elPos.h > rootHeight){
					elPos.h = rootHeight - elPos.y;
				}
				if(elPos.x + elPos.w > rootWidth){
					elPos.w = rootWidth - elPos.x;
				}
			}
			// calculate overflow in all 4 directions
			var	l = nodePos.x - elPos.x, // beyond left: < 0
//						t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
				t = nodePos.y - elPos.y, // beyond top: < 0
				r = l + nodePos.w - elPos.w, // beyond right: > 0
				bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
			var s, old;
			if(r * l > 0 && (!!el.scrollLeft || el == scrollRoot || el.scrollWidth > el.offsetHeight)){
				s = Math[l < 0? "max" : "min"](l, r);
				if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9)){ s = -s; }
				old = el.scrollLeft;
				el.scrollLeft += s;
				s = el.scrollLeft - old;
				nodePos.x -= s;
			}
			if(bot * t > 0 && (!!el.scrollTop || el == scrollRoot || el.scrollHeight > el.offsetHeight)){
				s = Math.ceil(Math[t < 0? "max" : "min"](t, bot));
				old = el.scrollTop;
				el.scrollTop += s;
				s = el.scrollTop - old;
				nodePos.y -= s;
			}
			el = (el != scrollRoot) && !fixedPos && el.parentNode;
		}
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

return window;
});

},
'com.ibm.bpm.coach/coachUtil':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all common utilities

define(["dojo/_base/array", "./CONSTANTS", "./dataBindingHelper"], function(dojoArray, CONSTANTS, dataBindingHelper) {
				
	var coachUtil = new Object();

	/*
	 returns true if the array contains the item, otherwise returns false.
	 Note: update the implementation to array.indexOf() once IE8 is not supported.
	*/
	coachUtil.contains = function(array, item){
    	return dojoArray.indexOf(array, item) >= 0;
  	};

  	/*
  	 dateArray is an array of Date objects, dateInMilliseconds is the number of milliseconds since midnight Jan 1, 1970
  	 returns true if one date object in the dateArray match dateInMilliseconds, otherwise returns false.
  	*/
  	coachUtil.isDateMatched = function(dateArray, dateInMilliseconds) {
  		if(!!dateArray) {
  			for(var k=0; k<dateArray.length; k++) {
  				if(dateArray[k].getTime && dateArray[k].getTime() == dateInMilliseconds) {
  					return true;
  				}
  			}
  		}
  		return false;	
  	};
  	
  	/*
  	 returns true if one item in the list has "id" property which is equal to the specified id parameter, otherwise returns false.
  	*/
  	coachUtil.isIdMatched = function(array, id) {
  		if(!!array) {
  			for(var k=0; k<array.length; k++) {
  				if(array[k].id == id) {
  					return true;
  				}
  			}
  		}
  		return false;	
  	};
  	
  	/*
  	 bindingStr may represent many variables that are under root variable local, e.g. bindingStr is "local.contactBook.contacts[index].phones[index].number".
  	 This API tests whether some variable (represented by bindingStr and local) passes the test implemented by the provided callback.
  	 	
  	 	bindingStr: examples are "local.contactBook.owner", "local.contactBook.contacts.listSelected.phones[index].name"
  	 	local: the root Statful object that holds all human service variables used by Coach
  	 	callback: function to test for each variable
  	 	thisObject: OBject to use as this when executing callback
  	*/
  	coachUtil.some = function(bindingStr, local, callback, thisObject) {
  		if(!bindingStr) {
  			// bindingStr shouldn't be null or undefined
  			return false;
  		}
  		
		var sourcePathArray = bindingStr.split(".");
		if(sourcePathArray.length < 2 || sourcePathArray[0] != "local") {
			// bindingStr should always start with "local." and contains at least 2 paths
			return false;
		}
		
		var pathArray = [];
		var onePath;
		for(var k=0; k<sourcePathArray.length; k++) {
			onePath = sourcePathArray[k];			
			if(onePath.indexOf(CONSTANTS.INDEX_WITH_BRACKET) != -1) {
				// e.g. a[currentItem]
				var virtualPathForList = onePath.replace(CONSTANTS.INDEX_WITH_BRACKET, "");
				pathArray.push(virtualPathForList);
				pathArray.push(CONSTANTS.INDEX_WITH_BRACKET);				
			} else {			
				pathArray.push(onePath);
			}
		}
		
		// if bindingStr is "local.contactBook.contacts[index].phones[index].number", 
		// pathArray is ["local", "contactBook", "contacts", "[index]", "phones", "[index]", "number"]
		
		return coachUtil._traverse(pathArray, 1, local, callback, thisObject);
  	};
  	
	coachUtil._traverse = function(pathArray, index, dataObject, compareFunc, thisObject) {
		if(!dataObject) {
			// for example, pathArray is ["local", "contactBook", "contacts", "listSelected", "phones"]
			// if no selection on "local.contactBook.contacts" list, the dataObject for "local.contactBook.contacts.listSelected.phones" is undefined
			return false;
		} 
	
		var path = pathArray[index];
		var isLastPath = (index == pathArray.length - 1);
		
		if(path == CONSTANTS.INDEX_WITH_BRACKET) {
			// dataObject represents a com.ibm.bpm.coach.List
			if(isLastPath) {
				for(var i=0; i<dataObject.length(); i++) {
					var itemPlainValue = dataBindingHelper.getPlainValue(dataObject, i);
					if(compareFunc.call(thisObject, itemPlainValue)) {
						return true;
					}
				}
			} else {
				var newIndex = index + 1;
				for(var j=0; j<dataObject.length(); j++) {
					// newDataObject is a Stateful object or List object
					var newDataObject = dataObject.get(j);
					if(coachUtil._traverse(pathArray, newIndex, newDataObject, compareFunc, thisObject)) {
						return true;
					}
				}
			}
			return false;
		} else {
			// dataObject represents a Stateful object usually. Could be List object when path is "listSelected"
			if(isLastPath) {
				var itemPlainValue = dataBindingHelper.getPlainValue(dataObject, path);
				return compareFunc.call(thisObject, itemPlainValue);
			} else {
				// newDataObject is a Stateful object or List object
				var newDataObject = dataObject.get(path);
				return coachUtil._traverse(pathArray, index+1, newDataObject, compareFunc, thisObject);
			}
		}
	}  	
  	
	return coachUtil;
});

},
'dojo/_base/xhr':function(){
define([
	"./kernel", "./sniff", "require", "../io-query", "../dom", "../dom-form", "./Deferred", "./json", "./lang", "./array", "../on"
], function(dojo, has, require, ioq, dom, domForm, deferred, json, lang, array, on){
	//	module:
	//		dojo/_base.xhr
	// summary:
	//		This modules defines the dojo.xhr* API.

	has.add("native-xhr", function() {
		// if true, the environment has a native XHR implementation
		return typeof XMLHttpRequest !== 'undefined';
	});

	if(1 && require.getXhr){
		dojo._xhrObj = require.getXhr;
	}else if (has("native-xhr")){
		dojo._xhrObj = function(){
			// summary:
			//		does the work of portably generating a new XMLHTTPRequest object.
			try{
				return new XMLHttpRequest();
			}catch(e){
				throw new Error("XMLHTTP not available: "+e);
			}
		};
	}else{
		// PROGIDs are in order of decreasing likelihood; this will change in time.
		for(var XMLHTTP_PROGIDS = ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP', 'Msxml2.XMLHTTP.4.0'], progid, i = 0; i < 3;){
			try{
				progid = XMLHTTP_PROGIDS[i++];
				if (new ActiveXObject(progid)) {
					// this progid works; therefore, use it from now on
					break;
				}
			}catch(e){
				// squelch; we're just trying to find a good ActiveX PROGID
				// if they all fail, then progid ends up as the last attempt and that will signal the error
				// the first time the client actually tries to exec an xhr
			}
		}
		dojo._xhrObj= function() {
			return new ActiveXObject(progid);
		};
	}

	var cfg = dojo.config;

	// mix in io-query and dom-form
	dojo.objectToQuery = ioq.objectToQuery;
	dojo.queryToObject = ioq.queryToObject;
	dojo.fieldToObject = domForm.fieldToObject;
	dojo.formToObject = domForm.toObject;
	dojo.formToQuery = domForm.toQuery;
	dojo.formToJson = domForm.toJson;

	// need to block async callbacks from snatching this thread as the result
	// of an async callback might call another sync XHR, this hangs khtml forever
	// must checked by watchInFlight()

	dojo._blockAsync = false;

	// MOW: remove dojo._contentHandlers alias in 2.0
	var handlers = dojo._contentHandlers = dojo.contentHandlers = {
		// summary:
		//		A map of availble XHR transport handle types. Name matches the
		//		`handleAs` attribute passed to XHR calls.
		//
		// description:
		//		A map of availble XHR transport handle types. Name matches the
		//		`handleAs` attribute passed to XHR calls. Each contentHandler is
		//		called, passing the xhr object for manipulation. The return value
		//		from the contentHandler will be passed to the `load` or `handle`
		//		functions defined in the original xhr call.
		//
		// example:
		//		Creating a custom content-handler:
		//	|	dojo.contentHandlers.makeCaps = function(xhr){
		//	|		return xhr.responseText.toUpperCase();
		//	|	}
		//	|	// and later:
		//	|	dojo.xhrGet({
		//	|		url:"foo.txt",
		//	|		handleAs:"makeCaps",
		//	|		load: function(data){ /* data is a toUpper version of foo.txt */ }
		//	|	});

		"text": function(xhr){
			// summary: A contentHandler which simply returns the plaintext response data
			return xhr.responseText;
		},
		"json": function(xhr){
			// summary: A contentHandler which returns a JavaScript object created from the response data
			return json.fromJson(xhr.responseText || null);
		},
		"json-comment-filtered": function(xhr){
			// summary: A contentHandler which expects comment-filtered JSON.
			// description:
			//		A contentHandler which expects comment-filtered JSON.
			//		the json-comment-filtered option was implemented to prevent
			//		"JavaScript Hijacking", but it is less secure than standard JSON. Use
			//		standard JSON instead. JSON prefixing can be used to subvert hijacking.
			//
			//		Will throw a notice suggesting to use application/json mimetype, as
			//		json-commenting can introduce security issues. To decrease the chances of hijacking,
			//		use the standard `json` contentHandler, and prefix your "JSON" with: {}&&
			//
			//		use djConfig.useCommentedJson = true to turn off the notice
			if(!dojo.config.useCommentedJson){
				console.warn("Consider using the standard mimetype:application/json."
					+ " json-commenting can introduce security issues. To"
					+ " decrease the chances of hijacking, use the standard the 'json' handler and"
					+ " prefix your json with: {}&&\n"
					+ "Use djConfig.useCommentedJson=true to turn off this message.");
			}

			var value = xhr.responseText;
			var cStartIdx = value.indexOf("\/*");
			var cEndIdx = value.lastIndexOf("*\/");
			if(cStartIdx == -1 || cEndIdx == -1){
				throw new Error("JSON was not comment filtered");
			}
			return json.fromJson(value.substring(cStartIdx+2, cEndIdx));
		},
		"javascript": function(xhr){
			// summary: A contentHandler which evaluates the response data, expecting it to be valid JavaScript

			// FIXME: try Moz and IE specific eval variants?
			return dojo.eval(xhr.responseText);
		},
		"xml": function(xhr){
			// summary: A contentHandler returning an XML Document parsed from the response data
			var result = xhr.responseXML;

			if(has("ie")){
				if((!result || !result.documentElement)){
					//WARNING: this branch used by the xml handling in dojo.io.iframe,
					//so be sure to test dojo.io.iframe if making changes below.
					var ms = function(n){ return "MSXML" + n + ".DOMDocument"; };
					var dp = ["Microsoft.XMLDOM", ms(6), ms(4), ms(3), ms(2)];
					array.some(dp, function(p){
						try{
							var dom = new ActiveXObject(p);
							dom.async = false;
							dom.loadXML(xhr.responseText);
							result = dom;
						}catch(e){ return false; }
						return true;
					});
				}
		 }
			return result; // DOMDocument
		},
		"json-comment-optional": function(xhr){
			// summary: A contentHandler which checks the presence of comment-filtered JSON and
			//		alternates between the `json` and `json-comment-filtered` contentHandlers.
			if(xhr.responseText && /^[^{\[]*\/\*/.test(xhr.responseText)){
				return handlers["json-comment-filtered"](xhr);
			}else{
				return handlers["json"](xhr);
			}
		}
	};

	/*=====
	dojo.__IoArgs = function(){
		//	url: String
		//		URL to server endpoint.
		//	content: Object?
		//		Contains properties with string values. These
		//		properties will be serialized as name1=value2 and
		//		passed in the request.
		//	timeout: Integer?
		//		Milliseconds to wait for the response. If this time
		//		passes, the then error callbacks are called.
		//	form: DOMNode?
		//		DOM node for a form. Used to extract the form values
		//		and send to the server.
		//	preventCache: Boolean?
		//		Default is false. If true, then a
		//		"dojo.preventCache" parameter is sent in the request
		//		with a value that changes with each request
		//		(timestamp). Useful only with GET-type requests.
		//	handleAs: String?
		//		Acceptable values depend on the type of IO
		//		transport (see specific IO calls for more information).
		//	rawBody: String?
		//		Sets the raw body for an HTTP request. If this is used, then the content
		//		property is ignored. This is mostly useful for HTTP methods that have
		//		a body to their requests, like PUT or POST. This property can be used instead
		//		of postData and putData for dojo.rawXhrPost and dojo.rawXhrPut respectively.
		//	ioPublish: Boolean?
		//		Set this explicitly to false to prevent publishing of topics related to
		//		IO operations. Otherwise, if djConfig.ioPublish is set to true, topics
		//		will be published via dojo.publish for different phases of an IO operation.
		//		See dojo.__IoPublish for a list of topics that are published.
		//	load: Function?
		//		This function will be
		//		called on a successful HTTP response code.
		//	error: Function?
		//		This function will
		//		be called when the request fails due to a network or server error, the url
		//		is invalid, etc. It will also be called if the load or handle callback throws an
		//		exception, unless djConfig.debugAtAllCosts is true.	 This allows deployed applications
		//		to continue to run even when a logic error happens in the callback, while making
		//		it easier to troubleshoot while in debug mode.
		//	handle: Function?
		//		This function will
		//		be called at the end of every request, whether or not an error occurs.
		this.url = url;
		this.content = content;
		this.timeout = timeout;
		this.form = form;
		this.preventCache = preventCache;
		this.handleAs = handleAs;
		this.ioPublish = ioPublish;
		this.load = function(response, ioArgs){
			// ioArgs: dojo.__IoCallbackArgs
			//		Provides additional information about the request.
			// response: Object
			//		The response in the format as defined with handleAs.
		}
		this.error = function(response, ioArgs){
			// ioArgs: dojo.__IoCallbackArgs
			//		Provides additional information about the request.
			// response: Object
			//		The response in the format as defined with handleAs.
		}
		this.handle = function(loadOrError, response, ioArgs){
			// loadOrError: String
			//		Provides a string that tells you whether this function
			//		was called because of success (load) or failure (error).
			// response: Object
			//		The response in the format as defined with handleAs.
			// ioArgs: dojo.__IoCallbackArgs
			//		Provides additional information about the request.
		}
	}
	=====*/

	/*=====
	dojo.__IoCallbackArgs = function(args, xhr, url, query, handleAs, id, canDelete, json){
		//	args: Object
		//		the original object argument to the IO call.
		//	xhr: XMLHttpRequest
		//		For XMLHttpRequest calls only, the
		//		XMLHttpRequest object that was used for the
		//		request.
		//	url: String
		//		The final URL used for the call. Many times it
		//		will be different than the original args.url
		//		value.
		//	query: String
		//		For non-GET requests, the
		//		name1=value1&name2=value2 parameters sent up in
		//		the request.
		//	handleAs: String
		//		The final indicator on how the response will be
		//		handled.
		//	id: String
		//		For dojo.io.script calls only, the internal
		//		script ID used for the request.
		//	canDelete: Boolean
		//		For dojo.io.script calls only, indicates
		//		whether the script tag that represents the
		//		request can be deleted after callbacks have
		//		been called. Used internally to know when
		//		cleanup can happen on JSONP-type requests.
		//	json: Object
		//		For dojo.io.script calls only: holds the JSON
		//		response for JSONP-type requests. Used
		//		internally to hold on to the JSON responses.
		//		You should not need to access it directly --
		//		the same object should be passed to the success
		//		callbacks directly.
		this.args = args;
		this.xhr = xhr;
		this.url = url;
		this.query = query;
		this.handleAs = handleAs;
		this.id = id;
		this.canDelete = canDelete;
		this.json = json;
	}
	=====*/


	/*=====
	dojo.__IoPublish = function(){
		//	summary:
		//		This is a list of IO topics that can be published
		//		if djConfig.ioPublish is set to true. IO topics can be
		//		published for any Input/Output, network operation. So,
		//		dojo.xhr, dojo.io.script and dojo.io.iframe can all
		//		trigger these topics to be published.
		//	start: String
		//		"/dojo/io/start" is sent when there are no outstanding IO
		//		requests, and a new IO request is started. No arguments
		//		are passed with this topic.
		//	send: String
		//		"/dojo/io/send" is sent whenever a new IO request is started.
		//		It passes the dojo.Deferred for the request with the topic.
		//	load: String
		//		"/dojo/io/load" is sent whenever an IO request has loaded
		//		successfully. It passes the response and the dojo.Deferred
		//		for the request with the topic.
		//	error: String
		//		"/dojo/io/error" is sent whenever an IO request has errored.
		//		It passes the error and the dojo.Deferred
		//		for the request with the topic.
		//	done: String
		//		"/dojo/io/done" is sent whenever an IO request has completed,
		//		either by loading or by erroring. It passes the error and
		//		the dojo.Deferred for the request with the topic.
		//	stop: String
		//		"/dojo/io/stop" is sent when all outstanding IO requests have
		//		finished. No arguments are passed with this topic.
		this.start = "/dojo/io/start";
		this.send = "/dojo/io/send";
		this.load = "/dojo/io/load";
		this.error = "/dojo/io/error";
		this.done = "/dojo/io/done";
		this.stop = "/dojo/io/stop";
	}
	=====*/


	dojo._ioSetArgs = function(/*dojo.__IoArgs*/args,
			/*Function*/canceller,
			/*Function*/okHandler,
			/*Function*/errHandler){
		//	summary:
		//		sets up the Deferred and ioArgs property on the Deferred so it
		//		can be used in an io call.
		//	args:
		//		The args object passed into the public io call. Recognized properties on
		//		the args object are:
		//	canceller:
		//		The canceller function used for the Deferred object. The function
		//		will receive one argument, the Deferred object that is related to the
		//		canceller.
		//	okHandler:
		//		The first OK callback to be registered with Deferred. It has the opportunity
		//		to transform the OK response. It will receive one argument -- the Deferred
		//		object returned from this function.
		//	errHandler:
		//		The first error callback to be registered with Deferred. It has the opportunity
		//		to do cleanup on an error. It will receive two arguments: error (the
		//		Error object) and dfd, the Deferred object returned from this function.

		var ioArgs = {args: args, url: args.url};

		//Get values from form if requestd.
		var formObject = null;
		if(args.form){
			var form = dom.byId(args.form);
			//IE requires going through getAttributeNode instead of just getAttribute in some form cases,
			//so use it for all. See #2844
			var actnNode = form.getAttributeNode("action");
			ioArgs.url = ioArgs.url || (actnNode ? actnNode.value : null);
			formObject = domForm.toObject(form);
		}

		// set up the query params
		var miArgs = [{}];

		if(formObject){
			// potentially over-ride url-provided params w/ form values
			miArgs.push(formObject);
		}
		if(args.content){
			// stuff in content over-rides what's set by form
			miArgs.push(args.content);
		}
		if(args.preventCache){
			miArgs.push({"dojo.preventCache": new Date().valueOf()});
		}
		ioArgs.query = ioq.objectToQuery(lang.mixin.apply(null, miArgs));

		// .. and the real work of getting the deferred in order, etc.
		ioArgs.handleAs = args.handleAs || "text";
		var d = new deferred(canceller);
		d.addCallbacks(okHandler, function(error){
			return errHandler(error, d);
		});

		//Support specifying load, error and handle callback functions from the args.
		//For those callbacks, the "this" object will be the args object.
		//The callbacks will get the deferred result value as the
		//first argument and the ioArgs object as the second argument.
		var ld = args.load;
		if(ld && lang.isFunction(ld)){
			d.addCallback(function(value){
				return ld.call(args, value, ioArgs);
			});
		}
		var err = args.error;
		if(err && lang.isFunction(err)){
			d.addErrback(function(value){
				return err.call(args, value, ioArgs);
			});
		}
		var handle = args.handle;
		if(handle && lang.isFunction(handle)){
			d.addBoth(function(value){
				return handle.call(args, value, ioArgs);
			});
		}

		//Plug in topic publishing, if dojo.publish is loaded.
		if(cfg.ioPublish && dojo.publish && ioArgs.args.ioPublish !== false){
			d.addCallbacks(
				function(res){
					dojo.publish("/dojo/io/load", [d, res]);
					return res;
				},
				function(res){
					dojo.publish("/dojo/io/error", [d, res]);
					return res;
				}
			);
			d.addBoth(function(res){
				dojo.publish("/dojo/io/done", [d, res]);
				return res;
			});
		}

		d.ioArgs = ioArgs;

		// FIXME: need to wire up the xhr object's abort method to something
		// analagous in the Deferred
		return d;
	};

	var _deferredCancel = function(/*Deferred*/dfd){
		// summary: canceller function for dojo._ioSetArgs call.

		dfd.canceled = true;
		var xhr = dfd.ioArgs.xhr;
		var _at = typeof xhr.abort;
		if(_at == "function" || _at == "object" || _at == "unknown"){
			xhr.abort();
		}
		var err = dfd.ioArgs.error;
		if(!err){
			err = new Error("xhr cancelled");
			err.dojoType="cancel";
		}
		return err;
	};
	var _deferredOk = function(/*Deferred*/dfd){
		// summary: okHandler function for dojo._ioSetArgs call.

		var ret = handlers[dfd.ioArgs.handleAs](dfd.ioArgs.xhr);
		return ret === undefined ? null : ret;
	};
	var _deferError = function(/*Error*/error, /*Deferred*/dfd){
		// summary: errHandler function for dojo._ioSetArgs call.

		if(!dfd.ioArgs.args.failOk){
			console.error(error);
		}
		return error;
	};

	// avoid setting a timer per request. It degrades performance on IE
	// something fierece if we don't use unified loops.
	var _inFlightIntvl = null;
	var _inFlight = [];


	//Use a separate count for knowing if we are starting/stopping io calls.
	//Cannot use _inFlight.length since it can change at a different time than
	//when we want to do this kind of test. We only want to decrement the count
	//after a callback/errback has finished, since the callback/errback should be
	//considered as part of finishing a request.
	var _pubCount = 0;
	var _checkPubCount = function(dfd){
		if(_pubCount <= 0){
			_pubCount = 0;
			if(cfg.ioPublish && dojo.publish && (!dfd || dfd && dfd.ioArgs.args.ioPublish !== false)){
				dojo.publish("/dojo/io/stop");
			}
		}
	};

	var _watchInFlight = function(){
		//summary:
		//		internal method that checks each inflight XMLHttpRequest to see
		//		if it has completed or if the timeout situation applies.

		var now = (new Date()).getTime();
		// make sure sync calls stay thread safe, if this callback is called
		// during a sync call and this results in another sync call before the
		// first sync call ends the browser hangs
		if(!dojo._blockAsync){
			// we need manual loop because we often modify _inFlight (and therefore 'i') while iterating
			// note: the second clause is an assigment on purpose, lint may complain
			for(var i = 0, tif; i < _inFlight.length && (tif = _inFlight[i]); i++){
				var dfd = tif.dfd;
				var func = function(){
					if(!dfd || dfd.canceled || !tif.validCheck(dfd)){
						_inFlight.splice(i--, 1);
						_pubCount -= 1;
					}else if(tif.ioCheck(dfd)){
						_inFlight.splice(i--, 1);
						tif.resHandle(dfd);
						_pubCount -= 1;
					}else if(dfd.startTime){
						//did we timeout?
						if(dfd.startTime + (dfd.ioArgs.args.timeout || 0) < now){
							_inFlight.splice(i--, 1);
							var err = new Error("timeout exceeded");
							err.dojoType = "timeout";
							dfd.errback(err);
							//Cancel the request so the io module can do appropriate cleanup.
							dfd.cancel();
							_pubCount -= 1;
						}
					}
				};
				if(dojo.config.debugAtAllCosts){
					func.call(this);
				}else{
					try{
						func.call(this);
					}catch(e){
						dfd.errback(e);
					}
				}
			}
		}

		_checkPubCount(dfd);

		if(!_inFlight.length){
			clearInterval(_inFlightIntvl);
			_inFlightIntvl = null;
		}
	};

	dojo._ioCancelAll = function(){
		//summary: Cancels all pending IO requests, regardless of IO type
		//(xhr, script, iframe).
		try{
			array.forEach(_inFlight, function(i){
				try{
					i.dfd.cancel();
				}catch(e){/*squelch*/}
			});
		}catch(e){/*squelch*/}
	};

	//Automatically call cancel all io calls on unload
	//in IE for trac issue #2357.
	if(has("ie")){
		on(window, "unload", dojo._ioCancelAll);
	}

	dojo._ioNotifyStart = function(/*Deferred*/dfd){
		// summary:
		//		If dojo.publish is available, publish topics
		//		about the start of a request queue and/or the
		//		the beginning of request.
		// description:
		//		Used by IO transports. An IO transport should
		//		call this method before making the network connection.
		if(cfg.ioPublish && dojo.publish && dfd.ioArgs.args.ioPublish !== false){
			if(!_pubCount){
				dojo.publish("/dojo/io/start");
			}
			_pubCount += 1;
			dojo.publish("/dojo/io/send", [dfd]);
		}
	};

	dojo._ioWatch = function(dfd, validCheck, ioCheck, resHandle){
		// summary:
		//		Watches the io request represented by dfd to see if it completes.
		// dfd: Deferred
		//		The Deferred object to watch.
		// validCheck: Function
		//		Function used to check if the IO request is still valid. Gets the dfd
		//		object as its only argument.
		// ioCheck: Function
		//		Function used to check if basic IO call worked. Gets the dfd
		//		object as its only argument.
		// resHandle: Function
		//		Function used to process response. Gets the dfd
		//		object as its only argument.
		var args = dfd.ioArgs.args;
		if(args.timeout){
			dfd.startTime = (new Date()).getTime();
		}

		_inFlight.push({dfd: dfd, validCheck: validCheck, ioCheck: ioCheck, resHandle: resHandle});
		if(!_inFlightIntvl){
			_inFlightIntvl = setInterval(_watchInFlight, 50);
		}
		// handle sync requests
		//A weakness: async calls in flight
		//could have their handlers called as part of the
		//_watchInFlight call, before the sync's callbacks
		// are called.
		if(args.sync){
			_watchInFlight();
		}
	};

	var _defaultContentType = "application/x-www-form-urlencoded";

	var _validCheck = function(/*Deferred*/dfd){
		return dfd.ioArgs.xhr.readyState; //boolean
	};
	var _ioCheck = function(/*Deferred*/dfd){
		return 4 == dfd.ioArgs.xhr.readyState; //boolean
	};
	var _resHandle = function(/*Deferred*/dfd){
		var xhr = dfd.ioArgs.xhr;
		if(dojo._isDocumentOk(xhr)){
			dfd.callback(dfd);
		}else{
			var err = new Error("Unable to load " + dfd.ioArgs.url + " status:" + xhr.status);
			err.status = xhr.status;
			err.responseText = xhr.responseText;
			err.xhr = xhr;
			dfd.errback(err);
		}
	};

	dojo._ioAddQueryToUrl = function(/*dojo.__IoCallbackArgs*/ioArgs){
		//summary: Adds query params discovered by the io deferred construction to the URL.
		//Only use this for operations which are fundamentally GET-type operations.
		if(ioArgs.query.length){
			ioArgs.url += (ioArgs.url.indexOf("?") == -1 ? "?" : "&") + ioArgs.query;
			ioArgs.query = null;
		}
	};

	/*=====
	dojo.declare("dojo.__XhrArgs", dojo.__IoArgs, {
		constructor: function(){
			//	summary:
			//		In addition to the properties listed for the dojo._IoArgs type,
			//		the following properties are allowed for dojo.xhr* methods.
			//	handleAs: String?
			//		Acceptable values are: text (default), json, json-comment-optional,
			//		json-comment-filtered, javascript, xml. See `dojo.contentHandlers`
			//	sync: Boolean?
			//		false is default. Indicates whether the request should
			//		be a synchronous (blocking) request.
			//	headers: Object?
			//		Additional HTTP headers to send in the request.
			//	failOk: Boolean?
			//		false is default. Indicates whether a request should be
			//		allowed to fail (and therefore no console error message in
			//		the event of a failure)
			//	contentType: String|Boolean
			//		"application/x-www-form-urlencoded" is default. Set to false to
			//		prevent a Content-Type header from being sent, or to a string
			//		to send a different Content-Type.
			this.handleAs = handleAs;
			this.sync = sync;
			this.headers = headers;
			this.failOk = failOk;
		}
	});
	=====*/

	dojo.xhr = function(/*String*/ method, /*dojo.__XhrArgs*/ args, /*Boolean?*/ hasBody){
		//	summary:
		//		Sends an HTTP request with the given method.
		//	description:
		//		Sends an HTTP request with the given method.
		//		See also dojo.xhrGet(), xhrPost(), xhrPut() and dojo.xhrDelete() for shortcuts
		//		for those HTTP methods. There are also methods for "raw" PUT and POST methods
		//		via dojo.rawXhrPut() and dojo.rawXhrPost() respectively.
		//	method:
		//		HTTP method to be used, such as GET, POST, PUT, DELETE. Should be uppercase.
		//	hasBody:
		//		If the request has an HTTP body, then pass true for hasBody.

		//Make the Deferred object for this xhr request.
		var dfd = dojo._ioSetArgs(args, _deferredCancel, _deferredOk, _deferError);
		var ioArgs = dfd.ioArgs;

		//Pass the args to _xhrObj, to allow alternate XHR calls based specific calls, like
		//the one used for iframe proxies.
		var xhr = ioArgs.xhr = dojo._xhrObj(ioArgs.args);
		//If XHR factory fails, cancel the deferred.
		if(!xhr){
			dfd.cancel();
			return dfd;
		}

		//Allow for specifying the HTTP body completely.
		if("postData" in args){
			ioArgs.query = args.postData;
		}else if("putData" in args){
			ioArgs.query = args.putData;
		}else if("rawBody" in args){
			ioArgs.query = args.rawBody;
		}else if((arguments.length > 2 && !hasBody) || "POST|PUT".indexOf(method.toUpperCase()) == -1){
			//Check for hasBody being passed. If no hasBody,
			//then only append query string if not a POST or PUT request.
			dojo._ioAddQueryToUrl(ioArgs);
		}

		// IE 6 is a steaming pile. It won't let you call apply() on the native function (xhr.open).
		// workaround for IE6's apply() "issues"
		xhr.open(method, ioArgs.url, args.sync !== true, args.user || undefined, args.password || undefined);
		if(args.headers){
			for(var hdr in args.headers){
				if(hdr.toLowerCase() === "content-type" && !args.contentType){
					args.contentType = args.headers[hdr];
				}else if(args.headers[hdr]){
					//Only add header if it has a value. This allows for instnace, skipping
					//insertion of X-Requested-With by specifying empty value.
					xhr.setRequestHeader(hdr, args.headers[hdr]);
				}
			}
		}
		// FIXME: is this appropriate for all content types?
		if(args.contentType !== false){
			xhr.setRequestHeader("Content-Type", args.contentType || _defaultContentType);
		}
		if(!args.headers || !("X-Requested-With" in args.headers)){
			xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
		}
		// FIXME: set other headers here!
		dojo._ioNotifyStart(dfd);
		if(dojo.config.debugAtAllCosts){
			xhr.send(ioArgs.query);
		}else{
			try{
				xhr.send(ioArgs.query);
			}catch(e){
				ioArgs.error = e;
				dfd.cancel();
			}
		}
		dojo._ioWatch(dfd, _validCheck, _ioCheck, _resHandle);
		xhr = null;
		return dfd; // dojo.Deferred
	};

	dojo.xhrGet = function(/*dojo.__XhrArgs*/ args){
		//	summary:
		//		Sends an HTTP GET request to the server.
		return dojo.xhr("GET", args); // dojo.Deferred
	};

	dojo.rawXhrPost = dojo.xhrPost = function(/*dojo.__XhrArgs*/ args){
		//	summary:
		//		Sends an HTTP POST request to the server. In addtion to the properties
		//		listed for the dojo.__XhrArgs type, the following property is allowed:
		//	postData:
		//		String. Send raw data in the body of the POST request.
		return dojo.xhr("POST", args, true); // dojo.Deferred
	};

	dojo.rawXhrPut = dojo.xhrPut = function(/*dojo.__XhrArgs*/ args){
		//	summary:
		//		Sends an HTTP PUT request to the server. In addtion to the properties
		//		listed for the dojo.__XhrArgs type, the following property is allowed:
		//	putData:
		//		String. Send raw data in the body of the PUT request.
		return dojo.xhr("PUT", args, true); // dojo.Deferred
	};

	dojo.xhrDelete = function(/*dojo.__XhrArgs*/ args){
		//	summary:
		//		Sends an HTTP DELETE request to the server.
		return dojo.xhr("DELETE", args); //dojo.Deferred
	};

	/*
	dojo.wrapForm = function(formNode){
		//summary:
		//		A replacement for FormBind, but not implemented yet.

		// FIXME: need to think harder about what extensions to this we might
		// want. What should we allow folks to do w/ this? What events to
		// set/send?
		throw new Error("dojo.wrapForm not yet implemented");
	}
	*/

	dojo._isDocumentOk = function(http){
		var stat = http.status || 0;
		stat =
			(stat >= 200 && stat < 300) || // allow any 2XX response code
			stat == 304 ||                 // or, get it out of the cache
			stat == 1223 ||                // or, Internet Explorer mangled the status code
			!stat;                         // or, we're Titanium/browser chrome/chrome extension requesting a local file
		return stat; // Boolean
	};

	dojo._getText = function(url){
		var result;
		dojo.xhrGet({url:url, sync:true, load:function(text){
			result = text;
		}});
		return result;
	};

	// Add aliases for static functions to dojo.xhr since dojo.xhr is what's returned from this module
	lang.mixin(dojo.xhr, {
		_xhrObj: dojo._xhrObj,
		fieldToObject: domForm.fieldToObject,
		formToObject: domForm.toObject,
		objectToQuery: ioq.objectToQuery,
		formToQuery: domForm.toQuery,
		formToJson: domForm.toJson,
		queryToObject: ioq.queryToObject,
		contentHandlers: handlers,
		_ioSetArgs: dojo._ioSetArgs,
		_ioCancelAll: dojo._ioCancelAll,
		_ioNotifyStart: dojo._ioNotifyStart,
		_ioWatch: dojo._ioWatch,
		_ioAddQueryToUrl: dojo._ioAddQueryToUrl,
		_isDocumentOk: dojo._isDocumentOk,
		_getText: dojo._getText,
		get: dojo.xhrGet,
		post: dojo.xhrPost,
		put: dojo.xhrPut,
		del: dojo.xhrDelete	// because "delete" is a reserved word
	});

	return dojo.xhr;
});

},
'dojo/has':function(){
define(["require"], function(require) {
	// module:
	//		dojo/has
	// summary:
	//		Defines the has.js API and several feature tests used by dojo.
	// description:
	//		This module defines the has API as described by the project has.js with the following additional features:
	//
	//			* the has test cache is exposed at has.cache.
	//			* the method has.add includes a forth parameter that controls whether or not existing tests are replaced
	//			* the loader's has cache may be optionally copied into this module's has cahce.
	//
	//		This module adopted from https://github.com/phiggins42/has.js; thanks has.js team!

	// try to pull the has implementation from the loader; both the dojo loader and bdLoad provide one
	// WARNING: if a foreign loader defines require.has to be something other than the has.js API, then this implementation fail
	var has = require.has || function(){};
	if(!1){
		// notice the condition is written so that if 1 is transformed to 1 during a build
		// the conditional will be (!1 && typeof has=="function") which is statically false and the closure
		// compiler will discard the block.
		var
			isBrowser =
				// the most fundamental decision: are we in the browser?
				typeof window != "undefined" &&
				typeof location != "undefined" &&
				typeof document != "undefined" &&
				window.location == location && window.document == document,

			// has API variables
			global = this,
			doc = isBrowser && document,
			element = doc && doc.createElement("DiV"),
			cache = {};

		has = /*===== dojo.has= =====*/ function(name){
			//	summary:
			//		Return the current value of the named feature.
			//
			//	name: String|Integer
			//		The name (if a string) or identifier (if an integer) of the feature to test.
			//
			//	description:
			//		Returns the value of the feature named by name. The feature must have been
			//		previously added to the cache by has.add.

			return typeof cache[name] == "function" ? (cache[name] = cache[name](global, doc, element)) : cache[name]; // Boolean
		};

		has.cache = cache;

		has.add = /*====== dojo.has.add= ======*/ function(name, test, now, force){
			// summary:
			//	 Register a new feature test for some named feature.
			//
			// name: String|Integer
			//	 The name (if a string) or identifier (if an integer) of the feature to test.
			//
			// test: Function
			//	 A test function to register. If a function, queued for testing until actually
			//	 needed. The test function should return a boolean indicating
			//	 the presence of a feature or bug.
			//
			// now: Boolean?
			//	 Optional. Omit if `test` is not a function. Provides a way to immediately
			//	 run the test and cache the result.
			//
			// force: Boolean?
			//	 Optional. If the test already exists and force is truthy, then the existing
			//	 test will be replaced; otherwise, add does not replace an existing test (that
			//	 is, by default, the first test advice wins).
			//
			// example:
			//			A redundant test, testFn with immediate execution:
			//	|				has.add("javascript", function(){ return true; }, true);
			//
			// example:
			//			Again with the redundantness. You can do this in your tests, but we should
			//			not be doing this in any internal has.js tests
			//	|				has.add("javascript", true);
			//
			// example:
			//			Three things are passed to the testFunction. `global`, `document`, and a generic element
			//			from which to work your test should the need arise.
			//	|				has.add("bug-byid", function(g, d, el){
			//	|						// g	== global, typically window, yadda yadda
			//	|						// d	== document object
			//	|						// el == the generic element. a `has` element.
			//	|						return false; // fake test, byid-when-form-has-name-matching-an-id is slightly longer
			//	|				});

			(typeof cache[name]=="undefined" || force) && (cache[name]= test);
			return now && has(name);
		};

		// since we're operating under a loader that doesn't provide a has API, we must explicitly initialize
		// has as it would have otherwise been initialized by the dojo loader; use has.add to the builder
		// can optimize these away iff desired
		true || has.add("host-browser", isBrowser);
		true || has.add("dom", isBrowser);
		true || has.add("dojo-dom-ready-api", 1);
		true || has.add("dojo-sniff", 1);
	}

	if(1){
		var agent = navigator.userAgent;
		// Common application level tests
		has.add("dom-addeventlistener", !!document.addEventListener);
		has.add("touch", "ontouchstart" in document);
		// I don't know if any of these tests are really correct, just a rough guess
		has.add("device-width", screen.availWidth || innerWidth);
		has.add("agent-ios", !!agent.match(/iPhone|iP[ao]d/));
		has.add("agent-android", agent.indexOf("android") > 1);
	}

	has.clearElement = /*===== dojo.has.clearElement= ======*/ function(element) {
		// summary:
		//	 Deletes the contents of the element passed to test functions.
		element.innerHTML= "";
		return element;
	};

	has.normalize = /*===== dojo.has.normalize= ======*/ function(id, toAbsMid){
		// summary:
		//	 Resolves id into a module id based on possibly-nested tenary expression that branches on has feature test value(s).
		//
		// toAbsMid: Function
		//	 Resolves a relative module id into an absolute module id
		var
			tokens = id.match(/[\?:]|[^:\?]*/g), i = 0,
			get = function(skip){
				var term = tokens[i++];
				if(term == ":"){
					// empty string module name, resolves to 0
					return 0;
				}else{
					// postfixed with a ? means it is a feature to branch on, the term is the name of the feature
					if(tokens[i++] == "?"){
						if(!skip && has(term)){
							// matched the feature, get the first value from the options
							return get();
						}else{
							// did not match, get the second value, passing over the first
							get(true);
							return get(skip);
						}
					}
					// a module
					return term || 0;
				}
			};
		id = get();
		return id && toAbsMid(id);
	};

	has.load = /*===== dojo.has.load= ======*/ function(id, parentRequire, loaded){
		// summary:
		//	 Conditional loading of AMD modules based on a has feature test value.
		//
		// id: String
		//	 Gives the resolved module id to load.
		//
		// parentRequire: Function
		//	 The loader require function with respect to the module that contained the plugin resource in it's
		//	 dependency list.
		//
		// loaded: Function
		//	 Callback to loader that consumes result of plugin demand.

		if(id){
			parentRequire([id], loaded);
		}else{
			loaded();
		}
	};

	return has;
});

},
'dojo/cldr/nls/en/currency':function(){
define(
//begin v1.x content
{
	"HKD_displayName": "Hong Kong Dollar",
	"CHF_displayName": "Swiss Franc",
	"JPY_symbol": "¥",
	"CAD_displayName": "Canadian Dollar",
	"CNY_displayName": "Chinese Yuan",
	"USD_symbol": "$",
	"AUD_displayName": "Australian Dollar",
	"JPY_displayName": "Japanese Yen",
	"USD_displayName": "US Dollar",
	"GBP_displayName": "British Pound Sterling",
	"EUR_displayName": "Euro"
}
//end v1.x content
);
},
'com.ibm.bpm.coach/List':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach data list class js

define(["dojo/_base/declare", "dojo/store/Memory", "dojo/store/Observable", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/json", "dojo/Stateful", "./commonHelper", "./dataBindingHelper", "./Stateful", "require", "./controller", "./logger", "./templateStore"], 
		function(declare, memory, observable, lang, array, json, dojoStateful, commonHelper, dataBindingHelper, stateful, require, controller, logger, templateStore) {
	//List object that uses dojo.store.Observable to "watch" the list for updates.
	//Maintains an internal representation of the list to map ids, so the user of this list does not need to supply unique ids
	//for each list item
	return declare("com.ibm.bpm.coach.List", [dojoStateful], {
		childrenCache: null,
		dataStore: null,
		propertyList: null,
		//index: 0,  //index is the index for the next available spot
		managedList: null,
		indexToId: null, // maitains a map from index to id. Think about remove from middle case; remove once, and remove again
		items: null,
		_systemCallbackHandle: null,
		_systemCallbackPropertyHandle: null,	
		
		//
		//1. bindAll: right now, it regisgter on the property list, should that also be on all properties
		//2. listAllSelectedIndices is plain array object, which are agreed.
		// following 4 properties for selection, they can be selected in design time for binding
		listAllSelectedIndices: null, //  (array) - return an array of all selected indices
		listSelectedIndex: null, // (int) - return the first element of listAllSelectedIndices
		listAllSelected: null, // (array) - return an array of selected elements (derived from listAllSelectedIndices and the array)
		listSelected: null, // (object) - return the selected element (derived from listSelectedIndex and the array)
		_masterList : null, // if the List is for listAllSelected, then the list have a masterList, which is the source of the data for this list
		
		constructor: function(listObj, path) {
		    
			this.childrenCache = {};
			this.managedList=[];
			this.indexToId = {};
			this._objectPath = path;
		
			//Store the items and populate other properites if this is an object contains items and listAllSelectedIndices
			if (!listObj){
				this.items = [];
			}
			else if (!!listObj.items && (!!listObj.listAllSelectedIndices || (!!listObj["selected"] && !!listObj["@metadata"]))){
				this.items = listObj.items;
				//defect 121645 - lists returned from AJAX services use "selected" instead of listAllSelectedIndecies
				if (!!listObj["listAllSelectedIndices"]) {
					this.listAllSelectedIndices = listObj.listAllSelectedIndices;
				} else if (!!listObj["selected"]) {
					this.listAllSelectedIndices = listObj.selected;
				}
				this._setDerivedProperties(true);
				
				// this.te is undefined when listObj.te doesn't exist. It's ok, we can try to figure out te when it is time to use it to create new element
				// when list is constructed as the response to server, likely litObj doesn't have "te" property
				if(listObj.te) {
					this.te = listObj.te;
					templateStore.save(path, listObj.te);
				}
			}
			else {
				//happens when the list is for ListAllSelected
				this.items = listObj;
			}
			
			//populate the internal managedList object with generated ids
			for (var index in this.items) {		
				this.managedList.push({"bpm_id":  this.updateId(index), "listItem": this.items[index]});
			};
			this.dataStore = observable(new memory({data: this.managedList, idProperty:"bpm_id"}));
			this.propertyList = this.dataStore.query();

			//register the system callback
			this._systemCallbackHandle = null,
			this._systemCallbackPropertyHandle = null,
			this.enableSystemNotifications("all");
		},
		
		disableSystemNotifications: function (property) {
			if (!property)
				this._systemCallbackHandle.cancel();
				
			if(!!this._systemCallbackPropertyHandle && property == "listAllSelectedIndices"){
				this._systemCallbackPropertyHandle.unwatch();
			}
		},
		
		enableSystemNotifications: function (property) {
			//set system call back, for notifications to controller
			if (!property || property == "all"){
				this._systemCallbackHandle = this.registerCallback(null, lang.hitch(this, function(changedObject, removedFrom, insertedInto) {
											controller.systemListChangeHandler(this._objectPath, changedObject, removedFrom, insertedInto);
										}));
			};
			
			//only register for listAllSelectedIndices
			if ( property == "all" || property == "listAllSelectedIndices"){
				this._systemCallbackPropertyHandle = this.registerCallback("listAllSelectedIndices", lang.hitch(this, function(property, oldValue, newValue) {
											controller.systemChangeHandler(this._objectPath, property, newValue);
										}));	
            }										
		},
		
		updateId: function(index){
			
		    var id;
            if (this.length() == 0){
				id = 1;
			}
			else{
				id = this.indexToId[this.length()-1] + 1;
			}			
			this.indexToId[index] = id;
			return id;
		},
				
		//registers a callback for this object that will be invoked if this object is modified
		registerCallback: function (property, callback) {
			var handle;
			if (callback){
				if (!property){				
				   handle =  this.propertyList.observe(callback, true);
				   handle.unbind = function (){						
						this.cancel();
					}
				}
				else {//if (property == "listSelected" || property  == "listAllSelectedIndices"		|| property == "listSelectedIndex" || property == "listAllSelected"){
					handle = this.watch(property, callback);
					
					handle.unbind = function (){						
						this.unwatch();
					}
					
				}
			}
			return handle;
		},
		
		//puts a new object on the list, replacing object at index
		put: function(index, object, options, disableSystemNotification) {
			logger.log("List", "put", "List: " + this._objectPath + ", index: " + index + ", object: " + object + ", options: " + options, logger.FINER);
		    if (index < 0 || index > this.length() - 1){
				return;
			}						   
			
			// check if it makes sense to accept the put request
			var currentPlainValue = dataBindingHelper.getPlainValue(this, index);
			if(commonHelper.isValuesEqual(currentPlainValue, object) == commonHelper.YES) {
				// new value is equal to current value, ignore this put request
				logger.log("List", "put", "Ignore this put request because new value is equal to current value. value = " + currentPlainValue, logger.FINE);
				return;
			}

		    //update items too
			if (this.items){
				this.items.splice(index, 1, object);		
			}	
			if (!!this.childrenCache[index]){
				delete this.childrenCache[index];
			}

			var result = null;
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				result = this.dataStore.put({"bpm_id": this.indexToId[index], listItem: object}, options); 
				this.enableSystemNotifications();
			} else {
				result = this.dataStore.put({"bpm_id": this.indexToId[index], listItem: object}, options); 
			}
			//Use the same id that is used by the object in this position
			return result;
		},
		
		//adds a new object to the end of the list
		add: function(object, options, disableSystemNotification) {		
			logger.log("List", "add", "List: " + this._objectPath + ", object: " + object + ", options: " + options, logger.FINER);
			var index = this.length();
			this.items.push(object);
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				this.dataStore.add({"bpm_id": this.updateId(index), listItem: object}, options)	;
				this.enableSystemNotifications();
			} else {
				this.dataStore.add({"bpm_id": this.updateId(index), listItem: object}, options)	;				
			}
			return index;
		},
		
		//removes the object at index from the list					
		remove: function(index, disableSystemNotification) {
			logger.log("List", "remove", "List: " + this._objectPath + ", index: " + index, logger.FINER);
			if (index < 0 || index >= this.length()){
				return;
			}
		
			if (this.items) {
				this.items.splice(index, 1);
			}
			
			var idToRemove = this.indexToId[index];
						
			//update all the index to Id mappings			
			for (var i=index; i < this.length(); i++){			   
				this.indexToId[i] = this.indexToId[i + 1];
				this.childrenCache[i] = this.childrenCache[i + 1];
			}
			delete this.indexToId[i];
			delete this.childrenCache[i];
			
			var result = null;
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				result = this.dataStore.remove(idToRemove);
				this.enableSystemNotifications();
			} else {
				result = this.dataStore.remove(idToRemove);
			}
			return result;
		},
		
		set: function(property, value, disableSystemNotification){
			logger.log("List", "set", "List: " + this._objectPath + ", property: " + property + ", value: " + value, logger.FINER);
			
			// dojo Statful calls set() during the execution of its constructor in version 1.8.3.
			// If first parameter (i.e. property) is an object and no other parameters are specified, we know set() is called by the constructor of dojo Stateful. 
			// The set() call can be ignored since com.ibm.bpm.coach.List doesn't handle it at all.
			// The set() call is ignored in follow way: when property is an complex object, currentPlainValue will be null, and value (input parameter) is also null.
			// currentPlainValue is equal to value, so we ignore the set() call accordingly.

			// check if it makes sense to accept the set request
			var currentPlainValue = dataBindingHelper.getPlainValue(this, property);
			if(commonHelper.isValuesEqual(currentPlainValue, value) == commonHelper.YES) {
				// new value is equal to current value, ignore this set request
				logger.log("List", "set", "Ignore this set request because new value is equal to current value. value = " + currentPlainValue, logger.FINE);
				return;
			}
			
			if (!!disableSystemNotification && property == "listAllSelectedIndices")
				this.disableSystemNotifications(property);	
				
			if (property != "listAllSelectedIndices" && property != "listSelectedIndex" 
			    && property != "listAllSelected" && property != "listSelected") {
				var index = this.idToIndex(parseInt(property));
				if(!!index && index >=0 && index < this.length()){
					this.put(index, value, null, disableSystemNotification);
				}
			} else {
				this.inherited(arguments);
			}
			if (!!disableSystemNotification && property == "listAllSelectedIndices")
				this.enableSystemNotifications(property);	
						
			if (property == "listAllSelectedIndices"){	                			
				this._setDerivedProperties(false);
			}	  
		},

		//gets the object at index or get the value of properties
		//also maintains a cache of any embedded lists or obejcts and makes them watchable.
		get: function(indexOrProperty) {
			var result;
			
			//special case for the 4 properties
			if (indexOrProperty == "listAllSelectedIndices" || indexOrProperty == "listSelectedIndex"){
				return this[indexOrProperty];  //not stateful or List
			}
			else if (indexOrProperty == "listSelected"){
				//Using get to get it from the main list, so that they points to the same wrapped object
				return this.get(this.listSelectedIndex);				
			}
			else if(indexOrProperty  == "listAllSelected"){
				var value = this.childrenCache[indexOrProperty];
				if (!value){
				
					//NOTE: that if listAllSelected also want to be a list,should we also give a listAllSelected property to it
					//but it does not make sense to do that
					value = new com.ibm.bpm.coach.List(this[indexOrProperty], this._objectPath + "." + indexOrProperty);
					value._masterList = this;
				}
				this.childrenCache[indexOrProperty] = value;	
				return value;
			}
			
			//The call is for getting an object in the items array in this list
			var index = indexOrProperty;
		
			//Special case: if this list object is for listAllSelected, then it has a main list where the data should come from, then get it from the mater list
			if (!!this._masterList){
			   return this._masterList.get(index);		
			}
			
			//first check the cache
			var property = this.childrenCache[index];

			if (typeof(property) == "undefined") {
				var obj = this.items[index];  
								
				//check if it is a simple type
				var dataTypes = ["number", "string", "boolean"];
				array.forEach(dataTypes, function(dataType) {
					if (typeof obj == dataType) {
						result = obj;
					}
				});

				if (typeof result == "undefined") {                                        
					if (!!obj && (lang.isArray(obj)|| !!obj.items && !!obj.listAllSelectedIndices)) {
						result = new com.ibm.bpm.coach.List(obj, this._objectPath + "[" +index + "]");
						//add result to the cache
						this.childrenCache[index] = result;
		        	} else if (typeof obj == "object") {
		        		if (obj == null || !obj.prototype) { // check that this is a literal
		        			result = obj;
		        		} else if (obj.constructor) {
		        			result = new obj.constructor();
		        		}
		        		
					//treat Date like  primitive type
					if (result != null && !(result instanceof Date)) {
						var objectPath = this._objectPath;
		    				require(["./Stateful"], function(stateful) {
		    					result = new stateful(result, objectPath + "[" +index + "]");
		    				});		
	        				//add result to the cache
	        				this.childrenCache[index] = result;
	        			}
	        		}
				} else {
					result = obj;
				}
			} else {
				result = property;
			}
			return result;
		}, //get 
		
		getById: function(bpm_id) {
			var index = this.idToIndex(bpm_id);
			if (!!index) {
				return this.get(index);
			}
			return null;
		},
		
		idToIndex: function(bpm_id) {
			for (var index in this.indexToId) {		
				if (this.indexToId[index] == bpm_id) {
					return index;
				}
			}
			return null;
		},
		
		//retuns the length of the list
		length: function() {
			return this.managedList.length; 
		},
		
		//list item add/remove 
		bindAll: function(callback, scope) {
		    var wrapperFunction = function(object, removedFrom, insertedInto) {
				var newVal, oldVal;		
				if (insertedInto > -1) {
					// "add" event
					newVal = object.listItem;
	             }
				if (removedFrom > -1){
					if(removedFrom == insertedInto) {
						// removedFrom and insertedInto have same value and the value is not -1, means that this is a "put" event.
						// we do not have the old value for the "put" event based on current design
						oldVal = null; 
					} else {
						// "delete" event
						oldVal = object.listItem;
					}			 
				 }
				var event = {property: null, oldVal: oldVal, newVal: newVal, removedFrom: removedFrom, insertedInto: insertedInto};
				try {
					callback.call(scope, event);
				} catch(ex) {
					commonHelper.consoleLog(ex, commonHelper.ERROR);
				}
			}
			return this.registerCallback(null, wrapperFunction);
				
	    },
		
	    bindAllWithIds: function(callback, scope) {
            var wrapperFunction = function(object, removedFrom, insertedInto) {
                var newVal, oldVal;     
                if (insertedInto > -1) {
                    newVal = object;
                 }
                if (removedFrom > -1){
                    oldVal = object;             
                 }
                var event = {property: null, oldVal: oldVal, newVal: newVal, removedFrom: removedFrom, insertedInto: insertedInto};
				try {
					callback.call(scope, event);
				} catch(ex) {
					commonHelper.consoleLog(ex, commonHelper.ERROR);
				}
            }
            return this.registerCallback(null, wrapperFunction);
                
        },
		
		bind: function (property, callback, scope){
		     var wrapperFunction = function(property, oldVal, newVal) {
				var event = {property: property, oldVal: oldVal, newVal: newVal};
				try {
					callback.call(scope, event);
				} catch(ex) {
					commonHelper.consoleLog(ex, commonHelper.ERROR);
				}
			}
			return this.registerCallback(property, wrapperFunction);
		},
		
		toJson: function() {
			logger.log("List", "toJson", "Serializing list: " + this._objectPath, logger.FINER);
			//first get all of the stateful properties and do a toJson on them and 
			//then print the simple properties
			//debugger;
			var resultList = [];
			for (var x=0; x< this.items.length; x++) {
				if ((typeof this.items[x] == 'object') && this.items[x] instanceof Date) {
					var curItem = this.items[x];
					require(["./Stateful"], function(stateful) {
						resultList.push("\"" + stateful.prototype.dateToString(curItem) + "\"");
					});
				}
                else if (this.items[x] == null) {
                    resultList.push(null);
                } else if (typeof this.items[x] == 'object' ||
					lang.isArray(this.items[x])||
					typeof this.items[x] == 'com.ibm.bpm.coach.Stateful' ||
					typeof this.items[x] == 'com.ibm.bpm.coach.List') {
						resultList.push(this.get(x).toJson());
				} else {
					resultList.push(json.toJson(this.items[x]));
				}
			}
			
			//include listAllSelectedIndices in the json, the original array data become vlaue of items
			if (!this.listAllSelectedIndices) {
				this.listAllSelectedIndices = [];
			}
			var result = "{ \"listAllSelectedIndices\" : " + "[" + this.listAllSelectedIndices.join(",") + "]";
			result += ", \"items\" : " + "[";
            // we can not use join() here since null values may be present which will be turned into empty value by join.
            for (var x = 0; x < resultList.length; x++) {
                result += resultList[x];
                if (x < resultList.length - 1) {
                    result += ",";
                }
            }
            result += "]}";
			//return "[" + resultList.join(",") + "]";
			logger.log("List", "toJson", result, logger.FINER);
			return result;
		},
		
		//All the values are none wrapped when set, the one get from get method will be wrapped
		_setDerivedProperties: function(initialize){
				//set other properties
				if (!this.listAllSelectedIndices || this.listAllSelectedIndices.length == 0){
					if (!initialize){
						this.set("listSelectedIndex", null);
						this.set("listSelected", null);
			            
						//this is cached so delete the cache
						if (!!this.childrenCache["listAllSelected"]){
							delete this.childrenCache["listAllSelected"];
						}
						this.set("listAllSelected", []);
					}
					return;
				}
				var listSelectedIndex = this.listAllSelectedIndices[0];
				var listSelected = this.items[listSelectedIndex]; 	
				
				var listAllSelected = [];			
				var selectedIndex;
                for (var index in this.listAllSelectedIndices){
					selectedIndex = this.listAllSelectedIndices[index];
					listAllSelected.push(this.items[selectedIndex]);
				}
								
				if (!!initialize){
					//do not trigger notification
					this.listSelectedIndex = listSelectedIndex;
					this.listSelected = listSelected;
					this.listAllSelected = listAllSelected;
				}
				else{			
					
					this.set("listSelectedIndex", listSelectedIndex);
					
					//when get, please get it from this.get("..."), is using this way, we do not need to cache it
					this.set("listSelected", listSelected);
		            
					//this is cached so delete the cache
					if (!!this.childrenCache["listAllSelected"]){
						delete this.childrenCache["listAllSelected"];
					}
					this.set("listAllSelected", listAllSelected);
				}

		},
		createListElement : function(){
			// Two example of this.local, the value of "te" property is the type of list element.
			// this.local = {var1:{listAllSelectedIndices:[],items:[],te:"t1"}}
			// this.local = {strList:{listAllSelectedIndices:[],items:[],te:"S"}}
			
			if(!this.te) {
				this.te = templateStore.getTypeByPath(this._objectPath);
			}
			
			if(this.te) {
				var fullTemplate = templateStore.getByType(this.te);
				// "" and 0 are valid template
				if(fullTemplate != undefined && fullTemplate != null) {
					return lang.clone(fullTemplate);
				}
			}
			return null;
		},
		
		/*
		 * Duplicate the array. It is only used internally by coach runtime
		 * @param obj maybe an array or an object in the same format as the inital data
		 */
		duplicate: function(obj){
			/* Make the obj to the following format. If the obj does not contains those field. use the value from this object
			 * {
				items: [],
				listAllSelectedIndices : [],
				te				
			}*/
			var newItems = [];
			var newListAllSelectedIndices = this.listAllSelectedIndices;
			var newTemplateElement = this.te;
			
			if (lang.isArray(obj)){
				newItems = obj;
			}
			else if ((!!obj) && (lang.isObject(obj))) {
				newItems = obj.items;
				if (obj.listAllSelectedIndices){
					newListAllSelectedIndices = obj.listAllSelectedIndices;
				}
				if (obj.te){
					newTemplateElement = obj.te;
				}
			}
			return {items : newItems, listAllSelectedIndices : newListAllSelectedIndices, te : newTemplateElement};
		}
		
	}); //com.ibm.bpm.coach.List
});


},
'dojo/cldr/nls/en/number':function(){
define(
//begin v1.x content
{
	"group": ",",
	"percentSign": "%",
	"exponential": "E",
	"scientificFormat": "#E0",
	"percentFormat": "#,##0%",
	"list": ";",
	"infinity": "∞",
	"patternDigit": "#",
	"minusSign": "-",
	"decimal": ".",
	"nan": "NaN",
	"nativeZeroDigit": "0",
	"perMille": "‰",
	"decimalFormat": "#,##0.###",
	"currencyFormat": "¤#,##0.00;(¤#,##0.00)",
	"plusSign": "+",
	"decimalFormat-short": "000T"
}
//end v1.x content
);
},
'dojo/back':function(){
define(["./_base/kernel", "./_base/lang", "./_base/sniff", "./dom", "./dom-construct", "./_base/window", "require"], function(dojo, lang, sniff, dom, domConstruct, baseWindow, require) {
	// module:
	//		dojo/back
	// summary:
	//		TODOC

	lang.getObject("back", true, dojo);

/*=====
dojo.back = {
	// summary: Browser history management resources
};
=====*/

	var back = dojo.back,

	// everyone deals with encoding the hash slightly differently

	getHash = back.getHash = function(){
		var h = window.location.hash;
		if(h.charAt(0) == "#"){ h = h.substring(1); }
		return sniff("mozilla") ? h : decodeURIComponent(h);
	},

	setHash = back.setHash = function(h){
		if(!h){ h = ""; }
		window.location.hash = encodeURIComponent(h);
		historyCounter = history.length;
	};

	var initialHref = (typeof(window) !== "undefined") ? window.location.href : "";
	var initialHash = (typeof(window) !== "undefined") ? getHash() : "";
	var initialState = null;

	var locationTimer = null;
	var bookmarkAnchor = null;
	var historyIframe = null;
	var forwardStack = [];
	var historyStack = [];
	var moveForward = false;
	var changingUrl = false;
	var historyCounter;

	function handleBackButton(){
		//summary: private method. Do not call this directly.

		//The "current" page is always at the top of the history stack.
		var current = historyStack.pop();
		if(!current){ return; }
		var last = historyStack[historyStack.length-1];
		if(!last && historyStack.length == 0){
			last = initialState;
		}
		if(last){
			if(last.kwArgs["back"]){
				last.kwArgs["back"]();
			}else if(last.kwArgs["backButton"]){
				last.kwArgs["backButton"]();
			}else if(last.kwArgs["handle"]){
				last.kwArgs.handle("back");
			}
		}
		forwardStack.push(current);
	}

	back.goBack = handleBackButton;

	function handleForwardButton(){
		//summary: private method. Do not call this directly.
		var last = forwardStack.pop();
		if(!last){ return; }
		if(last.kwArgs["forward"]){
			last.kwArgs.forward();
		}else if(last.kwArgs["forwardButton"]){
			last.kwArgs.forwardButton();
		}else if(last.kwArgs["handle"]){
			last.kwArgs.handle("forward");
		}
		historyStack.push(last);
	}

	back.goForward = handleForwardButton;

	function createState(url, args, hash){
		//summary: private method. Do not call this directly.
		return {"url": url, "kwArgs": args, "urlHash": hash};	//Object
	}

	function getUrlQuery(url){
		//summary: private method. Do not call this directly.
		var segments = url.split("?");
		if(segments.length < 2){
			return null; //null
		}
		else{
			return segments[1]; //String
		}
	}

	function loadIframeHistory(){
		//summary: private method. Do not call this directly.
		var url = (dojo.config["dojoIframeHistoryUrl"] || require.toUrl("./resources/iframe_history.html")) + "?" + (new Date()).getTime();
		moveForward = true;
		if(historyIframe){
			sniff("webkit") ? historyIframe.location = url : window.frames[historyIframe.name].location = url;
		}else{
			//console.warn("dojo.back: Not initialised. You need to call dojo.back.init() from a <script> block that lives inside the <body> tag.");
		}
		return url; //String
	}

	function checkLocation(){
		if(!changingUrl){
			var hsl = historyStack.length;

			var hash = getHash();

			if((hash === initialHash||window.location.href == initialHref)&&(hsl == 1)){
				// FIXME: could this ever be a forward button?
				// we can't clear it because we still need to check for forwards. Ugg.
				// clearInterval(this.locationTimer);
				handleBackButton();
				return;
			}

			// first check to see if we could have gone forward. We always halt on
			// a no-hash item.
			if(forwardStack.length > 0){
				if(forwardStack[forwardStack.length-1].urlHash === hash){
					handleForwardButton();
					return;
				}
			}

			// ok, that didn't work, try someplace back in the history stack
			if((hsl >= 2)&&(historyStack[hsl-2])){
				if(historyStack[hsl-2].urlHash === hash){
					handleBackButton();
				}
			}
		}
	}

	back.init = function(){
		//summary: Initializes the undo stack. This must be called from a <script>
		//		   block that lives inside the <body> tag to prevent bugs on IE.
		// description:
		//		Only call this method before the page's DOM is finished loading. Otherwise
		//		it will not work. Be careful with xdomain loading or djConfig.debugAtAllCosts scenarios,
		//		in order for this method to work, dojo.back will need to be part of a build layer.

		// prevent reinit
		if(dom.byId("dj_history")){ return; } 

		var src = dojo.config["dojoIframeHistoryUrl"] || require.toUrl("./resources/iframe_history.html");
		if (dojo._postLoad) {
			console.error("dojo.back.init() must be called before the DOM has loaded. "
						+ "If using xdomain loading or djConfig.debugAtAllCosts, include dojo.back "
						+ "in a build layer.");
		} else {
			document.write('<iframe style="border:0;width:1px;height:1px;position:absolute;visibility:hidden;bottom:0;right:0;" name="dj_history" id="dj_history" src="' + src + '"></iframe>');
		}
	};

	back.setInitialState = function(/*Object*/args){
		//summary:
		//		Sets the state object and back callback for the very first page
		//		that is loaded.
		//description:
		//		It is recommended that you call this method as part of an event
		//		listener that is registered via dojo.addOnLoad().
		//args: Object
		//		See the addToHistory() function for the list of valid args properties.
		initialState = createState(initialHref, args, initialHash);
	};

	//FIXME: Make these doc comments not be awful. At least they're not wrong.
	//FIXME: Would like to support arbitrary back/forward jumps. Have to rework iframeLoaded among other things.
	//FIXME: is there a slight race condition in moz using change URL with the timer check and when
	//		 the hash gets set? I think I have seen a back/forward call in quick succession, but not consistent.


	/*=====
	dojo.__backArgs = function(kwArgs){
		// back: Function?
		//		A function to be called when this state is reached via the user
		//		clicking the back button.
		//	forward: Function?
		//		Upon return to this state from the "back, forward" combination
		//		of navigation steps, this function will be called. Somewhat
		//		analgous to the semantic of an "onRedo" event handler.
		//	changeUrl: Boolean?|String?
		//		Boolean indicating whether or not to create a unique hash for
		//		this state. If a string is passed instead, it is used as the
		//		hash.
	}
	=====*/

	back.addToHistory = function(/*dojo.__backArgs*/ args){
		//	summary:
		//		adds a state object (args) to the history list.
		//	args: dojo.__backArgs
		//		The state object that will be added to the history list.
		//	description:
		//		To support getting back button notifications, the object
		//		argument should implement a function called either "back",
		//		"backButton", or "handle". The string "back" will be passed as
		//		the first and only argument to this callback.
		//
		//		To support getting forward button notifications, the object
		//		argument should implement a function called either "forward",
		//		"forwardButton", or "handle". The string "forward" will be
		//		passed as the first and only argument to this callback.
		//
		//		If you want the browser location string to change, define "changeUrl" on the object. If the
		//		value of "changeUrl" is true, then a unique number will be appended to the URL as a fragment
		//		identifier (http://some.domain.com/path#uniquenumber). If it is any other value that does
		//		not evaluate to false, that value will be used as the fragment identifier. For example,
		//		if changeUrl: 'page1', then the URL will look like: http://some.domain.com/path#page1
		//
		//		There are problems with using dojo.back with semantically-named fragment identifiers
		//		("hash values" on an URL). In most browsers it will be hard for dojo.back to know
		//		distinguish a back from a forward event in those cases. For back/forward support to
		//		work best, the fragment ID should always be a unique value (something using new Date().getTime()
		//		for example). If you want to detect hash changes using semantic fragment IDs, then
		//		consider using dojo.hash instead (in Dojo 1.4+).
		//
		//	example:
		//		|	dojo.back.addToHistory({
		//		|		back: function(){ console.log('back pressed'); },
		//		|		forward: function(){ console.log('forward pressed'); },
		//		|		changeUrl: true
		//		|	});

		//	BROWSER NOTES:
		//	Safari 1.2:
		//	back button "works" fine, however it's not possible to actually
		//	DETECT that you've moved backwards by inspecting window.location.
		//	Unless there is some other means of locating.
		//	FIXME: perhaps we can poll on history.length?
		//	Safari 2.0.3+ (and probably 1.3.2+):
		//	works fine, except when changeUrl is used. When changeUrl is used,
		//	Safari jumps all the way back to whatever page was shown before
		//	the page that uses dojo.undo.browser support.
		//	IE 5.5 SP2:
		//	back button behavior is macro. It does not move back to the
		//	previous hash value, but to the last full page load. This suggests
		//	that the iframe is the correct way to capture the back button in
		//	these cases.
		//	Don't test this page using local disk for MSIE. MSIE will not create
		//	a history list for iframe_history.html if served from a file: URL.
		//	The XML served back from the XHR tests will also not be properly
		//	created if served from local disk. Serve the test pages from a web
		//	server to test in that browser.
		//	IE 6.0:
		//	same behavior as IE 5.5 SP2
		//	Firefox 1.0+:
		//	the back button will return us to the previous hash on the same
		//	page, thereby not requiring an iframe hack, although we do then
		//	need to run a timer to detect inter-page movement.

		//If addToHistory is called, then that means we prune the
		//forward stack -- the user went back, then wanted to
		//start a new forward path.
		forwardStack = [];

		var hash = null;
		var url = null;
		if(!historyIframe){
			if(dojo.config["useXDomain"] && !dojo.config["dojoIframeHistoryUrl"]){
				console.warn("dojo.back: When using cross-domain Dojo builds,"
					+ " please save iframe_history.html to your domain and set djConfig.dojoIframeHistoryUrl"
					+ " to the path on your domain to iframe_history.html");
			}
			historyIframe = window.frames["dj_history"];
		}
		if(!bookmarkAnchor){
			bookmarkAnchor = domConstruct.create("a", {style: {display: "none"}}, baseWindow.body());
		}
		if(args["changeUrl"]){
			hash = ""+ ((args["changeUrl"]!==true) ? args["changeUrl"] : (new Date()).getTime());

			//If the current hash matches the new one, just replace the history object with
			//this new one. It doesn't make sense to track different state objects for the same
			//logical URL. This matches the browser behavior of only putting in one history
			//item no matter how many times you click on the same #hash link, at least in Firefox
			//and Safari, and there is no reliable way in those browsers to know if a #hash link
			//has been clicked on multiple times. So making this the standard behavior in all browsers
			//so that dojo.back's behavior is the same in all browsers.
			if(historyStack.length == 0 && initialState.urlHash == hash){
				initialState = createState(url, args, hash);
				return;
			}else if(historyStack.length > 0 && historyStack[historyStack.length - 1].urlHash == hash){
				historyStack[historyStack.length - 1] = createState(url, args, hash);
				return;
			}

			changingUrl = true;
			setTimeout(function() {
					setHash(hash);
					changingUrl = false;
				}, 1);
			bookmarkAnchor.href = hash;

			if(sniff("ie")){
				url = loadIframeHistory();

				var oldCB = args["back"]||args["backButton"]||args["handle"];

				//The function takes handleName as a parameter, in case the
				//callback we are overriding was "handle". In that case,
				//we will need to pass the handle name to handle.
				var tcb = function(handleName){
					if(getHash() != ""){
						setTimeout(function() { setHash(hash); }, 1);
					}
					//Use apply to set "this" to args, and to try to avoid memory leaks.
					oldCB.apply(this, [handleName]);
				};

				//Set interceptor function in the right place.
				if(args["back"]){
					args.back = tcb;
				}else if(args["backButton"]){
					args.backButton = tcb;
				}else if(args["handle"]){
					args.handle = tcb;
				}

				var oldFW = args["forward"]||args["forwardButton"]||args["handle"];

				//The function takes handleName as a parameter, in case the
				//callback we are overriding was "handle". In that case,
				//we will need to pass the handle name to handle.
				var tfw = function(handleName){
					if(getHash() != ""){
						setHash(hash);
					}
					if(oldFW){ // we might not actually have one
						//Use apply to set "this" to args, and to try to avoid memory leaks.
						oldFW.apply(this, [handleName]);
					}
				};

				//Set interceptor function in the right place.
				if(args["forward"]){
					args.forward = tfw;
				}else if(args["forwardButton"]){
					args.forwardButton = tfw;
				}else if(args["handle"]){
					args.handle = tfw;
				}

			}else if(!sniff("ie")){
				// start the timer
				if(!locationTimer){
					locationTimer = setInterval(checkLocation, 200);
				}

			}
		}else{
			url = loadIframeHistory();
		}

		historyStack.push(createState(url, args, hash));
	};

	back._iframeLoaded = function(evt, ifrLoc){
		//summary:
		//		private method. Do not call this directly.
		var query = getUrlQuery(ifrLoc.href);
		if(query == null){
			// alert("iframeLoaded");
			// we hit the end of the history, so we should go back
			if(historyStack.length == 1){
				handleBackButton();
			}
			return;
		}
		if(moveForward){
			// we were expecting it, so it's not either a forward or backward movement
			moveForward = false;
			return;
		}

		//Check the back stack first, since it is more likely.
		//Note that only one step back or forward is supported.
		if(historyStack.length >= 2 && query == getUrlQuery(historyStack[historyStack.length-2].url)){
			handleBackButton();
		}else if(forwardStack.length > 0 && query == getUrlQuery(forwardStack[forwardStack.length-1].url)){
			handleForwardButton();
		}
	};

	return dojo.back;
	
});

},
'com.ibm.bpm.coach/specificBindingHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all common utilities

define(["dojo/_base/lang", "dojo/_base/array", "./CONSTANTS", "./commonHelper", "./bindingResolver", "./dynamicBindingHelper"], function(lang, array, CONSTANTS, commonHelper, bindingResolver, dynamicBindingHelper) {
				
	var specificBindingHelper = new Object();
	
	/*	
	 Specific bind
	 example of bindingStr: 
	 "local.name", "local.contactBook.owner", 
	 "local.contactBook.phones", "local.contactBook.contacts[2]", "local.contactBook.contacts[1].name", 
	 "local.contactBook.contacts[1].phones[]", "local.contactBook.contacts[1].phones[2].number",
	 "local.contactBook.contacts.listSelected", "local.contactBook.contacts.listSelected.phones.listSelected.name", "local.contactBook.contacts.listSelected.phones[0].name",
	 "config.config1.myOption", "config.config2._metadata.label", "config.config3.myOption.p1",
	 "config.config3.myListOption[1]", "config.config3.myListOption[1].p1",
	 "config.config3.myListOption.listSelected", "config.config3.myListOption.listSelected.p2"
	 
	 "local.contactBook.phones[]" is acceptable, it is interpreted as the recommended notation "local.contactBook.phones". Those 2 notations mean that the change on list itself.
	 
	  When callback is invoked, input parameter event object is expected to be readonly, should not be modified by callback function.
	*/
	specificBindingHelper.bind = function(specificBindingStr, callback, scope) {
		if(!specificBindingStr || !callback) {
			return;
		}
				
		var sourcePathArray = specificBindingStr.split(".");
		
		// only accept the binding path that starts with "config" or "local". 
		// For example, some binding paths may start with "resource", which is supposed to be readonly. Make no sense to listen to the change which never occurs.
		if(sourcePathArray[0] != "config" && sourcePathArray[0] != "local") {			
			return;
		}
		
		if(!!scope) {
			callback = lang.hitch(scope, callback); 
		}

		if(sourcePathArray[0] == "config") {
			var isMetadata = (sourcePathArray[2] == "_metadata" && sourcePathArray.length == 4);
			if((sourcePathArray.length == 3 && sourcePathArray[2].indexOf("[") == -1) || isMetadata) {
				// those options are top level, so no need to hook up them on dynamicBindingHelper for the structure change
				var resolvedVariable = bindingResolver.resolveBinding(specificBindingStr);
				
				if(!!resolvedVariable.property) {
					// for "config", resolvedVariable.property is "value". property = "value" doesn't make sense in the callback event.
					// so we need to pass the real property for the callback event.
					var realProperty = isMetadata ? (sourcePathArray[2] + "." + sourcePathArray[3]) : sourcePathArray[2];
					return resolvedVariable.boundObject.bind(resolvedVariable.property, function(rawEvent) {
						var event = {property: realProperty, oldVal: rawEvent.oldVal, newVal: rawEvent.newVal};
						// the event that is from dynamicBindingHelper always contains "path" property. So prepare "path" here to be consistent.
						event.path = specificBindingStr;
						callback(event);
					});
				}
			}
		}
		
		// lastPath that ends with "[]" (e.g. "local.contactBook.phones[]") is translated as bare list (e.g. "local.contactBook.phones")
		var lastPath = sourcePathArray[sourcePathArray.length - 1];
		if(lastPath.indexOf("[]") != -1) {
			sourcePathArray[sourcePathArray.length - 1] = lastPath.replace("[]", "");
			specificBindingStr = sourcePathArray.join(".");
		}
	
		var generalizedBindingStr = specificBindingStr.replace(/\[\d*\]/g, CONSTANTS.INDEX_WITH_BRACKET);
		if(generalizedBindingStr != specificBindingStr) {
			// 
			var generalizedBindingInfo = specificBindingHelper.generalizedBindingStrMap[generalizedBindingStr];
			if(!generalizedBindingInfo) {
				var generalizedCallbackFunc = function(event) {
					// "this" is generalizedBindingInfo object
					var specificCallbackArray = this.specificBindingStrToCallbackMap[event.path];
					if(specificCallbackArray) {
						for(var k=0; k<specificCallbackArray.length; k++) {
							try {
								specificCallbackArray[k](event);
							} catch(ex) {
								var errorMsg = "specificBindingHelper caught Exception when invoking callback function for change on " + event.path + ", Exception: " + ex;
								commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
							}
						}
					}
				}
				
				generalizedBindingInfo = {
					"specificBindingStrToCallbackMap": {}
				}
				generalizedBindingInfo.bindingHandler = dynamicBindingHelper.bind(generalizedBindingStr, generalizedCallbackFunc, generalizedBindingInfo);
								
				specificBindingHelper.generalizedBindingStrMap[generalizedBindingStr] = generalizedBindingInfo;
			}
			
			var callbackList = generalizedBindingInfo.specificBindingStrToCallbackMap[specificBindingStr];
			if(!callbackList) {
				callbackList = [];
				generalizedBindingInfo.specificBindingStrToCallbackMap[specificBindingStr] = callbackList;
			}
			callbackList.push(callback);
			
			var handle = {
				"_type": CONSTANTS.HANDLE_TYPE_SPECIFIC,
				"_generalizedBindingStr": generalizedBindingStr,
				"bindingStr": specificBindingStr,
				"callback": callback,
				"unbind": function() {
					specificBindingHelper.unbindOne(handle);
				}
			}			
			return handle;
		} else {
			return dynamicBindingHelper.bind(specificBindingStr, callback);
		}
	}
	
	// perform better than unbindOne() in general
	specificBindingHelper.unbindMultiple = function(handleArray) {
		if(!!handleArray) {
			var dynamicBindingHandleArray = [];
			for(var k=0; k<handleArray.length; k++) {
				var handle = handleArray[k];
				if(handle._type == CONSTANTS.HANDLE_TYPE_SPECIFIC) {
					var dynamicBindingHandle = specificBindingHelper_removeCallback(handle);
					if(dynamicBindingHandle) {
						dynamicBindingHandleArray.push(dynamicBindingHandle);
					}
				} else if(handle._type == CONSTANTS.HANDLE_TYPE_DYNAMIC) {
					dynamicBindingHandleArray.push(handle);
				} else {
					// assume the type is CONSTANTS.HANDLE_TYPE_NATIVE			
					handleArray[k].unbind();
				}		
			}
			
			if(dynamicBindingHandleArray.length > 0) {
				dynamicBindingHelper.unbindMultiple(dynamicBindingHandleArray);
			}
		}
	}
	
	specificBindingHelper.unbindOne = function(handle) {
		if(handle._type == CONSTANTS.HANDLE_TYPE_SPECIFIC) {
			var dynamicBindingHandle = specificBindingHelper_removeCallback(handle);
			if(dynamicBindingHandle) {
				dynamicBindingHelper.unbindOne(dynamicBindingHandle);
			}
		} else if(handle._type == CONSTANTS.HANDLE_TYPE_DYNAMIC) {
			dynamicBindingHelper.unbindOne(handle);
		} else {
			// assume the type is CONSTANTS.HANDLE_TYPE_NATIVE			
			handle.unbind();
		}
	}
	
	specificBindingHelper_removeCallback = function(handle) {
		var generalizedBindingInfo = specificBindingHelper.generalizedBindingStrMap[handle._generalizedBindingStr];
		if(generalizedBindingInfo) {
			var callbackArray = generalizedBindingInfo.specificBindingStrToCallbackMap[handle.bindingStr];
			if(callbackArray) {
				var index = array.indexOf(callbackArray, handle.callback);
				if(index != -1) {
					callbackArray.splice(index, 1);
					if(callbackArray.length == 0) {
						generalizedBindingInfo.specificBindingStrToCallbackMap[handle.bindingStr] = null;
						delete generalizedBindingInfo.specificBindingStrToCallbackMap[handle.bindingStr];
						
						if(commonHelper.isMapEmpty(generalizedBindingInfo.specificBindingStrToCallbackMap)) {
							specificBindingHelper.generalizedBindingStrMap[handle._generalizedBindingStr] = null;
							delete specificBindingHelper.generalizedBindingStrMap[handle._generalizedBindingStr];
							return generalizedBindingInfo.bindingHandler;
						}
					}
				}
			}
		}
		return null;
	}
	
	specificBindingHelper.generalizedBindingStrMap = {};
	
	return specificBindingHelper;
});

},
'dojo/_base/sniff':function(){
define(["./kernel", "../has"], function(dojo, has){
	// module:
	//		dojo/sniff
	// summary:
	//		This module populates the dojo browser version sniffing properties.

	if(!1){
		return has;
	}

	dojo.isBrowser = true,
	dojo._name = "browser";

	var hasAdd = has.add,
		n = navigator,
		dua = n.userAgent,
		dav = n.appVersion,
		tv = parseFloat(dav),
		isOpera,
		isAIR,
		isKhtml,
		isWebKit,
		isChrome,
		isMac,
		isSafari,
		isMozilla ,
		isMoz,
		isIE,
		isFF,
		isQuirks,
		isIos,
		isAndroid,
		isWii;

	/*=====
	dojo.isBrowser = {
		//	example:
		//	| if(dojo.isBrowser){ ... }
	};

	dojo.isFF = {
		//	example:
		//	| if(dojo.isFF > 1){ ... }
	};

	dojo.isIE = {
		// example:
		//	| if(dojo.isIE > 6){
		//	|		// we are IE7
		//	| }
	};

	dojo.isSafari = {
		//	example:
		//	| if(dojo.isSafari){ ... }
		//	example:
		//		Detect iPhone:
		//	| if(dojo.isSafari && navigator.userAgent.indexOf("iPhone") != -1){
		//	|		// we are iPhone. Note, iPod touch reports "iPod" above and fails this test.
		//	| }
	};

	dojo.mixin(dojo, {
		// isBrowser: Boolean
		//		True if the client is a web-browser
		isBrowser: true,
		//	isFF: Number | undefined
		//		Version as a Number if client is FireFox. undefined otherwise. Corresponds to
		//		major detected FireFox version (1.5, 2, 3, etc.)
		isFF: 2,
		//	isIE: Number | undefined
		//		Version as a Number if client is MSIE(PC). undefined otherwise. Corresponds to
		//		major detected IE version (6, 7, 8, etc.)
		isIE: 6,
		//	isKhtml: Number | undefined
		//		Version as a Number if client is a KHTML browser. undefined otherwise. Corresponds to major
		//		detected version.
		isKhtml: 0,
		//	isWebKit: Number | undefined
		//		Version as a Number if client is a WebKit-derived browser (Konqueror,
		//		Safari, Chrome, etc.). undefined otherwise.
		isWebKit: 0,
		//	isMozilla: Number | undefined
		//		Version as a Number if client is a Mozilla-based browser (Firefox,
		//		SeaMonkey). undefined otherwise. Corresponds to major detected version.
		isMozilla: 0,
		//	isOpera: Number | undefined
		//		Version as a Number if client is Opera. undefined otherwise. Corresponds to
		//		major detected version.
		isOpera: 0,
		//	isSafari: Number | undefined
		//		Version as a Number if client is Safari or iPhone. undefined otherwise.
		isSafari: 0,
		//	isChrome: Number | undefined
		//		Version as a Number if client is Chrome browser. undefined otherwise.
		isChrome: 0,
		//	isMac: Boolean
		//		True if the client runs on Mac
		isMac: 0,
		// isIos: Boolean
		//		True if client is iPhone, iPod, or iPad
		isIos: 0,
		// isAndroid: Number | undefined
		//		Version as a Number if client is android browser. undefined otherwise.
		isAndroid: 0,
		// isWii: Boolean
		//		True if client is Wii
		isWii: 0
	});
	=====*/

	// fill in the rendering support information in dojo.render.*
	if(dua.indexOf("AdobeAIR") >= 0){ isAIR = 1; }
	isKhtml = (dav.indexOf("Konqueror") >= 0) ? tv : 0;
	isWebKit = parseFloat(dua.split("WebKit/")[1]) || undefined;
	isChrome = parseFloat(dua.split("Chrome/")[1]) || undefined;
	isMac = dav.indexOf("Macintosh") >= 0;
	isIos = /iPhone|iPod|iPad/.test(dua);
	isAndroid = parseFloat(dua.split("Android ")[1]) || undefined;
	isWii = typeof opera != "undefined" && opera.wiiremote;

	// safari detection derived from:
	//		http://developer.apple.com/internet/safari/faq.html#anchor2
	//		http://developer.apple.com/internet/safari/uamatrix.html
	var index = Math.max(dav.indexOf("WebKit"), dav.indexOf("Safari"), 0);
	if(index && !isChrome){
		// try to grab the explicit Safari version first. If we don't get
		// one, look for less than 419.3 as the indication that we're on something
		// "Safari 2-ish".
		isSafari = parseFloat(dav.split("Version/")[1]);
		if(!isSafari || parseFloat(dav.substr(index + 7)) <= 419.3){
			isSafari = 2;
		}
	}

	if (!has("dojo-webkit")) {
		if(dua.indexOf("Opera") >= 0){
			isOpera = tv;
			// see http://dev.opera.com/articles/view/opera-ua-string-changes and http://www.useragentstring.com/pages/Opera/
			// 9.8 has both styles; <9.8, 9.9 only old style
			if(isOpera >= 9.8){
				isOpera = parseFloat(dua.split("Version/")[1]) || tv;
			}
		}

		if(dua.indexOf("Gecko") >= 0 && !isKhtml && !isWebKit){
			isMozilla = isMoz = tv;
		}
		if(isMoz){
			//We really need to get away from this. Consider a sane isGecko approach for the future.
			isFF = parseFloat(dua.split("Firefox/")[1] || dua.split("Minefield/")[1]) || undefined;
		}
		if(document.all && !isOpera){
			isIE = parseFloat(dav.split("MSIE ")[1]) || undefined;
			//In cases where the page has an HTTP header or META tag with
			//X-UA-Compatible, then it is in emulation mode.
			//Make sure isIE reflects the desired version.
			//document.documentMode of 5 means quirks mode.
			//Only switch the value if documentMode's major version
			//is different from isIE's major version.
			var mode = document.documentMode;
			if(mode && mode != 5 && Math.floor(isIE) != mode){
				isIE = mode;
			}
		}
	}

	isQuirks = document.compatMode == "BackCompat";

	hasAdd("opera", dojo.isOpera = isOpera);
	hasAdd("air", dojo.isAIR = isAIR);
	hasAdd("khtml", dojo.isKhtml = isKhtml);
	hasAdd("webkit", dojo.isWebKit = isWebKit);
	hasAdd("chrome", dojo.isChrome = isChrome);
	hasAdd("mac", dojo.isMac = isMac );
	hasAdd("safari", dojo.isSafari = isSafari);
	hasAdd("mozilla", dojo.isMozilla = dojo.isMoz = isMozilla );
	hasAdd("ie", dojo.isIE = isIE );
	hasAdd("ff", dojo.isFF = isFF);
	hasAdd("quirks", dojo.isQuirks = isQuirks);
	hasAdd("ios", dojo.isIos = isIos);
	hasAdd("android", dojo.isAndroid = isAndroid);

	dojo.locale = dojo.locale || (isIE ? n.userLanguage : n.language).toLowerCase();

	return has;
});

},
'dojo/_base/window':function(){
define(["./kernel", "../has", "./sniff"], function(dojo, has){
	// module:
	//		dojo/window
	// summary:
	//		This module provides an API to save/set/restore the global/document scope.

/*=====
dojo.doc = {
	// summary:
	//		Alias for the current document. 'dojo.doc' can be modified
	//		for temporary context shifting. Also see dojo.withDoc().
	// description:
	//		Refer to dojo.doc rather
	//		than referring to 'window.document' to ensure your code runs
	//		correctly in managed contexts.
	// example:
	//	|	n.appendChild(dojo.doc.createElement('div'));
}
=====*/
dojo.doc = this["document"] || null;

dojo.body = function(){
	// summary:
	//		Return the body element of the document
	//		return the body object associated with dojo.doc
	// example:
	//	|	dojo.body().appendChild(dojo.doc.createElement('div'));

	// Note: document.body is not defined for a strict xhtml document
	// Would like to memoize this, but dojo.doc can change vi dojo.withDoc().
	return dojo.doc.body || dojo.doc.getElementsByTagName("body")[0]; // Node
};

dojo.setContext = function(/*Object*/globalObject, /*DocumentElement*/globalDocument){
	// summary:
	//		changes the behavior of many core Dojo functions that deal with
	//		namespace and DOM lookup, changing them to work in a new global
	//		context (e.g., an iframe). The varibles dojo.global and dojo.doc
	//		are modified as a result of calling this function and the result of
	//		`dojo.body()` likewise differs.
	dojo.global = ret.global = globalObject;
	dojo.doc = ret.doc = globalDocument;
};

dojo.withGlobal = function(	/*Object*/globalObject,
							/*Function*/callback,
							/*Object?*/thisObject,
							/*Array?*/cbArguments){
	// summary:
	//		Invoke callback with globalObject as dojo.global and
	//		globalObject.document as dojo.doc.
	// description:
	//		Invoke callback with globalObject as dojo.global and
	//		globalObject.document as dojo.doc. If provided, globalObject
	//		will be executed in the context of object thisObject
	//		When callback() returns or throws an error, the dojo.global
	//		and dojo.doc will be restored to its previous state.

	var oldGlob = dojo.global;
	try{
		dojo.global = ret.global = globalObject;
		return dojo.withDoc.call(null, globalObject.document, callback, thisObject, cbArguments);
	}finally{
		dojo.global = ret.global = oldGlob;
	}
};

dojo.withDoc = function(	/*DocumentElement*/documentObject,
							/*Function*/callback,
							/*Object?*/thisObject,
							/*Array?*/cbArguments){
	// summary:
	//		Invoke callback with documentObject as dojo.doc.
	// description:
	//		Invoke callback with documentObject as dojo.doc. If provided,
	//		callback will be executed in the context of object thisObject
	//		When callback() returns or throws an error, the dojo.doc will
	//		be restored to its previous state.

	var oldDoc = dojo.doc,
		oldQ = dojo.isQuirks,
		oldIE = dojo.isIE, isIE, mode, pwin;

	try{
		dojo.doc = ret.doc = documentObject;
		// update dojo.isQuirks and the value of the has feature "quirks"
		dojo.isQuirks = has.add("quirks", dojo.doc.compatMode == "BackCompat", true, true); // no need to check for QuirksMode which was Opera 7 only

		if(has("ie")){
			if((pwin = documentObject.parentWindow) && pwin.navigator){
				// re-run IE detection logic and update dojo.isIE / has("ie")
				// (the only time parentWindow/navigator wouldn't exist is if we were not
				// passed an actual legitimate document object)
				isIE = parseFloat(pwin.navigator.appVersion.split("MSIE ")[1]) || undefined;
				mode = documentObject.documentMode;
				if(mode && mode != 5 && Math.floor(isIE) != mode){
					isIE = mode;
				}
				dojo.isIE = has.add("ie", isIE, true, true);
			}
		}

		if(thisObject && typeof callback == "string"){
			callback = thisObject[callback];
		}

		return callback.apply(thisObject, cbArguments || []);
	}finally{
		dojo.doc = ret.doc = oldDoc;
		dojo.isQuirks = has.add("quirks", oldQ, true, true);
		dojo.isIE = has.add("ie", oldIE, true, true);
	}
};

var ret = {
	global: dojo.global,
	doc: dojo.doc,
	body: dojo.body,
	setContext: dojo.setContext,
	withGlobal: dojo.withGlobal,
	withDoc: dojo.withDoc
};

return ret;

});

},
'com.ibm.bpm.coach/broadcastHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Mainly for the communication with Portal.

define(["dojo/_base/json", "./logger"], function(json, logger) {
				
	var broadcastHelper = new Object();
	
	/*
	 event:				the object to broadcast
	 target(optional):	If not provided, use "*" as targetOrigin and parent window (if exists) as target window.
						If provided, target object is expected to have property "window" and "origin".	 
	 
	 Implementation remark - use window.postMessage for now. Browser Hash is a candidate solution if postMessage doesn't satisfy the security requirement.
	*/
	broadcastHelper.broadcastMessage = function(event, target) {
		var targetWindow = broadcastHelper.parentWindow;
		var targetOrigin = "*";
		
		if(!!target) {
			if(!!target.window) {
				targetWindow = target.window; 
			}
			if(!!target.origin) {
				targetOrigin = target.origin; 
			}
		}
		
		if(!!targetWindow) {
			var serializedEvent = json.toJson(event);
			// work item 131966
			try {
				targetWindow.postMessage(serializedEvent, targetOrigin);
			} catch (e) {
				// postMessage() failed, use direct call onMessage instead with faked event. Container MUST be co-located
				logger.log("broadcastHelper", "broadcastMessage", "iframe postMessage() failed, call onMessage() directly - event=" + serializedEvent, logger.FINE);
				if (!!targetWindow.onMessage && typeof targetWindow.onMessage === 'function') {
					targetWindow.onMessage({origin: window.location.protocol + "//" + window.location.host,
											data: serializedEvent, 
											source: window});
				} else {
					logger.log("broadcastHelper", "broadcastMessage", "onMessage() does not exist", logger.FINE);				
				}
			}			
		}
	}
	
	broadcastHelper.parentWindow = (!!window.parent &&  window.parent != window)? window.parent : null;

	return broadcastHelper;
});

},
'dijit/popup':function(){
define([
	"dojo/_base/array", // array.forEach array.some
	"dojo/aspect",
	"dojo/_base/connect",	// connect._keypress
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-construct", // domConstruct.create domConstruct.destroy
	"dojo/dom-geometry", // domGeometry.isBodyLtr
	"dojo/dom-style", // domStyle.set
	"dojo/_base/event", // event.stop
	"dojo/has",
	"dojo/keys",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/_base/window", // win.body
	"./place",
	"./BackgroundIframe",
	"."	// dijit (defining dijit.popup to match API doc)
], function(array, aspect, connect, declare, dom, domAttr, domConstruct, domGeometry, domStyle, event, has, keys, lang, on, win,
			place, BackgroundIframe, dijit){

	// module:
	//		dijit/popup
	// summary:
	//		Used to show drop downs (ex: the select list of a ComboBox)
	//		or popups (ex: right-click context menus)


	/*=====
	dijit.popup.__OpenArgs = function(){
		// popup: Widget
		//		widget to display
		// parent: Widget
		//		the button etc. that is displaying this popup
		// around: DomNode
		//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
		// x: Integer
		//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		// y: Integer
		//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		// orient: Object|String
		//		When the around parameter is specified, orient should be a list of positions to try, ex:
		//	|	[ "below", "above" ]
		//		For backwards compatibility it can also be an (ordered) hash of tuples of the form
		//		(around-node-corner, popup-node-corner), ex:
		//	|	{ "BL": "TL", "TL": "BL" }
		//		where BL means "bottom left" and "TL" means "top left", etc.
		//
		//		dijit.popup.open() tries to position the popup according to each specified position, in order,
		//		until the popup appears fully within the viewport.
		//
		//		The default value is ["below", "above"]
		//
		//		When an (x,y) position is specified rather than an around node, orient is either
		//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
		//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
		//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
		//		and the top-right corner.
		// onCancel: Function
		//		callback when user has canceled the popup by
		//			1. hitting ESC or
		//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
		// onClose: Function
		//		callback whenever this popup is closed
		// onExecute: Function
		//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		// padding: dijit.__Position
		//		adding a buffer around the opening position. This is only useful when around is not set.
		this.popup = popup;
		this.parent = parent;
		this.around = around;
		this.x = x;
		this.y = y;
		this.orient = orient;
		this.onCancel = onCancel;
		this.onClose = onClose;
		this.onExecute = onExecute;
		this.padding = padding;
	}
	=====*/

	/*=====
	dijit.popup = {
		// summary:
		//		Used to show drop downs (ex: the select list of a ComboBox)
		//		or popups (ex: right-click context menus).
		//
		//		Access via require(["dijit/popup"], function(popup){ ... }).

		moveOffScreen: function(widget){
			// summary:
			//		Moves the popup widget off-screen.
			//		Do not use this method to hide popups when not in use, because
			//		that will create an accessibility issue: the offscreen popup is
			//		still in the tabbing order.
			// widget: dijit._WidgetBase
			//		The widget
		},

		hide: function(widget){
			// summary:
			//		Hide this popup widget (until it is ready to be shown).
			//		Initialization for widgets that will be used as popups
			//
			// 		Also puts widget inside a wrapper DIV (if not already in one)
			//
			//		If popup widget needs to layout it should
			//		do so when it is made visible, and popup._onShow() is called.
			// widget: dijit._WidgetBase
			//		The widget
		},

		open: function(args){
			// summary:
			//		Popup the widget at the specified position
			// example:
			//		opening at the mouse position
			//		|		popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
			// example:
			//		opening the widget as a dropdown
			//		|		popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
			//
			//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
			//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.
			// args: dijit.popup.__OpenArgs
			//		Parameters
			return {};	// Object specifying which position was chosen
		},

		close: function(popup){
			// summary:
			//		Close specified popup and any popups that it parented.
			//		If no popup is specified, closes all popups.
			// widget: dijit._WidgetBase?
			//		The widget, optional
		}
	};
	=====*/

	function destroyWrapper(){
		// summary:
		//		Function to destroy wrapper when popup widget is destroyed.
		//		Left in this scope to avoid memory leak on IE8 on refresh page, see #15206.
		if(this._popupWrapper){
			domConstruct.destroy(this._popupWrapper);
			delete this._popupWrapper;
		}
	}

	var PopupManager = declare(null, {
		// _stack: dijit._Widget[]
		//		Stack of currently popped up widgets.
		//		(someone opened _stack[0], and then it opened _stack[1], etc.)
		_stack: [],

		// _beginZIndex: Number
		//		Z-index of the first popup.   (If first popup opens other
		//		popups they get a higher z-index.)
		_beginZIndex: 1000,

		_idGen: 1,

		_createWrapper: function(/*Widget*/ widget){
			// summary:
			//		Initialization for widgets that will be used as popups.
			//		Puts widget inside a wrapper DIV (if not already in one),
			//		and returns pointer to that wrapper DIV.

			var wrapper = widget._popupWrapper,
				node = widget.domNode;

			if(!wrapper){
				// Create wrapper <div> for when this widget [in the future] will be used as a popup.
				// This is done early because of IE bugs where creating/moving DOM nodes causes focus
				// to go wonky, see tests/robot/Toolbar.html to reproduce
				wrapper = domConstruct.create("div", {
					"class":"dijitPopup",
					style:{ display: "none"},
					role: "presentation"
				}, win.body());
				wrapper.appendChild(node);

				var s = node.style;
				s.display = "";
				s.visibility = "";
				s.position = "";
				s.top = "0px";

				widget._popupWrapper = wrapper;
				aspect.after(widget, "destroy", destroyWrapper, true);
			}

			return wrapper;
		},

		moveOffScreen: function(/*Widget*/ widget){
			// summary:
			//		Moves the popup widget off-screen.
			//		Do not use this method to hide popups when not in use, because
			//		that will create an accessibility issue: the offscreen popup is
			//		still in the tabbing order.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			domStyle.set(wrapper, {
				visibility: "hidden",
				top: "-9999px",		// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
				display: ""
			});
		},

		hide: function(/*Widget*/ widget){
			// summary:
			//		Hide this popup widget (until it is ready to be shown).
			//		Initialization for widgets that will be used as popups
			//
			// 		Also puts widget inside a wrapper DIV (if not already in one)
			//
			//		If popup widget needs to layout it should
			//		do so when it is made visible, and popup._onShow() is called.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			domStyle.set(wrapper, "display", "none");
		},

		getTopPopup: function(){
			// summary:
			//		Compute the closest ancestor popup that's *not* a child of another popup.
			//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
			var stack = this._stack;
			for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
				/* do nothing, just trying to get right value for pi */
			}
			return stack[pi];
		},

		open: function(/*dijit.popup.__OpenArgs*/ args){
			// summary:
			//		Popup the widget at the specified position
			//
			// example:
			//		opening at the mouse position
			//		|		popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
			//
			// example:
			//		opening the widget as a dropdown
			//		|		popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
			//
			//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
			//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

			var stack = this._stack,
				widget = args.popup,
				orient = args.orient || ["below", "below-alt", "above", "above-alt"],
				ltr = args.parent ? args.parent.isLeftToRight() : domGeometry.isBodyLtr(),
				around = args.around,
				id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);

			// If we are opening a new popup that isn't a child of a currently opened popup, then
			// close currently opened popup(s).   This should happen automatically when the old popups
			// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
			while(stack.length && (!args.parent || !dom.isDescendant(args.parent.domNode, stack[stack.length-1].widget.domNode))){
				this.close(stack[stack.length-1].widget);
			}

			// Get pointer to popup wrapper, and create wrapper if it doesn't exist
			var wrapper = this._createWrapper(widget);


			domAttr.set(wrapper, {
				id: id,
				style: {
					zIndex: this._beginZIndex + stack.length
				},
				"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
				dijitPopupParent: args.parent ? args.parent.id : ""
			});

			if(has("bgIframe") && !widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _Widget.destroy()
				widget.bgIframe = new BackgroundIframe(wrapper);
			}

			// position the wrapper node and make it visible
			var best = around ?
				place.around(wrapper, around, orient, ltr, widget.orient ? lang.hitch(widget, "orient") : null) :
				place.at(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

			wrapper.style.display = "";
			wrapper.style.visibility = "visible";
			widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

			var handlers = [];

			// provide default escape and tab key handling
			// (this will work for any widget, not just menu)
			handlers.push(on(wrapper, connect._keypress, lang.hitch(this, function(evt){
				if(evt.charOrCode == keys.ESCAPE && args.onCancel){
					event.stop(evt);
					args.onCancel();
				}else if(evt.charOrCode === keys.TAB){
					event.stop(evt);
					var topPopup = this.getTopPopup();
					if(topPopup && topPopup.onCancel){
						topPopup.onCancel();
					}
				}
			})));

			// watch for cancel/execute events on the popup and notify the caller
			// (for a menu, "execute" means clicking an item)
			if(widget.onCancel && args.onCancel){
				handlers.push(widget.on("cancel", args.onCancel));
			}

			handlers.push(widget.on(widget.onExecute ? "execute" : "change", lang.hitch(this, function(){
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onExecute){
					topPopup.onExecute();
				}
			})));

			stack.push({
				widget: widget,
				parent: args.parent,
				onExecute: args.onExecute,
				onCancel: args.onCancel,
				onClose: args.onClose,
				handlers: handlers
			});

			if(widget.onOpen){
				// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
				widget.onOpen(best);
			}

			return best;
		},

		close: function(/*Widget?*/ popup){
			// summary:
			//		Close specified popup and any popups that it parented.
			//		If no popup is specified, closes all popups.

			var stack = this._stack;

			// Basically work backwards from the top of the stack closing popups
			// until we hit the specified popup, but IIRC there was some issue where closing
			// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
			// closing C might close B indirectly and then the while() condition will run where stack==[A]...
			// so the while condition is constructed defensively.
			while((popup && array.some(stack, function(elem){return elem.widget == popup;})) ||
				(!popup && stack.length)){
				var top = stack.pop(),
					widget = top.widget,
					onClose = top.onClose;

				if(widget.onClose){
					// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
					widget.onClose();
				}

				var h;
				while(h = top.handlers.pop()){ h.remove(); }

				// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
				if(widget && widget.domNode){
					this.hide(widget);
				}

				if(onClose){
					onClose();
				}
			}
		}
	});

	return (dijit.popup = new PopupManager());
});

},
'dojo/cldr/nls/en/islamic':function(){
define(
//begin v1.x content
{
	"dateFormatItem-yM": "M/y",
	"dateFormatItem-yyyyMMMEd": "EEE, MMM d, y G",
	"dateFormatItem-yQ": "Q y",
	"eraNames": [
		"AH"
	],
	"dateFormatItem-MMMEd": "E, MMM d",
	"dateFormatItem-hms": "h:mm:ss a",
	"dateFormatItem-yQQQ": "QQQ y",
	"dateFormatItem-MMM": "LLL",
	"months-standAlone-narrow": [
		"1",
		"2",
		"3",
		"4",
		"5",
		"6",
		"7",
		"8",
		"9",
		"10",
		"11",
		"12"
	],
	"timeFormat-full": "h:mm:ss a zzzz",
	"dateFormatItem-yyyy": "y G",
	"months-standAlone-abbr": [
		"Muh.",
		"Saf.",
		"Rab. I",
		"Rab. II",
		"Jum. I",
		"Jum. II",
		"Raj.",
		"Sha.",
		"Ram.",
		"Shaw.",
		"Dhuʻl-Q.",
		"Dhuʻl-H."
	],
	"dateFormatItem-yMMM": "MMM y",
	"days-standAlone-narrow": [
		"S",
		"M",
		"T",
		"W",
		"T",
		"F",
		"S"
	],
	"eraAbbr": [
		"AH"
	],
	"dateFormat-long": "MMMM d, y G",
	"timeFormat-medium": "h:mm:ss a",
	"dateFormatItem-EEEd": "d EEE",
	"dateFormatItem-Hm": "HH:mm",
	"dateFormat-medium": "MMM d, y G",
	"dateFormatItem-Hms": "HH:mm:ss",
	"dateFormatItem-ms": "mm:ss",
	"months-standAlone-wide": [
		"Muharram",
		"Safar",
		"Rabiʻ I",
		"Rabiʻ II",
		"Jumada I",
		"Jumada II",
		"Rajab",
		"Shaʻban",
		"Ramadan",
		"Shawwal",
		"Dhuʻl-Qiʻdah",
		"Dhuʻl-Hijjah"
	],
	"dateFormatItem-yyyyMEd": "EEE, M/d/y G",
	"dateFormatItem-MMMd": "MMM d",
	"timeFormat-long": "h:mm:ss a z",
	"months-format-abbr": [
		"Muh.",
		"Saf.",
		"Rab. I",
		"Rab. II",
		"Jum. I",
		"Jum. II",
		"Raj.",
		"Sha.",
		"Ram.",
		"Shaw.",
		"Dhuʻl-Q.",
		"Dhuʻl-H."
	],
	"timeFormat-short": "h:mm a",
	"days-format-abbr": [
		"Sun",
		"Mon",
		"Tue",
		"Wed",
		"Thu",
		"Fri",
		"Sat"
	],
	"dateFormatItem-M": "L",
	"dateFormatItem-yyyyQQQ": "QQQ y G",
	"dateFormatItem-MEd": "E, M/d",
	"months-format-narrow": [
		"1",
		"2",
		"3",
		"4",
		"5",
		"6",
		"7",
		"8",
		"9",
		"10",
		"11",
		"12"
	],
	"dateFormatItem-hm": "h:mm a",
	"dateFormat-short": "M/d/yy G",
	"dateFormatItem-yyyyM": "M/y G",
	"dateFormatItem-yMMMEd": "EEE, MMM d, y",
	"dateFormat-full": "EEEE, MMMM d, y G",
	"dateFormatItem-Md": "M/d",
	"dateFormatItem-yyyyQ": "Q y G",
	"dateFormatItem-yMEd": "EEE, M/d/y",
	"months-format-wide": [
		"Muharram",
		"Safar",
		"Rabiʻ I",
		"Rabiʻ II",
		"Jumada I",
		"Jumada II",
		"Rajab",
		"Shaʻban",
		"Ramadan",
		"Shawwal",
		"Dhuʻl-Qiʻdah",
		"Dhuʻl-Hijjah"
	],
	"dateFormatItem-yyyyMMM": "MMM y G",
	"dateFormatItem-d": "d",
	"quarters-format-wide": [
		"1st quarter",
		"2nd quarter",
		"3rd quarter",
		"4th quarter"
	],
	"eraNarrow": [
		"AH"
	],
	"days-format-wide": [
		"Sunday",
		"Monday",
		"Tuesday",
		"Wednesday",
		"Thursday",
		"Friday",
		"Saturday"
	]
}
//end v1.x content
);
},
'com.ibm.bpm.coach/controller':function(){
/*
Licensed Materials - Property of IBM
5725-C95
Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
 
// controller main js
define(["require", "./Stateful", "./engine", "dojo/_base/xhr", "dojo/_base/json", "dojo/_base/lang", "dojo/_base/connect", "dojo/_base/kernel", "dojo/dom", "dojo/has", "./collaboration", "./BPM_CONSTANTS", "./CONSTANTS", "./boundaryEventHelper", "./broadcastHelper", "./logger", "dojo/_base/config"], function(require, stateful, engine, xhr, json, lang, connect, kernel, dom, has, p_collaboration, BPM_CONSTANTS, CONSTANTS, boundaryEventHelper, broadcastHelper, logger, dojoConfig) {
	// controller start
	var controller = new Object();

	controller.inCollaboration = false;
    controller.readOnly = false;
    controller.destroyed = false;
	controller.notificationQueue = [];
	controller.notificationTimeoutHandler = null;
	controller.setAsEditorTimeoutHandler = null;

	controller.init = function() {
		//This function is called in the main routine of coach. 
		//Catch all the exceptions here to give views a chance to render themselves. 
		//Then customer at least won't get a blank page just because this init() fails.
		try {
		    controller.initNotificationBus();
		    controller.initFocus();
		    controller.subscribeToEvents();
		    controller.initProgressIcon();
		    controller.initCustomCollaborations();
		    controller.initBoundaryEvents();
            controller.initValidationErrors();
	    } catch(ex) {
	    	logger.log("controller", "init", "Exception initializing: " + ex, logger.INFO);
	    }
	}

	controller.initProgressIcon = function() {
		//initializes the icon that spins as boundary events are being processed
	    var img = document.createElement('img');
	    img.src = 'images/common/loading.gif';
        require(["./engine"], function(engine) {
		    img.alt = engine.localizedMessages.coachNG_loading;
        });	    
	    
	    controller.progressIcon = document.createElement('div');
	    controller.progressIcon.style.position = 'fixed';
	    controller.progressIcon.style.top = '10px';
	    controller.progressIcon.style.right = '10px';
	    controller.progressIcon.visibility = "hidden";
	    controller.progressIcon.style.display = "none";
	    controller.progressIcon.appendChild(img);
	    document.body.appendChild(controller.progressIcon);
	}

	controller.initCustomCollaborations = function() {
		controller.customCollabEvents = {};
	}

	controller.initBoundaryEvents = function() {
		controller.boundayEventQueue = [];
		controller.boundayEventInProgress = false;		
	}

    controller.initValidationErrors = function() {
        controller.validationErrors = null;
    }

	controller.showProgressIcon = function() {
		//shows the spinner and sets the progress flag
	    controller.progressIcon.style.display = "block";
	    controller.progressIcon.style.visibility = "visible";
	    
	    controller.boundayEventInProgress = true;
	}

	controller.hideProgressIcon = function() {
		//hides the spinner and sets the progress flag
	    controller.progressIcon.style.display = "none";
	    controller.progressIcon.style.visibility = "hidden";
	    
	    controller.boundayEventInProgress = false;
	}
		
	controller.clearBoundaryEventQueue = function(requestedByView) {
		if(!!controller.boundayEventQueue) {
			for(var k=0; k<controller.boundayEventQueue.length; k++) {
				var callbackFun = controller.boundayEventQueue[k].callback;
				var alwaysInvokeCallback = controller.boundayEventQueue[k].alwaysInvokeCallback;
				if(!requestedByView) {
					boundaryEventHelper.callbackForBoundaryEvent(callbackFun, alwaysInvokeCallback, BPM_CONSTANTS.BOUNDARY_EVENT_STATUS_ABANDONED);
				} else {
					boundaryEventHelper.callbackForBoundaryEvent(callbackFun, alwaysInvokeCallback, BPM_CONSTANTS.BOUNDARY_EVENT_STATUS_CANCELLED);
				}
			}
		}
		controller.boundayEventQueue = [];
	}	

	controller.initNotificationBus = function() {
		var _this = this;
		require(["./engine"], function(engine) {
			if(engine.GlobalAssets.systemData.zTaskId&&engine.GlobalAssets.systemData.zComponentId){
				controller.sessionId = {task_id: engine.GlobalAssets.systemData.zTaskId, coach_session_id: engine.GlobalAssets.systemData.zComponentId};
				//check to see if this coach is launched as part of a task
				var taskID = controller.sessionId.task_id.split("t")[1];
				if(taskID){
					// joining session with the given session id, if session does not exist, it is being initialized
					logger.log("controller", "initNotificationBus", "Initializing Bus with session=" + json.toJson(controller.sessionId) +
							   ", owner=" + engine.GlobalAssets.systemData.taskOwnerUserName + ", task=" + taskID, logger.FINE);
					var roster = p_collaboration.RegisterForCollaboration(controller.sessionId,
																  engine.GlobalAssets.systemData.taskOwnerUserName,
																  // fullSynchReceivedCallbackMethod
																  lang.hitch(_this, "handleFullSync"),
																  // publishFullSynchCallbackMethod
																  lang.hitch(_this, "publishFullSync"),
																  // incomingChangesetCallbackMethod
																  lang.hitch(_this, "handleBusNotifications"),
																  // rosterChangeCallbackMethod
																  lang.hitch(_this, "handleRosterChange"),
																  lang.hitch(_this,"setToReadOnly"));
					controller.handleRosterChange(roster);
				}
			}
		});
	}

	controller.publishFullSync = function() {
		//called when a new collaboration user joins
        var result = null;
        require(["./engine"], function(engine) {
            controller.inCollaboration = true;
            
            result = "{\"pendingEvents\":" + controller.getPendingCustomCollabEvents() + "," +
            "\"data\":" + controller.localDataToJson() + "," +
            "\"errors\":" + json.toJson(controller.validationErrors) + "}";
        });
        
		logger.log("controller", "publishFullSync", "Sending JSON: " + result, logger.FINER);
        return result;
	}

	controller.handleFullSync = function(data) {
		//called to receive the current coach data when we join the collaboration
		require(["./engine"], function(engine) {
		    // we assume that we are in collaboration if we are handling full sync
		    controller.inCollaboration = true;
			logger.log("controller", "handleFullSync", "Received JSON: " + data, logger.FINER);
		    var dataObject = json.fromJson(data);
		    var baseObject = dataObject.data;
		    var pendingEvents = dataObject.pendingEvents;
            var validationErrors = dataObject.errors;
            // handling validation errors
            if (!!validationErrors) {
                engine.clearValidationEvent();
                engine.handleValidationEvent(validationErrors);
            }
		    // setting new properties and refreshing views at the same time
		    for (var x in baseObject) {
                if (!!baseObject[x] && !!baseObject[x].items && !!baseObject[x].listAllSelectedIndices && Object.prototype.toString.call(baseObject[x].items) === '[object Array]') {
                    engine.GlobalAssets.local.set(x, baseObject[x], true);
                    engine.GlobalAssets.local.get(x).set("listAllSelectedIndices", baseObject[x].listAllSelectedIndices, true);
                } else {
                    if (!!baseObject[x]) {
                        controller.adjustDateProperties(baseObject[x]);
                    }
                    engine.GlobalAssets.local.set(x, baseObject[x], true);
                }
		    }
		    if (!!pendingEvents) {
			    for (var y=0; y<pendingEvents.length; y++) {
			    	if(!!pendingEvents[y]) {
			    		controller.processNotification(pendingEvents[y]);
			    	}
			    }		    	
		    }
		});    
	}

	controller.handleBusNotifications = function(data) {
		//called by the notification bus for all activity from the bus to the controller
        require(["./engine"], function(engine) {
	        var event = json.fromJson(data.data);
	        controller.processNotification(event);
        });
	}
	
	controller.setToReadOnly = function(){
		//called by the notification bus to force coach view to read only when the same user is joined collaboration somewhere else
        require(["./engine"], function(engine) {
            controller.readOnly = true;
            engine.setReadonly(true);
        });
	}
	
	controller._delaySetAsEditor = function() {
		// Need to check if there is one already exist, if yes, then don't do another one.
		if (controller.setAsEditorTimeoutHandler == null) {
			controller.setAsEditorTimeoutHandler = setTimeout(function() {
				controller.setAsEditorTimeoutHandler = null;
				if (controller.notificationTimeoutHandler == null) {
					require(["./engine"], function(engine) {
						engine.setReadonly(false);
					});
				} else {
					controller._delaySetAsEditor();
				}
			}, 1000);
		}
	}

	controller.handleRosterChange = function(sessionRoster) {
		//called by the bus when a user adds/leaves the collab
        require(["./engine"], function(engine) {
            var username = engine.GlobalAssets.system.user_loginName;
            if (sessionRoster && sessionRoster.allParticipants) {
    			logger.log("controller", "handleRosterChange", "sessionRoster: " + json.toJson(sessionRoster), logger.FINE);
                if (!!sessionRoster.editorID) {
                    controller.readOnly = sessionRoster.editorID != username;
					if (controller.readOnly) {
						// Need to make sure any unexecuted previously set as editor timout handler does not execute.
						if (controller.setAsEditorTimeoutHandler != null) {
							clearTimeout(controller.setAsEditorTimeoutHandler);
							controller.setAsEditorTimeoutHandler = null;
						}
						engine.setReadonly(controller.readOnly);
					} else {
						// make sure all previous notification has been processed before setting to editor
						if (controller.notificationTimeoutHandler != null) {
							controller._delaySetAsEditor();
						} else {
							engine.setReadonly(controller.readOnly);
						}
					}
                }
                var rosterSize = 0;
                var currentUserInCollab = false;
                for (var i in sessionRoster.allParticipants){
                    rosterSize++;
                    if (sessionRoster.allParticipants[i].fullName == engine.GlobalAssets.system.user_fullName) {
                        currentUserInCollab = true;
                    }
                }

                if (rosterSize > 1 && currentUserInCollab) {
                    controller.inCollaboration = true;
                } else {
                    controller.inCollaboration = false;
                }
            } else {
                controller.inCollaboration = false;
            }
        });
	}

	controller._adjustCollaborationURL = function(nextURL) {
		var tempParser = document.createElement('a');
		tempParser.href = nextURL;
		var tempStr = tempParser.protocol + '//' + tempParser.host;
		if (nextURL.indexOf(tempStr) > -1) {
			nextURL = nextURL.substring(tempStr.length);
		}
		return nextURL;
	}

	controller.processBoundaryEvent = function(boundaryEvent) {
		//processes the boundary event that was triggered by another collaboration user
		require(["./engine"], function(engine) {
			logger.log("controller", "processBoundaryEvent", "Processing boundary event: " + json.toJson(boundaryEvent), logger.FINE);
		    var event = boundaryEvent.event;
            if (event == 'taskCompleted') {
                engine.TaskCompleted(controller._adjustCollaborationURL(boundaryEvent.url), boundaryEvent.taskId);
            }
            if (event == 'taskPostponed') {
                engine.TaskCompleted(controller._adjustCollaborationURL(boundaryEvent.url));
            }
            if (event == 'taskClosed') {
                engine.TaskCompletedAlreadyClosed(controller._adjustCollaborationURL(boundaryEvent.url));
            }
            if (event == 'taskNotOwned') {
                engine.TaskCompletedNotOwned(controller._adjustCollaborationURL(boundaryEvent.url));
            }
            if (event == 'taskStateChanged') {
                // we need to indicate that we are not leaving collaboration session
                p_collaboration.setSkipUserLeavePublish(true);
                p_collaboration.setUnsubscribeAll(true);

                // rtc-67838. we need to force collaborating user to get the right coach with correct execution context
				var url = dojoConfig.App._bpmContextRootMap.teamworks + "/process.lsw?zWorkflowState=1&zTaskId=" +
                          engine.GlobalAssets.systemData.zTaskId + "&zResetContext=true";
                engine.StateChange(url);
            }
            if (event == 'taskDataChanged') {
                var baseObject = json.fromJson(boundaryEvent.data);
                // setting new properties and refreshing views at the same time
                if (!!engine.GlobalAssets.local.set) {
                    for (var x in baseObject) {
                        engine.GlobalAssets.local.set(x, baseObject[x], true);
                    }
                }
                // clearing validation events if any
                engine.clearValidationEvent();
            }
            if (event == 'taskDebug') {
                engine.handleDebugMode(controller._adjustCollaborationURL(boundaryEvent.url));
            }
            if (event == 'taskError') {
                engine.ErrorBoundaryEvent(boundaryEvent.errorMessage);
            }
            if (event == 'dataInputValidation') {
                engine.clearValidationEvent();
                engine.handleValidationEvent(boundaryEvent.validationErrors);
            }
		});
	}

	controller.subscribeToEvents = function() {
		//boundary
		connect.subscribe("com.ibm.bpm.coach.BoundaryEventTriggered", function(obj) {
	        controller.handleBoundaryEvent(obj.boundaryEventInfo, obj.validateBoundaryEventInfo, obj.callback, obj.alwaysInvokeCallback, true);
	    });
	}

	controller.publishNotification = function(notificationObject) {
		//send a message on the bus
	    if (controller.inCollaboration) {
	    	p_collaboration.publishData(json.toJson(notificationObject));
	    }
	}

	controller.handleBoundaryEvent = function(boundaryEventInfo, validateBoundaryEventInfo, callback, alwaysInvokeCallback, broadcast) {
		//called to handle the boundary event that the user triggers.  Adds the event to a queue for processing
		var boundaryEventInfo = {boundaryEventInfo: boundaryEventInfo, validateBoundaryEventInfo: validateBoundaryEventInfo,
                                callback: callback, alwaysInvokeCallback: alwaysInvokeCallback, broadcast: broadcast};
		controller.boundayEventQueue.push(boundaryEventInfo);
		controller.sendBoundaryEvent();
	}

	controller.sendBoundaryEvent = function() {
		//sends the boundary event (including all local data) to the server
		require(["./engine"], function(engine) {
			//only send the event if there is no other boundary event in progress
			if (controller.boundayEventInProgress == false && controller.boundayEventQueue.length > 0) {
			    controller.showProgressIcon();
				logger.log("controller", "sendBoundaryEvent", "Boundary event queue length: " + controller.boundayEventQueue.length, logger.FINER);
			    var event = controller.boundayEventQueue.shift();
			    var boundaryEventId = event.boundaryEventInfo.boundaryEventId;
			    var boundaryEventEndStateId = event.boundaryEventInfo.boundaryEventEndStateId;
                var fireValidation = event.boundaryEventInfo.fireValidation;
                var validateBoundaryEventSection = "";
                if (!!event.validateBoundaryEventInfo) {
                    validateBoundaryEventSection = "\"validateBoundaryEventInfo\":" + json.toJson(event.validateBoundaryEventInfo) + ",";
                }
			    var callback = event.callback;
			    var alwaysInvokeCallback = event.alwaysInvokeCallback;
			    var broadcast = event.broadcast;
				logger.log("controller", "sendBoundaryEvent", "Sending boundary event: " + event.boundaryEventInfo.boundaryEventId + ", " +
                                                              event.boundaryEventInfo.boundaryEventEndStateId, logger.FINE);
			    var localData = controller.localDataToJson();
				var coachData = "{\"boundaryEventInfo\":{" +
			                    "\"boundaryEventId\":\"" + boundaryEventId + "\"," +
			                    "\"boundaryEventEndStateId\":\"" + boundaryEventEndStateId + "\"," +
                                "\"fireValidation\":\"" + fireValidation + "\"}," +
                                validateBoundaryEventSection +
			                    "\"stateInfo\":{" +
			                    "\"zComponentId\":\"" + engine.GlobalAssets.systemData.zComponentId + "\"," +
			                    "\"zComponentName\":\"CoachNG\"," +
			                    "\"zTaskId\":\"" + engine.GlobalAssets.systemData.zTaskId + "\"," +
			                    "\"zWorkflowState\":\"" + engine.GlobalAssets.systemData.zWorkflowState + "\"," +
			                    "\"applicationId\":\"" + engine.GlobalAssets.systemData.applicationId + "\"," +
			                    "\"applicationInstanceId\":\"" + engine.GlobalAssets.systemData.applicationInstanceId + "\"," +
			                    "\"zDbg\":\"" + engine.GlobalAssets.systemData.zDbg + "\"}," +
			                    "\"data\":{\"local\":" + localData + "}}";
			    var xhrArgs = {
			        url: "ajaxCoach",
			        handleAs: "text",
			        headers: { 
			            "Content-Type":"application/json; charset=utf-8" 
			        },
			        postData: coachData,
			        load: function(data) {
						logger.log("controller", "sendBoundaryEvent", "Boundary event response: " + data, logger.FINER);
	                    // check for session timeout
	                    var sessionExpired = controller.checkSessionTimeOut(data);
	                    if (!sessionExpired) {
	                        // convert the response, because handleAs is text
	                        var response = json.fromJson(data);
		                    controller.hideProgressIcon();
		                    
		                    boundaryEventHelper.callbackForBoundaryEvent(callback, alwaysInvokeCallback, BPM_CONSTANTS.BOUNDARY_EVENT_STATUS_EXECUTED);

	                        // process response
	                        controller.processServerResponse(response, fireValidation);
	                        
	                        if (!!broadcast) {
	                            //get path from id
				                var boundaryEventPath = "";

	                        	if (!!engine.GlobalAssets.boundaryEventData)
	                        	{
	                        		for (var z in engine.GlobalAssets.boundaryEventData) {
	                        			if (engine.GlobalAssets.boundaryEventData[z] == boundaryEventId) {
	                        				boundaryEventPath = z;
	                        				break;
	                        			}
	                        		}
	                        	}
	                        	var event = {name: "notifyBoundaryEvent", path: boundaryEventPath, result: "success", coach: document.title, nextUrl: response.url};
	                        	broadcastHelper.broadcastMessage(event, engine.boundaryEventBroadcastTarget);
	                        } 
	                    } else {
	                    	//TODO: call a harmless rest call to trigger an auth challenge if session expired, and clear the queue	                    	
	                    }
			        },
			        error: function(error) {
			            controller.hideProgressIcon();
			            controller.clearBoundaryEventQueue(false);
			            logger.log("controller", "sendBoundaryEvent", "An unexpected error occurred: " + error, logger.INFO);
			            engine.ErrorBoundaryEvent(error);
			        }
			    }
			    
			    xhr.post(xhrArgs);				
			} else {
				logger.log("controller", "sendBoundaryEvent", "Boundary event already in progress, or nothing in the queue, waiting...", logger.FINER);
			}
	    });
	}

    controller.getPendingCustomCollabEvents = function () {
    	//Returns the json serialization for any pending custom collab and focus events.  This is useful in a late joiner case to make sure the 
    	//navigational state of tabs and tables is correct on the late joiner coach
    	var result = new Array(); 
        for (var viewId in controller.customCollabEvents) {
        	if (!!viewId) {
        		var event = controller.customCollabEvents[viewId];
        		if (!!event) {
        			result.push(event);
        		}
        	}
        }
        if (!!controller.currentUserFocusEvent) {
        	result.push(controller.currentUserFocusEvent);
        }
        return json.toJson(result);
    }
   
    controller.processCustomCollaborationEvent = function(event) {
		//called when notification event comes in from the bus where type = 'custom'
		require(["./engine"], function(engine) {
			engine.handleCustomCollaborationEvent(event);
		});
	}
	
    controller.checkSessionTimeOut = function(data) {
        var expired = false;
        require(["./engine"], function(engine) {
            // there is no way to intercept HTTP 302 response by means of dojo framework,
            // redirect happens completely transparent and we get contents of login page with HTTP 200 response
            if (data.indexOf("form method=\"POST\" action=\"j_security_check\"") != -1) {
                expired = true;
				logger.log("controller", "checkSessionTimeOut", "session has expired", logger.INFO);
                var msg = engine.localizedMessages.coachNG_session_expired;
                engine.ErrorBoundaryEvent(msg);
            }
        });
        return expired;
    }

    controller.processServerResponse = function(response, fireValidation) {
    	//processes the response of the boundary event from the server
        require(["./engine"], function(engine) {
            var notificationEvent;
            var event = response.event;
            var system = engine.GlobalAssets.system;
            if (event == 'taskCompleted') {
				logger.log("controller", "processServerResponse", "task is completed", logger.FINE);
                var nextUrl = response.url;
                var nextTaskId;
                if (!!response.taskId) {
                    nextTaskId = response.taskId;
                }
                notificationEvent = {type: 'boundary', event: 'taskCompleted', url: nextUrl, taskId: nextTaskId, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.TaskCompleted(nextUrl, nextTaskId);
            }
            if (event == 'taskPostponed') {
				logger.log("controller", "processServerResponse", "task is postponed", logger.FINE);
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskPostponed', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.TaskCompleted(nextUrl);
            }
            if (event == 'taskClosed') {
				logger.log("controller", "processServerResponse", "task is completed and already closed", logger.FINE);
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskClosed', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.TaskCompletedAlreadyClosed(nextUrl);
            }
            if (event == 'taskNotOwned') {
				logger.log("controller", "processServerResponse", "task is completed and not owned by the current user", logger.FINE);
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskNotOwned', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.TaskCompletedNotOwned(nextUrl);
            }
            if (event == 'taskStateChanged') {
				logger.log("controller", "processServerResponse", "task state is changed", logger.FINE);
                // we need to indicate that we are not leaving collaboration session
                if (controller.inCollaboration) {
                    p_collaboration.setSkipUserLeavePublish(true);
                    p_collaboration.setUnsubscribeAll(true);
                }
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskStateChanged', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.StateChange(nextUrl);
            }
            if (event == 'taskDataChanged') {
				logger.log("controller", "processServerResponse", "task data is changed", logger.FINE);
                //clear validation only if fireValidation is on
                if (fireValidation == 1){
                	engine.clearValidationEvent();
                }
                var delta = response.delta;
                controller.processDataDeltas(delta);

                var data = controller.localDataToJson();
                notificationEvent = {type: 'boundary', event: 'taskDataChanged', data: data, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                
                //we can now process the next boundary event that is queued
                controller.sendBoundaryEvent();
            }
            if (event == 'taskDebug') {
				logger.log("controller", "processServerResponse", "debug response is received", logger.FINE);
                var nextUrl = response.url;
                notificationEvent = {type: 'boundary', event: 'taskDebug', url: nextUrl, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.handleDebugMode(nextUrl);
            }
            if (event == 'taskError') {
                var message = response.errorMessage;
				logger.log("controller", "processServerResponse", "error has occured: " + message, logger.FINE);
                notificationEvent = {type: 'boundary', event: 'taskError', errorMessage: message, userName: system.user_fullName};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                engine.ErrorBoundaryEvent(message);
            }
            if (event == 'dataInputValidation') {
                logger.log("controller", "processServerResponse", "data input validation response is received", logger.FINE);
                notificationEvent = {type: 'boundary', event: 'dataInputValidation', validationErrors: response.validationErrors};
                // sending boundary event notification
                controller.publishNotification(notificationEvent);
                // we need to keep track of validation errors for late joiner case
                controller.validationErrors = response.validationErrors;
                //clear views that have previous validation errors decorations before handling new validation errors
				engine.clearValidationEvent();
                engine.handleValidationEvent(response.validationErrors);
            }
        });
    }

    controller.localDataToJson = function() {
    	//returns the serialized local obejct (all local data)
        var result = "{}";
        require(["./engine"], function(engine) {
            // check for uninitialized data
            if (!!engine.GlobalAssets.local.toJson) {
                result = engine.GlobalAssets.local.toJson();
            }
        });
        return result;
    }

    controller.processDataDeltas = function(delta) {
    	//called when a boundary event is a data change (stay on same coach).  The response is the delta data that needs to be applied.
        var allPathsValid = true;
        require(["./engine"], function(engine) {
			logger.log("controller", "processDataDeltas", "processing deltas", logger.FINE);
            // check for uninitialized local data
            if (!!engine.GlobalAssets.local.set) {
                var restoreCollaboration = controller.inCollaboration;
                // stop sending collaboration events
                if (restoreCollaboration) {
                    controller.inCollaboration = false;
                }
                for (var x in delta) {
                    var pos = x.indexOf("tw.local.");
                    if (pos == 0) {
                        // stripping off 'tw.local.'
                        var property = x.substring(9);
                        // check if given property exists
                        if ((engine.GlobalAssets.local.get(property)) !== undefined) {
                            if (Object.prototype.toString.call(delta[x]) === '[object Array]') {
                                // retrieving the correct list property
                                var list = engine.GlobalAssets.local.get(property);
                                if (!!list) {
                                    engine.GlobalAssets.local.set(property, delta[x], true);
                                    engine.GlobalAssets.local.get(property).set("listAllSelectedIndices", [], true);
                                }
                            } else {
                                // rtc-90015 date strings must be converted into date objects
                                // in order to avoid inconsistencies in stateful objects and
                                // prevent issues in date/time pickers
                                controller.adjustDateProperties(delta[x]);
                                engine.GlobalAssets.local.set(property, delta[x], true);
                            }
                        } else {
                        	allPathsValid = false;
                        }
                    } else {
                    	allPathsValid = false;
                    }
                    
                }
                // restoring sending collaboration events if required
                if (restoreCollaboration) {
                    controller.inCollaboration = true;
                }
            } else {
            	allPathsValid = false;
            }
        });
        return allPathsValid;
    }

    controller.adjustDateProperties = function(value) {
        for (var x in value) {
            if (value.hasOwnProperty(x) && !!value[x]) {
                if (Object.prototype.toString.call(value[x]) === '[object Array]') {
                    controller.adjustDateProperties(value[x]);
                } else if (/[0-9]{4}(\-[0-9]{2}){2}T([0-9]{2}\:){2}[0-9]{2}/.test("" + value[x])) {
                    // converting date string to Date object
                    var datetime = value[x].split("T");
                    var date = datetime[0].split("-");
                    var time = datetime[1].split(":");
                    var epochValue = Date.UTC(parseInt(date[0]), parseInt(date[1],10)-1, parseInt(date[2],10),
                        parseInt(time[0],10), parseInt(time[1],10), parseInt(time[2],10));
                    value[x] = new Date(epochValue);
                } else if (typeof value[x] !== "string") {
                    // skip simple strings and check complex objects
                    controller.adjustDateProperties(value[x]);
                }
            }
        }
    }


	controller.processNotification = function(event) {
		controller.notificationQueue.push(event);
		if (controller.notificationTimeoutHandler == null) {
			// Do a 1 sec delay here to allow any previous collaboration to be shown for at least 1 sec
			controller.notificationTimeoutHandler = setTimeout(function() {
				controller._processNotification(controller.notificationQueue.shift());
			}, 1000);		
		}
	}
	
	controller._processNotification = function(event) {
		controller.notificationTimeoutHandler = null;
		if (controller.notificationQueue.length > 0) {
			// Do a 1 sec delay here to allow any previous collaboration to be shown for at least 1 sec
			controller.notificationTimeoutHandler = setTimeout(function() {
				controller._processNotification(controller.notificationQueue.shift());
			}, 1000);
		}
		//router for notification events from the bus
		if (event.type && controller.destroyed == false) {
            if (event.type == 'focus') {
                controller.processFocusEvent(event);
            } else if (event.type == 'data') {
                controller.processDataEvent(event);
            } else if (event.type == 'boundary') {
                controller.processBoundaryEvent(event);
            } else if (event.type == 'custom') {
                controller.processCustomCollaborationEvent(event);
            } else {
    			logger.log("controller", "processNotification", "could not process notification event: " + json.toJson(event), logger.FINE);
            }
		} else {
			logger.log("controller", "processNotification", "could not process notification event: " + json.toJson(event), logger.FINE);
		}
	}
	
	controller.initFocus = function() {
		controller.setFocusHandlers();
		
		//init data structures to handle focus collaboration
		controller.currentUserFocus = {};
		controller.previousUserFocus = {};
		controller.focusTextColor = "#ffffff";
		controller.colorList = ["#07e5e2", "#ec8705", "#59a043", "#1b75bc", "#8d0ee6", "#0818f1", "#e5c307", "#e50707", "#cd462f"];

		controller.userColorMap = {};
		controller.userFocusMap = {};
		controller.focusElementToUserMap = {};
		controller.isCurrentUserAnEditor = true;

		controller.focusTimestamp = new Date().getTime();
	}


	controller.handleFocusEvent = function (event) {
		require(["./engine"], function(engine) {
			if ((engine._isCoachUnloaded)) {
				// coach is already unloaded. no need to continue
				return;
			}

			//finds the id of the focus target and the id of the view it belongs to 
			//and passes the ids and focus event to the controller
	
			//IE
			if (!event.target) {
				event.target = event.srcElement;
			}
			
			var id = event.target.id;
			var viewId = null;
			var parent = event.target.parentNode;
			while (parent) {
				if (parent.className && parent.className.indexOf(engine.CONSTANTS.COACH_VIEW) != -1) {
					viewId = parent.id;
					break;
				}
				parent = parent.parentNode;
			}
			if (!id) {
				id = viewId;
			}
			var viewIdMetaData = engine.getViewIdMetaData(viewId);
			var currentFocusTimeStamp = new Date().getTime();
			e = {'type': 'focus', 'domId': id, 'viewId': viewId, 'userName': engine.GlobalAssets.system.user_fullName, 'viewIdMetaData': viewIdMetaData};
			controller.currentUserFocusEvent = e;
			
			//only change the previous focus if the coach stays on the field for some time
			//helps in the case where a user quickly tabs several fields after making a data change
			if ( (currentFocusTimeStamp - controller.focusTimestamp) > 600) {
				controller.previousUserFocus.domId = controller.currentUserFocus.domId;
				controller.previousUserFocus.viewId = controller.currentUserFocus.viewId;
				controller.previousUserFocus.viewIdMetaData = controller.currentUserFocus.viewIdMetaData;
			}
			
			controller.currentUserFocus.domId = id;
			controller.currentUserFocus.viewId = viewId;
			controller.currentUserFocus.viewIdMetaData = viewIdMetaData;
			controller.focusTimestamp = currentFocusTimeStamp;
			
			// publishing focus notification
		    controller.publishNotification(e);
		    
		    //clean up any highlights left by the previous editor
		    if (controller.isCurrentUserAnEditor == false) {
		    	for (var x in controller.userFocusMap) {
		    		if (!!x && !!controller.userFocusMap[x]) {
		    			var userFocus = controller.userFocusMap[x];
		    			engine.handleBlurEvent({viewId: userFocus.viewId, domId: userFocus.domId, viewIdMetaData: userFocus.viewIdMetaData});
		    		}
		    	}
		    	controller.isCurrentUserAnEditor = true;
		    }
		});
	}	

	controller.customCollaborationHandler = function(viewId, payload) {
		//creates a new event: type='custom', viewId = viewId and payload = payload
		//publishes the event on the notification bus
		require(["./engine"], function(engine) {
			var viewIdMeta = engine.getViewIdMetaData(viewId);
			var e = {'type': 'custom', 'viewId': viewId, 'payload': payload, 'viewIdMetaData': viewIdMeta};
			//keep track of custom collab events (per view) in case we need to send them later to a late joiner
			controller.customCollabEvents[viewId] = e;
			controller.publishNotification(e);
		});
	}
	
	controller.mouseOnHandler = function (event) {
		//detects a hover event.  Need to check if the current domId+viewId has any users who are focused on it.
		//If so, notify the framework to show a bubble on the element.
		//Note: this code no longer applies since the collab always has a shimer in front of the coach
		require(["./engine"], function(engine) {
	
			//IE
			if (!event.target) {
				event.target = event.srcElement;
			}
			
			if (!!event && !!event.target && event.target.id != "mainBody") {
				var id = event.target.id;
				var viewId = null;
				var parent = event.target.parentNode;
				while (parent) {
					if (parent.className && parent.className.indexOf(engine.CONSTANTS.COACH_VIEW) != -1) {
						viewId = parent.id;
						break;
					}
					parent = parent.parentNode;
				}
				if (!id) {
					id = viewId;
				}
				var focusedUsers = controller.focusElementToUserMap[id+viewId];
				if (focusedUsers && focusedUsers.length >0) {
					var e = 	{viewId: viewId, domId: id};
					e.color = controller.focusTextColor;
					e.bgColor = controller.userColorMap[focusedUsers[0]];
					if (focusedUsers.length >1) {
						e.message = focusedUsers[0] + ",...";
					} else {
						e.message = focusedUsers[0];
					}
					
					engine.handleMouseOnEvent(e);
				}
			}
		});
	}

	controller.mouseOffHandler = function (event) {
		require(["./engine"], function(engine) {
			//detected a hover leave event.  Need to check if the current domId+viewId has any users who are focused on it.
			//If so, notify the framework to hide the bubble on the element.
			//Note: this code no longer applies since the collab always has a shimer in front of the coach
	
			//IE
			if (!event.target) {
				event.target = event.srcElement;
			}
			
			if (!!event && !!event.target && event.target.id != "mainBody") {
				var id = event.target.id;
				var viewId = null;
				var parent = event.target.parentNode;
				while (parent) {
					if (parent.className && parent.className.indexOf(engine.CONSTANTS.COACH_VIEW) != -1) {
						viewId = parent.id;
						break;
					}
					parent = parent.parentNode;
				}
				if (!id) {
					id = viewId;
				}
				var focusedUsers = controller.focusElementToUserMap[id+viewId];
				if (focusedUsers && focusedUsers.length >0) {
					var e = 	{viewId: viewId, domId: id};
	
					engine.handleMouseOffEvent(e);
				}
			}
		});
	}


	controller.setFocusHandlers = function () {
		require(["./engine"], function(engine) {
			//sets a focus handler on the mainBody element to process all focus changes on the coach
			var mainBody = dom.byId("mainBody");
			
			if (mainBody.addEventListener) { //non-IE
				mainBody.addEventListener('focus', lang.hitch(engine.GlobalAssets,controller.handleFocusEvent), true); 
				//mainBody.onmouseover = controller.mouseOnHandler;
				//mainBody.onmouseout = controller.mouseOffHandler;
			} else {  // IE
			  //mainBody.onfocusin = controller.handleFocusEvent;
				mainBody.attachEvent('onfocusin', lang.hitch(engine.GlobalAssets, controller.handleFocusEvent));
				//mainBody.attachEvent('onmouseover', controller.mouseOnHandler);
				//mainBody.attachEvent('onmouseout', controller.mouseOffHandler);
			}
		});
	}

	controller.processFocusEvent = function(e) {
		require(["./engine"], function(engine) {
			//a focus event has come in, need to check if the user already is assigned a color, if not, assign one
			if (!controller.userColorMap[e.userName]) {
				controller.userColorMap[e.userName] = controller.colorList.pop();
			}
			var userColor = controller.userColorMap[e.userName];
			e.outlineColor = userColor;
			e.bgColor = userColor;
			e.color = controller.focusTextColor;
			
			//check the user focus map to see if this user has an existing focus
			var userFocus = controller.userFocusMap[e.userName];
			if (userFocus) {
				//use this key to remove the user from the focusElementToUserMap
				var listOfUsers = controller.focusElementToUserMap[userFocus.domId+userFocus.viewId];
				for (var x=0; x<listOfUsers.length; x++) {
					if (listOfUsers[x] == e.userName) {
						listOfUsers.splice(x,1);
						break;
					}
				}
				//if there are any other users who are currently focused on this element, send a focus event to the framework on their behalf,
				//otherwise ask the framework to blur this element
				if (listOfUsers.length > 0) {
					var altColor = controller.userColorMap[listOfUsers[0]];
					engine.handleFocusEvent({viewId: userFocus.viewId, domId: userFocus.domId, outlineColor: altColor, bgColor: altColor, color: controller.focusTextColor});
				} else {
					engine.handleBlurEvent({viewId: userFocus.viewId, domId: userFocus.domId, viewIdMetaData: userFocus.viewIdMetaData});
				}
			}
			//change the users new focus
			controller.userFocusMap[e.userName] = {domId: e.domId, viewId: e.viewId, viewIdMetaData: e.viewIdMetaData};
			//add the user to the focus map of the new dom
			if (!controller.focusElementToUserMap[e.domId+e.viewId]) {
				controller.focusElementToUserMap[e.domId+e.viewId] = [];
			}
			controller.focusElementToUserMap[e.domId+e.viewId].push(e.userName);
			
			engine.handleFocusEvent(e);
			
			//since we are processing a focus event, this implies that we are a viewer
			//not an editor, so we should reset the flag
			controller.isCurrentUserAnEditor = false;
		});
	}

	controller.processDataEvent = function(e) {
		//processes a data change event that has come in on the bus
		require(["./engine", "./Stateful"], function(engine, stateful) {
			//process data event
			var statefulObject = null; 
			if (e.path == "local") {
				logger.log("controller", "proccessing collab event for local.", logger.FINE);
				statefulObject = engine.GlobalAssets.local;
			}
			else {
				var path;
				
				//Now that option value are bindable and editable, even for static value (ie, not bo). 
				//Those static values are stored under engine.GlobalAssets.options
				if (e.path.indexOf("config") == 0){
					path = e.path.substring("config.".length);
					logger.log("controller", "proccessing collab event for config." +path, logger.FINE);
					statefulObject = engine.GlobalAssets.options;
					var propArray = path.split(".");
					for (var x=0; x<propArray.length; x++) {
						statefulObject = statefulObject[propArray[x]];
					}
					if (!!statefulObject && !!(statefulObject.boundObject)) {
						statefulObject = statefulObject.boundObject;
					}
				}
				else {
					path = e.path.substring("local.".length);
					logger.log("controller", "proccessing collab event for local." +path, logger.FINE);
					statefulObject = engine.GlobalAssets.local.get(path);
				}
			}
				
			logger.log("controller", "event is: " + json.toJson(e), logger.FINE);
			if(statefulObject instanceof com.ibm.bpm.coach.Stateful) {
				var processedValue = e.value;
				if (e.value[0] == "{" || e.value[0] == "[")
				{
					processedValue = kernel.eval('('+e.value+')'); 
				}
	
				//disable system notifications before doing the set to prevent an infinite
				//loop of notifications
				statefulObject.set(e.property, processedValue, true);			
			} else {
				//must be a list
                if (e.property == "listAllSelectedIndices") {
                    // listAllSelectedIndices is a special case for lists
                    processedValue = kernel.eval('('+e.value+')');
                    statefulObject.set(e.property, processedValue, true);
                } else if (e.insertedInto == -1) {
					//must be a delete
					if (e.removedFrom != -1) {
						statefulObject.remove(e.removedFrom, true)
					} else {
		    			logger.log("controller", "processDataEvent", "error could not process update: " + json.toJson(e), logger.FINE);
					}
				} else {
					//either an insert or a update
					processedValue = e.value;
					if (e.value[0] == "{" || e.value[0] == "[")
					{
						processedValue = kernel.eval('('+e.value+')'); 
					}
					if (e.removedFrom != -1) {
						//update
						statefulObject.put(e.insertedInto, processedValue.listItem, null, true);
					} else {
						//add
						statefulObject.add(processedValue.listItem, null, true);
					}
				}
//                var listObj = statefulObject;
//                var list = engine.GlobalAssets.local.get(path);
//                engine.GlobalAssets.local.set(list, listObj, true);
			}	
			e.color = controller.focusTextColor;
			e.bgColor = controller.userColorMap[e.userName];
            var message = engine.localizedMessages.coachNG_change_notification;
			e.message = message.replace("{0}", e.userName);
			engine.handleCollabDataChangeEvent(e);
		});
	}

	/*system change handlers, 
	 * handles properties change on a @Stateful object or
	 * property on a @List object
	*/
	controller.systemChangeHandler = function (path, prop, newValue) {
		require(["./engine"], function(engine) {
			var currTimeStamp = new Date().getTime();
			var curFocus;
			var curView;
			var curViewMeta;
			//only send the previous focus dom if the user has tabbed off the field
			//we know this if the focus change happens within a few milliseconds of the data change 
			if (!!controller.previousUserFocus.domId && ( (currTimeStamp - controller.focusTimestamp) < 600) ) {
				curFocus = controller.previousUserFocus.domId;
				curView = controller.previousUserFocus.viewId;
				curViewMeta = controller.previousUserFocus.viewIdMetaData;
			} else {
				curFocus = controller.currentUserFocus.domId;
				curView = controller.currentUserFocus.viewId;
				curViewMeta = controller.currentUserFocus.viewIdMetaData;
			}
			controller.previousUserFocus.domId = controller.currentUserFocus.domId;
			controller.previousUserFocus.viewId = controller.currentUserFocus.viewId;
			controller.previousUserFocus.viewIdMetaData = controller.currentUserFocus.viewIdMetaData;

            if (controller.readOnly == false) {
                // we need to check if coach is in editor mode
                // in order to prevent mirroring notifications
                if (typeof newValue == 'object' ||
                      Object.prototype.toString.call(newValue) === '[object Array]') {
                    var shortPath = path;
                    if (prop && prop != "") {
                        shortPath = shortPath + "." + prop;
                    }
                    if (shortPath.indexOf("local.") != -1)
                    {
                        shortPath = shortPath.substring("local.".length);
                    }
                    var storedValue = engine.GlobalAssets.local.get(shortPath);

                    if(storedValue instanceof Date) {
                        newValue = engine.GlobalAssets.local.dateToString(storedValue);
                    } else {
                        //There are properties on a List that requires this checking
                        newValue = storedValue instanceof com.ibm.bpm.coach.Stateful || storedValue instanceof com.ibm.bpm.coach.List
                            ? storedValue.toJson() : json.toJson(newValue);
                    }
                }

                var system = engine.GlobalAssets.system;
                var event = {type: 'data', path: path, property: prop, value: newValue, userName: system.user_fullName,
                    domId: curFocus, viewId: curView, viewIdMetaData: curViewMeta };

                // publishing data change (collaboration) notification
                
                // do not send notification if it is only a meta data change
                var metaDataOnly = false;
                if (!!path && path.indexOf("_metadata") != -1) {
                	logger.log("controller", "systemChangeHandler", "Meta data only, not sending", logger.FINER);
                	metaDataOnly = true;
                }
                
                if (!metaDataOnly) {
                 logger.log("controller", "systemChangeHandler", "Sending data change event on bus: " + json.toJson(event), logger.FINER);
                 controller.publishNotification(event);
                }
            }
		}); 
	}

	controller.systemListChangeHandler = function (path, object, removedFrom, insertedInto) {
		require(["./engine"], function(engine) {
			var currTimeStamp = new Date().getTime();
			var curFocus;
			var curView;
			var curViewMeta;
			//only send the previous focus dom if the user has tabbed off the field
			//we know this if the focus change happens within a few milliseconds of the data change 
			if (!!controller.previousUserFocus.domId && ( (currTimeStamp - controller.focusTimestamp) < 600) ) {
				curFocus = controller.previousUserFocus.domId;
				curView = controller.previousUserFocus.viewId;
				curViewMeta = controller.previousUserFocus.viewIdMetaData;
			} else {
				curFocus = controller.currentUserFocus.domId;
				curView = controller.currentUserFocus.viewId;
				curViewMeta = controller.currentUserFocus.viewIdMetaData;
			}
			controller.previousUserFocus.domId = controller.currentUserFocus.domId;
			controller.previousUserFocus.viewId = controller.currentUserFocus.viewId;
			controller.previousUserFocus.viewIdMetaData = controller.currentUserFocus.viewIdMetaData;

            if (controller.readOnly == false) {
                // we need to check if coach is in editor mode
                // in order to prevent mirroring notifications
                var objectString = ""
                if (insertedInto != -1) {
                    //this is not a remove operation
                    objectString = json.toJson(object);
                }
                var system = engine.GlobalAssets.system;
                var event = {type: 'data', path: path, removedFrom: removedFrom, insertedInto: insertedInto, value: objectString, userName: system.user_fullName,
                         domId: curFocus, viewId: curView, viewIdMetaData: curViewMeta };

                logger.log("controller", "systemListChangeHandler", "Sending list change event on bus: " + json.toJson(event), logger.FINER);
                // publishing data change notification (collaboration)
                controller.publishNotification(event);
            }
		});
	}

	controller.destroy = function() {
		logger.log("controller", "destroy", "removing controller objects", logger.FINE);
		if(!!controller.progressIcon) {
			controller.progressIcon.parentNode.removeChild(controller.progressIcon);
		}
        controller.destroyed = true;
		controller.currentUserFocus = null;
		controller.previousUserFocus = null;
		controller.focusTextColor = null
		controller.colorList = null;
		controller.userColorMap = null;
		controller.userFocusMap = null;
		controller.focusElementToUserMap = null;
		controller.focusTimestamp = null;
		controller.boundayEventQueue = null;
        controller.validationErrors = null;
		
		if (controller.notificationTimeoutHandler != null) {
			clearTimeout(controller.notificationTimeoutHandler);
			controller.notificationTimeoutHandler = null;
		}
		if (controller.setAsEditorTimeoutHandler != null) {
			clearTimeout(controller.setAsEditorTimeoutHandler);
			controller.setAsEditorTimeoutHandler = null;		
		}
	}

	//process an event originating from an external wrapper (like a portlet) 
	controller.processExternalEvent = function(event, returnTicket) {
		require(["./engine"], function(engine) {
			if (event.name == "setCoachData") {
				//sets the given data on the coach data model
				var result = controller.processDataDeltas(event.data);
				if (result == false) {
					event.result = "error";
				} else {
					event.result = "success";
				}
				event.data = null;
				broadcastHelper.broadcastMessage(event, returnTicket);
			} else if (event.name == "getCoachData") {
				//gets the specified data from the coach and returns it
				var result = {};
				event.result = "success";
				for (var x=0; x<event.data.length; x++) {
					var property = event.data[x];
					var value = null;
					var translatedProperty = null;
					if (property == "tw.local") {
						value = engine.GlobalAssets.local;
					} else {
                    				var pos = property.indexOf("tw.local.");
                    				if (pos == 0) {
                        				// stripping off 'tw.local.'
                    					translatedProperty = property.substring(9);
                        				 value = engine.GlobalAssets.local.get(translatedProperty);
                        			}
                        		}
            		if (value != null && value != undefined) {
						if (!!value.toJson) {
							//need to ensure stateful and list objects are traslated into normal JS objects
							value = json.fromJson(value.toJson());
						}
					} else {
						event.result = "error";
					}
					result[property] = value;
				}
				event.data = result;
				broadcastHelper.broadcastMessage(event, returnTicket);
			} else if (event.name == "triggerBoundaryEvent") {
				//Translate boundary event path to boundary event id
				var boundaryEventId = null;
				if (!!engine.GlobalAssets.boundaryEventData && !!engine.GlobalAssets.boundaryEventData[event.path]) {
                    // coach id must match the current coach
                    if (!!event.coach && event.coach == document.title) {
                        boundaryEventId = engine.GlobalAssets.boundaryEventData[event.path];
                    }
				}
				
				if (!!boundaryEventId) {
					//create closure callback that sends response including correlationid
					var callback = function() {
						event.result = "success";
						broadcastHelper.broadcastMessage(event, returnTicket);
					}
                    var boundaryEventInfo = engine.getBoundaryEventInfo(boundaryEventId);
                    var validateEventInfo = engine.getValidateBoundaryEventInfo();
					controller.handleBoundaryEvent(boundaryEventInfo, validateEventInfo, callback, false);
				} else {
					event.result = "skipped";
					broadcastHelper.broadcastMessage(event, returnTicket);
				}
			} else if (event.name == "getCoachContentBox") {
				event.data = dojo.contentBox(document.body);
				event.result = "success";
				broadcastHelper.broadcastMessage(event, returnTicket);
			}
		});		
	}
	// controller end
	
	return controller;
});

},
'dijit/layout/_ContentPaneResizeMixin':function(){
define([
	"dojo/_base/array", // array.filter array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr",	// domAttr.has
	"dojo/dom-class",	// domClass.contains domClass.toggle
	"dojo/dom-geometry",// domGeometry.contentBox domGeometry.marginBox
	"dojo/_base/lang", // lang.mixin
	"dojo/query", // query
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/window", // win.global
	"../registry",	// registry.byId
	"./utils",	// marginBox2contextBox
	"../_Contained"
], function(array, declare, domAttr, domClass, domGeometry, lang, query, has, win,
			registry, layoutUtils, _Contained){

/*=====
var _Contained = dijit._Contained;
=====*/

// module:
//		dijit/layout/_ContentPaneResizeMixin
// summary:
//		Resize() functionality of ContentPane.   If there's a single layout widget
//		child then it will call resize() with the same dimensions as the ContentPane.
//		Otherwise just calls resize on each child.


return declare("dijit.layout._ContentPaneResizeMixin", null, {
	// summary:
	//		Resize() functionality of ContentPane.   If there's a single layout widget
	//		child then it will call resize() with the same dimensions as the ContentPane.
	//		Otherwise just calls resize on each child.
	//
	//		Also implements basic startup() functionality, where starting the parent
	//		will start the children

	// doLayout: Boolean
	//		- false - don't adjust size of children
	//		- true - if there is a single visible child widget, set it's size to
	//				however big the ContentPane is
	doLayout: true,

	// isLayoutContainer: [protected] Boolean
	//		Indicates that this widget will call resize() on it's child widgets
	//		when they become visible.
	isLayoutContainer: true,

	startup: function(){
		// summary:
		//		See `dijit.layout._LayoutWidget.startup` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		if(this._started){ return; }

		var parent = this.getParent();
		this._childOfLayoutWidget = parent && parent.isLayoutContainer;

		// I need to call resize() on my child/children (when I become visible), unless
		// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
		this._needLayout = !this._childOfLayoutWidget;

		this.inherited(arguments);

		if(this._isShown()){
			this._onShow();
		}

		if(!this._childOfLayoutWidget){
			// If my parent isn't a layout container, since my style *may be* width=height=100%
			// or something similar (either set directly or via a CSS class),
			// monitor when my size changes so that I can re-layout.
			// For browsers where I can't directly monitor when my size changes,
			// monitor when the viewport changes size, which *may* indicate a size change for me.
			this.connect(has("ie") ? this.domNode : win.global, 'onresize', function(){
				// Using function(){} closure to ensure no arguments to resize.
				this._needLayout = !this._childOfLayoutWidget;
				this.resize();
			});
		}
	},

	_checkIfSingleChild: function(){
		// summary:
		//		Test if we have exactly one visible widget as a child,
		//		and if so assume that we are a container for that widget,
		//		and should propagate startup() and resize() calls to it.
		//		Skips over things like data stores since they aren't visible.

		var childNodes = query("> *", this.containerNode).filter(function(node){
				return node.tagName !== "SCRIPT"; // or a regexp for hidden elements like script|area|map|etc..
			}),
			childWidgetNodes = childNodes.filter(function(node){
				return domAttr.has(node, "data-dojo-type") || domAttr.has(node, "dojoType") || domAttr.has(node, "widgetId");
			}),
			candidateWidgets = array.filter(childWidgetNodes.map(registry.byNode), function(widget){
				return widget && widget.domNode && widget.resize;
			});

		if(
			// all child nodes are widgets
			childNodes.length == childWidgetNodes.length &&

			// all but one are invisible (like dojo.data)
			candidateWidgets.length == 1
		){
			this._singleChild = candidateWidgets[0];
		}else{
			delete this._singleChild;
		}

		// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
		domClass.toggle(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
	},

	resize: function(changeSize, resultSize){
		// summary:
		//		See `dijit.layout._LayoutWidget.resize` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
		// never called, so resize() is our trigger to do the initial href download (see [20099]).
		// However, don't load href for closed TitlePanes.
		if(!this._wasShown && this.open !== false){
			this._onShow();
		}

		this._resizeCalled = true;

		this._scheduleLayout(changeSize, resultSize);
	},

	_scheduleLayout: function(changeSize, resultSize){
		// summary:
		//		Resize myself, and call resize() on each of my child layout widgets, either now
		//		(if I'm currently visible) or when I become visible
		if(this._isShown()){
			this._layout(changeSize, resultSize);
		}else{
			this._needLayout = true;
			this._changeSize = changeSize;
			this._resultSize = resultSize;
		}
	},

	_layout: function(changeSize, resultSize){
		// summary:
		//		Resize myself according to optional changeSize/resultSize parameters, like a layout widget.
		//		Also, since I am a Container widget, each of my children expects me to
		//		call resize() or layout() on them.
		//
		//		Should be called on initialization and also whenever we get new content
		//		(from an href, or from set('content', ...))... but deferred until
		//		the ContentPane is visible

		// Set margin box size, unless it wasn't specified, in which case use current size.
		if(changeSize){
			domGeometry.setMarginBox(this.domNode, changeSize);
		}

		// Compute content box size of containerNode in case we [later] need to size our single child.
		var cn = this.containerNode;
		if(cn === this.domNode){
			// If changeSize or resultSize was passed to this method and this.containerNode ==
			// this.domNode then we can compute the content-box size without querying the node,
			// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
			var mb = resultSize || {};
			lang.mixin(mb, changeSize || {}); // changeSize overrides resultSize
			if(!("h" in mb) || !("w" in mb)){
				mb = lang.mixin(domGeometry.getMarginBox(cn), mb); // just use domGeometry.setMarginBox() to fill in missing values
			}
			this._contentBox = layoutUtils.marginBox2contentBox(cn, mb);
		}else{
			this._contentBox = domGeometry.getContentBox(cn);
		}

		this._layoutChildren();

		delete this._needLayout;
	},

	_layoutChildren: function(){
		// Call _checkIfSingleChild() again in case app has manually mucked w/the content
		// of the ContentPane (rather than changing it through the set("content", ...) API.
		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(this._singleChild && this._singleChild.resize){
			var cb = this._contentBox || domGeometry.getContentBox(this.containerNode);

			// note: if widget has padding this._contentBox will have l and t set,
			// but don't pass them to resize() or it will doubly-offset the child
			this._singleChild.resize({w: cb.w, h: cb.h});
		}else{
			// All my child widgets are independently sized (rather than matching my size),
			// but I still need to call resize() on each child to make it layout.
			array.forEach(this.getChildren(), function(widget){
				if(widget.resize){
					widget.resize();
				}
			});
		}
	},

	_isShown: function(){
		// summary:
		//		Returns true if the content is currently shown.
		// description:
		//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
		//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
		//		tree every call, and at least solves the performance problem on page load by deferring loading
		//		hidden ContentPanes until they are first shown

		if(this._childOfLayoutWidget){
			// If we are TitlePane, etc - we return that only *IF* we've been resized
			if(this._resizeCalled && "open" in this){
				return this.open;
			}
			return this._resizeCalled;
		}else if("open" in this){
			return this.open;		// for TitlePane, etc.
		}else{
			var node = this.domNode, parent = this.domNode.parentNode;
			return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !domClass.contains(node, "dijitHidden") &&
					parent && parent.style && (parent.style.display != 'none');
		}
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does layout/resize of child widget(s)

		if(this._needLayout){
			// If a layout has been scheduled for when we become visible, do it now
			this._layout(this._changeSize, this._resultSize);
		}

		this.inherited(arguments);

		// Need to keep track of whether ContentPane has been shown (which is different than
		// whether or not it's currently visible).
		this._wasShown = true;
	}
});

});

},
'dijit/_WidgetBase':function(){
define([
	"require",			// require.toUrl
	"dojo/_base/array", // array.forEach array.map
	"dojo/aspect",
	"dojo/_base/config", // config.blankGif
	"dojo/_base/connect", // connect.connect
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.set domAttr.remove
	"dojo/dom-class", // domClass.add domClass.replace
	"dojo/dom-construct", // domConstruct.create domConstruct.destroy domConstruct.place
	"dojo/dom-geometry",	// isBodyLtr
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/_base/kernel",
	"dojo/_base/lang", // mixin(), isArray(), etc.
	"dojo/on",
	"dojo/ready",
	"dojo/Stateful", // Stateful
	"dojo/topic",
	"dojo/_base/window", // win.doc.createTextNode
	"./registry"	// registry.getUniqueId(), registry.findWidgets()
], function(require, array, aspect, config, connect, declare,
			dom, domAttr, domClass, domConstruct, domGeometry, domStyle, kernel,
			lang, on, ready, Stateful, topic, win, registry){

/*=====
var Stateful = dojo.Stateful;
=====*/

// module:
//		dijit/_WidgetBase
// summary:
//		Future base class for all Dijit widgets.

// For back-compat, remove in 2.0.
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dijit/_base/manager"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

// Nested hash listing attributes for each tag, all strings in lowercase.
// ex: {"div": {"style": true, "tabindex" true}, "form": { ...
var tagAttrs = {};
function getAttrs(obj){
	var ret = {};
	for(var attr in obj){
		ret[attr.toLowerCase()] = true;
	}
	return ret;
}

function nonEmptyAttrToDom(attr){
	// summary:
	//		Returns a setter function that copies the attribute to this.domNode,
	//		or removes the attribute from this.domNode, depending on whether the
	//		value is defined or not.
	return function(val){
		domAttr[val ? "set" : "remove"](this.domNode, attr, val);
		this._set(attr, val);
	};
}

return declare("dijit._WidgetBase", Stateful, {
	// summary:
	//		Future base class for all Dijit widgets.
	// description:
	//		Future base class for all Dijit widgets.
	//		_Widget extends this class adding support for various features needed by desktop.
	//
	//		Provides stubs for widget lifecycle methods for subclasses to extend, like postMixInProperties(), buildRendering(),
	//		postCreate(), startup(), and destroy(), and also public API methods like set(), get(), and watch().
	//
	//		Widgets can provide custom setters/getters for widget attributes, which are called automatically by set(name, value).
	//		For an attribute XXX, define methods _setXXXAttr() and/or _getXXXAttr().
	//
	//		_setXXXAttr can also be a string/hash/array mapping from a widget attribute XXX to the widget's DOMNodes:
	//
	//		- DOM node attribute
	// |		_setFocusAttr: {node: "focusNode", type: "attribute"}
	// |		_setFocusAttr: "focusNode"	(shorthand)
	// |		_setFocusAttr: ""		(shorthand, maps to this.domNode)
	// 		Maps this.focus to this.focusNode.focus, or (last example) this.domNode.focus
	//
	//		- DOM node innerHTML
	//	|		_setTitleAttr: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		_setTitleAttr: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		_setMyClassAttr: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value of _setXXXAttr is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		If the custom setter is null, no action is performed other than saving the new value
	//		in the widget (in this).
	//
	//		If no custom setter is defined for an attribute, then it will be copied
	//		to this.focusNode (if the widget defines a focusNode), or this.domNode otherwise.
	//		That's only done though for attributes that match DOMNode attributes (title,
	//		alt, aria-labelledby, etc.)

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",
	_setIdAttr: "domNode",	// to copy to this.domNode even for auto-generated id's

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",
	// set on domNode even when there's a focus node.   but don't set lang="", since that's invalid.
	_setLangAttr: nonEmptyAttrToDom("lang"),

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",
	// set on domNode even when there's a focus node.   but don't set dir="", since that's invalid.
	_setDirAttr: nonEmptyAttrToDom("dir"),	// to set on domNode even when there's a focus node

	// textDir: String
	//		Bi-directional support,	the main variable which is responsible for the direction of the text.
	//		The text direction can be different than the GUI direction by using this parameter in creation
	//		of a widget.
	// 		Allowed values:
	//			1. "ltr"
	//			2. "rtl"
	//			3. "auto" - contextual the direction of a text defined by first strong letter.
	//		By default is as the page direction.
	textDir: "",

	// class: String
	//		HTML class attribute
	"class": "",
	_setClassAttr: { node: "domNode", type: "class" },

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's data-dojo-attach-point syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div data-dojo-type=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span data-dojo-type=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span data-dojo-type=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		data-dojo-attach-point assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

/*=====
	// _started: Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		Deprecated.   Instead of attributeMap, widget should have a _setXXXAttr attribute
	//		for each XXX attribute to be mapped to the DOM.
	//
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling set('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {},

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by <img> nodes in templates that really get their image via CSS background-image.
	_blankGif: config.blankGif || require.toUrl("dojo/resources/blank.gif"),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (DOM node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://dojotoolkit.org/reference-guide/dijit/_WidgetBase.html
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dom.byId(srcNodeRef);

		// For garbage collection.  An array of listener handles returned by this.connect() / this.subscribe()
		this._connects = [];

		// For widgets internal to this widget, invisible to calling code
		this._supportingWidgets = [];

		// this is here for back-compat, remove in 2.0 (but check NodeList-instantiate.html test)
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }

		// mix in our passed parameters
		if(params){
			this.params = params;
			lang.mixin(this, params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = registry.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			// Also calls custom setters for all attributes with custom setters.
			this._applyAttributes();

			// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
			// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
			// widget being attached to the DOM since it isn't when a widget is created programmatically like
			// new MyWidget({}).   See #11635.
			var source = this.srcNodeRef;
			if(source && source.parentNode && this.domNode !== source){
				source.parentNode.replaceChild(this.domNode, source);
			}
		}

		if(this.domNode){
			// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
			// assuming that dojo._scopeName even exists in 2.0
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy  widget attributes to the
		//		DOM according to attributeMap and _setXXXAttr objects, and also to call
		//		custom _setXXXAttr() methods.
		//
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		For backwards-compatibility reasons attributeMap overrides _setXXXAttr when
		//		_setXXXAttr is a hash/string/array, but _setXXXAttr as a functions override attributeMap.
		// tags:
		//		private

		// Get list of attributes where this.set(name, value) will do something beyond
		// setting this[name] = value.  Specifically, attributes that have:
		//		- associated _setXXXAttr() method/hash/string/array
		//		- entries in attributeMap.
		var ctor = this.constructor,
			list = ctor._setterAttrs;
		if(!list){
			list = (ctor._setterAttrs = []);
			for(var attr in this.attributeMap){
				list.push(attr);
			}

			var proto = ctor.prototype;
			for(var fxName in proto){
				if(fxName in this.attributeMap){ continue; }
				var setterName = "_set" + fxName.replace(/^[a-z]|-[a-zA-Z]/g, function(c){ return c.charAt(c.length-1).toUpperCase(); }) + "Attr";
				if(setterName in proto){
					list.push(fxName);
				}
			}
		}

		// Call this.set() for each attribute that was either specified as parameter to constructor,
		// or was found above and has a default non-null value.   For correlated attributes like value and displayedValue, the one
		// specified as a parameter should take precedence, so apply attributes in this.params last.
		// Particularly important for new DateTextBox({displayedValue: ...}) since DateTextBox's default value is
		// NaN and thus is not ignored like a default value of "".
		array.forEach(list, function(attr){
			if(this.params && attr in this.params){
				// skip this one, do it below
			}else if(this[attr]){
				this.set(attr, this[attr]);
			}
		}, this);
		for(var param in this.params){
			this.set(param, this[param]);
		}
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode.
		//		Most widgets will mixin `dijit._TemplatedMixin`, which implements this method.
		// tags:
		//		protected

		if(!this.domNode){
			// Create root node if it wasn't created by _Templated
			this.domNode = this.srcNodeRef || domConstruct.create('div');
		}

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
		// TODO: make baseClass custom setter
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( array.map(classes, function(name){ return name+"Rtl"; }));
			}
			domClass.add(this.domNode, classes);
		}
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		if(this._started){ return; }
		this._started = true;
		array.forEach(this.getChildren(), function(obj){
			if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
				obj.startup();
				obj._started = true;
			}
		});
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		// 		should call to cleanly discard with a widget. Once a widget is
		// 		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();

		// remove this.connect() and this.subscribe() listeners
		var c;
		while((c = this._connects.pop())){
			c.remove();
		}

		// destroy widgets created as part of template, etc.
		var w;
		while((w = this._supportingWidgets.pop())){
			if(w.destroyRecursive){
				w.destroyRecursive();
			}else if(w.destroy){
				w.destroy();
			}
		}

		this.destroyRendering(preserveDom);
		registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				domAttr.remove(this.domNode, "widgetId");
			}else{
				domConstruct.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				domConstruct.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		array.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},

	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribute of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this.domNode;

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(lang.isObject(value)){
			domStyle.set(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this._set("style", value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value, /*Object?*/ commands){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified by commands parameter.
		//		If commands isn't specified then it's looked up from attributeMap.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		commands = arguments.length >= 3 ? commands : this.attributeMap[attr];

		array.forEach(lang.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(lang.isFunction(value)){ // functions execute in the context of the widget
						value = lang.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					domAttr.set(mapNode, attrName, value);
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(win.doc.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					domClass.replace(mapNode, value, this[attr]);
					break;
			}
		}, this);
	},

	get: function(name){
		// summary:
		//		Get a property from a widget.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.
		//
		// 		For example, if the widget has properties `foo` and `bar`
		//		and a method named `_getFooAttr()`, calling:
		//		`myWidget.get("foo")` would be equivalent to calling
		//		`widget._getFooAttr()` and `myWidget.get("bar")`
		//		would be equivalent to the expression
		//		`widget.bar2`
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},

	set: function(name, value){
		// summary:
		//		Set a property on a widget
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a
		// 		setter in the widget.
		//
		// 		For example, if the widget has properties `foo` and `bar`
		//		and a method named `_setFooAttr()`, calling
		//		`myWidget.set("foo", "Howdy!")` would be equivalent to calling
		//		`widget._setFooAttr("Howdy!")` and `myWidget.set("bar", 3)`
		//		would be equivalent to the statement `widget.bar = 3;`
		//
		//		set() may also be called with a hash of name/value pairs, ex:
		//
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	});
		//
		//	This is equivalent to calling `set(foo, "Howdy")` and `set(bar, 3)`

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var names = this._getAttrNames(name),
			setter = this[names.s];
		if(lang.isFunction(setter)){
			// use the explicit setter
			var result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// Mapping from widget attribute to DOMNode attribute/value/etc.
			// Map according to:
			//		1. attributeMap setting, if one exists (TODO: attributeMap deprecated, remove in 2.0)
			//		2. _setFooAttr: {...} type attribute in the widget (if one exists)
			//		3. apply to focusNode or domNode if standard attribute name, excluding funcs like onClick.
			// Checks if an attribute is a "standard attribute" by whether the DOMNode JS object has a similar
			// attribute name (ex: accept-charset attribute matches jsObject.acceptCharset).
			// Note also that Tree.focusNode() is a function not a DOMNode, so test for that.
			var defaultNode = this.focusNode && !lang.isFunction(this.focusNode) ? "focusNode" : "domNode",
				tag = this[defaultNode].tagName,
				attrsForTag = tagAttrs[tag] || (tagAttrs[tag] = getAttrs(this[defaultNode])),
				map =	name in this.attributeMap ? this.attributeMap[name] :
						names.s in this ? this[names.s] :
						((names.l in attrsForTag && typeof value != "function") ||
							/^aria-|^data-|^role$/.test(name)) ? defaultNode : null;
			if(map != null){
				this._attrToDom(name, value, map);
			}
			this._set(name, value);
		}
		return result || this;
	},

	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.replace(/^[a-z]|-[a-zA-Z]/g, function(c){ return c.charAt(c.length-1).toUpperCase(); });
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",	// converts dashes to camel case, ex: accept-charset --> _setAcceptCharsetAttr
			g: "_get"+uc+"Attr",
			l: uc.toLowerCase()		// lowercase name w/out dashes, ex: acceptcharset
		});
	},

	_set: function(/*String*/ name, /*anything*/ value){
		// summary:
		//		Helper function to set new value for specified attribute, and call handlers
		//		registered with watch() if the value has changed.
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks && this._created && value !== oldValue){
			this._watchCallbacks(name, oldValue, value);
		}
	},

	on: function(/*String*/ type, /*Function*/ func){
		// summary:
		//		Call specified function when event occurs, ex: myWidget.on("click", function(){ ... }).
		// description:
		//		Call specified function when event `type` occurs, ex: `myWidget.on("click", function(){ ... })`.
		//		Note that the function is not run in any particular scope, so if (for example) you want it to run in the
		//		widget's scope you must do `myWidget.on("click", lang.hitch(myWidget, func))`.

		return aspect.after(this, this._onMap(type), func, true);
	},

	_onMap: function(/*String*/ type){
		// summary:
		//		Maps on() type parameter (ex: "mousemove") to method name (ex: "onMouseMove")
		var ctor = this.constructor, map = ctor._onMap;
		if(!map){
			map = (ctor._onMap = {});
			for(var attr in ctor.prototype){
				if(/^on/.test(attr)){
					map[attr.replace(/^on/, "").toLowerCase()] = attr;
				}
			}
		}
		return map[type.toLowerCase()];	// String
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? registry.findWidgets(this.containerNode) : []; // dijit._Widget[]
	},

	getParent: function(){
		// summary:
		//		Returns the parent widget of this widget
		return registry.getEnclosingWidget(this.domNode.parentNode);
	},

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var handle = connect.connect(obj, event, this, method);
		this._connects.push(handle);
		return handle;		// _Widget.Handle
	},

	disconnect: function(handle){
		// summary:
		//		Disconnects handle created by `connect`.
		//		Also removes handle from this widget's list of connects.
		// tags:
		//		protected
		var i = array.indexOf(this._connects, handle);
		if(i != -1){
			handle.remove();
			this._connects.splice(i, 1);
		}
	},

	subscribe: function(t, method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// t: String
		//		The topic
		// method: Function
		//		The callback
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|   // be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		// tags:
		//		protected
		var handle = topic.subscribe(t, lang.hitch(this, method));
		this._connects.push(handle);
		return handle;		// _Widget.Handle
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		// tags:
		//		protected
		this.disconnect(handle);
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : domGeometry.isBodyLtr(); //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (domStyle.get(this.domNode, "display") != "none");
	},

	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard domConstruct.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference:
		//		The String id of a domNode, a domNode reference, or a reference to a Widget possessing
		//		an addChild method.
		//
		// position:
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as domConstruct.place does, one of: "first", "last",
		//		"before", or "after".
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns:
		//		dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(win.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	button.on("click", function(e){ console.log('click'); }));
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example:
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference.declaredClass && reference.addChild){
			reference.addChild(this, position);
		}else{
			domConstruct.place(this.domNode, reference, position);
		}
		return this;
	},

	getTextDir: function(/*String*/ text,/*String*/ originalDir){
		// summary:
		//		Return direction of the text.
		//		The function overridden in the _BidiSupport module,
		//		its main purpose is to calculate the direction of the
		//		text, if was defined by the programmer through textDir.
		//	tags:
		//		protected.
		return originalDir;
	},

	applyTextDir: function(/*===== element, text =====*/){
		// summary:
		//		The function overridden in the _BidiSupport module,
		//		originally used for setting element.dir according to this.textDir.
		//		In this case does nothing.
		// element: DOMNode
		// text: String
		// tags:
		//		protected.
	},

	defer: function(fcn, delay){ 
		// summary:
		//		Wrapper to setTimeout to avoid deferred functions executing
		//		after the originating widget has been destroyed.
		//		Returns an object handle with a remove method (that returns null) (replaces clearTimeout).
		// fcn: function reference
		// delay: Optional number (defaults to 0)
		// tags:
		//		protected.
		var timer = setTimeout(lang.hitch(this, 
			function(){ 
				timer = null;
				if(!this._destroyed){ 
					lang.hitch(this, fcn)(); 
				} 
			}),
			delay || 0
		);
		return {
			remove:	function(){
					if(timer){
						clearTimeout(timer);
						timer = null;
					}
					return null; // so this works well: handle = handle.remove();
				}
		};
	}
});

});

},
'com.ibm.bpm.coach/validationHelper':function(){
/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// This module handles data validation in caoch framework

define(["./dataBindingHelper", "./BPM_CONSTANTS", "./commonHelper", "./coachViewHelper", "dojo/_base/array"], function(dataBindingHelper, BPM_CONSTANTS, commonHelper, coachViewHelper, array) {
				
	var validationHelper = new Object();
	
	// only the public APIs of this module can access validationHelper properties which are specified here.
	validationHelper.initialize = function() {
		validationHelper.rawEvents = null;
		
		validationHelper.bindingToViewDomIdsMap = {};
		
		validationHelper.viewTree = {viewDomIdToTreeNodeMap: {}, rootNode: {}};
		
		validationHelper.lastListItemRemoved = {bindingListStr: null, removedItem: null};
		
		// transient info
		validationHelper.clearTransientInfo();
	}
	
	validationHelper.clearTransientInfo = function() {
		validationHelper.viewDomIdToViewMap = {};
	}
	
	validationHelper.validateAllViews = function(rawEvent, topLevelViews) {
		validationHelper.rawEvents = rawEvent;
	
		var viewDomIdToViewMap = validationHelper.viewDomIdToViewMap;
		var viewDomIdToTreeNodeMap = validationHelper.viewTree.viewDomIdToTreeNodeMap;
		var databindingToViewDomIdsMap = validationHelper.bindingToViewDomIdsMap;
		validationHelper._collectInfoForAllViews(topLevelViews, viewDomIdToViewMap, validationHelper.viewTree.rootNode, viewDomIdToTreeNodeMap, databindingToViewDomIdsMap);
		
		return validationHelper._process(rawEvent, databindingToViewDomIdsMap, viewDomIdToViewMap, viewDomIdToTreeNodeMap, true);
	}

	validationHelper.validateView = function(coachView) {
		var foundvalidationEvent = false;
		
		if(!validationHelper.rawEvents || validationHelper.rawEvents.length == 0) {
			// there is even no raw events, no nothing needs to do
			return foundvalidationEvent;
		}
		
		var treeNodeForView = null;
		
		// see if this coachView is a replacement of disposed view. If so, the validation event is already there, just need to make it up to date
		var myBinding = dataBindingHelper.rtDataBinding._get(coachView.context.element);
		var viewDomIds = validationHelper.bindingToViewDomIdsMap[myBinding];
		if(!!viewDomIds  && viewDomIds.length != 0) {
			var myLocationInfo = coachViewHelper.getViewIdMetaData(coachView);
			var myViewDomId =  coachView.context.element.id;
			// some other view has same binding as mine
			for(var k=0; k<viewDomIds.length; k++) {
				var treeNode = validationHelper.viewTree.viewDomIdToTreeNodeMap[viewDomIds[k]];
				if(!!treeNode) {
					if(coachViewHelper.hasSameStructure(myLocationInfo, treeNode.locationInfo)) {
						// coachView is the replacement for the view described by treeNode. So update that treeNode (and a few maps) to make sure it's up to date.
						validationHelper._updateTreeNode(treeNode, myViewDomId, myBinding, myLocationInfo, validationHelper.viewTree.viewDomIdToTreeNodeMap, validationHelper.bindingToViewDomIdsMap);
						treeNodeForView = treeNode;
						break;
					}
				}
			}		
		}
		
		if(!!treeNodeForView) {
			foundvalidationEvent = !!(treeNodeForView.validationEvent);
			
			// handle the subviews of this coachView
			for (var viewId in coachView.context.subview) {
				var subviews = coachView.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					var result = validationHelper.validateView(subViewScope);
					if(!foundvalidationEvent) {
						foundvalidationEvent = result;
					}					
				}
			}		
		} else {
			// this is a brand new coach view, need to check if it has validation errors
			foundvalidationEvent = validationHelper._validateView(coachView);
		}
		
		return foundvalidationEvent;
	}

	validationHelper.listItemRemoved = function(bindingListStr, removedIndex, removedItem) {

		if(!validationHelper.rawEvents || validationHelper.rawEvents.length == 0) {
			// there is even no raw events, no nothing needs to do
			return;
		}
		
		if(validationHelper.lastListItemRemoved.bindingListStr == bindingListStr && validationHelper.lastListItemRemoved.removedItem === removedItem) {
			// multiple views could bind to same bindingListStr. And those views may all try to tell validationHelper
			// that one item has been removed.
			// removedItem has been handled, so ignore it here.
			return;
		}
		
		validationHelper.lastListItemRemoved.bindingListStr = bindingListStr;
		validationHelper.lastListItemRemoved.removedItem = removedItem;

		// update validationHelper.rawEvents
		var invalidEventIndexs = [];
		for(var i=0; i<validationHelper.rawEvents.length; i++) {
			var errorBoPath = validationHelper.rawEvents[i].errorBOPath;
			var newErrorBoPath = validationHelper._updateBindingStrPerDelete(bindingListStr, removedIndex, errorBoPath);
			if(newErrorBoPath == null) {
				// list item that contains errorBoPath has been deleted. so remove this raw event too
				invalidEventIndexs.push(i);
			} else if(newErrorBoPath != errorBoPath) {
				validationHelper.rawEvents[i].errorBOPath = newErrorBoPath;
			}
		}	
		for(var k=invalidEventIndexs.length-1; k>=0; k--) {
			validationHelper.rawEvents.splice(invalidEventIndexs[k], 1);
		}
		
		// item has been deleted from list. As the result, existing information might be out-of-date, 
		// e.g. relativePath in computed validation events, the validation event that roots on deleted item, data in validationHelper.bindingToViewDomIdsMap etc.
		// the best way is to clean up, when new view is created, we evaluate it against the reserved (and updated) rawEvents.
		var cacheRawEvents = validationHelper.rawEvents;
		var cacheLastListItemRemoved = validationHelper.lastListItemRemoved;
		validationHelper.initialize();
		validationHelper.rawEvents = cacheRawEvents;
		validationHelper.lastListItemRemoved = cacheLastListItemRemoved;		
	}
	
	validationHelper._updateBindingStrPerDelete = function(bindingListStr, removedIndex, bindingToUpdate) {
		// bindingListStr is sth like a[0].b[1].c[]
		var strToMatch = bindingListStr.substring(0, bindingListStr.length-1);
		if(bindingToUpdate.indexOf(strToMatch) != -1) {
			var restPart = bindingToUpdate.substring(bindingListStr.length-1);
			var rightBracketPosition = restPart.indexOf("]");
			var index = restPart.substring(0, rightBracketPosition);
			if(index != "") {
				if(index == removedIndex) {
					return null;
				} else if(index > removedIndex) {
					var newBindingStr = strToMatch + (index-1) + restPart.substring(rightBracketPosition);
					return newBindingStr;
				}
			}			
		}
		
		return bindingToUpdate;
	}
	
	// validate the coachView against the raw events, prepare the validationEvent if exists
	validationHelper._validateView = function(coachView) {
	
		var myViewDomId = coachView.context.element.id;

		var rawEvent = validationHelper.rawEvents;
		
		var viewDomIdToViewMap = validationHelper.viewDomIdToViewMap;
		viewDomIdToViewMap[myViewDomId] = coachView;
		
		var viewDomIdToTreeNodeMap = validationHelper.viewTree.viewDomIdToTreeNodeMap;
		
		// insert into right place in validationHelper.viewTree
		var parentTreeNode = null;		
		var parentView = coachView.context.parentView();
		if(!!parentView) {
			parentTreeNode = viewDomIdToTreeNodeMap[parentView.context.element.id];
		}
		if(!parentTreeNode) {
			// can't find proper parent treeNode, let the rootNode to be parent
			parentTreeNode = validationHelper.viewTree.rootNode;
		}
			
		var databindingToViewDomIdsMap = new Object();
		validationHelper._collectInfoForView(coachView, databindingToViewDomIdsMap, viewDomIdToViewMap, parentTreeNode, viewDomIdToTreeNodeMap);
		
		return validationHelper._process(rawEvent, databindingToViewDomIdsMap, viewDomIdToViewMap, viewDomIdToTreeNodeMap, false);		
	}
			
	validationHelper._updateTreeNode = function(treeNode, newViewDomId, dataBinding, newLocationInfo, viewDomIdToTreeNodeMap, bindingToViewDomIdsMap) {
		var oldViewDomId = treeNode.domId;
		
		// update the domId of the treeNode
		treeNode.domId = newViewDomId;
		
		// coachView is a new one, we need to deliver the validation event to this coachView again if it's already done
		if(treeNode.isHandled) {
			treeNode.isHandled = false;
		}
		
		// update validationEvent
		validationHelper._updateIdInValidationEvent(oldViewDomId, newViewDomId, treeNode.validationEvent);
		
		// update the domId referenced by the parent views
		var tempNode = treeNode.parent;
		while(!!tempNode) {
			validationHelper._updateIdInValidationEvent(oldViewDomId, newViewDomId, tempNode.validationEvent, true);
			tempNode = tempNode.parent;
		}
		
		// update locationInfo
		treeNode.locationInfo = newLocationInfo;
		
		// update validationHelper.viewTree.viewDomIdToTreeNodeMap
		viewDomIdToTreeNodeMap[newViewDomId] = viewDomIdToTreeNodeMap[oldViewDomId];
		delete viewDomIdToTreeNodeMap[oldViewDomId];
		
		// update validationHelper.bindingToViewDomIdsMap
		var domIdsOnOneBinding = bindingToViewDomIdsMap[dataBinding];
		validationHelper._replaceIdInArray(oldViewDomId, newViewDomId, domIdsOnOneBinding);
	}
	
	validationHelper._updateIdInValidationEvent = function(oldId, newId, event, updateSubscriptionError) {
		if(!event) {
			return;
		}
		
		if(!!event.errors) {
			for(var k=0; k<event.errors.length; k++) {
				var domIdsInOneError = event.errors[k].view_dom_ids;
				validationHelper._replaceIdInArray(oldId, newId, domIdsInOneError);
			}
		}
		
		if(updateSubscriptionError == true) {
			if(!!event.errors_subscription) {
				for(var k=0; k<event.errors_subscription.length; k++) {
					if(event.errors_subscription[k].view_dom_id == oldId) {
						event.errors_subscription[k].view_dom_id = newId;
					}
				}
			}
		}
	}
	
	validationHelper._replaceIdInArray = function(oldId, newId, idArray) {
		index = array.indexOf(idArray, oldId);
		if(index != -1) {
			idArray[index] = newId;
		}
	}
	
	// prepare the validation events if exists
	validationHelper._process = function(rawEvent, databindingToViewDomIdsMap, viewDomIdToViewMap, viewDomIdToTreeNodeMap, reportUnResolvedErrorBoPath) {	
		var foundValidationEvent = false;
		
		// map between the dom id of views and their list of error messages.
		var viewDomIdToErrorMessagesMap = new Object();			

		for(var i=0; i<rawEvent.length; i++) {
			var foundOneView = false;
			var errorBoPath = rawEvent[i].errorBOPath;
			var errorMsg = rawEvent[i].errorMessage;
			var pathArray = errorBoPath.split(".");
			// path always start with "local."
			var databindingToTry = pathArray[0];
			
			var relatedErrors = new Array();
			for(var k=1; k<pathArray.length; k++) { 
				var relativePath = validationHelper.computeRelativePath(pathArray, k+1);
				var endPosition = pathArray[k].indexOf("[");
				if(endPosition != -1 && pathArray[k].indexOf("[]") == -1) {
					// if pathArray[k] is "abc[number]", we need to check "abc[]" as well
					var otherDatabindingToTry = databindingToTry + "." + pathArray[k].substring(0, endPosition) + "[]";
					
					if(!!databindingToViewDomIdsMap[otherDatabindingToTry]) {
						var newRelativePath = pathArray[k].substring(endPosition);
						if(relativePath != "") {
							newRelativePath += "." + relativePath;
						}
						var viewDomIds = databindingToViewDomIdsMap[otherDatabindingToTry];
						validationHelper.handleOneRelativePath(newRelativePath, errorMsg, viewDomIds, viewDomIdToViewMap, viewDomIdToTreeNodeMap, viewDomIdToErrorMessagesMap, relatedErrors);
						if(!foundOneView) {
							foundOneView = true;
						}
					}				
				}
				databindingToTry += "." + pathArray[k];
				if(!!databindingToViewDomIdsMap[databindingToTry]) {
					var viewDomIds = databindingToViewDomIdsMap[databindingToTry];
					validationHelper.handleOneRelativePath(relativePath, errorMsg, viewDomIds, viewDomIdToViewMap, viewDomIdToTreeNodeMap, viewDomIdToErrorMessagesMap, relatedErrors);
					if(!foundOneView) {
						foundOneView = true;
					}
				}				
			}
			
			if(!!reportUnResolvedErrorBoPath && !databindingToViewDomIdsMap[errorBoPath]) {
				// no view binds to errorBoPath
		    	var message = "validationHelper._process(), Warning: No view binds errorBoPath: " + errorBoPath;
		    	commonHelper.consoleLog(message, commonHelper.WARN);
			}
			
			foundValidationEvent = foundValidationEvent || foundOneView;
		}
			
		// handle subscription
		for(errorViewDomId in viewDomIdToErrorMessagesMap) {
			var errorView = viewDomIdToViewMap[errorViewDomId];
			var parentView = errorView.context.parentView();
			while(!!parentView) {
				if(parentView.context._validationSubscribed) {
					var parentViewDomId = parentView.context.element.id;
					var parentViewFineEvent = viewDomIdToTreeNodeMap[parentViewDomId].validationEvent;
					if(!parentViewFineEvent || !validationHelper.contains(parentViewFineEvent, errorViewDomId)) {
						if(!parentViewFineEvent) {
							parentViewFineEvent = new Object();
							parentViewFineEvent.type = BPM_CONSTANTS.VALIDATION_EVENT_TYPE_ERROR;
							viewDomIdToTreeNodeMap[parentViewDomId].validationEvent = parentViewFineEvent;
							// when there is validationEvent, we also collect 
							viewDomIdToTreeNodeMap[parentViewDomId].locationInfo = coachViewHelper.getViewIdMetaData(parentView);
						} 
						if(!parentViewFineEvent.errors_subscription) {
							parentViewFineEvent.errors_subscription = new Array();
						}
						
						var oneEntry = new Object();
						oneEntry.view_dom_id = errorViewDomId;
						oneEntry.messages = viewDomIdToErrorMessagesMap[errorViewDomId];
						parentViewFineEvent.errors_subscription.push(oneEntry);
					}
				}
				parentView = parentView.context.parentView();					
			}
		}
		
		return foundValidationEvent;	
	}
	
	validationHelper.handleOneRelativePath = function(relativePath, errorMsg, viewDomIds, viewDomIdToViewMap, viewDomIdToTreeNodeMap, viewDomIdToErrorMessagesMap, viewDomIdToSameErrorMap) {

		for(var viewCounter=0; viewCounter<viewDomIds.length; viewCounter++) {
			var viewDomId = viewDomIds[viewCounter];
			var aView = viewDomIdToViewMap[viewDomId];

			var fineEvent = viewDomIdToTreeNodeMap[viewDomId].validationEvent; 			
			if(!fineEvent) {
				fineEvent = new Object();
				fineEvent.type = BPM_CONSTANTS.VALIDATION_EVENT_TYPE_ERROR;
				viewDomIdToTreeNodeMap[viewDomId].validationEvent = fineEvent;
				// when there is validationEvent, we also collect 
				viewDomIdToTreeNodeMap[viewDomId].locationInfo = coachViewHelper.getViewIdMetaData(aView);
			}
			if(!fineEvent.errors) {
				fineEvent.errors = new Array();
			}
			var error = validationHelper.getErrorByPath(fineEvent.errors, relativePath, errorMsg);
			error.view_dom_ids.push(viewDomId);
			
			var parentView = aView.context.parentView();
			while(!!parentView) {
				var parentViewDomId = parentView.context.element.id;
				var sameErrorInParent = viewDomIdToSameErrorMap[parentViewDomId];
				if(!!sameErrorInParent) {
					// parentView has same error as aView, so add id of aView to the error of parentView
					sameErrorInParent.view_dom_ids.push(viewDomId);
				}
				parentView = parentView.context.parentView();										
			}
			viewDomIdToSameErrorMap[viewDomId] = error;
			
			if(!!viewDomIdToErrorMessagesMap[viewDomId]) {
				viewDomIdToErrorMessagesMap[viewDomId].push(errorMsg);
			} else {
				var errorMessages = new Array();
				errorMessages.push(errorMsg);
				viewDomIdToErrorMessagesMap[viewDomId] = errorMessages;
			}					
		}				
	}
	
	validationHelper.computeRelativePath = function(pathArray, pathArrayStartPosition) {
		var relativePath = "";
		for(var j=pathArrayStartPosition; j<pathArray.length; j++) {
			if(relativePath != "") {
				relativePath += "." + pathArray[j];
			} else {
				relativePath = pathArray[j];
			}						
		}
		
		return relativePath;
	}
	
	// check if validationEvent.errors already contains specified viewDomId
	validationHelper.contains = function(validationEvent, viewDomId) {
		if(!!validationEvent.errors) {
			for(var k=0; k<validationEvent.errors.length; k++) {			
				if(array.indexOf(validationEvent.errors[k].view_dom_ids, viewDomId) != -1) {
					return true;
				}
			}
		}
		return false;
	}
	
	// if there's no error for specified relativePath, create one and add it to the error array
	validationHelper.getErrorByPath = function(errors, relativePath, errorMessage) {
		for(var k=0; k<errors.length; k++) {			
			if(errors[k].binding === relativePath) {
				return errors[k];
			}
		}
		
		var error = new Object();
		error.binding = relativePath;
		error.message = errorMessage;
		error.view_dom_ids = new Array();
		
		errors.push(error);
		
		return error;
	}
	
	validationHelper._collectInfoForAllViews = function(topLevelViews, viewDomIdToViewMap, viewTreeRootNode, viewDomIdToTreeNodeMap, databindingToViewDomIdsMap) {
		var bindingStrToListMap = new Object();
		for (var viewId in topLevelViews) {
			var viewScope = topLevelViews[viewId];
			validationHelper._collectInfoForView(viewScope, databindingToViewDomIdsMap, viewDomIdToViewMap, viewTreeRootNode, viewDomIdToTreeNodeMap);
		}
	}
	
	// crete the treeNode and collect binding info for hte viewScope and its descendant views
	validationHelper._collectInfoForView = function(viewScope, databindingToViewDomIdsMap, viewDomIdToViewMap, parentTreeNode, viewDomIdToTreeNodeMap) {
		var treeNode = validationHelper._createTreeNode(viewScope, parentTreeNode, viewDomIdToTreeNodeMap);		
		validationHelper._collectBindingForOneView(viewScope, databindingToViewDomIdsMap);
		
		viewDomIdToViewMap[viewScope.context.element.id] = viewScope;

		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					validationHelper._collectInfoForView(subViewScope, databindingToViewDomIdsMap, viewDomIdToViewMap, treeNode, viewDomIdToTreeNodeMap);
				}
			}		
		}			
	}
	
	validationHelper._collectBindingForOneView = function(coachView, databindingToViewDomIdsMap) {
		var bindingStrArray = coachView._getBindingStrings();
		for(var k=0; k<bindingStrArray.length; k++) {
			var bindingStr = bindingStrArray[k];
			var coachViewDomIdArray = databindingToViewDomIdsMap[bindingStr];
			if(!coachViewDomIdArray) {
				coachViewDomIdArray = new Array();
				databindingToViewDomIdsMap[bindingStr] = coachViewDomIdArray;
			}
			coachViewDomIdArray.push(coachView.context.element.id);		
		}
	}
	
	validationHelper._createTreeNode = function(viewScope, parentTreeNode, viewDomIdToTreeNodeMap) {
		var viewDomId = viewScope.context.element.id;
		var treeNode = {"viewId": viewScope.context.viewid, 
						"domId": viewDomId, 
						"isHandled": false};
		treeNode.parent = parentTreeNode;
		if(!parentTreeNode.firstChild) {
			parentTreeNode.firstChild = treeNode;
		} else {
			parentTreeNode.lastChild.next = treeNode;
		}
		parentTreeNode.lastChild = treeNode;
		
		viewDomIdToTreeNodeMap[viewDomId] = treeNode;
		
		return treeNode;
	}
		
	
	validationHelper.initialize();
	
	return validationHelper;
});
	
},
'dojo/io/script':function(){
define(["../main"], function(dojo) {
	// module:
	//		dojo/io/script
	// summary:
	//		TODOC

	dojo.getObject("io", true, dojo);

/*=====
dojo.declare("dojo.io.script.__ioArgs", dojo.__IoArgs, {
	constructor: function(){
		//	summary:
		//		All the properties described in the dojo.__ioArgs type, apply to this
		//		type as well, EXCEPT "handleAs". It is not applicable to
		//		dojo.io.script.get() calls, since it is implied by the usage of
		//		"jsonp" (response will be a JSONP call returning JSON)
		//		or the response is pure JavaScript defined in
		//		the body of the script that was attached.
		//	callbackParamName: String
		//		Deprecated as of Dojo 1.4 in favor of "jsonp", but still supported for
		//		legacy code. See notes for jsonp property.
		//	jsonp: String
		//		The URL parameter name that indicates the JSONP callback string.
		//		For instance, when using Yahoo JSONP calls it is normally,
		//		jsonp: "callback". For AOL JSONP calls it is normally
		//		jsonp: "c".
		//	checkString: String
		//		A string of JavaScript that when evaluated like so:
		//		"typeof(" + checkString + ") != 'undefined'"
		//		being true means that the script fetched has been loaded.
		//		Do not use this if doing a JSONP type of call (use callbackParamName instead).
		//	frameDoc: Document
		//		The Document object for a child iframe. If this is passed in, the script
		//		will be attached to that document. This can be helpful in some comet long-polling
		//		scenarios with Firefox and Opera.
		this.callbackParamName = callbackParamName;
		this.jsonp = jsonp;
		this.checkString = checkString;
		this.frameDoc = frameDoc;
	}
});
=====*/

	var loadEvent = dojo.isIE ? "onreadystatechange" : "load",
		readyRegExp = /complete|loaded/;

	dojo.io.script = {
		get: function(/*dojo.io.script.__ioArgs*/args){
			//	summary:
			//		sends a get request using a dynamically created script tag.
			var dfd = this._makeScriptDeferred(args);
			var ioArgs = dfd.ioArgs;
			dojo._ioAddQueryToUrl(ioArgs);

			dojo._ioNotifyStart(dfd);

			if(this._canAttach(ioArgs)){
				var node = this.attach(ioArgs.id, ioArgs.url, args.frameDoc);

				//If not a jsonp callback or a polling checkString case, bind
				//to load event on the script tag.
				if(!ioArgs.jsonp && !ioArgs.args.checkString){
					var handle = dojo.connect(node, loadEvent, function(evt){
						if(evt.type == "load" || readyRegExp.test(node.readyState)){
							dojo.disconnect(handle);
							ioArgs.scriptLoaded = evt;
						}
					});
				}
			}

			dojo._ioWatch(dfd, this._validCheck, this._ioCheck, this._resHandle);
			return dfd;
		},

		attach: function(/*String*/id, /*String*/url, /*Document?*/frameDocument){
			//	summary:
			//		creates a new <script> tag pointing to the specified URL and
			//		adds it to the document.
			//	description:
			//		Attaches the script element to the DOM.	 Use this method if you
			//		just want to attach a script to the DOM and do not care when or
			//		if it loads.
			var doc = (frameDocument || dojo.doc);
			var element = doc.createElement("script");
			element.type = "text/javascript";
			element.src = url;
			element.id = id;
			element.async = true;
			element.charset = "utf-8";
			return doc.getElementsByTagName("head")[0].appendChild(element);
		},

		remove: function(/*String*/id, /*Document?*/frameDocument){
			//summary: removes the script element with the given id, from the given frameDocument.
			//If no frameDocument is passed, the current document is used.
			dojo.destroy(dojo.byId(id, frameDocument));

			//Remove the jsonp callback on dojo.io.script, if it exists.
			if(this["jsonp_" + id]){
				delete this["jsonp_" + id];
			}
		},

		_makeScriptDeferred: function(/*Object*/args){
			//summary:
			//		sets up a Deferred object for an IO request.
			var dfd = dojo._ioSetArgs(args, this._deferredCancel, this._deferredOk, this._deferredError);

			var ioArgs = dfd.ioArgs;
			ioArgs.id = dojo._scopeName + "IoScript" + (this._counter++);
			ioArgs.canDelete = false;

			//Special setup for jsonp case
			ioArgs.jsonp = args.callbackParamName || args.jsonp;
			if(ioArgs.jsonp){
				//Add the jsonp parameter.
				ioArgs.query = ioArgs.query || "";
				if(ioArgs.query.length > 0){
					ioArgs.query += "&";
				}
				ioArgs.query += ioArgs.jsonp
					+ "="
					+ (args.frameDoc ? "parent." : "")
					+ dojo._scopeName + ".io.script.jsonp_" + ioArgs.id + "._jsonpCallback";

				ioArgs.frameDoc = args.frameDoc;

				//Setup the Deferred to have the jsonp callback.
				ioArgs.canDelete = true;
				dfd._jsonpCallback = this._jsonpCallback;
				this["jsonp_" + ioArgs.id] = dfd;
			}
			return dfd; // dojo.Deferred
		},

		_deferredCancel: function(/*Deferred*/dfd){
			//summary: canceller function for dojo._ioSetArgs call.

			//DO NOT use "this" and expect it to be dojo.io.script.
			dfd.canceled = true;
			if(dfd.ioArgs.canDelete){
				dojo.io.script._addDeadScript(dfd.ioArgs);
			}
		},

		_deferredOk: function(/*Deferred*/dfd){
			//summary: okHandler function for dojo._ioSetArgs call.

			//DO NOT use "this" and expect it to be dojo.io.script.
			var ioArgs = dfd.ioArgs;

			//Add script to list of things that can be removed.
			if(ioArgs.canDelete){
				dojo.io.script._addDeadScript(ioArgs);
			}

			//Favor JSONP responses, script load events then lastly ioArgs.
			//The ioArgs are goofy, but cannot return the dfd since that stops
			//the callback chain in Deferred. The return value is not that important
			//in that case, probably a checkString case.
			return ioArgs.json || ioArgs.scriptLoaded || ioArgs;
		},

		_deferredError: function(/*Error*/error, /*Deferred*/dfd){
			//summary: errHandler function for dojo._ioSetArgs call.

			if(dfd.ioArgs.canDelete){
				//DO NOT use "this" and expect it to be dojo.io.script.
				if(error.dojoType == "timeout"){
					//For timeouts, remove the script element immediately to
					//avoid a response from it coming back later and causing trouble.
					dojo.io.script.remove(dfd.ioArgs.id, dfd.ioArgs.frameDoc);
				}else{
					dojo.io.script._addDeadScript(dfd.ioArgs);
				}
			}
			console.log("dojo.io.script error", error);
			return error;
		},

		_deadScripts: [],
		_counter: 1,

		_addDeadScript: function(/*Object*/ioArgs){
			//summary: sets up an entry in the deadScripts array.
			dojo.io.script._deadScripts.push({id: ioArgs.id, frameDoc: ioArgs.frameDoc});
			//Being extra paranoid about leaks:
			ioArgs.frameDoc = null;
		},

		_validCheck: function(/*Deferred*/dfd){
			//summary: inflight check function to see if dfd is still valid.

			//Do script cleanup here. We wait for one inflight pass
			//to make sure we don't get any weird things by trying to remove a script
			//tag that is part of the call chain (IE 6 has been known to
			//crash in that case).
			var _self = dojo.io.script;
			var deadScripts = _self._deadScripts;
			if(deadScripts && deadScripts.length > 0){
				for(var i = 0; i < deadScripts.length; i++){
					//Remove the script tag
					_self.remove(deadScripts[i].id, deadScripts[i].frameDoc);
					deadScripts[i].frameDoc = null;
				}
				dojo.io.script._deadScripts = [];
			}

			return true;
		},

		_ioCheck: function(/*Deferred*/dfd){
			//summary: inflight check function to see if IO finished.
			var ioArgs = dfd.ioArgs;
			//Check for finished jsonp
			if(ioArgs.json || (ioArgs.scriptLoaded && !ioArgs.args.checkString)){
				return true;
			}

			//Check for finished "checkString" case.
			var checkString = ioArgs.args.checkString;
			return checkString && eval("typeof(" + checkString + ") != 'undefined'");


		},

		_resHandle: function(/*Deferred*/dfd){
			//summary: inflight function to handle a completed response.
			if(dojo.io.script._ioCheck(dfd)){
				dfd.callback(dfd);
			}else{
				//This path should never happen since the only way we can get
				//to _resHandle is if _ioCheck is true.
				dfd.errback(new Error("inconceivable dojo.io.script._resHandle error"));
			}
		},

		_canAttach: function(/*Object*/ioArgs){
			//summary: A method that can be overridden by other modules
			//to control when the script attachment occurs.
			return true;
		},

		_jsonpCallback: function(/*JSON Object*/json){
			//summary:
			//		generic handler for jsonp callback. A pointer to this function
			//		is used for all jsonp callbacks.  NOTE: the "this" in this
			//		function will be the Deferred object that represents the script
			//		request.
			this.ioArgs.json = json;
		}
	};

	return dojo.io.script;
});

},
'dojo/html':function(){
define(["./_base/kernel", "./_base/lang", "./_base/array", "./_base/declare", "./dom", "./dom-construct", "./parser"], function(dojo, lang, darray, declare, dom, domConstruct, parser) {
	// module:
	//		dojo/html
	// summary:
	//		TODOC

	lang.getObject("html", true, dojo);

	// the parser might be needed..

	// idCounter is incremented with each instantiation to allow asignment of a unique id for tracking, logging purposes
	var idCounter = 0;

	dojo.html._secureForInnerHtml = function(/*String*/ cont){
		// summary:
		//		removes !DOCTYPE and title elements from the html string.
		//
		//		khtml is picky about dom faults, you can't attach a style or <title> node as child of body
		//		must go into head, so we need to cut out those tags
		//	cont:
		//		An html string for insertion into the dom
		//
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
	};

/*====
	dojo.html._emptyNode = function(node){
		// summary:
		//		removes all child nodes from the given node
		//	node: DOMNode
		//		the parent element
	};
=====*/
	dojo.html._emptyNode = domConstruct.empty;

	dojo.html._setNodeContent = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont){
		// summary:
		//		inserts the given content into the given node
		//	node:
		//		the parent element
		//	content:
		//		the content to be set on the parent element.
		//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes

		// always empty
		domConstruct.empty(node);

		if(cont) {
			if(typeof cont == "string") {
				cont = domConstruct.toDom(cont, node.ownerDocument);
			}
			if(!cont.nodeType && lang.isArrayLike(cont)) {
				// handle as enumerable, but it may shrink as we enumerate it
				for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0) {
					domConstruct.place( cont[i], node, "last");
				}
			} else {
				// pass nodes, documentFragments and unknowns through to dojo.place
				domConstruct.place(cont, node, "last");
			}
		}

		// return DomNode
		return node;
	};

	// we wrap up the content-setting operation in a object
	declare("dojo.html._ContentSetter", null,
		{
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",

			// id: String?
			//		Usually only used internally, and auto-generated with each instance
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,

			// extractContent: Boolean
			//		Should the content be treated as a full html document, and the real content stripped of <html>, <body> wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,

			// parserScope: String
			//		Flag passed to parser.	Root for attribute names to search for.	  If scopeName is dojo,
			//		will search for data-dojo-type (or dojoType).  For backwards compatibility
			//		reasons defaults to dojo._scopeName (which is "dojo" except when
			//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			parserScope: dojo._scopeName,

			// startup: Boolean
			//		Start the child widgets after parsing them.	  Only obeyed if parseContent is true.
			startup: true,

			// lifecyle methods
			constructor: function(/* Object */params, /* String|DomNode */node){
				//	summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..

				// the original params are mixed directly into the instance "this"
				lang.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dom.byId( this.node || node );

				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "",
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /* Object? */ params){
				// summary:
				//		front-end to the set-content sequence
				//	cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();
				this.onEnd();

				return this.node;
			},
			setContent: function(){
				// summary:
				//		sets the content on the node

				var node = this.node;
				if(!node) {
					// can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = dojo.html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV

					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e);
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},

			empty: function() {
				// summary
				//	cleanly empty out existing content

				// destroy any widgets from a previous run
				// NOTE: if you dont want this you'll need to empty
				// the parseResults array property yourself to avoid bad things happenning
				if(this.parseResults && this.parseResults.length) {
					darray.forEach(this.parseResults, function(w) {
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could
				// override empty to skip this step
				dojo.html._emptyNode(this.node);
			},

			onBegin: function(){
				// summary
				//		Called after instantiation, but before set();
				//		It allows modification of any of the object properties
				//		- including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to
				//		optionally pre-process html string content
				var cont = this.content;

				if(lang.isString(cont)){
					if(this.cleanContent){
						cont = dojo.html._secureForInnerHtml(cont);
					}

					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();

				this.content = cont;
				return this.node; /* DomNode */
			},

			onEnd: function(){
				// summary
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults if you need those..
					this._parse();
				}
				return this.node; /* DomNode */
			},

			tearDown: function(){
				// summary
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description
				//		tearDown() is not called automatically.
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults;
				delete this.node;
				delete this.content;
			},

			onContentError: function(err){
				return "Error occured setting content: " + err;
			},

			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we dont consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key];
				}
			},
			_parse: function(){
				// summary:
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					var inherited = {};
					darray.forEach(["dir", "lang", "textDir"], function(name){
						if(this[name]){
							inherited[name] = this[name];
						}
					}, this);
					this.parseResults = parser.parse({
						rootNode: rootNode,
						noStart: !this.startup,
						inherited: inherited,
						scope: this.parserScope
					});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#"+this.id);
				}
			},

			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		overide/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					dojo.html._setNodeContent(this.node, errText, true);
				}
			}
	}); // end dojo.declare()

	dojo.html.set = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont, /* Object? */ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo.place(cont, node, "only"). dojo.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			//	node:
			//		the parent element that will receive the content
			//	cont:
			//		the content to be set on the parent element.
			//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
			//	params:
			//		Optional flags/properties to configure the content-setting. See dojo.html._ContentSetter
			//	example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage:
			//		dojo.html.set(node, "some string");
			//		dojo.html.set(node, contentNode, {options});
			//		dojo.html.set(node, myNode.childNodes, {options});
		if(undefined == cont){
			console.warn("dojo.html.set: no cont argument provided, using empty string");
			cont = "";
		}
		if(!params){
			// simple and fast
			return dojo.html._setNodeContent(node, cont, true);
		}else{
			// more options but slower
			// note the arguments are reversed in order, to match the convention for instantiation via the parser
			var op = new dojo.html._ContentSetter(lang.mixin(
					params,
					{ content: cont, node: node }
			));
			return op.set();
		}
	};

	return dojo.html;
});

},
'*now':function(r){r(['dojo/i18n!*preload*dojo/nls/dojo*["ar","cs","de","el","en","es","fr","he","he-il","hu","it","iw","iw-il","ja","ko","pl","pt","pt-br","ro","ru","tr","xx","zh","zh-tw","nb","no","sv","fi","da","sk","nl","ROOT"]']);}
}});
(function(){
	// must use this.require to make this work in node.js
	var require = this.require;
	// consume the cached dojo layer
	require({cache:{}});
	!require.async && require(["dojo"]);
	require.boot && require.apply(null, require.boot);
})();
